@@ -207,7 +207,9 @@ static double U[] = {
 
 #define UTHRESH 37.519379347
 
-static double erfce (double);
+static double cephes_erfc (double a);
+static double cephes_erf (double x);
+static double erfce (double a);
 
 double ndtr (double a)
 {
@@ -218,7 +220,7 @@ double ndtr (double a)
 
     /* if( z < SQRTH ) */
     if (z < 1.0) {
-	y = 0.5 + 0.5 * erf(x);
+	y = 0.5 + 0.5 * cephes_erf(x);
 
     } else {
 #ifdef USE_EXPXSQ
@@ -228,7 +230,7 @@ double ndtr (double a)
 	z = expx2(a, -1);
 	y = y * sqrt(z);
 #else
-	y = 0.5 * erfc(z);
+	y = 0.5 * cephes_erfc(z);
 #endif
 	if (x > 0) {
 	    y = 1.0 - y;
@@ -238,7 +240,7 @@ double ndtr (double a)
     return y;
 }
 
-double erfc (double a)
+static double cephes_erfc (double a)
 {
     double p, q, x, y, z;
 
@@ -249,14 +251,14 @@ double erfc (double a)
     }
 
     if (x < 1.0) {
-	return 1.0 - erf(a);
+	return 1.0 - cephes_erf(a);
     }
 
     z = -a * a;
 
     if (z < -MAXLOG) {
     under:
-	mtherr("erfc", UNDERFLOW);
+	mtherr("erfc", CEPHES_UNDERFLOW);
 	if (a < 0) {
 	    return 2.0;
 	} else {
@@ -313,12 +315,12 @@ static double erfce (double x)
     return p/q;
 }
 
-double erf (double x)
+static double cephes_erf (double x)
 {
     double y, z;
 
     if (fabs(x) > 1.0) {
-	return 1.0 - erfc(x);
+	return 1.0 - cephes_erfc(x);
     }
 
     z = x * x;
