@@ -156,6 +156,11 @@ static int saved_object_action (const char *line, PRN *prn)
 	return 0;
     }
 
+    /* special: display icon view window, not applicable */
+    if (!strncmp(line, "iconview", 8)) {
+	return 1;
+    }
+
     action = cli_parse_object_request(line, objname, &param, &ptr, &type, prn);
 
     if (action == OBJ_ACTION_NONE) {
@@ -174,8 +179,8 @@ static int saved_object_action (const char *line, PRN *prn)
 	} else if (type == GRETL_OBJ_VAR) {
 	    gretl_VAR_print((GRETL_VAR *) ptr, datainfo, OPT_NONE, prn);
 	} else if (type == GRETL_OBJ_SYS) {
-	    err = estimate_saved_equation_system((gretl_equation_system *) ptr, 
-						 &Z, datainfo, prn);
+	    err = gretl_equation_system_estimate((gretl_equation_system *) ptr, 
+						 &Z, datainfo, OPT_NONE, prn);
 	} 
     } else if (action == OBJ_ACTION_SHOW_STAT) {
 	err = saved_object_print_scalar(objname, param, prn);
