@@ -25,6 +25,8 @@
 #include <math.h>
 #include <dirent.h>
 
+#define GRETLCLI
+
 #include "libgretl.h"
 #include "var.h"
 #include "system.h"
@@ -36,6 +38,7 @@
 #include "cmd_private.h"
 #include "objstack.h"
 #include "gretl_xml.h"
+#include "gretl_panel.h"
 
 #ifdef WIN32
 # include <windows.h>
@@ -149,17 +152,19 @@ void noalloc (const char *str)
     exit(EXIT_FAILURE);
 }
 
-int model_test_start (int test_ci, int model_id, PRN *prn)
+static int model_test_check (CMD *cmd, PRN *prn)
 {
-    int m, err = 0;
+    return last_model_test_ok(cmd->ci, cmd->opt, datainfo, prn);
+}
 
-    if (model_id != 0) {
-	m = modelspec_index_from_model_id(modelspec, model_id);
-    } else {
-	m = modelspec_last_index(modelspec);
-    }
+#define MSPEC_DEBUG 0
 
-#ifdef MSPEC_DEBUG
+static int modelspec_test_check (int test_ci, int model_id, PRN *prn)
+{
+    int m = modelspec_index_from_model_id(modelspec, model_id);
+    int err = 0;
+
+#if MSPEC_DEBUG
     fprintf(stderr, "model_test_start: test_ci=%d, model_id=%d, m=%d\n",
 	    test_ci, model_id, m);
 #endif
@@ -177,7 +182,7 @@ int model_test_start (int test_ci, int model_id, PRN *prn)
 	pputs(prn, _("Sorry, command not available for this estimator"));
 	pputc(prn, '\n');
 	err = 1;
-    } else if (model_sample_issue(NULL, modelspec, m, datainfo)) {
+    } else if (modelspec_sample_problem(modelspec, m, datainfo)) {
 	pputs(prn, _("Can't do: the current data set is different from "
 		     "the one on which\nthe reference model was estimated\n"));
 	err = 1;
@@ -228,19 +233,6 @@ int fn_get_line (void)
     return err;
 }
 
-gnuplot_flags gp_flags (int batch, gretlopt opt)
-{
-    gnuplot_flags flags = 0;
-
-    if (batch) flags |= GP_BATCH;
-
-    if (opt & OPT_M) flags |= GP_IMPULSES;
-    else if (opt & OPT_Z) flags |= GP_DUMMY;
-    else if (opt & OPT_S) flags |= GP_OLS_OMIT;
-
-    return flags;
-}
-
 #ifdef ENABLE_NLS
 
 void nls_init (void)
@@ -721,28 +713,16 @@ static void printf_strip (char *s)
 
 static int do_autofit_plot (PRN *prn)
 {
-    int *plotlist;
-    int pv, err = 0;
-
-    pv = plotvar(&Z, datainfo);
-    if (pv < 0) {
-	return 1;
-    }
-
-    plotlist = gretl_list_new(3);
-    if (plotlist == NULL) {
-	return 1;
-    }
+    int plotlist[3];
+    int err = 0;
 
+    plotlist[0] = 2;
     plotlist[1] = gretl_model_get_depvar(models[0]);
     plotlist[2] = varindex(datainfo, "autofit");
-    plotlist[3] = pv;
 
     lines[0] = 1;
     err = gnuplot(plotlist, lines, NULL, &Z, datainfo,
-		  &plot_count, gp_flags(batch, 0));
-
-    free(plotlist);
+		  &plot_count, gp_flags(batch, OPT_T));
 
     if (err) {
 	pputs(prn, _("gnuplot command failed\n"));
@@ -751,6 +731,19 @@ static int do_autofit_plot (PRN *prn)
     return err;
 }
 
+static int maybe_print_model (MODEL *pmod, gretlopt opt, PRN *prn)
+{
+    int err = pmod->errcode;
+
+    if (err) {
+	errmsg(err, prn);
+    } else {
+	printmodel(pmod, datainfo, opt, prn);
+    }
+
+    return err;
+}
+
 static int exec_line (char *line, PRN *prn) 
 {
     GRETL_VAR *var = NULL;
@@ -762,6 +755,8 @@ static int exec_line (char *line, PRN *prn)
     double rho;
     int k, err = 0;
 
+    static int in_comment;
+
     if (string_is_blank(line)) {
 	return 0;
     }
@@ -774,17 +769,18 @@ static int exec_line (char *line, PRN *prn)
 	return err;
     } 
 
-    /* catch requests relating to saved objects, which are not
-       really "commands" as such */
-    k = saved_object_action(line, prn);
-    if (k == 1) return 0;   /* action was OK */
-    if (k == -1) return 1;  /* action was faulty */
+    if (!in_comment) {
+	/* catch requests relating to saved objects, which are not
+	   really "commands" as such */
+	k = saved_object_action(line, prn);
+	if (k == 1) return 0;   /* action was OK */
+	if (k == -1) return 1;  /* action was faulty */
 
-    /* are we ready for this? */
-    if (!data_status && !cmd.ignore && 
-	!ready_for_command(line)) {
-	fprintf(stderr, _("You must open a data file first\n"));
-	return 1;
+	/* are we ready for this? */
+	if (!data_status && !cmd.ignore && !ready_for_command(line)) {
+	    fprintf(stderr, _("You must open a data file first\n"));
+	    return 1;
+	}
     }
 
     /* tell libgretl if we're in batch mode */
@@ -805,6 +801,9 @@ static int exec_line (char *line, PRN *prn)
 	return err;
     }
 
+    /* are we in a multi-line comment block? */
+    in_comment = cmd.ignore;
+
     /* if in batch mode, echo comments from input */
     if (batch && cmd.ci == CMD_COMMENT && gretl_echo_on()) {
 	printf_strip(linebak);
@@ -863,6 +862,16 @@ static int exec_line (char *line, PRN *prn)
 
     check_for_loop_only_options(cmd.ci, cmd.opt, prn);
 
+    if (NEEDS_MODEL_CHECK(cmd.ci)) {
+	err = model_test_check(&cmd, prn);
+	if (err) {
+	    if (gretl_executing_function()) {
+		gretl_function_stop_on_error(&Z, &datainfo, prn);
+	    }
+	    return err;
+	}
+    }
+
     switch (cmd.ci) {
 
     case ADDOBS:
@@ -874,6 +883,8 @@ static int exec_line (char *line, PRN *prn)
     case CRITICAL: 
     case DATA:
     case DIFF: 
+    case DISCRETE:
+    case DUMMIFY:
     case ESTIMATE:
     case FNCALL:
     case FUNC:
@@ -883,7 +894,6 @@ static int exec_line (char *line, PRN *prn)
     case HURST:
     case INFO: 
     case KPSS:
-    case LABEL:
     case LABELS: 
     case LAGS: 
     case LDIFF: 
@@ -901,6 +911,7 @@ static int exec_line (char *line, PRN *prn)
     case RMPLOT: 
     case RUNS: 
     case SDIFF:
+    case SETINFO:
     case SHELL:
     case SPEARMAN:
     case SQUARE: 
@@ -909,6 +920,7 @@ static int exec_line (char *line, PRN *prn)
     case TRANSPOSE:
     case VARLIST:
     case VARTEST: 
+    case XTAB:
 	err = simple_commands(&cmd, line, &Z, datainfo, prn);
 	if (err) {
 	    errmsg(err, prn);
@@ -917,7 +929,6 @@ static int exec_line (char *line, PRN *prn)
 
     case ADD:
     case OMIT:
-	if ((err = model_test_start(cmd.ci, 0, prn))) break;
     plain_add_omit:
 	clear_model(models[1]);
 	if (cmd.ci == ADD || cmd.ci == ADDTO) {
@@ -929,13 +940,12 @@ static int exec_line (char *line, PRN *prn)
 	}
 	if (err) {
 	    errmsg(err, prn);
-	    clear_model(models[1]);
-	} else {
+	} else if (!(cmd.opt & OPT_Q) && !(cmd.opt & OPT_W)) {
 	    /* for command-line use, we keep a "stack" of 
 	       two models, and recycle the places */
-	    if (!(cmd.opt & OPT_Q)) {
-		swap_models(models[0], models[1]);
-	    } 
+	    swap_models(models[0], models[1]);
+	}
+	if (!(cmd.opt & OPT_W)) {
 	    clear_model(models[1]);
 	}
 	break;	
@@ -943,7 +953,7 @@ static int exec_line (char *line, PRN *prn)
     case ADDTO:
     case OMITFROM:
 	k = atoi(cmd.param);
-	if ((err = model_test_start(cmd.ci, k, prn))) break;
+	if ((err = modelspec_test_check(cmd.ci, k, prn))) break;
 	if (k == models[0]->ID) goto plain_add_omit;
 	err = re_estimate(modelspec_get_command_by_id(modelspec, k), 
 			  &tmpmod, &Z, datainfo);
@@ -974,11 +984,15 @@ static int exec_line (char *line, PRN *prn)
 	break;
 
     case AR:
+    case ARMA:
 	clear_model(models[0]);
-	*models[0] = ar_func(cmd.list, &Z, datainfo, cmd.opt, prn);
-	if ((err = models[0]->errcode)) { 
-	    errmsg(err, prn); 
+	if (cmd.ci == AR) {
+	    *models[0] = ar_func(cmd.list, &Z, datainfo, cmd.opt, prn);
+	} else {
+	    *models[0] = arma(cmd.list, (const double **) Z, datainfo,
+			      cmd.opt, prn);
 	}
+	err = maybe_print_model(models[0], cmd.opt, prn);
 	break;
 
     case ARCH:
@@ -995,30 +1009,19 @@ static int exec_line (char *line, PRN *prn)
 	clear_model(models[1]);
 	break;
 
-    case ARMA:
-	clear_model(models[0]);
-	*models[0] = arma(cmd.list, (const double **) Z, datainfo,
-			  cmd.opt, prn);
-	if ((err = (models[0])->errcode)) { 
-	    errmsg(err, prn); 
-	} else {	
-	    printmodel(models[0], datainfo, cmd.opt, prn);
-	}	
-	break;
-
     case COEFFSUM:
     case CUSUM:
     case RESET:
     case CHOW:
+    case QLRTEST:
     case VIF:
-        if ((err = model_test_start(cmd.ci, 0, prn))) break;
 	if (cmd.ci == COEFFSUM) {
 	    err = sum_test(cmd.list, models[0], &Z, datainfo, prn);
 	} else if (cmd.ci == CUSUM) {
 	    err = cusum_test(models[0], &Z, datainfo, OPT_NONE, prn);
 	} else if (cmd.ci == RESET) {
 	    err = reset_test(models[0], &Z, datainfo, OPT_NONE, prn);
-	} else if (cmd.ci == CHOW) {
+	} else if (cmd.ci == CHOW || cmd.ci == QLRTEST) {
 	    err = chow_test(line, models[0], &Z, datainfo, OPT_NONE, prn);
 	} else {
 	    err = vif_test(models[0], &Z, datainfo, prn);
@@ -1039,16 +1042,12 @@ static int exec_line (char *line, PRN *prn)
 	}
 	clear_model(models[0]);
 	*models[0] = ar1_lsq(cmd.list, &Z, datainfo, cmd.ci, cmd.opt, rho);
-	if ((err = (models[0])->errcode)) {
-	    errmsg(err, prn);
-	} else {
-	    printmodel(models[0], datainfo, cmd.opt, prn); 
-	}
+	err = maybe_print_model(models[0], cmd.opt, prn);
 	break;
 
     case CORRGM:
 	k = atoi(cmd.param);
-	err = corrgram(cmd.list[1], k, &Z, datainfo, prn, OPT_A);
+	err = corrgram(cmd.list[1], k, 0, &Z, datainfo, prn, OPT_A);
 	if (err) {
 	    pputs(prn, _("Failed to generate correlogram\n"));
 	}
@@ -1082,14 +1081,13 @@ static int exec_line (char *line, PRN *prn)
 	} else if (!strcmp(cmd.param, "mle") || !strcmp(cmd.param, "nls")) {
 	    clear_model(models[0]);
 	    *models[0] = nls(&Z, datainfo, cmd.opt, prn);
-	    if ((err = models[0]->errcode)) {
-		errmsg(err, prn);
-	    } else {
+	    err = maybe_print_model(models[0], cmd.opt, prn);
+	    if (!err) {
 		alt_model = 1;
-		printmodel(models[0], datainfo, cmd.opt, prn);
 	    }
 	} else if (!strcmp(cmd.param, "restrict")) {
-	    err = gretl_restriction_set_finalize(rset, datainfo, prn);
+	    err = gretl_restriction_set_finalize(rset, (const double **) Z, 
+						 datainfo, prn);
 	    if (err) {
 		errmsg(err, prn);
 	    }
@@ -1117,9 +1115,6 @@ static int exec_line (char *line, PRN *prn)
     case TABPRINT:
     case EQNPRINT:
 	strcpy(texfile, cmd.param);
-	if ((err = model_test_start(cmd.ci, 0, prn))) {
-	    break;
-	}
 	err = texprint(models[0], datainfo, texfile, 
 		       (cmd.ci == EQNPRINT)? (cmd.opt | OPT_E) :
 		       cmd.opt);
@@ -1132,7 +1127,6 @@ static int exec_line (char *line, PRN *prn)
 
     case FCAST:
     case FIT:
-	if ((err = model_test_start(cmd.ci, 0, prn))) break;
 	if (cmd.ci == FIT) {
 	    err = add_forecast("fcast autofit", models[0], &Z, datainfo, cmd.opt);
 	} else {
@@ -1152,7 +1146,6 @@ static int exec_line (char *line, PRN *prn)
 	break;
 
     case FCASTERR:
-	if ((err = model_test_start(cmd.ci, 0, prn))) break;
 	err = display_forecast(line, models[0], &Z, datainfo, cmd.opt, prn);
 	if (err) {
 	    errmsg(err, prn);
@@ -1161,7 +1154,7 @@ static int exec_line (char *line, PRN *prn)
 
     case FREQ:
 	err = freqdist(cmd.list[1], (const double **) Z, 
-		       datainfo, !batch, prn, cmd.opt);
+		       datainfo, !batch, cmd.opt, prn);
 	break;
 
     case GENR:
@@ -1188,7 +1181,7 @@ static int exec_line (char *line, PRN *prn)
 			  &Z, datainfo, &plot_count, 
 			  gp_flags(batch, 0));
 	}
-	if (err < 0) {
+	if (err) {
 	    pputs(prn, _("gnuplot command failed\n"));
 	} else if (batch) {
 	    pprintf(prn, _("wrote %s\n"), gretl_plotfile());
@@ -1196,8 +1189,8 @@ static int exec_line (char *line, PRN *prn)
 	break;
 
     case SCATTERS:
-	err = multi_scatters(cmd.list, &Z, datainfo, &plot_count, 
-			     gp_flags(batch, cmd.opt));
+	err = multi_scatters(cmd.list, (const double **) Z, datainfo, 
+			     &plot_count, gp_flags(batch, cmd.opt));
 	if (err) {
 	    pputs(prn, _("scatters command failed\n"));
 	} else if (batch) {
@@ -1206,10 +1199,7 @@ static int exec_line (char *line, PRN *prn)
 	break;
 
     case HAUSMAN:
-	err = model_test_start(cmd.ci, 0, prn);
-	if (!err) {
-	    err = hausman_test(models[0], &Z, datainfo, cmd.opt, prn);
-	}
+	err = panel_hausman_test(models[0], &Z, datainfo, cmd.opt, prn);
 	break;
 
     case HELP:
@@ -1252,7 +1242,7 @@ static int exec_line (char *line, PRN *prn)
 	}
 
 	k = detect_filetype(datfile, &paths, prn);
-	dbdata = (k == GRETL_NATIVE_DB || k == GRETL_RATS_DB);
+	dbdata = (k == GRETL_NATIVE_DB || k == GRETL_RATS_DB || k == GRETL_PCGIVE_DB);
 
 	if (data_status && !batch && !dbdata && cmd.ci != APPEND &&
 	    strcmp(datfile, paths.datfile)) {
@@ -1290,22 +1280,21 @@ static int exec_line (char *line, PRN *prn)
 
 	if (err) {
 	    errmsg(err, prn);
-	    break;
+	} else {
+	    if (!dbdata && cmd.ci != APPEND) {
+		strncpy(paths.datfile, datfile, MAXLEN-1);
+	    }
+	    data_status = 1;
+	    if (datainfo->v > 0 && !dbdata) {
+		varlist(datainfo, prn);
+	    }
+	    paths.currdir[0] = '\0';
 	}
-	if (!dbdata && cmd.ci != APPEND) {
-	    strncpy(paths.datfile, datfile, MAXLEN-1);
-	}
-	data_status = 1;
-	if (datainfo->v > 0 && !dbdata) {
-	    varlist(datainfo, prn);
-	}
-	paths.currdir[0] = '\0';
 	break;
 
     case LEVERAGE:
-	if ((err = model_test_start(cmd.ci, 0, prn))) break;	
 	err = leverage_test(models[0], &Z, datainfo, cmd.opt, prn);
-	if (err > 1) {
+	if (err) {
 	    errmsg(err, prn);
 	} else if (cmd.opt & OPT_S) {
 	    maybe_list_vars(datainfo, prn);
@@ -1313,40 +1302,9 @@ static int exec_line (char *line, PRN *prn)
 	break;
 
     case LMTEST:
-	if ((err = model_test_start(cmd.ci, 0, prn))) break;
-	/* non-linearity (squares) */
-	if ((cmd.opt & OPT_S) || (cmd.opt & OPT_O) || !cmd.opt) {
-	    clear_model(models[1]);
-	    err = nonlinearity_test(models[0], &Z, datainfo, 
-				    AUX_SQ, OPT_NONE, prn);
-	    if (err) errmsg(err, prn);
-	    if (cmd.opt == OPT_S) break;
-	    if (!err && !batch && scroll_pause_or_quit()) break; 
-	}
-	/* non-linearity (logs) */
-	if ((cmd.opt & OPT_L) || (cmd.opt & OPT_O) || !cmd.opt) {
-	    err = nonlinearity_test(models[0], &Z, datainfo, 
-				    AUX_LOG, OPT_NONE, prn);
-	    if (err) errmsg(err, prn);
-	    if (cmd.opt == OPT_L) break;
-	    if (!err && !batch && scroll_pause_or_quit()) break;
-	}
-	/* autocorrelation */
-	if ((cmd.opt & OPT_M) || (cmd.opt & OPT_O)) {
-	    int order = atoi(cmd.param);
-
-	    err = autocorr_test(models[0], order, &Z, datainfo, OPT_NONE, prn);
-	    if (err) errmsg(err, prn);
-	}
-	/* heteroskedasticity */
-	if ((cmd.opt & OPT_W) || !cmd.opt) {
-	    err = whites_test(models[0], &Z, datainfo, OPT_NONE, prn);
-	    if (err) errmsg(err, prn);
-	}
-	/* groupwise heteroskedasticity */
-	if (cmd.opt & OPT_P) {
-	    err = groupwise_hetero_test(models[0], &Z, datainfo, prn);
-	    if (err) errmsg(err, prn);
+	err = lmtest_driver(cmd.param, &Z, datainfo, cmd.opt, prn);
+	if (err) {
+	    errmsg(err, prn);
 	}
 	break;
 
@@ -1355,6 +1313,7 @@ static int exec_line (char *line, PRN *prn)
     case LAD:
     case LOGISTIC:
     case LOGIT:
+    case PANEL:
     case POISSON:
     case PROBIT:
     case TOBIT:
@@ -1378,16 +1337,14 @@ static int exec_line (char *line, PRN *prn)
 	    *models[0] = lad(cmd.list, &Z, datainfo);
 	} else if (cmd.ci == GARCH) {
 	    *models[0] = garch(cmd.list, &Z, datainfo, cmd.opt, prn);
+	} else if (cmd.ci == PANEL) {
+	    *models[0] = panel_model(cmd.list, &Z, datainfo, cmd.opt, prn);
 	} else {
 	    /* can't happen */
 	    err = 1;
 	    break;
 	}
-	if ((err = (models[0])->errcode)) {
-	    errmsg(err, prn);
-	} else {
-	    printmodel(models[0], datainfo, cmd.opt, prn);
-	}
+	err = maybe_print_model(models[0], cmd.opt, prn);
 	break;
 
     case MLE:
@@ -1401,15 +1358,13 @@ static int exec_line (char *line, PRN *prn)
 	break;
 
     case NULLDATA:
-	k = atoi(cmd.param);
-	if (k < 2) {
-	    pputs(prn, _("Data series length count missing or invalid\n"));
+	k = gretl_int_from_string(cmd.param, (const double **) Z, 
+				  datainfo, &err);
+	if (!err && k < 2) {
 	    err = 1;
-	    break;
 	}
-	if (k > 1000000) {
-	    pputs(prn, _("Data series too long\n"));
-	    err = 1;
+	if (err) {
+	    pputs(prn, _("Data series length count missing or invalid\n"));
 	    break;
 	}
 	if (data_status) {
@@ -1426,36 +1381,21 @@ static int exec_line (char *line, PRN *prn)
     case OLS:
     case WLS:
     case HCCM:
-    case POOLED:
 	clear_model(models[0]);
-	if (cmd.ci == POOLED) {
-	    *models[0] = pooled(cmd.list, &Z, datainfo, cmd.opt, prn);
-	} else {
-	    *models[0] = lsq(cmd.list, &Z, datainfo, cmd.ci, cmd.opt);
-	}
-	if ((err = models[0]->errcode)) {
-	    errmsg(err, prn);
-	} else {
-	    printmodel(models[0], datainfo, cmd.opt, prn);
-	}
+	*models[0] = lsq(cmd.list, &Z, datainfo, cmd.ci, cmd.opt);
+	err = maybe_print_model(models[0], cmd.opt, prn);
 	break;
 
 #ifdef ENABLE_GMP
     case MPOLS:
-	err = mp_ols(cmd.list, cmd.param, &Z, datainfo, prn);
-	if (err) {
-	    pputs(prn, _("mpols command failed\n"));
-	    errmsg(err, prn);
-	}
+	clear_model(models[0]);
+	*models[0] = mp_ols(cmd.list, (const double **) Z, datainfo);
+	err = maybe_print_model(models[0], cmd.opt, prn);
 	break;
 #endif
 
-    case PANEL:	
-	err = set_panel_structure(cmd.opt, datainfo, prn);
-	break;
-
     case PERGM:
-	err = periodogram(cmd.list[1], &Z, datainfo, prn, cmd.opt | OPT_N);
+	err = periodogram(cmd.list[1], &Z, datainfo, cmd.opt | OPT_N, prn);
 	if (err) {
 	    pputs(prn, _("Failed to generate periodogram\n"));
 	}
@@ -1522,6 +1462,13 @@ static int exec_line (char *line, PRN *prn)
 	    pputs(prn, _("Command is malformed\n"));
 	    break;
 	}
+	if (cmd.ci == INCLUDE && gretl_is_xml_file(runfile)) {
+	    err = load_user_function_file(runfile);
+	    if (err) {
+		pputs(prn, _("Error reading function definitions\n"));
+	    }
+	    break;
+	}
 	if (fb != NULL) {
 	    push_input_file(fb);
 	}
@@ -1540,14 +1487,14 @@ static int exec_line (char *line, PRN *prn)
 	break;
 
     case SET:
-	err = execute_set_line(line, prn);
+	err = execute_set_line(line, datainfo, prn);
 	if (err) {
 	    errmsg(err, prn);
 	}
 	break;
 
     case SETOBS:
-	err = set_obs(line, datainfo, cmd.opt);
+	err = set_obs(line, &Z, datainfo, cmd.opt);
 	if (err) {
 	    errmsg(err, prn);
 	} else {
@@ -1584,11 +1531,11 @@ static int exec_line (char *line, PRN *prn)
 	if (rset == NULL) {
 	    if (*cmd.param == '\0') {
 		/* if param is non-blank, we're restricting a named system */
-		if ((err = model_test_start(cmd.ci, 0, prn))) break;
+		err = model_test_check(&cmd, prn);
+		if (err) break;
 	    }
-	    rset = restriction_set_start(line, models[0], datainfo);
-	    if (rset == NULL) {
-		err = 1;
+	    rset = restriction_set_start(line, cmd.opt, &err);
+	    if (err) {
 		errmsg(err, prn);
 	    } else {
 		gretl_cmd_set_context(&cmd, RESTRICT);
@@ -1622,9 +1569,10 @@ static int exec_line (char *line, PRN *prn)
 	break;
 
     case TESTUHAT:
-	if ((err = model_test_start(cmd.ci, 0, prn))) break;
-	err = model_error_dist(models[0], &Z, datainfo,
-			       prn);
+	err = last_model_test_uhat(&Z, datainfo, prn);
+	if (err) {
+	    errmsg(err, prn);
+	}
 	break;
 
     case VAR:
@@ -1659,7 +1607,7 @@ static int exec_line (char *line, PRN *prn)
     if (!err && (is_model_cmd(cmd.word) || alt_model)
 	&& !is_quiet_model_test(cmd.ci, cmd.opt)) { 
 	attach_subsample_to_model(models[0], datainfo);
-#ifdef MSPEC_DEBUG
+#if MSPEC_DEBUG
 	fprintf(stderr, "\ngretlcli: saving spec: model.ID = %d, model_count = %d\n",
 		(models[0])->ID, get_model_count());
 #endif
