@@ -28,11 +28,7 @@
 # include <windows.h>
 #endif
 
-#ifdef OLD_GTK
-#include <gtkextra/gtkextra.h>
-#else
 #include "gtkplot-lite.h"
-#endif
 
 typedef struct {
     int n;
@@ -82,20 +78,16 @@ static int cb_copy_image (gpointer data);
 int plot_to_xpm (const char *fname, gpointer data);
 #endif
 
-/* ............................................................. */
 
 /* Create a new backing pixmap of the appropriate size */
+
 static gint
 configure_event (GtkWidget *widget, GdkEventConfigure *event, gpointer data)
 {
     PLOTGROUP *grp = (PLOTGROUP *) data;
 
     if (grp->pixmap) {
-#ifndef OLD_GTK
 	g_object_unref(G_OBJECT(grp->pixmap));
-#else
-	gdk_pixmap_unref(grp->pixmap);
-#endif
     }
 
     grp->pixmap = gdk_pixmap_new(widget->window,
@@ -113,9 +105,8 @@ configure_event (GtkWidget *widget, GdkEventConfigure *event, gpointer data)
     return TRUE;
 }
 
-/* ............................................................. */
-
 /* Redraw the screen from the backing pixmap */
+
 static gint
 expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer data)
 {
@@ -131,19 +122,15 @@ expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer data)
     return FALSE;
 }
 
-/* ............................................................. */
-
 static gboolean 
 box_key_handler (GtkWidget *w, GdkEventKey *key, gpointer data)
 {
     if (key->keyval == GDK_q) {
 	gtk_widget_destroy(w);
-    }
-    else if (key->keyval == GDK_s) {
+    } else if (key->keyval == GDK_s) {
         file_selector(_("Save boxplot file"), SAVE_BOXPLOT_EPS, 
 		      FSEL_DATA_MISC, data);
-    }
-    else if (key->keyval == GDK_p) {  
+    } else if (key->keyval == GDK_p) {  
 	five_numbers(data);
     }
 #ifdef G_OS_WIN32
@@ -159,16 +146,10 @@ box_key_handler (GtkWidget *w, GdkEventKey *key, gpointer data)
     return TRUE;
 }
 
-/* ........................................................... */
-
 static gint box_popup_activated (GtkWidget *w, gpointer data)
 {
     gchar *item = (gchar *) data;
-#ifndef OLD_GTK
     gpointer ptr = g_object_get_data(G_OBJECT(w), "group");
-#else
-    gpointer ptr = gtk_object_get_data(GTK_OBJECT(w), "group");
-#endif
     PLOTGROUP *grp = (PLOTGROUP *) ptr;
 
     if (!strcmp(item, _("Five-number summary"))) {
@@ -176,7 +157,7 @@ static gint box_popup_activated (GtkWidget *w, gpointer data)
     }
     else if (!strcmp(item, _("Save to session as icon"))) {
         if (dump_boxplot(grp) == 0) {
-	    add_graph_to_session(NULL, GRETL_BOXPLOT, NULL);
+	    add_graph_to_session(NULL, GRETL_OBJ_PLOT, NULL);
 	    grp->saved = 1;
 	}
     }
@@ -214,8 +195,6 @@ static gint box_popup_activated (GtkWidget *w, gpointer data)
     return TRUE;
 }
 
-/* ........................................................... */
-
 static GtkWidget *build_menu (PLOTGROUP *grp)
 {
     GtkWidget *menu, *item;    
@@ -247,19 +226,13 @@ static GtkWidget *build_menu (PLOTGROUP *grp)
 			 G_CALLBACK(box_popup_activated),
 			 _(items[i]));
 	g_object_set_data(G_OBJECT(item), "group", grp);
-#ifndef OLD_GTK	
         gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-#else
-        gtk_menu_append(GTK_MENU(menu), item);
-#endif
 	gtk_widget_show(item);
 	i++;
     }
     return menu;
 }
 
-/* ........................................................... */
-
 static gint box_popup (GtkWidget *widget, GdkEventButton *event, 
 		       gpointer data)
 {
@@ -272,8 +245,6 @@ static gint box_popup (GtkWidget *widget, GdkEventButton *event,
     return TRUE;
 }
 
-/* ............................................................. */
-
 static void
 setup_text (GtkWidget *area, GdkPixmap *pixmap,
 	    GdkGC *gc, GtkPlotPC *pc, const char *text, 
@@ -295,7 +266,6 @@ setup_text (GtkWidget *area, GdkPixmap *pixmap,
 				 just,
 				 text);
     } else {
-#ifndef OLD_GTK
 	PangoContext *context;
 	PangoLayout *pl;
 	int width, height;
@@ -315,32 +285,9 @@ setup_text (GtkWidget *area, GdkPixmap *pixmap,
 
 	gdk_draw_layout (pixmap, gc, x, y, pl);
 	g_object_unref (G_OBJECT(pl));
-#else
-	GdkRectangle rect; 
-	size_t len = strlen(text);
-	int cw = gdk_char_width(fixed_font, 'X');
-	
-	if (just == GTK_JUSTIFY_CENTER) {
-	    x -= len * cw / 2.0;
-	}
-	else if (just == GTK_JUSTIFY_RIGHT) {
-	    int ch = gdk_char_height(fixed_font, '1');
-
-	    y += (double) ch / 2.0;
-	    x -= len * cw;
-	}
-	rect.x = x;
-	rect.y = y;
-	rect.width = 80;
-	rect.height = 10;	
-	gdk_draw_string (pixmap, fixed_font, gc, x, y, text);
-	gtk_widget_draw (area, &rect);
-#endif
     }
 }
 
-/* ............................................................. */
-
 static void 
 draw_line (double *points, GtkWidget *area, GdkPixmap *pixmap,
 	   GdkGC *gc, GtkPlotPC *pc)
@@ -348,23 +295,10 @@ draw_line (double *points, GtkWidget *area, GdkPixmap *pixmap,
     if (pc != NULL) {
 	gtk_plot_pc_draw_line (pc, points[0], points[1], points[2], points[3]); 
     } else {
-#ifndef OLD_GTK
 	gdk_draw_line (pixmap, gc, points[0], points[1], points[2], points[3]);
-#else
-	GdkRectangle rect;
-
-	rect.x = points[0];
-	rect.y = points[1];
-	rect.width = points[2] - points[1] + 1;
-	rect.height = points[3] - points[1] + 1;
-	gdk_draw_line (pixmap, gc, points[0], points[1], points[2], points[3]);
-	gtk_widget_draw (area, &rect);
-#endif
     }
 }
 
-/* ............................................................. */
-
 static void
 draw_outlier (double x, double y,
 	      GtkWidget *area, GdkPixmap *pixmap, 
@@ -373,24 +307,11 @@ draw_outlier (double x, double y,
     if (pc != NULL) {
 	gtk_plot_pc_draw_circle (pc, FALSE, x, y, 8);
     } else {
-#ifndef OLD_GTK
 	gdk_draw_line (pixmap, gc, x - 4, y - 4, x + 4, y + 4);
 	gdk_draw_line (pixmap, gc, x - 4, y + 4, x + 4, y - 4);
-#else
-	GdkRectangle rect;
-
-	rect.x = x - 4;
-	rect.y = y - 4;
-	rect.width = rect.height = 8;
-	gdk_draw_line (pixmap, gc, rect.x, rect.y, x + 4, y + 4);
-	gdk_draw_line (pixmap, gc, rect.x, y + 4, x + 4, rect.y);
-	gtk_widget_draw (area, &rect);	
-#endif
     }
 }
 
-/* ............................................................. */
-
 static void 
 place_plots (PLOTGROUP *plotgrp)
      /* calculate placement of plots based on the number of them
@@ -415,8 +336,6 @@ place_plots (PLOTGROUP *plotgrp)
     }
 }
 
-/* ............................................................. */
-
 static void 
 gtk_boxplot_yscale (PLOTGROUP *grp, GtkPlotPC *pc)
 {
@@ -464,8 +383,6 @@ gtk_boxplot_yscale (PLOTGROUP *grp, GtkPlotPC *pc)
     g_free(numstr);
 }
 
-/* ............................................................. */
-
 static void 
 gtk_area_boxplot (BOXPLOT *plot, GtkWidget *area, GdkPixmap *pixmap,
 		  GtkStyle *style, GtkPlotPC *pc,
@@ -480,15 +397,9 @@ gtk_area_boxplot (BOXPLOT *plot, GtkWidget *area, GdkPixmap *pixmap,
     double conflo = 0., confhi = 0.;
     double nameoff = headroom / 4.0;
     GdkGC *gc = NULL;
-    /* GdkGC *whitegc = NULL; */
-#ifdef OLD_GTK
-    GdkRectangle rect;
-#endif
 
     if (pc == NULL) {
 	gc = style->fg_gc[GTK_STATE_NORMAL];
-	/* gc = style->bg_gc[GTK_STATE_SELECTED]; */
-	/* whitegc = style->fg_gc[GTK_STATE_SELECTED]; */
     }
 
     median = ybase + (gmax - plot->median) * scale;
@@ -516,21 +427,15 @@ gtk_area_boxplot (BOXPLOT *plot, GtkWidget *area, GdkPixmap *pixmap,
 	}
     }
 
-    /* no notches: draw simple inter-quartile box */
     if (confhi == 0.) {
-	if (pc != NULL) 
+	/* no notches: draw simple inter-quartile box */
+	if (pc != NULL) { 
 	    gtk_plot_pc_draw_rectangle (pc,
 					FALSE,
 					plot->xbase, uq,
 					boxwidth,
 					lq - uq);
-	else {
-#ifdef OLD_GTK
-	    rect.x = plot->xbase;
-	    rect.y = uq;
-	    rect.width = boxwidth;
-	    rect.height = uq - lq; 
-#endif
+	} else {
 	    gdk_draw_rectangle (pixmap, 
 				gc, 
 				FALSE, /* filled ? */
@@ -538,11 +443,9 @@ gtk_area_boxplot (BOXPLOT *plot, GtkWidget *area, GdkPixmap *pixmap,
 				uq, 
 				boxwidth, 
 				lq - uq);
-#ifdef OLD_GTK
-	    gtk_widget_draw (area, &rect);
-#endif
 	}
-    } else { /* draw notched boxes */
+    } else { 
+	/* draw notched boxes */
 	if (pc != NULL) {
 	    GtkPlotPoint points[10];
 
@@ -565,13 +468,6 @@ gtk_area_boxplot (BOXPLOT *plot, GtkWidget *area, GdkPixmap *pixmap,
 	} else {
 	    GdkPoint points[10];
 
-#ifdef OLD_GTK
-	    rect.x = plot->xbase;
-	    rect.y = uq;
-	    rect.width = boxwidth;
-	    rect.height = uq - lq; 
-#endif
-
 	    points[0].x = points[6].x = points[7].x = points[9].x = 
 		plot->xbase;
 	    points[0].y = points[1].y = uq;
@@ -589,9 +485,6 @@ gtk_area_boxplot (BOXPLOT *plot, GtkWidget *area, GdkPixmap *pixmap,
 			      FALSE, /* filled ? */
 			      points,
 			      10);
-#ifdef OLD_GTK
-	    gtk_widget_draw (area, &rect);
-#endif
 	}
     }
 
@@ -658,7 +551,9 @@ gtk_area_boxplot (BOXPLOT *plot, GtkWidget *area, GdkPixmap *pixmap,
     }
 
     /* write name of variable beneath */
-    if (plot->bool) nameoff = headroom / 3.5;
+    if (plot->bool) {
+	nameoff = headroom / 3.5;
+    }
     setup_text (area, pixmap, gc, pc, plot->varname, xcenter, 
 		height * (1.0 - nameoff), GTK_JUSTIFY_CENTER);
     if (plot->bool) {
@@ -667,8 +562,6 @@ gtk_area_boxplot (BOXPLOT *plot, GtkWidget *area, GdkPixmap *pixmap,
     }
 }
 
-/* ............................................................. */
-
 static void 
 destroy_boxplots (GtkWidget *w, gpointer data)
 {
@@ -679,28 +572,21 @@ destroy_boxplots (GtkWidget *w, gpointer data)
 	free(grp->plots[i].outliers);
 	free(grp->plots[i].bool);
     }
+
     free(grp->plots);
     free(grp->numbers);
-#ifndef OLD_GTK
     g_object_unref(G_OBJECT(grp->pixmap));
-#else
-    gdk_pixmap_unref(grp->pixmap);
-#endif
     free(grp);
-}
 
-/* ............................................................. */
+    remove(boxplottmp);
+}
 
 static GtkWidget *
 make_area (PLOTGROUP *grp)
 {
     grp->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     gtk_window_set_title(GTK_WINDOW(grp->window), _("gretl: boxplots"));
-#ifndef OLD_GTK
     gtk_window_set_resizable(GTK_WINDOW(grp->window), FALSE);
-#else
-    gtk_window_set_policy(GTK_WINDOW(grp->window), FALSE, FALSE, FALSE);
-#endif
 
     /* Create the drawing area */
     grp->area = gtk_drawing_area_new ();
@@ -717,7 +603,6 @@ make_area (PLOTGROUP *grp)
     g_signal_connect(G_OBJECT(grp->area), "expose_event",
 		     G_CALLBACK(expose_event), grp);
 
-#ifndef OLD_GTK
     g_signal_connect(G_OBJECT(grp->area), "button_press_event", 
 		     G_CALLBACK(box_popup), grp);
 
@@ -729,19 +614,6 @@ make_area (PLOTGROUP *grp)
 
     gtk_widget_set_size_request (GTK_WIDGET(grp->area),
 				 grp->width, grp->height); 
-#else
-    gtk_signal_connect(GTK_OBJECT(grp->window), "button_press_event", 
-		       GTK_SIGNAL_FUNC(box_popup), grp);
-
-    gtk_signal_connect(GTK_OBJECT(grp->window), "key_press_event", 
-		       GTK_SIGNAL_FUNC(box_key_handler), grp);
-
-    gtk_signal_connect(GTK_OBJECT(grp->window), "destroy",
-		       GTK_SIGNAL_FUNC(destroy_boxplots), grp);
-
-    gtk_drawing_area_size (GTK_DRAWING_AREA(grp->area), 
-			   grp->width, grp->height); 
-#endif
 
     gtk_widget_show (grp->area);
 
@@ -750,8 +622,6 @@ make_area (PLOTGROUP *grp)
     return grp->window;
 }
 
-/* ............................................................. */
-
 static int 
 compare_doubles (const void *a, const void *b)
 {
@@ -761,8 +631,6 @@ compare_doubles (const void *a, const void *b)
     return (*da > *db) - (*da < *db);
 }
 
-/* ............................................................. */
-
 static double 
 median (double *x, const int n)
 {
@@ -776,8 +644,6 @@ median (double *x, const int n)
     return xx;
 }
 
-/* ............................................................. */
-
 static double 
 quartiles (const double *x, const int n, BOXPLOT *box)
 {
@@ -800,8 +666,6 @@ quartiles (const double *x, const int n, BOXPLOT *box)
     return xx;
 }
 
-/* ............................................................. */
-
 static int 
 add_outliers (const double *x, const int n, BOXPLOT *box)
 {
@@ -843,8 +707,6 @@ add_outliers (const double *x, const int n, BOXPLOT *box)
     return 0;
 }
 
-/* ............................................................. */
-
 #define ITERS 560
 #define CONFIDENCE 90
 
@@ -890,16 +752,6 @@ median_interval (double *x, int n, double *low, double *high)
     return 0;
 }
 
-/* ............................................................. */
-
-#ifdef OLD_GTK
-/* At this point we need to borrow a few functions from gtkplotps.c,
-   which is part of gtkextra.  These functions are declared as
-   static in that context, but we want direct access to them. */
-
-#include "plotps.c"
-#endif
-
 int ps_print_plots (const char *fname, int flag, gpointer data) 
 {
     PLOTGROUP *grp = (PLOTGROUP *) data;
@@ -927,11 +779,10 @@ int ps_print_plots (const char *fname, int flag, gpointer data)
 	ps->page_height = pscale * grp->height;
     }
 
-#ifndef OLD_GTK
-    if (!psinit(GTK_PLOT_PC(ps))) return 1;
-#else
-    if (!psinit(ps)) return 1;
-#endif
+    if (!psinit(GTK_PLOT_PC(ps))) {
+	return 1;
+    }
+
     gtk_psfont_init();
 
     for (i=0; i<grp->nplots; i++)
@@ -942,21 +793,15 @@ int ps_print_plots (const char *fname, int flag, gpointer data)
     
     gtk_boxplot_yscale (grp, &ps->pc);
 
-#ifndef OLD_GTK
     psleave(GTK_PLOT_PC(ps));
-#else
-    psleave(ps);
-#endif
+
     gtk_object_destroy(GTK_OBJECT(ps));
     gtk_psfont_unref();
 
     return 0;
 }
 
-/* ............................................................. */
-
-static int
-five_numbers (gpointer data) 
+static int five_numbers (gpointer data) 
 {
     PLOTGROUP *grp = (PLOTGROUP *) data;
     int i;
@@ -1029,10 +874,8 @@ five_numbers (gpointer data)
 
 static void read_boxrc (PLOTGROUP *grp);
 
-/* ............................................................. */
-
 int boxplots (int *list, char **bools, double ***pZ, const DATAINFO *pdinfo, 
-	      int notches)
+	      gretlopt opt)
 {
     int i, j, n = pdinfo->t2 - pdinfo->t1 + 1;
     double *x;
@@ -1075,14 +918,16 @@ int boxplots (int *list, char **bools, double ***pZ, const DATAINFO *pdinfo,
 		continue;
 	    }
 	}
+
 	plotgrp->plots[i].outliers = NULL;
 	qsort(x, n, sizeof *x, compare_doubles);
 	plotgrp->plots[i].min = x[0];
 	plotgrp->plots[i].max = x[n-1];
 	quartiles(x, n, &plotgrp->plots[i]);
 	plotgrp->plots[i].n = n;
-	/* notched boxplots wanted? */
-	if (notches) {
+
+	if (opt & OPT_O) {
+	    /* notched boxplots wanted */
 	    if (median_interval(x, n, &plotgrp->plots[i].conf[0],
 				&plotgrp->plots[i].conf[1])) {
 		errbox (_("Couldn't obtain confidence interval"));
@@ -1092,7 +937,9 @@ int boxplots (int *list, char **bools, double ***pZ, const DATAINFO *pdinfo,
 	} else {
 	    plotgrp->plots[i].conf[0] = plotgrp->plots[i].conf[1] = NADBL;
 	}
+
 	strcpy(plotgrp->plots[i].varname, pdinfo->varname[list[i+1]]);
+
 	if (bools) { 
 	    plotgrp->plots[i].bool = bools[j];
 	} else {
@@ -1118,7 +965,7 @@ int boxplots (int *list, char **bools, double ***pZ, const DATAINFO *pdinfo,
 
     plotgrp->popup = NULL;
     plotgrp->pixmap = NULL;
-    place_plots (plotgrp);
+    place_plots(plotgrp);
 
     if (make_area(plotgrp) == NULL) {
 	free(plotgrp->plots);
@@ -1137,6 +984,10 @@ int boxplots (int *list, char **bools, double ***pZ, const DATAINFO *pdinfo,
 			  plotgrp->numbers);
     
     gtk_boxplot_yscale(plotgrp, NULL);
+
+    if (opt & OPT_B) {
+	dump_boxplot(plotgrp);
+    }
     
     return 0;
 }
@@ -1145,6 +996,12 @@ int boxplots (int *list, char **bools, double ***pZ, const DATAINFO *pdinfo,
 
 #ifdef G_OS_WIN32
 
+static void image_copy_err (const char *msg)
+{
+    fprintf(stderr, "%s\n", msg);
+    errbox(_("Failed to set clipboard data"));
+}
+
 static int cb_copy_image (gpointer data)
 {
     PLOTGROUP *grp = (PLOTGROUP *) data;
@@ -1169,7 +1026,7 @@ static int cb_copy_image (gpointer data)
 
     hDIB = GlobalAlloc(GMEM_MOVEABLE | GMEM_DDESHARE, dibsize);
     if (hDIB == NULL) {
-	errbox (_("Failed to allocate DIB"));
+	image_copy_err("Failed to allocate DIB");
 	return FALSE;
     }
 
@@ -1192,7 +1049,7 @@ static int cb_copy_image (gpointer data)
 	GlobalUnlock(hDIB);
 	ret = TRUE;
     } else {
-	errbox(_("Failed to lock DIB Header"));
+	image_copy_err("Failed to lock DIB Header");
     }
 
     /* fill color map */
@@ -1213,7 +1070,7 @@ static int cb_copy_image (gpointer data)
 	    ret = TRUE;
 	    GlobalUnlock(hDIB);
 	} else {
-	    errbox (_("Failed to lock DIB Palette"));
+	    image_copy_err("Failed to lock DIB Palette");
 	}
     } 
   
@@ -1246,35 +1103,36 @@ static int cb_copy_image (gpointer data)
 	    ret = TRUE;
 	    GlobalUnlock (hDIB);
 	} else {
-	    errbox(_("Failed to lock DIB Data"));
+	    image_copy_err("Failed to lock DIB Data");
 	}
     } /* copy data to DIB */
   
     /* copy DIB to ClipBoard */
     if (ret) {      
 	if (!OpenClipboard (NULL)) {
-	    errbox (_("Cannot open the Clipboard!"));
+	    image_copy_err("Cannot open the Clipboard!");
 	    ret = FALSE;
 	} else {
 	    if (ret && !EmptyClipboard ()) {
-		errbox (_("Cannot empty the Clipboard"));
+		image_copy_err("Cannot empty the Clipboard");
 		ret = FALSE;
 	    }
 	    if (ret) {
-		if (NULL != SetClipboardData (CF_DIB, hDIB))
+		if (NULL != SetClipboardData(CF_DIB, hDIB)) {
 		    hDIB = NULL; /* data now owned by clipboard */
-		else
-		    errbox (_("Failed to set clipboard data"));
+		} else {
+		    errbox(_("Failed to set clipboard data"));
+		}
 	    }
-	    if (!CloseClipboard ()) {
-		errbox (_("Failed to close Clipboard"));
+	    if (!CloseClipboard()) {
+		errbox(_("Failed to close Clipboard"));
 	    }
 	}
     }
 
     if (hDIB) GlobalFree(hDIB);
   
-    g_object_unref (G_OBJECT(image));
+    g_object_unref(G_OBJECT(image));
   
     return ret;
 } 
@@ -1319,11 +1177,7 @@ int plot_to_xpm (const char *fname, gpointer data)
 	fprintf(fp, "\"%s\n", (i<grp->height-1)? "," : "};");
     }
 
-#ifndef OLD_GTK
     g_object_unref(G_OBJECT(image));
-#else
-    gdk_image_destroy(image);
-#endif
 
     fclose(fp);
 
@@ -1413,7 +1267,7 @@ static int dump_boxplot (PLOTGROUP *grp)
     int i;
     BOXPLOT *plt;
 
-    build_path(paths.userdir, "boxdump.tmp", boxplottmp, NULL);
+    build_path(boxplottmp, paths.userdir, "boxdump.tmp", NULL);
 
     fp = gretl_fopen(boxplottmp, "w");
     if (fp == NULL) {
@@ -1464,6 +1318,7 @@ static int dump_boxplot (PLOTGROUP *grp)
     }
 
     fclose(fp);
+
     return 0;
 }
 
@@ -1697,7 +1552,7 @@ const char *get_boxplots_string (void)
 }
 
 int boolean_boxplots (const char *str, double ***pZ, DATAINFO *pdinfo, 
-		      int notches)
+		      gretlopt opt)
 {
     int i, k, v, nvars, nbool, err = 0;
     int n = pdinfo->n, origv = pdinfo->v;
@@ -1803,7 +1658,7 @@ int boolean_boxplots (const char *str, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (!err) {
-	err = boxplots(list, bools, pZ, pdinfo, notches);
+	err = boxplots(list, bools, pZ, pdinfo, opt);
     } 
     
     free(list);
