@@ -33,10 +33,6 @@
 # include <windows.h>
 #endif
 
-#if GTK_MAJOR_VERSION < 2
-# define OLD_GTK
-#endif
-
 typedef struct GretlChild_ GretlChild;
 typedef struct test_t_ test_t;
 typedef struct lookup_t_ lookup_t;
@@ -252,9 +248,9 @@ static void get_critical (GtkWidget *w, gpointer data)
 	case F_DIST:
 	    pprintf(prn, _("Approximate critical values of F(%d, %d)\n\n"),
 		    df, n);
-	    pprintf(prn, _(" 10%% in right tail %.2f\n"), f_crit_a(.10, df, n));
-	    pprintf(prn, "  5%%               %.2f\n", f_crit_a(.05, df, n));	
-	    pprintf(prn, "  1%%               %.2f\n", f_crit_a(.01, df, n));
+	    pprintf(prn, _(" 10%% in right tail %.2f\n"), f_critval(.10, df, n));
+	    pprintf(prn, "  5%%               %.2f\n", f_critval(.05, df, n));	
+	    pprintf(prn, "  1%%               %.2f\n", f_critval(.01, df, n));
 	    break;
 
 	case DW_DIST:
@@ -300,7 +296,7 @@ static int do_binomial_pdf (const char *s, PRN *prn)
     }
 
     x1 = binomial_cdf(k, n, p);
-    x2 = binomial_pvalue(k, n, p);
+    x2 = binomial_cdf_comp(k, n, p);
 
     if (na(x1) || na(x2)) {
 	err = 1;
@@ -311,7 +307,9 @@ static int do_binomial_pdf (const char *s, PRN *prn)
 	pputs(prn, "\n\n");
 	pprintf(prn, " P(x <= %d) = %g\n", k, x1);
 	pprintf(prn, " P(x > %d)  = %g\n", k, x2);
-	pprintf(prn, " P(x = %d)  = %g", k, binomial_cdf(k+1, n, p) - x1);
+	if (k > 0) {
+	    pprintf(prn, " P(x = %d)  = %g", k, x1 - binomial_cdf(k-1, n, p));
+	}
     }
 
     return err;
@@ -440,32 +438,14 @@ static void print_pv (PRN *prn, double p1, double p2)
 	    p1, p2);
 }
 
-/* v. rough 99.9 percent critical value of chi-square */
-
-static double chi_crit (const int df)
-{
-    double x = 10.0;
-    
-    while (chisq(x, df) > .001) x += .5;
-    return x;
-}
-
-/* v. rough 99.9 percent critical value of F */
-
-static double f_crit (const int df1, const int df2)
-{
-    double x = 2.0;
-
-    while (fdist(x, df1, df2) > .001) x += .5;
-    return x;
-}
-
 static void htest_graph (int dist, double x, int df1, int df2)
 {
     double xx, prange, spike = 0.0;
     FILE *fp = NULL;
 
-    if (gnuplot_init(PLOT_SAMPLING_DIST, &fp)) return;
+    if (gnuplot_init(PLOT_SAMPLING_DIST, &fp)) {
+	return;
+    }
 
     fprintf(fp, "set key right top\n");
 
@@ -479,9 +459,11 @@ static void htest_graph (int dist, double x, int df1, int df2)
 	fprintf(fp, "set yrange [0:.50]\n");
 	fprintf(fp, "set xlabel '%s'\n", I_("Standard errors"));
     } else if (dist == CHISQ_DIST || dist == F_DIST) {
-	prange = (dist == CHISQ_DIST)? chi_crit(df1) : f_crit(df1, df2);
-	if (x > prange) 
+	prange = (dist == CHISQ_DIST)? chisq_critval(0.001, df1) : 
+	    f_critval(0.001, df1, df2);
+	if (x > prange) {
 	    prange = 1.1 * x;
+	}
 	spike = 1.0 / prange;
 	fprintf(fp, "set xrange [0:%.3f]\n", prange);
     } 
@@ -573,83 +555,125 @@ static void h_test (GtkWidget *w, gpointer data)
 	    /* get sample mean and std dev */
 	    tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[j]));
 	    x[j] = getval(tmp, prn, (j==1)? 1 : 0);
-	    if (na(x[j])) return;
+	    if (na(x[j])) {
+		return;
+	    }
 	}
+
 	tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[2]));
-	if ((n1 = getint(tmp, prn)) == -1) return;
+	if ((n1 = getint(tmp, prn)) == -1) {
+	    return;
+	}
+
 	tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[3]));
 	x[2] = getval(tmp, prn, 0);
-	if (na(x[2])) return;
+	if (na(x[2])) {
+	    return;
+	}
+
 	sderr = x[1] / sqrt((double) n1);
 	ts = (x[0] - x[2])/sderr;
+
 	pprintf(prn, _("Null hypothesis: population mean = %g\n"), x[2]);
 	pprintf(prn, _("Sample size: n = %d\n"), n1);
 	pprintf(prn, _("Sample mean = %g, std. deviation = %g\n"), 
 		x[0], x[1]);
+
 	if (GTK_TOGGLE_BUTTON(test->check)->active) {
 	    pprintf(prn, _("Test statistic: z = (%g - %g)/%g = %g\n"), 
 		    x[0], x[2], sderr, ts);
 	    pv = normal_pvalue_2(ts);
 	    print_pv(prn, pv, pv / 2.0);
-	    if (grf) htest_graph(0, ts, 0, 0);
+	    if (grf) {
+		htest_graph(0, ts, 0, 0);
+	    }
 	} else {
 	    pprintf(prn, _("Test statistic: t(%d) = (%g - %g)/%g = %g\n"), n1-1,
 		    x[0], x[2], sderr, ts);
 	    pv = t_pvalue_2(ts, n1 - 1);
 	    print_pv(prn, pv, 0.5 * pv);
-	    if (grf) htest_graph(1, ts, n1-1, 0);
+	    if (grf) {
+		htest_graph(1, ts, n1-1, 0);
+	    }
 	}
 	break;
 
     case ONE_VARIANCE:
 	tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[0]));
 	x[0] = getval(tmp, prn, 1);
-	if (na(x[0])) return;
+	if (na(x[0])) {
+	    return;
+	}
+
 	tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[1]));
-	if ((n1 = getint(tmp, prn)) == -1) return;
+	if ((n1 = getint(tmp, prn)) == -1) {
+	    return;
+	}
+
 	tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[2]));
 	x[1] = getval(tmp, prn, 1);
-	if (na(x[1])) return;
+	if (na(x[1])) {
+	    return;
+	}
+
 	ts = (n1 - 1) * x[0] / x[1];
+
 	pprintf(prn, _("Null hypothesis: population variance = %g\n"), x[1]);
 	pprintf(prn, _("Sample size: n = %d\n"), n1);
 	pprintf(prn, _("Sample variance = %g\n"), x[0]);
 	pprintf(prn, _("Test statistic: chi-square(%d) = %d * %g/%g = %g\n"), 
 		n1-1, n1-1, x[0], x[1], ts);
+
 	if (x[0] > x[1]) {
-	    pv = chisq(ts, n1 - 1);
+	    pv = chisq_cdf_comp(ts, n1 - 1);
 	} else {
-	    pv = 1.0 - chisq(ts, n1 -1);
+	    pv = chisq_cdf(ts, n1 - 1);
 	}
 	print_pv(prn, 2.0 * pv, pv);
-	if (grf) htest_graph(2, ts, n1 - 1, 0);
+	if (grf) {
+	    htest_graph(2, ts, n1 - 1, 0);
+	}
 	break;
 
     case ONE_PROPN: /* proportion */
 	tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[0]));
 	x[0] = getval(tmp, prn, 1);
-	if (na(x[0])) return;
+	if (na(x[0])) {
+	    return;
+	}
+
 	tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[1]));
-	if ((n1 = getint(tmp, prn)) == -1) return;
+	if ((n1 = getint(tmp, prn)) == -1) {
+	    return;
+	}
+
 	tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[2]));
 	x[1] = getval(tmp, prn, 1);
-	if (na(x[1])) return;
+	if (na(x[1])) {
+	    return;
+	}
+
 	if (n1 * x[1] < 5.0 || n1 * (1.0 - x[1]) < 5.0) {
 	    infobox(_("The assumption of a normal sampling distribution\n"
 		      "is not justified here.  Abandoning the test."));
 	    gretl_print_destroy(prn);
 	    return;
 	}
+
 	sderr = sqrt(x[1] * (1.0 - x[1]) / n1);
 	ts = (x[0] - x[1]) / sderr;
+
 	pprintf(prn, _("Null hypothesis: population proportion = %g\n"), x[1]);
 	pprintf(prn, _("Sample size: n = %d\n"), n1);
 	pprintf(prn, _("Sample proportion = %g\n"), x[0]);
 	pprintf(prn, _("Test statistic: z = (%g - %g)/%g = %g\n"), 
 		x[0], x[1], sderr, ts);
+
 	pv = normal_pvalue_2(ts);
 	print_pv(prn, pv, pv / 2.0);
-	if (grf) htest_graph(0, ts, 0, 0);
+	if (grf) {
+	    htest_graph(0, ts, 0, 0);
+	}
 	break;
 
     case TWO_MEANS:
@@ -657,28 +681,44 @@ static void h_test (GtkWidget *w, gpointer data)
 	    /* mean and std dev, sample 1 */
 	    tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[j]));
 	    x[j] = getval(tmp, prn, (j==1)? 1 : 0);
-	    if (na(x[j])) return;
+	    if (na(x[j])) {
+		return;
+	    }
 	}
+
 	tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[2]));
-	if ((n1 = getint(tmp, prn)) == -1) return;
+	if ((n1 = getint(tmp, prn)) == -1) {
+	    return;
+	}
+
 	for (j=2; j<4; j++) {
 	    /* mean and std dev, sample 2 */
 	    tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[j+1]));
 	    x[j] = getval(tmp, prn, (j==3)? 1 : 0);
-	    if (na(x[j])) return;
+	    if (na(x[j])) {
+		return;
+	    }
 	}
+
 	tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[5]));
-	if ((n2 = getint(tmp, prn)) == -1) return;
+	if ((n2 = getint(tmp, prn)) == -1) {
+	    return;
+	}
+
 	tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[6]));
 	x[4] = getval(tmp, prn, 0);
-	if (na(x[4])) return;
+	if (na(x[4])) {
+	    return;
+	}
 
 	pprintf(prn, _("Null hypothesis: Difference of means = %g\n"), x[4]);
 	pputc(prn, '\n');
 
 	pprintf(prn, _("Sample 1:\n n = %d, mean = %g, s.d. = %g\n"),
 		n1, x[0], x[1]);
+
 	z = x[1] / sqrt((double) n1);
+
 	pprintf(prn, _(" standard error of mean = %g\n"), z);
 	z *= tcrit95(n1 - 1);
 	pprintf(prn, _(" 95%% confidence interval for mean: %g to %g\n"), 
@@ -687,25 +727,28 @@ static void h_test (GtkWidget *w, gpointer data)
 
 	pprintf(prn, _("Sample 2:\n n = %d, mean = %g, s.d. = %g\n"),
 		n2, x[2], x[3]);
+
 	z = x[3] / sqrt((double) n2);
+
 	pprintf(prn, _(" standard error of mean = %g\n"), z);
 	z *= tcrit95(n2 - 1);
 	pprintf(prn, _(" 95%% confidence interval for mean: %g to %g\n"), 
 		x[2] - z, x[2] + z);
 	pputc(prn, '\n');
 
-	/* are we assuming a common variance? */
-	j = 0;
 	if (GTK_TOGGLE_BUTTON(test->check)->active) {
+	    /* the user specified a common variance */
 	    j = 1;
-	}
-	if (n1 < 30 || n2 < 30) {
+	} else if (n1 < 30 || n2 < 30) {
+	    /* flag for warning: unequal variances and small samples */
 	    j = 2;
+	} else {
+	    j = 0;
 	}
 
-	if (j > 0) {
-	    ts = ((n1-1)*x[1]*x[1] + (n2-1)*x[3]*x[3])/(n1 + n2 - 2);
-	    sderr = sqrt(ts / n1 + ts /n2);
+	if (j == 1) {
+	    ts = ((n1-1) * x[1] * x[1] + (n2-1) * x[3] * x[3]) / (n1 + n2 - 2);
+	    sderr = sqrt(ts / n1 + ts / n2);
 	} else {
 	    double v1 = x[1] * x[1] / n1;
 	    double v2 = x[3] * x[3] / n2;
@@ -714,10 +757,8 @@ static void h_test (GtkWidget *w, gpointer data)
 	}
 
 	ts = (x[0] - x[2] - x[4]) / sderr;
-	if (j) {
-	    if (j == 2)
-		pprintf(prn, _("Small samples: assuming normality and common "
-			       "variance\n"));
+
+	if (j == 1) {
 	    pprintf(prn, _("Test statistic: t(%d) = (%g - %g)/%g = %g\n"),
 		    n1 + n2 - 2, x[0], x[2], sderr, ts);
 	    if (ts > 0) {
@@ -726,7 +767,9 @@ static void h_test (GtkWidget *w, gpointer data)
 		pv = t_pvalue_2(-ts, n1 + n2 - 2);
 	    }
 	    print_pv(prn, pv, 0.5 * pv);
-	    if (grf) htest_graph(1, ts, n1 + n2 - 2, 0);
+	    if (grf) {
+		htest_graph(1, ts, n1 + n2 - 2, 0);
+	    }
 	} else {
 	    if (x[4] > 0.0) {
 		pprintf(prn, _("Test statistic: z = (%g - %g - %g)/%g = %g\n"),
@@ -740,53 +783,87 @@ static void h_test (GtkWidget *w, gpointer data)
 	    }
 	    pv = normal_pvalue_2(ts);
 	    print_pv(prn, pv, pv / 2.0);
-	    if (grf) htest_graph(0, ts, 0, 0);
+	    if (grf) {
+		htest_graph(0, ts, 0, 0);
+	    }
 	}
+	
+	if (j == 2) {
+	    pputc(prn, '\n');
+	    pprintf(prn, _("Warning: with small samples, asymptotic "
+			   "approximation may be poor\n"));
+	}
+
 	break;
 
     case TWO_VARIANCES:
 	tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[0]));
 	x[0] = getval(tmp, prn, 1);
-	if (na(x[0])) return;
+	if (na(x[0])) {
+	    return;
+	}
+
 	tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[1]));
-	if ((n1 = getint(tmp, prn)) == -1) return;
+	if ((n1 = getint(tmp, prn)) == -1) {
+	    return;
+	}
+
 	tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[2]));
 	x[1] = getval(tmp, prn, 1);
-	if (na(x[1])) return;
+	if (na(x[1])) {
+	    return;
+	}
+
 	tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[3]));
-	if ((n2 = getint(tmp, prn)) == -1) return;
+	if ((n2 = getint(tmp, prn)) == -1) {
+	    return;
+	}
+
 	pprintf(prn, _("Null hypothesis: The population variances are "
 		       "equal\n"));
 	pprintf(prn, _("Sample 1:\n n = %d, variance = %g\n"), n1, x[0]);
 	pprintf(prn, _("Sample 2:\n n = %d, variance = %g\n"), n2, x[1]);
+
 	if (x[0] > x[1]) {
 	    ts = x[0] / x[1];
 	    pprintf(prn, _("Test statistic: F(%d, %d) = %g\n"), 
 		    n1 - 1, n2 - 1, ts);
-	    pv = fdist(ts, n1 - 1, n2 - 1);
+	    pv = f_cdf_comp(ts, n1 - 1, n2 - 1);
 	} else {
-	    ts = x[1]/x[0];
+	    ts = x[1] / x[0];
 	    pprintf(prn, _("Test statistic: F(%d, %d) = %g\n"), 
 		    n2 - 1, n1 - 1, ts);
-	    pv = fdist(ts, n2 - 1, n1 - 1);
+	    pv = f_cdf_comp(ts, n2 - 1, n1 - 1);
 	}
+
 	print_pv(prn, 2.0 * pv, pv);
-	if (grf) htest_graph(3, ts, n1 - 1, n2 - 1);
+	if (grf) {
+	    htest_graph(3, ts, n1 - 1, n2 - 1);
+	}
 	break;
 
     case TWO_PROPNS: /* two proportions */
 	tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[0]));
 	x[0] = getval(tmp, prn, 1);
-	if (na(x[0])) return;
+	if (na(x[0])) {
+	    return;
+	}
 
 	tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[1]));
-	if ((n1 = getint(tmp, prn)) == -1) return;
+	if ((n1 = getint(tmp, prn)) == -1) {
+	    return;
+	}
+
 	tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[2]));
 	x[1] = getval(tmp, prn, 1);
-	if (na(x[1])) return;
+	if (na(x[1])) {
+	    return;
+	}
 
 	tmp = gtk_entry_get_text(GTK_ENTRY(test->entry[3]));
-	if ((n2 = getint(tmp, prn)) == -1) return;
+	if ((n2 = getint(tmp, prn)) == -1) {
+	    return;
+	}
 
 	pprintf(prn, _("Null hypothesis: the population proportions are "
 		       "equal\n"));
@@ -795,14 +872,19 @@ static void h_test (GtkWidget *w, gpointer data)
 	pputc(prn, '\n');
 	pprintf(prn, _("Sample 2:\n n = %d, proportion = %g\n"), n2, x[1]);
 	pputc(prn, '\n');
+
 	x[2] = (n1*x[0] + n2*x[1]) / (n1 + n2);
 	sderr = sqrt((x[2] * (1.0-x[2])) * (1.0/n1 + 1.0/n2));
 	ts = (x[0] - x[1]) / sderr;
+
 	pprintf(prn, _("Test statistic: z = (%g - %g) / %g = %g\n"),
 		x[0], x[1], sderr, ts);
+
 	pv = normal_pvalue_2(ts);
 	print_pv(prn, pv, pv / 2.0);
-	if (grf) htest_graph(0, ts, 0, 0);
+	if (grf) {
+	    htest_graph(0, ts, 0, 0);
+	}
 	break;
 
     default:
@@ -1150,7 +1232,7 @@ static void populate_stats (GtkWidget *w, gpointer p)
     }
 
     /* scalars are not valid input in this context */
-    if (!datainfo->vector[vx] || (vy > 0 && !datainfo->vector[vy])) {
+    if (var_is_scalar(datainfo, vx) || (vy > 0 && var_is_scalar(datainfo, vy))) {
 	errbox(_("Invalid entry"));
 	return;
     }
@@ -1211,9 +1293,9 @@ static int var_is_ok (int i, int code)
 {
     int ret = 1;
 
-    if (is_hidden_variable(i, datainfo)) {
+    if (var_is_hidden(datainfo, i)) {
 	ret = 0;
-    } else if (!datainfo->vector[i]) {
+    } else if (var_is_scalar(datainfo, i)) {
 	ret = 0;
     } else if ((code == ONE_PROPN || code == TWO_PROPNS) &&
 	       !gretl_isdummy(datainfo->t1, datainfo->t2, Z[i])) {
@@ -1368,13 +1450,15 @@ static void add_test_label (GtkWidget *tbl, gint *tbl_len,
 }
 
 static void add_test_check (GtkWidget *tbl, gint *tbl_len, 
-			    const gchar *label, test_t *test)
+			    const gchar *label, test_t *test,
+			    gboolean val)
 {
     GtkWidget *tempwid;
 
     *tbl_len += 1;
     gtk_table_resize(GTK_TABLE(tbl), *tbl_len, 2);
     tempwid = gtk_check_button_new_with_label(label);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tempwid), val);
     gtk_table_attach_defaults(GTK_TABLE(tbl), 
 			      tempwid, 0, 2, *tbl_len - 1, *tbl_len);
     gtk_widget_show(tempwid);
@@ -1387,7 +1471,7 @@ static int n_ok_series (void)
 
     if (datainfo != NULL) {
 	for (i=1; i<datainfo->v; i++) {
-	    if (datainfo->vector[i] && !is_hidden_variable(i, datainfo)) {
+	    if (var_is_series(datainfo, i) && !var_is_hidden(datainfo, i)) {
 		nv++;
 	    }
 	}
@@ -1402,8 +1486,8 @@ static int n_ok_dummies (void)
 
     if (datainfo != NULL) {
 	for (i=1; i<datainfo->v; i++) {
-	    if (datainfo->vector[i] && 
-		!is_hidden_variable(i, datainfo) &&
+	    if (var_is_series(datainfo, i) && 
+		!var_is_hidden(datainfo, i) &&
 		gretl_isdummy(datainfo->t1, datainfo->t2, Z[i])) {
 		nv++;
 	    }
@@ -1470,7 +1554,7 @@ static void make_test_tab (GtkWidget *notebook, int code, test_t *test)
 	add_test_entry(tbl, &tbl_len, _("sample size"), test, 2);
 	add_test_entry(tbl, &tbl_len, _("H0: mean ="), test, 3);
 	add_test_check(tbl, &tbl_len, _("Assume standard deviation is "
-		       "population value"), test);
+		       "population value"), test, FALSE);
 	break;
 
     case ONE_VARIANCE: 
@@ -1498,7 +1582,7 @@ static void make_test_tab (GtkWidget *notebook, int code, test_t *test)
 	add_test_entry(tbl, &tbl_len, _("H0: Difference of means ="), test, 6);
 	gtk_entry_set_text(GTK_ENTRY(test->entry[6]), "0");
 	add_test_check(tbl, &tbl_len, _("Assume common population standard "
-		       "deviation"), test);
+		       "deviation"), test, TRUE);
 	break;
 
     case TWO_VARIANCES:
@@ -1552,7 +1636,7 @@ static void gretl_child_destroy (GtkWidget *w, gpointer data)
 static GretlChild *gretl_child_new (const gchar *title)
 {
     GretlChild *gchild;
-    GtkWidget *base, *hsep;
+    GtkWidget *base;
 
     gchild = mymalloc(sizeof *gchild);
     if (gchild == NULL) return NULL;
@@ -1570,11 +1654,11 @@ static GretlChild *gretl_child_new (const gchar *title)
     gtk_container_add(GTK_CONTAINER(base), gchild->vbox);
     gtk_container_set_border_width(GTK_CONTAINER(gchild->vbox), 5);
 
-    hsep = gtk_hseparator_new();
-    gtk_widget_show(hsep);
-    gtk_container_add(GTK_CONTAINER(base), hsep);
-    
-    gchild->action_area = gtk_hbox_new(TRUE, 5);
+    gchild->action_area = gtk_hbutton_box_new();
+    gtk_button_box_set_layout(GTK_BUTTON_BOX(gchild->action_area), 
+			      GTK_BUTTONBOX_END);
+    gtk_button_box_set_spacing(GTK_BUTTON_BOX(gchild->action_area),
+			       10);
     gtk_widget_show(gchild->action_area);
     gtk_container_add(GTK_CONTAINER(base), gchild->action_area);
     gtk_container_set_border_width(GTK_CONTAINER(gchild->action_area), 5);
@@ -1664,11 +1748,11 @@ void stats_calculator (gpointer data, guint code, GtkWidget *widget)
 	}
     }	
 
-    /* OK button */
-    tempwid = standard_button(GTK_STOCK_OK);
+    /* Apply button */
+    tempwid = standard_button(GTK_STOCK_APPLY);
     GTK_WIDGET_SET_FLAGS(tempwid, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(dialog->action_area), 
-		       tempwid, TRUE, TRUE, 0);
+    gtk_container_add(GTK_CONTAINER(dialog->action_area), 
+		      tempwid);
 
     g_signal_connect(G_OBJECT (tempwid), "clicked", 
 		     (code == CALC_PVAL)? G_CALLBACK(get_pvalue) :
@@ -1680,8 +1764,8 @@ void stats_calculator (gpointer data, guint code, GtkWidget *widget)
     /* Close button */
     tempwid = standard_button(GTK_STOCK_CLOSE);
     GTK_WIDGET_SET_FLAGS(tempwid, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(dialog->action_area), 
-		       tempwid, TRUE, TRUE, 0);
+    gtk_container_add(GTK_CONTAINER(dialog->action_area), 
+		      tempwid);
 
     g_signal_connect(G_OBJECT(tempwid), "clicked", 
 		     (code == CALC_PVAL)? G_CALLBACK(trash_pval) :
@@ -1697,8 +1781,10 @@ void stats_calculator (gpointer data, guint code, GtkWidget *widget)
     if (code == CALC_TEST) {
 	tempwid = standard_button(GTK_STOCK_HELP);
 	GTK_WIDGET_SET_FLAGS(tempwid, GTK_CAN_DEFAULT);
-	gtk_box_pack_start(GTK_BOX(dialog->action_area), 
-			   tempwid, TRUE, TRUE, 0);
+	gtk_container_add(GTK_CONTAINER(dialog->action_area), 
+			  tempwid);
+	gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(dialog->action_area),
+					   tempwid, TRUE);
 	g_signal_connect(G_OBJECT(tempwid), "clicked", 
 			 G_CALLBACK(context_help), 
 			 GINT_TO_POINTER(HTEST));
