@@ -24,18 +24,13 @@
 #include "session.h"
 #include "database.h"
 #include "datafiles.h"
-#include "ssheet.h"
 #include "textbuf.h"
 #include "textutil.h"
 #include "boxplots.h"
 #include "dlgutils.h"
 #include "fileselect.h"
-
-#ifdef OLD_GTK
-# include <gtkextra/gtkiconfilesel.h>
-#else
-# include "treeutils.h"
-#endif
+#include "ssheet.h"
+#include "treeutils.h"
 
 /* these live in dialogs.c */
 extern GtkWidget *active_edit_id;
@@ -54,15 +49,21 @@ static void doubleclick_action (windata_t *win)
     case PS_FILES:
 	browser_open_ps(NULL, win);
 	break;
+    case FUNC_FILES:
+	browser_load_func(NULL, win);
+	break;
+    case FUNC_EDIT:
+	browser_edit_func(NULL, win);
+	break;
     case NATIVE_DB:
-    case RATS_DB:	    
-	open_db_list(NULL, win); 
+	open_db_index(NULL, win); 
 	break;
     case REMOTE_DB:
-	open_remote_db_list(NULL, win);
+	open_remote_db_index(NULL, win);
 	break;
     case NATIVE_SERIES:
     case RATS_SERIES:
+    case PCGIVE_SERIES:
     case REMOTE_SERIES:
 	gui_get_db_series(win, 0, NULL);
 	break;
@@ -71,8 +72,6 @@ static void doubleclick_action (windata_t *win)
     }
 }
 
-#ifndef OLD_GTK
-
 void listbox_select_row (GtkTreeSelection *selection, gpointer data)
 {
     windata_t *win = (windata_t *) data;
@@ -151,64 +150,6 @@ gboolean listbox_drag (GtkWidget *listbox, GdkEventMotion *event,
     return FALSE;
 }
 
-#else /* now comes an old gtk function */
-
-void selectrow (GtkCList *clist, gint row, gint column, 
-	        GdkEventButton *event, gpointer data) 
-{
-    gchar *numstr, *edttext;
-    windata_t *win = (windata_t *) data;
-
-    if (win == mdata) { /* main window */
-	gtk_clist_get_text(clist, row, 0, &numstr);
-	win->active_var = atoi(numstr);
-    } else {
-	win->active_var = row;
-    }
-
-    if (active_edit_id != NULL) {
-	gchar addvar[VNAMELEN];
-
-	edttext = gtk_entry_get_text(GTK_ENTRY(active_edit_id));
-	if (*edttext != '\0') {
-	    sprintf(addvar, " %d", win->active_var);
-	} else {
-	    sprintf(addvar, "%d", win->active_var);
-	}
-	gtk_entry_append_text(GTK_ENTRY(active_edit_id), addvar);
-    } else if (active_edit_name != NULL) {
-	edttext = gtk_entry_get_text (GTK_ENTRY(active_edit_name));
-	gtk_entry_append_text(GTK_ENTRY(active_edit_name), 
-			      datainfo->varname[win->active_var]);
-	gtk_entry_append_text(GTK_ENTRY(active_edit_name), " ");
-    }
-
-    /* response to double-click */
-    if (event != NULL && event->type == GDK_2BUTTON_PRESS 
-	&& event->button == 1) {
-	doubleclick_action(win);
-    }
-}
-
-void unselectrow (GtkCList *clist, gint row, gint column, 
-		  GdkEventButton *event, gpointer data) 
-{
-    windata_t *win = (windata_t *) data;
-
-    if (win != mdata) { /* main window */
-	return;
-    } else {
-	gchar *numstr;
-
-	gtk_clist_get_text(clist, row, 0, &numstr);
-	if (win->active_var == atoi(numstr)) {
-	    win->active_var = clist->focus_row;
-	}
-    }
-}
-
-#endif /* old versus new gtk */
-
 void open_data (gpointer data, guint code, GtkWidget *widget)
 {
     if (dataset_locked()) {
@@ -250,9 +191,17 @@ void open_data (gpointer data, guint code, GtkWidget *widget)
     case APPEND_DTA:
 	file_selector(_("Open Stata file"), code, FSEL_DATA_NONE, NULL);
 	break;
+    case OPEN_JMULTI:
+    case APPEND_JMULTI:
+	file_selector(_("Open JMulTi file"), code, FSEL_DATA_NONE, NULL);
+	break;
     case OPEN_MARKERS:
 	file_selector(_("gretl: add markers"), code, FSEL_DATA_NONE, NULL);
 	break;
+    case OPEN_RATS_DB:
+    case OPEN_PCGIVE_DB:
+	file_selector(_("gretl: open database"), code, FSEL_DATA_NONE, NULL);
+	break;
     default:
 	errbox("Unrecognized data code");
 	break;
@@ -286,10 +235,8 @@ void file_save (gpointer data, guint file_code, GtkWidget *widget)
 	break;
     case SAVE_DATA:
     case SAVE_DATA_AS:
-    case SAVE_GZDATA:
-    case SAVE_BIN1:
-    case SAVE_BIN2:
     case SAVE_DBDATA:
+    case SAVE_FUNCTIONS:	
 	data_save_selection_wrapper(file_code, NULL);
 	break;
     case EXPORT_CSV:
@@ -298,6 +245,7 @@ void file_save (gpointer data, guint file_code, GtkWidget *widget)
     case EXPORT_R:
     case EXPORT_OCTAVE:
     case EXPORT_DAT:
+    case EXPORT_JM:
 	data_save_selection_wrapper(file_code, p);
 	break;
     case SAVE_TEX:
@@ -377,20 +325,13 @@ void model_callback (gpointer data, guint model_code, GtkWidget *widget)
 		     presel);
 }
 
-#ifdef ENABLE_GMP
-void mp_ols_callback (gpointer data, guint model_code, GtkWidget *widget)
-{
-    selection_dialog(_("gretl: specify model"), do_mp_ols, model_code, 0);
-}
-#endif /* ENABLE_GMP */
-
 void model_genr_callback (gpointer data, guint u, GtkWidget *widget)
 {
     windata_t *mydata = (windata_t *) data;
 
     edit_dialog(_("gretl: add var"), _("Enter formula for new variable:"),
 		"", do_model_genr, mydata, 
-		MODEL_GENR, VARCLICK_INSERT_NAME);   
+		MODEL_GENR, VARCLICK_INSERT_NAME, NULL);   
 }
 
 void selector_callback (gpointer data, guint action, GtkWidget *widget)
@@ -443,7 +384,9 @@ void selector_callback (gpointer data, guint action, GtkWidget *widget)
     } else if (action == ELLIPSE) {
 	simple_selection(_("gretl: model tests"), do_confidence_region, action, vwin);
     } else if (action == GR_PLOT) {
-	simple_selection(_("gretl: define graph"), do_graph_from_selector, action, vwin);
+	simple_selection(_("gretl: define graph"), do_graph_from_selector, action, NULL);
+    } else if (action == TSPLOTS) {
+	simple_selection(_("gretl: define graph"), do_scatters, action, vwin);
     } else if (action == SPEARMAN) {
 	strcat(title, _("rank correlation"));
 	simple_selection(title, do_spearman, action, vwin);
@@ -473,18 +416,6 @@ void gretl_callback (gpointer data, guint action, GtkWidget *widget)
 	okfunc = do_samplebool;
 	varclick = VARCLICK_INSERT_NAME;
 	break;
-    case SETSEED:
-	title = N_("gretl: random variables");
-	query = N_("Enter integer seed for\n"
-	       "pseudo-random number generator:");
-	okfunc = do_seed;
-	break; 
-    case NULLDATA:
-	title = N_("gretl: simulation data");
-	query = N_("Series length for simulation data set:");
-	defstr = "100";
-	okfunc = do_simdata;
-	break;         
     case GENR:
 	title = N_("gretl: add var");
 	query = N_("Enter formula for new variable:");
@@ -543,7 +474,7 @@ void gretl_callback (gpointer data, guint action, GtkWidget *widget)
     }
 
     edit_dialog(_(title), _(query), defstr, okfunc, data, 
-		action, varclick);   
+		action, varclick, NULL);   
 }
 
 void run_script_callback (GtkWidget *w, gpointer data)
@@ -568,6 +499,7 @@ void file_save_callback (GtkWidget *w, gpointer data)
 	case EDIT_SCRIPT:
 	case VIEW_SCRIPT:
 	case VIEW_LOG:
+	case VIEW_FUNC_CODE:
 	    u = SAVE_SCRIPT;
 	    break;
 	case GR_PLOT:
@@ -582,128 +514,59 @@ void file_save_callback (GtkWidget *w, gpointer data)
     file_save(data, u, w);
 }
 
-void add_random_callback (gpointer data, guint code, GtkWidget *widget) 
+void add_rand_callback (gpointer data, guint r, GtkWidget *widget) 
 {
-    if (code == GENR_UNIFORM) {
+    if (r == RANDOM_UNIFORM) {
 	edit_dialog (_("gretl: uniform variable"), 
 		     _("Enter name for variable, and\n"
 		       "minimum and maximum values:"), 
 		     "unif 0 1",  
-		     do_random, NULL, 
-		     GENR_UNIFORM, GENR);
-    } else if (code == GENR_NORMAL) {
+		     do_random_uniform, NULL, 
+		     GENR_RANDOM, VARCLICK_NONE, NULL);
+    } else if (r == RANDOM_NORMAL) {
 	edit_dialog (_("gretl: normal variable"), 
 		     _("Enter name, mean and standard deviation:"), 
 		     "norm 0 1", 
-		     do_random, NULL, 
-		     GENR_NORMAL, GENR);
-    }
-}
-
-static void name_first_ssheet_var (GtkWidget *widget, dialog_t *dlg) 
-{
-    const gchar *buf;
-
-    buf = edit_dialog_get_text(dlg);
-
-    if (buf == NULL || validate_varname(buf)) {
-	return;
-    }
-
-    datainfo->varname[1][0] = 0;
-    strncat(datainfo->varname[1], buf, VNAMELEN - 1);
-
-    close_dialog(dlg);
-
-    show_spreadsheet(SHEET_NEW_DATASET);
-}
-
-static int prep_spreadsheet (const char *dataspec)
-{
-    char stobs[OBSLEN], endobs[OBSLEN];
-    int t1, t2;
-
-    if (sscanf(dataspec, "%10s %10s", stobs, endobs) != 2) {
-	errbox(_("Insufficient dataset information supplied"));
-	return 1;
-    }
-
-    t1 = dateton(stobs, datainfo);
-    t2 = dateton(endobs, datainfo);
-
-    strcpy(datainfo->stobs, stobs);
-    strcpy(datainfo->endobs, endobs);
-
-    datainfo->sd0 = get_date_x(datainfo->pd, datainfo->stobs);
-
-    datainfo->n = t2 - t1 + 1;
-    datainfo->t1 = 0;
-    datainfo->t2 = datainfo->n - 1;
-
-    datainfo->v = 2;
-    start_new_Z(&Z, datainfo, 0);
-    datainfo->markers = 0;
-
-    edit_dialog (_("gretl: name variable"), 
-		 _("Enter name for new variable\n"
-		   "(max. 8 characters)"),
-		 NULL, name_first_ssheet_var, NULL, 
-		 0, 0);
-
-    return 0;
-}
-
-static void n_obs_callback (GtkWidget *w, dialog_t *dlg)
-{
-    const gchar *buf;
-    char obsstr[32];
-    int n;
-
-    buf = edit_dialog_get_text(dlg);
-    if (buf == NULL) return;
-
-    n = atoi(buf);
-    if (n < 1 || n > 10000) {
-	errbox(_("Invalid number of observations"));
-	return;
-    }
-
-    close_dialog(dlg);
-
-    datainfo->n = n;
-    sprintf(obsstr, "1 %d", n);
-    prep_spreadsheet(obsstr);
+		     do_random_normal, NULL, 
+		     GENR_RANDOM, VARCLICK_NONE, NULL);
+    } else if (r == RANDOM_CHISQ) {
+	edit_dialog (_("gretl: chi-square variable"), 
+		     _("Enter name and degrees of freedom:"), 
+		     "chi 5", 
+		     do_random_chisq, NULL, 
+		     GENR_RANDOM, VARCLICK_NONE, NULL);
+    } else if (r == RANDOM_ST) {
+	edit_dialog (_("gretl: Student's t variable"), 
+		     _("Enter name and degrees of freedom:"), 
+		     "st 20", 
+		     do_random_st, NULL, 
+		     GENR_RANDOM, VARCLICK_NONE, NULL);
+    }	
 }
 
 void newdata_callback (gpointer data, guint pd_code, GtkWidget *widget) 
 {
+    int resp, n = 50;
+
     if (dataset_locked()) {
 	return;
     }
 
-    if (pd_code == 0) {
-	/* cross-sectional dataset */
-	datainfo->structure = CROSS_SECTION;
-	datainfo->pd = 1;
-	datainfo->sd0 = 1.0;
-	strcpy(datainfo->stobs, "1");
-	edit_dialog (_("gretl: create data set"), 
-		     _("Number of observations:"), "50",
-		     n_obs_callback, NULL, 0, 0);
-    } else {
-	gchar *obsstr = NULL;
+    resp = spin_dialog (_("gretl: create data set"), &n, 
+			_("Number of observations:"), 
+			2, 100000, 0);
 
-	datainfo->structure = TIME_SERIES;
-	datainfo->pd = pd_code;
-	compute_default_ts_info(datainfo, 1);
-
-	sample_range_dialog(&obsstr, CREATE_DATASET, NULL);
-
-	if (obsstr != NULL) {
-	    prep_spreadsheet(obsstr);
-	    g_free(obsstr);
-	}
+    if (resp < 0) {
+	/* canceled */
+	return;
     }
+
+    if (open_nulldata(&Z, datainfo, data_status, n, NULL)) {
+	errbox(_("Failed to create empty data set"));
+	return;
+    }
+
+    data_structure_wizard(NULL, 1, NULL);
 }
 
 void do_nistcheck (gpointer p, guint v, GtkWidget *w)
