@@ -5,15 +5,6 @@
 
 #include <gtk/gtk.h>
 
-/* functions follow */
-
-#if GTK_MAJOR_VERSION == 1
-void selectrow (GtkCList *clist, gint row, gint column, 
-                GdkEventButton *event, gpointer data);
-void unselectrow (GtkCList *clist, gint row, gint column, 
-		  GdkEventButton *event, gpointer data);
-#else
-
 void listbox_select_row (GtkTreeSelection *selection, gpointer data);
 
 gint listbox_double_click (GtkWidget *widget, GdkEventButton *event,
@@ -21,7 +12,6 @@ gint listbox_double_click (GtkWidget *widget, GdkEventButton *event,
 
 gboolean listbox_drag (GtkWidget *widget, GdkEventMotion *event,
 		       gpointer data);
-#endif
  
 void open_data (gpointer data, guint dir_code, GtkWidget *widget);
 
@@ -45,10 +35,6 @@ void model_stat_callback (gpointer data, guint which, GtkWidget *widget);
 
 void model_callback (gpointer data, guint model_code, GtkWidget *widget);
 
-#ifdef ENABLE_GMP
-void mp_ols_callback (gpointer data, guint model_code, GtkWidget *widget);
-#endif
-
 void selector_callback (gpointer data, guint action, GtkWidget *widget);
 
 void gretl_callback (gpointer data, guint action, GtkWidget *widget);
@@ -61,7 +47,7 @@ void gp_send_callback (GtkWidget *w, gpointer data);
 
 void file_save_callback (GtkWidget *w, gpointer data);
 
-void add_random_callback (gpointer data, guint code, GtkWidget *widget);
+void add_rand_callback (gpointer data, guint r, GtkWidget *widget);
 
 void newdata_callback (gpointer data, guint pd_code, GtkWidget *widget);
 
