@@ -160,30 +160,6 @@ static char *pop_history_line (int keyval)
     return ret;
 }
 
-#ifdef OLD_GTK
-
-static int last_console_line_len (int len)
-{
-    int i, c;
-
-    for (i=len; i>0; i--) {
-	c = GTK_TEXT_INDEX(GTK_TEXT(console_view), i - 1);
-	if (c == '\n') {
-	    break; 
-	}
-    }
-    return len - i - 2;
-}
-
-static void console_scroll_to_end (void)
-{
-    int len = gtk_text_get_length(GTK_TEXT(console_view));
-
-    gtk_editable_set_position(GTK_EDITABLE(console_view), len);
-}
-
-#else
-
 static void console_scroll_to_end (GtkTextBuffer *buf, 
 				   GtkTextIter *start)
 {
@@ -195,8 +171,6 @@ static void console_scroll_to_end (GtkTextBuffer *buf,
 				       mark);
 }
 
-#endif
-
 enum {
     SAMPLE_RECORD,
     SAMPLE_CHECK
@@ -258,25 +232,17 @@ static int console_function_exec (char *execline)
 
 static void console_exec (void)
 {
+    GtkTextBuffer *buf;
+    GtkTextIter start, end;
     static int redirected;
     int oldv = datainfo->v;
     char execline[MAXLINE];
-#ifndef OLD_GTK
-    GtkTextBuffer *buf;
-    GtkTextIter start, end;
-#else
-    int len;
-    extern GdkColor red;
-#endif
+    int err = 0;
 
-#ifndef OLD_GTK
     buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(console_view));
     gtk_text_buffer_get_end_iter(buf, &end);
     start = end;
     gtk_text_iter_set_line_offset(&start, 2);
-#else
-    len = gtk_text_get_length(GTK_TEXT(console_view));
-#endif
 
     top_n_tail(cbuf);
 
@@ -306,46 +272,25 @@ static void console_exec (void)
     push_history_line(execline); 
 
     /* actually execute the command line */
-    gui_exec_line(execline, console_prn, CONSOLE_EXEC, NULL);
+    err = gui_exec_line(execline, console_prn, CONSOLE_EXEC, NULL);
 
-    /* the control structure below is rather weird and needs more
-       testing/thought.  The issue is the nesting of loops inside
-       functions or vice versa. */
-
-    if (gretl_execute_loop()) { 
-    fn_run_loop:
-	gretl_loop_exec(execline, &Z, &datainfo, models, console_prn);
-	if (gretl_executing_function()) {
-	    console_function_exec(execline);
-	}
-    } else if (gretl_executing_function()) {
-    fn_run_fn:
-	console_function_exec(execline);
-	if (gretl_execute_loop()) {
-	    /* the function we are exec'ing includes a loop */ 
-	    goto fn_run_loop;
+    while (!err && (gretl_execute_loop() || gretl_executing_function())) {
+	if (gretl_execute_loop()) { 
+	    err = gretl_loop_exec(execline, &Z, &datainfo, models, console_prn);
 	} else if (gretl_executing_function()) {
-	    /* the function we are exec'ing includes a function */ 
-	    goto fn_run_fn;
-	}
-    } 
+	    err = console_function_exec(execline);
+	} 
+    }
 
     redirected = printing_is_redirected(console_prn);
 
-#ifndef OLD_GTK
     gtk_text_buffer_get_end_iter(buf, &start);
     gtk_text_buffer_insert(buf, &start, "\n", 1);
-#else
-    gtk_text_freeze(GTK_TEXT(console_view));
-    gtk_editable_insert_text(GTK_EDITABLE(console_view), 
-			     "\n", 1, &len);
-#endif
 
     if (!redirected) {
 	const char *cbuf = gretl_print_get_buffer(console_prn);
 
         /* put results into console window */
-#ifndef OLD_GTK
 	if (!g_utf8_validate(cbuf, -1, NULL)) {
 	    gchar *trbuf = my_locale_to_utf8(cbuf);
 
@@ -355,32 +300,18 @@ static void console_exec (void)
 	} else {
 	    gtk_text_buffer_insert(buf, &start, cbuf, strlen(cbuf));
 	}
-#else
-	gtk_text_insert(GTK_TEXT(console_view), fixed_font, 
-			NULL, NULL, cbuf, strlen(cbuf));
-#endif
 	gretl_print_destroy(console_prn);
 	console_prn = NULL;
     } else {
 	gretl_print_reset_buffer(console_prn);
     }
 
-#ifndef OLD_GTK
     gtk_text_buffer_insert_with_tags_by_name(buf, &start, 
 					     (gretl_compiling_loop())? "\n> " : "\n? ", 
 					     3, "redtext", NULL);
-#else
-    gtk_text_insert(GTK_TEXT(console_view), fixed_font,
-		    &red, NULL, (gretl_compiling_loop())? "\n> " : "\n? ", 3);
-    gtk_text_thaw(GTK_TEXT(console_view));
-#endif
 
     /* scroll to end of buffer */
-#ifndef OLD_GTK
     console_scroll_to_end(buf, &start);
-#else
-    console_scroll_to_end();
-#endif
 
     /* update variable listing in main window if needed */
     if (datainfo->v != oldv || !strncmp(execline, "rename", 6)) {
@@ -398,12 +329,8 @@ void show_gretl_console (void)
     PRN *prn;
     char fname[MAXLEN];
     windata_t *vwin;
-#ifndef OLD_GTK
     GtkTextBuffer *buf;
     GtkTextIter end;
-#else
-    size_t introlen;
-#endif
     const char *intro = 
 	N_("gretl console: type 'help' for a list of commands\n? ");
 
@@ -423,40 +350,21 @@ void show_gretl_console (void)
 
     pputs(prn, _(intro));
     gretl_print_destroy(prn);
-#ifdef OLD_GTK
-    introlen = strlen(_(intro));
-#endif
 
-    vwin = view_file(fname, 1, 0, 78, 400, CONSOLE);
+    vwin = view_file(fname, 1, 1, 78, 400, CONSOLE);
     console_view = vwin->w;
 
-#ifndef OLD_GTK
     g_signal_connect(G_OBJECT(console_view), "destroy",
 		     G_CALLBACK(gretl_console_free),
 		     NULL);
     g_signal_connect(G_OBJECT(console_view), "destroy",
 		     G_CALLBACK(gtk_widget_destroyed),
 		     &console_view);
-#else
-    gtk_signal_connect(GTK_OBJECT(console_view), "destroy",
-		       GTK_SIGNAL_FUNC(gretl_console_free),
-		       NULL);
-    gtk_signal_connect(GTK_OBJECT(console_view), "destroy",
-		       GTK_SIGNAL_FUNC(gtk_widget_destroyed),
-		       &console_view);
-#endif
 
     /* go to end of last line of text */
-#ifndef OLD_GTK
     buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(console_view));
     gtk_text_buffer_get_end_iter(buf, &end);
     gtk_text_buffer_place_cursor(buf, &end);
-#else
-    gtk_text_set_point(GTK_TEXT(console_view), introlen); 
-    GTK_TEXT(console_view)->cursor_pos_x = 2 * gdk_char_width(fixed_font, 'X');
-    gtk_editable_set_position(GTK_EDITABLE(console_view), introlen);
-    gtk_editable_changed(GTK_EDITABLE(console_view));
-#endif
 
     gtk_widget_grab_focus(console_view);
 }
@@ -493,8 +401,6 @@ static int bslash_cont (const gchar *line)
     return bslash;
 }
 
-#ifndef OLD_GTK
-
 gint console_key_handler (GtkWidget *w, GdkEventKey *key, gpointer d)
 {
     gint last_line, curr_line, line_pos;
@@ -614,129 +520,8 @@ gint console_key_handler (GtkWidget *w, GdkEventKey *key, gpointer d)
     return FALSE;
 }
 
-#else /* alternate gtk versions */
-
-static void replace_command_line (int cw, const char *repl)
-{
-    int len = gtk_text_get_length(GTK_TEXT(console_view));
-    int adjust;
-
-    adjust = GTK_TEXT(console_view)->cursor_pos_x / cw - 2;
-    gtk_editable_delete_text(GTK_EDITABLE(console_view), 
-			     len - adjust, len);
-    len = gtk_text_get_length(GTK_TEXT(console_view));
-    if (repl != NULL) {
-	gtk_editable_insert_text(GTK_EDITABLE(console_view), 
-				 repl, strlen(repl),
-				 &len);
-    }
-}
-
-gint console_key_handler (GtkWidget *w, GdkEventKey *key, gpointer d)
-{
-    int cw = gdk_char_width(fixed_font, 'X'); 
-    int len = gtk_text_get_length(GTK_TEXT(console_view));
-    int currpos, linelen, xpos;
-    GdkModifierType mods;
-
-    /* where are we? */
-    currpos = GTK_EDITABLE(console_view)->current_pos;
-    xpos = GTK_TEXT(console_view)->cursor_pos_x / cw; 
-    linelen = last_console_line_len(len);
-
-    /* if at start of command line, backspacing does nothing */
-    if (IS_BACKKEY(key->keyval) && xpos < 3) {
-	gtk_signal_emit_stop_by_name(GTK_OBJECT(w), "key-press-event");
-	return TRUE;
-    }
-
-    /* if not on prompt line, return to (the end of) it */
-    if (currpos < len - linelen) {
-	gtk_text_set_point(GTK_TEXT(console_view), len);
-	gtk_editable_set_position(GTK_EDITABLE(console_view), len);
-	gtk_text_set_editable(GTK_TEXT(console_view), TRUE);
-	return TRUE;
-    }
-
-    /* make return key execute the command, unless backslash-
-       continuation is happening */
-    if (key->keyval == GDK_Return) {
-	gchar *bit;
-	int start, cont = 0;
-	extern GdkColor red;
-
-	start = len - last_console_line_len(len);
-	bit = gtk_editable_get_chars(GTK_EDITABLE(console_view), 
-				     start, -1);
-	if (bit != NULL) {
-	    cont = bslash_cont(bit);
-	    g_free(bit);
-	}
-
-	if (cont) {
-	    gtk_text_insert(GTK_TEXT(console_view), fixed_font,
-			    &red, NULL, "\n> ", 3);
-	    console_scroll_to_end();
-	} else {
-	    console_exec();
-	}
-
-	key->keyval = GDK_End;
-	return FALSE;
-    }
-
-    /* up and down arrows: command history */
-    if (key->keyval == GDK_Up || key->keyval == GDK_Down) {
-	char *histline;
-
-	histline = pop_history_line(key->keyval);
-
-	if (histline != NULL || key->keyval == GDK_Down) {
-	    replace_command_line(cw, histline);
-	}
-	key->keyval = GDK_VoidSymbol;
-	return TRUE;
-    }
-
-    /* response to Ctrl-A: go to start of typing area */
-    gdk_window_get_pointer(console_view->window, NULL, NULL, &mods);
-    if (mods & GDK_CONTROL_MASK && 
-	gdk_keyval_to_upper(key->keyval) == GDK_A) {
-	gtk_editable_set_position(GTK_EDITABLE(console_view), 
-				  len - last_console_line_len(len));	
-	gtk_signal_emit_stop_by_name(GTK_OBJECT(w), "key-press-event");
-	return TRUE;
-    }
-
-    /* tab completion for gretl commands */
-    if (key->keyval == GDK_Tab) {
-	gchar *bit;
-	int start;
-
-	start = len - last_console_line_len(len);
-	bit = gtk_editable_get_chars(GTK_EDITABLE(console_view), 
-				     start, -1);
-
-	if (bit != NULL && *bit != 0) {
-	    const char *complete = gretl_command_complete(bit);
-
-	    g_free(bit);
-	    if (complete != NULL) {
-		replace_command_line(cw, complete);
-	    }
-	}
-	gtk_signal_emit_stop_by_name(GTK_OBJECT(w), "key-press-event");
-	return TRUE;
-    }    
-
-    return FALSE;
-}
-
-#endif
-
 static gint on_last_line (void)
 {
-#ifndef OLD_GTK
     GtkTextBuffer *buf;
     GtkTextIter iter;
     
@@ -745,12 +530,6 @@ static gint on_last_line (void)
 
     return (gtk_text_iter_get_line(&iter) == 
 	    gtk_text_buffer_get_line_count(buf) - 1);
-#else
-    int len = gtk_text_get_length(GTK_TEXT(console_view));
-    int currpos = GTK_EDITABLE(console_view)->current_pos;
-
-    return (currpos >= len - last_console_line_len(len));
-#endif
 }
 
 gint console_mouse_handler (GtkWidget *w, GdkEventButton *event,
@@ -758,11 +537,7 @@ gint console_mouse_handler (GtkWidget *w, GdkEventButton *event,
 {
     int lline = on_last_line();
 
-#ifndef OLD_GTK
     gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(w), lline);
-#else
-    gtk_text_set_editable(GTK_TEXT(w), lline);
-#endif
 
     return FALSE;
 }
