@@ -27,13 +27,12 @@
 #include "filelists.h"
 #include "webget.h"
 #include "menustate.h"
-#include "gretl_xml.h"
+#include "fnsave.h"
+#include "fncall.h"
+#include "treeutils.h"
 
-#if !GLIB_CHECK_VERSION(2,0,0)
-# define OLD_GTK
-#else
-# include "treeutils.h"
-#endif
+#include "gretl_xml.h"
+#include "gretl_func.h"
 
 #include <sys/stat.h>
 #include <unistd.h>
@@ -45,10 +44,6 @@
 #define BROWSER_BUSY    1
 #define BROWSER_OK      0
 
-#ifdef OLD_GTK
-extern GdkColor gray;
-#endif
-
 static GtkWidget *browsers[N_BROWSER_TYPES];
 
 static GtkWidget *files_window (windata_t *fdata);
@@ -88,6 +83,8 @@ enum {
     COLL_PS
 };
 
+#define REMOTE_ACTION(c) (c == REMOTE_DB || c == REMOTE_FUNC_FILES)
+
 static char *full_path (char *s1, const char *s2)
 {
     static char fpath[FILENAME_MAX];
@@ -504,7 +501,8 @@ char *strip_extension (char *s)
     
     if (p != NULL && 
 	(!strcmp(p, ".gdt") || !strcmp(p, ".inp") ||
-	 !strcmp(p, ".bin"))) {
+	 !strcmp(p, ".bin") || !strcmp(p, ".gfn") ||
+	 !strcmp(p, ".bn7"))) {
 	*p = '\0';
     }
 
@@ -514,13 +512,9 @@ char *strip_extension (char *s)
 static int read_file_descriptions (windata_t *win, gpointer p)
 {
     FILE *fp;
-#ifndef OLD_GTK
     GtkListStore *store;
     GtkTreeSelection *selection;
     GtkTreeIter iter;
-#else
-    gint i;    
-#endif
     char line[MAXLEN];
     char *index;
     file_collection *coll = (file_collection *) p;
@@ -532,12 +526,8 @@ static int read_file_descriptions (windata_t *win, gpointer p)
 	return 1;
     }
 
-#ifndef OLD_GTK
     store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(win->listbox)));
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
-#else
-    i = 0;
-#endif
     
     while (fgets(line, sizeof line, fp)) {
 	char fname[24], descrip[80], data[64];
@@ -547,56 +537,29 @@ static int read_file_descriptions (windata_t *win, gpointer p)
 	if (win->role == TEXTBOOK_DATA) {
 	    if (sscanf(line, " \"%23[^\"]\",\"%79[^\"]\"", 
 		       fname, descrip) == 2) {
-#ifdef OLD_GTK
-		gchar *row[2];	
-
-		row[0] = strip_extension(fname);
-		row[1] = descrip;
-		gtk_clist_append(GTK_CLIST(win->listbox), row);
-#else
 		gtk_list_store_append(store, &iter);
 		gtk_list_store_set(store, &iter, 
 				   0, strip_extension(fname), 
 				   1, descrip, -1);
-#endif
 	    }
 	} else { /* script files */
 	    if (sscanf(line, " \"%23[^\"]\",\"%79[^\"]\",\"%63[^\"]\"", 
 		       fname, descrip, data) == 3) {
-#ifdef OLD_GTK
-		gchar *row[3];
-
-		row[0] = strip_extension(fname);
-		row[1] = descrip;
-		row[2] = data;
-		gtk_clist_append(GTK_CLIST(win->listbox), row);
-#else
 		gtk_list_store_append(store, &iter);
 		gtk_list_store_set(store, &iter, 
 				   0, strip_extension(fname), 
 				   1, descrip, 
 				   2, data, -1);
-#endif
 	    }
 	}
-#ifdef OLD_GTK
-	if (i % 2) {
-	    gtk_clist_set_background(GTK_CLIST(win->listbox), i, &gray);
-	}
-	i++;
-#endif	
     }
 
     fclose(fp);
 
     /* select the first row */
-#ifndef OLD_GTK
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(win->listbox));
     gtk_tree_selection_select_iter(selection, &iter);
-#else
-    gtk_clist_select_row(GTK_CLIST(win->listbox), 0, 0);
-#endif
     
     return 0;
 }
@@ -610,21 +573,12 @@ static void display_datafile_info (GtkWidget *w, gpointer data)
     gchar *fname;
     file_collection *coll;
 
-#ifndef OLD_GTK
     tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), vwin->active_var,
 			 0, &fname);
     coll = g_object_get_data(G_OBJECT(vwin->listbox), "coll");
-#else
-    gtk_clist_get_text(GTK_CLIST(vwin->listbox), vwin->active_var, 
-		       0, &fname);
-    coll = gtk_object_get_data(GTK_OBJECT(vwin->listbox), "coll");
-#endif
 
-    build_path(coll->path, fname, hdrname, ".gdt");
-
-#ifndef OLD_GTK
+    build_path(hdrname, coll->path, fname, ".gdt");
     g_free(fname);
-#endif
 
     descrip = gretl_get_gdt_description(hdrname);
 
@@ -643,21 +597,12 @@ void browser_open_data (GtkWidget *w, gpointer data)
     gchar *datname;
     file_collection *coll;
 
-#ifndef OLD_GTK
     tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), vwin->active_var, 
 			 0, &datname);
     coll = g_object_get_data(G_OBJECT(vwin->listbox), "coll");
-#else
-    gtk_clist_get_text(GTK_CLIST(vwin->listbox), vwin->active_var, 
-		       0, &datname);
-    coll = gtk_object_get_data(GTK_OBJECT(vwin->listbox), "coll");
-#endif
 
-    build_path(coll->path, datname, trydatfile, ".gdt");
-
-#ifndef OLD_GTK
+    build_path(tryfile, coll->path, datname, ".gdt");
     g_free(datname);
-#endif
 
     set_datapage(coll->title);
 
@@ -670,21 +615,12 @@ void browser_open_ps (GtkWidget *w, gpointer data)
     gchar *fname;
     file_collection *coll;
 
-#ifndef OLD_GTK
     tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), vwin->active_var, 
 			 0, &fname);
     coll = g_object_get_data(G_OBJECT(vwin->listbox), "coll");
-#else
-    gtk_clist_get_text(GTK_CLIST(vwin->listbox), vwin->active_var, 
-		       0, &fname);
-    coll = gtk_object_get_data(GTK_OBJECT(vwin->listbox), "coll");
-#endif
 
-    build_path(coll->path, fname, scriptfile, ".inp");
-
-#ifndef OLD_GTK
+    build_path(scriptfile, coll->path, fname, ".inp");
     g_free(fname);
-#endif
 
     gtk_widget_destroy(GTK_WIDGET(vwin->w));
 
@@ -695,35 +631,165 @@ void browser_open_ps (GtkWidget *w, gpointer data)
     view_file(scriptfile, 0, 0, 78, 370, VIEW_SCRIPT);
 } 
 
-void browser_load_func (GtkWidget *w, gpointer data)
+enum {
+    VIEW_FN_PKG,
+    LOAD_FN_PKG,
+    EDIT_FN_PKG,
+    DELETE_FN_PKG,
+    CALL_FN_PKG
+};
+
+static void gui_load_user_functions (const char *fname)
+{
+    int err;
+
+    err = load_user_function_file(fname);
+    if (err) {
+	gui_errmsg(err);
+    } 
+}
+
+static void gui_delete_fn_pkg (const char *fname, windata_t *vwin)
+{
+    char *msg = g_strdup_printf(_("Really delete %s?"), fname);
+    int err;
+    
+    if (yes_no_dialog(_("gretl: delete"), msg, 0) != GRETL_YES) {
+        return;
+    }
+
+    err = remove(fname);
+    if (err) {
+	errbox("Couldn't delete %s", fname);
+    } else {
+	GtkTreeModel *mod;
+	GtkTreeIter iter;
+	int i = 0;
+
+	mod = gtk_tree_view_get_model(GTK_TREE_VIEW(vwin->listbox));
+	gtk_tree_model_get_iter_first(mod, &iter);
+	while (i < vwin->active_var) {
+	    gtk_tree_model_iter_next(mod, &iter);
+	    i++;
+	}
+	gtk_list_store_remove(GTK_LIST_STORE(mod), &iter);
+    }
+}
+
+windata_t *gui_show_function_info (const char *fname, int role)
+{
+    char *pkgname = NULL;
+    windata_t *vwin = NULL;
+    PRN *prn;
+    int err;
+
+    if (bufopen(&prn)) {
+	return NULL;
+    }
+
+    if (role == VIEW_FUNC_INFO) {
+	err = get_function_file_info(fname, prn, &pkgname);
+    } else {
+	err = get_function_file_code(fname, prn, &pkgname);
+    }
+	
+    if (err) {
+	gretl_print_destroy(prn);
+	gui_errmsg(err);
+    } else {
+	gchar *title;
+
+	title = g_strdup_printf("gretl: %s", (pkgname)? 
+				pkgname : _("function code"));
+	vwin = view_buffer(prn, 78, 350, title, role, NULL);
+	strcpy(vwin->fname, fname);
+	if (strstr(fname, "dltmp")) {
+	    set_window_delete_filename(vwin);
+	}
+	g_free(title);
+    }
+
+    free(pkgname);
+
+    return vwin;
+}
+
+static void browser_functions_handler (windata_t *vwin, int task)
 {
-    windata_t *vwin = (windata_t *) data;
     char fnfile[FILENAME_MAX];
     gchar *fname;
     gchar *dir;
+    int dircol = 0;
+
+    if (vwin->role == FUNC_EDIT) {
+	dircol = 2;
+    } else if (vwin->role == FUNC_FILES) {
+	dircol = 3;
+    } else if (vwin->role != REMOTE_FUNC_FILES) {
+	dummy_call();
+	return;
+    }
 
-#ifndef OLD_GTK
     tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), vwin->active_var, 
 			 0, &fname);
-    tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), vwin->active_var, 
-			 2, &dir);
-#else
-    gtk_clist_get_text(GTK_CLIST(vwin->listbox), vwin->active_var, 
-		       0, &fname);
-    gtk_clist_get_text(GTK_CLIST(vwin->listbox), vwin->active_var, 
-		       2, &dir); /* wrong */
-#endif
 
-    build_path(dir, fname, fnfile, NULL);
+    if (dircol != 0) {
+	tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), vwin->active_var, 
+			     dircol, &dir);
+	build_path(fnfile, dir, fname, ".gfn");
+	g_free(dir);
+    } else {
+	strcpy(fnfile, fname);
+    }
 
-    fprintf(stderr, "fnfile: '%s'\n", fnfile);
-
-#ifndef OLD_GTK
     g_free(fname);
-    g_free(dir);
-#endif
 
-    gtk_widget_destroy(GTK_WIDGET(vwin->w));
+    if (task == LOAD_FN_PKG) {
+	gui_load_user_functions(fnfile);
+    } else if (task == DELETE_FN_PKG) {
+	gui_delete_fn_pkg(fnfile, vwin);
+    } else if (task == VIEW_FN_PKG) {
+	gui_show_function_info(fnfile, VIEW_FUNC_INFO);
+    } else if (task == EDIT_FN_PKG) {
+	edit_function_package(fnfile);
+    } else if (task == CALL_FN_PKG) {
+	call_function_package(fnfile, vwin->w);
+    }
+}
+
+void browser_load_func (GtkWidget *w, gpointer data)
+{
+    windata_t *vwin = (windata_t *) data;
+
+    browser_functions_handler(vwin, LOAD_FN_PKG);
+} 
+
+void browser_edit_func (GtkWidget *w, gpointer data)
+{
+    windata_t *vwin = (windata_t *) data;
+
+    browser_functions_handler(vwin, EDIT_FN_PKG);
+}
+
+void browser_call_func (GtkWidget *w, gpointer data)
+{
+    windata_t *vwin = (windata_t *) data;
+
+    browser_functions_handler(vwin, CALL_FN_PKG);
+}
+
+static void display_function_info (GtkWidget *w, gpointer data)
+{
+    windata_t *vwin = (windata_t *) data;
+
+    browser_functions_handler(vwin, VIEW_FN_PKG);
+}
+
+static void browser_del_func (GtkWidget *w, gpointer data)
+{
+    windata_t *vwin = (windata_t *) data;
+
+    browser_functions_handler(vwin, DELETE_FN_PKG);
 } 
 
 static void set_browser_status (windata_t *vwin, int status)
@@ -756,7 +822,6 @@ static void build_datafiles_popup (windata_t *vwin)
 	add_popup_item(_("Info"), vwin->popup, 
 		       G_CALLBACK(display_datafile_info), 
 		       vwin);
-
 	add_popup_item(_("Open"), vwin->popup, 
 		       G_CALLBACK(browser_open_data), 
 		       vwin);
@@ -785,7 +850,10 @@ void display_files (gpointer p, guint code, GtkWidget *w)
     GtkWidget *filebox, *button;
     GtkWidget *main_vbox, *button_box;
     windata_t *vwin;
+    const gchar *label;
     void (*browse_func)() = NULL;
+    void (*delete_func)() = NULL;
+    void (*call_func)() = NULL;
     int err = 0;
 
     if (browser_busy(code)) {
@@ -816,8 +884,10 @@ void display_files (gpointer p, guint code, GtkWidget *w)
 	break;
     case FUNC_FILES:
 	gtk_window_set_title(GTK_WINDOW(vwin->w), 
-			     _("gretl: function files"));
-	browse_func = browser_load_func;
+			     _("gretl: function packages"));
+	browse_func = browser_edit_func;
+	call_func = browser_call_func;
+	delete_func = browser_del_func;
 	break;
     case TEXTBOOK_DATA:
 	gtk_window_set_title(GTK_WINDOW(vwin->w), 
@@ -825,17 +895,22 @@ void display_files (gpointer p, guint code, GtkWidget *w)
 	browse_func = browser_open_data;
 	break;
     case NATIVE_DB:
-    case RATS_DB:
 	gtk_window_set_title(GTK_WINDOW(vwin->w), 
 			     _("gretl: database files"));
-	browse_func = open_db_list;
+	browse_func = open_db_index;
 	break;
     case REMOTE_DB:
 	gtk_window_set_title(GTK_WINDOW(vwin->w), 
 			     _("gretl: databases on server"));
-	browse_func = open_remote_db_list;
+	browse_func = open_remote_db_index;
 	gtk_widget_set_usize(vwin->w, 640, 480);
 	break;
+    case REMOTE_FUNC_FILES:
+	gtk_window_set_title(GTK_WINDOW(vwin->w), 
+			     _("gretl: function packages on server"));
+	call_func = browser_call_func;
+	browse_func = file_info_from_server;
+	break;
     }
 
     /* set up grids */
@@ -854,7 +929,7 @@ void display_files (gpointer p, guint code, GtkWidget *w)
     if (code == TEXTBOOK_DATA) { 
 	file_collection *coll;
 
-	/* crate popup menu */
+	/* create popup menu */
 	build_datafiles_popup(vwin);
 
 	while ((coll = pop_data_collection())) {
@@ -863,7 +938,7 @@ void display_files (gpointer p, guint code, GtkWidget *w)
 			     vwin->popup);
 	}
 	reset_data_stack();
-    } else if (code == REMOTE_DB) {
+    } else if (REMOTE_ACTION(code)) {
 	GtkWidget *hbox;
 
 	hbox = gtk_hbox_new(FALSE, 0);
@@ -871,36 +946,60 @@ void display_files (gpointer p, guint code, GtkWidget *w)
 	vwin->status = gtk_label_new(_("Network status: OK"));
 	gtk_label_set_justify(GTK_LABEL(vwin->status), GTK_JUSTIFY_LEFT);
 	gtk_box_pack_start(GTK_BOX(hbox), vwin->status, FALSE, FALSE, 0);
-    }
+    } 
 
     button_box = gtk_hbox_new(FALSE, 5);
     gtk_box_set_homogeneous(GTK_BOX(button_box), TRUE);
     gtk_box_pack_start(GTK_BOX(main_vbox), button_box, FALSE, FALSE, 0);
 
-    button = gtk_button_new_with_label 
-	((code == REMOTE_DB)? _("Get series listing") : _("Open"));
-    gtk_box_pack_start(GTK_BOX(button_box), button, FALSE, TRUE, 0);
+    if (browse_func != NULL) {
+	label = (code == REMOTE_DB)? N_("Get series listing") :
+	    (code == REMOTE_FUNC_FILES)? N_("Info") :
+	    (code == FUNC_FILES)? N_("Edit") :
+	    N_("Open");
 
-    g_signal_connect(G_OBJECT(button), "clicked",
-		     G_CALLBACK(browse_func), vwin);
+	button = gtk_button_new_with_label(_(label));
+	gtk_box_pack_start(GTK_BOX(button_box), button, FALSE, TRUE, 0);
 
-    if (code != NATIVE_DB && code != RATS_DB && code != REMOTE_DB) {
-       	g_signal_connect(G_OBJECT(button), "clicked", 
-			 G_CALLBACK(delete_widget), vwin->w); 
+	g_signal_connect(G_OBJECT(button), "clicked",
+			 G_CALLBACK(browse_func), vwin);
+
+	if (code != NATIVE_DB && code != FUNC_FILES && !REMOTE_ACTION(code)) {
+	    g_signal_connect(G_OBJECT(button), "clicked", 
+			     G_CALLBACK(delete_widget), vwin->w); 
+	}
     }
 
-    if (code == TEXTBOOK_DATA || code == REMOTE_DB) {
-	button = gtk_button_new_with_label 
-	    ((code == REMOTE_DB)? _("Install") : _("Info"));
+    if (code == TEXTBOOK_DATA || code == FUNC_FILES || REMOTE_ACTION(code)) {
+	label = (REMOTE_ACTION(code))? N_("Install") : N_("Info");
+	button = gtk_button_new_with_label(_(label));
 	gtk_box_pack_start(GTK_BOX(button_box), button, FALSE, TRUE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked",
-			 (code == REMOTE_DB)?
-			 G_CALLBACK(grab_remote_db) :
+			 (REMOTE_ACTION(code))?
+			 G_CALLBACK(install_file_from_server) :
+			 (code == FUNC_FILES)? 
+			 G_CALLBACK(display_function_info) :
 			 G_CALLBACK(display_datafile_info), vwin);
-	button = gtk_button_new_with_label(_("Find"));
-	gtk_box_pack_start(GTK_BOX (button_box), button, FALSE, TRUE, 0);
+	if (code != FUNC_FILES && code != REMOTE_FUNC_FILES) {
+	    button = gtk_button_new_with_label(_("Find"));
+	    gtk_box_pack_start(GTK_BOX(button_box), button, FALSE, TRUE, 0);
+	    g_signal_connect(G_OBJECT(button), "clicked",
+			     G_CALLBACK(datafile_find), vwin);
+	}
+    }
+
+    if (call_func != NULL) {
+	button = gtk_button_new_with_label(_("Execute"));
+	gtk_box_pack_start(GTK_BOX(button_box), button, FALSE, TRUE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked",
-			 G_CALLBACK(datafile_find), vwin);
+			 G_CALLBACK(call_func), vwin);
+    }
+
+    if (delete_func != NULL) {
+	button = gtk_button_new_with_label(_("Delete"));
+	gtk_box_pack_start(GTK_BOX(button_box), button, FALSE, TRUE, 0);
+	g_signal_connect(G_OBJECT(button), "clicked",
+			 G_CALLBACK(delete_func), vwin);
     }
 
     button = gtk_button_new_with_label(_("Close"));
@@ -916,10 +1015,6 @@ void display_files (gpointer p, guint code, GtkWidget *w)
     }
 
     if (err) {
-	if (0) {
-	    /* FIXME? */
-	    errbox(_("Couldn't open database"));
-	}
 	gtk_widget_destroy(vwin->w);
     } else {
 	gtk_widget_show_all(vwin->w); 
@@ -929,130 +1024,86 @@ void display_files (gpointer p, guint code, GtkWidget *w)
 static char *get_func_description (const char *fname, const char *fndir)
 {
     char fullname[FILENAME_MAX];
-    char line[128];
-    FILE *fp;
+    char *descrip = NULL;
+    int err = 0;
 
-    build_path(fndir, fname, fullname, NULL);
-    
-    fp = fopen(fullname, "r");
-    if (fp == NULL) {
-	return NULL;
-    }
-    
-    while (fgets(line, sizeof line, fp)) {
-	if (*line != '#') {
-	    break;
-	}
-	printf("%s", line + 2);
+    build_path(fullname, fndir, fname, NULL);
+    descrip = get_function_file_header(fullname, &err);
+    if (err) {
+	gui_errmsg(err);
     }
 
-    fclose(fp);
-
-    return g_strdup("Not ready yet");
+    return descrip;
 }
 
-#ifndef OLD_GTK
-
 static int
-read_fn_files_in_dir (DIR *dir, const char *fndir, 
+read_fn_files_in_dir (int role, DIR *dir, const char *fndir, 
 		      GtkListStore *store, GtkTreeIter *iter)
 {
     struct dirent *dirent;
-    const char *fname;
+    char fullname[MAXLEN];
+    gboolean loaded;
+    char *fname;
     char *descrip;
     int n, nfn = 0;
 
     while ((dirent = readdir(dir)) != NULL) {
-	fname = dirent->d_name;
+	fname = g_strdup(dirent->d_name);
+	if (fname == NULL) {
+	    break;
+	}
 	n = strlen(fname);
-	if (!g_ascii_strcasecmp(fname + n - 4, ".inp")) {
+	if (!g_ascii_strcasecmp(fname + n - 4, ".gfn")) {
 	    descrip = get_func_description(fname, fndir);
 	    if (descrip != NULL) {
 		gtk_list_store_append(store, iter);
-		gtk_list_store_set(store, iter, 0, fname, 1, descrip,
-				   2, fndir, -1);
+		if (role == FUNC_EDIT) {
+		    fname[n - 4] = '\0';
+		    gtk_list_store_set(store, iter, 0, fname, 1, descrip,
+				       2, fndir, -1);
+		} else {
+		    build_path(fullname, fndir, fname, NULL);
+		    fname[n - 4] = '\0';
+		    loaded = user_function_file_is_loaded(fullname);
+		    gtk_list_store_set(store, iter, 0, fname, 1, descrip,
+				       2, loaded, 3, fndir, -1);
+		}
 		g_free(descrip);
 		nfn++;
 	    }
-	} 
-    }
-
-    return nfn;
-}
-
-#else
-
-static int
-read_fn_files_in_dir (DIR *dir, char *fndir, windata_t *vwin, int nfn)
-{
-    struct dirent *dirent;
-    char *fname;
-    char *descrip;
-    gchar *row[2];
-    int n, i;
-
-    while ((dirent = readdir(dir)) != NULL) {
-	fname = dirent->d_name;
-	n = strlen(fname);
-	if (!g_strcasecmp(fname + n - 4, ".inp")) {
-	    descrip = get_func_description(fname, fndir);
-	    if (descrip != NULL) {
-		row[0] = fname;
-		row[1] = descrip;
-		i = gtk_clist_append(GTK_CLIST(vwin->listbox), row);
-		gtk_clist_set_row_data(GTK_CLIST(vwin->listbox), i, fndir);
-		g_free(descrip);
-		nfn++;
-		if (nfn % 2) {
-		    gtk_clist_set_background(GTK_CLIST(vwin->listbox), i, &gray);
-		}
-	    } 
 	}
+	g_free(fname);
     }
 
     return nfn;
 }
 
-#endif
-
 gint populate_func_list (windata_t *vwin)
 {
-#ifndef OLD_GTK
     GtkListStore *store;
     GtkTreeIter iter;
-#endif
     char fndir[FILENAME_MAX];
     DIR *dir;
     int nfn = 0;
 
-#ifndef OLD_GTK
     store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(vwin->listbox)));
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
-#endif
 
     /* pick up any function files in system dir */
-    sprintf(fndir, "%sfunctions", paths.gretldir);
+    build_path(fndir, paths.gretldir, "functions", NULL);
     dir = opendir(fndir);
 
     if (dir != NULL) {
-#ifndef OLD_GTK
-	nfn += read_fn_files_in_dir(dir, fndir, store, &iter);
-#else
-	nfn = read_fn_files_in_dir(dir, fndir, vwin, 0);
-#endif
+	nfn += read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter);
 	closedir(dir);
     }
 
     /* pick up any function files in the user's personal dir */
-    sprintf(fndir, "%sfunctions", paths.userdir);
+    build_path(fndir, paths.userdir, "functions", NULL);
     dir = opendir(fndir);
 
     if (dir != NULL) {
-#ifndef OLD_GTK
-	nfn += read_fn_files_in_dir(dir, fndir, store, &iter);
-#else
-	nfn = read_fn_files_in_dir(dir, fndir, vwin, nfn);
-#endif
+	nfn += read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter);
 	closedir(dir);
     }
 
@@ -1062,19 +1113,15 @@ gint populate_func_list (windata_t *vwin)
 	return 1;
     }
 
-#ifdef OLD_GTK
-    gtk_clist_select_row(GTK_CLIST(vwin->listbox), 0, 0);
-#endif
-
     return 0;
 }
 
 gint populate_filelist (windata_t *vwin, gpointer p)
 {
-    if (vwin->role == NATIVE_DB || vwin->role == RATS_DB) {
+    if (vwin->role == NATIVE_DB) {
 	return populate_dbfilelist(vwin);
-    } else if (vwin->role == REMOTE_DB) {
-	return populate_remote_db_list(vwin);
+    } else if (REMOTE_ACTION(vwin->role)) {
+	return populate_remote_object_list(vwin);
     } else if (vwin->role == FUNC_FILES) {
 	return populate_func_list(vwin);
     } else {
@@ -1082,30 +1129,59 @@ gint populate_filelist (windata_t *vwin, gpointer p)
     }
 }
 
-#ifndef OLD_GTK
-
 static GtkWidget *files_window (windata_t *vwin) 
 {
     const char *data_titles[] = {
-	_("File"), 
-	_("Summary")
+	N_("File"), 
+	N_("Summary")
     };
     const char *ps_titles[] = {
-	_("Script"), 
-	_("Topic"), 
-	_("Data")
+	N_("Script"), 
+	N_("Topic"), 
+	N_("Data")
     };
     const char *db_titles[] = {
-	_("Database"), 
-	_("Source")
+	N_("Database"), 
+	N_("Source")
     };
-    const char *remote_titles[] = {
-	_("Database"), 
-	_("Source"), 
-	_("Local status")
+    const char *remote_db_titles[] = {
+	N_("Database"), 
+	N_("Source"), 
+	N_("Local status")
+    };
+    const char *func_titles[] = {
+	N_("Package"), 
+	N_("Summary"), 
+	N_("Loaded?")
+    };
+    const char *remote_func_titles[] = {
+	N_("Package"), 
+	N_("Summary"), 
+	N_("Local status")
+    };
+    const char *func_edit_titles[] = {
+	N_("Package"), 
+	N_("Summary") 
+    };
+
+    GType types_2[] = {
+	G_TYPE_STRING,
+	G_TYPE_STRING
+    };
+    GType types_3[] = {
+	G_TYPE_STRING,
+	G_TYPE_STRING,
+	G_TYPE_STRING
+    };
+    GType types_4[] = {
+	G_TYPE_STRING,
+	G_TYPE_STRING,
+	G_TYPE_BOOLEAN,
+	G_TYPE_STRING
     };
 
     const char **titles = data_titles;
+    GType *types = types_2;
 
     int full_width = 500, file_height = 260;
     int hidden_col = 0;
@@ -1116,146 +1192,54 @@ static GtkWidget *files_window (windata_t *vwin)
     switch (vwin->role) {
     case NATIVE_DB:
 	titles = db_titles;
-	hidden_col = 1;
+	cols = 3;
+	hidden_col = TRUE;
 	break;
     case REMOTE_DB:
-	titles = remote_titles;
+	titles = remote_db_titles;
 	cols = 3;
 	full_width = 560;
 	break;
-    case RATS_DB:
-	titles = db_titles;
-	cols = 1;
-	full_width = 240;
-	hidden_col = 1;
-	break;
     case PS_FILES:
 	titles = ps_titles;
 	cols = 3;
 	full_width = 480;
 	break;
     case FUNC_FILES:
-	hidden_col = 1;
+	titles = func_titles;
+	cols = 4;
+	types = types_4;
+	hidden_col = TRUE;
 	break;
+    case REMOTE_FUNC_FILES:
+	titles = remote_func_titles;
+	cols = 3;
+	break;
+    case FUNC_EDIT:
+	titles = func_edit_titles;
+	cols = 3;
+	hidden_col = TRUE;
+	break;	
     default:
 	break;
     }
 
+    if (cols == 3) {
+	types = types_3;
+    }
+
     full_width *= gui_scale;
     file_height *= gui_scale;
 
     box = gtk_vbox_new(FALSE, 0);
     gtk_widget_set_size_request(box, full_width, file_height);
-
-    vwin->listbox = list_box_create(vwin, GTK_BOX(box), cols, 
-				    hidden_col, titles);
-
+    vwin_add_list_box(vwin, GTK_BOX(box), cols, hidden_col, 
+		      types, titles, 0);
     gtk_widget_show(box);
 
     return box;
 }
 
-#else /* now the old gtk version */
-
-static GtkWidget *files_window (windata_t *fdata) 
-{
-    char *data_titles[] = {
-	_("File"), 
-	_("Summary")
-    };
-    char *ps_titles[] = {
-	_("Script"), 
-	_("Topic"), 
-	_("Data")
-    };
-    char *db_titles[] = {
-	_("Database"), 
-	_("Source")
-    };
-    char *remote_titles[] = {
-	_("Database"), 
-	_("Source"), 
-	_("Local status")
-    };
-
-    char **titles = data_titles;
-
-    int data_col_width[] = {128, 256}; 
-    int ps_col_width[] = {68, 180, 160};
-    int db_col_width[] = {80, 304};
-    int remote_col_width[] = {80, 256, 180};
-    int *col_width = data_col_width;
-    int full_width = 500, file_height = 260;
-
-    GtkWidget *box, *scroller;
-    int i, cols = 2;
-
-    switch (fdata->role) {
-    case NATIVE_DB:
-	titles = db_titles;
-	col_width = db_col_width;
-	break;
-    case REMOTE_DB:
-	titles = remote_titles;
-	cols = 3;
-	col_width = remote_col_width;
-	full_width = 560;
-	break;
-    case RATS_DB:
-	titles = db_titles;
-	cols = 1;
-	col_width = db_col_width;
-	col_width[0] = 200;
-	full_width = 240;
-	break;
-    case PS_FILES:
-	titles = ps_titles;
-	cols = 3;
-	col_width = ps_col_width;
-	full_width = 480;
-	break;
-    default:
-	break;
-    }
-
-    fdata->active_var = 1; 
-
-    box = gtk_vbox_new (FALSE, 0);
-
-    full_width *= gui_scale;
-    file_height *= gui_scale;
-
-    gtk_widget_set_usize(box, full_width, file_height);
-   
-    scroller = gtk_scrolled_window_new (NULL, NULL);
-    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroller),
-				    GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-    fdata->listbox = gtk_clist_new_with_titles(cols, titles);
-    gtk_clist_column_titles_passive(GTK_CLIST(fdata->listbox));
-    gtk_container_add (GTK_CONTAINER (scroller), fdata->listbox);
-    gtk_clist_set_selection_mode (GTK_CLIST (fdata->listbox), 
-				  GTK_SELECTION_BROWSE);
-    for (i=0; i<cols; i++) {
-	col_width[i] *= gui_scale;
-	gtk_clist_set_column_width (GTK_CLIST (fdata->listbox), i,
-				    col_width[i]);
-	gtk_clist_set_column_justification (GTK_CLIST (fdata->listbox), i, 
-					    GTK_JUSTIFY_LEFT);
-    }
-
-    gtk_box_pack_start (GTK_BOX (box), scroller, TRUE, TRUE, TRUE);
-    gtk_signal_connect_after (GTK_OBJECT (fdata->listbox), "select_row", 
-			      GTK_SIGNAL_FUNC (selectrow), fdata);
-    gtk_widget_show (fdata->listbox);
-    gtk_widget_show (scroller);
-
-    gtk_widget_show (box);
-
-    return box;
-}
-
-#endif /* old vs new gtk */
-
 static void 
 switch_file_page_callback (GtkNotebook *notebook, GtkNotebookPage *page,
 			   guint page_num, windata_t *vwin)
