@@ -26,41 +26,14 @@
 
 #include "system.h"
 
-#ifdef OLD_GTK
-
-/* compatibility functions */
-
-GtkWidget *standard_button (int code)
+void vbox_add_hsep (GtkWidget *vbox)
 {
-    const char *button_strings[] = {
-	N_("OK"),
-	N_("Cancel"),
-	N_("Clear"),
-	N_("Close"),
-	N_("Apply"),
-	N_("Help"),
-	N_("Forward"),
-	N_("Back"),
-	N_("Find next"),
-    };
-
-    return gtk_button_new_with_label(_(button_strings[code]));
+    GtkWidget *h = gtk_hseparator_new();
+    
+    gtk_box_pack_start(GTK_BOX(vbox), h, FALSE, FALSE, 0);
+    gtk_widget_show(h);
 }
 
-GtkWidget *
-gtk_spin_button_new_with_range (double lo, double hi, double step)
-{
-    GtkAdjustment *adj;
-    GtkWidget *sb;
-
-    adj = (GtkAdjustment *) gtk_adjustment_new(lo, lo, hi, step, 10 * step, 0);
-    sb = gtk_spin_button_new(adj, 0, 0);
-
-    return sb;
-}
-
-#endif
-
 /* Various buttons, usable in several sorts of dialogs */
 
 GtkWidget *context_help_button (GtkWidget *hbox, int cmdcode)
@@ -69,25 +42,41 @@ GtkWidget *context_help_button (GtkWidget *hbox, int cmdcode)
 
     w = standard_button(GTK_STOCK_HELP);
     GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 0);
-    g_signal_connect(G_OBJECT(w), "clicked", 
-		     G_CALLBACK(context_help), 
-		     GINT_TO_POINTER(cmdcode));
+    gtk_container_add(GTK_CONTAINER(hbox), w);
+    gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(hbox),
+				       w, TRUE);
+    if (cmdcode >= 0) {
+	g_signal_connect(G_OBJECT(w), "clicked", 
+			 G_CALLBACK(context_help), 
+			 GINT_TO_POINTER(cmdcode));
+    }
     gtk_widget_show(w);
 
     return w;
 }
 
-GtkWidget *cancel_delete_button (GtkWidget *hbox, GtkWidget *targ)
+static void set_canceled (GtkWidget *w, int *c)
+{
+    if (c != NULL) *c = 1;
+}
+
+GtkWidget *cancel_delete_button (GtkWidget *hbox, GtkWidget *targ,
+				 int *canceled)
 {
     GtkWidget *w;
 
     w = standard_button(GTK_STOCK_CANCEL);
     GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 0);
+    if (canceled != NULL) {
+	g_signal_connect(G_OBJECT(w), "clicked", 
+			 G_CALLBACK(set_canceled), 
+			 canceled);
+    }
     g_signal_connect(G_OBJECT(w), "clicked", 
 		     G_CALLBACK(delete_widget), 
 		     targ);
+	
     gtk_widget_show(w);
 
     return w;
@@ -98,14 +87,12 @@ static void opt_invalid (GtkWidget *w, int *opt)
     *opt = -1;
 }
 
-#ifndef OLD_GTK
 static void maybe_opt_invalid (GtkDialog *d, int resp, int *opt)
 {
     if (resp == GTK_RESPONSE_NONE || resp == GTK_RESPONSE_DELETE_EVENT) {
 	*opt = -1;
     }
 }
-#endif
 
 GtkWidget *cancel_options_button (GtkWidget *hbox, GtkWidget *targ,
 				  int *opt)
@@ -119,15 +106,9 @@ GtkWidget *cancel_options_button (GtkWidget *hbox, GtkWidget *targ,
 	g_signal_connect(G_OBJECT(w), "clicked", 
 			 G_CALLBACK(opt_invalid), 
 			 opt);
-#ifdef OLD_GTK
-	g_signal_connect(G_OBJECT(targ), "delete_event", 
-			 G_CALLBACK(opt_invalid), 
-			 opt);
-#else
 	g_signal_connect(GTK_DIALOG(targ), "response", 
 			 G_CALLBACK(maybe_opt_invalid), 
 			 opt);
-#endif
     }
 
     g_signal_connect(G_OBJECT(w), "clicked", 
@@ -144,7 +125,7 @@ GtkWidget *ok_button (GtkWidget *hbox)
 
     w = standard_button(GTK_STOCK_OK);
     GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 0);
+    gtk_container_add(GTK_CONTAINER(hbox), w);
 
     return w;
 }
@@ -155,7 +136,18 @@ GtkWidget *apply_button (GtkWidget *hbox)
 
     w = standard_button(GTK_STOCK_APPLY);
     GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 0);
+    gtk_container_add(GTK_CONTAINER(hbox), w);
+
+    return w;
+}
+
+GtkWidget *cancel_button (GtkWidget *hbox)
+{
+    GtkWidget *w;
+
+    w = standard_button(GTK_STOCK_CANCEL);
+    GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
+    gtk_container_add(GTK_CONTAINER(hbox), w);
 
     return w;
 }
@@ -166,7 +158,7 @@ GtkWidget *next_button (GtkWidget *hbox)
 
     w = standard_button(GTK_STOCK_GO_FORWARD);
     GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 0);
+    gtk_container_add(GTK_CONTAINER(hbox), w);
 
     return w;
 }
@@ -177,7 +169,7 @@ GtkWidget *back_button (GtkWidget *hbox)
 
     w = standard_button(GTK_STOCK_GO_BACK);
     GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 0);
+    gtk_container_add(GTK_CONTAINER(hbox), w);
 
     return w;
 }
@@ -186,28 +178,16 @@ static void set_dialog_border_widths (GtkWidget *dlg)
 {
     int w1 = 10, w2 = 5;
 
-#ifdef OLD_GTK
-    gtk_container_border_width(GTK_CONTAINER 
-			       (GTK_DIALOG(dlg)->vbox), w1);
-    gtk_container_border_width(GTK_CONTAINER 
-			       (GTK_DIALOG(dlg)->action_area), w2);
-#else
     gtk_container_set_border_width(GTK_CONTAINER 
 				   (GTK_DIALOG(dlg)->vbox), w1);
     gtk_container_set_border_width(GTK_CONTAINER 
 				   (GTK_DIALOG(dlg)->action_area), w2);
-#endif
-
     gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dlg)->vbox), w2);
 }
 
 static void gretl_dialog_set_resizeable (GtkWidget *w, gboolean s)
 {
-#ifdef OLD_GTK
-    gtk_window_set_policy(GTK_WINDOW(w), FALSE, s, !s);
-#else
     gtk_window_set_resizable(GTK_WINDOW(w), s);
-#endif
 }
 
 static GtkWidget *current_dialog;
@@ -234,9 +214,8 @@ static gint dialog_unblock (GtkWidget *w, gpointer p)
 static gint dialog_set_destruction (GtkWidget *w, gpointer p)
 {
     gtk_window_set_transient_for(GTK_WINDOW(w), GTK_WINDOW(p));
-#ifndef OLD_GTK
     gtk_window_set_destroy_with_parent(GTK_WINDOW(w), TRUE);
-#endif
+
     return FALSE;
 }
 
@@ -249,7 +228,8 @@ GtkWidget *gretl_dialog_new (const char *title, GtkWidget *parent,
 	gtk_window_set_title(GTK_WINDOW(d), title);
     }
 
-    gtk_box_set_homogeneous(GTK_BOX(GTK_DIALOG(d)->action_area), TRUE);
+    gtk_button_box_set_layout(GTK_BUTTON_BOX(GTK_DIALOG(d)->action_area), 
+			      GTK_BUTTONBOX_END);
     set_dialog_border_widths(d);
     gtk_window_set_position(GTK_WINDOW(d), GTK_WIN_POS_MOUSE);
 
@@ -295,6 +275,7 @@ struct dialog_t_ {
     GtkWidget *popup;
     gpointer data;
     gint code;
+    gint blocking;
     gretlopt opt;
 };
 
@@ -304,6 +285,15 @@ static void destroy_dialog_data (GtkWidget *w, gpointer data)
 {
     dialog_t *d = (dialog_t *) data;
 
+    if (d->blocking) {
+	gtk_main_quit();
+    }
+
+    if (d->code == GENR_RANDOM) {
+	/* pointer to double */
+	free(d->data);
+    } 
+
     g_free(d); 
 
     open_edit_dialog = NULL;
@@ -313,8 +303,16 @@ static void destroy_dialog_data (GtkWidget *w, gpointer data)
     if (active_edit_text) active_edit_text = NULL;
 }
 
+static void cancel_on_delete (GtkDialog *d, int resp, int *c)
+{
+    if (resp == GTK_RESPONSE_NONE || resp == GTK_RESPONSE_DELETE_EVENT) {
+	*c = 1;
+    }
+}
+
 static dialog_t *
-dialog_data_new (gpointer data, gint code, const char *title)
+dialog_data_new (gpointer data, gint code, const char *title,
+		 int *canceled)
 {
     dialog_t *d = mymalloc(sizeof *d);
 
@@ -328,13 +326,22 @@ dialog_data_new (gpointer data, gint code, const char *title)
     d->dialog = gtk_dialog_new();
     d->popup = NULL;
 
+    if (canceled != NULL) {
+	g_signal_connect(G_OBJECT(d->dialog), "delete_event",
+			 G_CALLBACK(cancel_on_delete), 
+			 canceled);
+	d->blocking = 1;
+    } else {
+	d->blocking = 0;
+    }
+
     gtk_window_set_title(GTK_WINDOW(d->dialog), title);
 
     gtk_box_set_homogeneous(GTK_BOX 
 			    (GTK_DIALOG(d->dialog)->action_area), TRUE);
     gtk_window_set_position(GTK_WINDOW(d->dialog), GTK_WIN_POS_MOUSE);
 
-    g_signal_connect(G_OBJECT (d->dialog), "destroy", 
+    g_signal_connect(G_OBJECT(d->dialog), "destroy", 
 		     G_CALLBACK(destroy_dialog_data), d);
 
     return d;
@@ -345,24 +352,63 @@ void close_dialog (dialog_t *dlg)
     gtk_widget_destroy(dlg->dialog);
 }
 
+/* saved material from "complex" edit dialog */
+
+static int edit_save_code;
+static char *edit_save_buf;
+
+static void edit_save_buf_clear (void)
+{
+    g_free(edit_save_buf);
+    edit_save_buf = NULL;
+    edit_save_code = 0;
+}
+
+static void set_edit_save_buf (const char *buf, int code)
+{
+    edit_save_buf_clear();
+
+    if (buf != NULL) {
+	edit_save_buf = g_strdup(buf);
+	edit_save_code = code;
+    }
+}
+
+static int dlg_text_set_previous (dialog_t *d)
+{
+    if (d->code == edit_save_code && 
+	edit_save_buf != NULL) {
+	textview_set_text(d->edit, edit_save_buf);
+	return 1;
+    } else {
+	return 0;
+    }
+}
+
+/* end edit saver apparatus */
+
 gchar *edit_dialog_special_get_text (dialog_t *dlg)
 {
     gchar *buf;
 
-#ifdef OLD_GTK
-    buf = gtk_editable_get_chars(GTK_EDITABLE(dlg->edit), 0, -1);
-#else
-    buf = textview_get_text(GTK_TEXT_VIEW(dlg->edit));
-#endif
+    if (dlg == NULL) {
+	edit_save_buf_clear();
+	return NULL;
+    }
+
+    buf = textview_get_text(dlg->edit);
 
     if (buf != NULL && *buf == '\0') {
+	/* nothing was entered */
 	g_free(buf);
 	buf = NULL;
     }
 
     if (buf == NULL) {
 	gtk_widget_destroy(dlg->dialog);
-    }
+    } 
+
+    set_edit_save_buf(buf, dlg->code);
 
     return buf;
 }
@@ -374,7 +420,7 @@ const gchar *edit_dialog_get_text (dialog_t *dlg)
     buf = gtk_entry_get_text(GTK_ENTRY(dlg->edit));
 
     if (buf == NULL || *buf == '\0') {
-	if (dlg->code != CORRGM) {
+	if (dlg->code != CORRGM && dlg->code != CREATE_DATASET) {
 	    gtk_widget_destroy(dlg->dialog);
 	}
 	return NULL;
@@ -403,44 +449,20 @@ static void dialog_table_setup (dialog_t *dlg, int hsize)
     GtkWidget *sw;
 
     sw = gtk_scrolled_window_new (NULL, NULL);
-#ifdef OLD_GTK
-    gtk_widget_set_usize(sw, hsize, 200);
-#else
     gtk_widget_set_size_request(sw, hsize, 200);
-#endif
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg->dialog)->vbox), 
 		       sw, TRUE, TRUE, FALSE);
-    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
-				    GTK_POLICY_AUTOMATIC,
-				    GTK_POLICY_AUTOMATIC);
-#ifndef OLD_GTK
-    gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
-					 GTK_SHADOW_IN);
-#endif
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (sw),
+				   GTK_POLICY_AUTOMATIC,
+				   GTK_POLICY_AUTOMATIC);
+    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW (sw),
+					GTK_SHADOW_IN);
     gtk_container_add(GTK_CONTAINER(sw), dlg->edit); 
     gtk_widget_show(dlg->edit);
     gtk_widget_show(sw);
 }
 
-#ifdef OLD_GTK
-
-static GtkWidget *text_edit_new (int *hsize)
-{
-    GtkWidget *tbuf;
-
-    tbuf = gtk_text_new(NULL, NULL);
-
-    gtk_text_set_editable(GTK_TEXT(tbuf), TRUE);
-    gtk_text_set_word_wrap(GTK_TEXT(tbuf), FALSE);
-    *hsize *= gdk_char_width(fixed_font, 'W');
-    *hsize += 48;
-
-    return tbuf;
-}
-
-#else
-
-static GtkWidget *text_edit_new (int *hsize)
+static GtkWidget *dlg_text_edit_new (int *hsize, gboolean s)
 {
     GtkTextBuffer *tbuf;
     GtkWidget *tview;
@@ -455,13 +477,27 @@ static GtkWidget *text_edit_new (int *hsize)
     gtk_widget_modify_font(GTK_WIDGET(tview), fixed_font);
     *hsize *= get_char_width(tview);
     *hsize += 48;
-    gtk_text_view_set_editable(GTK_TEXT_VIEW(tview), TRUE);
-    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(tview), TRUE);
+    gtk_text_view_set_editable(GTK_TEXT_VIEW(tview), s);
+    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(tview), s);
 
     return tview;
 }
 
-#endif
+static void dlg_text_set_from_sys (gretl_equation_system *sys,
+				   dialog_t *d)
+{
+    const char *buf;
+    PRN *prn;
+
+    if (bufopen(&prn)) {
+	return;
+    }
+
+    print_equation_system_info(sys, datainfo, OPT_NONE, prn);
+    buf = gretl_print_get_buffer(prn);
+    textview_set_text(d->edit, buf);
+    gretl_print_destroy(prn);
+}
 
 enum {
     ADD_EQN,
@@ -476,11 +512,7 @@ static int edit_has_list (GtkWidget *w, int i)
     gchar *buf;
     int ret = 0;
 
-#ifdef OLD_GTK
-    buf = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
-#else
-    buf = textview_get_text(GTK_TEXT_VIEW(w));
-#endif
+    buf = textview_get_text(w);
 
     if (buf != NULL) {
 	if (i == ADD_ENDO_LIST) {
@@ -513,19 +545,12 @@ static gint edit_popup_click (GtkWidget *w, dialog_t *d)
     }
 
     if (ins != NULL) {
-#ifndef OLD_GTK
 	GtkTextBuffer *tbuf;
 	GtkTextIter pos;
 
 	tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(d->edit));
 	gtk_text_buffer_get_end_iter(tbuf, &pos);
 	gtk_text_buffer_insert(tbuf, &pos, ins, strlen(ins));
-#else
-	int pos = gtk_editable_get_position(GTK_EDITABLE(d->edit));
-
-	gtk_editable_insert_text(GTK_EDITABLE(d->edit), 
-				 ins, strlen(ins), &pos);
-#endif
     }
 
     gtk_widget_destroy(d->popup);
@@ -566,12 +591,7 @@ static GtkWidget *build_edit_popup (dialog_t *d)
 	g_signal_connect(G_OBJECT(item), "activate",
 			 G_CALLBACK(edit_popup_click), d);
 	gtk_widget_show(item);
-#ifndef OLD_GTK
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-#else
-	GTK_WIDGET_SET_FLAGS(item, GTK_SENSITIVE | GTK_CAN_FOCUS);
-	gtk_menu_append(GTK_MENU(menu), item);
-#endif
     }
 
     return menu;
@@ -670,13 +690,33 @@ static gboolean opt_v_callback (GtkWidget *w, dialog_t *dlg)
     return FALSE;
 }
 
-static void verbosity_switch (GtkWidget *vbox, dialog_t *dlg)
+static gboolean opt_r_callback (GtkWidget *w, dialog_t *dlg)
+{
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
+	dlg->opt |= OPT_R;
+    } else {
+	dlg->opt &= ~OPT_R;
+    }
+
+    return FALSE;
+}
+
+static void dialog_option_switch (GtkWidget *vbox, dialog_t *dlg,
+				  gretlopt opt)
 {
     GtkWidget *b, *hbox;
 
-    b = gtk_check_button_new_with_label(_("Show details of iterations"));
-    g_signal_connect(G_OBJECT(b), "toggled", 
-		     G_CALLBACK(opt_v_callback), dlg);
+    if (opt == OPT_V) {
+	b = gtk_check_button_new_with_label(_("Show details of iterations"));
+	g_signal_connect(G_OBJECT(b), "toggled", 
+			 G_CALLBACK(opt_v_callback), dlg);
+    } else if (opt == OPT_R) {
+	b = gtk_check_button_new_with_label(_("Robust standard errors"));
+	g_signal_connect(G_OBJECT(b), "toggled", 
+			 G_CALLBACK(opt_r_callback), dlg);
+    } else {
+	return;
+    }
 
     hbox = gtk_hbox_new(FALSE, 5);
     gtk_box_pack_start(GTK_BOX(hbox), b, TRUE, TRUE, 5);
@@ -686,19 +726,28 @@ static void verbosity_switch (GtkWidget *vbox, dialog_t *dlg)
     gtk_widget_show(hbox);
 }
 
-static void system_estimator_list (GtkWidget *vbox, gpointer data)
+static void system_estimator_list (GtkWidget *vbox, dialog_t *d)
 {
+    gretl_equation_system *sys = NULL;
     GList *items = NULL;
     GtkWidget *w, *hbox;
     gchar **strs;
+    int method = -1;
     int i;
 
-    strs = create_strings_array(SYS_MAX);
+    if (d->data != NULL) {
+	sys = (gretl_equation_system *) d->data;
+    }
+
+    strs = strings_array_new(SYS_MAX);
 
     for (i=SYS_SUR; i<SYS_MAX; i++) {
 	strs[i] = g_strdup_printf("%s (%s)", _(system_method_full_string(i)),
 				  system_method_short_string(i));
 	items = g_list_append(items, strs[i]);
+	if (sys != NULL && sys->method == i) {
+	    method = i;
+	}
     }
 
     hbox = gtk_hbox_new(FALSE, 5);
@@ -714,11 +763,33 @@ static void system_estimator_list (GtkWidget *vbox, gpointer data)
     gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO(w)->entry), FALSE);
     g_signal_connect(G_OBJECT(w), "destroy", G_CALLBACK(free_sys_strings), strs);
     g_signal_connect(G_OBJECT(GTK_COMBO(w)->entry), "changed",
-		     G_CALLBACK(set_sys_method), data);
+		     G_CALLBACK(set_sys_method), d);
+
+    if (method >= 0) {
+	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(w)->entry), strs[method]);
+    }
+
     gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 5);
     gtk_widget_show(w);  
 }
 
+static void dlg_display_sys (dialog_t *d)
+{
+    gretl_equation_system *sys = d->data;
+    GtkWidget *w;
+    int hsize = 62;
+
+    w = gtk_label_new(sys->name);
+    gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_CENTER);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(d->dialog)->vbox), 
+		       w, TRUE, TRUE, 10);
+    gtk_widget_show(w);
+
+    d->edit = dlg_text_edit_new(&hsize, FALSE);
+    dialog_table_setup(d, hsize);
+    dlg_text_set_from_sys(sys, d); 
+}
+
 static void raise_and_focus_dialog (GtkEditable *editable, gpointer p)
 {
     dialog_t *d = (dialog_t *) p;
@@ -733,8 +804,10 @@ static int edit_dialog_help_code (int ci, void *p)
 {
     int hc = ci;
 
-    if (ci == PRINT || ci == CREATE_USERDIR ||
-	ci == GENR_NORMAL || ci == GENR_UNIFORM) {
+    if (ci == SYSTEM && p != NULL) {
+	hc = 0;
+    } else if (ci == PRINT || ci == CREATE_USERDIR || 
+	       ci == CREATE_DATASET || ci == MINIBUF) {
 	hc = 0;
     } else if (ci == RESTRICT) {
 	windata_t *vwin = (windata_t *) p;
@@ -751,21 +824,34 @@ static int edit_dialog_help_code (int ci, void *p)
     return hc;
 }
 
+static void clear_dlg_previous (GtkWidget *w, dialog_t *d)
+{
+    edit_save_buf_clear();
+    textview_set_text(d->edit, NULL);
+}
+
+static void edit_dialog_ok (GtkWidget *w, dialog_t *d)
+{
+    gtk_widget_destroy(d->dialog);
+}
+
 void edit_dialog (const char *diagtxt, const char *infotxt, const char *deftext, 
 		  void (*okfunc)(), void *okptr,
-		  guint cmdcode, guint varclick)
+		  guint cmdcode, guint varclick, 
+		  int *canceled)
 {
     dialog_t *d;
-    GtkWidget *tempwid;
+    GtkWidget *w;
     GtkWidget *top_vbox, *button_box;
     int hlpcode, modal = 0;
+    int clear = 0;
 
-    if (open_edit_dialog != NULL) {
+    if (open_edit_dialog != NULL && cmdcode != MINIBUF) {
 	gdk_window_raise(open_edit_dialog->window);
 	return;
     }
 
-    d = dialog_data_new(okptr, cmdcode, diagtxt);
+    d = dialog_data_new(okptr, cmdcode, diagtxt, canceled);
     if (d == NULL) return;
 
     open_edit_dialog = d->dialog;
@@ -775,37 +861,51 @@ void edit_dialog (const char *diagtxt, const char *infotxt, const char *deftext,
     /* convenience pointers */
     top_vbox = GTK_DIALOG(d->dialog)->vbox;
     button_box = GTK_DIALOG(d->dialog)->action_area;
+    gtk_button_box_set_layout(GTK_BUTTON_BOX(button_box), 
+			      GTK_BUTTONBOX_END);
+    
 
-    if (cmdcode == NLS || cmdcode == MLE || 
+    if (cmdcode == SYSTEM && d->data != NULL) {
+	/* revisiting saved equation system */
+	dlg_display_sys(d);
+    } else if (cmdcode == NLS || cmdcode == MLE || 
 	cmdcode == SYSTEM || cmdcode == RESTRICT) {
 	int hsize = 62;
 	gchar *lbl;
 
 	lbl = g_strdup_printf("%s\n%s", infotxt,
 			      _("(Please refer to Help for guidance)"));
-	tempwid = gtk_label_new(lbl);
-	gtk_label_set_justify(GTK_LABEL(tempwid), GTK_JUSTIFY_CENTER);
-	gtk_box_pack_start(GTK_BOX(top_vbox), tempwid, TRUE, TRUE, 10);
-	gtk_widget_show(tempwid);
+	w = gtk_label_new(lbl);
+	gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_CENTER);
+	gtk_box_pack_start(GTK_BOX(top_vbox), w, TRUE, TRUE, 10);
+	gtk_widget_show(w);
 	g_free(lbl);
 
-	d->edit = text_edit_new(&hsize);
+	d->edit = dlg_text_edit_new(&hsize, TRUE);
 	dialog_table_setup(d, hsize);
 
+	/* insert previous text, if any and if the command
+	   is the same as previously */
+	if (dlg_text_set_previous(d)) {
+	    clear = 1;
+	}
+
 	if (cmdcode != RESTRICT) {
 	    g_signal_connect(G_OBJECT(d->edit), "button_press_event", 
 			     G_CALLBACK(edit_dialog_popup_handler), d);
 	}
     } else {
-	tempwid = gtk_label_new(infotxt);
-	gtk_box_pack_start (GTK_BOX(top_vbox), tempwid, TRUE, TRUE, 5);
-	gtk_widget_show (tempwid);
+	if (infotxt != NULL) {
+	    w = gtk_label_new(infotxt);
+	    gtk_box_pack_start(GTK_BOX(top_vbox), w, TRUE, TRUE, 5);
+	    gtk_widget_show(w);
+	}
 
 	d->edit = gtk_entry_new();
 	gtk_box_pack_start(GTK_BOX(top_vbox), d->edit, TRUE, TRUE, 5);
 
 	/* make the Enter key do the business */
-	if (okfunc) {
+	if (okfunc != NULL) {
 	    g_signal_connect(G_OBJECT(d->edit), "activate", 
 			     G_CALLBACK(okfunc), d);
 	}
@@ -813,7 +913,7 @@ void edit_dialog (const char *diagtxt, const char *infotxt, const char *deftext,
 	if (deftext != NULL && *deftext != '\0') {
 	    gtk_entry_set_text(GTK_ENTRY(d->edit), deftext);
 	    gtk_editable_select_region(GTK_EDITABLE(d->edit), 0, strlen(deftext));
-	}
+	} 
 
 	gtk_widget_show(d->edit);
 
@@ -826,8 +926,9 @@ void edit_dialog (const char *diagtxt, const char *infotxt, const char *deftext,
     } else if (cmdcode == SYSTEM) {
 	system_estimator_list(top_vbox, d);
     } else if (cmdcode == NLS || cmdcode == MLE) {
-	verbosity_switch(top_vbox, d);
-    }
+	dialog_option_switch(top_vbox, d, OPT_V);
+	dialog_option_switch(top_vbox, d, OPT_R);
+    } 
 
     if (varclick == VARCLICK_INSERT_ID) { 
 	active_edit_id = d->edit; 
@@ -835,40 +936,83 @@ void edit_dialog (const char *diagtxt, const char *infotxt, const char *deftext,
 	active_edit_name = d->edit;
     } else if (varclick == VARCLICK_INSERT_TEXT) { 
 	active_edit_text = d->edit;
-    } else {
+    } else if (cmdcode != SYSTEM) {
 	modal = 1;
     }
 
     gtk_widget_grab_focus(d->edit);
 
-    /* Create the "OK" button */
-    tempwid = ok_button(button_box);
-    if (okfunc) {
-	g_signal_connect(G_OBJECT(tempwid), "clicked", 
-			 G_CALLBACK(okfunc), d);
-    }
-    gtk_widget_grab_default(tempwid);
-    gtk_widget_show(tempwid);
+    /* "Clear" button? */
+    if (clear) {
+	w = gtk_button_new_from_stock(GTK_STOCK_CLEAR);
+	gtk_container_add(GTK_CONTAINER(button_box), w);
+	g_signal_connect(G_OBJECT(w), "clicked", 
+			 G_CALLBACK(clear_dlg_previous), d);
+	gtk_widget_show(w);  
+    }    
 
-    /* Create a "Cancel" button */
+    /* "Cancel" button? */
     if (cmdcode != CREATE_USERDIR) {
-	cancel_delete_button(button_box, d->dialog);
+	cancel_delete_button(button_box, d->dialog, canceled);
     }
 
-    /* Create a "Help" button if wanted */
+    /* "OK" button */
+    w = ok_button(button_box);
+    if (okfunc != NULL) {
+	g_signal_connect(G_OBJECT(w), "clicked", 
+			 G_CALLBACK(okfunc), d);
+    } else {
+	g_signal_connect(G_OBJECT(w), "clicked", 
+			 G_CALLBACK(edit_dialog_ok), d);
+    }	
+    gtk_widget_grab_default(w);
+    gtk_widget_show(w);    
+
+    /* "Help" button, if wanted */
     hlpcode = edit_dialog_help_code(cmdcode, okptr);
     if (hlpcode > 0) {
 	context_help_button(button_box, hlpcode);
 	modal = 0;
     }
 
-#ifndef OLD_GTK
     gtk_window_set_destroy_with_parent(GTK_WINDOW(d->dialog), TRUE);
-#endif
 
     gtk_widget_show(d->dialog); 
 
     if (modal) {
 	gretl_set_window_modal(d->dialog);
     }
-} 
+
+    if (d->blocking) {
+	gtk_main();
+    }
+}
+
+char *entry_box_get_trimmed_text (GtkWidget *w)
+{
+    const gchar *s = gtk_entry_get_text(GTK_ENTRY(w));
+    char *ret = NULL;
+    int i, len;
+
+    if (s == NULL || *s == '\0') {
+	return NULL;
+    }
+
+    while (isspace(*s)) s++;
+    if (*s == '\0') {
+	return NULL;
+    }
+
+    len = strlen(s);
+    for (i=len-1; i>0; i--) {
+	if (!isspace(s[i])) break;
+	len--;
+    }
+
+    if (len > 0) {
+	ret = g_strndup(s, len);
+    }
+
+    return ret;
+}
+
