@@ -31,13 +31,18 @@ GtkWidget *gretl_dialog_new (const char *title, GtkWidget *parent,
 
 int maybe_raise_dialog (void);
 
+void vbox_add_hsep (GtkWidget *vbox);
+
 GtkWidget *context_help_button (GtkWidget *hbox, int cmdcode);
 
-GtkWidget *cancel_delete_button (GtkWidget *hbox, GtkWidget *targ);
+GtkWidget *cancel_delete_button (GtkWidget *hbox, GtkWidget *targ,
+				 int *canceled);
 
 GtkWidget *cancel_options_button (GtkWidget *hbox, GtkWidget *targ,
 				  int *opt);
 
+GtkWidget *cancel_button (GtkWidget *hbox);
+
 GtkWidget *ok_button (GtkWidget *hbox);
 
 GtkWidget *apply_button (GtkWidget *hbox);
@@ -48,7 +53,8 @@ GtkWidget *back_button (GtkWidget *hbox);
 
 void edit_dialog (const char *diagtxt, const char *infotxt, const char *deftext, 
 		  void (*okfunc)(), void *okptr,
-		  guint hlpcode, guint varclick);
+		  guint hlpcode, guint varclick,
+		  int *canceled);
 
 const gchar *edit_dialog_get_text (dialog_t *dlg);
 
@@ -62,11 +68,6 @@ gpointer edit_dialog_get_data (dialog_t *dlg);
 
 void close_dialog (dialog_t *dlg);
 
-#ifdef OLD_GTK
-GtkWidget *standard_button (int code);
-
-GtkWidget *gtk_spin_button_new_with_range (double lo, double hi, 
-					   double step);
-#endif
+char *entry_box_get_trimmed_text (GtkWidget *w);
 
 #endif /* DLGUTILS_H */
