@@ -78,7 +78,7 @@ static const char *file_sections[] = {
     "recent_script_files"
 };
 
-#if defined(USE_GNOME) && !defined(OLD_GTK)
+#if defined(USE_GNOME)
 
 static void printfilelist (int filetype, GConfClient *client)
 {
@@ -335,19 +335,14 @@ static void clear_files_list (int filetype, char **filep)
     gchar itempath[80];
     const gchar *fpath[] = {
 	N_("/File/Open data"), 
-	N_("/Session"),
-	N_("/File/Open command file")
+	N_("/File/Session files"),
+	N_("/File/Script files")
     };
     int i;
 
     for (i=0; i<MAXRECENT; i++) {
-#ifndef OLD_GTK
 	sprintf(itempath, "%s/%d. %s", fpath[filetype],
 		i+1, endbit(tmpname, filep[i], 0)); 
-#else
-	sprintf(itempath, "%s/%d. %s", fpath[filetype],
-		i+1, endbit(tmpname, filep[i], -1)); 
-#endif
 	w = gtk_item_factory_get_widget(mdata->ifac, itempath);
 	if (w != NULL) {
 	    gtk_item_factory_delete_item(mdata->ifac, itempath);
@@ -389,19 +384,13 @@ void mkfilelist (int filetype, char *fname_in)
     char **filep;
     char *fname;
     int i, match = -1;
-#ifndef OLD_GTK
     char trfname[MAXLEN];
-#endif
 
     cut_multiple_slashes(fname_in);
 
-#ifndef OLD_GTK
     strcpy(trfname, fname_in);
     my_filename_to_utf8(trfname);
     fname = trfname;
-#else
-    fname = fname_in;
-#endif
 
     filep = get_file_list(filetype);
     if (filep == NULL) {
@@ -526,7 +515,7 @@ static void copy_sys_filename (char *targ, const char *src)
 {
     strcpy(targ, src);
     /* check this: very confusing! */
-#if !defined(OLD_GTK) && !defined(G_OS_WIN32)
+#if !defined(G_OS_WIN32)
     my_filename_from_utf8(targ);
 #endif
 }    
@@ -534,8 +523,8 @@ static void copy_sys_filename (char *targ, const char *src)
 static void set_data_from_filelist (gpointer data, guint i, 
 				    GtkWidget *widget)
 {
-    copy_sys_filename(trydatfile, datap[i]);
-    if (strstr(trydatfile, ".csv")) {
+    copy_sys_filename(tryfile, datap[i]);
+    if (strstr(tryfile, ".csv")) {
 	delimiter_dialog(NULL);
     }
     verify_open_data(NULL, 0);
@@ -544,14 +533,14 @@ static void set_data_from_filelist (gpointer data, guint i,
 static void set_session_from_filelist (gpointer data, guint i, 
 				       GtkWidget *widget)
 {
-    copy_sys_filename(tryscript, sessionp[i]);
-    verify_open_session(NULL);
+    copy_sys_filename(tryfile, sessionp[i]);
+    verify_open_session();
 }
 
 static void set_script_from_filelist (gpointer data, guint i, 
 				      GtkWidget *widget)
 {
-    copy_sys_filename(tryscript, scriptp[i]);
+    copy_sys_filename(tryfile, scriptp[i]);
     do_open_script();
 }
 
@@ -561,14 +550,14 @@ static void real_add_files_to_menus (int ftype)
     void (*callfunc)() = NULL;
     GtkItemFactoryEntry fileitem;
     const gchar *msep[] = {
-	N_("/File/Open data/sep"),
-	N_("/Session/sep"),
-	N_("/File/Open command file/sep")
+	"/File/Open data/sep",
+	"/File/Session files/sep",
+	"/File/Script files/sep"
     };
     const gchar *mpath[] = {
 	N_("/File/Open data"),
-	N_("/Session"),
-	N_("/File/Open command file")
+	N_("/File/Session files"),
+	N_("/File/Script files")
     };
     int jmin = 0, jmax = 3;
     int i, j;
