@@ -17,7 +17,7 @@
  *
  */
 
-/* fileselect.c for gretl -- use the gtkextra file selector under X11,
+/* fileselect.c for gretl -- use the gtk file chooser under X11,
    the native MS file selector under MS Windows */
 
 #include "gretl.h"
@@ -28,22 +28,11 @@
 #include "textutil.h"
 #include "filelists.h"
 #include "fileselect.h"
-
-#if (GTK_MAJOR_VERSION >= 2) && (GTK_MINOR_VERSION >= 4)
-# ifndef G_OS_WIN32
-#  define USE_GTK_CHOOSER
-# endif
-#endif
-
-#ifdef OLD_GTK
-# include "menustate.h"
-#endif
+#include "fnsave.h"
+#include "database.h"
 
 #define IS_DAT_ACTION(i) (i == SAVE_DATA || \
                           i == SAVE_DATA_AS || \
-                          i == SAVE_GZDATA || \
-                          i == SAVE_BIN1 || \
-                          i == SAVE_BIN2 || \
                           i == OPEN_DATA)
 
 #define OPEN_DATA_ACTION(i)  (i == OPEN_DATA || \
@@ -54,7 +43,8 @@
                               i == OPEN_GNUMERIC || \
 	                      i == OPEN_EXCEL || \
                               i == OPEN_WF1 || \
-                              i == OPEN_DTA)
+                              i == OPEN_DTA || \
+                              i == OPEN_JMULTI)
 
 #define APPEND_DATA_ACTION(i) (i == APPEND_DATA || \
                                i == APPEND_CSV || \
@@ -63,7 +53,8 @@
                                i == APPEND_EXCEL || \
                                i == APPEND_ASCII || \
                                i == APPEND_WF1 || \
-                               i == APPEND_DTA)
+                               i == APPEND_DTA || \
+                               i == APPEND_JMULTI)
 
 #define SAVE_GRAPH_ACTION(i) (i == SAVE_GNUPLOT || \
                               i == SAVE_THIS_GRAPH || \
@@ -75,11 +66,8 @@
 #define EXPORT_ACTION(a,s) ((a == EXPORT_OCTAVE || \
                              a == EXPORT_R || \
                              a == EXPORT_CSV || \
-                             a == EXPORT_DAT) && s != FSEL_DATA_PRN)
-
-#ifdef REMEMBER_DIR
-static char remember_dir[MAXLEN];
-#endif
+                             a == EXPORT_DAT || \
+                             a == EXPORT_JM) && s != FSEL_DATA_PRN)
 
 struct extmap {
     int action;
@@ -89,9 +77,6 @@ struct extmap {
 static struct extmap action_map[] = {
     { SAVE_DATA,         ".gdt" },
     { SAVE_DATA_AS,      ".gdt" },
-    { SAVE_GZDATA,       ".gdt" },
-    { SAVE_BIN1,         ".gdt" },
-    { SAVE_BIN2,         ".gdt" },
     { SAVE_DBDATA,       ".bin" },
     { SAVE_SCRIPT,       ".inp" },
     { SAVE_CONSOLE,      ".inp" },
@@ -100,6 +85,7 @@ static struct extmap action_map[] = {
     { SAVE_BOXPLOT_EPS,  ".eps" },
     { SAVE_BOXPLOT_PS,   ".ps" },
     { SAVE_BOXPLOT_XPM,  ".xpm" },
+    { SAVE_FUNCTIONS,    ".gfn" },
     { EXPORT_CSV,        ".csv" },
     { EXPORT_R,          ".R" },
     { OPEN_OCTAVE,       ".m" },
@@ -126,6 +112,10 @@ static struct extmap action_map[] = {
     { APPEND_WF1,        ".wf1" },
     { OPEN_DTA,          ".dta" },
     { APPEND_DTA,        ".dta" },
+    { OPEN_JMULTI,       ".dat" },
+    { APPEND_JMULTI,     ".dat" },
+    { OPEN_RATS_DB,      ".rat" },
+    { OPEN_PCGIVE_DB,    ".bn7" },
     { FILE_OP_MAX,       NULL }
 };
 
@@ -134,14 +124,14 @@ static gretlopt save_action_to_opt (int action, gpointer p)
     gretlopt opt = OPT_NONE;
 
     switch (action) {
-    case SAVE_GZDATA:   opt = OPT_Z; break;
-    case SAVE_BIN1:     opt = OPT_S; break;
-    case SAVE_BIN2:     opt = OPT_O; break;
+    case SAVE_DATA:     opt = OPT_Z; break;
+    case SAVE_DATA_AS:  opt = OPT_Z; break;
     case SAVE_DBDATA:   opt = OPT_D; break;
     case EXPORT_OCTAVE: opt = OPT_M; break;
     case EXPORT_R:      opt = OPT_R; break;
     case EXPORT_CSV:    opt = OPT_C; break;
     case EXPORT_DAT:    opt = OPT_G; break; /* PcGive */
+    case EXPORT_JM:     opt = OPT_J; break; /* JMulti */
     default: break;
     }
 
@@ -175,7 +165,7 @@ static int dat_ext (const char *str, int showerr)
 
     suff = strrchr(str, '.');
 
-    if (suff != NULL && (!strcmp(suff, ".dat") || !strcmp(suff, ".gdt"))) {
+    if (suff != NULL && !strcmp(suff, ".gdt")) {
 	if (showerr) {
 	    errbox(_("The suffix you selected should be used\n"
 		   "only for gretl datafiles"));
@@ -190,10 +180,6 @@ static const char *get_ext (int action, gpointer data)
 {
     const char *s = NULL;
 
-    if (using_olddat() && IS_DAT_ACTION(action)) { 
-	return ".dat";
-    }
-
     if (action == SAVE_GNUPLOT || action == SAVE_THIS_GRAPH) {
 	GPT_SPEC *plot = (GPT_SPEC *) data;
 	s = get_gp_ext(plot->termtype);
@@ -224,7 +210,7 @@ static int check_maybe_add_ext (char *fname, int action, gpointer data)
 
     /* don't mess if the fname is really a dir */
     if (isdir(fname)) {
-	return 1;
+	return (action != SET_DIR);
     }
 
     /* don't mess with the name of a previously existing file */
@@ -259,37 +245,30 @@ static void script_window_update (windata_t *vwin, const char *fname)
     strcpy(vwin->fname, fname);
     g_free(title);
 
-    if (vwin->role == VIEW_LOG || vwin->role == VIEW_SCRIPT) {
+    if (vwin->role == VIEW_LOG || vwin->role == VIEW_SCRIPT ||
+	vwin->role == VIEW_FUNC_CODE) {
+	/* change role of window for editing */
 	vwin->role = EDIT_SCRIPT;
     }
 
+    mark_content_saved(vwin);
+
     /* make the window editable */
-#ifndef OLD_GTK
     if (!gtk_text_view_get_editable(GTK_TEXT_VIEW(vwin->w))) {
-	file_view_set_editable(vwin);
+	view_window_set_editable(vwin);
     }
-#else
-    if (vwin->role == EDIT_SCRIPT) {
-	file_view_set_editable(vwin);
-    } 
-#endif
 }
 
-static void save_editable_content (int action, const char *fname,
-				   windata_t *vwin)
+static void 
+save_editable_content (int action, const char *fname, windata_t *vwin)
 {
     FILE *fp;
     gchar *buf;
-#if defined(ENABLE_NLS) && !defined(OLD_GTK)
+#ifdef ENABLE_NLS
     gchar *trbuf;
 #endif
 
-#ifndef OLD_GTK
-    buf = textview_get_text(GTK_TEXT_VIEW(vwin->w));
-#else
-    buf = gtk_editable_get_chars(GTK_EDITABLE(vwin->w), 0, -1);
-#endif
-
+    buf = textview_get_text(vwin->w);
     if (buf == NULL) {
 	errbox("Couldn't retrieve buffer");
 	return;
@@ -302,7 +281,7 @@ static void save_editable_content (int action, const char *fname,
 	return;
     }
 
-#if defined(ENABLE_NLS) && !defined(OLD_GTK)
+#ifdef ENABLE_NLS
     trbuf = my_locale_from_utf8(buf);
     if (trbuf != NULL) {
 	system_print_buf(trbuf, fp);
@@ -324,15 +303,7 @@ static void save_editable_content (int action, const char *fname,
 
 static void set_startdir (char *startdir, int action)
 {
-#ifdef REMEMBER_DIR
-    if (*remember_dir != '\0') {
-	strcpy(startdir, remember_dir);
-    } else {
-	get_default_dir(startdir, action);
-    }
-#else
     get_default_dir(startdir, action);
-#endif
 
 #ifndef G_OS_WIN32
     if (startdir[strlen(startdir) - 1] != '/') {
@@ -359,10 +330,10 @@ static void filesel_open_script (const char *fname)
 {
     int spos;
 
-    strcpy(tryscript, fname);
+    strcpy(tryfile, fname);
 
-    if (view_file(tryscript, 1, 0, 78, 370, EDIT_SCRIPT) != NULL) {
-	strcpy(scriptfile, tryscript);
+    if (view_file(tryfile, 1, 0, 78, 370, EDIT_SCRIPT) != NULL) {
+	strcpy(scriptfile, tryfile);
 	mkfilelist(FILE_LIST_SCRIPT, scriptfile);
 	spos = slashpos(scriptfile);
 	if (spos) {
@@ -374,15 +345,17 @@ static void filesel_open_script (const char *fname)
 
 static void filesel_open_session (const char *fname)
 {
-    int pub = !strncmp(tryscript, paths.scriptdir, strlen(paths.scriptdir));
+    if (is_session_file(fname)) {
+	strcpy(tryfile, fname);
+	verify_open_session();
+    } else {
+	int pub;
 
-    strcpy(tryscript, fname);
-
-    if (saved_objects(tryscript)) {
-	verify_open_session(NULL);
-    } else if (view_file(tryscript, 1, 0, 78, 370, 
-			 pub ? VIEW_SCRIPT : EDIT_SCRIPT)) {
-	strcpy(scriptfile, tryscript);
+	strcpy(tryfile, fname);
+	pub = !strncmp(tryfile, paths.scriptdir, strlen(paths.scriptdir));
+	if (view_file(tryfile, 1, 0, 78, 370, (pub)? VIEW_SCRIPT : EDIT_SCRIPT)) {
+	    strcpy(scriptfile, tryfile);
+	}
     }
 }
 
@@ -400,10 +373,8 @@ static char *suggested_savename (const char *fname)
     sfx = strrchr(s, '.');
 
     if (sfx != NULL && (strlen(sfx) == 4 || !strcmp(sfx, ".gnumeric"))) {
-	const char *test = (using_olddat())? ".dat" : ".gdt";
-
-	if (strcmp(test, sfx)) {
-	    strcpy(sfx, test);
+	if (strcmp(sfx, ".gdt")) {
+	    strcpy(sfx, ".gdt");
 	}
     }
 
@@ -437,6 +408,7 @@ static char *suggested_exportname (const char *fname, int action)
 	    test = ".csv";
 	    break;
 	case EXPORT_DAT:
+	case EXPORT_JM:
 	    test = ".dat";
 	    break;
 	default:
@@ -452,18 +424,6 @@ static char *suggested_exportname (const char *fname, int action)
     return s;
 }
 
-#ifdef REMEMBER_DIR
-static void remember_this_dir (const char *fname)
-{
-    int spos = slashpos(fname);
-    
-    if (spos > 0 && spos < sizeof remember_dir) {
-	*remember_dir = '\0';
-	strncat(remember_dir, fname, spos);
-    }
-}
-#endif
-
 static void
 file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 			      gpointer data)
@@ -484,17 +444,11 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 	}
     } 
 
-#ifdef REMEMBER_DIR
-    if (action != SET_PATH) {
-	remember_this_dir(fname);
-    }
-#endif
-
     if (OPEN_DATA_ACTION(action)) {
-	strcpy(trydatfile, fname);
+	strcpy(tryfile, fname);
 	verify_open_data(NULL, action);
     } else if (APPEND_DATA_ACTION(action)) {
-	strcpy(trydatfile, fname);
+	strcpy(tryfile, fname);
 	do_open_data(NULL, NULL, action);
     } else if (action == OPEN_SCRIPT) {
 	filesel_open_script(fname);
@@ -502,6 +456,10 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 	filesel_open_session(fname);
     } else if (action == OPEN_MARKERS) {
 	do_add_markers(fname);
+    } else if (action == OPEN_RATS_DB) {
+	open_rats_window(fname);
+    } else if (action == OPEN_PCGIVE_DB) {
+	open_bn7_window(fname);
     }
 
     if (action < END_OPEN) {
@@ -510,7 +468,7 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 
     /* now for the save/export options */
 
-    if (action > SAVE_BIN2 && action != EXPORT_DAT && dat_ext(fname, 1)) { 
+    if (action > SAVE_DBDATA && action != EXPORT_DAT && dat_ext(fname, 1)) { 
 	return;
     }
 
@@ -527,12 +485,7 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
     } else if (src == FSEL_DATA_PRN) {
 	filesel_save_prn_buffer((PRN *) data, fname);
     } else if (SAVE_DATA_ACTION(action)) {
-	int overwrite = 0;
-
-	if (!strcmp(fname, paths.datfile) || action == SAVE_DBDATA) {
-	    overwrite = 1;
-	}
-	do_store(fname, save_action_to_opt(action, data), overwrite);
+	do_store(fname, save_action_to_opt(action, data));
     } else if (action == SAVE_GNUPLOT) {
 	int err = 0;
 	GPT_SPEC *plot = (GPT_SPEC *) data;
@@ -556,7 +509,9 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 	}
     } else if (action == SAVE_SESSION) {
 	save_session(fname);
-    } else if (action == SET_PATH) {
+    } else if (action == SAVE_FUNCTIONS) {
+	save_user_functions(fname, data);
+    } else if (action == SET_PROG || action == SET_DIR) {
 	char *strvar = (char *) data;
 
 	filesel_set_path_callback(fname, strvar);
@@ -567,15 +522,17 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
     }
 }
 
+#ifdef G_OS_WIN32 
+
 /* ........................................................... */
 
           /* MS Windows version of file selection code */
 
 /* ........................................................... */
 
-#ifdef G_OS_WIN32 
 
 #include <windows.h>
+#include <shlobj.h>
 
 struct winfilter {
     const char *descrip;
@@ -620,59 +577,54 @@ static struct winfilter get_filter (int action, gpointer data)
     int i;
     struct winfilter filter;
     static struct win32_filtermap map[] = {
-	{SAVE_DATA,    { N_("gretl data files (*.gdt)"), "*.gdt" }},
-	{SAVE_DATA_AS, { N_("gretl data files (*.gdt)"), "*.gdt" }},
-	{SAVE_GZDATA,  { N_("gretl data files (*.gdt)"), "*.gdt" }},
-	{SAVE_BIN1,    { N_("gretl data files (*.gdt)"), "*.gdt" }},
-	{SAVE_BIN2,    { N_("gretl data files (*.gdt)"), "*.gdt" }},
-	{SAVE_DBDATA,  { N_("gretl database files (*.bin)"), "*.bin" }},
-	{SAVE_SCRIPT,  { N_("gretl script files (*.inp)"), "*.inp" }},
-	{SAVE_CONSOLE, { N_("gretl command files (*.inp)"), "*.inp" }},
-	{SAVE_SESSION, { N_("session files (*.gretl)"), "*.gretl" }},
-	{SAVE_BOXPLOT_EPS, { N_("postscript files (*.eps)"), "*.eps" }},
-	{SAVE_BOXPLOT_PS,  { N_("postscript files (*.ps)"), "*.ps" }},
-	{SAVE_GP_CMDS, { N_("gnuplot files (*.plt)"), "*.plt" }},
-	{EXPORT_CSV,   { N_("CSV files (*.csv)"), "*.csv" }},
-	{EXPORT_R,     { N_("GNU R files (*.R)"), "*.R" }},
-	{EXPORT_OCTAVE, { N_("GNU Octave files (*.m)"), "*.m" }},
-	{OPEN_OCTAVE,  { N_("GNU Octave files (*.m)"), "*.m" }},
-	{APPEND_OCTAVE, { N_("GNU Octave files (*.m)"), "*.m" }},
-	{EXPORT_DAT,   { N_("PcGive files (*.dat)"), "*.dat" }},
-	{SAVE_OUTPUT,  { N_("text files (*.txt)"), "*.txt" }},
-	{SAVE_TEX,     { N_("TeX files (*.tex)"), "*.tex" }},
-	{SAVE_RTF,     { N_("RTF files (*.rtf)"), "*.rtf" }},
-	{OPEN_DATA,    { N_("gretl data files (*.gdt)"), "*.gdt" }},
-	{APPEND_DATA,  { N_("gretl data files (*.gdt)"), "*.gdt" }},
-	{OPEN_SCRIPT,  { N_("gretl script files (*.inp)"), "*.inp" }},
-	{OPEN_SESSION, { N_("session files (*.gretl)"), "*.gretl" }},
-	{OPEN_CSV,     { N_("CSV files (*.csv)"), "*.csv" }},
-	{APPEND_CSV,   { N_("CSV files (*.csv)"), "*.csv" }},
-	{OPEN_ASCII,   { N_("ASCII files (*.txt)"), "*.txt" }},
-	{APPEND_ASCII, { N_("ASCII files (*.txt)"), "*.txt" }},
-	{OPEN_BOX,     { N_("BOX data files (*.box)"), "*.box" }},
-	{OPEN_GNUMERIC,   { N_("Gnumeric files (*.gnumeric)"), "*.gnumeric" }},
-	{APPEND_GNUMERIC, { N_("Gnumeric files (*.gnumeric)"), "*.gnumeric" }},
-	{OPEN_EXCEL,   { N_("Excel files (*.xls)"), "*.xls" }},
-	{APPEND_EXCEL, { N_("Excel files (*.xls)"), "*.xls" }},
-	{OPEN_WF1,     { N_("Eviews workfiles (*.wf1)"), "*.wf1" }},
-	{APPEND_WF1,   { N_("Eviews workfiles (*.wf1)"), "*.wf1" }},
-	{OPEN_DTA,     { N_("Stata files (*.dta)"), "*.dta" }},
-	{APPEND_DTA,   { N_("Stata files (*.dta)"), "*.dta" }},
-	{SET_PATH,     { N_("program files (*.exe)"), "*.exe" }}
+	{ SAVE_DATA,    { N_("gretl data files (*.gdt)"), "*.gdt" }},
+	{ SAVE_DATA_AS, { N_("gretl data files (*.gdt)"), "*.gdt" }},
+	{ SAVE_DBDATA,  { N_("gretl database files (*.bin)"), "*.bin" }},
+	{ SAVE_SCRIPT,  { N_("gretl script files (*.inp)"), "*.inp" }},
+	{ SAVE_CONSOLE, { N_("gretl command files (*.inp)"), "*.inp" }},
+	{ SAVE_SESSION, { N_("session files (*.gretl)"), "*.gretl" }},
+	{ SAVE_BOXPLOT_EPS, { N_("postscript files (*.eps)"), "*.eps" }},
+	{ SAVE_BOXPLOT_PS,  { N_("postscript files (*.ps)"), "*.ps" }},
+	{ SAVE_GP_CMDS, { N_("gnuplot files (*.plt)"), "*.plt" }},
+	{ SAVE_FUNCTIONS,   { N_("gretl function files (*.gfn)"), "*.gfn" }},
+	{ EXPORT_CSV,   { N_("CSV files (*.csv)"), "*.csv" }},
+	{ EXPORT_R,     { N_("GNU R files (*.R)"), "*.R" }},
+	{ EXPORT_OCTAVE, { N_("GNU Octave files (*.m)"), "*.m" }},
+	{ OPEN_OCTAVE,  { N_("GNU Octave files (*.m)"), "*.m" }},
+	{ APPEND_OCTAVE, { N_("GNU Octave files (*.m)"), "*.m" }},
+	{ EXPORT_DAT,   { N_("PcGive files (*.dat)"), "*.dat" }},
+	{ EXPORT_JM,    { N_("JMulti files (*.dat)"), "*.dat" }},
+	{ SAVE_OUTPUT,  { N_("text files (*.txt)"), "*.txt" }},
+	{ SAVE_TEX,     { N_("TeX files (*.tex)"), "*.tex" }},
+	{ SAVE_RTF,     { N_("RTF files (*.rtf)"), "*.rtf" }},
+	{ OPEN_DATA,    { N_("gretl data files (*.gdt)"), "*.gdt" }},
+	{ APPEND_DATA,  { N_("gretl data files (*.gdt)"), "*.gdt" }},
+	{ OPEN_SCRIPT,  { N_("gretl script files (*.inp)"), "*.inp" }},
+	{ OPEN_SESSION, { N_("session files (*.gretl)"), "*.gretl" }},
+	{ OPEN_CSV,     { N_("CSV files (*.csv)"), "*.csv" }},
+	{ APPEND_CSV,   { N_("CSV files (*.csv)"), "*.csv" }},
+	{ OPEN_ASCII,   { N_("ASCII files (*.txt)"), "*.txt" }},
+	{ APPEND_ASCII, { N_("ASCII files (*.txt)"), "*.txt" }},
+	{ OPEN_BOX,     { N_("BOX data files (*.box)"), "*.box" }},
+	{ OPEN_GNUMERIC,   { N_("Gnumeric files (*.gnumeric)"), "*.gnumeric" }},
+	{ APPEND_GNUMERIC, { N_("Gnumeric files (*.gnumeric)"), "*.gnumeric" }},
+	{ OPEN_EXCEL,   { N_("Excel files (*.xls)"), "*.xls" }},
+	{ APPEND_EXCEL, { N_("Excel files (*.xls)"), "*.xls" }},
+	{ OPEN_WF1,     { N_("Eviews workfiles (*.wf1)"), "*.wf1" }},
+	{ APPEND_WF1,   { N_("Eviews workfiles (*.wf1)"), "*.wf1" }},
+	{ OPEN_DTA,     { N_("Stata files (*.dta)"), "*.dta" }},
+	{ APPEND_DTA,   { N_("Stata files (*.dta)"), "*.dta" }},
+	{ OPEN_JMULTI,  { N_("JMulTi files (*.dat)"), "*.dat" }},
+	{ APPEND_JMULTI,{ N_("JMulTi files (*.dat)"), "*.dat" }},
+	{ OPEN_RATS_DB, { N_("RATS databases (*.rat)"), "*.rat" }},
+	{ OPEN_PCGIVE_DB, { N_("PcGive data files (*.bn7)"), "*.bn7" }},
+	{ SET_PROG,     { N_("program files (*.exe)"), "*.exe" }}
     };
 
-    static struct winfilter olddat_filter = {
-	N_("gretl data files (*.dat)"), "*.dat"
-    };    
-
     static struct winfilter default_filter = {
 	N_("all files (*.*)"), "*.*" 
     };
 
-    if (using_olddat() && IS_DAT_ACTION(action)) {
-	return olddat_filter;
-    }
-
     if (action == SAVE_GNUPLOT || action == SAVE_THIS_GRAPH) {
 	GPT_SPEC *plot = (GPT_SPEC *) data;
 
@@ -724,6 +676,35 @@ static char *make_winfilter (int action, gpointer data)
     return start;
 }
 
+static int select_dirname (char *fname, char *trmsg)
+{
+    BROWSEINFO bi;
+    LPITEMIDLIST pidl;
+    char dirname[MAX_PATH];
+    int ret = 1;
+
+    CoInitialize(NULL);
+
+    bi.hwndOwner = NULL;
+    bi.pidlRoot = NULL; /* FIXME? */
+    bi.pszDisplayName = dirname;
+    bi.lpszTitle = trmsg;
+    bi.ulFlags = 0;
+    bi.lpfn = NULL;
+    bi.lParam = 0;
+    bi.iImage = 0;   
+
+    pidl = SHBrowseForFolder(&bi);
+    if (pidl == NULL) {
+	ret = 0;
+    } else {
+	SHGetPathFromIDList(pidl, fname);
+	CoTaskMemFree(pidl);
+    }
+
+    return ret;
+}
+
 void file_selector (const char *msg, int action, FselDataSrc src, gpointer data) 
 {
     OPENFILENAME of;
@@ -738,7 +719,7 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
     set_startdir(startdir, action);
 
     /* special cases */
-    if ((action == SAVE_DATA || action == SAVE_GZDATA) && paths.datfile[0]) {
+    if (action == SAVE_DATA && *paths.datfile != '\0') {
 	char *savename = suggested_savename(paths.datfile);
 
 	strcpy(fname, savename);
@@ -746,13 +727,13 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
 	if (!(data_status & BOOK_DATA)) {
 	    get_base(startdir, paths.datfile, SLASH);
 	}
-    } else if (EXPORT_ACTION(action, src) && paths.datfile[0]) {
+    } else if (EXPORT_ACTION(action, src) && *paths.datfile != '\0') {
 	char *savename = suggested_exportname(paths.datfile, action);
 
 	strcpy(fname, savename);
 	g_free(savename);
 	get_base(startdir, paths.datfile, SLASH);
-    } else if (action == SET_PATH) {
+    } else if (action == SET_PROG) {
 	char *strvar = (char *) data;
 
 	if (strvar != NULL && *strvar != '\0') {
@@ -768,32 +749,36 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
 	trmsg = g_strdup(msg);
     }
 
-    /* initialize file dialog info struct */
-    memset(&of, 0, sizeof of);
-#ifdef OPENFILENAME_SIZE_VERSION_400
-    of.lStructSize = OPENFILENAME_SIZE_VERSION_400;
-#else
-    of.lStructSize = sizeof of;
-#endif
-    of.hwndOwner = NULL;
-    filter = make_winfilter(action, data);
-    of.lpstrFilter = filter;
-    of.lpstrCustomFilter = NULL;
-    of.nFilterIndex = 1;
-    of.lpstrFile = fname;
-    of.nMaxFile = sizeof fname;
-    of.lpstrFileTitle = endname;
-    of.nMaxFileTitle = sizeof endname;
-    of.lpstrInitialDir = startdir;
-    of.lpstrTitle = trmsg;
-    of.lpstrDefExt = NULL;
-    of.Flags = OFN_HIDEREADONLY;
-
-    if (action < END_OPEN) {
-	retval = GetOpenFileName(&of);
+    if (action == SET_DIR) {
+	retval = select_dirname(fname, trmsg);
     } else {
-	/* a file save action */
-	retval = GetSaveFileName(&of);
+	/* initialize file dialog info struct */
+	memset(&of, 0, sizeof of);
+#ifdef OPENFILENAME_SIZE_VERSION_400
+	of.lStructSize = OPENFILENAME_SIZE_VERSION_400;
+#else
+	of.lStructSize = sizeof of;
+#endif
+	of.hwndOwner = NULL;
+	filter = make_winfilter(action, data);
+	of.lpstrFilter = filter;
+	of.lpstrCustomFilter = NULL;
+	of.nFilterIndex = 1;
+	of.lpstrFile = fname;
+	of.nMaxFile = sizeof fname;
+	of.lpstrFileTitle = endname;
+	of.nMaxFileTitle = sizeof endname;
+	of.lpstrInitialDir = startdir;
+	of.lpstrTitle = trmsg;
+	of.lpstrDefExt = NULL;
+	of.Flags = OFN_HIDEREADONLY;
+
+	if (action < END_OPEN || action == SET_PROG) {
+	    retval = GetOpenFileName(&of);
+	} else {
+	    /* a file save action */
+	    retval = GetSaveFileName(&of);
+	}
     }
 
     free(filter);
@@ -813,45 +798,6 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
 
 #else /* End of MS Windows file selection code, start GTK */
 
-/* ........................................................... */
-
-# ifndef USE_GTK_CHOOSER
-
-struct fsinfo_t {
-    GtkWidget *w;
-    char fname[FILENAME_MAX];
-};
-
-static void filesel_callback (GtkWidget *w, struct fsinfo_t *fsinfo) 
-{
-# ifdef OLD_GTK
-    GtkIconFileSel *fsel = GTK_ICON_FILESEL(fsinfo->w);
-    char *test;
-# endif
-    const gchar *path;
-
-    *fsinfo->fname = '\0';
-
-# ifndef OLD_GTK
-    path = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fsinfo->w));
-    if (path == NULL || *path == '\0' || isdir(path)) {
-	return;
-    }
-    strcpy(fsinfo->fname, path);
-# else
-    test = gtk_entry_get_text(GTK_ENTRY(fsel->file_entry));
-    if (test == NULL || *test == '\0') {
-	return;
-    }
-    path = gtk_file_list_get_path(GTK_FILE_LIST(fsel->file_list));
-    sprintf(fsinfo->fname, "%s%s", path, test);
-# endif
-
-    gtk_widget_destroy(GTK_WIDGET(fsinfo->w));
-}
-
-# endif /* !USE_GTK_CHOOSER */
-
 static char *get_filter_suffix (int action, gpointer data, char *suffix)
 {
     
@@ -866,10 +812,6 @@ static char *get_filter_suffix (int action, gpointer data, char *suffix)
     return suffix;
 }
 
-# ifndef OLD_GTK
-
-# ifdef USE_GTK_CHOOSER
-
 static GtkFileFilter *get_file_filter (int action, gpointer data)
 {
     GtkFileFilter *filter;
@@ -884,29 +826,38 @@ static GtkFileFilter *get_file_filter (int action, gpointer data)
 
 void file_selector (const char *msg, int action, FselDataSrc src, gpointer data) 
 {
+    GtkWindow *parent = GTK_WINDOW(mdata->w);
     GtkWidget *filesel;
     char startdir[MAXLEN];
     GtkFileFilter *filter;
+    GtkFileChooserAction fa;
+    const gchar *okstr;
 
     set_startdir(startdir, action);
 
+    if (action == SET_DIR) {
+	fa = GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
+	okstr = GTK_STOCK_OK;
+    } else if (action == SET_PROG) {
+	fa = GTK_FILE_CHOOSER_ACTION_OPEN;
+	okstr = GTK_STOCK_OK;
+    } else if (action < END_OPEN) {
+	fa = GTK_FILE_CHOOSER_ACTION_OPEN;
+	okstr = GTK_STOCK_OPEN;
+    } else {
+	fa = GTK_FILE_CHOOSER_ACTION_SAVE;
+	okstr = GTK_STOCK_SAVE;
+	parent = NULL;
+    }
+
     /* FIXME: parent window below should not always be mdata->w,
        in particular when action == SAVE_THIS_GRAPH
     */
 
-    if (action > END_OPEN && action != SET_PATH) {
-	filesel = gtk_file_chooser_dialog_new(msg, NULL, /* GTK_WINDOW(mdata->w), */
-					      GTK_FILE_CHOOSER_ACTION_SAVE,
-					      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					      NULL);
-    } else {
-	filesel = gtk_file_chooser_dialog_new(msg, GTK_WINDOW(mdata->w), 
-					      GTK_FILE_CHOOSER_ACTION_OPEN,
-					      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
-					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					      NULL);
-    }
+    filesel = gtk_file_chooser_dialog_new(msg, GTK_WINDOW(parent), fa,
+					  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					  okstr, GTK_RESPONSE_ACCEPT,
+					  NULL);
 
     gtk_dialog_set_default_response(GTK_DIALOG(filesel), GTK_RESPONSE_ACCEPT);
 
@@ -929,32 +880,39 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
 
     /* special cases */
 
-    if ((action == SAVE_DATA || action == SAVE_GZDATA) && paths.datfile[0]) {
+    if (action == SAVE_DATA && *paths.datfile != '\0') {
 	char *savename = suggested_savename(paths.datfile);
 
 	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(filesel), 
 					  savename);
 	g_free(savename);
-    } else if (EXPORT_ACTION(action, src) && paths.datfile[0]) {
+    } else if (EXPORT_ACTION(action, src) && *paths.datfile != '\0') {
 	char *savename = suggested_exportname(paths.datfile, action);
 
 	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(filesel), 
 					  savename);
 	g_free(savename);
-    } else if ((action == SAVE_SESSION) && *scriptfile != '\0') {
+    } else if (action == SAVE_SESSION && *scriptfile != '\0') {
 	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(filesel), 
 				      scriptfile);
-    } else if (action == SET_PATH) {
+    } else if (action == SET_PROG) {
 	char *strvar = (char *) data;
 
-	if (strvar != NULL && slashpos(strvar) > 0) {
+	if (strvar != NULL && g_path_is_absolute(strvar)) {
 	    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(filesel), 
 					  strvar);
 	} else {
 	    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(filesel), 
 						"/usr/bin");
 	}	    
-    } 
+    } else if (action == SET_DIR) {
+	char *strvar = (char *) data;
+
+	if (strvar != NULL && g_path_is_absolute(strvar)) {
+	    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(filesel), 
+					  strvar);
+	} 
+    }	
 
     if (gtk_dialog_run(GTK_DIALOG(filesel)) == GTK_RESPONSE_ACCEPT) {
 	char *fname;
@@ -968,210 +926,4 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
     }
 }
 
-# else
-
-#include <glob.h> /* POSIX */
-
-static void
-gtk_file_selection_glob_populate (GtkFileSelection *fs, 
-				  const gchar *dirname, const gchar *suffix)
-{
-    glob_t globbuf;
-    GtkTreeIter iter;
-    GtkListStore *file_model;
-    gchar *pattern;
-    size_t i, dirlen;
-
-    g_return_if_fail (GTK_IS_FILE_SELECTION (fs));
-
-    if (dirname == NULL || *dirname == 0 || suffix == NULL || *suffix == 0)
-	return;
-
-    pattern = g_build_path(G_DIR_SEPARATOR_S, dirname, suffix, NULL);
-    dirlen = strlen(pattern) - strlen(suffix);
-
-    glob(pattern, 0, NULL, &globbuf);
-
-    file_model = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(fs->file_list)));
-
-    gtk_list_store_clear (file_model);
-
-    for (i=0; i<globbuf.gl_pathc; i++) {
-	gtk_list_store_append (file_model, &iter);
-	gtk_list_store_set (file_model, &iter, 0, globbuf.gl_pathv[i] + dirlen, -1);
-    }
-
-    globfree(&globbuf);
-    g_free(pattern);
-}
-
-void file_selector (const char *msg, int action, FselDataSrc src, gpointer data) 
-{
-    struct fsinfo_t fsinfo;
-    GtkWidget *filesel;
-    char suffix[16], startdir[MAXLEN];
-    int do_glob = 1;
-
-    *fsinfo.fname = '\0';
-
-    set_startdir(startdir, action);
-
-    filesel = gtk_file_selection_new(msg);
-    fsinfo.w = filesel;
-    g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(filesel)->ok_button),
-		     "clicked", 
-		     G_CALLBACK(filesel_callback), &fsinfo);
-
-    gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel), startdir);
-
-    /* special cases */
-
-    if ((action == SAVE_DATA || action == SAVE_GZDATA) && paths.datfile[0]) {
-	char *savename = suggested_savename(paths.datfile);
-
-	gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel), 
-					savename);
-	if (!strstr(savename, startdir)) do_glob = 0;
-	g_free(savename);
-    } else if (EXPORT_ACTION(action, src) && paths.datfile[0]) {
-	char *savename = suggested_exportname(paths.datfile, action);
-
-	gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel), 
-					savename);
-	if (!strstr(savename, startdir)) do_glob = 0;
-	g_free(savename);
-    } else if ((action == SAVE_SESSION) && *scriptfile != '\0') {
-	gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel), 
-					scriptfile);
-	if (!strstr(scriptfile, startdir)) do_glob = 0;
-    } else if (action == SET_PATH) {
-	char *strvar = (char *) data;
-
-	if (strvar != NULL && slashpos(strvar) > 0) {
-	    gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel), 
-					    strvar);
-	} else {
-	    gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel), 
-					    "/usr/bin/");
-	}	    
-	do_glob = 0;
-    }	
-
-    /* end special cases */
-
-    g_signal_connect(G_OBJECT(filesel), "destroy",
-		     gtk_main_quit, NULL);
-    g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(filesel)->cancel_button),
-		     "clicked", G_CALLBACK(delete_widget), filesel);
-
-    gtk_widget_show(filesel);
-
-    if (do_glob) {
-	get_filter_suffix(action, data, suffix);
-	gtk_file_selection_glob_populate (GTK_FILE_SELECTION(filesel), 
-					  startdir, suffix);
-    }
-
-    gtk_window_set_modal(GTK_WINDOW(filesel), TRUE);
-    gtk_main(); 
-
-    if (*fsinfo.fname != '\0') {
-	file_selector_process_result(fsinfo.fname, action, src, data);
-    } 
-}
-
-# endif
-
-# else /* gtk version diffs continue */
-
-void file_selector (const char *msg, int action, FselDataSrc src, gpointer data) 
-{
-    struct fsinfo_t fsinfo;
-    GtkWidget *filesel;
-    int gotdir = 0;
-    char suffix[8], startdir[MAXLEN];
-
-    *fsinfo.fname = '\0';
-
-    set_startdir(startdir, action);
-
-    filesel = gtk_icon_file_selection_new(msg);
-
-    if (strstr(startdir, "/.")) {
-	gtk_icon_file_selection_show_hidden(GTK_ICON_FILESEL(filesel), TRUE);
-    } else {
-	gtk_icon_file_selection_show_hidden(GTK_ICON_FILESEL(filesel), FALSE);
-    }
-
-    get_filter_suffix(action, data, suffix);
-    gtk_icon_file_selection_set_filter(GTK_ICON_FILESEL(filesel), suffix);
-
-    fsinfo.w = filesel;
-    gtk_signal_connect(GTK_OBJECT(GTK_ICON_FILESEL(filesel)->ok_button),
-		       "clicked", 
-		       GTK_SIGNAL_FUNC(filesel_callback), &fsinfo);
-
-    /* special cases */
-
-    if ((action == SAVE_DATA || action == SAVE_GZDATA) && paths.datfile[0]) {
-	char *savename = suggested_savename(paths.datfile);
-	char startd[MAXLEN];
-
-	gtk_entry_set_text(GTK_ENTRY(GTK_ICON_FILESEL(filesel)->file_entry),
-			   savename);
-	if (!(data_status & BOOK_DATA) && get_base(startd, paths.datfile, SLASH)) {
-	    gtk_icon_file_selection_open_dir(GTK_ICON_FILESEL(filesel), startd);
-	    gotdir = 1;
-	}
-	g_free(savename);
-    } else if (EXPORT_ACTION(action, src) && paths.datfile[0]) {
-	char *savename = suggested_exportname(paths.datfile, action);
-	char startd[MAXLEN];
-
-	gtk_entry_set_text(GTK_ENTRY(GTK_ICON_FILESEL(filesel)->file_entry),
-			   savename);
-	if (get_base(startd, paths.datfile, SLASH)) {
-	    gtk_icon_file_selection_open_dir(GTK_ICON_FILESEL(filesel), startd);
-	    gotdir = 1;
-	}
-	g_free(savename);
-    } else if (action == SET_PATH) {
-	char *strvar = (char *) data;
-	char startd[MAXLEN];
-
-	if (get_base(startd, strvar, SLASH) == 1) {
-	    gtk_icon_file_selection_open_dir(GTK_ICON_FILESEL(filesel), startd);
-	    gtk_entry_set_text(GTK_ENTRY(GTK_ICON_FILESEL(filesel)->file_entry),
-			       strvar + slashpos(strvar) + 1);
-	} else {
-	    gtk_icon_file_selection_open_dir(GTK_ICON_FILESEL(filesel), 
-					     "/usr/bin/");
-	}
-	gotdir = 1;
-    }
-
-    if (!gotdir) {
-	gtk_icon_file_selection_open_dir(GTK_ICON_FILESEL(filesel), startdir);
-    }
-
-    gtk_signal_connect(GTK_OBJECT(GTK_ICON_FILESEL(filesel)), "destroy",
-                       gtk_main_quit, NULL);
-    gtk_signal_connect_object(GTK_OBJECT(GTK_ICON_FILESEL(filesel)->cancel_button),
-			      "clicked", (GtkSignalFunc) gtk_widget_destroy,
-			      GTK_OBJECT(filesel));
-
-    gtk_widget_show(filesel);
-    gretl_set_window_modal(filesel);
-    gtk_main(); 
-
-    if (*fsinfo.fname != '\0') {
-	file_selector_process_result(fsinfo.fname, action, src, data);
-    } 
-}
-
-# endif /* old gtk */
-
 #endif /* end of non-MS Windows code */
-
-
-
