@@ -40,10 +40,6 @@
 # include <png.h>
 #endif
 
-#if !GLIB_CHECK_VERSION(2,0,0)
-# define OLD_GTK
-#endif
-
 enum {
     PLOT_SAVED          = 1 << 0,
     PLOT_HAS_CONTROLLER = 1 << 1,
@@ -277,11 +273,11 @@ add_or_remove_png_term (const char *fname, int add, GPT_SPEC *spec)
     int l2 = 0;
 #endif
 
-    sprintf(temp, "%sgpttmp.XXXXXX", paths.userdir);
-    if (mktemp(temp) == NULL) return 1;
-
-    ftmp = open_gp_file(temp, "w");
-    if (ftmp == NULL) return 1;
+    sprintf(temp, "%sgpttmp", paths.userdir);
+    ftmp = gretl_tempfile_open(temp);
+    if (ftmp == NULL) {
+	return 1;
+    }
 
     fsrc = open_gp_file(fname, "r");
     if (fsrc == NULL) {
@@ -398,24 +394,30 @@ static int gnuplot_png_init (GPT_SPEC *spec, FILE **fpp)
 /* take saved plot source file and make PNG from it, then display
    the PNG */
 
-void display_session_graph_png (char *fname) 
+void display_session_graph_png (const char *fname) 
 {
-    char *myfname = fname;
+    char fullname[MAXLEN];
     gchar *plotcmd;
     int err = 0;
 
-    if (add_png_term_to_plotfile(myfname)) {
+    if (g_path_is_absolute(fname)) {
+	strcpy(fullname, fname);
+    } else {
+	sprintf(fullname, "%s%s", paths.userdir, fname);
+    }
+
+    if (add_png_term_to_plotfile(fullname)) {
 	return;
     }
 
-    plotcmd = g_strdup_printf("\"%s\" \"%s\"", paths.gnuplot, myfname);
+    plotcmd = g_strdup_printf("\"%s\" \"%s\"", paths.gnuplot, fullname);
     err = gretl_spawn(plotcmd);
     g_free(plotcmd);
 
     if (err) {
 	errbox(_("Gnuplot error creating graph"));
     } else {
-	gnuplot_show_png(myfname, NULL, 1);
+	gnuplot_show_png(fullname, NULL, 1);
     }
 }
 
@@ -551,6 +553,7 @@ static int get_gpt_marker (const char *line, char *label)
                       p == PLOT_MULTI_SCATTER || \
                       p == PLOT_PANEL || \
                       p == PLOT_TRI_GRAPH || \
+                      p == PLOT_BI_GRAPH || \
                       p == PLOT_VAR_ROOTS || \
 		      p == PLOT_ELLIPSE)
 
@@ -587,6 +590,7 @@ static GPT_SPEC *plotspec_new (void)
 	spec->lines[i].scale[0] = 0;
 	spec->lines[i].yaxis = 1;
 	spec->lines[i].type = 0;
+	spec->lines[i].width = 1;
 	spec->lines[i].ncols = 0;
     }
 
@@ -962,7 +966,7 @@ static int get_plot_nobs (FILE *fp, PlotType *ptype, int *do_markers)
 /* parse the "using..." portion of plot specification for a
    given plot line: full form is like:
   
-     using XX axes XX title XX w XX lt XX
+     using XX axes XX title XX w XX lt XX lw XX
 */
 
 static int parse_gp_line_line (const char *s, GPT_SPEC *spec, int i)
@@ -1000,7 +1004,6 @@ static int parse_gp_line_line (const char *s, GPT_SPEC *spec, int i)
 	}
     }
 
-    /* axes */
     if (strstr(s, "axes x1y2")) {
 	spec->lines[i].yaxis = 2;
     } 
@@ -1017,6 +1020,10 @@ static int parse_gp_line_line (const char *s, GPT_SPEC *spec, int i)
 	sscanf(p + 4, "%d", &spec->lines[i].type);
     } 
 
+    if ((p = strstr(s, " lw "))) {
+	sscanf(p + 4, "%d", &spec->lines[i].width);
+    } 
+
     if (spec->lines[i].ncols == 0 && spec->lines[i].formula[0] == '\0') {
 	/* got neither data column spec nor formula */
 	err = 1;
@@ -1495,59 +1502,6 @@ static int make_alt_label (gchar *alt, const gchar *label)
     return err;
 }
 
-#ifdef OLD_GTK
-
-static void
-write_label_to_plot (png_plot *plot, int i, gint x, gint y)
-{
-    const gchar *label = plot->spec->markers[i];
-    static GdkFont *label_font;
-
-    if (plot_is_roots(plot)) {
-	gchar alt_label[12];
-	
-	if (make_alt_label(alt_label, label)) {
-	    return;
-	}
-
-	label = alt_label;
-    }
-
-    if (plot->invert_gc == NULL) {
-	create_selection_gc(plot);
-    }
-
-    if (label_font == NULL) {
-	label_font = gdk_font_load("fixed");
-    }
-
-    /* draw the label */
-    gdk_draw_text (plot->pixmap,
-		   label_font,
-		   plot->invert_gc,
-		   x, y,
-		   label,
-		   strlen(label));
-
-    /* show the modified pixmap */
-    gdk_window_copy_area(plot->canvas->window,
-			 plot->canvas->style->fg_gc[GTK_STATE_NORMAL],
-			 0, 0,
-			 plot->pixmap,
-			 0, 0,
-			 plot->pixel_width, plot->pixel_height);
-
-    /* draw (invert) again to erase the text */
-    gdk_draw_text (plot->pixmap,
-		   label_font,
-		   plot->invert_gc,
-		   x, y,
-		   label,
-		   strlen(label));
-}
-
-#else
-
 static void
 write_label_to_plot (png_plot *plot, int i, gint x, gint y)
 {
@@ -1591,8 +1545,6 @@ write_label_to_plot (png_plot *plot, int i, gint x, gint y)
     plot->format |= PLOT_MARKERS_UP;
 }
 
-#endif /* GTK versions */
-
 #define TOLDIST 0.01
 
 static gint
@@ -1622,7 +1574,6 @@ identify_point (png_plot *plot, int pixel_x, int pixel_y,
 	return TRUE;
     }
 
-#ifndef OLD_GTK
     /* need array to keep track of which points are labeled */
     if (plot->spec->labeled == NULL) {
 	plot->spec->labeled = calloc(plot->spec->nobs, 1);
@@ -1630,7 +1581,6 @@ identify_point (png_plot *plot, int pixel_x, int pixel_y,
 	    return TRUE;
 	}
     }
-#endif
 
     if (plot_is_zoomed(plot)) {
 	min_xdist = xrange = plot->zoom_xmax - plot->zoom_xmin;
@@ -1680,12 +1630,10 @@ identify_point (png_plot *plot, int pixel_x, int pixel_y,
 	}
     }
 
-#ifndef OLD_GTK
     /* if the point is already labeled, skip */
     if (plot->spec->labeled[best_match]) {
 	return TRUE;
     }
-#endif
 
 #if GPDEBUG > 2
     fprintf(stderr, " best_match=%d, with data_x[%d]=%g, data_y[%d]=%g\n", 
@@ -1698,10 +1646,8 @@ identify_point (png_plot *plot, int pixel_x, int pixel_y,
 	min_xdist < TOLDIST * xrange &&
 	min_ydist < TOLDIST * yrange) {
 	write_label_to_plot(plot, best_match, pixel_x, pixel_y);
-#ifndef OLD_GTK
 	/* flag the point as labeled already */
 	plot->spec->labeled[best_match] = 1;
-#endif
     }
 
     return TRUE;
@@ -1944,21 +1890,17 @@ static gint plot_popup_activated (GtkWidget *w, gpointer data)
         file_selector(_("Save gnuplot graph"), SAVE_THIS_GRAPH, 
 		      FSEL_DATA_MISC, plot->spec);
     } else if (!strcmp(item, _("Save to session as icon"))) { 
-	add_graph_to_session(plot->spec, GRETL_GNUPLOT_GRAPH, NULL);
+	add_graph_to_session(plot->spec, GRETL_OBJ_GRAPH, NULL);
     } else if (plot_is_range_mean(plot) && !strcmp(item, _("Help"))) { 
 	context_help(NULL, GINT_TO_POINTER(RMPLOT));
     } else if (plot_is_hurst(plot) && !strcmp(item, _("Help"))) { 
 	context_help(NULL, GINT_TO_POINTER(HURST));
-    }
-#ifndef OLD_GTK
-    else if (!strcmp(item, _("Freeze data labels"))) {
+    } else if (!strcmp(item, _("Freeze data labels"))) {
 	plot->spec->flags |= GPTSPEC_ALL_MARKERS;
 	redisplay_edited_png(plot);
     } else if (!strcmp(item, _("Clear data labels"))) { 
 	zoom_unzoom_png(plot, PNG_REDISPLAY);
-    }
-#endif
-    else if (!strcmp(item, _("Zoom..."))) { 
+    } else if (!strcmp(item, _("Zoom..."))) { 
 	GdkCursor* cursor;
 
 	cursor = gdk_cursor_new(GDK_CROSSHAIR);
@@ -2034,10 +1976,8 @@ static void build_plot_menu (png_plot *plot)
 	N_("Copy to clipboard"),
 #endif
 	N_("Save to session as icon"),
-#ifndef OLD_GTK
 	N_("Freeze data labels"),
 	N_("Clear data labels"),
-#endif
 	N_("Zoom..."),
 #ifdef USE_GNOME
 	N_("Print..."),
@@ -2099,14 +2039,12 @@ static void build_plot_menu (png_plot *plot)
 	    i++;
 	    continue;
 	}	    
-#ifndef OLD_GTK
 	if (!plot_has_data_markers(plot) &&
 	    (!strcmp(plot_items[i], "Freeze data labels") ||
 	     !strcmp(plot_items[i], "Clear data labels"))) {
 	    i++;
 	    continue;
 	}
-#endif
 	if (!plot_has_regression_list(plot) &&
 	    !strcmp(plot_items[i], "OLS estimates")) {
 	    i++;
@@ -2202,7 +2140,7 @@ static int zoom_unzoom_png (png_plot *plot, int view)
 	    return 1;
 	}
 
-	build_path(paths.userdir, "zoomplot.gp", zoomname, NULL);
+	build_path(zoomname, paths.userdir, "zoomplot.gp", NULL);
 	fpout = gretl_fopen(zoomname, "w");
 	if (fpout == NULL) {
 	    fclose(fpin);
@@ -2342,7 +2280,7 @@ plot_key_handler (GtkWidget *w, GdkEventKey *key, png_plot *plot)
 	break;
     case GDK_s:
     case GDK_S:
-	add_graph_to_session(plot->spec, GRETL_GNUPLOT_GRAPH, NULL);
+	add_graph_to_session(plot->spec, GRETL_OBJ_GRAPH, NULL);
 	break;
 #ifdef G_OS_WIN32
     case GDK_c:
@@ -2379,52 +2317,16 @@ void plot_expose (GtkWidget *widget, GdkEventExpose *event,
 			 event->area.width, event->area.height);
 }
 
-#ifdef OLD_GTK
-
-#include <errno.h>
-
-static int test_file_open (const char *fname)
-{
-    FILE *fp;
-    int err = 0;
-
-    errno = 0;
-
-    fp = fopen(fname, "r");
-    if (fp == NULL) {
-	errbox(_("Couldn't open %s: %s"), fname, strerror(errno));
-	err = 1;
-    } else {
-	fclose(fp);
-    }
-
-    return err;
-}
-#endif
-
 static void render_pngfile (png_plot *plot, int view)
 {
     gint width;
     gint height;
     GdkPixbuf *pbuf;
     char pngname[MAXLEN];
-#ifndef OLD_GTK
     GError *error = NULL;
-#endif
 
-    build_path(paths.userdir, "gretltmp.png", pngname, NULL);
+    build_path(pngname, paths.userdir, "gretltmp.png", NULL);
 
-#ifdef OLD_GTK
-    if (test_file_open(pngname)) {
-	return;
-    }
-    pbuf = gdk_pixbuf_new_from_file(pngname);
-    if (pbuf == NULL) {
-	errbox(_("Failed to create pixbuf from file"));
-	remove(pngname);
-	return;
-    }
-#else
     pbuf = gdk_pixbuf_new_from_file(pngname, &error);
     if (pbuf == NULL) {
         errbox(error->message);
@@ -2432,41 +2334,30 @@ static void render_pngfile (png_plot *plot, int view)
 	remove(pngname);
 	return;
     }
-#endif
 
     width = gdk_pixbuf_get_width(pbuf);
     height = gdk_pixbuf_get_height(pbuf);
 
     if (width == 0 || height == 0) {
 	errbox(_("Malformed PNG file for graph"));
-#ifdef OLD_GTK
-	gdk_pixbuf_unref(pbuf);
-#else
 	g_object_unref(pbuf);
-#endif
 	remove(pngname);
 	return;
     }
 
-#ifndef OLD_GTK
     /* scrap any old record of which points are labeled */
     if (plot->spec->labeled != NULL) {
 	free(plot->spec->labeled);
 	plot->spec->labeled = NULL;
 	plot->format &= ~PLOT_MARKERS_UP;
     }
-#endif
 
     gdk_pixbuf_render_to_drawable(pbuf, plot->pixmap, 
 				  plot->canvas->style->fg_gc[GTK_STATE_NORMAL],
 				  0, 0, 0, 0, width, height,
 				  GDK_RGB_DITHER_NONE, 0, 0);
 
-#ifdef OLD_GTK
-    gdk_pixbuf_unref(pbuf);
-#else
     g_object_unref(pbuf);
-#endif
     remove(pngname);
     
     if (view != PNG_START) { 
@@ -2575,8 +2466,8 @@ static int get_dumb_plot_yrange (png_plot *plot)
 	return 1;
     }
 
-    build_path(paths.userdir, "dumbplot.gp", dumbgp, NULL);
-    build_path(paths.userdir, "gptdumb.txt", dumbtxt, NULL);
+    build_path(dumbgp, paths.userdir, "dumbplot.gp", NULL);
+    build_path(dumbtxt, paths.userdir, "gptdumb.txt", NULL);
     fpout = gretl_fopen(dumbgp, "w");
     if (fpout == NULL) {
 	fclose(fpin);
@@ -2916,25 +2807,13 @@ int gnuplot_show_png (const char *plotfile, GPT_SPEC *spec, int saved)
 	plot->pixel_width = plot->pixel_height;
     }
 
-#ifdef OLD_GTK
-    gtk_widget_push_visual(gdk_rgb_get_visual());
-    gtk_widget_push_colormap(gdk_rgb_get_cmap());
     plot->shell = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-    gtk_widget_pop_visual();
-    gtk_widget_pop_colormap();
-#else
-    plot->shell = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-#endif
 
     /* note need for corresponding unref */
     gtk_widget_ref(plot->shell);
 
     gtk_window_set_title(GTK_WINDOW(plot->shell), _("gretl: gnuplot graph")); 
-#ifdef OLD_GTK
-    gtk_window_set_policy(GTK_WINDOW(plot->shell), FALSE, FALSE, FALSE);
-#else
     gtk_window_set_resizable(GTK_WINDOW(plot->shell), FALSE);
-#endif
 
     vbox = gtk_vbox_new(FALSE, 2);
     gtk_container_add(GTK_CONTAINER(plot->shell), vbox);
@@ -2961,13 +2840,8 @@ int gnuplot_show_png (const char *plotfile, GPT_SPEC *spec, int saved)
 
     /* Create drawing-area widget */
     plot->canvas = gtk_drawing_area_new();
-#ifdef OLD_GTK
-    gtk_drawing_area_size(GTK_DRAWING_AREA(plot->canvas), 
-			  plot->pixel_width, plot->pixel_height);
-#else
     gtk_widget_set_size_request(GTK_WIDGET(plot->canvas), 
 				plot->pixel_width, plot->pixel_height);
-#endif
     gtk_widget_set_events (plot->canvas, GDK_EXPOSURE_MASK
                            | GDK_LEAVE_NOTIFY_MASK
                            | GDK_BUTTON_PRESS_MASK
@@ -2996,12 +2870,8 @@ int gnuplot_show_png (const char *plotfile, GPT_SPEC *spec, int saved)
     /* the statusbar */
     plot->statusbar = gtk_statusbar_new();
 
-#ifdef OLD_GTK
-    gtk_widget_set_usize(plot->statusbar, 1, -1);
-#else
     gtk_widget_set_size_request(plot->statusbar, 1, -1);
     gtk_statusbar_set_has_resize_grip(GTK_STATUSBAR(plot->statusbar), FALSE);
-#endif
 
     gtk_container_set_resize_mode(GTK_CONTAINER (plot->statusbar),
 				  GTK_RESIZE_QUEUE);
@@ -3043,11 +2913,7 @@ int gnuplot_show_png (const char *plotfile, GPT_SPEC *spec, int saved)
 
     if (plot_has_xrange(plot)) {
 	gtk_widget_realize(plot->cursor_label);
-#ifdef OLD_GTK
-	gtk_widget_set_usize(plot->cursor_label, 140, -1);
-#else
 	gtk_widget_set_size_request(plot->cursor_label, 140, -1);
-#endif
     }
 
     gtk_widget_show(vbox);
@@ -3129,7 +2995,7 @@ static int get_png_bounds_info (png_bounds *bounds)
     int i, num_text;
     volatile int ret = GRETL_PNG_OK;
 
-    build_path(paths.userdir, "gretltmp.png", pngname, NULL); 
+    build_path(pngname, paths.userdir, "gretltmp.png", NULL); 
 
     fp = gretl_fopen(pngname, "rb");
     if (fp == NULL) {
