@@ -44,6 +44,7 @@ GtkWidget *linetitle[MAX_PLOT_LINES];
 GtkWidget *stylecombo[MAX_PLOT_LINES];
 GtkWidget *yaxiscombo[MAX_PLOT_LINES];
 GtkWidget *linescale[MAX_PLOT_LINES];
+GtkWidget *linewidth[MAX_PLOT_LINES];
 
 GtkWidget *labeltext[MAX_PLOT_LABELS];
 GtkWidget *labeljust[MAX_PLOT_LABELS];
@@ -130,16 +131,12 @@ static void entry_to_gp_string (GtkWidget *w, char *targ, size_t n)
     wstr = gtk_entry_get_text(GTK_ENTRY(w));
 
     if (wstr != NULL && *wstr != '\0') {
-#ifndef OLD_GTK
 	gchar *trstr = force_locale_from_utf8(wstr);
 
 	if (trstr != NULL) {
 	    strncat(targ, trstr, n-1);
 	    g_free(trstr);
 	}
-#else
-	strncat(targ, wstr, n-1);
-#endif /* OLD_GTK */
     }
 }
 
@@ -200,12 +197,11 @@ static double entry_to_gp_double (GtkWidget *w)
 
 static void gp_string_to_entry (GtkWidget *w, const char *str)
 {
-#ifndef OLD_GTK
-# ifdef ENABLE_NLS
+#ifdef ENABLE_NLS
     int l2 = use_latin_2();
-# else
+#else
     int l2 = 0;
-# endif /* ENABLE_NLS */
+#endif
     gchar *trstr;
 
     if (l2) {
@@ -221,9 +217,6 @@ static void gp_string_to_entry (GtkWidget *w, const char *str)
 	gtk_entry_set_text(GTK_ENTRY(w), trstr);
 	g_free(trstr);
     }    
-#else 
-    gtk_entry_set_text(GTK_ENTRY(w), str);
-#endif /* OLD_GTK */
 }
 
 static int
@@ -346,15 +339,14 @@ static void apply_gpt_changes (GtkWidget *widget, GPT_SPEC *spec)
 				   spec->lines[i].scale, 
 				   sizeof spec->lines[0].scale);
 	    }
+	    if (linewidth[i] != NULL) {
+		spec->lines[i].width = 
+		    gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(linewidth[i]));
+	    }
 	}
     }
 
     for (i=0; i<MAX_PLOT_LABELS && !err; i++) {
-#ifdef OLD_GTK
-	GtkWidget *active_item;
-	int opt;
-#endif
-
 	entry_to_gp_string(labeltext[i], spec->labels[i].text, 
 			   sizeof spec->labels[0].text);
 	if (string_is_blank(spec->labels[i].text)) {
@@ -364,15 +356,8 @@ static void apply_gpt_changes (GtkWidget *widget, GPT_SPEC *spec)
 	if (err) {
 	    break;
 	}
-#ifdef OLD_GTK
-	active_item = GTK_OPTION_MENU(labeljust[i])->menu_item;
-	opt = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(active_item), 
-						  "option"));
-	spec->labels[i].just = opt;
-#else
 	spec->labels[i].just = 
 	    gtk_option_menu_get_history(GTK_OPTION_MENU(labeljust[i]));
-#endif
     } 
 
     if (!err && border_check != NULL) {
@@ -403,15 +388,11 @@ static void apply_gpt_changes (GtkWidget *widget, GPT_SPEC *spec)
 
     if (!err && ttfcombo != NULL && ttfspin != NULL) {
 	const gchar *tmp = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(ttfcombo)->entry));
-#ifdef OLD_GTK
 	int ptsize = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ttfspin));
-#else
-	int ptsize = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(ttfspin));
-#endif
 
 	if (tmp != NULL && *tmp != '\0') {
 	    const char *fname = get_font_filename(tmp);
-	    char pngfont[32];
+	    char pngfont[128];
 
 	    if (fname != NULL && ptsize > 5 && ptsize < 25) {
 		sprintf(pngfont, "%s %d", fname, ptsize);
@@ -717,9 +698,6 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
     /* set TT font (if gnuplot uses libgd and freetype) */
     if (gnuplot_has_ttf()) {
 	GtkWidget *ebox, *hsep;
-#ifdef OLD_GTK
-	GtkObject *adj;
-#endif
 	GList *fontnames = NULL;
 	struct font_info *ttflist;
 	const char *default_font = NULL;
@@ -755,30 +733,21 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
 	gtk_widget_show(label);
 	gtk_widget_show(ebox);
 
+	/* FIXME max length of font name? */
+
 	ttfcombo = gtk_combo_new();
 	gtk_entry_set_max_length(GTK_ENTRY(GTK_COMBO(ttfcombo)->entry), 15);
 	gtk_table_attach_defaults(GTK_TABLE(tbl), ttfcombo, 1, 2, 
 				  tbl_len-1, tbl_len);
 	gtk_combo_set_popdown_strings(GTK_COMBO(ttfcombo), fontnames); 
 	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(ttfcombo)->entry), default_font);
-#ifdef OLD_GTK
-	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(ttfcombo)->entry), "activate", 
-			   GTK_SIGNAL_FUNC(apply_gpt_changes), 
-			   spec);
-#else
 	gtk_entry_set_width_chars(GTK_ENTRY(GTK_COMBO(ttfcombo)->entry), 15);
 	g_signal_connect(G_OBJECT(GTK_COMBO(ttfcombo)->entry), "activate", 
 			 G_CALLBACK(apply_gpt_changes), 
 			 spec);
-#endif
 	gtk_widget_show (ttfcombo);
 
-#ifdef OLD_GTK
-	adj = gtk_adjustment_new(10, 6, 24, 1, 1, 1);
-	ttfspin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 0);
-#else
 	ttfspin = gtk_spin_button_new_with_range(6, 24, 1);
-#endif
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(ttfspin), 
 				  get_point_size(gretl_png_font()));
 	gtk_table_attach_defaults(GTK_TABLE(tbl), ttfspin, 2, 3, 
@@ -915,7 +884,8 @@ static void linetitle_callback (GtkWidget *w, GPT_SPEC *spec)
 
 static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec) 
 {
-    GtkWidget *label, *vbox, *tbl;
+    GtkWidget *label, *tbl;
+    GtkWidget *vbox, *hbox;
     int i, tbl_len, tbl_num, tbl_col;
     char label_text[32];
     GList *plot_types = NULL;
@@ -988,6 +958,7 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
 	gtk_widget_show(label);
 
 	label = gtk_label_new(_("legend"));
+	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 	gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				  label, 1, 2, tbl_len-1, tbl_len);
 	gtk_widget_show(label);
@@ -1011,6 +982,7 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
 	tbl_len++;
 	gtk_table_resize(GTK_TABLE(tbl), tbl_len, 3);
 	label = gtk_label_new(_("type"));
+	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 	gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				  label, 1, 2, tbl_len-1, tbl_len);
 	gtk_widget_show(label);
@@ -1038,6 +1010,7 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
 	    tbl_len++;
 	    gtk_table_resize(GTK_TABLE(tbl), tbl_len, 3);
 	    label = gtk_label_new(_("scale"));
+	    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 	    gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				      label, 1, 2, tbl_len-1, tbl_len);
 	    gtk_widget_show(label);
@@ -1045,16 +1018,10 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
 	    linescale[i] = gtk_entry_new();
 	    gtk_entry_set_max_length(GTK_ENTRY(linescale[i]), 6);
 	    gtk_entry_set_text(GTK_ENTRY(linescale[i]), spec->lines[i].scale);
-#ifdef OLD_GTK
-	    gtk_signal_connect(GTK_OBJECT(linescale[i]), "activate", 
-			       GTK_SIGNAL_FUNC(apply_gpt_changes), 
-			       spec);
-#else
 	    gtk_entry_set_width_chars(GTK_ENTRY(linescale[i]), 6);
 	    g_signal_connect(G_OBJECT(linescale[i]), "activate", 
 			     G_CALLBACK(apply_gpt_changes), 
 			     spec);
-#endif
 	    gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				      linescale[i], 2, 3, tbl_len-1, tbl_len);
 	    gtk_widget_show(linescale[i]);
@@ -1063,6 +1030,7 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
 	    tbl_len++;
 	    gtk_table_resize(GTK_TABLE(tbl), tbl_len, 3);
 	    label = gtk_label_new(_("y axis"));
+	    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 	    gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				      label, 1, 2, tbl_len-1, tbl_len);
 	    gtk_widget_show(label);
@@ -1074,7 +1042,31 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
 	    gtk_entry_set_text (GTK_ENTRY(GTK_COMBO(yaxiscombo[i])->entry), 
 				(spec->lines[i].yaxis == 1)? "left" : "right");  
 	    gtk_widget_show(yaxiscombo[i]);	
-	} 
+	}
+
+	/* line-width adjustment */
+	tbl_len++;
+	gtk_table_resize(GTK_TABLE(tbl), tbl_len, 3);
+	label = gtk_label_new(_("line width"));
+	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
+	gtk_table_attach_defaults(GTK_TABLE(tbl), 
+				  label, 1, 2, tbl_len-1, tbl_len);
+	gtk_widget_show(label);
+
+	hbox = gtk_hbox_new(FALSE, 5);
+	linewidth[i] = gtk_spin_button_new_with_range(1, 6, 1);
+	if (spec->lines[i].width > 1) {
+	    gtk_spin_button_set_value(GTK_SPIN_BUTTON(linewidth[i]),
+				      spec->lines[i].width);
+	}
+	g_signal_connect(G_OBJECT(linewidth[i]), "activate", 
+			 G_CALLBACK(apply_gpt_changes), 
+			 spec);
+	gtk_box_pack_start(GTK_BOX(hbox), linewidth[i], FALSE, FALSE, 0);
+	gtk_widget_show(linewidth[i]);
+	gtk_table_attach_defaults(GTK_TABLE(tbl), hbox, 2, 3, 
+				  tbl_len-1, tbl_len);
+	gtk_widget_show(hbox);
     }
 }
 
@@ -1117,12 +1109,7 @@ static void gpt_tab_labels (GtkWidget *notebook, GPT_SPEC *spec)
 
     for (i=0; i<MAX_PLOT_LABELS; i++) {
 	GtkWidget *hbox, *button, *image;
-#ifdef OLD_GTK
-	GdkPixmap *pixmap;
-	GdkBitmap *mask;	
-#else
 	GdkPixbuf *icon;
-#endif
 
 	/* label text */
 	tbl_len++;
@@ -1135,6 +1122,7 @@ static void gpt_tab_labels (GtkWidget *notebook, GPT_SPEC *spec)
 	gtk_widget_show(label);
 
 	label = gtk_label_new(_("text"));
+	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 	gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				  label, 1, 2, tbl_len-1, tbl_len);
 	gtk_widget_show(label);
@@ -1142,16 +1130,10 @@ static void gpt_tab_labels (GtkWidget *notebook, GPT_SPEC *spec)
 	labeltext[i] = gtk_entry_new();
 	gtk_entry_set_max_length(GTK_ENTRY(labeltext[i]), PLOT_LABEL_TEXT_LEN);
 	gp_string_to_entry(labeltext[i], spec->labels[i].text);
-#ifdef OLD_GTK
-	gtk_signal_connect (GTK_OBJECT(labeltext[i]), "activate", 
-			    GTK_SIGNAL_FUNC(apply_gpt_changes), 
-			    spec);
-#else
 	gtk_entry_set_width_chars(GTK_ENTRY(labeltext[i]), PLOT_LABEL_TEXT_LEN);
 	g_signal_connect (G_OBJECT(labeltext[i]), "activate", 
 			  G_CALLBACK(apply_gpt_changes), 
 			  spec);
-#endif
 	gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				  labeltext[i], 2, 3, tbl_len-1, tbl_len);
 	gtk_widget_show(labeltext[i]);
@@ -1161,6 +1143,7 @@ static void gpt_tab_labels (GtkWidget *notebook, GPT_SPEC *spec)
 
 	gtk_table_resize(GTK_TABLE(tbl), tbl_len, 3);
 	label = gtk_label_new(_("position (X Y)"));
+	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 	gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				  label, 1, 2, tbl_len-1, tbl_len);
 	gtk_widget_show(label);
@@ -1172,16 +1155,10 @@ static void gpt_tab_labels (GtkWidget *notebook, GPT_SPEC *spec)
 	labelpos[i] = gtk_entry_new();
 	gtk_entry_set_max_length(GTK_ENTRY(labelpos[i]), PLOT_LABEL_POS_LEN);
 	label_pos_to_entry(spec->labels[i].pos, labelpos[i]);
-#ifdef OLD_GTK
-	gtk_signal_connect(GTK_OBJECT(labelpos[i]), "activate", 
-			   GTK_SIGNAL_FUNC(apply_gpt_changes), 
-			   spec);
-#else
 	gtk_entry_set_width_chars(GTK_ENTRY(labelpos[i]), PLOT_LABEL_POS_LEN);
 	g_signal_connect(G_OBJECT(labelpos[i]), "activate", 
 			 G_CALLBACK(apply_gpt_changes), 
 			 spec);
-#endif
 	gtk_container_add(GTK_CONTAINER(hbox), labelpos[i]);
 	gtk_widget_show(labelpos[i]);
 
@@ -1191,15 +1168,9 @@ static void gpt_tab_labels (GtkWidget *notebook, GPT_SPEC *spec)
 	    g_object_set_data(G_OBJECT(button), "labelpos_entry", labelpos[i]);
 	    g_signal_connect(G_OBJECT(button), "clicked",
 			     G_CALLBACK(plot_label_position_click), spec);
-#ifdef OLD_GTK
-	    pixmap = gdk_pixmap_create_from_xpm_d(mdata->w->window,
-						  &mask, NULL, mini_mouse_xpm);
-	    image = gtk_pixmap_new(pixmap, mask);
-#else
 	    icon = gdk_pixbuf_new_from_xpm_data((const char **) mini_mouse_xpm);
 	    image = gtk_image_new_from_pixbuf(icon);
 	    gtk_widget_set_size_request(button, 32, 26);
-#endif
 	    gtk_container_add (GTK_CONTAINER(button), image);
 	    gtk_container_add(GTK_CONTAINER(hbox), button);
 	    gtk_widget_show_all(button);
@@ -1213,6 +1184,7 @@ static void gpt_tab_labels (GtkWidget *notebook, GPT_SPEC *spec)
 	tbl_len++;
 	gtk_table_resize(GTK_TABLE(tbl), tbl_len, 3);
 	label = gtk_label_new(_("justification"));
+	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 	gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				  label, 1, 2, tbl_len-1, tbl_len);
 	gtk_widget_show(label);
@@ -1224,9 +1196,6 @@ static void gpt_tab_labels (GtkWidget *notebook, GPT_SPEC *spec)
 
 	    item = gtk_menu_item_new_with_label(gp_justification_string(j));
 	    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-#ifdef OLD_GTK
-	    gtk_object_set_data(GTK_OBJECT(item), "option", GINT_TO_POINTER(j));
-#endif
 	}
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(labeljust[i]), menu);	
 	gtk_option_menu_set_history(GTK_OPTION_MENU(labeljust[i]),
@@ -1390,6 +1359,7 @@ int show_gnuplot_dialog (GPT_SPEC *spec)
 {
     png_plot *plot = (png_plot *) spec->ptr;
     GtkWidget *button, *notebook;
+    GtkWidget *hbox;
     int i;
 
     if (gpt_control != NULL) {
@@ -1406,16 +1376,9 @@ int show_gnuplot_dialog (GPT_SPEC *spec)
 	gpt_titles[i].widget = NULL;
     }
 
-    gpt_control = gtk_dialog_new();
-    gtk_window_set_title(GTK_WINDOW(gpt_control), _("gretl plot controls"));
-    gtk_container_set_border_width 
-        (GTK_CONTAINER(GTK_DIALOG(gpt_control)->vbox), 10);
-    gtk_container_set_border_width 
-        (GTK_CONTAINER(GTK_DIALOG(gpt_control)->action_area), 5);
+    gpt_control = gretl_dialog_new(_("gretl plot controls"), NULL, 0);
     gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(gpt_control)->vbox), 2);
-    gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(gpt_control)->action_area), 15);
-    gtk_box_set_homogeneous(GTK_BOX(GTK_DIALOG(gpt_control)->action_area), TRUE);
-    gtk_window_set_position(GTK_WINDOW(gpt_control), GTK_WIN_POS_MOUSE);
+    gtk_dialog_set_has_separator(GTK_DIALOG(gpt_control), FALSE);
 
     if (plot != NULL) {
 	gtk_window_set_transient_for(GTK_WINDOW(gpt_control), 
@@ -1443,20 +1406,17 @@ int show_gnuplot_dialog (GPT_SPEC *spec)
     gpt_tab_labels(notebook, spec); 
     gpt_tab_output(notebook, spec);
 
+    hbox = GTK_DIALOG(gpt_control)->action_area;
+
     /* "Apply" button */
-    button = standard_button(GTK_STOCK_APPLY);
-    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(gpt_control)->action_area), 
-		       button, TRUE, TRUE, 0);
+    button = apply_button(hbox);
     g_signal_connect(G_OBJECT(button), "clicked", 
 		     G_CALLBACK(apply_gpt_changes), spec);
     gtk_widget_grab_default(button);
     gtk_widget_show(button);
 
     /* "OK" button (apply and close) */
-    button = standard_button(GTK_STOCK_OK);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(gpt_control)->action_area), 
-		       button, TRUE, TRUE, 0);
+    button = ok_button(hbox);
     g_signal_connect(G_OBJECT(button), "clicked", 
 		     G_CALLBACK(apply_gpt_changes), spec);
     g_signal_connect(G_OBJECT(button), "clicked",
@@ -1466,20 +1426,13 @@ int show_gnuplot_dialog (GPT_SPEC *spec)
     /* Close button (do not apply changes) */
     button = standard_button(GTK_STOCK_CLOSE);
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(gpt_control)->action_area), 
-		       button, TRUE, TRUE, 0);
+    gtk_container_add(GTK_CONTAINER(hbox), button);
     g_signal_connect(G_OBJECT(button), "clicked",
 		     G_CALLBACK(close_gnuplot_dialog), gpt_control);
     gtk_widget_show(button);
 
-    button = standard_button(GTK_STOCK_HELP);
-    GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(gpt_control)->action_area), 
-		       button, TRUE, TRUE, 0);
-    g_signal_connect(G_OBJECT(button), "clicked", 
-		     G_CALLBACK(context_help), 
-		     GINT_TO_POINTER(GR_PLOT));
-    gtk_widget_show(button);
+    /* Help button */
+    context_help_button(hbox, GR_PLOT);
 
     set_keyspec_sensitivity(spec);
 
