@@ -278,6 +278,19 @@ int graph_page_add_file (const char *fname)
     return 0;
 }
 
+int in_graph_page (const char *fname)
+{
+    int i;
+
+    for (i=0; i<gpage.ngraphs; i++) {
+	if (!strcmp(fname, gpage.fnames[i])) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
 static int gnuplot_compile (const char *fname)
 {
     char plotcmd[MAXLEN];
@@ -363,7 +376,7 @@ static int get_dvips_path (char *path)
     return (ret == 0);
 }
 
-#elif !defined(OLD_GTK)
+#else
 
 #include <signal.h>
 
@@ -423,8 +436,6 @@ int dvips_compile (char *texshort)
 {
 #ifdef G_OS_WIN32
     static char dvips_path[MAXLEN];
-#endif
-#if defined(G_OS_WIN32) || defined(OLD_GTK)
     char tmp[MAXLEN];
 #endif
     int err = 0;
@@ -440,9 +451,6 @@ int dvips_compile (char *texshort)
     if (winfork(tmp, paths.userdir, SW_SHOWMINIMIZED, CREATE_NEW_CONSOLE)) {
 	return 1;
     }
-#elif defined(OLD_GTK)
-    sprintf(tmp, "cd \"%s\" && dvips -o %s.ps %s", paths.userdir, texshort, texshort);
-    err = system(tmp);
 #else
     err = spawn_dvips(texshort);
 #endif 
@@ -515,12 +523,14 @@ static int real_display_gpage (void)
 	fname = gpage_fname(".ps", 0);
     }
 
-#ifdef G_OS_WIN32
+#if defined(G_OS_WIN32)
     if ((int) ShellExecute(NULL, "open", fname, NULL, NULL, SW_SHOW) <= 32) {
 	DWORD dw = GetLastError();
 	win_show_error(dw);
 	err = 1;
     }
+#elif defined(OSX_BUILD)
+    err = osx_open_file(fname);
 #else
     viewer = (gpage.output == PDF_OUTPUT)? viewpdf : viewps;
     err = gretl_fork(viewer, fname);
