@@ -39,12 +39,7 @@
 #include <stdlib.h>
 #include <time.h>
 
-#if GTK_MAJOR_VERSION < 2
-# define OLD_GTK
-# include <gtkextra/gtkiconfilesel.h>
-#else
-# define GNULL (gconstpointer) NULL
-#endif
+#define GNULL (gconstpointer) NULL
 
 #ifdef USE_GTKSOURCEVIEW
 # include <gtksourceview/gtksourceview.h>
@@ -57,13 +52,14 @@
 #include "library.h"
 #include "settings.h"
 #include "helpfiles.h"
+#include "gui_recode.h"
 
 #ifdef ENABLE_NLS
 # include "locale.h"
 #endif
 
 #define GRETL_BUFSIZE 8192
-#define MAXSTR 255
+#define MAXSTR FILENAME_MAX
 
 /* basic global program vars */
 extern double **Z;
@@ -83,8 +79,8 @@ extern float gui_scale;
 /* global filenames */
 extern char cmdfile[MAXLEN];
 extern char scriptfile[MAXLEN];
-extern char trydatfile[MAXLEN];
-extern char tryscript[MAXLEN];
+extern char sessionfile[MAXLEN];
+extern char tryfile[MAXLEN];
 
 /* global option-related vars */
 extern int expert;
@@ -94,15 +90,21 @@ extern int main_x;
 extern int main_y;
 extern int mainwin_width;
 extern int mainwin_height;
-#ifdef G_OS_WIN32
+
+#if defined(G_OS_WIN32)
 extern int wimp;
 #else
+# ifndef OSX_BUILD
 extern char viewps[MAXSTR];
 extern char viewpdf[MAXSTR];
+extern char Browser[MAXSTR];
+# endif
 #endif
+
 extern char calculator[MAXSTR];
 extern char latex[MAXSTR];
 extern char viewdvi[MAXSTR];
+
 #if defined(HAVE_AUDIO) && !defined(G_OS_WIN32)
 extern char midiplayer[MAXSTR];
 #endif
@@ -110,11 +112,7 @@ extern char midiplayer[MAXSTR];
 /* global GUI equipment */
 extern windata_t *mdata;
 extern GtkTargetEntry gretl_drag_targets[];
-#ifdef OLD_GTK
-extern GdkFont *fixed_font;
-#else
 extern PangoFontDescription *fixed_font;
-#endif
 
 #include "gretl_enums.h"
 
@@ -126,6 +124,7 @@ void set_wm_icon (GtkWidget *w, gpointer data);
 #endif
 
 int mdata_selection_count (void);
+int mdata_active_var (void);
 void populate_varlist (void);
 void clear_varlist (GtkWidget *widget);
 int gui_restore_sample (void);
