@@ -8,14 +8,13 @@ enum extra_cmds {
     SMPLDUM,
     SMPLBOOL,
     SMPLRAND,
-    SETSEED,
     MARKERS,
     STORE_MODEL,
     VAR_SUMMARY,
-    CORR_SELECTED,
-    SUMMARY_SELECTED,
-    GENR_NORMAL,
-    GENR_UNIFORM,
+    ALL_SUMMARY,
+    ALL_CORR,
+    GENR_RANDOM,
+    SEED_RANDOM,
     ONLINE,
     EXPORT,
     MEANTEST2,
@@ -43,6 +42,8 @@ enum extra_cmds {
     VIEW_SCRIPT,
     VIEW_CODEBOOK,
     VIEW_MODELTABLE,
+    VIEW_FUNC_INFO,
+    VIEW_FUNC_CODE,
     VIEW_FILE,
     DATA_REPORT,
     SCRIPT_OUT,
@@ -50,6 +51,7 @@ enum extra_cmds {
     EDIT_HEADER,
     EDIT_SCRIPT,
     EDIT_NOTES,
+    EDIT_FUNC_CODE,
     CLI_HELP,
     GUI_HELP,
     CLI_HELP_EN,
@@ -79,13 +81,18 @@ enum extra_cmds {
     MINIBUF,
     VLAGSEL,
     VAROMIT,
+    DEFINE_LIST,
+    PANEL_WLS,
+    PANEL_MODE,
+    TSPLOTS,
+    ITERATIONS,
     GUI_CMD_MAX
 };
 
 enum file_ops {
     OPEN_DATA = GUI_CMD_MAX + 1, /* don't collide with extra_cmds */
-    OPEN_DB,
-    OPEN_RATSDB,
+    OPEN_RATS_DB,
+    OPEN_PCGIVE_DB,
     OPEN_SCRIPT,
     OPEN_CSV,
     APPEND_DATA,
@@ -103,19 +110,19 @@ enum file_ops {
     APPEND_WF1,
     OPEN_DTA,
     APPEND_DTA,
+    OPEN_JMULTI,
+    APPEND_JMULTI,
     OPEN_SESSION,
     OPEN_MARKERS,
     END_OPEN,      /* marker for end of file open section */
     SAVE_DATA,
     SAVE_DATA_AS,
-    SAVE_GZDATA,
     SAVE_DBDATA,
-    SAVE_BIN1,
-    SAVE_BIN2,
     EXPORT_OCTAVE,
     EXPORT_R,
     EXPORT_CSV,
     EXPORT_DAT,
+    EXPORT_JM,
     COPY_CSV,
     END_SAVE_DATA,  /* marker for end of data-saving section */
     SAVE_TEX,
@@ -131,7 +138,9 @@ enum file_ops {
     SAVE_THIS_GRAPH,
     SAVE_GP_CMDS,
     SAVE_CONSOLE,
-    SET_PATH,
+    SAVE_FUNCTIONS,
+    SET_PROG,
+    SET_DIR,
     FILE_OP_MAX
 };
 
@@ -141,11 +150,13 @@ enum browser_codes {
     TEXTBOOK_DATA = FILE_OP_MAX + 1, /* don't collide with file_ops enum */
     PS_FILES,
     FUNC_FILES,
+    REMOTE_FUNC_FILES,
+    FUNC_EDIT,
     NATIVE_DB,
-    RATS_DB,
     REMOTE_DB,
     NATIVE_SERIES,
     RATS_SERIES,
+    PCGIVE_SERIES,
     REMOTE_SERIES,
     MAINWIN
 };
@@ -166,9 +177,7 @@ enum exec_codes {
     CONSOLE_EXEC      = 1 << 0,
     SCRIPT_EXEC       = 1 << 1,
     SESSION_EXEC      = 1 << 2,
-    REBUILD_EXEC      = 1 << 3,
-    SAVE_SESSION_EXEC = 1 << 4,
-    INCLUDE_EXEC      = 1 << 5
+    INCLUDE_EXEC      = 1 << 3
 };
 
 enum clipstuff {
@@ -200,11 +209,6 @@ enum file_lists {
     FILE_LIST_SCRIPT,
 };
 
-enum graph_types {
-    GRETL_GNUPLOT_GRAPH,
-    GRETL_BOXPLOT
-};
-
 enum varclick_actions {
     VARCLICK_NONE,
     VARCLICK_INSERT_ID,
@@ -224,14 +228,28 @@ enum calc_functions {
     CALC_TEST
 };
 
+enum auto_dummies {
+    TS_DUMMIES,
+    PANEL_UNIT_DUMMIES,
+    PANEL_TIME_DUMMIES
+};
+
 enum dynamic_forecast_status {
     DYNAMIC_NA,
     DYNAMIC_OK,
     DYNAMIC_FORCED
 };
 
+enum random_types {
+    RANDOM_UNIFORM,
+    RANDOM_NORMAL,
+    RANDOM_CHISQ,
+    RANDOM_ST
+};
+
 #define MULTI_FORMAT_ENABLED(c) (c == SUMMARY || c == VAR_SUMMARY || \
-	                         c == CORR || c == FCASTERR || \
+                                 c == ALL_SUMMARY || \
+	                         c == CORR || c == ALL_CORR || c == FCASTERR || \
 	                         c == FCAST || c == COEFFINT || \
 	                         c == COVAR || c == VIEW_MODEL || \
                                  c == VIEW_MODELTABLE || c == VAR || c == VECM || \
