@@ -31,13 +31,10 @@
 #define GRETL_STOCK_PDF    "gretl-pdf"
 #define GRETL_STOCK_BOOK   "gretl-book"
 
-#if (GTK_MAJOR_VERSION >= 2) && (GTK_MINOR_VERSION < 4)
-# define GTK_STOCK_NETWORK  "gretl-network"
-#endif
-
 enum windata_flags {
-    VWIN_HELP_ACTIVE = 1 << 0,
-    VWIN_BUSY        = 1 << 1
+    VWIN_HELP_ACTIVE  = 1 << 0,
+    VWIN_BUSY         = 1 << 1,
+    VWIN_DELETE_FNAME = 1 << 2
 };
 
 typedef struct _windata_t windata_t;
@@ -73,4 +70,8 @@ struct _windata_t {
 #define set_window_help_active(w)   (w->flags |= VWIN_HELP_ACTIVE)
 #define unset_window_help_active(w) (w->flags &= ~VWIN_HELP_ACTIVE)
 
+#define window_delete_filename(w)       (w->flags & VWIN_DELETE_FNAME)
+#define set_window_delete_filename(w)   (w->flags |= VWIN_DELETE_FNAME)
+#define unset_window_delete_filename(w) (w->flags &= ~VWIN_DELETE_FNAME)
+
 #endif /* GRETLTYPES_H */
