@@ -24,6 +24,7 @@
 #include "textutil.h"
 #include "guiprint.h"
 #include "gpt_control.h"
+#include "menustate.h"
 
 #include <gdk/gdkwin32.h>
 #include <dirent.h>
@@ -91,7 +92,7 @@ void startR (const char *Rcommand)
 	return;
     }
 
-    build_path(paths.userdir, "gretl.Rprofile", Rprofile, NULL);
+    build_path(Rprofile, paths.userdir, "gretl.Rprofile", NULL);
     fp = gretl_fopen(Rprofile, "w");
     if (fp == NULL) {
 	errbox(_("Couldn't write R startup file"));
@@ -105,7 +106,7 @@ void startR (const char *Rcommand)
 	return;
     } 	
 
-    build_path(paths.userdir, "Rdata.tmp", Rdata, NULL);
+    build_path(Rdata, paths.userdir, "Rdata.tmp", NULL);
 
     sprintf(Rline, "store \"%s\" -r", Rdata);
     list = command_list_from_string(Rline);
@@ -129,13 +130,14 @@ void startR (const char *Rcommand)
 	char Rtmp[MAXLEN];
 	FILE *fq;
 
-	build_path(paths.userdir, "Rtmp", Rtmp, NULL);
+	build_path(Rtmp, paths.userdir, "Rtmp", NULL);
 	fq = gretl_fopen(Rtmp, "w");
-	fprintf(fq, "gretldata <- read.table(\"%s\")\n", 
-		slash_convert(Rdata, FROM_BACKSLASH));
-	fprintf(fq, "attach(gretldata)\n");
-	fclose(fq);
-
+	if (fq != NULL) {
+	    fprintf(fq, "gretldata <- read.table(\"%s\")\n", 
+		    slash_convert(Rdata, FROM_BACKSLASH));
+	    fprintf(fq, "attach(gretldata)\n");
+	    fclose(fq);
+	}
 	fprintf(fp, "source(\"%s\", echo=TRUE)\n", 
 		slash_convert(Rtmp, FROM_BACKSLASH));
     }
@@ -349,8 +351,13 @@ void set_up_windows_look (void)
 	int needslash = (paths.gretldir[n-1] != SLASH);
 	gchar *wimprc;
 
+#ifdef NEW_GTK
+	wimprc = g_strdup_printf("%s%sshare\\themes\\MS-Windows\\gtk-2.0\\gtkrc", 
+				 paths.gretldir, (needslash)? "\\" : "");
+#else
 	wimprc = g_strdup_printf("%s%setc\\gtk-2.0\\gtkrc.wimp", 
 				 paths.gretldir, (needslash)? "\\" : "");
+#endif
 	gtk_rc_parse(wimprc);
 	g_free(wimprc);
     } else {
@@ -427,7 +434,7 @@ void gretl_win32_init (const char *progname)
     atexit(write_rc);
 }
 
-static int win_mkdir (const char *path)
+int gretl_mkdir (const char *path)
 {
     DIR *test;
     int done;
@@ -456,7 +463,7 @@ void win32_make_user_dirs (void)
 	dirname[n-1] = '\0';
     }
 
-    if (win_mkdir(dirname)) {
+    if (gretl_mkdir(dirname)) {
 	gchar *msg;
 
 	msg = g_strdup_printf("Couldn't open or create gretl "
@@ -466,28 +473,28 @@ void win32_make_user_dirs (void)
 	return;
     }
 
-    build_path(paths.userdir, "x12arima", paths.x12adir, NULL);
-    win_mkdir(paths.x12adir);
+    build_path(paths.x12adir, paths.userdir, "x12arima", NULL);
+    gretl_mkdir(paths.x12adir);
 
-    build_path(paths.userdir, "tramo", tramodir, NULL);
-    if (win_mkdir(tramodir)) return;
+    build_path(tramodir, paths.userdir, "tramo", NULL);
+    if (gretl_mkdir(tramodir)) return;
 
     sprintf(dirname, "%s\\output", tramodir);
-    win_mkdir(dirname);
+    gretl_mkdir(dirname);
 
     sprintf(dirname, "%s\\graph", tramodir);
-    if (win_mkdir(dirname)) return;
+    if (gretl_mkdir(dirname)) return;
 
     sprintf(dirname, "%s\\graph\\acf", tramodir);
-    win_mkdir(dirname);
+    gretl_mkdir(dirname);
     sprintf(dirname, "%s\\graph\\filters", tramodir);
-    win_mkdir(dirname);
+    gretl_mkdir(dirname);
     sprintf(dirname, "%s\\graph\\forecast", tramodir);
-    win_mkdir(dirname);
+    gretl_mkdir(dirname);
     sprintf(dirname, "%s\\graph\\series", tramodir);
-    win_mkdir(dirname);
+    gretl_mkdir(dirname);
     sprintf(dirname, "%s\\graph\\spectra", tramodir);
-    win_mkdir(dirname);
+    gretl_mkdir(dirname);
 }
 
 static int win_copy_buf (const char *buf, int fmt, size_t buflen)
@@ -648,6 +655,7 @@ int send_file (char *fullname)
     } else {
 	char *fname = fname_from_fullname(fullname);
 	gchar *note = NULL;
+	gchar *tmp = NULL;
 	MapiFileDesc mfd;
 	MapiMessage msg;
 	ULONG sd;
@@ -660,13 +668,18 @@ int send_file (char *fullname)
 	mfd.nPosition = 1; /* ? */
 
 	if (strstr(fname, ".gdt") != NULL) {
-	    note = g_strdup_printf("Please find the gretl data file %s attached.\n", fname);
+	    tmp = g_strdup_printf(_("Please find the gretl data file %s attached."), 
+				  fname);
 	    msg.lpszSubject  = "dataset";
 	} else {
-	    note = g_strdup_printf("Please find the gretl script %s attached.\n", fname);
+	    tmp = g_strdup_printf(_("Please find the gretl script %s attached."), 
+				  fname);
 	    msg.lpszSubject  = "script";
 	}
 
+	note = g_strdup_printf("%s\n", tmp);
+	g_free(tmp);
+
 	msg.lpszNoteText = note;
 	msg.nFileCount = 1;
 	msg.lpFiles = &mfd;
@@ -802,3 +815,60 @@ void win32_process_graph (GPT_SPEC *spec, int color, int dest)
     remove(emfname);
     g_free(emfname);
 }
+
+static long GetRegKey (HKEY key, char *subkey, char *retdata)
+{
+    long err;
+    HKEY hkey;
+
+    err = RegOpenKeyEx(key, subkey, 0, KEY_QUERY_VALUE, &hkey);
+
+    if (err == ERROR_SUCCESS) {
+	long datasize = MAX_PATH;
+	char data[MAX_PATH];
+
+	RegQueryValue(hkey, NULL, (LPSTR) data, &datasize);
+
+	lstrcpy(retdata, data);
+	RegCloseKey(hkey);
+    }
+
+    return err;
+}
+
+int browser_open (const char *url)
+{
+    char key[MAX_PATH + MAX_PATH];
+    int err = 0;
+
+    if ((long) ShellExecute(NULL, "open", url, NULL, NULL, SW_SHOW) <= 32) {
+	/* if the above fails, get the .htm regkey and 
+	   look up the program */
+	if (GetRegKey(HKEY_CLASSES_ROOT, ".htm", key) == ERROR_SUCCESS) {
+	    lstrcat(key,"\\shell\\open\\command");
+	    if (GetRegKey(HKEY_CLASSES_ROOT, key, key) == ERROR_SUCCESS) {
+		char *p;
+
+		p = strstr(key, "\"%1\"");
+		if (p == NULL) {    
+		    /* so check for %1 without the quotes */
+		    p = strstr(key, "%1");
+		    if (p == NULL) {
+			/* if no parameter */
+			p = key + lstrlen(key) - 1;
+		    } else {
+			*p = '\0';    /* remove the param */
+		    }
+		} else {
+		    *p = '\0';        /* remove the param */
+		}
+
+		lstrcat(p, " ");
+		lstrcat(p, url);
+		if (WinExec(key, SW_SHOW) < 32) err = 1;
+	    }
+	}
+    }
+
+    return err;
+}
