@@ -20,62 +20,38 @@
 #ifndef GUI_UTILS_H
 #define GUI_UTILS_H
 
-#ifdef OLD_GTK
+#define standard_button(s) gtk_button_new_from_stock(s)
 
-/* various forward-compatibility bodges for GTK+ 1.2 */
-
-enum {
-    GTK_STOCK_OK,
-    GTK_STOCK_CANCEL,
-    GTK_STOCK_CLEAR,
-    GTK_STOCK_CLOSE,
-    GTK_STOCK_APPLY,
-    GTK_STOCK_HELP,
-    GTK_STOCK_GO_FORWARD,
-    GTK_STOCK_GO_BACK,
-    GTK_STOCK_FIND
-};
-
-# define G_OBJECT(o)                    GTK_OBJECT(o)
-# define g_object_set_data(o,s,d)       gtk_object_set_data(o,s,d)
-# define g_object_get_data(o,s)         gtk_object_get_data(o,s)
-# define G_CALLBACK(f)                  GTK_SIGNAL_FUNC(f)
-# define g_signal_connect(o,s,f,p)      gtk_signal_connect(o,s,f,p)
-# define gtk_radio_button_get_group(b)  gtk_radio_button_group(b)
-# define gtk_notebook_set_current_page(n, p) gtk_notebook_set_page(n, p)
-# define gtk_widget_set_size_request(v,w,h) gtk_widget_set_usize(v,w,h)
-
-void gtk_entry_set_activates_default (GtkEntry *entry, gboolean setting);
-
-#else
-
-# define standard_button(s) gtk_button_new_from_stock(s)
-
-gchar *my_filename_from_utf8 (char *fname);
-gchar *my_locale_from_utf8 (const gchar *src);
-gchar *force_locale_from_utf8 (const gchar *src);
-gchar *my_filename_to_utf8 (char *fname);
-gchar *my_locale_to_utf8 (const gchar *src);
 void gretl_stock_icons_init (void);
 
-#endif /* OLD_GTK */
-
 #ifdef ENABLE_NLS
 gchar *menu_translate (const gchar *path, gpointer p);
 #endif
 
-void flip (GtkItemFactory *ifac, const char *path, gboolean s);
+int probably_native_datafile (const char *fname);
+
+int probably_script_file (const char *fname);
+
+int probably_session_file (const char *fname);
+
+int gretl_mkdir (const char *path);
 
 int copyfile (const char *src, const char *dest);
 
 int isdir (const char *path);
 
+FILE *gretl_tempfile_open (char *fname);
+
+int gretl_tempname (char *fname);
+
 void delete_widget (GtkWidget *widget, gpointer data);
 
 void *mymalloc (size_t size); 
 
 void *myrealloc (void *ptr, size_t size);
 
+void nomem (void);
+
 void mark_dataset_as_modified (void);
 
 void register_data (char *fname, const char *user_fname,
@@ -85,9 +61,7 @@ void do_open_data (GtkWidget *w, gpointer data, int code);
 
 void verify_open_data (gpointer userdata, int code);
 
-void verify_open_session (gpointer userdata);
-
-void save_session (char *fname);
+void verify_open_session (void);
 
 void close_window (gpointer data, guint win_code, GtkWidget *widget);
 
@@ -103,6 +77,8 @@ int winstack_match_data (gpointer p);
 
 GtkWidget *match_window_by_data (gpointer p);
 
+void mark_content_saved (windata_t *vwin);
+
 windata_t *view_buffer (PRN *prn, int hsize, int vsize, 
 			const char *title, int role,
 			gpointer data);
@@ -121,23 +97,17 @@ int view_model (PRN *prn, MODEL *pmod, int hsize, int vsize,
 
 int highest_numbered_variable_in_winstack (void);
 
-void file_view_set_editable (windata_t *vwin);
+void view_window_set_editable (windata_t *vwin);
 
 int validate_varname (const char *varname);
 
-gint catch_listbox_key (GtkWidget *w, GdkEventKey *key, 
-			windata_t *vwin);
+gretlopt get_tex_eqn_opt (void);
 
 gint popup_menu_handler (GtkWidget *widget, GdkEvent *event,
 			 gpointer data);
 
-#if GTK_MAJOR_VERSION >= 2
 void add_popup_item (const gchar *label, GtkWidget *menu,
 		     GCallback callback, gpointer data);
-#else
-void add_popup_item (const gchar *label, GtkWidget *menu,
-		     GtkSignalFunc callback, gpointer data);
-#endif
 
 void get_stats_table (void);
 
@@ -151,6 +121,7 @@ char *double_underscores (char *targ, const char *src);
 
 #ifndef G_OS_WIN32
 void startR (const char *Rcommand);
+int browser_open (const char *url);
 #endif
 
 #if defined(HAVE_FLITE) || defined(G_OS_WIN32)
@@ -160,6 +131,7 @@ enum {
 } audio_render_keys;
 
 void audio_render_window (windata_t *vwin, int key);
+void stop_talking (void);
 #endif
 
 #endif /* GUI_UTILS_H */
