@@ -22,12 +22,8 @@
 #include "gretl.h"
 #include "textbuf.h"
 #include "webget.h"
-
-#ifdef OLD_GTK
-# include "dlgutils.h"
-#else
-# include "treeutils.h"
-#endif
+#include "treeutils.h"
+#include "dlgutils.h"
 
 #ifndef G_OS_WIN32
 # include <sys/stat.h>
@@ -58,21 +54,17 @@ static help_head **en_cli_heads, **en_gui_heads;
 
 static windata_t *helpwin (int cli, int en);
 static void real_do_help (int hcode, int pos, int cli, int en);
+static void en_text_cmdref (gpointer p, guint u, GtkWidget *w);
 
 /* searching stuff */
 static void find_in_text (GtkWidget *widget, gpointer data);
 static void find_in_listbox (GtkWidget *widget, gpointer data);
 static void find_string_dialog (void (*findfunc)(), gpointer data);
-#ifdef OLD_GTK
-static int clist_start_row;
-#endif
 
 static GtkWidget *find_window = NULL;
 static GtkWidget *find_entry;
 static char *needle;
 
-#ifndef OLD_GTK
-
 GtkItemFactoryEntry help_menu_items[] = {
     { N_("/_Topics"), NULL, NULL, 0, "<Branch>", GNULL },    
     { N_("/_Find"), NULL, NULL, 0, "<Branch>", GNULL },   
@@ -80,16 +72,6 @@ GtkItemFactoryEntry help_menu_items[] = {
     { NULL, NULL, NULL, 0, NULL, GNULL }
 };
 
-#else
-
-GtkItemFactoryEntry help_menu_items[] = {
-    { N_("/_Topics"), NULL, NULL, 0, "<Branch>" },    
-    { N_("/_Find"), NULL, menu_find, 0, NULL },
-    { NULL, NULL, NULL, 0, NULL}
-};
-
-#endif
-
 struct gui_help_item {
     int code;
     char *string;
@@ -116,7 +98,8 @@ static struct gui_help_item gui_help_items[] = {
     { GUI_HELP,       "dialog" },
     { MODELTABLE,     "modeltab" },
     { GRAPHPAGE,      "graphpag" },
-    { SETSEED,        "seed" },
+    { GENR_RANDOM,    "genrand" },
+    { SEED_RANDOM,    "genseed" },
     { KERNEL_DENSITY, "density" },
     { HCCME,          "hccme" },
     { IRF_BOOT,       "irfboot" },
@@ -129,6 +112,8 @@ static struct gui_help_item gui_help_items[] = {
     { MINIBUF,        "minibuffer" },
     { VLAGSEL,        "VAR-lagselect" },
     { VAROMIT,        "VAR-omit" },
+    { PANEL_MODE,     "panel-mode" },
+    { PANEL_WLS,      "panel-wls" },
     { -1,          NULL },
 };
 
@@ -457,12 +442,10 @@ static void free_help_heads (help_head **heads, int nh)
     free(heads);
 }
 
-static int add_topic_to_head (help_head *head, int j, const char *word,
-			      char *label, int gui)
+static void add_topic_to_head (help_head *head, int j, const char *word,
+			       char *label, int gui)
 {
-    int hnum, err = 0;
-
-    hnum = help_topic_number(word, gui);
+    int hnum = help_topic_number(word, gui);
 
     if (hnum > 0) {
 	head->topics[j] = hnum;
@@ -470,17 +453,20 @@ static int add_topic_to_head (help_head *head, int j, const char *word,
 	    head->topicnames[j] = label;
 	}
     } else {
-	fprintf(stderr, "helpfile: add_topic_to_head: hnum=%d for word='%s'\n", 
-		hnum, word);
-	err = 1;
+	fprintf(stderr, "helpfile error: word='%s' not recognized\n", word);
+#if 0
+	head->topics[j] = 0;
+	head->topicnames[j] = NULL;
+#endif
+	if (label != NULL) {
+	    free(label);
+	}
     }
 
 #if HDEBUG
     fprintf(stderr, "add_topic_to_head: topic %d: word = %s: hnum = %d\n", 
 	    j+1, word, hnum);
 #endif
-
-    return err;
 }
 
 static int 
@@ -556,7 +542,7 @@ get_helpfile_structure (help_head ***pheads, int gui, const char *fname)
 	/* heading with at least one topic */
 	heads[nh2] = help_head_new(tmp, nt, gui);
 	if (heads[nh2] == NULL) {
-	    fprintf(stderr, "healp_head_new() failed\n");
+	    fprintf(stderr, "help_head_new() failed\n");
 	    err = 1;
 	}
 
@@ -580,12 +566,7 @@ get_helpfile_structure (help_head ***pheads, int gui, const char *fname)
 	    }
 
 	    if (!err) {
-		err = add_topic_to_head(heads[nh2], j, tmp, label, gui);
-		if (err) {
-		    if (label != NULL) {
-			free(label);
-		    }
-		}
+		add_topic_to_head(heads[nh2], j, tmp, label, gui);
 	    }
 	}
 
@@ -645,7 +626,7 @@ static int real_helpfile_init (int gui, int en)
     err = get_helpfile_structure(&heads, gui, helpfile);
 
     if (err) {
-	errbox(_("help file %s is not accessible\n"), helpfile);
+	errbox(_("help file %s is not up to date\n"), helpfile);
     } else {
 	if (gui) {
 	    if (en) {
@@ -821,10 +802,10 @@ static void add_en_help_item (windata_t *hwin, int cli)
 
 static void add_help_topics (windata_t *hwin, int cli, int en)
 {
-    int i, j;
     GtkItemFactoryEntry hitem;
     const gchar *mpath = N_("/_Topics");
     help_head **hds;
+    int i, j;
 
     if (en) {
 	hds = (cli)? en_cli_heads : en_gui_heads;
@@ -832,33 +813,36 @@ static void add_help_topics (windata_t *hwin, int cli, int en)
 	hds = (cli)? cli_heads : gui_heads;
     }
 
+    /* Are there any topics to add? */
+    if (hds == NULL) {
+	return;
+    }
+
     hitem.accelerator = NULL;
 
-    /* See if there are any topics to add */
-    if (hds == NULL) return;
-
-#ifndef OLD_GTK
     if (cli) {
 	/* Add general index as "topic" */
 	hitem.callback_action = 0; 
 	hitem.item_type = NULL;
-	hitem.path = g_strdup_printf("%s/%s", mpath, _("Index"));
-	hitem.callback = plain_text_cmdref;
+	hitem.path = g_strdup_printf("%s/%s", mpath, (en)? "Index" : _("Index"));
+	hitem.callback = (en)? en_text_cmdref : plain_text_cmdref;
 	gtk_item_factory_create_item(hwin->ifac, &hitem, 
 				     GINT_TO_POINTER(0), 
 				     1);
 	g_free(hitem.path);
     }
-#endif
 
     /* put the topics under the menu heading */
     for (i=0; hds[i] != NULL; i++) {
+	const char *headname = (en)? hds[i]->name : _(hds[i]->name);
 
-	if (hds[i]->ntopics == 0) continue;
+	if (hds[i]->ntopics == 0) {
+	    continue;
+	}
 
 	hitem.callback_action = 0; 
 	hitem.item_type = "<Branch>";
-	hitem.path = g_strdup_printf("%s/%s", mpath, _(hds[i]->name));
+	hitem.path = g_strdup_printf("%s/%s", mpath, headname);
 	hitem.callback = NULL; 
 
 	gtk_item_factory_create_item(hwin->ifac, &hitem, NULL, 1);
@@ -877,11 +861,11 @@ static void add_help_topics (windata_t *hwin, int cli, int en)
 	    if (hds[i]->topicnames != NULL) {
 		hitem.path = 
 		    g_strdup_printf("%s/%s/%s", 
-				    mpath, _(hds[i]->name), 
+				    mpath, headname, 
 				    hds[i]->topicnames[j]);
 #if HDEBUG
 		fprintf(stderr, "(1) Built help topic path from\n"
-			" '%s', '%s' and '%s'\n", mpath, _(hds[i]->name),
+			" '%s', '%s' and '%s'\n", mpath, headname),
 			hds[i]->topicnames[j]);
 #endif
 	    } else {
@@ -889,11 +873,11 @@ static void add_help_topics (windata_t *hwin, int cli, int en)
 		    /* a regular gretl command */
 		    hitem.path = 
 			g_strdup_printf("%s/%s/%s", 
-					mpath, _(hds[i]->name), 
+					mpath, headname, 
 					gretl_command_word(tnum));
 #if HDEBUG
 		    fprintf(stderr, "(2) Built help topic path from\n"
-			    " '%s', '%s' and '%s'\n", mpath, _(hds[i]->name),
+			    " '%s', '%s' and '%s'\n", mpath, headname),
 			    gretl_command_word(tnum));
 #endif
 		} else if (!cli) {
@@ -903,10 +887,10 @@ static void add_help_topics (windata_t *hwin, int cli, int en)
 		    if (gstr != NULL) {
 			hitem.path = 
 			    g_strdup_printf("%s/%s/%s", 
-					    mpath, _(hds[i]->name), gstr);
+					    mpath, headname, gstr);
 #if HDEBUG
 			fprintf(stderr, "(3) Built help topic path from\n"
-				" '%s', '%s' and '%s'\n", mpath, _(hds[i]->name),
+				" '%s', '%s' and '%s'\n", mpath, headname,
 				gstr);
 #endif
 		    } else {
@@ -1076,6 +1060,7 @@ void plain_text_cmdref (gpointer p, guint cmdnum, GtkWidget *w)
     if (cmdnum > 0) {
 	pos = cli_pos_from_cmd(cmdnum, en);
 	if (pos < 0 && !en && translated_helpfile == 1) {
+	    /* no translated entry: fall back on English */
 	    en = 1;
 	    pos = cli_pos_from_cmd(cmdnum, en);
 	}
@@ -1084,7 +1069,10 @@ void plain_text_cmdref (gpointer p, guint cmdnum, GtkWidget *w)
     real_do_help(cmdnum, pos, cli, en);
 } 
 
-#ifndef OLD_GTK
+static void en_text_cmdref (gpointer p, guint u, GtkWidget *w)
+{
+    real_do_help(0, 0, 1, 1);
+} 
 
 gint edit_script_help (GtkWidget *widget, GdkEventButton *b,
 		       windata_t *vwin)
@@ -1142,58 +1130,6 @@ gint edit_script_help (GtkWidget *widget, GdkEventButton *b,
     return FALSE;
 }
 
-#else /* now old gtk */
-
-gint edit_script_help (GtkWidget *widget, GdkEventButton *b,
-		       windata_t *vwin)
-{
-    if (!window_help_is_active(vwin)) { 
-	/* command help not activated */
-	return FALSE;
-    } else {
-	gchar *text = NULL;
-	int pos = -1;
-	int hcode = 0;
-	int en = 0;
-	int pt = GTK_EDITABLE(vwin->w)->current_pos;
-	int len = gtk_text_get_length(GTK_TEXT(vwin->w));
-
-	text = gtk_editable_get_chars(GTK_EDITABLE(vwin->w), 
-				      0, (pt + 9 > len)? -1 : pt + 8);
-
-	if (text != NULL && *text != '\0') {
-	    char *p, *q;
-	    char word[9];
-
-	    p = q = text + pt;
-	    if (pt > 0) {
-		while (p - text && !isspace(*(p-1))) p--;
-	    }
-	    if (pt < (int) strlen(text)) {
-		while (*q && !isspace(*q)) q++;
-	    }
-	    *word = '\0';
-	    strncat(word, p, (q - p > 8)? 8 : q - p);
-	    hcode = gretl_command_number(word);
-	    pos = cli_pos_from_cmd(hcode, en);
-	    if (pos < 0 && translated_helpfile) {
-		en = 1;
-		pos = cli_pos_from_cmd(hcode, en);
-	    }
-	} 
-
-	g_free(text);
-	unset_window_help_active(vwin);
-	gdk_window_set_cursor(GTK_TEXT(vwin->w)->text_area, NULL);
-
-	real_do_help(hcode, pos, 1, en);
-    }
-
-    return FALSE;
-}
-
-#endif /* gtk versions */
-
 void menu_find (gpointer data, guint db, GtkWidget *widget)
 {
     if (db) {
@@ -1235,82 +1171,30 @@ static int look_for_string (const char *haystack, const char *needle,
     return -1;
 }
 
-#ifdef OLD_GTK
+/* case-insensitive search in text buffer */
 
-static int is_all_lower (const char *s)
-{
-    int ret = 1;
-
-    while (*s) {
-	if (!islower(*s)) {
-	    ret = 0;
-	    break;
-	}
-	s++;
-    }
-
-    return ret;
-}
-
-static void find_in_text (GtkWidget *widget, gpointer data)
-{
-    int found = 0;
-    char *haystack;
-    windata_t *vwin = 
-	(windata_t *) gtk_object_get_data(GTK_OBJECT(data), "windat");
-
-    haystack = gtk_editable_get_chars(GTK_EDITABLE(vwin->w), 0,
-				      gtk_text_get_length(GTK_TEXT(vwin->w)));
-
-    if (needle) g_free(needle);
-
-    needle = gtk_editable_get_chars(GTK_EDITABLE(find_entry), 0, -1);
-    found = GTK_EDITABLE(vwin->w)->selection_end_pos;
-
-    if (is_all_lower(needle)) {
-	lower(haystack);
-    }
-
-    found = look_for_string(haystack, needle, found);
-
-    if (found >= 0) {
-	gtk_text_set_point(GTK_TEXT(vwin->w), found);
-	gtk_editable_set_position(GTK_EDITABLE(vwin->w), found);
-        gtk_editable_select_region(GTK_EDITABLE(vwin->w), 
-				   found, found + strlen(needle));
-    } else {
-	infobox(_("String was not found."));
-    }
-
-    g_free(haystack);
-}
-
-#else /* !OLD_GTK */
-
-static gboolean real_find_in_text (GtkTextView *view, const gchar* str, 
+static gboolean real_find_in_text (GtkTextView *view, const gchar *s, 
 				   gboolean from_cursor)
 {
     GtkTextBuffer *buf;
-    GtkTextIter iter;
-    gboolean found = FALSE;
-    gboolean wrapped = FALSE;
-    GtkTextSearchFlags search_flags;
+    GtkTextIter iter, start, end;
+    GtkTextMark *vis;
+    int found = 0;
+    int wrapped = 0;
+    int n = strlen(s);
+    gchar *got;
 
-    buf = gtk_text_view_get_buffer (view);
-
-    search_flags = GTK_TEXT_SEARCH_VISIBLE_ONLY | GTK_TEXT_SEARCH_TEXT_ONLY;
+    buf = gtk_text_view_get_buffer(view);
 
  text_search_wrap:
 	
     if (from_cursor) {
 	GtkTextIter sel_bound;
 		
-	gtk_text_buffer_get_iter_at_mark(buf,			
-					 &iter,
+	gtk_text_buffer_get_iter_at_mark(buf, &iter,
 					 gtk_text_buffer_get_mark(buf,
 								  "insert"));
-	gtk_text_buffer_get_iter_at_mark(buf,			
-					 &sel_bound,
+	gtk_text_buffer_get_iter_at_mark(buf, &sel_bound,
 					 gtk_text_buffer_get_mark(buf,
 								  "selection_bound"));
 	gtk_text_iter_order(&sel_bound, &iter);		
@@ -1318,24 +1202,34 @@ static gboolean real_find_in_text (GtkTextView *view, const gchar* str,
 	gtk_text_buffer_get_iter_at_offset(buf, &iter, 0);
     }
 
-    if (*str != '\0') {
-	GtkTextIter match_start, match_end;
-	GtkTextMark *vis;
+    start = end = iter;
 
-	found = gtk_text_iter_forward_search(&iter, str, search_flags,
-					     &match_start, &match_end,
-					     NULL);	
-	if (found) {
-	    gtk_text_buffer_place_cursor(buf, &match_start);
-	    gtk_text_buffer_move_mark_by_name(buf, "selection_bound", &match_end);
-	    vis = gtk_text_buffer_create_mark(buf, "vis", &match_end, FALSE);
-	    gtk_text_view_scroll_to_mark(view, vis, 0.0, TRUE, 0.1, 0.0);
-	} else if (from_cursor && !wrapped) {
-	    /* try wrapping */
-	    from_cursor = FALSE;
-	    wrapped = TRUE;
-	    goto text_search_wrap;
+    if (!gtk_text_iter_forward_chars(&end, n)) {
+	return 0;
+    }
+
+    while (1) {
+	got = gtk_text_buffer_get_text(buf, &start, &end, FALSE);
+	if (g_ascii_strcasecmp(got, s) == 0) {
+	    found = 1;
 	}
+	g_free(got);
+	if (found || !gtk_text_iter_forward_char(&start) ||
+	    !gtk_text_iter_forward_char(&end)) {
+	    break;
+	}
+    }
+
+    if (found) {
+	gtk_text_buffer_place_cursor(buf, &start);
+	gtk_text_buffer_move_mark_by_name(buf, "selection_bound", &end);
+	vis = gtk_text_buffer_create_mark(buf, "vis", &end, FALSE);
+	gtk_text_view_scroll_to_mark(view, vis, 0.0, TRUE, 0.1, 0.0);
+    } else if (from_cursor && !wrapped) {
+	/* try wrapping */
+	from_cursor = FALSE;
+	wrapped = 1;
+	goto text_search_wrap;
     }
 
     if (found && wrapped) {
@@ -1347,11 +1241,13 @@ static gboolean real_find_in_text (GtkTextView *view, const gchar* str,
 
 static void find_in_text (GtkWidget *widget, gpointer data)
 {
+    windata_t *vwin = g_object_get_data(G_OBJECT(data), "windat");
     gboolean found;
-    windata_t *vwin = 
-	(windata_t *) g_object_get_data(G_OBJECT(data), "windat");
 
     needle = gtk_editable_get_chars(GTK_EDITABLE(find_entry), 0, -1);
+    if (*needle == '\0') {
+	return;
+    }
 
     found = real_find_in_text(GTK_TEXT_VIEW(vwin->w), needle, TRUE);
 
@@ -1360,21 +1256,15 @@ static void find_in_text (GtkWidget *widget, gpointer data)
     }
 }
 
-#endif /* new vs old gtk */
-
 static void find_in_listbox (GtkWidget *w, gpointer data)
 {
     int found = 0, wrapped = 0, minvar = 0;
     gchar *tmp; 
     char haystack[MAXLEN];
     windata_t *win;
-#ifndef OLD_GTK
     gchar *pstr;
     GtkTreeModel *model;
     GtkTreeIter iter, iterhere;
-#else
-    int end, i;
-#endif
 
     win = (windata_t *) g_object_get_data(G_OBJECT(data), "windat");
     if (win == mdata) {
@@ -1382,11 +1272,13 @@ static void find_in_listbox (GtkWidget *w, gpointer data)
 	minvar = 1;
     }
 
-    if (needle) g_free(needle);
+    if (needle) {
+	g_free(needle);
+    }
+
     needle = gtk_editable_get_chars(GTK_EDITABLE(find_entry), 0, -1);
     lower(needle);
 
-#ifndef OLD_GTK
     model = gtk_tree_view_get_model(GTK_TREE_VIEW(win->listbox));
 
     /* try searching downward from the current line plus one */
@@ -1447,55 +1339,6 @@ static void find_in_listbox (GtkWidget *w, gpointer data)
     } else {
 	infobox(_("String was not found."));
     }
-
-#else /* GTK version switch */
-
-    end = GTK_CLIST(win->listbox)->rows;
-
- search_wrap: 
-
-    for (i=clist_start_row; i<end; i++) {  
-	/* try looking in column 1 first */
-	gtk_clist_get_text(GTK_CLIST(win->listbox), i, 1, &tmp);
-	strcpy(haystack, tmp);
-	lower(haystack);
-	found = look_for_string(haystack, needle, 0);
-	if (found >= 0) break;
-	if (win == mdata) {
-	    /* try column 2? */
-	    gtk_clist_get_text(GTK_CLIST(win->listbox), i, 2, &tmp);
-	} else {
-	    /* try column 0? */
-	    gtk_clist_get_text(GTK_CLIST(win->listbox), i, 0, &tmp);
-	}
-	strcpy(haystack, tmp);
-	lower(haystack);
-	found = look_for_string(haystack, needle, 0);
-	if (found >= 0) break;
-    }
-
-    if (found < 0 && win->active_var > minvar && !wrapped) {
-	/* try wrapping to start */
-	end = win->active_var;
-	clist_start_row = minvar;
-	wrapped = 1;
-	goto search_wrap;
-    }    
-
-    if (found >= 0) {
-	if (wrapped) infobox(_("Search wrapped"));
-	gtk_clist_freeze(GTK_CLIST(win->listbox));
-	gtk_clist_moveto(GTK_CLIST(win->listbox), i, 0, 0, .1);
-	gtk_clist_unselect_all(GTK_CLIST(win->listbox));
-	gtk_clist_select_row(GTK_CLIST(win->listbox), i, 0);
-	GTK_CLIST(win->listbox)->focus_row = i;
-	gtk_clist_thaw(GTK_CLIST(win->listbox));
-	win->active_var = i;
-	clist_start_row = i + 1;
-    } else {
-	infobox(_("String was not found."));
-    }
-#endif /* OLD_GTK */
 }
 
 static void cancel_find (GtkWidget *widget, gpointer data)
@@ -1508,7 +1351,6 @@ static void cancel_find (GtkWidget *widget, gpointer data)
 
 static void parent_find (GtkWidget *finder, windata_t *caller)
 {
-#ifndef OLD_GTK
     GtkWidget *w = NULL;
 
     if (caller->dialog != NULL) {
@@ -1522,25 +1364,6 @@ static void parent_find (GtkWidget *finder, windata_t *caller)
 				     GTK_WINDOW(w));
 	gtk_window_set_destroy_with_parent(GTK_WINDOW(finder), TRUE);
     }
-#else
-    if (caller != mdata) {
-	GtkWidget *w = NULL;
-
-	if (caller->dialog != NULL) {
-	    w = caller->dialog;
-	} else if (caller->w != NULL) {
-	    w = caller->w;
-	}
-
-	if (w != NULL) {
-	    gtk_window_set_transient_for(GTK_WINDOW(finder),
-					 GTK_WINDOW(w));
-	    gtk_signal_connect(GTK_OBJECT(w), "destroy",
-			       GTK_SIGNAL_FUNC(cancel_find),
-			       finder);
-	}
-    }
-#endif
 }
 
 static void find_string_dialog (void (*findfunc)(), gpointer data)
@@ -1550,10 +1373,6 @@ static void find_string_dialog (void (*findfunc)(), gpointer data)
     GtkWidget *hbox;
     windata_t *mydat = (windata_t *) data;
 
-#ifdef OLD_GTK
-    clist_start_row = 0;
-#endif
-
     if (find_window != NULL) {
 	g_object_set_data(G_OBJECT(find_window), "windat", mydat);
 	parent_find(find_window, mydat);
@@ -1561,7 +1380,7 @@ static void find_string_dialog (void (*findfunc)(), gpointer data)
 	return;
     }
 
-    find_window = gtk_dialog_new();
+    find_window = gretl_dialog_new(_("gretl: find"), NULL, 0);
     g_object_set_data(G_OBJECT(find_window), "windat", mydat);
     parent_find(find_window, mydat);
 
@@ -1569,9 +1388,6 @@ static void find_string_dialog (void (*findfunc)(), gpointer data)
 		     G_CALLBACK(close_find_dialog),
 		     find_window);
 
-    gtk_window_set_title(GTK_WINDOW(find_window), _("gretl: find"));
-    gtk_container_set_border_width(GTK_CONTAINER(find_window), 5);
-
     hbox = gtk_hbox_new(TRUE, TRUE);
     label = gtk_label_new(_(" Find what:"));
     gtk_widget_show (label);
@@ -1595,30 +1411,23 @@ static void find_string_dialog (void (*findfunc)(), gpointer data)
     gtk_box_pack_start(GTK_BOX (GTK_DIALOG(find_window)->vbox), 
 		       hbox, TRUE, TRUE, 0);
 
-    gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(find_window)->action_area), 15);
-    gtk_box_set_homogeneous(GTK_BOX 
-			    (GTK_DIALOG(find_window)->action_area), TRUE);
-    gtk_window_set_position(GTK_WINDOW(find_window), GTK_WIN_POS_MOUSE);
+    hbox = GTK_DIALOG(find_window)->action_area;
+
+    /* cancel button */
+    button = cancel_button(hbox);
+    g_signal_connect(G_OBJECT(button), "clicked",
+		     G_CALLBACK(cancel_find), find_window);
+    gtk_widget_show(button);
 
     /* find button -- make this the default */
     button = standard_button(GTK_STOCK_FIND);
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(find_window)->action_area), 
-		       button, TRUE, TRUE, FALSE);
+    gtk_container_add(GTK_CONTAINER(hbox), button);
     g_signal_connect(G_OBJECT(button), "clicked",
 		     G_CALLBACK(findfunc), find_window);
     gtk_widget_grab_default(button);
     gtk_widget_show(button);
 
-    /* cancel button */
-    button = standard_button(GTK_STOCK_CANCEL);
-    GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(find_window)->action_area), 
-		       button, TRUE, TRUE, FALSE);
-    g_signal_connect(G_OBJECT(button), "clicked",
-		     G_CALLBACK(cancel_find), find_window);
-    gtk_widget_show(button);
-
     gtk_widget_grab_focus(find_entry);
     gtk_widget_show(find_window);
 }
@@ -1628,34 +1437,12 @@ void text_find_callback (GtkWidget *w, gpointer data)
     find_string_dialog(find_in_text, data);
 }
 
-#ifdef OLD_GTK
-
-void colorize_tooltips (GtkTooltips *tip)
-{
-    GdkColor t_back;
-    GtkStyle *style;
-
-    if (gdk_color_parse("light yellow", &t_back)) {
-	gtk_tooltips_force_window(tip);
-	if (gdk_color_alloc(gtk_widget_get_colormap(tip->tip_window), &t_back)) {
-	    style = gtk_style_copy(gtk_widget_get_style(tip->tip_window));
-	    style->bg[GTK_STATE_NORMAL] = t_back;
-	    gtk_widget_set_style(tip->tip_window, style);
-	} 
-    } 
-}
-
-#endif
-
 static GtkTooltips *gretl_tips;
 
 void gretl_tooltips_init (void)
 {
     gretl_tips = gtk_tooltips_new();
     gtk_tooltips_enable(gretl_tips); /* redundant? */
-#ifdef OLD_GTK
-    colorize_tooltips(gretl_tips); 
-#endif
 }
 
 void gretl_tooltips_add (GtkWidget *w, const gchar *str)
@@ -1663,23 +1450,6 @@ void gretl_tooltips_add (GtkWidget *w, const gchar *str)
     gtk_tooltips_set_tip(gretl_tips, w, str, NULL);
 }
 
-#ifdef OSX_PKG
-static void osx_help (int uguide)
-{
-    char *prefix, *syscmd;
-   
-    prefix = getenv("GTK_EXE_PREFIX");
-    if (prefix == NULL) {
-        errbox("Couldn't find the manual");
-	return;
-    }
-    
-    syscmd = g_strdup_printf("%s/bin/manual.sh %s", prefix, (uguide)? "uguide" : "ref");
-    system(syscmd);
-    g_free(syscmd);
-}
-#endif 
-
 enum {
     EN_LETTER,
     EN_A4,
@@ -1687,105 +1457,50 @@ enum {
     SPANISH
 };
 
-#ifdef G_OS_WIN32
-
-static char *full_doc_path (char *path, const char *fname)
+static int get_writable_path (char *path, const char *fname)
 {
-    strcpy(path, paths.gretldir);
-    strcat(path, "doc");
-    strcat(path, SLASHSTR);
-    strcat(path, fname);
+    static int sysdoc_writable = -1;
+    static int userdoc_writable = -1;
 
-    return path;
-}
-
-#else
-
-static int helpfile_exists (char *path, const char *fname, int i)
-{
-    FILE *fp = NULL;
-    int ret = 0;
-
-    if (i == 0) {
-	/* standard location */
-	sprintf(path, "%sdoc/%s", paths.gretldir, fname);
-    } else if (i == 1) {
-	sprintf(path, "%s../doc/%s", paths.gretldir, fname);
-	/* "system" location */
-    } else {
-	/* "user" location */
-	sprintf(path, "%sdoc/%s", paths.userdir, fname);
-    }
-
-    fp = fopen(path, "r");
-
-    if (fp != NULL) {
-	fclose(fp);
-	ret = 1;
-    }
-
-    return ret;
-}
-
-static int helpfile_is_writable (char *path, const char *fname, int i)
-{
     FILE *fp;
     int err = 0;
-    int ret = 0;
 
-    if (i == 0) {
-	/* standard location */
-	sprintf(path, "%sdoc/%s", paths.gretldir, fname);
-    } else if (i == 1) {
-	sprintf(path, "%s../doc/%s", paths.gretldir, fname);
-	/* "system" location */
-    } else {
-	/* "user" location */
-	DIR *dir;
+    if (sysdoc_writable < 0) {
+	sysdoc_writable = 0;
+	sprintf(path, "%sdoc", paths.gretldir);
+	if (gretl_mkdir(path) == 0) {
+	    sprintf(path, "%sdoc%c%s", paths.gretldir, SLASH, fname);
+	    fp = gretl_fopen(path, "w");
+	    if (fp != NULL) {
+		sysdoc_writable = 1;
+		fclose(fp);
+		remove(path);
+	    } 
+	} 
+    }
 
+    if (!sysdoc_writable && userdoc_writable < 0) {
+	userdoc_writable = 0;
 	sprintf(path, "%sdoc", paths.userdir);
-	dir = opendir(path);
-
-	if (dir != NULL) {
-	    closedir(dir);
-	} else if (mkdir(path, 0755)) {
-	    err = 1;
-	}
-	if (!err) {
-	    sprintf(path, "%sdoc/%s", paths.userdir, fname);
-	}
+	if (gretl_mkdir(path) == 0) {
+	    sprintf(path, "%sdoc%c%s", paths.userdir, SLASH, fname);
+	    fp = gretl_fopen(path, "w");
+	    if (fp != NULL) {
+		userdoc_writable = 1;
+		fclose(fp);
+		remove(path);
+	    } 
+	} 
     }
 
-    if (!err) {
-	fp = fopen(path, "w");
-	if (fp != NULL) {
-	    fclose(fp);
-	    remove(path);
-	    ret = 1;
-	}
+    if (!sysdoc_writable && !userdoc_writable) {
+	err = 1;
     }
 
-    return ret;
+    return err;
 }
 
-static char *full_doc_path (char *path, const char *fname)
-{
-    int i;
-
-    for (i=0; i<3; i++) {
-	if (helpfile_exists(path, fname, i)) {
-	    return path;
-	} else if (helpfile_is_writable(path, fname, i)) {
-	    return path;
-	}
-    }
-
-    return NULL;
-}
-
-#endif
-
-static int maybe_grab_pdf (int uguide, int i, char *fullpath)
+static int find_or_download_pdf (int uguide, int i, char *fullpath)
 {
     const char *guide_files[] = {
 	"gretl-guide.pdf",
@@ -1804,35 +1519,43 @@ static int maybe_grab_pdf (int uguide, int i, char *fullpath)
     FILE *fp;
     int err = 0;
 
+    *errtext = '\0';
+
     if (i < 0 || i > 3) {
 	i = 0;
     }
 
-    if (uguide) {
-	fname = guide_files[i];
-    } else {
-	fname = ref_files[i];
-    }
+    fname = (uguide)? guide_files[i] : ref_files[i];
 
-    if (full_doc_path(fullpath, fname) == NULL) {
-	errbox("Failed to download file");
-	return 1;
-    }
-
-    /* see if file exists locally */
-    fp = fopen(fullpath, "r");
+    /* is the file available in public dir? */
+    sprintf(fullpath, "%sdoc%c%s", paths.gretldir, SLASH, fname);
+    fp = gretl_fopen(fullpath, "r");
     if (fp != NULL) {
 	fclose(fp);
 	return 0;
     }
 
-    /* if not, grab from server */
-    err = retrieve_manfile(fname, fullpath, errtext);
+    /* or maybe in user dir? */
+    sprintf(fullpath, "%sdoc%c%s", paths.userdir, SLASH, fname);
+    fp = gretl_fopen(fullpath, "r");
+    if (fp != NULL) {
+	fclose(fp);
+	return 0;
+    }
+
+    /* check for download location */
+    err = get_writable_path(fullpath, fname);
+
+    /* do actual download */
+    if (!err) {
+	err = retrieve_manfile(fname, fullpath, errtext);
+    }
+
     if (err) {
 	if (*errtext) {
 	    errbox(errtext);
 	} else {
-	    errbox("Failed to download file");
+	    errbox(_("Failed to download file"));
 	}
     }
 
@@ -1841,27 +1564,23 @@ static int maybe_grab_pdf (int uguide, int i, char *fullpath)
 
 void display_pdf_help (gpointer p, guint uguide, GtkWidget *w)
 {
-    char fullpath[FILENAME_MAX];
+    char fname[FILENAME_MAX];
     int pref, err = 0;
 
-#ifdef OSX_PKG
-    osx_help(uguide);
-    return;
-#endif  
-
     pref = get_manpref();
-
-    err = maybe_grab_pdf(uguide, pref, fullpath);
+    err = find_or_download_pdf(uguide, pref, fname);
     if (err) {
 	return;
     }
 
-#ifdef G_OS_WIN32
-    if ((int) ShellExecute(NULL, "open", fullpath, NULL, NULL, SW_SHOW) <= 32) {
+#if defined(G_OS_WIN32)
+    if ((int) ShellExecute(NULL, "open", fname, NULL, NULL, SW_SHOW) <= 32) {
 	DWORD dw = GetLastError();
 	win_show_error(dw);
     }
+#elif defined(OSX_BUILD)
+    osx_open_file(fname);
 #else
-    gretl_fork(viewpdf, fullpath);
+    gretl_fork(viewpdf, fname);
 #endif
 }
