@@ -37,6 +37,12 @@ int winfork (char *cmdline, const char *dir, int wshow,
 	     DWORD flags);
 #endif
 
+#ifdef OSX_BUILD
+int osx_open_file (const char *path);
+
+int osx_open_url (const char *url);
+#endif
+
 typedef struct _selector selector;
 
 void library_command_init (void);
@@ -69,12 +75,10 @@ void gui_errmsg (const int errcode);
 
 void register_graph (void);
 
-void clear_data (void);
-
-void exit_free_modelspec (void);
-
 /* sample-related functions */
 
+void set_sample_label_special (void);
+
 int bool_subsample (gretlopt opt);
 
 void do_samplebool (GtkWidget *widget, dialog_t *dlg);
@@ -135,6 +139,8 @@ void do_mle_model (GtkWidget *widget, dialog_t *dlg);
 
 void do_eqn_system (GtkWidget *widget, dialog_t *dlg);
 
+void do_saved_eqn_system (GtkWidget *widget, dialog_t *dlg);
+
 int do_model (selector *sr);
 
 int do_vector_model (selector *sr);
@@ -157,25 +163,23 @@ void do_panel_diagnostics (gpointer data, guint u, GtkWidget *w);
 
 int do_spearman (selector *sr);
 
-#ifdef ENABLE_GMP
-int do_mp_ols (selector *sr);
-#endif
-
 int out_of_sample_info (int add_ok, int *t2);
 
 /* variable-related functions */
 
-void do_simdata (GtkWidget *widget, dialog_t *dlg);
-
 void do_minibuf (GtkWidget *widget, dialog_t *dlg);
 
 void do_genr (GtkWidget *widget, dialog_t *dlg);
 
 void do_model_genr (GtkWidget *widget, dialog_t *dlg);
 
-void do_random (GtkWidget *widget, dialog_t *dlg);
+void do_random_uniform (GtkWidget *widget, dialog_t *dlg);
 
-void do_seed (GtkWidget *widget, dialog_t *dlg);
+void do_random_normal (GtkWidget *widget, dialog_t *dlg);
+
+void do_random_chisq (GtkWidget *widget, dialog_t *dlg);
+
+void do_random_st (GtkWidget *widget, dialog_t *dlg);
 
 void do_global_setmiss (GtkWidget *widget, dialog_t *dlg);
 
@@ -233,8 +237,6 @@ void fit_actual_plot (gpointer data, guint xvar, GtkWidget *widget);
 
 void fit_actual_splot (gpointer data, guint u, GtkWidget *widget);
 
-void display_data (gpointer data, guint select, GtkWidget *widget);
-
 void display_fit_resid (gpointer data, guint code, GtkWidget *widget);
 
 void do_graph_var (int varnum);
@@ -255,7 +257,9 @@ void do_box_graph (GtkWidget *widget, dialog_t *dlg);
 
 int do_dummy_graph (selector *sr);
 
-void delete_selected_vars (int id);
+void delete_selected_vars (void);
+
+void delete_single_var (int id);
 
 void display_selected (gpointer data, guint action, GtkWidget *widget);
 
@@ -271,7 +275,7 @@ void do_new_script (gpointer data, guint action, GtkWidget *widget);
 
 void do_open_csv_box (char *fname, int code, int append);
 
-int do_store (char *mydatfile, gretlopt oflag, int overwrite);
+int do_store (char *mydatfile, gretlopt oflag);
 
 void view_latex (PRN *prn);
 
