@@ -22,6 +22,7 @@
 #include "gretl.h"
 #include "console.h"
 #include "guiprint.h"
+#include "ssheet.h"
 
 void refresh_data (void)
 {
@@ -30,6 +31,19 @@ void refresh_data (void)
     }
 }
 
+void flip (GtkItemFactory *ifac, const char *path, gboolean s)
+{
+    if (ifac != NULL) {
+	GtkWidget *w = gtk_item_factory_get_item(ifac, path);
+
+	if (w != NULL) {
+	    gtk_widget_set_sensitive(w, s);
+	} else {
+	    fprintf(stderr, I_("Failed to flip state of \"%s\"\n"), path);
+	}
+    }
+}
+
 void edit_info_state (gboolean s)
 {
     flip(mdata->ifac, "/Data/Edit info", s);
@@ -37,8 +51,8 @@ void edit_info_state (gboolean s)
 
 void add_remove_markers_state (gboolean s)
 {
-    flip(mdata->ifac, "/Sample/Add case markers...", !s);
-    flip(mdata->ifac, "/Sample/Remove case markers", s);
+    flip(mdata->ifac, "/Data/Add case markers...", !s);
+    flip(mdata->ifac, "/Data/Remove case markers", s);
 }
 
 /* by using gretl_set_window_modal() we make the main
@@ -75,9 +89,19 @@ void gretl_set_window_modal (GtkWidget *w)
 		     NULL);
 }
 
+void variable_menu_state (gboolean s)
+{
+    if (mdata == NULL || mdata->ifac == NULL) return;
+
+    flip(mdata->ifac, "/Variable", s);
+    flip(mdata->ifac, "/View/Correlation matrix", !s);
+    flip(mdata->ifac, "/View/Principal components", !s);
+    flip(mdata->ifac, "/View/Mahalanobis distances", !s);
+}
+
 void main_menubar_state (gboolean s)
 {
-    if (mdata->ifac == NULL) return;
+    if (mdata == NULL || mdata->ifac == NULL) return;
 
     flip(mdata->ifac, "/File/Append data", s);
     flip(mdata->ifac, "/File/Clear data set", s);
@@ -85,11 +109,15 @@ void main_menubar_state (gboolean s)
     flip(mdata->ifac, "/File/Save data as", s);
     flip(mdata->ifac, "/File/Export data", s);
     flip(mdata->ifac, "/File/Send To...", s);
-    flip(mdata->ifac, "/File/Create data set", !s);
     flip(mdata->ifac, "/Data", s);
+    flip(mdata->ifac, "/View", s);
+    flip(mdata->ifac, "/Add", s);
     flip(mdata->ifac, "/Sample", s);
     flip(mdata->ifac, "/Variable", s);
     flip(mdata->ifac, "/Model", s);
+    flip(mdata->ifac, "/Tools/Sort variables", s);
+
+    flip(mdata->ifac, "/File/New data set", !s);
 
     if (s) {
 	edit_info_state(!(data_status & BOOK_DATA));
@@ -97,8 +125,6 @@ void main_menubar_state (gboolean s)
     }
 }
 
-#ifndef OLD_GTK
-
 static GtkItemFactoryEntry time_series_model_items[] = {
     { N_("/Model/Time series/_Cochrane-Orcutt..."), NULL, model_callback, CORC, NULL, GNULL },
     { N_("/Model/Time series/_Hildreth-Lu..."), NULL, model_callback, HILU, NULL, GNULL },
@@ -107,36 +133,20 @@ static GtkItemFactoryEntry time_series_model_items[] = {
     { N_("/Model/Time series/ARI_MA..."), NULL, model_callback, ARMA, NULL, GNULL },
     { N_("/Model/Time series/_GARCH..."), NULL, model_callback, GARCH, NULL, GNULL },
     { N_("/Model/Time series/_Vector Autoregression..."), NULL, selector_callback, VAR, NULL, GNULL },
-    { N_("/Model/Time series/VAR lag selection..."), NULL, selector_callback, VLAGSEL, NULL, GNULL },
-    { N_("/Model/Time series/VECM..."), NULL, selector_callback, VECM, NULL, GNULL },
-    { N_("/Model/Time series/Cointegration test"), NULL, NULL, 0, "<Branch>", GNULL },
-    { N_("/Model/Time series/Cointegration test/Engle-Granger..."), NULL, 
+    { N_("/Model/Time series/VAR _lag selection..."), NULL, selector_callback, VLAGSEL, NULL, GNULL },
+    { N_("/Model/Time series/V_ECM..."), NULL, selector_callback, VECM, NULL, GNULL },
+    { N_("/Model/Time series/_Cointegration test"), NULL, NULL, 0, "<Branch>", GNULL },
+    { N_("/Model/Time series/Cointegration test/_Engle-Granger..."), NULL, 
       selector_callback, COINT, NULL, GNULL },
-    { N_("/Model/Time series/Cointegration test/Johansen..."), NULL, 
+    { N_("/Model/Time series/Cointegration test/_Johansen..."), NULL, 
       selector_callback, COINT2, NULL, GNULL }
 };
 
-#else
-
-static GtkItemFactoryEntry time_series_model_items[] = {
-    { N_("/Model/Time series/_Cochrane-Orcutt..."), NULL, model_callback, CORC, NULL },
-    { N_("/Model/Time series/_Hildreth-Lu..."), NULL, model_callback, HILU, NULL },
-    { N_("/Model/Time series/_Prais-Winsten..."), NULL, model_callback, PWE, NULL },
-    { N_("/Model/Time series/_Autoregressive estimation..."), NULL, model_callback, AR, NULL },
-    { N_("/Model/Time series/ARI_MA..."), NULL, model_callback, ARMA, NULL },
-    { N_("/Model/Time series/_GARCH..."), NULL, model_callback, GARCH, NULL },
-    { N_("/Model/Time series/_Vector Autoregression..."), NULL, selector_callback, VAR, NULL },
-    { N_("/Model/Time series/VAR lag selection..."), NULL, selector_callback, VLAGSEL, NULL },
-    { N_("/Model/Time series/VECM..."), NULL, selector_callback, VECM, NULL },
-    { N_("/Model/Time series/Cointegration test"), NULL, NULL, 0, "<Branch>" },
-    { N_("/Model/Time series/Cointegration test/Engle-Granger..."), NULL, 
-      selector_callback, COINT, NULL },
-    { N_("/Model/Time series/Cointegration test/Johansen..."), NULL, 
-      selector_callback, COINT2, NULL }
+static GtkItemFactoryEntry panel_model_items[] = {
+    { N_("/Model/Panel/_Fixed or random effects..."), NULL, model_callback, PANEL, NULL, GNULL },
+    { N_("/Model/Panel/_Weighted least squares..."), NULL, model_callback, PANEL_WLS, NULL, GNULL }
 };
 
-#endif
-
 #define COMPACTABLE(d) (d->structure == TIME_SERIES && \
                         (d->pd == 4 || d->pd == 12 || \
                          d->pd == 5 || d->pd == 7))
@@ -150,19 +160,23 @@ static GtkItemFactoryEntry time_series_model_items[] = {
 			(d)->structure == SPECIAL_TIME_SERIES || \
                         (d)->structure == STACKED_TIME_SERIES)
 
+#define seasonal_ts(d) ((d)->structure == TIME_SERIES && (d->pd == 4 || d->pd == 12))
+
 void time_series_menu_state (gboolean s)
 {
     gboolean sx = extended_ts(datainfo);
+    gboolean ss = seasonal_ts(datainfo);
 
     if (mdata->ifac == NULL) {
 	return;
     }
 
     /* File menu */
-    flip(mdata->ifac, "/File/Save data as/database...", DATASET_DB_OK(datainfo));
+    flip(mdata->ifac, "/File/Save data as/Database...", DATASET_DB_OK(datainfo));
 
     /* Plots */
-    flip(mdata->ifac, "/Data/Graph specified vars/Time series plot...", sx);
+    flip(mdata->ifac, "/View/Graph specified vars/Time series plot...", sx);
+    flip(mdata->ifac, "/View/Multiple graphs/Time series...", sx);
     flip(mdata->ifac, "/Variable/Time series plot", sx);
 
     /* Variable menu */
@@ -171,19 +185,20 @@ void time_series_menu_state (gboolean s)
     flip(mdata->ifac, "/Variable/Runs test", s);
     flip(mdata->ifac, "/Variable/Augmented Dickey-Fuller test", s);
     flip(mdata->ifac, "/Variable/KPSS test", s);
+    flip(mdata->ifac, "/Variable/Filter", s);
 #ifdef HAVE_X12A
-    flip(mdata->ifac, "/Variable/X-12-ARIMA analysis", s);
+    flip(mdata->ifac, "/Variable/X-12-ARIMA analysis", ss);
 #endif
 #ifdef HAVE_TRAMO
-    flip(mdata->ifac, "/Variable/TRAMO analysis", s);
+    flip(mdata->ifac, "/Variable/TRAMO analysis", ss);
 #endif
     flip(mdata->ifac, "/Variable/Hurst exponent", s);
     /* Model menu */
     flip(mdata->ifac, "/Model/Time series", s);
     /* Sample menu */
-    flip(mdata->ifac, "/Sample/Compact data...", 
+    flip(mdata->ifac, "/Data/Compact data...", 
 	 s && (COMPACTABLE(datainfo) || dated_weekly_data(datainfo)));
-    flip(mdata->ifac, "/Sample/Expand data...", s && EXPANSIBLE(datainfo));
+    flip(mdata->ifac, "/Data/Expand data...", s && EXPANSIBLE(datainfo));
 
     if (s) {
 	GtkWidget *w =  
@@ -206,9 +221,26 @@ void time_series_menu_state (gboolean s)
 void panel_menu_state (gboolean s)
 {
     if (mdata->ifac != NULL) {
-	flip(mdata->ifac, "/Model/Pooled OLS (panel)...", s);
-	flip(mdata->ifac, "/Data/Add variables/unit dummies", s);
-	flip(mdata->ifac, "/Data/Add variables/panel dummies", s);
+	flip(mdata->ifac, "/Add/Unit dummies", s);
+	flip(mdata->ifac, "/Add/Time dummies", s);
+	flip(mdata->ifac, "/Model/Panel", s);
+    }
+
+    if (s) {
+	GtkWidget *w =  
+	    gtk_item_factory_get_widget(mdata->ifac, 
+					"/Model/Panel/Fixed or random effects...");
+
+	if (w == NULL) {
+	    int i, n = sizeof panel_model_items / 
+		sizeof panel_model_items[0];
+
+	    for (i=0; i<n; i++) {
+		gtk_item_factory_create_item(mdata->ifac, 
+					     &panel_model_items[i], 
+					     mdata, 1);
+	    }
+	}
     }
 }
 
@@ -216,23 +248,23 @@ void ts_or_panel_menu_state (gboolean s)
 {
     if (mdata->ifac == NULL) return;
 
-    flip(mdata->ifac, "/Data/Add variables/time trend", s);
-    flip(mdata->ifac, "/Data/Add variables/lags of selected variables", s);
-    flip(mdata->ifac, "/Data/Add variables/first differences of selected variables", s);
-    flip(mdata->ifac, "/Data/Add variables/log differences of selected variables", s);
+    flip(mdata->ifac, "/Add/Time trend", s);
+    flip(mdata->ifac, "/Add/Lags of selected variables", s);
+    flip(mdata->ifac, "/Add/First differences of selected variables", s);
+    flip(mdata->ifac, "/Add/Log differences of selected variables", s);
 
-    flip(mdata->ifac, "/Data/Add variables/seasonal differences of selected variables",
+    flip(mdata->ifac, "/Add/Seasonal differences of selected variables",
 	 dataset_is_seasonal(datainfo));
-    flip(mdata->ifac, "/Data/Add variables/periodic dummies", 
+    flip(mdata->ifac, "/Add/Periodic dummies", 
 	 dataset_is_seasonal(datainfo));
 }
 
 void session_menu_state (gboolean s)
 {
     if (mdata->ifac != NULL) {
-	flip(mdata->ifac, "/Session/Icon view", s);
-	flip(mdata->ifac, "/Session/Save", s);
-	flip(mdata->ifac, "/Session/Save as...", s);
+	flip(mdata->ifac, "/View/Icon view", s);
+	flip(mdata->ifac, "/File/Session files/Save session", s);
+	flip(mdata->ifac, "/File/Session files/Save session as...", s);
     }	
 }
 
@@ -253,7 +285,7 @@ void drop_obs_state (gboolean s)
 void compact_data_state (gboolean s)
 {
     if (mdata->ifac != NULL) {
-	flip(mdata->ifac, "/Sample/Compact data...", s);
+	flip(mdata->ifac, "/Data/Compact data...", s);
     }
 }
 
@@ -261,40 +293,47 @@ void main_menus_enable (gboolean s)
 {
     if (mdata->ifac != NULL) {
 	flip(mdata->ifac, "/File", s);
-	flip(mdata->ifac, "/Utilities", s);
-	flip(mdata->ifac, "/Session", s);
+	flip(mdata->ifac, "/Tools", s);
 	flip(mdata->ifac, "/Data", s);
+	flip(mdata->ifac, "/View", s);
+	flip(mdata->ifac, "/Add", s);
 	flip(mdata->ifac, "/Sample", s);
 	flip(mdata->ifac, "/Model", s);
     }
 }
 
-static gint var_popup_click (GtkWidget *widget, gpointer data)
+static gint var_popup_click (GtkWidget *w, gpointer p)
 {
-    gchar *item = (gchar *) data;
+    gchar *item = (gchar *) p;
+    int v = mdata_active_var();
 
     if (!strcmp(item, _("Display values"))) 
 	display_var();
     if (!strcmp(item, _("Descriptive statistics"))) 
 	do_menu_op(NULL, VAR_SUMMARY, NULL);
     else if (!strcmp(item, _("Time series plot"))) 
-	do_graph_var(mdata->active_var);
+	do_graph_var(v);
     else if (!strcmp(item, _("Frequency distribution"))) 
 	do_menu_op(NULL, FREQ, NULL);
     else if (!strcmp(item, _("Frequency plot"))) 
 	do_freqplot(NULL, 0, NULL);
     else if (!strcmp(item, _("Boxplot")))
-	do_boxplot_var(mdata->active_var);
+	do_boxplot_var(v);
     else if (!strcmp(item, _("Gini coefficient")))
 	do_gini(NULL, 0, NULL);
     else if (!strcmp(item, _("Correlogram")))
 	do_corrgm(NULL, CORRGM, NULL);
+#if 0
     else if (!strcmp(item, _("Spectrum"))) 
 	do_pergm(NULL, 0, NULL);
     else if (!strcmp(item, _("Spectrum (Bartlett)"))) 
 	do_pergm(NULL, 1, NULL);
+#else
+    else if (!strcmp(item, _("Spectrum"))) 
+	do_pergm(NULL, 1, NULL);
+#endif
     else if (!strcmp(item, _("ARIMA model"))) 
-	model_callback(GINT_TO_POINTER(mdata->active_var), ARMA, NULL);
+	model_callback(GINT_TO_POINTER(v), ARMA, NULL);
     else if (!strcmp(item, _("Dickey-Fuller test"))) 
 	unit_root_test(NULL, ADF, NULL);
     else if (!strcmp(item, _("KPSS test"))) 
@@ -304,11 +343,13 @@ static gint var_popup_click (GtkWidget *widget, gpointer data)
     else if (!strcmp(item, _("Hurst exponent"))) 
 	do_hurst(NULL, 0, NULL);
     else if (!strcmp(item, _("Edit attributes")))  
-	varinfo_dialog(mdata->active_var, 1);
+	varinfo_dialog(v, 1);
+    else if (!strcmp(item, _("Edit values")))  
+	show_spreadsheet(SHEET_EDIT_VARLIST);
     else if (!strcmp(item, _("Copy to clipboard"))) 
 	csv_selected_to_clipboard();
     else if (!strcmp(item, _("Delete"))) 
-	delete_selected_vars(mdata->active_var);
+	delete_single_var(v);
     else if (!strcmp(item, _("Define new variable..."))) 
 	gretl_callback(NULL, GENR, NULL);
 
@@ -323,19 +364,12 @@ GtkWidget *build_var_popup (void)
 	N_("Display values"),
 	N_("Descriptive statistics"),
 	N_("Time series plot"),
-	N_("Frequency distribution"),
 	N_("Frequency plot"),
 	N_("Boxplot"),
-	N_("Gini coefficient"),
 	N_("Correlogram"),
 	N_("Spectrum"),
-	N_("Spectrum (Bartlett)"),
-	N_("ARIMA model"),
-	N_("Dickey-Fuller test"),
-	N_("KPSS test"),
-	N_("Runs test"),
-	N_("Hurst exponent"),
 	N_("Edit attributes"),
+	N_("Edit values"),
 	N_("Copy to clipboard"),
 	N_("Delete"),
 	N_("Define new variable...")
@@ -348,7 +382,7 @@ GtkWidget *build_var_popup (void)
     var_menu = gtk_menu_new();
 
     for (i=0; i<n_items; i++) {
-	if (!dataset_is_time_series(datainfo) && (i >= 7 && i <= 12)) {
+	if ((i == 5 || i == 6) && !dataset_is_time_series(datainfo)) {
 	    continue;
 	}
 	if (i == 2 && !extended_ts(datainfo)) {
@@ -359,39 +393,32 @@ GtkWidget *build_var_popup (void)
 			 G_CALLBACK(var_popup_click),
 			 _(var_items[i]));
 	gtk_widget_show(var_item);
-#ifndef OLD_GTK
 	gtk_menu_shell_append(GTK_MENU_SHELL(var_menu), var_item);
-#else
-	GTK_WIDGET_SET_FLAGS (var_item, GTK_SENSITIVE | GTK_CAN_FOCUS); /* ?? */
-	gtk_menu_append(GTK_MENU(var_menu), var_item);
-#endif
     }
 
     return var_menu;
 }
 
-static gint selection_popup_click (GtkWidget *widget, gpointer data)
+static gint selection_popup_click (GtkWidget *w, gpointer p)
 {
-    gchar *item = (gchar *) data;
+    gchar *item = (gchar *) p;
 
     if (!strcmp(item, _("Display values"))) 
 	display_selected(NULL, 0, NULL); 
     else if (!strcmp(item, _("Descriptive statistics"))) 
-	do_menu_op(NULL, SUMMARY_SELECTED, NULL);
+	do_menu_op(NULL, SUMMARY, NULL);
     else if (!strcmp(item, _("Correlation matrix"))) 
-	do_menu_op(NULL, CORR_SELECTED, NULL);
-    else if (!strcmp(item, _("Principal components"))) 
-	do_menu_op(NULL, PCA, NULL);
-    else if (!strcmp(item, _("Mahalanobis distances"))) 
-	do_menu_op(NULL, MAHAL, NULL);
+	do_menu_op(NULL, CORR, NULL);
     else if (!strcmp(item, _("Time series plot"))) 
 	plot_from_selection(NULL, GR_PLOT, NULL);
     else if (!strcmp(item, _("XY scatterplot"))) 
 	plot_from_selection(NULL, GR_XY, NULL);
     else if (!strcmp(item, _("Copy to clipboard"))) 
 	csv_selected_to_clipboard();
+    else if (!strcmp(item, _("Edit"))) 
+	show_spreadsheet(SHEET_EDIT_VARLIST);
     else if (!strcmp(item, _("Delete"))) 
-	delete_selected_vars(0);
+	delete_selected_vars();
 
     gtk_widget_destroy(mdata->popup);
 
@@ -406,10 +433,9 @@ GtkWidget *build_selection_popup (void)
 	N_("Correlation matrix"),
 	N_("Time series plot"),
 	N_("XY scatterplot"),
-	N_("Principal components"),
-	N_("Mahalanobis distances"),
 	N_("Copy to clipboard"),
-	N_("Delete"),
+	N_("Edit"),
+	N_("Delete")
     };
 
     GtkWidget *sel_menu;
@@ -427,11 +453,7 @@ GtkWidget *build_selection_popup (void)
 			 G_CALLBACK(selection_popup_click),
 			 _(items[i]));
 	gtk_widget_show(item);
-#ifndef OLD_GTK
 	gtk_menu_shell_append(GTK_MENU_SHELL(sel_menu), item);
-#else 
-	gtk_menu_append(GTK_MENU(sel_menu), item);
-#endif
     }
 
     return sel_menu;
@@ -489,10 +511,7 @@ void set_sample_label (DATAINFO *pdinfo)
     ts_or_panel_menu_state(dataset_is_time_series(pdinfo) ||
 			   dataset_is_panel(pdinfo));
 
-    flip(mdata->ifac, "/Sample/Restructure panel...", 
-	 dataset_is_panel(pdinfo));
-
-    flip(mdata->ifac, "/Sample/Transpose data...", 
+    flip(mdata->ifac, "/Data/Transpose data...", 
 	 !dataset_is_panel(pdinfo));
 
     sprintf(labeltxt, _("%s: Full range %s - %s"), 
@@ -519,9 +538,7 @@ void set_sample_label (DATAINFO *pdinfo)
 	}
 	if (data_status & MODIFIED_DATA) { 
 	    strcat(labeltxt, "* ");
-	} else if (data_status & GZIPPED_DATA) {
-	    strcat(labeltxt, "[gz] ");
-	}
+	} 
 	if (dlabel != NULL) {
 	    gtk_label_set_text(GTK_LABEL(dlabel), labeltxt);
 	}
