@@ -65,7 +65,7 @@ static int model_table_too_many (int gui)
     return 0;
 }
 
-static int model_already_in_table (const MODEL *pmod)
+int in_model_table (const MODEL *pmod)
 {
     int i;
 
@@ -78,6 +78,20 @@ static int model_already_in_table (const MODEL *pmod)
     return 0;
 }
 
+int model_table_n_models (void)
+{
+    return n_models;
+}
+
+MODEL *model_table_model_by_index (int i)
+{
+    if (i >= 0 && i < n_models) {
+	return table_models[i];
+    } else {
+	return NULL;
+    }
+}
+
 void clear_model_table (PRN *prn)
 {
     int i;
@@ -163,7 +177,7 @@ int add_to_model_table (MODEL *pmod, int add_mode, PRN *prn)
 	}
 
 	/* check that model is not already on the list */
-	if (model_already_in_table(pmod)) {
+	if (in_model_table(pmod)) {
 	    mtable_errmsg(_("Model is already included in the table"), 0);
 	    return 1;
 	}
@@ -353,7 +367,6 @@ static const char *short_estimator_string (int ci, PRN *prn)
     else if (ci == HILU) return N_("HILU");
     else if (ci == PWE) return N_("PWE");
     else if (ci == ARCH) return N_("ARCH");
-    else if (ci == POOLED) return N_("OLS");
     else return estimator_string(ci, prn);
 }
 
@@ -657,7 +670,7 @@ static void print_n_r_squared (int wid, PRN *prn, int *binary)
 	/* print log-likelihoods */
 
 	if (tex) {
-	    pputs(prn, "ln$L$");
+	    pputs(prn, "$\\ell$");
 	} else if (rtf) {
 	    pputs(prn, "\\qc lnL\\cell ");
 	} else {
@@ -677,7 +690,11 @@ static void print_n_r_squared (int wid, PRN *prn, int *binary)
 		}		
 	    } else {
 		if (tex) {
-		    pprintf(prn, "& $-$%.2f ", -pmod->lnL);
+		    if (pmod->lnL > 0) {
+			pprintf(prn, "& %.2f ", pmod->lnL);
+		    } else {
+			pprintf(prn, "& $-$%.2f ", -pmod->lnL);
+		    }
 		} else if (rtf) {
 		    pprintf(prn, "\\qc %.3f\\cell ", pmod->lnL);
 		} else {
@@ -790,7 +807,7 @@ int display_model_table (int gui)
    
     if (binary) {
 	pprintf(prn, "%s\n", _("For logit and probit, R-squared is "
-				 "McFadden's pseudo-R-squared"));
+			       "McFadden's pseudo-R-squared"));
     }
 
     if (real_table_n_models() > 5) {
