@@ -148,46 +148,27 @@ int maybe_save_system (const CMD *cmd, gretl_equation_system *sys, PRN *prn)
     return err;
 }
 
-int maybe_save_graph (const CMD *cmd, const char *fname, int code, PRN *prn)
+int maybe_save_graph (const CMD *cmd, const char *fname, GretlObjType type, 
+		      PRN *prn)
 {
     char gname[MAXSAVENAME];
-    char savedir[MAXLEN];
-    gchar *tmp, *plotfile;
-    int err = 0;
+    int ret, err = 0;
 
     gretl_cmd_get_savename(gname);
     if (*gname == 0) {
 	return 0;
     }
 
-    get_default_dir(savedir, SAVE_THIS_GRAPH);
+    ret = cli_add_graph_to_session(fname, gname, type);
 
-    tmp = g_strdup(gname);
-    plotfile = g_strdup_printf("%ssession.%s", savedir, 
-			       space_to_score(tmp));
-    g_free(tmp);
-
-    if (code == GRETL_GNUPLOT_GRAPH) {
-	err = copyfile(fname, plotfile);
-	if (!err) {
-	    int ret;
-
-	    ret = real_add_graph_to_session(plotfile, gname, code);
-	    if (ret == ADD_OBJECT_FAIL) {
-		err = 1;
-	    } else {
-		remove(fname);
-		if (ret == ADD_OBJECT_REPLACE) {
-		    pprintf(prn, _("%s replaced\n"), gname);
-		} else {
-		    pprintf(prn, _("%s saved\n"), gname);
-		}
-	    }
-	}
+    if (ret == ADD_OBJECT_FAIL) {
+	err = 1;
+    } else if (ret == ADD_OBJECT_REPLACE) {
+	pprintf(prn, _("%s replaced\n"), gname);
+    } else {
+	pprintf(prn, _("%s saved\n"), gname);
     }
 
-    g_free(plotfile);
-
     return err;
 }
 
@@ -316,6 +297,16 @@ int saved_object_action (const char *line, PRN *prn)
 	return 0;
     }
 
+    /* special: display icon view window */
+    if (!strncmp(line, "iconview", 8)) {
+	if (data_status) {
+	    view_session();
+	    return 1;
+	} else {
+	    return -1;
+	}
+    }
+
     action = gui_parse_object_request(line, objname, &param, &ptr, &type, prn);
 
     if (action == OBJ_ACTION_NONE) {
@@ -339,9 +330,7 @@ int saved_object_action (const char *line, PRN *prn)
 	} else if (type == GRETL_OBJ_TEXT) {
 	    display_saved_text(ptr);
 	} else if (type == GRETL_OBJ_GRAPH) {
-	    GRAPHT *graph = (GRAPHT *) ptr;
-
-	    display_session_graph_png(graph->fname);
+	    display_session_graph_by_data(ptr);
 	}
     } else if (action == OBJ_ACTION_FREE) {
 	if (type == GRETL_OBJ_EQN || 
