@@ -26,7 +26,8 @@ int maybe_save_var (const CMD *cmd, GRETL_VAR **pvar, PRN *prn);
 
 int maybe_save_system (const CMD *cmd, gretl_equation_system *sys, PRN *prn);
 
-int maybe_save_graph (const CMD *cmd, const char *fname, int code, PRN *prn);
+int maybe_save_graph (const CMD *cmd, const char *fname, GretlObjType type, 
+		      PRN *prn);
 
 int save_text_buffer (PRN *prn, const char *savename, PRN *errprn);
 
