@@ -20,6 +20,14 @@
 #ifndef SELECTOR_H
 #define SELECTOR_H
 
+typedef struct iterinfo_t iterinfo;
+
+struct iterinfo_t {
+    int ci;
+    int maxiters;
+    double tol;
+};
+
 void clear_selector (void);
 
 void selection_dialog (const char *title, int (*callback)(), guint cmdcode,
@@ -42,6 +50,8 @@ gpointer selector_get_data (const selector *sr);
 
 gretlopt selector_get_opts (const selector *sr);
 
+const char *selector_entry_text (const selector *sr);
+
 int selector_error (const selector *sr);
 
 void maybe_clear_selector (const int *dlist);
