@@ -23,6 +23,7 @@
 #include "textutil.h"
 #include "dlgutils.h"
 #include "menustate.h"
+#include "textbuf.h"
 #include "series_view.h"
 
 #ifdef G_OS_WIN32
@@ -91,7 +92,7 @@ static int series_view_allocate (series_view *sview)
     if (sview->npoints != 0) {
 	/* already allocated */
 	return 0;
-    } else if (!datainfo->vector[sview->varnum]) {
+    } else if (var_is_scalar(datainfo, sview->varnum)) {
 	sview->npoints = 1;
 	return 0;
     } else {
@@ -151,22 +152,6 @@ static void mview_fill_points (multi_series_view *mview)
     }
 }
 
-static void replace_window_text (windata_t *vwin, const char *pbuf)
-{
-#ifndef OLD_GTK
-    GtkTextBuffer *tbuf;
-
-    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
-    gtk_text_buffer_set_text(tbuf, pbuf, -1);
-#else
-    gtk_text_freeze(GTK_TEXT(vwin->w));
-    gtk_editable_delete_text(GTK_EDITABLE(vwin->w), 0, -1);
-    gtk_text_insert(GTK_TEXT(vwin->w), fixed_font, 
-		    NULL, NULL, pbuf, strlen(pbuf));
-    gtk_text_thaw(GTK_TEXT(vwin->w));
-#endif
-}
-
 static PRN *series_view_print_csv (windata_t *vwin)
 {
     series_view *sview = (series_view *) vwin->data;
@@ -212,7 +197,7 @@ static void series_view_print (windata_t *vwin)
     } 
     
     /* print formatted data to buffer */
-    if (datainfo->vector[sview->varnum]) {
+    if (var_is_series(datainfo, sview->varnum)) {
 	pprintf(prn, "\n     Obs ");
 	pprintf(prn, "%*s\n\n", 13, datainfo->varname[sview->varnum]);
 	for (t=0; t<sview->npoints; t++) {
@@ -241,7 +226,7 @@ static void series_view_print (windata_t *vwin)
     }
 
     pbuf = gretl_print_get_buffer(prn);
-    replace_window_text(vwin, pbuf);
+    textview_set_text(vwin->w, pbuf);
 
     gretl_print_destroy(prn);
 }
@@ -286,7 +271,7 @@ static void multi_series_view_print_sorted (windata_t *vwin)
 	gui_errmsg(err);
     } else {
 	pbuf = gretl_print_get_buffer(prn);
-	replace_window_text(vwin, pbuf);
+	textview_set_text(vwin->w, pbuf);
     }
 
     free(obsvec);
@@ -473,19 +458,29 @@ void series_view_connect (windata_t *vwin, int varnum)
     }
 }
 
-multi_series_view *multi_series_view_new (int *list)
+int has_sortable_data (windata_t *vwin)
+{
+    multi_series_view *mview;
+
+    if (vwin == NULL || vwin->role != PRINT || vwin->data == NULL) {
+	return 0;
+    }
+
+    mview = vwin->data;
+
+    return mview->list != NULL && mview->list[0] <= 5;
+}
+
+multi_series_view *multi_series_view_new (const int *list)
 {
     multi_series_view *mview = NULL;
     int i, err = 0;
 
-    /* we do this only for a list of up to five variables,
-       with no scalars */
-
-    if (list == NULL || list[0] > 5) {
+    if (list == NULL) {
 	err = 1;
     } else {
 	for (i=1; i<=list[0]; i++) {
-	    if (!datainfo->vector[list[i]]) {
+	    if (var_is_scalar(datainfo, list[i])) {
 		err = 1;
 		break;
 	    }
@@ -493,20 +488,22 @@ multi_series_view *multi_series_view_new (int *list)
     }
 
     if (err) {
-	free(list);
 	return NULL;
     }
 
     mview = malloc(sizeof *mview);
 
     if (mview != NULL) {
-	mview->list = list;
-	mview->sortvar = 0;
-	mview->npoints = 0;
-	mview->points = NULL;
-    } else {
-	free(list);
-    }
+	mview->list = gretl_list_copy(list);
+	if (mview->list == NULL) {
+	    free(mview);
+	    mview = NULL;
+	} else {
+	    mview->sortvar = 0;
+	    mview->npoints = 0;
+	    mview->points = NULL;
+	}
+    } 
 
     return mview;
 }
@@ -593,42 +590,24 @@ void series_view_format_dialog (GtkWidget *src, windata_t *vwin)
     GTK_WIDGET_SET_FLAGS (tmp, GTK_CAN_DEFAULT);
     gtk_box_pack_start (GTK_BOX (hbox), 
                         tmp, TRUE, TRUE, 0);
-#ifndef OLD_GTK
     g_signal_connect_swapped (G_OBJECT (tmp), "clicked", 
 			      G_CALLBACK (gtk_widget_destroy), 
 			      G_OBJECT (w));
-#else
-    gtk_signal_connect (GTK_OBJECT (tmp), "clicked", 
-			GTK_SIGNAL_FUNC (delete_widget), 
-			GTK_OBJECT (w));
-#endif
 
     tmp = standard_button(GTK_STOCK_CANCEL);
     gtk_box_pack_start (GTK_BOX (hbox), 
                         tmp, TRUE, TRUE, 0);
-#ifndef OLD_GTK
     g_signal_connect (G_OBJECT (tmp), "clicked", 
 		      G_CALLBACK (series_view_format_cancel), sview);
     g_signal_connect_swapped (G_OBJECT (tmp), "clicked", 
 			      G_CALLBACK (gtk_widget_destroy), 
 			      G_OBJECT (w));
-#else
-    gtk_signal_connect (GTK_OBJECT (tmp), "clicked", 
-			GTK_SIGNAL_FUNC (series_view_format_cancel), sview);
-    gtk_signal_connect (GTK_OBJECT (tmp), "clicked", 
-			GTK_SIGNAL_FUNC (delete_widget), 
-			GTK_OBJECT (w));
-#endif
 
     gtk_container_add(GTK_CONTAINER(vbox), hbox);
     gtk_container_add(GTK_CONTAINER(w), vbox);
 
     gtk_widget_show_all(w);
 
-#ifdef OLD_GTK
-    gtk_window_set_transient_for(GTK_WINDOW(w), GTK_WINDOW(vwin->dialog));
-#endif
-
     gretl_set_window_modal(w);
 
     gtk_main(); /* block */
