@@ -7,8 +7,6 @@
 void read_rc (void);
 #endif
 
-int using_olddat (void);
-
 int using_hc_by_default (void);
 
 int get_manpref (void);
@@ -21,13 +19,15 @@ void dump_rc (void);
 
 void force_english_help (void);
 
-void options_dialog (gpointer p, guint page, GtkWidget *w);
+int options_dialog (int page);
+
+void options_dialog_callback (gpointer p, guint u, GtkWidget *w);
 
 void font_selector (gpointer data, guint which, GtkWidget *widget);
 
 void set_fixed_font (void);
 
-#if !defined(OLD_GTK) && !defined(USE_GNOME)
+#ifndef USE_GNOME
 
 void set_app_font (const char *fontname);
 
