@@ -21,11 +21,15 @@
 #define SSHEET_H
 
 typedef enum {
+    SHEET_EDIT_VARLIST,
     SHEET_EDIT_DATASET,
-    SHEET_NEW_DATASET
-} SheetCode;
+    SHEET_NEW_DATASET,
+    SHEET_EDIT_MATRIX
+} SheetCmd;
 
-void show_spreadsheet (SheetCode code);
+void show_spreadsheet (SheetCmd c);
+
+void edit_matrix (gretl_matrix *m);
 
 int dataset_locked (void);
 
