@@ -84,20 +84,38 @@ gint get_char_width (GtkWidget *widget)
     return width;
 }
 
-gchar *textview_get_text (GtkTextView *view)
+gchar *textview_get_text (GtkWidget *view)
 {
     GtkTextBuffer *tbuf;
     GtkTextIter start, end;
 
     g_return_val_if_fail(GTK_IS_TEXT_VIEW(view), NULL);
 
-    tbuf = gtk_text_view_get_buffer(view);
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
     gtk_text_buffer_get_start_iter(tbuf, &start);
     gtk_text_buffer_get_end_iter(tbuf, &end);
 
     return gtk_text_buffer_get_text(tbuf, &start, &end, FALSE);
 }
 
+int textview_set_text (GtkWidget *view, const gchar *text)
+{
+    GtkTextBuffer *tbuf;
+
+    g_return_val_if_fail(GTK_IS_TEXT_VIEW(view), 1);
+
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
+    g_return_val_if_fail(tbuf != NULL, 1);
+
+    if (text != NULL) {
+	gtk_text_buffer_set_text(tbuf, text, -1);
+    } else {
+	gtk_text_buffer_set_text(tbuf, "", -1);
+    }
+
+    return 0;
+}
+
 int viewer_char_count (windata_t *vwin)
 {
     GtkTextBuffer *tbuf;
@@ -108,16 +126,16 @@ int viewer_char_count (windata_t *vwin)
 
 void text_paste (windata_t *vwin, guint u, GtkWidget *widget)
 {
+    gchar *undo_buf = textview_get_text(vwin->w);
     gchar *old;
-    GtkTextBuffer *buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
-    gchar *undo_buf = textview_get_text(GTK_TEXT_VIEW(vwin->w));
 
     old = g_object_get_data(G_OBJECT(vwin->w), "undo");
     g_free(old);
 
     g_object_set_data(G_OBJECT(vwin->w), "undo", undo_buf);
 
-    gtk_text_buffer_paste_clipboard(buf, gtk_clipboard_get(GDK_NONE),
+    gtk_text_buffer_paste_clipboard(gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w)),
+				    gtk_clipboard_get(GDK_NONE),
 				    NULL, TRUE);
 }
 
@@ -162,18 +180,12 @@ void text_undo (windata_t *vwin, guint u, GtkWidget *widget)
 
 #ifdef USE_GTKSOURCEVIEW
 
-static int 
-gtk_source_buffer_load_file (GtkSourceBuffer *sbuf, 
-			     const char *fname)
+static int source_buffer_load_file (GtkSourceBuffer *sbuf, FILE *fp)
 {
-    FILE *fp;
-    GtkTextIter iter;    
     char readbuf[MAXSTR], *chunk = NULL;
+    GtkTextIter iter;   
 
-    fp = gretl_fopen(fname, "rb");
-    if (fp == NULL) return 1;
-
-    gtk_source_buffer_begin_not_undoable_action (sbuf);
+    gtk_source_buffer_begin_not_undoable_action(sbuf);
 
     gtk_text_buffer_set_text(GTK_TEXT_BUFFER(sbuf), "", 0);
     gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(sbuf), &iter, 0);
@@ -211,10 +223,7 @@ gtk_source_buffer_load_file (GtkSourceBuffer *sbuf,
 	}
     }
 
-    fclose(fp);
-	
     gtk_source_buffer_end_not_undoable_action(sbuf);
-
     gtk_text_buffer_set_modified(GTK_TEXT_BUFFER(sbuf), FALSE);
 
     /* move cursor to the beginning */
@@ -224,15 +233,50 @@ gtk_source_buffer_load_file (GtkSourceBuffer *sbuf,
     return 0;
 }
 
-void source_buffer_insert_file (GtkSourceBuffer *sbuf, const char *filename,
-				int role)
+static int source_buffer_load_buf (GtkSourceBuffer *sbuf, const char *buf)
+{
+    char line[MAXLINE];
+    GtkTextIter iter;   
+
+    gtk_source_buffer_begin_not_undoable_action(sbuf);
+    gtk_text_buffer_set_text(GTK_TEXT_BUFFER(sbuf), "", 0);
+    gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(sbuf), &iter, 0);
+
+    bufgets_init(buf);
+
+    while (bufgets(line, sizeof line, buf)) {
+	gtk_text_buffer_insert(GTK_TEXT_BUFFER(sbuf), &iter, line, -1);
+	gtk_text_buffer_insert(GTK_TEXT_BUFFER(sbuf), &iter, "\n", 1);
+    }
+
+    gtk_source_buffer_end_not_undoable_action(sbuf);
+    gtk_text_buffer_set_modified(GTK_TEXT_BUFFER(sbuf), FALSE);
+
+    /* move cursor to the beginning */
+    gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(sbuf), &iter);
+    gtk_text_buffer_place_cursor(GTK_TEXT_BUFFER(sbuf), &iter);
+
+    return 0;
+}
+
+static void 
+real_sourceview_insert (windata_t *vwin, const char *fname, const char *buf)
 {
     GtkSourceLanguagesManager *manager;    
     GtkSourceLanguage *language = NULL;
-		
-    manager = g_object_get_data(G_OBJECT (sbuf), "languages-manager");
+    FILE *fp = NULL;
 
-    if (role == GR_PLOT) {
+    if (fname != NULL) {
+	fp = gretl_fopen(fname, "rb");
+	if (fp == NULL) {
+	    errbox(_("Couldn't open %s"), fname);
+	    return;
+	}
+    }
+		
+    manager = g_object_get_data(G_OBJECT(vwin->sbuf), "languages-manager");
+
+    if (vwin->role == GR_PLOT) {
 	language = 
 	    gtk_source_languages_manager_get_language_from_mime_type 
 	    (manager, "application/x-gnuplot");
@@ -243,17 +287,32 @@ void source_buffer_insert_file (GtkSourceBuffer *sbuf, const char *filename,
     }
 
     if (language == NULL) {
-	g_object_set(G_OBJECT(sbuf), "highlight", FALSE, NULL);
+	g_object_set(G_OBJECT(vwin->sbuf), "highlight", FALSE, NULL);
     } else {
-	g_object_set(G_OBJECT(sbuf), "highlight", TRUE, NULL);
-	gtk_source_buffer_set_language(sbuf, language);
+	g_object_set(G_OBJECT(vwin->sbuf), "highlight", TRUE, NULL);
+	gtk_source_buffer_set_language(vwin->sbuf, language);
     }
 
-    gtk_source_buffer_load_file(sbuf, filename);
+    if (fp != NULL) {
+	source_buffer_load_file(vwin->sbuf, fp);
+	fclose(fp);
+    } else {
+	source_buffer_load_buf(vwin->sbuf, buf);
+    }
 }
 
-void create_source (windata_t *vwin, GtkSourceBuffer **buf, 
-		    int hsize, int vsize, gboolean editable)
+void sourceview_insert_file (windata_t *vwin, const char *fname)
+{
+    real_sourceview_insert(vwin, fname, NULL);
+}
+
+void sourceview_insert_buffer (windata_t *vwin, const char *buf)
+{
+    real_sourceview_insert(vwin, NULL, buf);
+}
+
+void create_source (windata_t *vwin, int hsize, int vsize, 
+		    gboolean editable)
 {
     GtkSourceLanguagesManager *lm;
     GtkSourceBuffer *sbuf;
@@ -263,29 +322,29 @@ void create_source (windata_t *vwin, GtkSourceBuffer **buf,
 
     /* set up paren-matching in blue */
 
-    cmap = gdk_colormap_get_system ();
-    gdk_color_parse ("blue", &blue);
-    gdk_colormap_alloc_color (cmap, &blue, FALSE, TRUE);
+    cmap = gdk_colormap_get_system();
+    gdk_color_parse("blue", &blue);
+    gdk_colormap_alloc_color(cmap, &blue, FALSE, TRUE);
 
-    lm = gtk_source_languages_manager_new ();
-    tagstyle = gtk_source_tag_style_new ();
+    lm = gtk_source_languages_manager_new();
+    tagstyle = gtk_source_tag_style_new();
     
     sbuf = GTK_SOURCE_BUFFER(gtk_source_buffer_new(NULL));
-    g_object_ref (lm);
-    g_object_set_data_full (G_OBJECT (sbuf), "languages-manager",
-			    lm, (GDestroyNotify) g_object_unref); 
-    g_object_unref (lm); 
+    g_object_ref(lm);
+    g_object_set_data_full(G_OBJECT(sbuf), "languages-manager",
+			   lm, (GDestroyNotify) g_object_unref); 
+    g_object_unref(lm); 
 
     tagstyle->mask = GTK_SOURCE_TAG_STYLE_USE_FOREGROUND;
     tagstyle->foreground = blue;
-    g_object_set_data_full (G_OBJECT (sbuf), "tag-style",
-			    tagstyle, 
-			    (GDestroyNotify) gtk_source_tag_style_free); 
+    g_object_set_data_full(G_OBJECT(sbuf), "tag-style",
+			   tagstyle, 
+			   (GDestroyNotify) gtk_source_tag_style_free); 
     gtk_source_buffer_set_bracket_match_style(sbuf, tagstyle);
     gtk_source_buffer_set_check_brackets(sbuf, TRUE);
 
     vwin->w = gtk_source_view_new_with_buffer(sbuf);
-    *buf = sbuf;
+    vwin->sbuf = sbuf;
 
     gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(vwin->w), GTK_WRAP_WORD);
     gtk_text_view_set_left_margin(GTK_TEXT_VIEW(vwin->w), 4);
@@ -296,7 +355,7 @@ void create_source (windata_t *vwin, GtkSourceBuffer **buf,
     hsize *= get_char_width(vwin->w);
     hsize += 48;
 
-    gtk_window_set_default_size (GTK_WINDOW(vwin->dialog), hsize, vsize); 
+    gtk_window_set_default_size(GTK_WINDOW(vwin->dialog), hsize, vsize); 
     gtk_text_view_set_editable(GTK_TEXT_VIEW(vwin->w), editable);
     gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(vwin->w), editable);
 
@@ -444,20 +503,22 @@ static GtkTextBuffer *gretl_text_buf_new (void)
     return tbuf;
 }
 
-void text_buffer_insert_colorized_buffer (GtkTextBuffer *tbuf, PRN *prn)
+void textview_set_text_colorized (GtkWidget *view, const char *buf)
 {
+    GtkTextBuffer *tbuf;
     GtkTextIter iter;    
     int thiscolor, nextcolor;
-    const char *pbuf;
     char readbuf[MAXSTR];
 
-    pbuf = gretl_print_get_buffer(prn);
+    g_return_if_fail(GTK_IS_TEXT_VIEW(view));
+
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
 
     thiscolor = PLAIN_TEXT;
     gtk_text_buffer_get_iter_at_offset(tbuf, &iter, 0);
-    bufgets_init(pbuf);
+    bufgets_init(buf);
 
-    while (bufgets(readbuf, sizeof readbuf, pbuf)) {
+    while (bufgets(readbuf, sizeof readbuf, buf)) {
 
 	if (ends_with_backslash(readbuf)) {
 	    nextcolor = BLUE_TEXT;
@@ -484,19 +545,22 @@ void text_buffer_insert_colorized_buffer (GtkTextBuffer *tbuf, PRN *prn)
     }
 }
 
-void text_buffer_insert_file (GtkTextBuffer *tbuf, const char *fname, 
-			      int role)
+void textview_insert_file (windata_t *vwin, const char *fname)
 {
     FILE *fp;
+    GtkTextBuffer *tbuf;
     GtkTextIter iter;    
     int thiscolor, nextcolor;
     char readbuf[MAXSTR], *chunk;
 
+    g_return_if_fail(GTK_IS_TEXT_VIEW(vwin->w));
+
     fp = gretl_fopen(fname, "r");
     if (fp == NULL) return;
 
     thiscolor = nextcolor = PLAIN_TEXT;
 
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
     gtk_text_buffer_get_iter_at_offset(tbuf, &iter, 0);
 
     memset(readbuf, 0, sizeof readbuf);
@@ -510,12 +574,12 @@ void text_buffer_insert_file (GtkTextBuffer *tbuf, const char *fname,
 
 	nextcolor = PLAIN_TEXT;
 	
-	if (role == SCRIPT_OUT && ends_with_backslash(chunk)) {
+	if (vwin->role == SCRIPT_OUT && ends_with_backslash(chunk)) {
 	    nextcolor = BLUE_TEXT;
 	}
 
 	if (*chunk == '?') {
-	    thiscolor = (role == CONSOLE)? RED_TEXT : BLUE_TEXT;
+	    thiscolor = (vwin->role == CONSOLE)? RED_TEXT : BLUE_TEXT;
 	} else if (*chunk == '#') {
 	    thiscolor = BLUE_TEXT;
 	} 
@@ -1163,41 +1227,44 @@ void correct_line_color (windata_t *vwin)
 
 #endif /* not USE_GTKSOURCEVIEW */
 
-void create_text (windata_t *vwin, GtkTextBuffer **buf, 
-		  int hsize, int vsize, gboolean editable)
+GtkWidget *create_text (GtkWidget *dlg, int hsize, int vsize, 
+			gboolean editable)
 {
     GtkTextBuffer *tbuf = gretl_text_buf_new();
+    GtkWidget *w = gtk_text_view_new_with_buffer(tbuf);
 
-    vwin->w = gtk_text_view_new_with_buffer(tbuf);
-    *buf = tbuf;
+    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(w), GTK_WRAP_WORD);
+    gtk_text_view_set_left_margin(GTK_TEXT_VIEW(w), 4);
+    gtk_text_view_set_right_margin(GTK_TEXT_VIEW(w), 4);
 
-    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(vwin->w), GTK_WRAP_WORD);
-    gtk_text_view_set_left_margin(GTK_TEXT_VIEW(vwin->w), 4);
-    gtk_text_view_set_right_margin(GTK_TEXT_VIEW(vwin->w), 4);
+    gtk_widget_modify_font(GTK_WIDGET(w), fixed_font);
 
-    gtk_widget_modify_font(GTK_WIDGET(vwin->w), fixed_font);
+    if (hsize > 0) {
+	hsize *= get_char_width(w);
+	hsize += 48;
+    }
 
-    hsize *= get_char_width(vwin->w);
-    hsize += 48;
+    if (dlg != NULL) {
+	gtk_window_set_default_size(GTK_WINDOW(dlg), hsize, vsize); 
+    }
+    gtk_text_view_set_editable(GTK_TEXT_VIEW(w), editable);
+    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(w), editable);
 
-    gtk_window_set_default_size(GTK_WINDOW(vwin->dialog), hsize, vsize); 
-    gtk_text_view_set_editable(GTK_TEXT_VIEW(vwin->w), editable);
-    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(vwin->w), editable);
+    return w;
 }
 
-void text_table_setup (windata_t *vwin)
+void text_table_setup (GtkWidget *vbox, GtkWidget *w)
 {
     GtkWidget *sw;
 
     sw = gtk_scrolled_window_new(NULL, NULL);
-    gtk_box_pack_start(GTK_BOX(vwin->vbox), 
-		       sw, TRUE, TRUE, FALSE);
+    gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, FALSE);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
 				   GTK_POLICY_AUTOMATIC,
 				   GTK_POLICY_AUTOMATIC);
     gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
 					GTK_SHADOW_IN);
-    gtk_container_add(GTK_CONTAINER(sw), vwin->w); 
-    gtk_widget_show(vwin->w);
+    gtk_container_add(GTK_CONTAINER(sw), w); 
+    gtk_widget_show(w);
     gtk_widget_show(sw);
 }
