@@ -31,7 +31,9 @@ void cursor_to_top (windata_t *vwin);
 
 gint get_char_width (GtkWidget *widget);
 
-gchar *textview_get_text (GtkTextView *view);
+gchar *textview_get_text (GtkWidget *view);
+
+int textview_set_text (GtkWidget *view, const gchar *text);
 
 int viewer_char_count (windata_t *vwin);
 
@@ -39,15 +41,14 @@ void text_paste (windata_t *vwin, guint u, GtkWidget *widget);
 
 void text_undo (windata_t *vwin, guint u, GtkWidget *widget);
 
-void text_buffer_insert_colorized_buffer (GtkTextBuffer *tbuf, PRN *prn);
+void textview_set_text_colorized (GtkWidget *view, const char *buf);
 
-void text_buffer_insert_file (GtkTextBuffer *tbuf, const char *fname, 
-			      int role);
+void textview_insert_file (windata_t *vwin, const char *fname);
 
-void create_text (windata_t *vwin, GtkTextBuffer **buf, 
-		  int hsize, int vsize, gboolean editable);
+GtkWidget *create_text (GtkWidget *dlg, int hsize, int vsize, 
+			gboolean editable);
 
-void text_table_setup (windata_t *vwin);
+void text_table_setup (GtkWidget *vbox, GtkWidget *w);
 
 void set_help_topic_buffer (windata_t *hwin, int hcode, int pos, int en);
 
@@ -56,11 +57,13 @@ gboolean help_popup_handler (GtkWidget *w, GdkEventButton *event,
 
 #ifdef USE_GTKSOURCEVIEW
 
-void create_source (windata_t *vwin, GtkSourceBuffer **buf, 
-		    int hsize, int vsize, gboolean editable);
+void create_source (windata_t *vwin, int hsize, int vsize, 
+		    gboolean editable);
+
+void sourceview_insert_file (windata_t *vwin, const char *fname);
+
+void sourceview_insert_buffer (windata_t *vwin, const char *buf);
 
-void source_buffer_insert_file (GtkSourceBuffer *sbuf, const char *filename,
-				int role);
 #else
 
 void correct_line_color (windata_t *vwin);
