@@ -84,6 +84,7 @@ static int use_proxy;
 #endif /* UPDATER */
 
 enum {
+    SAVE_NONE,
     SAVE_TO_BUFFER,
     SAVE_TO_FILE
 } save_opts;
@@ -254,15 +255,14 @@ struct proto {
     unsigned short port;
 };
 
-struct urlinfo
-{
+struct urlinfo {
     char *url;                   /* the URL */
     uerr_t proto;                /* URL protocol */
     unsigned short port;
     unsigned short saveopt;      /* save to buffer or file? */   
     char *path; 
     char *localfile;
-    char **savebuf;
+    char *getbuf;
     char host[32];
     char errbuf[80];
     FILE *fp;                   /* for saving content locally */
@@ -279,8 +279,7 @@ struct http_stat {
     long dltime;		/* time of the download */
 };
 
-struct rbuf
-{
+struct rbuf {
     int fd;
     char buffer[4096];		/* the input buffer */
     char *buffer_pos;		/* current position in the buffer */
@@ -343,34 +342,7 @@ enum header_get_flags {
 static struct urlinfo gretlproxy; 
 
 /* prototypes */
-static char *time_str (time_t *tm);
-static void rbuf_initialize (struct rbuf *rbuf, int fd);
-static int rbuf_peek (struct rbuf *rbuf, char *store);
 static size_t rbuf_flush (struct rbuf *rbuf, char *where, int maxsize);
-
-static int header_get (struct rbuf *rbuf, char **hdr, 
-		       enum header_get_flags flags);
-static int header_strdup (const char *header, void *closure);
-static int header_extract_number (const char *header, void *closure);
-static int skip_lws (const char *string);
-static int header_process (const char *header, const char *name,
-			   int (*procfun) (const char *, void *),
-			   void *arg);
-static int parse_http_status_line (const char *line, 
-				   const char **reason_phrase_ptr);
-static int http_process_range (const char *hdr, void *arg);
-static int http_process_none (const char *hdr, void *arg);
-static int http_process_type (const char *hdr, void *arg);
-static int numdigit (long a);
-static char *herrmsg (int error);
-static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs, int *dt,
-		       struct urlinfo *proxy);
-static uerr_t http_loop (struct urlinfo *u, int *dt, struct urlinfo *proxy);
-static struct urlinfo *newurl (void);
-static void freeurl (struct urlinfo *u, int delfile);
-static int get_contents (int fd, FILE *fp, char **getbuf, long *len, 
-			 long expected, struct rbuf *rbuf);
-static int store_hostaddress (unsigned char *where, const char *hostname);
 static uerr_t make_connection (int *sock, char *hostname, 
 			       unsigned short port);
 static int iread (int fd, char *buf, int len);
@@ -432,17 +404,18 @@ static int get_host_ip (char *h_ip, const char *h_name)
 
 static char *time_str (time_t *tm)
 {
-  static char tms[15];
-  struct tm *ptm;
-  time_t tim;
+    static char tms[15];
+    struct tm *ptm;
+    time_t tim;
 
-  *tms = '\0';
-  tim = time (tm);
-  if (tim == -1)
+    *tms = '\0';
+    tim = time (tm);
+    if (tim == -1) {
+	return tms;
+    }
+    ptm = localtime (&tim);
+    sprintf(tms, "%02d:%02d:%02d", ptm->tm_hour, ptm->tm_min, ptm->tm_sec);
     return tms;
-  ptm = localtime (&tim);
-  sprintf (tms, "%02d:%02d:%02d", ptm->tm_hour, ptm->tm_min, ptm->tm_sec);
-  return tms;
 }
 
 /* http header functions -- based on Wget */
@@ -553,8 +526,7 @@ static int header_get (struct rbuf *rbuf, char **hdr,
 		}
 		break;
 	    }
-	}
-	else if (res == 0) {
+	} else if (res == 0) {
 	    return HG_EOF;
 	} else {
 	    return HG_ERROR;
@@ -587,22 +559,11 @@ static int header_strdup (const char *header, void *closure)
     return 1;
 }
 
-static int skip_lws (const char *string)
-{
-    const char *p = string;
-
-    while (*p == ' ' || *p == '\t' || *p == '\r' || *p == '\n') {
-	++p;
-    }
-
-    return p - string;
-}
-
 static int header_process (const char *header, const char *name,
 			   int (*procfun) (const char *, void *),
 			   void *arg)
 {
-    while (*name && (tolower (*name) == tolower (*header))) {
+    while (*name && (tolower(*name) == tolower(*header))) {
 	++name, ++header;
     }
 
@@ -610,7 +571,7 @@ static int header_process (const char *header, const char *name,
 	return 0;
     }
 
-    header += skip_lws (header);
+    header += strspn(header, " \t\r\n");
 
     return ((*procfun) (header, arg));
 }
@@ -626,8 +587,6 @@ static void rbuf_initialize (struct rbuf *rbuf, int fd)
     rbuf->buffer_left = 0;
 }
 
-static int parse_http_status_line (const char *line, 
-				   const char **reason_phrase_ptr)
 /* Parse the HTTP status line, which is of format:
 
    HTTP-Version SP Status-Code SP Reason-Phrase
@@ -635,6 +594,9 @@ static int parse_http_status_line (const char *line,
    The function returns the status-code, or -1 if the status line is
    malformed.  The pointer to reason-phrase is returned in RP.  
 */
+
+static int parse_http_status_line (const char *line, 
+				   const char **reason_phrase_ptr)
 {
     int mjr, mnr, statcode;
     const char *p;
@@ -663,6 +625,7 @@ static int parse_http_status_line (const char *line,
     if (*line != ' ' || p == line) {
 	return -1;
     }
+
     /* Will accept only 1.0 and higher HTTP-versions.  The value of
        minor version can be safely ignored. */
     if (mjr < 1) {
@@ -689,8 +652,7 @@ static int parse_http_status_line (const char *line,
 	} else {
 	    return -1;
 	}
-    }
-    else {
+    } else {
 	*reason_phrase_ptr = line + 1;
     }
 
@@ -709,12 +671,12 @@ struct http_process_range_closure {
 static int http_process_range (const char *hdr, void *arg)
 {
     struct http_process_range_closure *closure
-	= (struct http_process_range_closure *)arg;
+	= (struct http_process_range_closure *) arg;
     long num;
 
     if (!strncasecmp (hdr, "bytes", 5)) {
 	hdr += 5;
-	hdr += skip_lws (hdr);
+	hdr += strspn(hdr, " \t\r\n");
 	if (!*hdr) {
 	    return 0;
 	}
@@ -821,6 +783,102 @@ static char *herrmsg (int error)
     }
 }
 
+static int get_contents (int fd, FILE *fp, char **getbuf, long *len, 
+			 long expected, struct rbuf *rbuf)
+{
+    int res = 0;
+    int sp_ret = SP_RETURN_OK;
+    static char cbuf[GRETL_BUFSIZE];
+    size_t allocated;
+    int nchunks;
+#ifdef UPDATER
+    int show = 1;
+#else
+    void *handle;
+    int (*show_progress) (long, long, int) = NULL;
+    int show = 0;
+
+    if (expected > 2 * GRETL_BUFSIZE) {
+	show_progress = gui_get_plugin_function("show_progress", 
+						&handle);
+	if (show_progress != NULL) show = 1;
+    }
+#endif
+    if (show) {
+	sp_ret = show_progress(res, expected, SP_LOAD_INIT);
+    }
+
+    *len = 0L;
+
+    if (rbuf && RBUF_FD(rbuf) == fd) {
+	while ((res = rbuf_flush(rbuf, cbuf, sizeof cbuf)) != 0) {
+	    if (fp == NULL) {
+		memcpy(*getbuf, cbuf, res);
+	    } else {
+		if (fwrite(cbuf, 1, res, fp) < (unsigned) res) {
+		    return -2;
+		}
+	    }
+	    *len += res;
+	    if (show) {
+		sp_ret = show_progress(res, expected, SP_NONE);
+	    }
+	}
+    }
+
+    if (sp_ret == SP_RETURN_CANCELED) goto canceled;
+
+    /* Read from fd while there is available data. */
+    nchunks = 1;
+    allocated = GRETL_BUFSIZE;
+    do {
+	res = iread(fd, cbuf, sizeof cbuf);
+	if (res > 0) {
+	    if (fp == NULL) {
+		if ((size_t) (*len + res) > allocated) {
+		    nchunks *= 2;
+		    *getbuf = myrealloc(*getbuf, nchunks * GRETL_BUFSIZE);
+		    if (*getbuf == NULL) {
+			return -2;
+		    }
+		    allocated = nchunks * GRETL_BUFSIZE;
+		}
+		memcpy(*getbuf + *len, cbuf, res);
+	    } else {
+		if (fwrite(cbuf, 1, res, fp) < (unsigned) res) {
+		    return -2;
+		}
+	    }
+	    *len += res;
+
+	    if (show) {
+		sp_ret = show_progress(res, expected, SP_NONE);
+		if (sp_ret == SP_RETURN_DONE || sp_ret == SP_RETURN_CANCELED) {
+		    break;
+		}
+	    }
+	}
+    } while (res > 0);
+
+    if (res < -1) res = -1;
+
+    if (show) {
+	show_progress(0, expected, SP_FINISH);
+#ifndef UPDATER
+	close_plugin(handle);
+#endif
+    }
+
+ canceled:
+
+    if (sp_ret == SP_RETURN_CANCELED) {
+	fprintf(stderr, "Got SP_RETURN_CANCELED\n");
+	res = -2;
+    }
+
+    return res;
+}
+
 static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs, 
 		       int *dt, struct urlinfo *proxy)
 {
@@ -1007,8 +1065,7 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
 		}
 		free(hdr);
 		break;
-	    }
-	    else if (!*error) {
+	    } else if (!*error) {
 		hs->error = g_strdup(_("(no description)"));
 	    } else {
 		hs->error = g_strdup(error);
@@ -1035,20 +1092,20 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
 
 	/* Try getting content-type */
 	if (!type) {
-	    if (header_process (hdr, "Content-Type", http_process_type, &type))
+	    if (header_process(hdr, "Content-Type", http_process_type, &type))
 		goto done_header;
 	}
 
 	/* Try getting location */
 	if (!hs->newloc) {
-	    if (header_process (hdr, "Location", header_strdup, &hs->newloc))
+	    if (header_process(hdr, "Location", header_strdup, &hs->newloc))
 		goto done_header;
 	}
 
 	/* Try getting last-modified */
 	if (!hs->remote_time) {
-	    if (header_process (hdr, "Last-Modified", header_strdup,
-				&hs->remote_time))
+	    if (header_process(hdr, "Last-Modified", header_strdup,
+			       &hs->remote_time))
 		goto done_header;
 	}
 
@@ -1057,8 +1114,8 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
 	if (*dt & ACCEPTRANGES) {
 	    int nonep;
 
-	    if (header_process (hdr, "Accept-Ranges", 
-				http_process_none, &nonep)) {
+	    if (header_process(hdr, "Accept-Ranges", http_process_none, 
+			       &nonep)) {
 		if (nonep) {
 		    *dt &= ~ACCEPTRANGES;
 		}
@@ -1069,6 +1126,7 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
 	/* Try getting content-range */
 	if (contrange == -1) {
 	    struct http_process_range_closure closure;
+
 	    if (header_process(hdr, "Content-Range", 
 			       http_process_range, &closure)) {
 		contrange = closure.first_byte_pos;
@@ -1125,11 +1183,12 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
     }
 
     /* Get the contents of the document */
-    hs->res = get_contents(sock, u->fp, u->savebuf, &hs->len, 
-			   (contlen != -1 ? contlen : 0), &rbuf);
+    hs->res = get_contents(sock, u->fp, &u->getbuf, 
+			   &hs->len, (contlen != -1 ? contlen : 0), 
+			   &rbuf);
 
 #if WDEBUG
-    fprintf(stderr, "get_contents returned %d\n", hs->res);
+    fprintf(stderr, "gethttp: get_contents returned %d\n", hs->res);
 #endif
 
     free(all_headers);
@@ -1146,14 +1205,14 @@ static uerr_t gethttp (struct urlinfo *u, struct http_stat *hs,
 
 static uerr_t http_loop (struct urlinfo *u, int *dt, struct urlinfo *proxy)
 {
+    struct http_stat hstat; /* HTTP status */
     int count = 0;
     char *tms;
     uerr_t err;
-    struct http_stat hstat; /* HTTP status */
 
     *dt = 0 | ACCEPTRANGES;
 
-    /* THE loop */
+    /* main loop */
     do {
 	++count;
 	tms = time_str(NULL);
@@ -1170,7 +1229,7 @@ static uerr_t http_loop (struct urlinfo *u, int *dt, struct urlinfo *proxy)
 	fprintf(stderr, "http_loop: err (from gethttp) = %d, errbuf = '%s'\n", 
 		err, (proxy)? proxy->errbuf : u->errbuf);
 	if (err == RETRFINISHED) {
-	    fprintf(stderr, " (%d == RETRFINISHED\n", err);
+	    fprintf(stderr, " (%d == RETRFINISHED)\n", err);
 	}
 #endif
 
@@ -1247,7 +1306,7 @@ static void url_init (struct urlinfo *u)
     u->url = NULL;
     u->path = NULL;
     u->localfile = NULL;
-    u->savebuf = NULL;
+    u->getbuf = NULL;
 
     u->proto = URLHTTP;
     u->port = DEFAULT_HTTP_PORT;
@@ -1299,108 +1358,6 @@ static void freeurl (struct urlinfo *u, int delfile)
     free(u);
 }
 
-static int get_contents (int fd, FILE *fp, char **getbuf, long *len, 
-			 long expected, struct rbuf *rbuf)
-{
-    int res = 0;
-    int sp_ret = SP_RETURN_OK;
-    static char cbuf[GRETL_BUFSIZE];
-    size_t allocated;
-    int nchunks;
-#ifdef UPDATER
-    int show = 1;
-#else
-    void *handle;
-    int (*show_progress) (long, long, int) = NULL;
-    int show = 0;
-
-    if (expected > 2 * GRETL_BUFSIZE) {
-	show_progress = gui_get_plugin_function("show_progress", 
-						&handle);
-	if (show_progress != NULL) show = 1;
-    }
-#endif
-    if (show) {
-	sp_ret = show_progress(res, expected, SP_LOAD_INIT);
-    }
-
-    *len = 0L;
-
-#if 0
-    if (fp != NULL) {
-	rewind(fp);
-    }
-#endif
-
-    if (rbuf && RBUF_FD(rbuf) == fd) {
-	while ((res = rbuf_flush(rbuf, cbuf, sizeof cbuf)) != 0) {
-	    if (fp == NULL) {
-		memcpy(*getbuf, cbuf, res);
-	    } else {
-		if (fwrite(cbuf, 1, res, fp) < (unsigned) res) {
-		    return -2;
-		}
-	    }
-	    *len += res;
-	    if (show) {
-		sp_ret = show_progress(res, expected, SP_NONE);
-	    }
-	}
-    }
-
-    if (sp_ret == SP_RETURN_CANCELED) goto canceled;
-
-    /* Read from fd while there is available data. */
-    nchunks = 1;
-    allocated = GRETL_BUFSIZE;
-    do {
-	res = iread(fd, cbuf, sizeof cbuf);
-	if (res > 0) {
-	    if (fp == NULL) {
-		if ((size_t) (*len + res) > allocated) {
-		    nchunks *= 2;
-		    *getbuf = myrealloc(*getbuf, nchunks * GRETL_BUFSIZE);
-		    if (*getbuf == NULL) {
-			return -2;
-		    }
-		    allocated = nchunks * GRETL_BUFSIZE;
-		}
-		memcpy(*getbuf + *len, cbuf, res);
-	    } else {
-		if (fwrite(cbuf, 1, res, fp) < (unsigned) res) {
-		    return -2;
-		}
-	    }
-	    *len += res;
-
-	    if (show) {
-		sp_ret = show_progress(res, expected, SP_NONE);
-		if (sp_ret == SP_RETURN_DONE || sp_ret == SP_RETURN_CANCELED) {
-		    break;
-		}
-	    }
-	}
-    } while (res > 0);
-
-    if (res < -1) res = -1;
-
-    if (show) {
-	show_progress(0, expected, SP_FINISH);
-#ifndef UPDATER
-	close_plugin(handle);
-#endif
-    }
-
- canceled:
-
-    if (sp_ret == SP_RETURN_CANCELED) {
-	fprintf(stderr, "Got SP_RETURN_CANCELED\n");
-	res = -2;
-    }
-
-    return res;
-}
-
 static int store_hostaddress (unsigned char *where, const char *hostname)
 {
     unsigned long addr = (unsigned long) inet_addr(hostname);
@@ -1421,9 +1378,10 @@ static int store_hostaddress (unsigned char *where, const char *hostname)
 # define ECONNREFUSED WSAECONNREFUSED
 #endif
 
-static uerr_t make_connection (int *sock, char *hostname, unsigned short port)
 /* Create an internet connection to HOSTNAME on PORT.  The created
-   socket will be stored to *SOCK.  */
+   socket will be stored to *sock.  */
+
+static uerr_t make_connection (int *sock, char *hostname, unsigned short port)
 {
     struct sockaddr_in sock_name;
 
@@ -1502,6 +1460,12 @@ static char *print_option (int opt)
 	return "GRAB_FILE";
     case QUERY:
 	return "QUERY";
+    case LIST_FUNCS:
+	return "LIST_FUNCS";
+    case GRAB_FUNC:
+	return "GRAB_FUNC";
+    case UPLOAD:
+	return "UPLOAD";
     default:
 	break;
     }
@@ -1518,6 +1482,8 @@ static int get_update_info (char **saver, char *errbuf, time_t filedate,
     const char *cgi = "/gretl/cgi-bin/gretl_update.cgi";
     struct urlinfo *proxy = NULL; 
 
+    *saver = NULL;
+
     if (use_proxy) {
 	proxy = &gretlproxy;
     }
@@ -1528,8 +1494,9 @@ static int get_update_info (char **saver, char *errbuf, time_t filedate,
     }
 
     u->path = mymalloc(strlen(cgi) + 64);
-
-    if (u->path == NULL) {
+    u->getbuf = calloc(2048, 1);
+    if (u->path == NULL || u->getbuf == NULL) {
+	free(u->getbuf);
 	freeurl(u, 0);
 	return 1;
     }
@@ -1541,6 +1508,7 @@ static int get_update_info (char **saver, char *errbuf, time_t filedate,
 #endif
 
     if (err) {
+	free(u->getbuf);
 	freeurl(u, 0);
 	return err;
     }
@@ -1556,13 +1524,12 @@ static int get_update_info (char **saver, char *errbuf, time_t filedate,
     strcat(u->path, datestr);
 
     u->saveopt = SAVE_TO_BUFFER;
-    u->savebuf = saver;
 
     result = http_loop(u, &dt, proxy); 
 
 #if WDEBUG
-    fprintf(stderr, "http_loop returned %d, u->errbuf='%s'\n",
-	    (int) result, u->errbuf);
+    fprintf(stderr, "http_loop returned %d (RETROK=%d), u->errbuf='%s'\n",
+	    (int) result, RETROK, u->errbuf);
 #endif
 
     if (result == RETROK) {
@@ -1572,6 +1539,8 @@ static int get_update_info (char **saver, char *errbuf, time_t filedate,
 	err = 1;
     }
 
+    *saver = u->getbuf;
+
     freeurl(u, 0);
 
     return err;
@@ -1680,7 +1649,7 @@ static int real_update_query (int queryopt)
     struct stat fbuf;
     time_t filedate = (time_t) 0;
 
-    build_path(paths.gretldir, "gretl.stamp", testfile, NULL);
+    build_path(testfile, paths.gretldir, "gretl.stamp", NULL);
 
     if (stat(testfile, &fbuf)) {
 	fprintf(stderr, "update_query: couldn't stat testfile '%s'\n", 
@@ -1692,7 +1661,7 @@ static int real_update_query (int queryopt)
 	*hometest = '\0';
 	if (getuid() != fbuf.st_uid) { 
 	    /* user is not owner of gretl.stamp */
-	    build_path(paths.userdir, "gretl.stamp", hometest, NULL);
+	    build_path(hometest, paths.userdir, "gretl.stamp", NULL);
 	    if (!stat(hometest, &fbuf)) {
 		filedate = get_time_from_stamp_file(hometest);
 	    }
@@ -1707,11 +1676,6 @@ static int real_update_query (int queryopt)
 	return 1;
     }
 
-    getbuf = malloc(2048); 
-    if (getbuf == NULL) return E_ALLOC;
-
-    clear(getbuf, 2048);
-
     err = get_update_info(&getbuf, errbuf, filedate, queryopt);
 
     if (err || getbuf == NULL) return 1;
@@ -1812,7 +1776,7 @@ static int open_local_file (struct urlinfo *u)
 #ifdef USE_G_FOPEN
 	u->fp = g_fopen(u->localfile, "wb");
 #else
-	u->fp = fopen(u->localfile, "wb");
+	u->fp = fopen(u->localfile, "wb"); /* FIXME? */
 #endif
 	if (u->fp == NULL) {
 	    fprintf(stderr, "Couldn't open local file '%s'\n", u->localfile);
@@ -1823,15 +1787,15 @@ static int open_local_file (struct urlinfo *u)
     return err;
 }
 
-/* grab data from URL.  If saveopt = SAVE_TO_FILE then data is stored to
-   a local file whose name is given by "savefile".  If saveopt = SAVE_TO_BUFFER
-   then "savebuf" is presumed to point to a char buffer to which the data
-   should be written.
+/* grab data from URL.  If saveopt = SAVE_TO_FILE then data is stored
+   to a local file whose name is given by "savefile".  If saveopt =
+   SAVE_TO_BUFFER then "getbuf" is presumed to point to a char buffer
+   to which the data should be written.
 */
 
 static int 
 retrieve_url (int opt, const char *fname, const char *dbseries, 
-	      int saveopt, const char *savefile, char **savebuf,
+	      int saveopt, const char *savefile, char **getbuf,
 	      char *errbuf)
 {
     uerr_t result;
@@ -1843,6 +1807,10 @@ retrieve_url (int opt, const char *fname, const char *dbseries,
     int dt, err = 0;
     size_t fnlen = 0L;
 
+    if (getbuf != NULL) {
+	*getbuf = NULL;
+    }
+
     if (use_proxy) {
 	proxy = &gretlproxy;
     }
@@ -1851,8 +1819,7 @@ retrieve_url (int opt, const char *fname, const char *dbseries,
 	fnlen = strlen(fname);
     }
 
-    if (opt == GRAB_FILE) cgi = updatecgi;
-    else cgi = datacgi;
+    cgi = (opt == GRAB_FILE)? updatecgi : datacgi;
 
     u = newurl();
     if (u == NULL) {
@@ -1879,8 +1846,8 @@ retrieve_url (int opt, const char *fname, const char *dbseries,
     sprintf(u->path, "%s?opt=%s", cgi, print_option(opt));
     u->saveopt = saveopt;
 
-    if (fnlen) {
-	if (opt == GRAB_FILE) {
+    if (fnlen > 0) {
+	if (opt == GRAB_FILE || opt == GRAB_FUNC) {
 	    strcat(u->path, "&fname=");
 	} else {
 	    strcat(u->path, "&dbase=");
@@ -1895,17 +1862,19 @@ retrieve_url (int opt, const char *fname, const char *dbseries,
 
     if (saveopt == SAVE_TO_FILE) {
 	u->localfile = g_strdup(savefile);
-	u->savebuf = NULL;
 	err = open_local_file(u);
-	if (err) {
-	    freeurl(u, 0);
-	    return err;
-	}
     } else {
-	u->localfile = NULL;
-	u->savebuf = savebuf;
+	u->getbuf = calloc(GRETL_BUFSIZE, 1);
+	if (u->getbuf == NULL) {
+	    err = 1;
+	}
     }
 
+    if (err) {
+	freeurl(u, 0);
+	return err;
+    }    
+
     result = http_loop(u, &dt, proxy);
 
 #if WDEBUG
@@ -1920,6 +1889,10 @@ retrieve_url (int opt, const char *fname, const char *dbseries,
 	err = 1;
     }
 
+    if (getbuf != NULL) {
+	*getbuf = u->getbuf;
+    }    
+
     freeurl(u, err);
 
     return err;
@@ -1960,8 +1933,6 @@ int retrieve_manfile (const char *fname, const char *savefile, char *errbuf)
 
     u->saveopt = SAVE_TO_FILE;
     u->localfile = g_strdup(savefile);
-    u->savebuf = NULL;
-
     err = open_local_file(u);
     if (err) {
 	freeurl(u, 0);
@@ -1987,34 +1958,154 @@ int retrieve_manfile (const char *fname, const char *savefile, char *errbuf)
     return err;
 }
 
-#endif /* !UPDATER */
+#define url_reserved(c) (strchr(";/?:@&=+$,<>%#\t\r\n\v\0", c) != NULL)
 
-#ifdef WIN32
-
-#include <windows.h>
-#include <shellapi.h>
-
-static long GetRegKey (HKEY key, char *subkey, char *retdata)
+static int count_specials (const char *s)
 {
-    long err;
-    HKEY hkey;
+    int n = 0;
 
-    err = RegOpenKeyEx(key, subkey, 0, KEY_QUERY_VALUE, &hkey);
-
-    if (err == ERROR_SUCCESS) {
-	long datasize = MAX_PATH;
-	char data[MAX_PATH];
-
-	RegQueryValue(hkey, NULL, (LPSTR)data, &datasize);
-
-	lstrcpy(retdata, data);
-	RegCloseKey(hkey);
+    while (*s) {
+	if (url_reserved(*s) || !isprint(*s)) {
+	    n++;
+	}
+	s++;
     }
 
+    return n;
+}
+
+static char *url_encode_string (const char *s)
+{
+    char *encstr;
+    char encc[4];
+    int n;
+
+    if (s == NULL) {
+	return NULL;
+    }
+
+    n = count_specials(s);
+    if (n == 0) {
+	return gretl_strdup(s);
+    }
+
+    encstr = malloc(strlen(s) + n * 2 + 1);
+
+    if (encstr != NULL) {
+	*encstr = '\0';
+	while (*s) {
+	    if (*s == ' ') {
+		strcat(encstr, "+");
+	    } else if (url_reserved(*s) || !isprint(*s)) {
+		sprintf(encc, "%%%.2X", *s);
+		strcat(encstr, encc);
+	    } else {
+		strncat(encstr, s, 1);
+	    } 
+	    s++;
+	}
+    }
+
+    return encstr;
+}
+
+int upload_function_package (const char *login, const char *pass, 
+			     const char *fullname, char **errbuf)
+{
+    const char *cgi = "/gretl/cgi-bin/gretldata.cgi";
+    struct urlinfo *proxy = NULL; 
+    struct urlinfo *u;
+    char *ulogin = NULL;
+    char *upass = NULL;
+    char *ufname = NULL;
+    char *ubuf = NULL;
+    char *buf = NULL;
+    int dt, err = 0;
+    uerr_t result;
+
+    g_file_get_contents(fullname, &buf, NULL, NULL);
+    if (buf == NULL) {
+	return 1;
+    }
+
+    ulogin = url_encode_string(login);
+    upass = url_encode_string(pass);
+    ufname = url_encode_string(path_last_element(fullname));
+    ubuf = url_encode_string(buf);
+
+    free(buf);
+
+    if (ulogin == NULL || upass == NULL || ufname == NULL || ubuf == NULL) {
+	err = E_ALLOC;
+	goto bailout;
+    }
+
+    if (use_proxy) {
+	proxy = &gretlproxy;
+    }
+
+    u = newurl();
+    if (u == NULL) {
+	err = E_ALLOC;
+	goto bailout;
+    }
+
+    u->path = g_strdup_printf("%s?opt=UPLOAD&login=%s&pass=%s"
+			      "&fname=%s&content=%s",
+			      cgi, ulogin, upass, ufname, ubuf);
+    if (u->path == NULL) {
+	freeurl(u, 0);
+	err = E_ALLOC;
+	goto bailout;
+    }
+
+    u->getbuf = calloc(256, 1);
+    if (u->getbuf == NULL) {
+	freeurl(u, 0);
+	err = E_ALLOC;
+	goto bailout;
+    }	
+
+    err = get_host_ip(u->host, paths.dbhost);
+    if (err) {
+	freeurl(u, 0);
+	goto bailout;
+    }
+
+    u->saveopt = SAVE_TO_BUFFER;
+
+    result = http_loop(u, &dt, proxy);
+
+    *errbuf = u->getbuf;
+
+    if (result == RETROK) {
+	*u->errbuf = 0;
+    } else {
+	if (*u->errbuf) {
+	    strcpy(u->getbuf, u->errbuf);
+	} 
+	err = 1;
+    }
+
+    freeurl(u, err);
+
+ bailout:
+
+    free(ulogin);
+    free(upass);
+    free(ufname);
+    free(ubuf);
+
     return err;
 }
 
-# ifdef UPDATER
+#endif /* !UPDATER */
+
+#ifdef WIN32
+# ifdef UPDATER /* standalone program for Windows */
+
+#include <windows.h>
+#include <shellapi.h>
 
 int read_reg_val (HKEY tree, char *keyname, char *keyval)
 {
@@ -2069,43 +2160,6 @@ static void read_proxy_info (void)
 
 # endif /* UPDATER */
 
-int goto_url (const char *url)
-{
-    char key[MAX_PATH + MAX_PATH];
-    int err = 0;
-
-    if ((long) ShellExecute(NULL, "open", url, NULL, NULL, SW_SHOW) <= 32) {
-	/* if the above fails, get the .htm regkey and 
-	   look up the program */
-	if (GetRegKey(HKEY_CLASSES_ROOT, ".htm", key) == ERROR_SUCCESS) {
-	    lstrcat(key,"\\shell\\open\\command");
-	    if (GetRegKey(HKEY_CLASSES_ROOT, key, key) == ERROR_SUCCESS) {
-		char *p;
-
-		p = strstr(key, "\"%1\"");
-		if (p == NULL) {    
-		    /* so check for %1 without the quotes */
-		    p = strstr(key, "%1");
-		    if (p == NULL) {
-			/* if no parameter */
-			p = key + lstrlen(key) - 1;
-		    } else {
-			*p = '\0';    /* remove the param */
-		    }
-		} else {
-		    *p = '\0';        /* remove the param */
-		}
-
-		lstrcat(p, " ");
-		lstrcat(p, url);
-		if (WinExec(key, SW_SHOW) < 32) err = 1;
-	    }
-	}
-    }
-
-    return err;
-}
-
 #endif /* WIN32 */
 
 /* public interfaces to some of the above */
@@ -2118,9 +2172,15 @@ int list_remote_dbs (char **getbuf, char *errbuf)
 			 NULL, getbuf, errbuf);
 }
 
-int retrieve_remote_db_list (const char *dbname, 
-			     char **getbuf, 
-			     char *errbuf)
+int list_remote_function_packages (char **getbuf, char *errbuf)
+{
+    return retrieve_url (LIST_FUNCS, NULL, NULL, SAVE_TO_BUFFER, 
+			 NULL, getbuf, errbuf);
+}
+
+int retrieve_remote_db_index (const char *dbname, 
+			      char **getbuf, 
+			      char *errbuf)
 {
     return retrieve_url (GRAB_IDX, dbname, NULL, SAVE_TO_BUFFER, 
 			 NULL, getbuf, errbuf);
@@ -2135,6 +2195,14 @@ int retrieve_remote_db (const char *dbname,
 			 localname, NULL, errbuf);
 }
 
+int retrieve_remote_function_package (const char *pkgname, 
+				      const char *localname,
+				      char *errbuf)
+{
+    return retrieve_url (GRAB_FUNC, pkgname, NULL, SAVE_TO_FILE, 
+			 localname, NULL, errbuf);
+}
+
 int retrieve_remote_db_data (const char *dbname,
 			     const char *varname,
 			     char **getbuf,
@@ -2155,7 +2223,7 @@ int files_query (char **getbuf, char *errbuf, time_t filedate)
     read_proxy_info();
 #endif
     proxy_init(dbproxy);
-    return get_update_info (getbuf, errbuf, filedate, QUERY_SILENT); 
+    return get_update_info(getbuf, errbuf, filedate, QUERY_SILENT); 
 }
 
 int get_remote_file (const char *fname, char *errbuf)
@@ -2166,8 +2234,8 @@ int get_remote_file (const char *fname, char *errbuf)
     read_proxy_info();
 #endif
     proxy_init(dbproxy);
-    return retrieve_url (GRAB_FILE, fname, NULL, SAVE_TO_FILE, 
-			 fname, NULL, errbuf);
+    return retrieve_url(GRAB_FILE, fname, NULL, SAVE_TO_FILE, 
+			fname, NULL, errbuf);
 }
 
 #endif /* UPDATER */
