@@ -11,22 +11,32 @@ enum cgi_options {
     SHOW_DBS,
     GRAB_NBO_DATA,
     GRAB_FILE,
-    QUERY
+    QUERY,
+    /* elements added April 2006 */
+    LIST_FUNCS,
+    GRAB_FUNC,
+    UPLOAD
 };
 
 # ifndef UPDATER
 
 int list_remote_dbs (char **getbuf, char *errbuf);
 
-int retrieve_remote_db_list (const char *dbname, 
-			     char **getbuf, 
-			     char *errbuf);
+int list_remote_function_packages (char **getbuf, char *errbuf);
+
+int retrieve_remote_db_index (const char *dbname, 
+			      char **getbuf, 
+			      char *errbuf);
 
 int retrieve_remote_db (const char *dbname, 
 			const char *localname,
 			char *errbuf, 
 			int opt);
 
+int retrieve_remote_function_package (const char *pkgname, 
+				      const char *localname,
+				      char *errbuf);
+
 int retrieve_remote_db_data (const char *dbname,
 			     const char *varname,
 			     char **getbuf,
@@ -37,6 +47,9 @@ int retrieve_manfile (const char *fname,
 		      const char *savefile, 
 		      char *errbuf);
 
+int upload_function_package (const char *login, const char *pass, 
+			     const char *fullname, char **errbuf);
+
 int update_query (void);
 
 int silent_update_query (void);
