@@ -21,7 +21,7 @@
 #include "libset.h"
 #include "bhhh_max.h"
 
-#undef BHHH_DEBUG
+#define BHHH_DEBUG 0
 
 struct _model_info {
 
@@ -30,6 +30,7 @@ struct _model_info {
 
     int k;              /* number of parameters */
     int t1, t2;         /* starting and ending point of sample */
+    int bign;           /* total observations in current real dataset */
     int n_series;       /* number of additional series needed in the
                            likelihood and/or score calculations */
     double tol;         /* tolerance for convergence */
@@ -54,22 +55,6 @@ struct _model_info {
     MODEL *pmod;
 };
 
-#define DEFAULT_MAX_ITER 1000
-
-static int get_maxiter (void)
-{
-    char *mistr = getenv("GRETL_MAX_ITER");
-    int mi = DEFAULT_MAX_ITER;
-
-    if (mistr != NULL) {
-	if (!sscanf(mistr, "%d", &mi)) {
-	    mi = DEFAULT_MAX_ITER;
-	}
-    }
-
-    return mi;
-}
-
 /**
  * model_info_free:
  * @minfo: model info pointer.
@@ -347,16 +332,15 @@ static int *make_opg_list (int k)
     int *list;
     int i;
 
-    list = malloc((k + 2) * sizeof *list);
+    list = gretl_list_new(k + 1);
     if (list == NULL) return NULL;
 
-    list[0] = k + 1;
     list[1] = 0;  /* dep var is the constant */
     for (i=0; i<k; i++) {
 	list[i+2] = i + 1; 
     }
 
-#ifdef BHHH_DEBUG
+#if BHHH_DEBUG
     printlist(list, "OPG regression list");
 #endif
 
@@ -409,6 +393,7 @@ static int model_info_init (model_info *minfo, const double *init_coeff)
  * @k: number of parameters to be estimated.
  * @t1: starting observation of estimation range.
  * @t2: ending observation of estimation range.
+ * @bign: total obserations in current dataset.
  * @tol: tolerance for assessing convergence.
  *
  * Creates the basic information structure required by
@@ -421,7 +406,7 @@ static int model_info_init (model_info *minfo, const double *init_coeff)
  * on failure.
  */
  
-model_info *model_info_new (int k, int t1, int t2, double tol)
+model_info *model_info_new (int k, int t1, int t2, int bign, double tol)
 {
     model_info *mi;
 
@@ -438,12 +423,45 @@ model_info *model_info_new (int k, int t1, int t2, double tol)
 	mi->t1 = t1;
 	mi->t2 = t2;
 	mi->n = t2 + 1;
+	mi->bign = bign;
 	mi->tol = tol;
     }
 
     return mi;
 }
 
+static int expand_model_series (MODEL *pmod, model_info *minfo)
+{
+    double *x;
+    int t;
+
+    x = realloc(pmod->uhat, minfo->bign * sizeof *x);
+    if (x == NULL) {
+	pmod->errcode = E_ALLOC;
+    } else {
+	pmod->uhat = x;
+	for (t=minfo->t2; t<minfo->bign; t++) {
+	    pmod->uhat[t] = NADBL;
+	}
+    }
+
+    x = realloc(pmod->yhat, minfo->bign * sizeof *x);
+    if (x == NULL) {
+	pmod->errcode = E_ALLOC;
+    } else {
+	pmod->yhat = x;
+	for (t=minfo->t2; t<minfo->bign; t++) {
+	    pmod->yhat[t] = NADBL;
+	}
+    }
+
+    if (pmod->errcode == 0) {
+	pmod->full_n = minfo->bign;
+    }
+
+    return 0;
+}
+
 static int bhhh_iter_info (int iter, double *theta, int m, double ll,
 			   double steplength, PRN *prn)
 {
@@ -560,7 +578,7 @@ int bhhh_max (LL_FUNC loglik,
 
     /* zero the dataset */
     for (i=1; i<=k; i++) {
-#ifdef BHHH_DEBUG
+#if BHHH_DEBUG
 	sprintf(tinfo->varname[i], "tZ%d", i);
 #endif
 	for (t=0; t<minfo->n; t++) {
@@ -572,7 +590,7 @@ int bhhh_max (LL_FUNC loglik,
     bmod = gretl_model_new();
 
     iters = 0;
-    itermax = get_maxiter();
+    itermax = get_bhhh_maxiter();
 
     while (crit > minfo->tol && iters++ < itermax) {
 
@@ -584,24 +602,24 @@ int bhhh_max (LL_FUNC loglik,
 	    break;
 	}
 
-#ifdef BHHH_DEBUG
+#if BHHH_DEBUG
 	pprintf(prn, "Top of loop: ll = %g\n", minfo->ll);
+	pprintf(prn, "Dataset for OPG regression:\n");
+	printdata(blist, (const double **) tZ, tinfo, OPT_O, prn);
 #endif
 
 	/* BHHH via OPG regression */
-	*bmod = lsq(blist, &tZ, tinfo, OLS, OPT_A, 0.0);
+	*bmod = lsq(blist, &tZ, tinfo, OLS, OPT_A);
 	if (bmod->errcode) {
+	    fprintf(stderr, "BHHH model error code = %d\n", bmod->errcode);
 	    err = E_NOCONV;
 	    break;
 	}
 
-#ifdef BHHH_DEBUG
-	pprintf(prn, "Dataset for OPG regression:\n");
-	printdata(blist, (const double **) tZ, tinfo, OPT_O, prn);
+#if BHHH_DEBUG
 	pprintf(prn, "OLS model, OPG regression:\n");
 	printmodel(bmod, tinfo, 0, prn);
 #endif
-
 	for (i=0; i<k; i++) {
 	    delta[i] = bmod->coeff[i] * stepsize;
 	    ctemp[i] = minfo->theta[i] + delta[i];
@@ -612,7 +630,7 @@ int bhhh_max (LL_FUNC loglik,
 	/* see if we've gone up... (0 means "don't compute score") */
 	err = loglik(ctemp, X, tZ, minfo, 0); 
 
-#ifdef BHHH_DEBUG
+#if BHHH_DEBUG
 	pprintf(prn, "bhhh loop: initial ll2 = %g\n", minfo->ll2);
 #endif
 
@@ -628,7 +646,7 @@ int bhhh_max (LL_FUNC loglik,
 		ctemp[i] = minfo->theta[i] + delta[i];
 	    }
 	    err = loglik(ctemp, X, tZ, minfo, 0);
-#ifdef BHHH_DEBUG
+#if BHHH_DEBUG
 	    pprintf(prn, "bhhh loop: modified ll2 = %g\n", minfo->ll2);
 #endif
 	}
@@ -640,8 +658,8 @@ int bhhh_max (LL_FUNC loglik,
 	    minfo->theta[i] = ctemp[i];
 	}	
 
-	/* double the steplength? */
-	if (stepsize < 4.0) {
+	/* double the steplength? (was < 4.0 below) */
+	if (stepsize < 1.0) {
 	    stepsize *= 2.0;
 	}
 
@@ -652,8 +670,8 @@ int bhhh_max (LL_FUNC loglik,
     }
 
     if (crit > minfo->tol || err != 0) {
-	fprintf(stderr, "bhhh_max: crit = %g, tol = %g, err = %d\n",
-		crit, minfo->tol, err);
+	fprintf(stderr, "bhhh_max: iters = %d, crit = %g, tol = %g, err = %d\n",
+		iters, crit, minfo->tol, err);
 	err = E_NOCONV;
     }
 
@@ -675,10 +693,13 @@ int bhhh_max (LL_FUNC loglik,
 
 	    /* run OPG once more using QR, to get packed VCV */
 	    set_use_qr(1);
-	    *bmod = lsq(blist, &tZ, tinfo, OLS, OPT_A, 0.0);
+	    *bmod = lsq(blist, &tZ, tinfo, OLS, OPT_A);
 	    set_use_qr(qr_bak);
 	    minfo->pmod = bmod;
 	    gretl_model_set_int(bmod, "iters", iters);
+	    if (minfo->bign > minfo->t2) {
+		expand_model_series(bmod, minfo);
+	    }
 	} 
 	minfo->iters = iters;
     }
