@@ -23,14 +23,25 @@
 
 #include "libgretl.h"
 #include "libset.h"
-#include "gretl_matrix.h"
+#include "gretl_panel.h"
+#include "var.h"
 #include "missing_private.h"
 
-#undef WDEBUG
+#define WDEBUG 0
 
-#define print_add_omit_model(m,o) (m->ci != AR && m->ci != ARCH && \
+#define print_add_omit_model(m,o) (m->ci != ARCH && \
                                    !(opt & OPT_Q) && !(opt & OPT_I))
 
+enum {
+    CHISQ_FORM,
+    F_FORM
+};
+
+enum {
+    ADD_TEST,
+    OMIT_TEST,
+    OMIT_WALD
+};
 
 struct COMPARE {
     int cmd;       /* ADD or OMIT */
@@ -42,8 +53,41 @@ struct COMPARE {
     double F;      /* F test statistic */
     double chisq;  /* Chi-square test statistic */
     double trsq;   /* T*R^2 test statistic */
-    int score;     /* "cases correct" for discrete models */
+    int score;     /* number of info stats showing improvement */
     int robust;    /* = 1 when robust vcv is in use, else 0 */
+    int err;       /* error code */
+};
+
+/* Critical values for Quandt likelihood ratio (break) test:
+   columns contain the 10%, 5% and 1% values respectively,
+   for the given number of restrictions, q, and 15% trimming.  
+   Taken from Stock and Watson, Introduction to Econometrics
+   (Addison-Wesley, 2003), based on Andrews.
+*/
+
+#define QLR_QMAX 20
+
+static double QLR_critvals[QLR_QMAX][3] = {
+    { 7.12, 8.68, 12.16 }, /*  1 */
+    { 5.00, 5.86,  7.78 }, /*  2 */
+    { 4.09, 4.71,  6.02 }, /*  3 */
+    { 3.59, 4.09,  5.12 }, /*  4 */
+    { 3.26, 3.66,  4.53 }, /*  5 */
+    { 3.02, 3.37,  4.12 }, /*  6 */
+    { 2.84, 3.15,  3.82 }, /*  7 */
+    { 2.69, 2.98,  3.57 }, /*  8 */
+    { 2.58, 2.84,  3.38 }, /*  9 */
+    { 2.48, 2.71,  3.23 }, /* 10 */
+    { 2.40, 2.62,  3.09 }, /* 11 */
+    { 2.33, 2.54,  2.97 }, /* 12 */
+    { 2.27, 2.46,  2.87 }, /* 13 */
+    { 2.21, 2.40,  2.78 }, /* 14 */
+    { 2.16, 2.34,  2.71 }, /* 15 */
+    { 2.12, 2.29,  2.64 }, /* 16 */
+    { 2.08, 2.25,  2.58 }, /* 17 */
+    { 2.05, 2.20,  2.53 }, /* 18 */
+    { 2.01, 2.17,  2.48 }, /* 19 */
+    { 1.99, 2.13,  2.43 }  /* 20 */
 };
 
 /* Given a list of variables, check them against the independent
@@ -53,21 +97,24 @@ struct COMPARE {
 */
 
 static char *
-mask_from_test_list (const int *test, const MODEL *pmod)
+mask_from_test_list (const int *list, const MODEL *pmod, int *err)
 {
     char *mask;
+    int nmask = 0;
     int i, j;
 
     mask = calloc(pmod->ncoeff, 1);
     if (mask == NULL) {
+	*err = E_ALLOC;
 	return NULL;
     }
 
     for (i=0; i<pmod->ncoeff; i++) {
-	if (test != NULL) {
-	    for (j=1; j<=test[0]; j++) {
-		if (pmod->list[i+2] == test[j]) {
+	if (list != NULL) {
+	    for (j=1; j<=list[0]; j++) {
+		if (pmod->list[i+2] == list[j]) {
 		    mask[i] = 1;
+		    nmask++;
 		}
 	    }
 	} else {
@@ -77,14 +124,15 @@ mask_from_test_list (const int *test, const MODEL *pmod)
 	}
     }
 
+    if (list != NULL && nmask != list[0]) {
+	fprintf(stderr, "mask from list: list[0] = %d but nmask = %d\n",
+		list[0], nmask);
+	*err = E_DATA;
+    }
+
     return mask;
 }
 
-enum {
-    CHI_SQUARE_FORM,
-    F_FORM
-};
-
 /* Wald (chi-square or F) test for a set of zero restrictions on the
    parameters of a given model, based on the covariance matrix of the
    unrestricted model. Suitable for use where the original model is
@@ -92,46 +140,43 @@ enum {
    automatic test, for the significance of all vars but the constant.
 */
 
-static double wald_test (const int *list, MODEL *pmod, int form)
+static double wald_test (const int *list, MODEL *pmod, int form,
+			 int *err)
 {
     char *mask = NULL;
     gretl_matrix *C = NULL;
     gretl_vector *b = NULL;
     double w = NADBL;
-    int err = 0;
 
-    mask = mask_from_test_list(list, pmod);
-    if (mask == NULL) {
-	err = 1;
-    } 
+    mask = mask_from_test_list(list, pmod, err);
 
-    if (!err) {
+    if (!*err) {
 	C = gretl_vcv_matrix_from_model(pmod, mask);
 	if (C == NULL) {
-	    err = 1;
+	    *err = E_ALLOC;
 	} 
     }
 
-    if (!err) {
+    if (!*err) {
 	b = gretl_coeff_vector_from_model(pmod, mask);
 	if (b == NULL) {
-	    err = 1;
+	    *err = E_ALLOC;
 	} 
     }  
 
-    if (!err) {
+    if (!*err) {
 #if WDEBUG
 	gretl_matrix_print(C, "Wald VCV matrix");
 	gretl_matrix_print(b, "Wald coeff vector");
 #endif
-	err = gretl_invert_symmetric_matrix(C);
+	*err = gretl_invert_symmetric_matrix(C);
     }
 
-    if (!err) {
-	w = gretl_scalar_b_X_b(b, GRETL_MOD_TRANSPOSE, C, &err);
+    if (!*err) {
+	w = gretl_scalar_b_X_b(b, GRETL_MOD_TRANSPOSE, C, err);
     }
 
-    if (form == F_FORM && !err) {
+    if (!*err && form == F_FORM) {
 	w /= gretl_vector_get_length(b);
     }
 
@@ -163,7 +208,9 @@ static double wald_test (const int *list, MODEL *pmod, int form)
 
 double robust_omit_F (const int *list, MODEL *pmod)
 {
-    return wald_test(list, pmod, F_FORM);
+    int err = 0;
+
+    return wald_test(list, pmod, F_FORM, &err);
 }
 
 /* ----------------------------------------------------- */
@@ -208,7 +255,7 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
     int verbosity = 2;
     int stat_ok, i;
 
-    if (cmp->ci == LAD) {
+    if (cmp->err || cmp->ci == LAD) {
 	return;
     }
 
@@ -224,7 +271,7 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
 	test = model_test_new((cmp->cmd == OMIT)? GRETL_TEST_OMIT : GRETL_TEST_ADD);
     }
 
-    if (verbosity > 1) {
+    if (verbosity > 1 && cmp->m2 >= 0) {
 	pprintf(prn, _("Comparison of Model %d and Model %d:\n"), 
 		cmp->m1, cmp->m2);
     } 
@@ -238,7 +285,7 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
     }
 
     if (!na(cmp->F)) {
-	pval = fdist(cmp->F, cmp->dfn, cmp->dfd);
+	pval = f_cdf_comp(cmp->F, cmp->dfn, cmp->dfd);
 	if (verbosity > 0) {
 	    pprintf(prn, "\n  %s: %s(%d, %d) = %g, ", _("Test statistic"), 
 		    (cmp->robust)? _("Robust F") : "F",
@@ -255,7 +302,7 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
 	    model_test_set_pvalue(test, pval);
 	}
     } else if (!na(cmp->chisq)) {
-	pval = chisq(cmp->chisq, cmp->dfn);
+	pval = chisq_cdf_comp(cmp->chisq, cmp->dfn);
 	if (verbosity > 0) {
 	    pprintf(prn, "\n  %s:%s%s(%d) = %g, ",  
 		    (LIMDEP(cmp->ci))? _("Test statistic") : 
@@ -280,7 +327,7 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
 	maybe_add_test_to_model(new, test);
     }
 
-    if (verbosity > 1) {
+    if (verbosity > 1 && cmp->score >= 0) {
 	pprintf(prn, _("  Of the %d model selection statistics, %d "), 
 		C_MAX, cmp->score);
 	if (cmp->score == 1) {
@@ -292,14 +339,16 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
 }
 
 static struct COMPARE 
-add_or_omit_compare (MODEL *pmodA, MODEL *pmodB, int add,
+add_or_omit_compare (MODEL *pmodA, MODEL *pmodB, int flag,
 		     const int *testvars)
 {
     struct COMPARE cmp;
     MODEL *umod, *rmod;
     int i;	
 
-    if (add) {
+    cmp.err = 0;
+
+    if (flag == ADD_TEST) {
 	umod = pmodB;
 	rmod = pmodA;
 	cmp.cmd = ADD;
@@ -310,19 +359,29 @@ add_or_omit_compare (MODEL *pmodA, MODEL *pmodB, int add,
     }
 
     cmp.m1 = pmodA->ID;
-    cmp.m2 = pmodB->ID;
     cmp.ci = pmodA->ci;
 
     cmp.F = cmp.chisq = cmp.trsq = NADBL;
-    cmp.score = 0;
+    cmp.score = -1;
     cmp.robust = 0;
-    
-    cmp.dfn = umod->ncoeff - rmod->ncoeff;
+    cmp.dfn = 0;
+
+    if (pmodB != NULL) {
+	/* may be NULL for Wald test */
+	cmp.m2 = pmodB->ID;
+	cmp.dfn = umod->ncoeff - rmod->ncoeff;
+    } else {
+	cmp.m2 = -1;
+	cmp.dfn = testvars[0];
+    }
+
     cmp.dfd = umod->dfd;
 
     /* FIXME TSLS (F or chi-square?) */
 
-    if (gretl_model_get_int(pmodA, "robust") || pmodA->ci == HCCM) {
+    if (flag == OMIT_WALD) {
+	cmp.chisq = wald_test(testvars, umod, CHISQ_FORM, &cmp.err);
+    } else if (gretl_model_get_int(pmodA, "robust") || pmodA->ci == HCCM) {
 	cmp.F = robust_omit_F(testvars, umod);
 	cmp.robust = 1;
     } else if (LIMDEP(cmp.ci)) {
@@ -330,15 +389,18 @@ add_or_omit_compare (MODEL *pmodA, MODEL *pmodB, int add,
     } else if (cmp.ci == OLS) {
 	cmp.F = ((rmod->ess - umod->ess) / umod->ess) * cmp.dfd / cmp.dfn;
     } else if (cmp.dfn > 1) {
-	cmp.chisq = wald_test(testvars, umod, CHI_SQUARE_FORM);
+	cmp.chisq = wald_test(testvars, umod, CHISQ_FORM, &cmp.err);
     }
 
-    for (i=0; i<C_MAX; i++) { 
-	if (na(pmodB->criterion[i]) || na(pmodA->criterion[i])) {
-	    continue;
-	}
-	if (pmodB->criterion[i] < pmodA->criterion[i]) {
-	    cmp.score++;
+    if (pmodB != NULL) {
+	cmp.score = 0;
+	for (i=0; i<C_MAX; i++) { 
+	    if (na(pmodB->criterion[i]) || na(pmodA->criterion[i])) {
+		continue;
+	    }
+	    if (pmodB->criterion[i] < pmodA->criterion[i]) {
+		cmp.score++;
+	    }
 	}
     }
 
@@ -406,13 +468,14 @@ full_model_list (const MODEL *pmod, const int *inlist)
 
 static MODEL replicate_estimator (MODEL *orig, int **plist,
 				  double ***pZ, DATAINFO *pdinfo,
-				  gretlopt lsqopt, PRN *prn)
+				  gretlopt myopt, PRN *prn)
 {
     MODEL rep;
     double rho = 0.0;
     int *list = *plist;
     int mc = get_model_count();
     int repci = orig->ci;
+    int order = 0;
 
     gretl_model_init(&rep);
 
@@ -426,8 +489,7 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
 			   &rep.errcode, hlopt, prn);
     } else if (gretl_model_get_int(orig, "unit_weights")) {
 	/* panel model with per-unit weights */
-	lsqopt |= OPT_W;
-	repci = POOLED;
+	myopt |= OPT_W;
     } else if (orig->ci == WLS || orig->ci == AR || 
 	       (orig->ci == POISSON && gretl_model_get_int(orig, "offset_var"))) {
 	int *full_list = full_model_list(orig, list);
@@ -447,19 +509,20 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
     switch (orig->ci) {
 
     case AR:
-	rep = ar_func(list, pZ, pdinfo, lsqopt, prn);
+	rep = ar_func(list, pZ, pdinfo, myopt, prn);
 	break;
     case ARCH:
-	rep = arch_model(list, orig->order, pZ, pdinfo, lsqopt, prn);
+	order = gretl_model_get_int(orig, "arch_order");
+	rep = arch_model(list, order, pZ, pdinfo, myopt, prn);
 	break;
     case LOGIT:
     case PROBIT:
 	if (gretl_model_get_int(orig, "robust")) {
-	    lsqopt = OPT_R;
+	    myopt = OPT_R;
 	} else {
-	    lsqopt = OPT_NONE;
+	    myopt = OPT_NONE;
 	}
-	rep = logit_probit(list, pZ, pdinfo, orig->ci, lsqopt);
+	rep = logit_probit(list, pZ, pdinfo, orig->ci, myopt);
 	break;
     case TOBIT:
 	rep = tobit_model(list, pZ, pdinfo, NULL);
@@ -471,7 +534,7 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
 	rep = poisson_model(list, pZ, pdinfo, NULL);
 	break;
     case TSLS:
-	rep = tsls_func(list, TSLS, pZ, pdinfo, lsqopt);
+	rep = tsls_func(list, TSLS, pZ, pdinfo, myopt);
 	break;
     case LOGISTIC: 
 	{
@@ -483,18 +546,25 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
 	    rep = logistic_model(list, pZ, pdinfo, lmaxstr);
 	}
 	break;
+    case PANEL:
+	if (gretl_model_get_int(orig, "random-effects")) {
+	    myopt |= OPT_R;
+	}
+	rep = real_panel_model(list, pZ, pdinfo, myopt, NULL);
+	break;
     default:
 	/* handles OLS, WLS, HSK, HCCM, etc. */
 	if (gretl_model_get_int(orig, "robust")) {
-	    lsqopt |= OPT_R;
+	    myopt |= OPT_R;
 	}
 	if (gretl_model_get_int(orig, "hc_version") == 4) {
 	    repci = HCCM;
 	}
 	if (rho != 0.0) {
-	    rep = ar1_lsq(list, pZ, pdinfo, repci, lsqopt, rho);
+	    rep = ar1_lsq(list, pZ, pdinfo, repci, myopt, rho);
 	} else {
-	    rep = lsq(list, pZ, pdinfo, repci, lsqopt);
+	    /* FIXME panel WLS by unit? */
+	    rep = lsq(list, pZ, pdinfo, repci, myopt);
 	}
 	break;
     }
@@ -518,7 +588,7 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
 
     /* if the model count went up for an aux regression,
        bring it back down */
-    if (be_quiet(lsqopt) && get_model_count() > mc) {
+    if (be_quiet(myopt) && get_model_count() > mc) {
 	model_count_minus();
     }
 
@@ -553,7 +623,7 @@ real_nonlinearity_test (MODEL *pmod, int *list,
     } else {
 	double trsq = aux.rsq * aux.nobs;
 	int df = list[0] - pmod->list[0];
-	double pval = chisq(trsq, df);
+	double pval = chisq_cdf_comp(trsq, df);
 
 	aux.aux = aux_code;
 	printmodel(&aux, pdinfo, opt, prn);
@@ -571,7 +641,7 @@ real_nonlinearity_test (MODEL *pmod, int *list,
 		model_test_set_teststat(test, GRETL_STAT_TR2);
 		model_test_set_dfn(test, df);
 		model_test_set_value(test, trsq);
-		model_test_set_pvalue(test, chisq(trsq, df));
+		model_test_set_pvalue(test, chisq_cdf_comp(trsq, df));
 		maybe_add_test_to_model(pmod, test);
 	    }
 	}
@@ -709,7 +779,7 @@ int add_test (const int *addvars, MODEL *orig, MODEL *new,
     const int orig_nvar = pdinfo->v; 
     int err = 0;
 
-    if (orig == NULL || orig->list == NULL) {
+    if (orig == NULL || orig->list == NULL || addvars == NULL) {
 	return 1;
     }
 
@@ -726,6 +796,8 @@ int add_test (const int *addvars, MODEL *orig, MODEL *new,
     /* create augmented regression list */
     if (orig->ci == TSLS) {
 	tmplist = tsls_list_add(orig->list, addvars, opt, &err);
+    } else if (orig->ci == PANEL) {
+	tmplist = panel_list_add(orig, addvars, &err);
     } else {
 	tmplist = gretl_list_add(orig->list, addvars, &err);
     }
@@ -766,7 +838,7 @@ int add_test (const int *addvars, MODEL *orig, MODEL *new,
 	    int *addlist;
 
 	    addlist = gretl_list_diff_new(new->list, orig->list, 2);
-	    cmp = add_or_omit_compare(orig, new, 1, addlist);
+	    cmp = add_or_omit_compare(orig, new, ADD_TEST, addlist);
 
 	    gretl_make_compare(&cmp, addlist, orig, pdinfo, opt, prn);
 	    free(addlist);
@@ -785,6 +857,45 @@ int add_test (const int *addvars, MODEL *orig, MODEL *new,
     return err;
 }
 
+static int wald_omit_test (const int *list, MODEL *pmod, 
+			   const DATAINFO *pdinfo, gretlopt opt,
+			   PRN *prn)
+{
+    struct COMPARE cmp;
+    int *test = NULL;
+    int err = 0;
+
+    /* test validity of omissions */
+    test = gretl_list_omit(pmod->list, list, 2, &err);
+    if (err) {
+	return err;
+    }
+
+    free(test);
+
+    cmp = add_or_omit_compare(pmod, NULL, OMIT_WALD, list);
+    if (cmp.err) {
+	return cmp.err;
+    }
+
+    gretl_make_compare(&cmp, list, pmod, pdinfo, opt, prn);
+
+    return err;
+}
+
+static int omit_options_inconsistent (gretlopt opt)
+{
+    if ((opt & OPT_T) || (opt & OPT_B)) {
+	/* 2sls: omitting variable as instrument */
+	if (opt & OPT_W) {
+	    /* can't use Wald method on original VCV */
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
 /**
  * omit_test:
  * @omitvars: list of variables to omit from original model.
@@ -821,11 +932,19 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *new,
     int err = 0;
 
     if (orig == NULL || orig->list == NULL) {
-	return 1;
+	err = E_DATA;
+    } else if (!command_ok_for_model(OMIT, orig->ci)) {
+	err = E_NOTIMP;
+    } else if (omit_options_inconsistent(opt)) {
+	err = E_BADOPT;
     }
 
-    if (!command_ok_for_model(OMIT, orig->ci)) {
-	return E_NOTIMP;
+    if (err) {
+	return err;
+    }
+
+    if (opt & OPT_W) {
+	return wald_omit_test(omitvars, orig, pdinfo, opt, prn);
     }
 
     /* check that vars to omit have not been redefined */
@@ -843,7 +962,9 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *new,
 
     /* create list for test model */
     if (orig->ci == TSLS) {
-	tmplist = tsls_list_omit(orig->list, omitvars, opt, &err);	
+	tmplist = tsls_list_omit(orig->list, omitvars, opt, &err);
+    } else if (orig->ci == PANEL) {
+	tmplist = panel_list_omit(orig, omitvars, &err);
     } else if (omitlast) {
 	/* special: just drop the last variable */
 	tmplist = gretl_list_omit_last(orig->list, &err);
@@ -871,7 +992,7 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *new,
     if (orig->ci == AR) { 
 	maxlag = orig->arinfo->arlist[orig->arinfo->arlist[0]];
     } else if (orig->ci == ARCH) {
-	maxlag = orig->order;
+	maxlag = gretl_model_get_int(orig, "arch_order");
     }
 
     pdinfo->t1 = orig->t1 - maxlag; /* FIXME: problem? */
@@ -902,11 +1023,10 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *new,
 
 	if (new->nobs == orig->nobs && !omitlast) {
 	    struct COMPARE cmp;
-	    int *omitlist;
+	    int *omitlist = NULL;
 
 	    omitlist = gretl_list_diff_new(orig->list, new->list, 2);
-	    cmp = add_or_omit_compare(orig, new, 0, omitlist);
-
+	    cmp = add_or_omit_compare(orig, new, OMIT_TEST, omitlist);
 	    gretl_make_compare(&cmp, omitlist, orig, pdinfo, opt, prn); 
 	    free(omitlist);
 	}
@@ -1044,7 +1164,7 @@ int reset_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	aux.aux = AUX_RESET;
 	printmodel(&aux, pdinfo, OPT_NONE, prn);
 	RF = ((pmod->ess - aux.ess) / 2) / (aux.ess / aux.dfd);
-	pval = fdist(RF, 2, aux.dfd);
+	pval = f_cdf_comp(RF, 2, aux.dfd);
 
 	pprintf(prn, "\n%s: F = %f,\n", _("Test statistic"), RF);
 	pprintf(prn, "%s = P(F(%d,%d) > %g) = %.3g\n", _("with p-value"), 
@@ -1110,21 +1230,7 @@ int autocorr_test (MODEL *pmod, int order,
     }
 
     if (dataset_is_panel(pdinfo)) {
-	void *handle;
-	int (*panel_autocorr_test)(MODEL *, int, 
-				   double **, DATAINFO *, 
-				   PRN *, ModelTest *);
-
-	panel_autocorr_test = get_plugin_function("panel_autocorr_test", 
-						  &handle);
-	if (panel_autocorr_test == NULL) {
-	    return 1;
-	}
-
-	err = panel_autocorr_test(pmod, order, *pZ, pdinfo,
-				  prn, NULL);
-	close_plugin(handle);
-	return err;
+	return panel_autocorr_test(pmod, order, *pZ, pdinfo, opt, prn);
     }
 
     /* impose original sample range */
@@ -1132,7 +1238,9 @@ int autocorr_test (MODEL *pmod, int order,
 
     gretl_model_init(&aux);
 
-    if (order <= 0) order = pdinfo->pd;
+    if (order <= 0) {
+	order = pdinfo->pd;
+    }
 
     if (pmod->ncoeff + order >= pdinfo->t2 - pdinfo->t1) {
 	return E_DF;
@@ -1185,11 +1293,11 @@ int autocorr_test (MODEL *pmod, int order,
 
     if (!err) {
 	aux.aux = AUX_AR;
-	aux.order = order;
+	gretl_model_set_int(&aux, "BG_order", order);
 	trsq = aux.rsq * aux.nobs;
 	LMF = (aux.rsq/(1.0 - aux.rsq)) * 
 	    (aux.nobs - pmod->ncoeff - order)/order; 
-	pval = fdist(LMF, order, aux.nobs - pmod->ncoeff - order);
+	pval = f_cdf_comp(LMF, order, aux.nobs - pmod->ncoeff - order);
 
 	if (pmod->aux != AUX_VAR) {
 	    printmodel(&aux, pdinfo, OPT_NONE, prn);
@@ -1199,11 +1307,11 @@ int autocorr_test (MODEL *pmod, int order,
 	    pprintf(prn, "\n%s: TR^2 = %f,\n", 
 		    _("Alternative statistic"), trsq);
 	    pprintf(prn, "%s = P(%s(%d) > %g) = %.3g\n\n", 	_("with p-value"), 
-		    _("Chi-square"), order, trsq, chisq(trsq, order));
+		    _("Chi-square"), order, trsq, chisq_cdf_comp(trsq, order));
 	    if (ljung_box(v, order, (const double **) *pZ, pdinfo, &lb) == 0) {
 		pprintf(prn, "Ljung-Box Q' = %g %s = P(%s(%d) > %g) = %.3g\n", 
 			lb, _("with p-value"), _("Chi-square"), order,
-			lb, chisq(lb, order));
+			lb, chisq_cdf_comp(lb, order));
 	    }
 	    record_test_result(LMF, pval, _("autocorrelation"));
 	}
@@ -1234,6 +1342,179 @@ int autocorr_test (MODEL *pmod, int order,
     return err;
 }
 
+#if 0
+/* shift the Chow split-point one period */
+
+static void shift_chow_split (int v, int n, int t, double **Z)
+{
+    for (i=0; i<n; i++) {
+	Z[v+i][t] = 0.0;
+    }
+}
+#endif
+
+/* compose list of variables to be added for Chow test and add
+   them to the data set */
+
+static int *
+make_chow_list (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
+		int split, int *err)
+{
+    int *chowlist = NULL;
+    int newvars = pmod->ncoeff;
+    int i, t, v = pdinfo->v;
+
+    if (dataset_add_series(newvars, pZ, pdinfo)) {
+	*err = E_ALLOC;
+    } else {
+	chowlist = gretl_list_new(pmod->list[0] + newvars);
+	if (chowlist == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    if (!*err) {
+	double *dum = (*pZ)[v];
+	int l0 = pmod->list[0];
+
+	for (i=1; i<=l0; i++) { 
+	    chowlist[i] = pmod->list[i];
+	}
+
+	/* generate the split variable */
+	for (t=0; t<pdinfo->n; t++) {
+	    dum[t] = (double) (t >= split); 
+	}
+
+	strcpy(pdinfo->varname[v], "splitdum");
+	strcpy(VARLABEL(pdinfo, v), _("dummy variable for Chow test"));
+	chowlist[l0 + 1] = v;
+
+	/* and the interaction terms */
+	for (i=1; i<newvars; i++) {
+	    int pv = pmod->list[i + 1 + pmod->ifc];
+	    int sv = v + i;
+
+	    for (t=0; t<pdinfo->n; t++) {
+		if (t < split) {
+		    (*pZ)[sv][t] = 0.0;
+		} else if (model_missing(pmod, t)) {
+		    (*pZ)[sv][t] = NADBL;
+		} else {
+		    (*pZ)[sv][t] = (*pZ)[pv][t];
+		}
+	    }
+
+	    strcpy(pdinfo->varname[sv], "sd_");
+	    strncat(pdinfo->varname[sv], pdinfo->varname[pv], VNAMELEN - 4);
+	    sprintf(VARLABEL(pdinfo, sv), "splitdum * %s", pdinfo->varname[pv]);
+	    chowlist[l0 + 1 + i] = sv;
+	}
+    }
+
+    return chowlist;
+}
+
+static void save_QLR_test (MODEL *pmod, char *datestr,
+			   double Fmax, double crit, double alpha,
+			   int dfn, int dfd)
+{
+    ModelTest *test = model_test_new(GRETL_TEST_QLR);
+
+    if (test != NULL) {
+	model_test_set_teststat(test, GRETL_STAT_SUP_WALD);
+	model_test_set_param(test, datestr);
+	model_test_set_value(test, Fmax);
+	model_test_set_crit_and_alpha(test, crit, alpha);
+	model_test_set_dfn(test, dfn);
+	model_test_set_dfd(test, dfd);
+	maybe_add_test_to_model(pmod, test);
+    }	  
+}
+
+static void QLR_print_result (MODEL *pmod,
+			      double Fmax, int tmax, int dfn, int dfd,
+			      const DATAINFO *pdinfo, gretlopt opt,
+			      PRN *prn)
+{
+    char datestr[OBSLEN];
+    double crit = 0.0;
+    int a = 0, approx = 0;
+    int i, j;
+
+    ntodate(datestr, tmax, pdinfo);
+
+    pputs(prn, _("Quandt likelihood ratio test for structural break at an "
+	  "unknown point,\nwith 15 percent trimming"));
+    pputs(prn, ":\n\n");
+    pprintf(prn, _("The maximum F(%d, %d) = %g occurs "
+	    "at observation %s"), dfn, dfd, Fmax, datestr);
+    pputc(prn, '\n');
+
+    j = dfn - 1;
+
+    if (j >= QLR_QMAX) {
+	j = QLR_QMAX - 1;
+	approx = 1;
+    } 
+
+    for (i=2; i>=0; i--) {
+	if (Fmax > QLR_critvals[j][i]) {
+	    a = (i == 2)? 1 : (i == 1)? 5 : 10;
+	    crit = QLR_critvals[j][i];
+	    break;
+	}
+    }
+
+    if (crit == 0.0) {
+	crit = QLR_critvals[j][0];
+    }
+
+    if (a > 0) {
+	pprintf(prn, _("Significant at the %d percent level "), a);
+	pprintf(prn, _("(%d%% critical value %s %.2f)"), a, 
+		(approx)? "<" : "=", crit);
+    } else {
+	if (approx) {
+	    pprintf(prn, _("10%% critical value for q = 20 is %.2f"),
+		    crit);
+	} else {
+	    pputs(prn, _("Not significant at the 10 percent level "));
+	    pprintf(prn, _("(10%% value = %.2f)"), crit);
+	}
+	a = 10;
+    }
+
+    pputs(prn, "\n\n");
+    pputs(prn, _("This statistic does not follow the standard "
+	  "F distribution;\ncritical values are from Stock and Watson "
+	  "(2003)."));
+    pputs(prn, "\n\n");
+	  
+
+    if (opt & OPT_S) {
+	save_QLR_test(pmod, datestr, Fmax, crit, a / 100.0,
+		      dfn, dfd);
+    }
+}
+
+static void save_chow_test (MODEL *pmod, char *chowdate,
+			    double F, double pval,
+			    int dfn, int dfd)
+{
+    ModelTest *test = model_test_new(GRETL_TEST_CHOW);
+
+    if (test != NULL) {
+	model_test_set_teststat(test, GRETL_STAT_F);
+	model_test_set_param(test, chowdate);
+	model_test_set_value(test, F);
+	model_test_set_pvalue(test, pval);
+	model_test_set_dfn(test, dfn);
+	model_test_set_dfd(test, dfd);
+	maybe_add_test_to_model(pmod, test);
+    }	  
+}
+
 /**
  * chow_test:
  * @line: command line for parsing.
@@ -1254,12 +1535,13 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
     int smpl_t1 = pdinfo->t1;
     int smpl_t2 = pdinfo->t2;
     int *chowlist = NULL;
-    int newvars = pmod->list[0] - 1;
-    int i, t, v = pdinfo->v;
-    char chowdate[OBSLEN], s[OBSLEN];
+    int origv = pdinfo->v;
+    char chowdate[OBSLEN];
     MODEL chow_mod;
     double F;
-    int split = 0, err = 0;
+    int QLR = 0;
+    int split, smax;
+    int err = 0;
 
     if (pmod->ci != OLS) {
 	return E_OLSONLY;
@@ -1272,94 +1554,82 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
     gretl_model_init(&chow_mod);
 
     if (sscanf(line, "%*s %8s", chowdate) != 1) {
-	err = E_PARSE;
+	QLR = 1;
+	/* 15 percent trimming */
+	split = pmod->t1 + 0.15 * pmod->nobs;
+	smax = pmod->t1 + 0.85 * pmod->nobs;
     } else {
-	split = dateton(chowdate, pdinfo) - 1;
+	split = dateton(chowdate, pdinfo);
 	if (split <= 0 || split >= pdinfo->n) { 
 	    err = E_SPLIT;
 	}
+	smax = split;
     }
 
     if (!err) {
-	/* take the original regression list, add a split dummy
-	   and interaction terms. */
-	if (pmod->ifc == 0) {
-	    newvars++;
-	}
-	if (dataset_add_series(newvars, pZ, pdinfo)) {
-	    newvars = 0;
-	    err = E_ALLOC;
-	} else {
-	    chowlist = malloc((pmod->list[0] + newvars + 1) * sizeof *chowlist);
-	    if (chowlist == NULL) {
-		err = E_ALLOC;
-	    }
-	}
+	chowlist = make_chow_list(pmod, pZ, pdinfo, split, &err);
     }
 
-    if (!err) {
-	chowlist[0] = pmod->list[0] + newvars;
-	for (i=1; i<=pmod->list[0]; i++) { 
-	    chowlist[i] = pmod->list[i];
-	}
-
-	/* generate the split variable */
-	for (t=0; t<pdinfo->n; t++) {
-	    (*pZ)[v][t] = (double) (t > split); 
-	}
-	strcpy(pdinfo->varname[v], "splitdum");
-	strcpy(VARLABEL(pdinfo, v), _("dummy variable for Chow test"));
-	chowlist[pmod->list[0] + 1] = v;
-
-	/* and the interaction terms */
-	for (i=1; i<newvars; i++) {
-	    int orig = i + 1 + pmod->ifc;
-	    int plo = pmod->list[orig];
-
-	    for (t=0; t<pdinfo->n; t++) {
-		if (model_missing(pmod, t)) {
-		    (*pZ)[v+i][t] = NADBL;
-		} else {
-		    (*pZ)[v+i][t] = (*pZ)[v][t] * (*pZ)[plo][t];
-		}
+    if (!err && QLR) {
+	/* Quandt likelihood ratio */
+	double Fmax = 0.0;
+	int dfn = 0, dfd = 0;
+	int tmax = 0;
+	int i, t;
+	
+	for (t=split; t<=smax; t++) {
+	    chow_mod = lsq(chowlist, pZ, pdinfo, OLS, OPT_A);
+	    if (chow_mod.errcode) {
+		err = chow_mod.errcode;
+		errmsg(err, prn);
+		break;
+	    }
+	    dfn = chow_mod.ncoeff - pmod->ncoeff;
+	    dfd = chow_mod.dfd;
+	    F = (pmod->ess - chow_mod.ess) * dfd / (chow_mod.ess * dfn);
+	    if (F > Fmax) {
+		tmax = t;
+		Fmax = F;
+	    }
+#if 0
+	    fprintf(stderr, "split at t=%d: F(%d,%d)=%g\n", t, 
+		    dfn, dfd, F);
+	    fprintf(stderr, " pmod->ess = %g, chow_mod.ess = %g\n", 
+		    pmod->ess, chow_mod.ess);
+#endif
+	    clear_model(&chow_mod);
+	    for (i=0; i<pmod->ncoeff; i++) {
+		(*pZ)[origv+i][t] = 0.0;
 	    }
-	    strcpy(s, pdinfo->varname[plo]); 
-	    gretl_trunc(s, 5);
-	    strcpy(pdinfo->varname[v+i], "sd_");
-	    strcat(pdinfo->varname[v+i], s);
-	    sprintf(VARLABEL(pdinfo, v+i), "splitdum * %s", pdinfo->varname[plo]);
-	    chowlist[pmod->list[0] + 1 + i] = v + i;
 	}
 
+	if (!err) {
+	    QLR_print_result(pmod, Fmax, tmax, dfn, dfd, pdinfo, opt, prn);
+	    record_test_result(Fmax, NADBL, "QLR");
+	}
+
+    } else if (!err) {
+	/* regular Chow test */
 	chow_mod = lsq(chowlist, pZ, pdinfo, OLS, OPT_A);
 
 	if (chow_mod.errcode) {
 	    err = chow_mod.errcode;
 	    errmsg(err, prn);
 	} else {
+	    int dfn = chow_mod.ncoeff - pmod->ncoeff;
 	    double pval;
 
 	    chow_mod.aux = AUX_CHOW;
 	    printmodel(&chow_mod, pdinfo, OPT_NONE, prn);
 	    F = (pmod->ess - chow_mod.ess) * chow_mod.dfd / 
-		(chow_mod.ess * newvars);
-	    pval = fdist(F, newvars, chow_mod.dfd);
+		(chow_mod.ess * dfn);
+	    pval = f_cdf_comp(F, dfn, chow_mod.dfd);
 	    pprintf(prn, _("\nChow test for structural break at observation %s:\n"
 		    "  F(%d, %d) = %f with p-value %f\n\n"), chowdate,
-		    newvars, chow_mod.dfd, F, pval);
-
+		    dfn, chow_mod.dfd, F, pval);
+	    
 	    if (opt & OPT_S) {
-		ModelTest *test = model_test_new(GRETL_TEST_CHOW);
-
-		if (test != NULL) {
-		    model_test_set_teststat(test, GRETL_STAT_F);
-		    model_test_set_param(test, chowdate);
-		    model_test_set_dfn(test, newvars);
-		    model_test_set_dfd(test, chow_mod.dfd);
-		    model_test_set_value(test, F);
-		    model_test_set_pvalue(test, pval);
-		    maybe_add_test_to_model(pmod, test);
-		}	  
+		save_chow_test(pmod, chowdate, F, pval, dfn, chow_mod.dfd);
 	    }
 
 	    record_test_result(F, pval, "Chow");
@@ -1368,7 +1638,7 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
     }
 
     /* clean up extra variables */
-    dataset_drop_last_variables(newvars, pZ, pdinfo);
+    dataset_drop_last_variables(pdinfo->v - origv, pZ, pdinfo);
     free(chowlist);
 
     pdinfo->t1 = smpl_t1;
@@ -1480,7 +1750,7 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 		}
 		cresid[j] = (*pZ)[yno][t] - yy;
 		cum_mod.ci = CUSUM;
-		makevcv(&cum_mod);
+		makevcv(&cum_mod, 1.0);
 		xx = vprime_M_v(xvec, cum_mod.vcv, cum_mod.ncoeff);
 		cresid[j] /= sqrt(1.0 + xx);
 #if 0
@@ -1593,7 +1863,7 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 }
 
 /**
- * hausman_test:
+ * panel_hausman_test:
  * @pmod: pointer to model to be tested.
  * @pZ: pointer to data matrix.
  * @pdinfo: information on the data set.
@@ -1603,13 +1873,12 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
  * Tests the given pooled model for fixed and random effects.
  * 
  * Returns: 0 on successful completion, error code on error.
- *
  */
 
-int hausman_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, 
-		  gretlopt opt, PRN *prn) 
+int panel_hausman_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, 
+			gretlopt opt, PRN *prn) 
 {
-    if (pmod->ci != POOLED) {
+    if (pmod->ci != OLS || !dataset_is_panel(pdinfo)) {
 	pputs(prn, _("This test is only relevant for pooled models\n"));
 	return 1;
     }
@@ -1619,23 +1888,15 @@ int hausman_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	return 1;
     }
 
-    if (!balanced_panel(pdinfo)) {
+    if (!balanced_panel(pdinfo)) { /* ?? */
 	pputs(prn, _("Sorry, can't do this test on an unbalanced panel.\n"
 		"You need to have the same number of observations\n"
 		"for each cross-sectional unit"));
 	return 1;
     } else {
-	void *handle;
-	void (*panel_diagnostics)(MODEL *, double ***, DATAINFO *, 
-				  gretlopt, PRN *);
-
-	panel_diagnostics = get_plugin_function("panel_diagnostics", &handle);
-	if (panel_diagnostics == NULL) {
-	    return 1;
-	}
-	(*panel_diagnostics) (pmod, pZ, pdinfo, opt, prn);
-	close_plugin(handle);
+	panel_diagnostics(pmod, pZ, pdinfo, opt, prn);
     }
+
     return 0;
 }
 
@@ -1665,8 +1926,7 @@ int add_leverage_values_to_dataset (double ***pZ, DATAINFO *pdinfo,
     if (flags & SAVE_DFFITS) addvars++;
 
     if (dataset_add_series(addvars, pZ, pdinfo)) {
-	strcpy(gretl_errmsg, _("Out of memory adding series"));
-	return 1;
+	return E_ALLOC;
     }
 
     t1 = gretl_matrix_get_int(m);
@@ -1756,7 +2016,8 @@ int leverage_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 {
     void *handle;
     gretl_matrix *(*model_leverage) (const MODEL *, double ***, 
-				     const DATAINFO *, PRN *, int);
+				     const DATAINFO *, gretlopt,
+				     PRN *, int *);
     gretl_matrix *m;
     int err = 0;
 
@@ -1767,19 +2028,17 @@ int leverage_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	return 1;
     }
 
-    m = (*model_leverage)(pmod, pZ, pdinfo, prn, 0);
-    if (m == NULL) {
-	err = 1;
-    } else {
-	if (opt & OPT_S) {
-	    err = add_leverage_values_to_dataset(pZ, pdinfo, m, 
-						 SAVE_LEVERAGE |
-						 SAVE_INFLUENCE| 
-						 SAVE_DFFITS);
-	}
-	gretl_matrix_free(m);
+    m = (*model_leverage)(pmod, pZ, pdinfo, OPT_NONE, prn, &err);
+
+    if (!err && (opt & OPT_S)) {
+	err = add_leverage_values_to_dataset(pZ, pdinfo, m, 
+					     SAVE_LEVERAGE |
+					     SAVE_INFLUENCE| 
+					     SAVE_DFFITS);
     }
 
+    gretl_matrix_free(m);
+
     close_plugin(handle);
 
     return err;
@@ -1823,107 +2082,6 @@ int vif_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, PRN *prn)
     return err;
 }
 
-int make_mp_lists (const int *list, const char *str,
-		   int **reglist, int **polylist)
-{
-    int i, pos;
-
-    pos = atoi(str);
-
-    *reglist = malloc(pos * sizeof **polylist);
-    *polylist = malloc((list[0] - pos + 2) * sizeof **reglist);
-
-    if (*reglist == NULL || *polylist == NULL) {
-	free(*reglist);
-	free(*polylist);
-	return 1;
-    }
-    
-    (*reglist)[0] = pos - 1;
-    for (i=1; i<pos; i++) {
-	(*reglist)[i] = list[i];
-    }
-
-    (*polylist)[0] = list[0] - pos;
-    for (i=1; i<=(*polylist)[0]; i++) {
-	(*polylist)[i] = list[i+pos];
-    }
-
-    return 0;
-}
-
-/**
- * mp_ols:
- * @list: specification of variables to use
- * @pos: string rep. of integer position in list at which
- * the regular list of variables ends and a list of polynomial
- * terms begins (or empty string in case of no polynomial terms)
- * @pZ: pointer to data matrix.
- * @pdinfo: information on the data set.
- * @prn: gretl printing struct.
- * 
- * 
- * Returns: 0 on successful completion, error code on error.
- */
-
-int mp_ols (const int *list, const char *pos,
-	    double ***pZ, DATAINFO *pdinfo, 
-	    PRN *prn) 
-{
-    void *handle = NULL;
-    int (*mplsq)(const int *, const int *, double ***, 
-		 DATAINFO *, PRN *, char *, mp_results *);
-    const int *reglist = NULL;
-    int *polylist = NULL, *tmplist = NULL;
-    mp_results *mpvals = NULL;
-    int nc, err = 0;
-
-    mplsq = get_plugin_function("mplsq", &handle);
-    if (mplsq == NULL) return 1;
-
-    if (!err && *pos) { 
-	/* got a list of polynomial terms? */
-	err = make_mp_lists(list, pos, &tmplist, &polylist);
-	if (err) {
-	    pputs(prn, _("Failed to parse mp_ols command\n"));
-	}
-	reglist = tmplist;
-    } 
-
-    if (!err && !*pos) {
-	reglist = list;
-    }
-
-    nc = list[0] - 1;
-    if (polylist != NULL) {
-	nc--;
-    }
-
-    mpvals = gretl_mp_results_new(nc);
-
-    if (mpvals == NULL || allocate_mp_varnames(mpvals)) {
-	pprintf(prn, "%s\n", _("Out of memory!"));
-	err = 1;
-    }
-
-    if (!err) {
-	err = (*mplsq)(reglist, polylist, pZ, pdinfo, prn, 
-		       gretl_errmsg, mpvals); 
-    }
-
-    if (!err) {
-	print_mpols_results(mpvals, pdinfo, prn);
-    }
-
-    close_plugin(handle);
-
-    free(polylist);
-    free(tmplist);
-    free_gretl_mp_results(mpvals);
-
-    return err;
-}
-
 static int varmatch (const int *sumvars, int test)
 {
     int j;
@@ -2085,3 +2243,104 @@ int sum_test (const int *sumvars, MODEL *pmod,
 
     return err;
 }
+
+/**
+ * lmtest_driver:
+ * @param: auxiliary parameter for some uses.
+ * @pZ: pointer to data matrix.
+ * @pdinfo: information on the data set.
+ * @opt: controls which test(s) will be performed.
+ * @prn: gretl printing struct.
+ * 
+ * Performs some subset of gretl's "lmtest" tests on the
+ * model last estimated, and prints the results to @prn.
+ * 
+ * Returns: 0 on successful completion, error code on error.
+ */
+
+int lmtest_driver (const char *param, 
+		   double ***pZ, DATAINFO *pdinfo, 
+		   gretlopt opt, PRN *prn)
+{
+    GretlObjType type;
+    void *ptr;
+    int k = 0;
+    int err = 0;
+
+    if (opt == OPT_NONE) {
+	pprintf(prn, "lmtest: no options selected\n");
+	return 0;
+    }
+
+    ptr = get_last_model(&type);  
+    if (ptr == NULL) {
+	return E_DATA;
+    }
+
+    if ((opt & OPT_A) || (opt & OPT_H)) {
+	k = atoi(param);
+    }
+
+    /* non-linearity (squares) */
+    if (!err && (opt & OPT_S)) {
+	if (type == GRETL_OBJ_EQN) {
+	    err = nonlinearity_test(ptr, pZ, pdinfo, 
+				    AUX_SQ, OPT_NONE, prn);
+	} else {
+	    err = E_NOTIMP;
+	}
+    }
+
+    /* non-linearity (logs) */
+    if (!err && (opt & OPT_L)) {
+	if (type == GRETL_OBJ_EQN) {
+	    err = nonlinearity_test(ptr, pZ, pdinfo, 
+				    AUX_LOG, OPT_NONE, prn);
+	} else {
+	    err = E_NOTIMP;
+	}
+    }
+
+    /* heteroskedasticity, White */
+    if (!err && (opt & OPT_W)) {
+	if (type == GRETL_OBJ_EQN) {
+	    err = whites_test(ptr, pZ, pdinfo, OPT_NONE, prn);
+	} else {
+	    err = E_NOTIMP;
+	}
+    }
+
+    /* autocorrelation */
+    if (!err && (opt & OPT_A)) {
+	if (type == GRETL_OBJ_EQN) {
+	    err = autocorr_test(ptr, k, pZ, pdinfo, 
+				OPT_NONE, prn);
+	} else if (type == GRETL_OBJ_VAR) {
+	    err = gretl_VAR_autocorrelation_test(ptr, k, pZ, pdinfo, prn);
+	} else {
+	    err = E_NOTIMP;
+	}
+    }
+
+    /* ARCH */
+    if (!err && (opt & OPT_H)) {
+	if (type == GRETL_OBJ_EQN) {
+	    err = arch_test_simple(ptr, k, pZ, pdinfo, OPT_NONE, prn);
+	} else if (type == GRETL_OBJ_VAR) {
+	    err = gretl_VAR_arch_test(ptr, k, pZ, pdinfo, prn);
+	} else {
+	    err = E_NOTIMP;
+	}
+    }    
+
+    /* groupwise heteroskedasticity */
+    if (!err && (opt & OPT_P)) {
+	if (type == GRETL_OBJ_EQN) {
+	    err = groupwise_hetero_test(ptr, pZ, pdinfo, prn);
+	} else {
+	    err = E_NOTIMP;
+	}
+    }
+
+    return err;
+}
