@@ -37,7 +37,6 @@ typedef enum {
     AUX_CHOW,  /* aux. regression for Chow test */
     AUX_ADD,   /* aux. regression for adding variables */
     AUX_AR,    /* aux. regression for autocorrelation test */
-    AUX_SCR,   /* regression showing serial correlation-robust std errs */
     AUX_WHITE, /* aux. regression for heteroskedasticity (White's test) */
     AUX_COINT, /* aux. regression for cointegreation test */
     AUX_ARCH,  /* aux. regression for ARCH test */
@@ -84,9 +83,9 @@ int cusum_test (MODEL *pmod,
 		double ***pZ, DATAINFO *pdinfo, 
 		gretlopt opt, PRN *prn);
 
-int hausman_test (MODEL *pmod, 
-		  double ***pZ, DATAINFO *pdinfo, 
-		  gretlopt opt, PRN *prn);
+int panel_hausman_test (MODEL *pmod, 
+			double ***pZ, DATAINFO *pdinfo, 
+			gretlopt opt, PRN *prn);
 
 int vif_test (MODEL *pmod, 
 	      double ***pZ, DATAINFO *pdinfo, 
@@ -99,11 +98,10 @@ int leverage_test (MODEL *pmod,
 int add_leverage_values_to_dataset (double ***pZ, DATAINFO *pdinfo,
 				    gretl_matrix *m, unsigned char flags);
 
-int mp_ols (const int *list, const char *pos,
-	    double ***pZ, DATAINFO *pdinfo, 
-	    PRN *prn); 
-
 int sum_test (const int *sumvars, MODEL *pmod, 
 	      double ***pZ, DATAINFO *pdinfo, 
 	      PRN *prn);
 
+int lmtest_driver (const char *param,
+		   double ***pZ, DATAINFO *pdinfo, 
+		   gretlopt opt, PRN *prn);
