@@ -22,12 +22,13 @@
 #include "dbwrite.h"
 #include "libset.h"
 #include "gretl_xml.h"
+#include "gretl_panel.h"
 
 #include <ctype.h>
 #include <time.h>
 #include <errno.h>
 
-#ifdef USE_GTK2
+#ifdef USE_GLIB2
 # include <glib.h>
 #endif
 
@@ -40,7 +41,6 @@ static int writelbl (const char *lblfile, const int *list,
 		     const DATAINFO *pdinfo);
 static int writehdr (const char *hdrfile, const int *list, 
 		     const DATAINFO *pdinfo, int opt);
-static int xmlfile (const char *fname);
 static int csv_time_series_check (DATAINFO *pdinfo, PRN *prn);
 
 static char STARTCOMMENT[3] = "(*";
@@ -426,7 +426,9 @@ static int readhdr (const char *hdrfile, DATAINFO *pdinfo,
     fscanf(fp, "%s", str);
     i += skipcomments(fp, str); 
 
-    while (1) { /* find number of variables */
+    /* find number of variables */
+
+    while (1) {
         if (fscanf(fp, "%s", str) != 1) {
 	    fclose(fp);
 	    sprintf(gretl_errmsg, _("Opened header file %s\n"
@@ -552,13 +554,12 @@ static int readhdr (const char *hdrfile, DATAINFO *pdinfo,
 		strcpy(pdinfo->descrip, dbuf);
 	    }
 	    free(dbuf);
-	}
-	else if (lines < 0) {
+	} else if (lines < 0) {
 	    fprintf(stderr, I_("Failed to store data comments\n"));
 	}
 	fclose(fp);
     } 
-	
+
     return 0;
 
     varname_error:
@@ -624,8 +625,7 @@ static int get_dot_pos (const char *s)
 #define DATES_DEBUG 0
 
 static int 
-real_dateton (const char *date, const DATAINFO *pdinfo,
-	      int nolimit)
+real_dateton (const char *date, const DATAINFO *pdinfo, int nolimit)
 {
     int t, n = -1;
 
@@ -781,6 +781,20 @@ static int merge_dateton (const char *date, const DATAINFO *pdinfo)
     return real_dateton(date, pdinfo, 1);
 }
 
+static char *
+out_of_range_panel_obs (char *s, int t, const DATAINFO *pdinfo)
+{
+    int i = t / pdinfo->pd + 1;
+    int j = (t + 1) % pdinfo->pd;
+
+    if (j == 0) {
+	j = pdinfo->pd;
+    }
+
+    sprintf(s, "%d:%0*d", i, pdinfo->paninfo->olen, j);
+    return s;
+}
+
 static char *
 real_ntodate (char *datestr, int t, const DATAINFO *pdinfo, int full)
 {
@@ -815,7 +829,16 @@ real_ntodate (char *datestr, int t, const DATAINFO *pdinfo, int full)
 	x = pdinfo->sd0 + 10 * t;
 	sprintf(datestr, "%d", (int) x);
 	return datestr;
-    }	
+    } else if (pdinfo->paninfo != NULL) {
+	/* indexed panel data */
+	if (t < 0 || t >= pdinfo->n) {
+	    return out_of_range_panel_obs(datestr, t, pdinfo);
+	}
+	sprintf(datestr, "%d:%0*d", pdinfo->paninfo->unit[t] + 1,
+		pdinfo->paninfo->olen,
+		pdinfo->paninfo->period[t] + 1);
+	return datestr;
+    }
 
     x = date(t, pdinfo->pd, pdinfo->sd0);
 
@@ -1086,6 +1109,8 @@ format_from_opt_or_name (gretlopt opt, const char *fname)
 	fmt = GRETL_DATA_DB;
     } else if (opt & OPT_G) {
 	fmt = GRETL_DATA_DAT;
+    } else if (opt & OPT_J) {
+	fmt = GRETL_DATA_JM;
     }
 
     if (fmt == 0) {
@@ -1101,10 +1126,15 @@ format_from_opt_or_name (gretlopt opt, const char *fname)
     return fmt;
 }
 
-static void date_maj_min (const char *s, int *maj, int *min)
+static void date_maj_min (int t, const DATAINFO *pdinfo, int *maj, int *min)
 {
-    *maj = atoi(s);
-    s = strchr(s, ':');
+    char obs[OBSLEN];
+    char *s;
+
+    ntodate(obs, t, pdinfo);
+
+    *maj = atoi(obs);
+    s = strchr(obs, ':');
     if (s != NULL && strlen(s) > 1) {
 	*min = atoi(s + 1);
     } else {
@@ -1181,7 +1211,7 @@ int write_data (const char *fname, const int *list,
     /* write header and label files if not exporting to other formats */
     if (fmt != GRETL_DATA_R && fmt != GRETL_DATA_R_TS && 
 	fmt != GRETL_DATA_CSV && fmt != GRETL_DATA_OCTAVE &&
-	fmt != GRETL_DATA_DAT) {
+	fmt != GRETL_DATA_DAT && fmt != GRETL_DATA_JM) {
 	if (!has_suffix(datfile, ".gz")) {
 	    switch_ext(hdrfile, datfile, "hdr");
 	    switch_ext(lblfile, datfile, "lbl");
@@ -1206,7 +1236,9 @@ int write_data (const char *fname, const int *list,
 	fp = gretl_fopen(datfile, "w");
     }
 
-    if (fp == NULL) return 1;
+    if (fp == NULL) {
+	return E_FOPEN;
+    }
 
     if (fmt == GRETL_DATA_FLOAT) { 
 	/* single-precision binary */
@@ -1220,7 +1252,7 @@ int write_data (const char *fname, const int *list,
 		x = (float) Z[v][0];
 	    }
 	    for (t=0; t<n; t++) {
-		if (pdinfo->vector[v]) {
+		if (var_is_series(pdinfo, v)) {
 		    if (na(Z[v][0])) {
 			x = -999.0;
 		    } else {	
@@ -1234,7 +1266,7 @@ int write_data (const char *fname, const int *list,
 	/* double-precision binary */
 	for (i=1; i<=l0; i++) {
 	    v = list[i];
-	    if (pdinfo->vector[v]) {
+	    if (var_is_series(pdinfo, v)) {
 		fwrite(&Z[v][0], sizeof(double), n, fp);
 	    } else {
 		for (t=0; t<n; t++) {
@@ -1245,7 +1277,8 @@ int write_data (const char *fname, const int *list,
     }
 
     if (fmt == GRETL_DATA_CSV || fmt == GRETL_DATA_OCTAVE || 
-	GRETL_DATA_R || fmt == GRETL_DATA_TRAD || fmt == GRETL_DATA_DAT) { 
+	GRETL_DATA_R || fmt == GRETL_DATA_TRAD || 
+	fmt == GRETL_DATA_DAT || fmt == GRETL_DATA_JM) { 
 	/* an ASCII variant of some sort */
 	pmax = malloc(l0 * sizeof *pmax);
 	if (pmax == NULL) {
@@ -1254,7 +1287,7 @@ int write_data (const char *fname, const int *list,
 	}
 	for (i=1; i<=l0; i++) {
 	    v = list[i];
-	    if (pdinfo->vector[v]) {
+	    if (var_is_series(pdinfo, v)) {
 		pmax[i-1] = get_precision(&Z[v][pdinfo->t1], tsamp, 10);
 	    } else {
 		pmax[i-1] = GRETL_SCALAR_DIGITS;
@@ -1277,17 +1310,16 @@ int write_data (const char *fname, const int *list,
 		    fprintf(fp, "-999 ");
 		} else if (pmax[i-1] == PMAX_NOT_AVAILABLE) {
 		    fprintf(fp, "%.12g ", 
-			    (pdinfo->vector[v])? 
+			    (var_is_series(pdinfo, v))? 
 			    Z[v][t] : Z[v][0]);
 		} else {
 		    fprintf(fp, "%.*f ", pmax[i-1], 
-			    (pdinfo->vector[v])? 
+			    (var_is_series(pdinfo, v))? 
 			    Z[v][t] : Z[v][0]);
 		}
 	    }
 	    fputc('\n', fp);
 	}
-	fputc('\n', fp);
     } else if (fmt == GRETL_DATA_CSV || fmt == GRETL_DATA_R) { 
 	/* export CSV or GNU R (dataframe) */
 	char delim;
@@ -1326,7 +1358,7 @@ int write_data (const char *fname, const int *list,
 	    }
 	    for (i=1; i<=l0; i++) { 
 		v = list[i];
-		xx = (pdinfo->vector[v])? Z[v][t] : Z[v][0];
+		xx = (var_is_series(pdinfo, v))? Z[v][t] : Z[v][0];
 		if (na(xx)) {
 		    fprintf(fp, "NA");
 		} else if (pmax[i-1] == PMAX_NOT_AVAILABLE) {
@@ -1341,7 +1373,6 @@ int write_data (const char *fname, const int *list,
 		}
 	    }
 	}
-	fputc('\n', fp);
     } else if (fmt == GRETL_DATA_R_TS) {
 	char *p, datestr[OBSLEN];
 	int subper = 1;
@@ -1351,7 +1382,7 @@ int write_data (const char *fname, const int *list,
 	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	    for (i=1; i<=l0; i++) {
 		v = list[i];
-		xx = (pdinfo->vector[v])? Z[v][t] : Z[v][0];
+		xx = (var_is_series(pdinfo, v))? Z[v][t] : Z[v][0];
 		if (na(xx)) {
 		    fputs("NA", fp);
 		} else {
@@ -1397,13 +1428,12 @@ int write_data (const char *fname, const int *list,
 		    fprintf(fp, "%.12g ", Z[v][t]);
 		} else {
 		    fprintf(fp, "%.*f ", pmax[i-1], 
-			    (pdinfo->vector[v])? 
+			    (var_is_series(pdinfo, v))? 
 			    Z[v][t] : Z[v][0]);
 		}
 	    }
 	    fputc('\n', fp);
 	}
-	fputc('\n', fp);
     } else if (fmt == GRETL_DATA_DAT) { 
 	/* PcGive: data file with load info */
 	int pd = pdinfo->pd;
@@ -1414,9 +1444,9 @@ int write_data (const char *fname, const int *list,
 		(pd == 1 || pd == 4 || pd == 12)) {
 		int maj, min;
 
-		date_maj_min(pdinfo->stobs, &maj, &min);
+		date_maj_min(pdinfo->t1, pdinfo, &maj, &min);
 		fprintf(fp, "%d %d ", maj, min);
-		date_maj_min(pdinfo->endobs, &maj, &min);
+		date_maj_min(pdinfo->t2, pdinfo, &maj, &min);
 		fprintf(fp, "%d %d %d", maj, min, pd);
 	    } else {
 		fprintf(fp, "%d 1 %d 1 1", pdinfo->t1, pdinfo->t2);
@@ -1426,7 +1456,7 @@ int write_data (const char *fname, const int *list,
 
 	    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 		v = list[i];
-		xx = (pdinfo->vector[v])? Z[v][t] : Z[v][0];
+		xx = (var_is_series(pdinfo, v))? Z[v][t] : Z[v][0];
 		if (na(xx)) {
 		    fprintf(fp, "-9999.99");
 		} else if (pmax[i-1] == PMAX_NOT_AVAILABLE) {
@@ -1438,7 +1468,42 @@ int write_data (const char *fname, const int *list,
 	    }
 	    fputc('\n', fp);
 	}
+    } else if (fmt == GRETL_DATA_JM) { 
+	/* JMulti: ascii with comments and date info */
+	int maj, min;
+
+	fputs("/*\n", fp);
+	for (i=1; i<=list[0]; i++) {
+	    fprintf(fp, " %s: %s\n", pdinfo->varname[list[i]], VARLABEL(pdinfo, i));
+	}
+	fputs("*/\n", fp);
+	date_maj_min(pdinfo->t1, pdinfo, &maj, &min);
+	if (pdinfo->pd == 4 || pdinfo->pd == 12) {
+	    fprintf(fp, "<%d %c%d>\n", maj, (pdinfo->pd == 4)? 'Q' : 'M', min);
+	} else if (pdinfo->pd == 1) {
+	    fprintf(fp, "<%d>\n", maj);
+	} else {
+	    fputs("<1>\n", fp);
+	}
+	for (i=1; i<=list[0]; i++) {
+	    fprintf(fp, " %s", pdinfo->varname[list[i]]);
+	}
 	fputc('\n', fp);
+	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	    for (i=1; i<=list[0]; i++) {
+		v = list[i];
+		if (na(Z[v][t])) {
+		    fputs("NaN ", fp);
+		} else if (pmax[i-1] == PMAX_NOT_AVAILABLE) {
+		    fprintf(fp, "%.12g ", Z[v][t]);
+		} else {
+		    fprintf(fp, "%.*f ", pmax[i-1], 
+			    (var_is_series(pdinfo, v))? 
+			    Z[v][t] : Z[v][0]);
+		}
+	    }
+	    fputc('\n', fp);
+	}
     }
 
     gretl_pop_c_numeric_locale();
@@ -1460,10 +1525,6 @@ static void dataset_type_string (char *str, const DATAINFO *pdinfo)
 	strcpy(str, _("time series"));
     } else if (dataset_is_panel(pdinfo)) {
         strcpy(str, _("panel"));
-	strcat(str, " (");
-	strcat(str, (pdinfo->structure == STACKED_TIME_SERIES)? 
-	       _("Stacked time series") : _("Stacked cross sections"));
-	strcat(str, ")");       
     } else {
         strcpy(str, _("undated"));
     }
@@ -1727,27 +1788,32 @@ int gretl_get_data (double ***pZ, DATAINFO **ppdinfo, char *datfile, PATHS *ppat
     double **tmpZ = NULL;
     FILE *dat = NULL;
     gzFile fz = NULL;
-    int err = 0, gzsuff = 0, add_gdt = 0;
-    int binary = 0, old_byvar = 0;
     char hdrfile[MAXLEN], lblfile[MAXLEN];
+    int gdtsuff, gzsuff = 0;
+    int binary = 0, old_byvar = 0;
+    int err = 0;
 
     *gretl_errmsg = '\0';
-
-    /* get filenames organized */
     *hdrfile = '\0';
-    gzsuff = has_suffix(datfile, ".gz");
+
+    gdtsuff = has_suffix(datfile, ".gdt");
+    if (!gdtsuff) {
+	gzsuff = has_suffix(datfile, ".gz");
+    }
 
     if (addpath(datfile, ppaths, 0) == NULL) { /* not found yet */
 	char tryfile[MAXLEN];
 	int found = 0;
 
-	/* try using the .gdt suffix? */
-	*tryfile = '\0';
-	strncat(tryfile, datfile, MAXLEN-1);
-	try_gdt(tryfile); 
-	found = (addpath(tryfile, ppaths, 0) != NULL);
-	if (found) {
-	    add_gdt = 1;
+	if (!gdtsuff) {
+	    /* try using the .gdt suffix? */
+	    *tryfile = '\0';
+	    strncat(tryfile, datfile, MAXLEN-1);
+	    try_gdt(tryfile); 
+	    found = (addpath(tryfile, ppaths, 0) != NULL);
+	    if (found) {
+		gdtsuff = 1;
+	    }
 	}
 
 	/* or maybe the file is gzipped but lacks a .gz extension? */
@@ -1768,7 +1834,7 @@ int gretl_get_data (double ***pZ, DATAINFO **ppdinfo, char *datfile, PATHS *ppat
     }
 
     /* catch XML files that have strayed in here? */
-    if (add_gdt && xmlfile(datfile)) {
+    if (gdtsuff && gretl_is_xml_file(datfile)) {
 	return gretl_read_gdt(pZ, ppdinfo, datfile, ppaths, 
 			      code, prn, 0);
     }
@@ -1829,6 +1895,26 @@ int gretl_get_data (double ***pZ, DATAINFO **ppdinfo, char *datfile, PATHS *ppat
 	}
     }
 
+    if (gzsuff) {
+	err = gz_readdata(fz, tmpdinfo, tmpZ, binary); 
+	gzclose(fz);
+    } else {
+	err = readdata(dat, tmpdinfo, tmpZ, binary, old_byvar); 
+	fclose(dat);
+    }
+
+    if (err) goto bailout;
+
+    if (tmpdinfo->structure == STACKED_CROSS_SECTION) {
+	err = switch_panel_orientation(tmpZ, tmpdinfo);
+    }
+
+    if (!err && tmpdinfo->structure == STACKED_TIME_SERIES) {
+	err = dataset_add_default_panel_indices(tmpdinfo);
+    }
+
+    if (err) goto bailout;
+
     /* print out basic info from the files read */
     pprintf(prn, I_("periodicity: %d, maxobs: %d,\n"
 	   "observations range: %s-%s\n"), tmpdinfo->pd, tmpdinfo->n,
@@ -1843,28 +1929,19 @@ int gretl_get_data (double ***pZ, DATAINFO **ppdinfo, char *datfile, PATHS *ppat
     }
     pprintf(prn, " %s\n\n", datfile);
 
-    if (gzsuff) {
-	err = gz_readdata(fz, tmpdinfo, tmpZ, binary); 
-	gzclose(fz);
-    } else {
-	err = readdata(dat, tmpdinfo, tmpZ, binary, old_byvar); 
-	fclose(dat);
-    }
-
-    if (err) goto bailout;
-
     /* Set sample range to entire length of dataset by default */
     tmpdinfo->t1 = 0; 
     tmpdinfo->t2 = tmpdinfo->n - 1;
 
-    strcpy(ppaths->datfile, datfile);
-
     err = readlbl(lblfile, tmpdinfo);
     if (err) goto bailout;
 
     if (code == DATA_APPEND) {
 	err = merge_data(pZ, *ppdinfo, tmpZ, tmpdinfo, prn);
     } else {
+	if (ppaths != NULL && datfile != ppaths->datfile) {
+	    strcpy(ppaths->datfile, datfile);
+	}
 	free_Z(*pZ, *ppdinfo);
 	if (code == DATA_CLEAR) {
 	    clear_datainfo(*ppdinfo, CLEAR_FULL);
@@ -1944,9 +2021,11 @@ int open_nulldata (double ***pZ, DATAINFO *pdinfo,
     return 0;
 }
 
-static int check_daily_dates (DATAINFO *pdinfo, int *pd)
+#define DAY_DEBUG 0
+
+static int check_daily_dates (DATAINFO *pdinfo, int *pd, PRN *prn)
 {
-    int fulln = 0, n, t;
+    int fulln = 0, n, t, nbak;
     int oldpd = pdinfo->pd;
     double oldsd0 = pdinfo->sd0;
     long ed1, ed2;
@@ -1959,10 +2038,14 @@ static int check_daily_dates (DATAINFO *pdinfo, int *pd)
 	err = 1;
     }
 
+#if DAY_DEBUG    
+    fprintf(stderr, "S[0] = '%s', ed1 = %ld\n", pdinfo->S[0], ed1);
+#endif
+
     pdinfo->pd = guess_daily_pd(pdinfo);
     pdinfo->structure = TIME_SERIES;
 
-#if 0    
+#if DAY_DEBUG    
     fprintf(stderr, "guessed daily pd = %d\n", pdinfo->pd);
 #endif
 
@@ -1984,35 +2067,42 @@ static int check_daily_dates (DATAINFO *pdinfo, int *pd)
 	    err = 1;
 	} else {
 	    nmiss = fulln - pdinfo->n;
-	    fprintf(stderr, "Observations: %d; days in sample: %d\n", 
+	    pprintf(prn, "Observations: %d; days in sample: %d\n", 
 		    pdinfo->n, fulln);
 	    if (nmiss > 300 * pdinfo->n) {
-		fprintf(stderr, "Probably annual data\n");
+		pprintf(prn, "Probably annual data\n");
 		*pd = 1;
 	    } else if (nmiss > 50 * pdinfo->n) {
-		fprintf(stderr, "Probably quarterly data\n");
+		pprintf(prn, "Probably quarterly data\n");
 		*pd = 4;
 	    } else if (nmiss > 20 * pdinfo->n) {
-		fprintf(stderr, "Probably monthly data\n");
+		pprintf(prn, "Probably monthly data\n");
 		*pd = 12;
 	    } else if (nmiss > 5 * pdinfo->n) {
-		fprintf(stderr, "Probably weekly data\n");
+		pprintf(prn, "Probably weekly data\n");
 		*pd = pdinfo->pd = 52;
 	    } else {
-		fprintf(stderr, "Missing daily observations: %d\n", nmiss);
+		pprintf(prn, "Missing daily observations: %d\n", nmiss);
 	    }
 	}
     }
 
+    nbak = 0;
     for (t=0; t<pdinfo->n && !err; t++) {
 	n = calendar_obs_number(pdinfo->S[t], pdinfo);
 	if (n < t) {
-	    fprintf(stderr, "Error: n = %d < t = %d\n", n, t);
+	    pprintf(prn, "Daily dates error at t = %d:\n"
+		    "  calendar_obs_number() for '%s' = %d but t = %d\n", 
+		    t, pdinfo->S[t], n, t);
 	    err = 1;
 	} else if (n > fulln - 1) {
-	    fprintf(stderr, "Error: n = %d >= fulln = %d\n", n, fulln);
+	    pprintf(prn, "Error: date '%s' out of bounds\n", pdinfo->S[t]);
+	    err = 1;
+	} else if (nbak > 0 && n == nbak) {
+	    pprintf(prn, "Error: date '%s' is repeated\n", pdinfo->S[t]);
 	    err = 1;
 	}
+	nbak = n;
     }
 
     if (err) {
@@ -2121,7 +2211,7 @@ pad_weekly_data (double ***pZ, DATAINFO *pdinfo, int add)
     int ttarg, offset = 0, skip = 0;
     int i, s, t, tc, err;
 
-    err = dataset_add_observations(add, pZ, pdinfo); 
+    err = dataset_add_observations(add, pZ, pdinfo, OPT_A); 
 
     if (!err) {
 	for (t=0; t<oldn; t++) {
@@ -2248,7 +2338,7 @@ csv_daily_date_check (double ***pZ, DATAINFO *pdinfo, PRN *prn)
 		}
 	    }
 	    pprintf(prn, "? %s - %s\n", lbl1, lbl2);
-	    ret = check_daily_dates(pdinfo, &pd);
+	    ret = check_daily_dates(pdinfo, &pd, prn);
 	    if (ret >= 0 && pd > 0) {
 		if (pd == 52) {
 		    if (csv_weekly_data(pZ, pdinfo)) {
@@ -2291,7 +2381,7 @@ static int csv_time_series_check (DATAINFO *pdinfo, PRN *prn)
     if (try > 0 && try < 3000) {
 	pprintf(prn, M_("   %s: probably a year... "), year);
     } else {
-	pprintf(prn, M_("   %s: out of bounds for a year?\n"), year);
+	pprintf(prn, M_("   %s: probably not a year\n"), year);
     }
 
     if (len == 5) {
@@ -2408,7 +2498,7 @@ static int count_add_vars (const DATAINFO *pdinfo, const DATAINFO *addinfo)
     for (i=1; i<addinfo->v; i++) {
 	for (j=1; j<pdinfo->v; j++) {
 	    if (!strcmp(addinfo->varname[i], pdinfo->varname[j])) {
-		if (!pdinfo->vector[j]) {
+		if (var_is_scalar(pdinfo, j)) {
 		    addvars = -1;
 		} else {
 		    addvars--;
@@ -2508,13 +2598,13 @@ static void merge_error (char *msg, PRN *prn)
  * gretl's current working data set.
  * 
  * Returns: 0 on successful completion, non-zero otherwise.
- *
  */
 
 int merge_data (double ***pZ, DATAINFO *pdinfo,
 		double **addZ, DATAINFO *addinfo,
 		PRN *prn)
 {
+    int addsimple = 0;
     int addvars = 0;
     int addobs = 0;
     int offset = 0;
@@ -2522,13 +2612,20 @@ int merge_data (double ***pZ, DATAINFO *pdinfo,
 
     /* first check for conformability */
 
-    if (pdinfo->pd != addinfo->pd) {
+    if (addinfo->pd == 1 && addinfo->structure == CROSS_SECTION &&
+	addinfo->n == pdinfo->n) {
+	/* we'll allow undated data to be merged with dated, provided
+	   the number of observations matches OK */
+	addsimple = 1;
+    } else if (pdinfo->pd != addinfo->pd) {
 	merge_error(_("Data frequency does not match\n"), prn);
 	err = 1;
     }
 
     if (!err) {
-	addobs = compare_ranges(pdinfo, addinfo, &offset);
+	if (!addsimple) {
+	    addobs = compare_ranges(pdinfo, addinfo, &offset);
+	}
 	addvars = count_add_vars(pdinfo, addinfo);
     }
 
@@ -2563,73 +2660,74 @@ int merge_data (double ***pZ, DATAINFO *pdinfo,
 	for (i=0; i<pdinfo->v && !err; i++) {
 	    double *x;
 
-	    if (!pdinfo->vector[i]) {
+	    if (var_is_scalar(pdinfo, i)) {
 		continue;
 	    }
 
-	   x = realloc((*pZ)[i], new_n * sizeof *x);
-	   if (x == NULL) {
-	       err = 1;
-	       break;
-	   }
+	    x = realloc((*pZ)[i], new_n * sizeof *x);
+	    if (x == NULL) {
+		err = 1;
+		break;
+	    }
 
-	   for (t=pdinfo->n; t<new_n; t++) {
-	       if (i == 0) {
-		   x[t] = 1.0;
-	       } else {
-		   x[t] = NADBL;
-	       }
-	   }
-	   (*pZ)[i] = x;
-       }
+	    for (t=pdinfo->n; t<new_n; t++) {
+		if (i == 0) {
+		    x[t] = 1.0;
+		} else {
+		    x[t] = NADBL;
+		}
+	    }
+	    (*pZ)[i] = x;
+	}
 
-       if (err) { 
-	   merge_error(_("Out of memory adding data\n"), prn);
-       } else {
-	   pdinfo->n = new_n;
-	   ntodate_full(pdinfo->endobs, new_n - 1, pdinfo);
-	   pdinfo->t2 = pdinfo->n - 1;
-       }
-   }
+	if (err) { 
+	    merge_error(_("Out of memory adding data\n"), prn);
+	} else {
+	    pdinfo->n = new_n;
+	    ntodate_full(pdinfo->endobs, new_n - 1, pdinfo);
+	    pdinfo->t2 = pdinfo->n - 1;
+	}
+    }
 
-   if (!err) { 
-       int k = pdinfo->v;
-       int i, t;
+    if (!err) { 
+	int k = pdinfo->v;
+	int i, t;
 
-       if (addvars > 0 && dataset_add_series(addvars, pZ, pdinfo)) {
-	   merge_error(_("Out of memory adding data\n"), prn);
-	   err = 1;
-       }
+	if (addvars > 0 && dataset_add_series(addvars, pZ, pdinfo)) {
+	    merge_error(_("Out of memory adding data\n"), prn);
+	    err = 1;
+	}
 
-       for (i=1; i<addinfo->v && !err; i++) {
-	   int v = varindex(pdinfo, addinfo->varname[i]);
-	   int newvar = 0;
+	for (i=1; i<addinfo->v && !err; i++) {
+	    int v = varindex(pdinfo, addinfo->varname[i]);
+	    int newvar = 0;
 
-	   if (v >= k) {
-	       /* a  new variable */
-	       v = k++;
-	       newvar = 1;
-	       strcpy(pdinfo->varname[v], addinfo->varname[i]);
-	   } 
+	    if (v >= k) {
+		/* a new variable */
+		v = k++;
+		newvar = 1;
+		strcpy(pdinfo->varname[v], addinfo->varname[i]);
+		copy_varinfo(pdinfo->varinfo[v], addinfo->varinfo[i]);
+	    } 
 
-	   for (t=0; t<pdinfo->n; t++) {
-	       if (t >= offset && t - offset < addinfo->n) {
-		   (*pZ)[v][t] = addZ[i][t - offset];
-	       } else if (newvar) {
-		   (*pZ)[v][t] = NADBL;
-	       }
-	   }
-       }
-   }
+	    for (t=0; t<pdinfo->n; t++) {
+		if (t >= offset && t - offset < addinfo->n) {
+		    (*pZ)[v][t] = addZ[i][t - offset];
+		} else if (newvar) {
+		    (*pZ)[v][t] = NADBL;
+		}
+	    }
+	}
+    }
 
-   if (!err && (addvars || addobs)) {
-       pputs(prn, _("Data appended OK\n"));
-   }
+    if (!err && (addvars || addobs)) {
+	pputs(prn, _("Data appended OK\n"));
+    }
 
-   free_Z(addZ, addinfo);
-   clear_datainfo(addinfo, CLEAR_FULL);
+    free_Z(addZ, addinfo);
+    clear_datainfo(addinfo, CLEAR_FULL);
 
-   return err;
+    return err;
 }
 
 /* The function below checks for the maximum line length in the given
@@ -2801,6 +2899,8 @@ static void check_first_field (const char *line, char delim,
                  !strcmp(s, "N.A.") || \
                  !strcmp(s, "n.a.") || \
                  !strcmp(s, "na") || \
+                 !strcmp(s, "N/A") || \
+                 !strcmp(s, "NaN") || \
                  !strcmp(s, ".") || \
                  !strcmp(s, "..") || \
                  !strncmp(s, "-999", 4))
@@ -2850,17 +2950,16 @@ static int add_obs_marker (DATAINFO *pdinfo, int n)
     return 0;
 }
 
-static int dataset_add_obs (double ***pZ, DATAINFO *pdinfo)
+static int add_single_obs (double ***pZ, DATAINFO *pdinfo)
 {
-    int i;
-    int err = 0;
+    double *x;
+    int i, err = 0;
 
     for (i=0; i<pdinfo->v; i++) {
-	if (pdinfo->vector[i]) {
-	    double *tmp = realloc((*pZ)[i], (pdinfo->n + 1) * sizeof ***pZ);
-
-	    if (tmp != NULL) {
-		(*pZ)[i] = tmp;
+	if (var_is_series(pdinfo, i)) {
+	    x = realloc((*pZ)[i], (pdinfo->n + 1) * sizeof *x);
+	    if (x != NULL) {
+		(*pZ)[i] = x;
 	    } else {
 		return 1;
 	    }
@@ -2872,7 +2971,7 @@ static int dataset_add_obs (double ***pZ, DATAINFO *pdinfo)
     (*pZ)[0][pdinfo->n - 1] = 1.0;
 
     for (i=1; i<pdinfo->v; i++) {
-	if (pdinfo->vector[i]) {
+	if (var_is_series(pdinfo, i)) {
 	    (*pZ)[i][pdinfo->n - 1] = NADBL;
 	}
     }
@@ -3048,7 +3147,7 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
 
     csvinfo->delim = delim;
 
-#if USE_GTK2
+#if USE_GLIB2
     if (!g_utf8_validate(fname, -1, NULL)) {
 	gchar *trfname = g_locale_to_utf8(fname, -1, NULL, NULL, NULL);
 
@@ -3224,7 +3323,7 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
 	obs_labels_no_varnames(obs_1, csvinfo, numcount)) {
 	pputs(prn, M_("it seems there are no variable names\n"));
 	/* then we undercounted the observations by one */
-	if (dataset_add_obs(&csvZ, csvinfo)) {
+	if (add_single_obs(&csvZ, csvinfo)) {
 	    pputs(prn, _("Out of memory\n"));
 	    goto csv_bailout;
 	}
@@ -3400,8 +3499,10 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
  * @fname: name of file containing case markers.
  * 
  * Read case markers (strings of %OBSLEN - 1 characters or less that identify
- * the observations) from a file, and associate tham with the 
- * current data set.
+ * the observations) from a file, and associate them with the 
+ * current data set.  The file should contain one marker per line,
+ * with a number of lines equal to the number of observations in
+ * the current data set.
  * 
  * Returns: 0 on successful completion, non-zero otherwise.
  */
@@ -3410,7 +3511,8 @@ int add_obs_markers_from_file (DATAINFO *pdinfo, const char *fname)
 {
     char **Sbak = NULL;
     FILE *fp;
-    char marker[OBSLEN], sformat[8];
+    char line[128];
+    char marker[OBSLEN], sformat[10];
     int t, err = 0;
 
     fp = gretl_fopen(fname, "r");
@@ -3419,7 +3521,7 @@ int add_obs_markers_from_file (DATAINFO *pdinfo, const char *fname)
     }
 
     if (pdinfo->S != NULL) {
-	/* keep backup copy */
+	/* keep backup copy of existing markers */
 	Sbak = pdinfo->S;
 	pdinfo->S = NULL;
     }
@@ -3429,15 +3531,15 @@ int add_obs_markers_from_file (DATAINFO *pdinfo, const char *fname)
 	goto bailout;
     }
     
-    sprintf(sformat, "%%%ds", OBSLEN - 1);
+    sprintf(sformat, "%%%d[^\n]", OBSLEN - 1);
 
-    for (t=0; t<pdinfo->n; t++) {
-	eatspace(fp);
-	if (fscanf(fp, sformat, marker)) {
-	    strcat(pdinfo->S[t], marker);
-	} else {
+    for (t=0; t<pdinfo->n && !err; t++) {
+	if (fgets(line, sizeof line, fp) == NULL) {
 	    err = E_DATA;
-	    goto bailout;
+	} else if (sscanf(line, sformat, marker) != 1) {
+	    err = E_DATA;
+	} else {
+	    strcat(pdinfo->S[t], marker);
 	}
     }
 
@@ -3980,29 +4082,23 @@ int import_other (double ***pZ, DATAINFO **ppdinfo,
 
     check_for_console(prn);
 
-    if (ftype == GRETL_GNUMERIC || ftype == GRETL_EXCEL) {
-	pputs(prn, "At present Gnumeric and Excel files can only be opened "
-	      "via the GUI menus.\n");
-	return E_DATA;
-    }
-
     fp = gretl_fopen(fname, "r");
     if (fp == NULL) {
 	pprintf(prn, M_("Couldn't open %s\n"), fname);
 	return E_FOPEN;
     }
-
     fclose(fp);
 
-    /* FIXME Gnumeric and XLS */
     if (ftype == GRETL_GNUMERIC) {
-	sheet_get_data = get_plugin_function("wbook_get_data", &handle);
+	sheet_get_data = get_plugin_function("cli_get_gnumeric", &handle);
     } else if (ftype == GRETL_EXCEL) {
-	sheet_get_data = get_plugin_function("excel_get_data", &handle);
+	sheet_get_data = get_plugin_function("cli_get_xls", &handle);
     } else if (ftype == GRETL_WF1) {
 	sheet_get_data = get_plugin_function("wf1_get_data", &handle);
     } else if (ftype == GRETL_DTA) {
 	sheet_get_data = get_plugin_function("dta_get_data", &handle);
+    } else if (ftype == GRETL_JMULTI) {
+	sheet_get_data = get_plugin_function("jmulti_get_data", &handle);
     } else {
 	pprintf(prn, M_("Unrecognized data type"));
 	pputc(prn, '\n');
@@ -4019,22 +4115,60 @@ int import_other (double ***pZ, DATAINFO **ppdinfo,
     return err;
 }
 
-static int xmlfile (const char *fname)
+static int is_jmulti_datafile (const char *fname)
 {
-    gzFile fz;
-    char test[6];
+    FILE *fp;
     int ret = 0;
 
-    fz = gretl_gzopen(fname, "rb");
-    if (fz != Z_NULL) {
-	if (gzread(fz, test, 5)) {
-	    test[5] = '\0';
-	    if (!strcmp(test, "<?xml")) ret = 1;
+    fp = gretl_fopen(fname, "r");
+
+    if (fp != NULL) {
+	char test[128] = {0};
+	int gotobs = 0;
+	int gotcomm = 0;
+	int incomm = 0;
+
+	/* look for characteristic C-style comment and
+	   <obs stuff> field, outside of comment */
+
+	while (fgets(test, sizeof test, fp)) {
+	    if (!incomm && strstr(test, "/*")) {
+		gotcomm = 1;
+		incomm = 1;
+	    }
+	    if (incomm && strstr(test, "*/")) {
+		incomm = 0;
+	    }
+	    if (!incomm && *test == '<' && strchr(test, '>')) {
+		gotobs = 1;
+	    }
+	    if (gotcomm && gotobs) {
+		ret = 1;
+		break;
+	    }
 	} 
-	gzclose(fz);
+	fclose(fp);
     } 
+
     return ret;
-} 
+}
+
+int gretl_is_pkzip_file (const char *fname)
+{
+    FILE *fp;
+    char test[3] = {0};
+    int ret = 0;
+
+    fp = gretl_fopen(fname, "rb");
+    if (fp != NULL) {
+	if (fread(test, 1, 2, fp) == 2) {
+	    if (!strcmp(test, "PK")) ret = 1;
+	} 
+	fclose(fp);
+    } 
+
+    return ret;
+}
 
 /**
  * detect_filetype:
@@ -4055,11 +4189,17 @@ GretlFileType detect_filetype (char *fname, PATHS *ppaths, PRN *prn)
     FILE *fp;
 
     /* might be a script file? (watch out for DOS-mangled names) */
-    if (has_suffix(fname, ".inp") ||
-	has_suffix(fname, ".gre"))
+    if (has_suffix(fname, ".inp")) 
 	return GRETL_SCRIPT;
-    if (has_suffix(fname, ".gretl"))
-	return GRETL_SCRIPT; 
+
+    if (has_suffix(fname, ".gretl")) {
+	if (gretl_is_pkzip_file(fname)) {
+	    return GRETL_SESSION;
+	} else {
+	    return GRETL_SCRIPT;
+	}
+    }
+
     if (has_suffix(fname, ".gnumeric"))
 	return GRETL_GNUMERIC;
     if (has_suffix(fname, ".xls"))
@@ -4078,13 +4218,19 @@ GretlFileType detect_filetype (char *fname, PATHS *ppaths, PRN *prn)
 	return GRETL_CSV_DATA;
     if (has_suffix(fname, ".m"))
 	return GRETL_OCTAVE;
+    if (has_suffix(fname, ".bn7"))
+	return GRETL_PCGIVE_DB;
 
     addpath(fname, ppaths, 0); 
 
-    if (xmlfile(fname)) {
+    if (gretl_is_xml_file(fname)) {
 	return GRETL_XML_DATA;  
     } 
 
+    if (has_suffix(fname, ".dat") && is_jmulti_datafile(fname)) {
+	return GRETL_JMULTI; 
+    }
+
     fp = gretl_fopen(fname, "r");
     if (fp == NULL) { 
 	/* may be native file in different location */
@@ -4186,7 +4332,7 @@ int transpose_data (double ***pZ, DATAINFO *pdinfo)
     int i, t;
 
     for (i=1; i<pdinfo->v; i++) {
-	if (!pdinfo->vector[i]) {
+	if (var_is_scalar(pdinfo, i)) {
 	    strcpy(gretl_errmsg, _("Dataset contains scalars, can't transpose"));
 	    return E_DATA;
 	}
@@ -4224,7 +4370,6 @@ int transpose_data (double ***pZ, DATAINFO *pdinfo)
 
     pdinfo->varname = tinfo->varname;
     pdinfo->varinfo = tinfo->varinfo;
-    pdinfo->vector = tinfo->vector;
 
     dataset_obs_info_default(pdinfo);
 
