@@ -75,6 +75,17 @@ static void free_sorted_markers (DATAINFO *pdinfo, int v)
     }    
 }
 
+void dataset_destroy_panel_info (DATAINFO *pdinfo)
+{
+    if (pdinfo->paninfo != NULL) {
+	free(pdinfo->paninfo->unit);
+	free(pdinfo->paninfo->period);
+	free(pdinfo->paninfo->padmask);
+	free(pdinfo->paninfo);
+	pdinfo->paninfo = NULL;
+    }
+}
+
 static void free_varinfo (DATAINFO *pdinfo, int v)
 {
     free_sorted_markers(pdinfo, v);
@@ -112,6 +123,10 @@ void clear_datainfo (DATAINFO *pdinfo, int code)
 
     pdinfo->submode = 0;
 
+    if (pdinfo->paninfo != NULL) {
+	dataset_destroy_panel_info(pdinfo);
+    }    
+
     /* if this is not a sub-sample datainfo, free varnames, labels, etc. */
     if (code == CLEAR_FULL) {
 	if (pdinfo->varname != NULL) {
@@ -132,10 +147,6 @@ void clear_datainfo (DATAINFO *pdinfo, int code)
 	    free(pdinfo->descrip);
 	    pdinfo->descrip = NULL;
 	}
-	if (pdinfo->vector != NULL) {
-	    free(pdinfo->vector);
-	    pdinfo->vector = NULL;
-	}
 
 	maybe_free_full_dataset(pdinfo);
 
@@ -200,29 +211,6 @@ void dataset_obs_info_default (DATAINFO *pdinfo)
     pdinfo->decpoint = '.';
 }
 
-static char **allocate_obslen_strings (int n)
-{
-    char **S;
-    int j, t;
-
-    S = malloc(n * sizeof *S);
-    if (S == NULL) return NULL;
-
-    for (t=0; t<n; t++) {
-	S[t] = malloc(OBSLEN);
-	if (S[t] == NULL) {
-	    for (j=0; j<t; j++) {
-		free(S[j]);
-	    }
-	    free(S);
-	    return NULL;
-	}
-	S[t][0] = '\0';
-    }
-
-    return S;
-}
-
 /**
  * dataset_allocate_obs_markers:
  * @pdinfo: dataset information struct
@@ -242,7 +230,7 @@ int dataset_allocate_obs_markers (DATAINFO *pdinfo)
 
     if (pdinfo->S == NULL) {
 	/* not already allocated */
-	S = allocate_obslen_strings(pdinfo->n);
+	S = strings_array_new_with_length(pdinfo->n, OBSLEN);
 	if (S == NULL) {
 	    err = E_ALLOC;
 	} else {
@@ -257,12 +245,235 @@ int dataset_allocate_obs_markers (DATAINFO *pdinfo)
     return err;
 }
 
+/**
+ * dataset_allocate_panel_info:
+ * @pdinfo: dataset information struct
+ *
+ * Allocates space in @pdinfo for two indices representing
+ * the unit or group and time-period, respectively,  of each 
+ * observation in a panel data set.
+ *
+ * Returns: 0 on success, %E_ALLOC on error.
+ */
+
+int dataset_allocate_panel_info (DATAINFO *pdinfo)
+{
+    PANINFO *pan;
+    int i;
+
+#if 0
+    fprintf(stderr, "dataset_allocate_panel_info: pdinfo at %p\n",
+	    (void *) pdinfo);
+#endif
+
+    /* just in case, clean out any previous stuff */
+    dataset_destroy_panel_info(pdinfo);  
+
+    pan = malloc(sizeof *pan);
+    if (pan == NULL) {
+	return E_ALLOC;
+    }
+
+    pan->unit = pan->period = NULL;
+    pan->padmask = NULL;
+
+    pan->unit = malloc(pdinfo->n * sizeof *pan->unit);
+    pan->period = malloc(pdinfo->n * sizeof *pan->period);
+
+    if (pan->unit == NULL || pan->period == NULL) {
+	free(pan->unit);
+	free(pan->period);
+	free(pan);
+	return E_ALLOC;
+    }
+
+    for (i=0; i<pdinfo->n; i++) {
+	pan->unit[i] = pan->period[i] = -1; /* deliberately invalid */
+    }
+
+    pan->nunits = 0;
+    pan->Tmin = pan->Tmax = 0;
+    pan->olen = 0;
+
+    pdinfo->paninfo = pan;
+
+    return 0;
+}
+
+static int resize_panel_info (DATAINFO *pdinfo, int n)
+{
+    PANINFO *pan = pdinfo->paninfo;
+    int *unit;
+    int *period;
+
+    unit = realloc(pan->unit, n * sizeof *unit);
+    if (unit == NULL) {
+	return E_ALLOC;
+    }
+    pan->unit = unit;
+
+    period = realloc(pan->period, n * sizeof *period);
+    if (period == NULL) {
+	return E_ALLOC;
+    }
+    pan->period = period;
+
+    if (n > pdinfo->n) {
+	int uadd = (n - pdinfo->n) / pdinfo->pd;
+	int i, t, j = pdinfo->paninfo->nunits;
+	int s = pdinfo->n;
+
+	for (i=0; i<uadd; i++) {
+	    for (t=0; t<pdinfo->pd; t++) {
+		pan->unit[s + i] = j;
+		pan->period[s + i] = t;
+		s++;
+	    }
+	    j++;
+	}
+    }
+
+    pdinfo->paninfo->nunits = n / pdinfo->pd;
+
+    return 0;
+}
+
+/**
+ * dataset_add_default_panel_indices:
+ * @pdinfo: dataset information struct.
+ *
+ * Adds a pair of indices for panel unit and panel period.
+ * The default is that both are zero-based and increase
+ * consecutively, per unit and per period respectively.
+ * This function assumes a balanced panel.
+ *
+ * Returns: 0 on success, non-zero code on error.
+ */
+
+int dataset_add_default_panel_indices (DATAINFO *pdinfo)
+{
+    int nunits, T;
+    int i, s, t, err;
+
+    if (pdinfo->n % pdinfo->pd != 0) {
+	fprintf(stderr, "dataset_add_default_panel_indices: "
+		"error: pdinfo->n %% pdinfo->pd = %d\n", 
+		pdinfo->n % pdinfo->pd);	
+	return E_DATA;
+    }
+
+    err = dataset_allocate_panel_info(pdinfo);
+
+    if (!err) {
+	char test[32];
+
+	T = pdinfo->pd;
+	nunits = pdinfo->n / pdinfo->pd;
+	s = 0;
+	for (i=0; i<nunits; i++) {
+	    for (t=0; t<T; t++) {
+		pdinfo->paninfo->unit[s] = i;
+		pdinfo->paninfo->period[s] = t;
+		s++;
+	    }
+	}
+	pdinfo->paninfo->nunits = nunits;
+	pdinfo->paninfo->Tmin = T;
+	pdinfo->paninfo->Tmax = T;
+
+	sprintf(test, "%d", T);
+	pdinfo->paninfo->olen = strlen(test);
+    }
+
+    return err;
+}
+
+/**
+ * dataset_finalize_panel_indices:
+ * @pdinfo: dataset information struct.
+ *
+ * Having already added a pair of indices for panel unit 
+ * and panel period, check these for consistency and
+ * calculate the number of panel units and the minimum
+ * and maximum observations per unit.  If it turns out
+ * there's only one unit, or only one period, in the
+ * dataset, then it's not really a panel: we destroy
+ * the panel info and return %E_PDWRONG.
+ *
+ * Returns: 0 on success, non-zero code on error.
+ */
+
+int dataset_finalize_panel_indices (DATAINFO *pdinfo)
+{
+    PANINFO *pan = pdinfo->paninfo;
+    char test[32];
+    int u0, ubak;
+    int i, Ti, len;
+
+    if (pan == NULL) {
+	return E_DATA;
+    }
+
+    pan->nunits = 1;
+    pan->Tmax = 0;
+    pan->Tmin = 999999;
+    pan->olen = 0;
+
+    u0 = ubak = pan->unit[0];
+    Ti = 0;
+
+    /* basic validity check */
+    for (i=0; i<pdinfo->n; i++) {
+	if (pan->unit[i] < 0 || pan->period[i] < 0) {
+	    strcpy(gretl_errmsg, "Panel index information is corrupted");
+	    return E_DATA;
+	}
+	sprintf(test, "%d", pan->period[i] + 1);
+	len = strlen(test);
+	if (len > pan->olen) {
+	    pan->olen = len;
+	}
+    }
+
+    for (i=0; i<pdinfo->n; i++) {
+	if (pan->unit[i] == ubak) {
+	    Ti++;
+	} else {
+	    pan->nunits += 1;
+	    if (Ti > pan->Tmax) {
+		pan->Tmax = Ti;
+	    }
+	    if (Ti < pan->Tmin) {
+		pan->Tmin = Ti;
+	    }	    
+	    Ti = 1;
+	    ubak = pan->unit[i];
+	}
+    }
+
+    if (pan->nunits == 1 || pan->Tmax < 2) {
+	dataset_destroy_panel_info(pdinfo);
+	pdinfo->structure = CROSS_SECTION;
+	return E_PDWRONG;
+    }
+
+#if 0
+    fprintf(stderr, "dataset_finalize_panel_indices:\n "
+	    "nunits=%d, Tmin=%d, Tmax=%d, olen=%d\n", pan->nunits,
+	    pan->Tmin, pan->Tmax, pan->olen);
+#endif
+
+    return 0;
+}
+
 static void gretl_varinfo_init (VARINFO *vinfo)
 {
     vinfo->label[0] = '\0';
     vinfo->display_name[0] = '\0';
+    vinfo->flags = 0;
     vinfo->compact_method = COMPACT_NONE;
     vinfo->stack_level = 0;
+    vinfo->line_width = 1;
     vinfo->sorted_markers = NULL;
 
     if (gretl_executing_function()) {
@@ -270,6 +481,29 @@ static void gretl_varinfo_init (VARINFO *vinfo)
     } 
 }
 
+/**
+ * copy_varinfo:
+ * @targ: target to which to copy.
+ * @src: source to copy from.
+ *
+ * Copies all relevant information from @src to @targ.
+ */
+
+void copy_varinfo (VARINFO *targ, const VARINFO *src)
+{
+    if (src == NULL || targ == NULL) {
+	return;
+    }
+
+    strcpy(targ->label, src->label);
+    strcpy(targ->display_name, src->display_name);
+    targ->flags = src->flags;
+    targ->compact_method = src->compact_method;
+    targ->stack_level = src->stack_level;
+    targ->line_width = src->line_width;
+    targ->sorted_markers = NULL;
+}
+
 /**
  * dataset_allocate_varnames:
  * @pdinfo: dataset information struct.
@@ -286,8 +520,8 @@ int dataset_allocate_varnames (DATAINFO *pdinfo)
 {
     int i, j, v = pdinfo->v;
     int err = 0;
-    
-    pdinfo->varname = malloc(v * sizeof *pdinfo->varname);
+
+    pdinfo->varname = strings_array_new_with_length(v, VNAMELEN);
     if (pdinfo->varname == NULL) {
 	return E_ALLOC;
     }
@@ -298,27 +532,7 @@ int dataset_allocate_varnames (DATAINFO *pdinfo)
 	return E_ALLOC;
     }
 
-    pdinfo->vector = malloc(v * sizeof *pdinfo->vector);
-    if (pdinfo->vector == NULL) {
-	free(pdinfo->varname);
-	free(pdinfo->varinfo);
-	return E_ALLOC;
-    }
-
     for (i=0; i<v; i++) {
-	pdinfo->varname[i] = malloc(VNAMELEN);
-	if (pdinfo->varname[i] == NULL) {
-	    for (j=0; j<i; j++) {
-		free(pdinfo->varname[j]);
-	    }
-	    free(pdinfo->varname);
-	    pdinfo->varname = NULL;
-	    err = E_ALLOC;
-	    break;
-	} else {
-	    pdinfo->varname[i][0] = '\0';
-	}
-
 	pdinfo->varinfo[i] = malloc(sizeof **pdinfo->varinfo);
 	if (pdinfo->varinfo[i] == NULL) {
 	    for (j=0; j<i; j++) {
@@ -331,17 +545,12 @@ int dataset_allocate_varnames (DATAINFO *pdinfo)
 	} else {
 	    gretl_varinfo_init(pdinfo->varinfo[i]);
 	}
-
-	pdinfo->vector[i] = 1;
     }
 
     if (!err) {
 	strcpy(pdinfo->varname[0], "const");
 	strcpy(VARLABEL(pdinfo, 0), _("auto-generated constant"));
-    } else {
-	free(pdinfo->vector);
-	pdinfo->vector = NULL;
-    }
+    } 
 
     return err;
 }
@@ -375,6 +584,7 @@ DATAINFO *datainfo_new (void)
 
     dinfo->varname = NULL;
     dinfo->varinfo = NULL;    
+    dinfo->paninfo = NULL;
 
     dinfo->markers = NO_MARKERS;  
     dinfo->delim = ',';
@@ -382,7 +592,6 @@ DATAINFO *datainfo_new (void)
 
     dinfo->S = NULL;
     dinfo->descrip = NULL;
-    dinfo->vector = NULL;
     dinfo->submask = NULL;
     dinfo->submode = 0;
 
@@ -446,8 +655,8 @@ create_new_dataset (double ***pZ, int nvar, int nobs, int markers)
  * Allocates the two-dimensional array to which @pZ points,
  * based on the %v (number of variables) and %n (number of
  * observations) members of @pdinfo.  The variable at 
- * position 0 is initialized to all 1s; no other variables
- * are initialized.
+ * position 0 is initialized to all 1s; other variables
+ * are initialized to #NADBL.
  *
  * Returns: 0 on success, %E_ALLOC on error.
  */
@@ -455,39 +664,21 @@ create_new_dataset (double ***pZ, int nvar, int nobs, int markers)
 int allocate_Z (double ***pZ, const DATAINFO *pdinfo)
 {
     double **Z;
-    int i, j, t;
+    int i, t;
     int err = 0;
 
     if (*pZ != NULL) {
 	free(*pZ);
     }
 
-    Z = malloc(pdinfo->v * sizeof *Z);
+    Z = doubles_array_new(pdinfo->v, pdinfo->n);
 
     if (Z == NULL) {
 	err = E_ALLOC;
     } else {
-	for (i=0; i<pdinfo->v && !err; i++) {
-	    Z[i] = malloc(pdinfo->n * sizeof **Z);
-	    if (Z[i] == NULL) {
-		for (j=0; j<i; j++) {
-		    free(Z[j]);
-		}
-		free(Z);
-		Z = NULL;
-		err = E_ALLOC;
-	    }
-	}
-    }
-
-    if (!err) {
 	for (i=0; i<pdinfo->v; i++) {
 	    for (t=0; t<pdinfo->n; t++) {
-		if (i == 0) {
-		    Z[i][t] = 1.0; 
-		} else {
-		    Z[i][t] = NADBL;
-		}
+		Z[i][t] = (i == 0)? 1.0 : NADBL;
 	    }
 	}
     }
@@ -532,6 +723,7 @@ int start_new_Z (double ***pZ, DATAINFO *pdinfo, int resample)
     pdinfo->delim = ',';
     pdinfo->descrip = NULL;
     pdinfo->data = NULL;
+    pdinfo->paninfo = NULL;
     pdinfo->submask = NULL;
     pdinfo->submode = 0;
     
@@ -783,28 +975,40 @@ maybe_extend_dummies (double **Z, const DATAINFO *pdinfo, int oldn)
  * @newobs: number of observations to add.
  * @pZ: pointer to data array.
  * @pdinfo: dataset information.
+ * @opt: use %OPT_A to attempt to recognize and
+ * automatically extend simple deterministic variables such 
+ * as a time trend and periodic dummy variables; also can
+ * use %OPT_D to drop any observation markers rather than
+ * expanding the set of markers and padding it out with
+ * dummy values.
  *
  * Extends all series in the dataset by the specified number of
  * extra observations.  The added values are initialized to
  * the missing value code, #NADBL, with the exception of
- * simple deterministic variables such as a time trend and
- * periodic dummy variables, which are extrapolated.
+ * simple deterministic variables when %OPT_A is given.
  *
- * Returns: 0 on success, %E_ALLOC on error.
+ * Returns: 0 on success, non-zero code on error.
  */
 
-int dataset_add_observations (int newobs, double ***pZ, DATAINFO *pdinfo)
+int dataset_add_observations (int newobs, double ***pZ, DATAINFO *pdinfo,
+			      gretlopt opt)
 {
     double *x;
     int oldn = pdinfo->n;
     int i, t, bign;
 
-    if (newobs <= 0) return 0;
+    if (newobs <= 0) {
+	return 0;
+    }
+
+    if (dataset_is_panel(pdinfo) && newobs % pdinfo->pd != 0) {
+	return E_PDWRONG;
+    }
 
     bign = pdinfo->n + newobs;
 
     for (i=0; i<pdinfo->v; i++) {
-	if (pdinfo->vector[i]) {
+	if (var_is_series(pdinfo, i)) {
 	    x = realloc((*pZ)[i], bign * sizeof *x);
 	    if (x == NULL) {
 		return E_ALLOC;
@@ -817,11 +1021,21 @@ int dataset_add_observations (int newobs, double ***pZ, DATAINFO *pdinfo)
     }
     
     if (pdinfo->markers && pdinfo->S != NULL) {
-	if (reallocate_markers(pdinfo, bign)) {
-	    return E_ALLOC;
+	if (opt & OPT_D) {
+	    dataset_destroy_obs_markers(pdinfo);
+	} else {
+	    if (reallocate_markers(pdinfo, bign)) {
+		return E_ALLOC;
+	    }
+	    for (t=oldn; t<bign; t++) {
+		sprintf(pdinfo->S[t], "%d", t + 1);
+	    }
 	}
-	for (t=oldn; t<bign; t++) {
-	    sprintf(pdinfo->S[t], "%d", t + 1);
+    }
+
+    if (pdinfo->paninfo != NULL) {
+	if (resize_panel_info(pdinfo, bign)) {
+	    return E_ALLOC;
 	}
     }
     
@@ -831,8 +1045,10 @@ int dataset_add_observations (int newobs, double ***pZ, DATAINFO *pdinfo)
 
     pdinfo->n = bign;
 
-    maybe_extend_trends(*pZ, pdinfo, oldn);
-    maybe_extend_dummies(*pZ, pdinfo, oldn);
+    if (opt & OPT_A) {
+	maybe_extend_trends(*pZ, pdinfo, oldn);
+	maybe_extend_dummies(*pZ, pdinfo, oldn);
+    }
 
     /* does daily data need special handling? */
     ntodate(pdinfo->endobs, bign - 1, pdinfo);
@@ -849,7 +1065,7 @@ int dataset_add_observations (int newobs, double ***pZ, DATAINFO *pdinfo)
  * Deletes @n observations from the end of each series in the 
  * dataset.
  *
- * Returns: 0 on success, %E_ALLOC on error.
+ * Returns: 0 on success, non-zero code on error.
  */
 
 int dataset_drop_observations (int n, double ***pZ, DATAINFO *pdinfo)
@@ -857,12 +1073,18 @@ int dataset_drop_observations (int n, double ***pZ, DATAINFO *pdinfo)
     double *x;
     int i, newn;
 
-    if (n <= 0) return 0;
+    if (n <= 0) {
+	return 0;
+    }
+
+    if (dataset_is_panel(pdinfo) && n % pdinfo->pd != 0) {
+	return E_PDWRONG;
+    }
 
     newn = pdinfo->n - n;
 
     for (i=0; i<pdinfo->v; i++) {
-	if (pdinfo->vector[i]) {
+	if (var_is_series(pdinfo, i)) {
 	    x = realloc((*pZ)[i], newn * sizeof *x);
 	    if (x == NULL) {
 		return E_ALLOC;
@@ -876,7 +1098,13 @@ int dataset_drop_observations (int n, double ***pZ, DATAINFO *pdinfo)
 	    return E_ALLOC;
 	}
     }
-    
+
+    if (pdinfo->paninfo != NULL) {
+	if (resize_panel_info(pdinfo, newn)) {
+	    return E_ALLOC;
+	}
+    }    
+
     if (pdinfo->t2 > newn - 1) {
 	pdinfo->t2 = newn - 1;
     }
@@ -889,11 +1117,74 @@ int dataset_drop_observations (int n, double ***pZ, DATAINFO *pdinfo)
     return 0;
 }
 
+/**
+ * dataset_shrink_obs_range:
+ * @pZ: pointer to data array.
+ * @pdinfo: dataset information.
+ *
+ * Truncates the range of observations in the dataset, based on
+ * the current values of the %t1 and %t2 members of @pdinfo.
+ *
+ * Returns: 0 on success, non-zero code on error.
+ */
+
+int dataset_shrink_obs_range (double ***pZ, DATAINFO *pdinfo)
+{
+    int head = pdinfo->t1;
+    int tail = pdinfo->n - 1 - pdinfo->t2;
+    int err = 0;
+
+    if (head > 0) {
+	int mvsize, rem = pdinfo->n - head;
+	double **Z = *pZ;
+	int i;
+
+	mvsize = rem * sizeof **Z;
+	for (i=0; i<pdinfo->v; i++) {
+	    if (var_is_series(pdinfo, i)) {
+		memmove(Z[i], Z[i] + head, mvsize);
+	    }
+	}
+
+	if (pdinfo->markers && pdinfo->S != NULL) {
+	    for (i=0; i<head; i++) {
+		free(pdinfo->S[i]);
+	    }
+	    mvsize = rem * sizeof *pdinfo->S;
+	    memmove(pdinfo->S, pdinfo->S + head, mvsize);
+	}
+
+	if (pdinfo->paninfo != NULL) {
+	    mvsize = rem * sizeof *pdinfo->paninfo->unit;
+	    memmove(pdinfo->paninfo->unit, pdinfo->paninfo->unit + head, mvsize);
+	    memmove(pdinfo->paninfo->period, pdinfo->paninfo->period + head, mvsize);
+	}
+
+	/* FIXME panel data */
+
+	if (pdinfo->structure == CROSS_SECTION) {
+	    ntodate(pdinfo->stobs, 0, pdinfo);
+	} else {
+	    ntodate(pdinfo->stobs, pdinfo->t1, pdinfo);
+	    pdinfo->sd0 = get_date_x(pdinfo->pd, pdinfo->stobs);
+	}
+	pdinfo->t1 = 0;
+	pdinfo->t2 -= head;
+	pdinfo->n -= head;
+	ntodate(pdinfo->endobs, pdinfo->n - 1, pdinfo);
+    }
+
+    if (tail > 0) {
+	err = dataset_drop_observations(tail, pZ, pdinfo);
+    }
+
+    return err;
+}
+
 static int 
 dataset_expand_varinfo (int newvars, DATAINFO *pdinfo)
 {
     char **varname = NULL;
-    char *vector = NULL;
     VARINFO **varinfo = NULL;
     int v = pdinfo->v;
     int bigv = v + newvars;
@@ -930,17 +1221,6 @@ dataset_expand_varinfo (int newvars, DATAINFO *pdinfo)
 	}
     }
 
-    vector = realloc(pdinfo->vector, bigv);
-    if (vector == NULL) {
-	return E_ALLOC;
-    }
-
-    pdinfo->vector = vector;
-
-    for (i=0; i<newvars; i++) {
-	pdinfo->vector[v+i] = 1;
-    }
-
     pdinfo->v += newvars;
 
     return 0;
@@ -1066,7 +1346,7 @@ int dataset_add_scalars (int n, double ***pZ, DATAINFO *pdinfo)
 
     if (!err) {
 	for (i=0; i<n; i++) {
-	    pdinfo->vector[v+i] = 0;
+	    set_var_scalar(pdinfo, v + i, 1);
 	}
     }
 
@@ -1117,7 +1397,7 @@ int dataset_scalar_to_vector (int v, double ***pZ, DATAINFO *pdinfo)
 	    tmp[t] = NADBL;
 	}
 	(*pZ)[v] = tmp;
-	pdinfo->vector[v] = 1;
+	set_var_scalar(pdinfo, v, 0);
     }
 
     return err;
@@ -1184,7 +1464,7 @@ int dataset_copy_variable_as (int v, const char *newname,
 {
     int t, err;
 
-    if (pdinfo->vector[v]) {
+    if (var_is_series(pdinfo, v)) {
 	err = real_dataset_add_series(1, NULL, pZ, pdinfo);
     } else {
 	err = dataset_add_scalar(pZ, pdinfo);
@@ -1193,7 +1473,7 @@ int dataset_copy_variable_as (int v, const char *newname,
     if (!err) {
 	int vnew = pdinfo->v - 1;
 
-	if (pdinfo->vector[v]) {
+	if (var_is_series(pdinfo, v)) {
 	    for (t=0; t<pdinfo->n; t++) {
 		(*pZ)[vnew][t] = (*pZ)[v][t];
 	    }
@@ -1226,7 +1506,6 @@ shrink_dataset_to_size (double ***pZ, DATAINFO *pdinfo, int nv, int drop)
 
     if (drop == DROP_NORMAL) {
 	char **varname;
-	char *vector;
 	VARINFO **varinfo;
     
 	varname = realloc(pdinfo->varname, nv * sizeof *varname);
@@ -1235,12 +1514,6 @@ shrink_dataset_to_size (double ***pZ, DATAINFO *pdinfo, int nv, int drop)
 	}
 	pdinfo->varname = varname;
 
-	vector = realloc(pdinfo->vector, nv * sizeof *vector);
-	if (vector == NULL) {
-	    return E_ALLOC;
-	}
-	pdinfo->vector = vector;
-
 	varinfo = realloc(pdinfo->varinfo, nv * sizeof *varinfo);
 	if (varinfo == NULL) {
 	    return E_ALLOC;
@@ -1281,11 +1554,6 @@ static int real_dataset_drop_listed_vars (const int *list, double ***pZ,
 #if DROPDBG
     fprintf(stderr, "dataset_drop_listed_variables: dropping %d vars:\n",
 	    list[0]);
-    if (drop == DROP_NORMAL) {
-	for (i=1; i<=list[0]; i++) {
-	    fprintf(stderr, " %d: %s\n", list[i], pdinfo->varname[list[i]]);
-	}
-    }
 #endif
 
     /* free and set to NULL all the vars to be deleted */
@@ -1305,6 +1573,10 @@ static int real_dataset_drop_listed_vars (const int *list, double ***pZ,
 	}
     }
 
+    if (ndel == 0) {
+	return 0;
+    }
+
     /* rearrange pointers if necessary */
 
     for (v=1; v<vmax; v++) {
@@ -1324,12 +1596,11 @@ static int real_dataset_drop_listed_vars (const int *list, double ***pZ,
 		for (i=v; i<vmax; i++) {
 		    if (drop == DROP_NORMAL) {
 			if (renumber != NULL && 
-			    !is_hidden_variable(i + gap, pdinfo)) {
+			    !var_is_hidden(pdinfo, i + gap)) {
 			    *renumber = 1;
 			}
 			pdinfo->varname[i] = pdinfo->varname[i + gap];
 			pdinfo->varinfo[i] = pdinfo->varinfo[i + gap];
-			pdinfo->vector[i] = pdinfo->vector[i + gap];
 		    }
 		    Z[i] = Z[i + gap];
 		}		    
@@ -1343,12 +1614,6 @@ static int real_dataset_drop_listed_vars (const int *list, double ***pZ,
     return shrink_dataset_to_size(pZ, pdinfo, oldv - ndel, drop);
 }
 
-/* in subsample.c */
-
-extern double ***fetch_full_Z (void);
-extern void reset_full_Z (double ***pZ);
-extern DATAINFO *fetch_full_datainfo (void);
-
 /**
  * dataset_drop_listed_variables:
  * @list: list of variable to drop, by ID number.
@@ -1408,22 +1673,27 @@ int dataset_drop_variable (int v, double ***pZ, DATAINFO *pdinfo)
  * dataset_destroy_hidden_variables:
  * @pZ: pointer to data array.
  * @pdinfo: dataset information.
+ * @vmin: do not drop variables with ID numbers less than this.
  *
- * Deletes from the dataset all "hidden" variables that have
+ * Deletes from the dataset any "hidden" variables that have
  * been added automatically (for example, auto-generated variables
- * used for the x-axis in graph plotting).  Does not delete the
+ * used for the x-axis in graph plotting), and that have ID
+ * numbers greater than or equal to @vmin.  Never deletes the
  * automatically generated constant (ID number 0).
  *
  * Returns: 0 on success, %E_ALLOC on error.
  */
 
-int dataset_destroy_hidden_variables (double ***pZ, DATAINFO *pdinfo)
+int dataset_destroy_hidden_variables (double ***pZ, DATAINFO *pdinfo,
+				      int vmin)
 {
     int i, nhid = 0;
     int err = 0;
 
-    for (i=1; i<pdinfo->v; i++) {
-	if (is_hidden_variable(i, pdinfo)) {
+    if (vmin <= 1) vmin = 1;
+
+    for (i=vmin; i<pdinfo->v; i++) {
+	if (var_is_hidden(pdinfo, i)) {
 	    nhid++;
 	}
     }
@@ -1436,8 +1706,8 @@ int dataset_destroy_hidden_variables (double ***pZ, DATAINFO *pdinfo)
 	} else {
 	    int j = 1;
 
-	    for (i=1; i<pdinfo->v; i++) {
-		if (is_hidden_variable(i, pdinfo)) {
+	    for (i=vmin; i<pdinfo->v; i++) {
+		if (var_is_hidden(pdinfo, i)) {
 		    hidlist[j++] = i;
 		}
 	    }	    
@@ -1729,7 +1999,7 @@ int dataset_stack_variables (double ***pZ, DATAINFO *pdinfo,
 
 	    j = (vnum == NULL)? i + v1 : vnum[i];
 
-	    if (pdinfo->vector[j]) {
+	    if (var_is_series(pdinfo, j)) {
 		ok = pdinfo->n - missing_tail((*pZ)[j], pdinfo->n);
 	    } else {
 		ok = 1;
@@ -1758,7 +2028,7 @@ int dataset_stack_variables (double ***pZ, DATAINFO *pdinfo,
     /* extend length of all series? */
     oldn = pdinfo->n;
     if (bign > oldn) {
-	err = dataset_add_observations(bign - oldn, pZ, pdinfo);
+	err = dataset_add_observations(bign - oldn, pZ, pdinfo, OPT_NONE);
 	if (err) {
 	    free(bigx);
 	    goto bailout;
@@ -1780,7 +2050,7 @@ int dataset_stack_variables (double ***pZ, DATAINFO *pdinfo,
 	}
 
 	for (t=offset; t<tmax; t++) {
-	    if (pdinfo->vector[j]) {
+	    if (var_is_series(pdinfo, j)) {
 		bigx[bigt] = (*pZ)[j][t];
 	    } else {
 		bigx[bigt] = (*pZ)[j][0];
@@ -1831,25 +2101,134 @@ int dataset_stack_variables (double ***pZ, DATAINFO *pdinfo,
 }
 
 /**
- * is_hidden_variable:
+ * is_log_variable:
  * @i: ID number of variable.
  * @pdinfo: dataset information.
+ * @parent: location to which to write the name of the
+ * "parent" variable if any.
  *
- * Used in various contexts to screen a list of variables being 
- * presented to the user.
+ * Tries to determine if the variable with ID number @i is
+ * the logarithm of some other variable.
  *
- * Returns: 1 if variable @i is a "hidden", automatically
- * generated variable, otherwise 0.  
+ * Returns: 1 if variable @i appears to be a log, else 0.
  */
 
-int is_hidden_variable (int i, const DATAINFO *pdinfo)
+int is_log_variable (int i, const DATAINFO *pdinfo, char *parent)
 {
-    if (strcmp(pdinfo->varname[i], "annual") == 0 ||
-	strcmp(pdinfo->varname[i], "qtrs") == 0 ||
-	strcmp(pdinfo->varname[i], "months") == 0 ||
-	strcmp(pdinfo->varname[i], "hrs") == 0 ||
-	strcmp(pdinfo->varname[i], "decdate") == 0) {
-	return 1;
+    const char *s = VARLABEL(pdinfo, i);
+
+    *parent = '\0';
+
+    if (s != NULL && *s != '\0') {
+	if (sscanf(s, "= log of %15s", parent) == 1) {
+	    return 1;
+	} else {
+	    s += strcspn(s, "=");
+	    if (!strncmp(s, "=log(", 5)) {
+		int len;
+
+		s += 5;
+		len = gretl_varchar_spn(s);
+		if (len < VNAMELEN && s[len] == ')') {
+		    sscanf(s, "%15[^)]", parent);
+		    return 1;
+		}
+	    }
+	}
+    }
+
+    return 0;
+}
+
+/**
+ * set_var_discrete:
+ * @pdinfo: pointer to data information struct.
+ * @i: index number of variable.
+ * @s: non-zero to mark variable as discrete, zero to 
+ * mark as not discrete.
+ *
+ * Mark a variable as being discrete or not.
+ */
+
+void set_var_discrete (DATAINFO *pdinfo, int i, int s) 
+{
+    if (i > 0 && i < pdinfo->v) {
+	if (s) {
+	    pdinfo->varinfo[i]->flags |= VAR_DISCRETE;
+	} else {
+	    pdinfo->varinfo[i]->flags &= ~VAR_DISCRETE;
+	}
+    }
+}
+
+/**
+ * set_var_scalar:
+ * @pdinfo: pointer to data information struct.
+ * @i: index number of variable.
+ * @s: non-zero to mark variable as a scalar, zero to 
+ * mark as not scalar (i.e. a series).
+ *
+ * Mark a variable as being a scalar or not.
+ */
+
+void set_var_scalar (DATAINFO *pdinfo, int i, int s) 
+{
+    if (i > 0 && i < pdinfo->v) {
+	if (s) {
+	    pdinfo->varinfo[i]->flags |= VAR_SCALAR;
+	} else {
+	    pdinfo->varinfo[i]->flags &= ~VAR_SCALAR;
+	}
+    }
+}
+
+/**
+ * set_var_hidden:
+ * @pdinfo: pointer to data information struct.
+ * @i: index number of variable.
+ *
+ * Mark a variable as being "hidden" (an automatically
+ * generated variable that will not be shown in the main
+ * GUI window).
+ */
+
+void set_var_hidden (DATAINFO *pdinfo, int i) 
+{
+    if (i > 0 && i < pdinfo->v) {
+	pdinfo->varinfo[i]->flags |= VAR_HIDDEN;
+    }
+}
+
+/**
+ * var_set_linewidth:
+ * @pdinfo: pointer to data information struct.
+ * @i: index number of variable.
+ * @w: with of plot line.
+ *
+ * Set the line width for use when this variable is displayed
+ * in a line graph.
+ */
+
+void var_set_linewidth (DATAINFO *pdinfo, int i, int w) 
+{
+    if (w >= 1 && w <= 32 && i > 0 && i < pdinfo->v) {
+	pdinfo->varinfo[i]->line_width = w;
+    }
+}
+
+/**
+ * var_get_linewidth:
+ * @pdinfo: pointer to data information struct.
+ * @i: index number of variable.
+ *
+ * Returns: the line width set for use when graphing
+ * variable @i.
+ */
+
+int var_get_linewidth (const DATAINFO *pdinfo, int i) 
+{
+    if (i > 0 && i < pdinfo->v) {
+	return pdinfo->varinfo[i]->line_width;
     } else {
 	return 0;
     }
