@@ -30,7 +30,7 @@
 # include <windows.h>
 #endif
 
-#if defined(ENABLE_NLS) && defined(USE_GTK2)
+#if defined(ENABLE_NLS) && defined(USE_GLIB2)
 # include <glib.h>
 #endif
 
@@ -116,6 +116,68 @@ int gretl_minmax (int t1, int t2, const double *x,
     return 0;
 }
 
+/**
+ * gretl_min:
+ * @t1: starting observation.
+ * @t2: ending observation.
+ * @x: data series.
+ *
+ * Returns: the minimum value of @x over the given range,
+ * or #NADBL if no valid vaues are found.
+ */
+
+double gretl_min (int t1, int t2, const double *x)
+{
+    double min, max;
+
+    gretl_minmax(t1, t2, x, &min, &max);
+    return min;
+}
+
+/**
+ * gretl_max:
+ * @t1: starting observation.
+ * @t2: ending observation.
+ * @x: data series.
+ *
+ * Returns: the maximum value of @x over the given range,
+ * or #NADBL if no valid vaues are found.
+ */
+
+double gretl_max (int t1, int t2, const double *x)
+{
+    double min, max;
+
+    gretl_minmax(t1, t2, x, &min, &max);
+    return max;
+}
+
+/**
+ * gretl_sum:
+ * @t1: starting observation.
+ * @t2: ending observation.
+ * @x: data series.
+ *
+ * Returns: the sum of the series @x from obs
+ * @t1 to obs @t2, skipping any missing values, or #NADBL 
+ * in case there are no valid observations.
+ */
+
+double gretl_sum (int t1, int t2, const double *x)
+{
+    double sum = 0.0;
+    int t, n = 0;
+
+    for (t=t1; t<=t2; t++) {
+	if (!(na(x[t]))) {
+	    sum += x[t];
+	    n++;
+	}
+    }
+    
+    return (n == 0)? NADBL : sum;
+}
+
 /**
  * gretl_mean:
  * @t1: starting observation.
@@ -129,21 +191,14 @@ int gretl_minmax (int t1, int t2, const double *x,
 
 double gretl_mean (int t1, int t2, const double *x)
 {
-    int n;
-    register int t;
     double xbar, sum = 0.0;
-
-    n = t2 - t1 + 1;
-    if (n <= 0) {
-	return NADBL;
-    }
+    int t, n = 0;
 
     for (t=t1; t<=t2; t++) {
 	if (!(na(x[t]))) {
 	    sum += x[t];
-	} else {
-	    n--;
-	}
+	    n++;
+	} 
     }
 
     if (n == 0) {
@@ -803,9 +858,14 @@ int gretl_moments (int t1, int t2, const double *x,
 
 void free_freq (FreqDist *freq)
 {
+    if (freq == NULL) {
+	return;
+    }
+
     free(freq->midpt);
     free(freq->endpt);
     free(freq->f);
+
     free(freq);
 }
 
@@ -867,7 +927,7 @@ double dh_root_b1_to_z1 (double rb1, double n)
  * normal score, z2, as set out in Doornik and Hansen, "An Omnibus 
  * Test for Normality", 1994.
  *
- * Returns: the z2 value.
+ * Returns: the z2 value, or #NADBL on failure.
  */
 
 double dh_b2_to_z2 (double b1, double b2, double n)
@@ -884,11 +944,15 @@ double dh_b2_to_z2 (double b1, double b2, double n)
     k = ((n+5) * (n+7) * (n2*n + 37*n2 + 11*n - 313.0)) / (12.0 * d);
 
     alpha = a + b1 * c;
+    z2 =  (1.0 / (9.0 * alpha)) - 1.0;
 
     chi = (b2 - 1.0 - b1) * 2.0 * k;
 
-    z2 = (pow(chi/(2*alpha), 1.0/3.0) - 1.0 + (1.0 / (9.0*alpha))) *
-	sqrt(9.0*alpha);
+    if (chi > 0.0) {
+       z2 += pow(chi/(2*alpha), 1.0 / 3.0);
+    }
+
+    z2 *= sqrt(9.0*alpha);
 
     return z2;
 }
@@ -918,7 +982,11 @@ double doornik_chisq (double skew, double xkurt, int n)
     z1 = dh_root_b1_to_z1(rb1, (double) n);
     z2 = dh_b2_to_z2(b1, b2, (double) n);
 
-    return z1*z1 + z2*z2;
+    if (!na(z2)) {
+	z2 = z1*z1 + z2*z2;
+    }
+
+    return z2;
 }
 
 static int
@@ -1080,8 +1148,13 @@ gretl_system_normality_test (const gretl_matrix *E, const gretl_matrix *Sigma,
 	    double z1i = dh_root_b1_to_z1(skew, n);
 	    double z2i = dh_b2_to_z2(skew * skew, kurt, n);
 
-	    gretl_vector_set(Z1, i, z1i);
-	    gretl_vector_set(Z2, i, z2i);
+	    if (na(z2i)) {
+		X2 = NADBL;
+		err = E_NAN;
+	    } else {
+		gretl_vector_set(Z1, i, z1i);
+		gretl_vector_set(Z2, i, z2i);
+	    }
 	}
     }
 	
@@ -1095,7 +1168,7 @@ gretl_system_normality_test (const gretl_matrix *E, const gretl_matrix *Sigma,
     } else {
 	pputs(prn, "Test for multivariate normality of residuals\n");
 	pprintf(prn, "Doornik-Hansen Chi-square(%d) = %g, ", 2 * p, X2);
-	pprintf(prn, "with p-value = %g\n", chisq(X2, 2 * p));
+	pprintf(prn, "with p-value = %g\n", chisq_cdf_comp(X2, 2 * p));
     }
 
  bailout:
@@ -1112,7 +1185,7 @@ gretl_system_normality_test (const gretl_matrix *E, const gretl_matrix *Sigma,
     return err;
 }
 
-static freq_add_arrays (FreqDist *freq, int nbins)
+static int freq_add_arrays (FreqDist *freq, int nbins)
 {
     int err = 0;
 	
@@ -1130,6 +1203,17 @@ static freq_add_arrays (FreqDist *freq, int nbins)
     return err;
 }
 
+static int count_distinct_int_values (int *x, int n)
+{
+    int i, c = 1;
+
+    for (i=1; i<n; i++) {
+	if (x[i] != x[i-1]) c++;
+    }
+
+    return c;
+}
+
 /**
  * get_freq:
  * @varno: ID number of variable to process.
@@ -1137,12 +1221,12 @@ static freq_add_arrays (FreqDist *freq, int nbins)
  * @pdinfo: information on the data set.
  * @params: degrees of freedom loss (generally = 1 unless we're dealing
  * with the residual from a regression)
- * @opt: if & OPT_O, compare with gamma distribution, not normal
+ * @opt: if & OPT_O, compare with gamma distribution, not normal;
+ * if includes OPT_Q, do not show a histogram.
  *
  * Calculates the frequency distribution for the specified variable.
  *
- * Returns: struct containing the distribution.
- *
+ * Returns: pointer to struct containing the distribution.
  */
 
 FreqDist *get_freq (int varno, const double **Z, const DATAINFO *pdinfo, 
@@ -1188,7 +1272,7 @@ FreqDist *get_freq (int varno, const double **Z, const DATAINFO *pdinfo,
 	return NULL;
     } 
 
-    freq->discrete = gretl_isdiscrete(pdinfo->t1, pdinfo->t2, x);
+    freq->discrete = var_is_discrete(pdinfo, varno);
 
     gretl_moments(pdinfo->t1, pdinfo->t2, x, 
 		  &freq->xbar, &freq->sdx, 
@@ -1201,10 +1285,10 @@ FreqDist *get_freq (int varno, const double **Z, const DATAINFO *pdinfo,
     if (freq->n > 7) {
 	if (opt & OPT_O) {
 	    freq->test = lockes_test(x, pdinfo->t1, pdinfo->t2);
-	    freq->dist = DIST_GAMMA;
+	    freq->dist = D_GAMMA;
 	} else {
 	    freq->test = doornik_chisq(skew, kurt, freq->n); 
-	    freq->dist = DIST_NORMAL;
+	    freq->dist = D_NORMAL;
 	}
     } else {
 	freq->test = NADBL;
@@ -1218,62 +1302,68 @@ FreqDist *get_freq (int varno, const double **Z, const DATAINFO *pdinfo,
     }
     
     if (freq->discrete) {
-	double *sorted = NULL;
 	int *ifreq = NULL, *ivals = NULL;
-	int i;
+	int *sorted = NULL;
+	int i, last, err = 0;
 
 	sorted = malloc(n * sizeof *sorted);
-	ifreq = malloc(n * sizeof *ifreq);
-	ivals = malloc(n * sizeof *ivals);
-
-	if (sorted == NULL || ifreq == NULL || ivals == NULL) {
-	    free(sorted);
-	    free(ifreq);
-	    free(ivals);
+	if (sorted == NULL) {
 	    free_freq(freq);
 	    return NULL;
 	}
 	
-	i = 0;
+	n = 0;
 	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	    if (!na(x[t])) {
-		sorted[i++] = x[t];
+		sorted[n++] = (int) x[t];
 	    }
 	}
 	
-	qsort(sorted, n, sizeof *sorted, gretl_compare_doubles); 
-	nbins = 1;
-	xx = sorted[0];
-	ifreq[0] = 1;
-	ivals[0] = floor(xx);
-	
+	qsort(sorted, n, sizeof *sorted, gretl_compare_ints); 
+	nbins = count_distinct_int_values(sorted, n);
+
+	ifreq = malloc(nbins * sizeof *ifreq);
+	ivals = malloc(nbins * sizeof *ivals);
+	if (ifreq == NULL || ivals == NULL) {
+	    err = E_ALLOC;
+	    goto discrete_end;
+	}
+
+	ivals[0] = last = sorted[0];
+	ifreq[0] = i = 1;
+
 	for (t=1; t<n; t++) {
-	    if (sorted[t] != xx) {
-		xx = sorted[t];
-		ifreq[nbins] = 1;
-		ivals[nbins++] = floor(xx);
+	    if (sorted[t] != last) {
+		last = sorted[t];
+		ifreq[i] = 1;
+		ivals[i++] = last;
 	    } else {
-		ifreq[nbins-1] += 1;
+		ifreq[i-1] += 1;
 	    }
 	}
 
 	if (freq_add_arrays(freq, nbins)) {
+	    err = E_ALLOC;
+	} else {
+	    for (k=0; k<nbins; k++) {
+		freq->endpt[k] = freq->midpt[k] = ivals[k];
+		freq->f[k] = ifreq[k];
+	    }
+	    freq->endpt[nbins] = xmax;
+	}
+
+    discrete_end:
+
+	free(sorted);
+	free(ivals);
+	free(ifreq);
+	
+	if (err) {
 	    gretl_errno = E_ALLOC;
 	    strcpy(gretl_errmsg, _("Out of memory for frequency distribution"));
-	    return freq;
+	    free_freq(freq);
+	    freq = NULL;
 	}
-	
-	for (k=0; k<nbins; k++) {
-	    freq->endpt[k] = ivals[k];
-	    freq->midpt[k] = ivals[k];
-	    freq->f[k] = ifreq[k];
-	}
-	
-	freq->endpt[nbins] = xmax;
-
-	free(ifreq);
-	free(ivals);
-	free(sorted);
     } else {
 	/* not discrete (integer) data */
 	if (n < 16) {
@@ -1343,7 +1433,7 @@ FreqDist *get_freq (int varno, const double **Z, const DATAINFO *pdinfo,
 }
 
 int freqdist (int varno, const double **Z, const DATAINFO *pdinfo,
-	      int graph, PRN *prn, gretlopt opt)
+	      int graph, gretlopt opt, PRN *prn)
 {
     FreqDist *freq;
 
@@ -1356,7 +1446,7 @@ int freqdist (int varno, const double **Z, const DATAINFO *pdinfo,
     print_freq(freq, prn); 
 
     if (graph && !(opt & OPT_Q)) {
-	if (plot_freq(freq, (opt)? DIST_GAMMA : DIST_NORMAL)) {
+	if (plot_freq(freq, (opt)? D_GAMMA : D_NORMAL)) {
 	    pputs(prn, _("gnuplot command failed\n"));
 	}
     }
@@ -1366,15 +1456,335 @@ int freqdist (int varno, const double **Z, const DATAINFO *pdinfo,
     return 0;
 }
 
+/**
+ * free_xtab:
+ * @xtab: gretl crosstab struct
+ *
+ * Frees all malloced elements of the struct, and then
+ * the pointer itself.
+ */
+
+void free_xtab (Xtab *tab)
+{
+    int i;
+
+    free(tab->rtotal);
+    free(tab->ctotal);
+    free(tab->rval);
+    free(tab->cval);
+
+    if (tab->f != NULL) {
+	for (i=0; i<tab->rows; i++) {
+	    free(tab->f[i]);
+	}
+	free(tab->f);
+    }
+
+    free(tab);
+}
+
+static Xtab *xtab_new (int n, int t1, int t2)
+{
+    Xtab *tab = malloc(sizeof *tab);
+
+    if (tab == NULL) return NULL;
+
+    tab->rtotal = NULL;
+    tab->ctotal = NULL;
+    tab->rval = NULL;
+    tab->cval = NULL;
+    tab->f = NULL;
+
+    tab->n = n;
+    tab->t1 = t1;
+    tab->t2 = t2;
+
+    return tab;
+}
+
+static int xtab_allocate_arrays (Xtab *tab, int rows, int cols)
+{
+    int i, j;
+
+    tab->rows = rows;
+    tab->cols = cols;
+
+    tab->rval = malloc(rows * sizeof tab->rval);
+    tab->rtotal = malloc(rows * sizeof tab->rtotal);
+
+    tab->cval = malloc(cols * sizeof tab->cval);
+    tab->ctotal = malloc(cols * sizeof tab->ctotal);
+
+    tab->f = malloc(rows * sizeof tab->f);
+
+    if (tab->rval == NULL || tab->rtotal == NULL ||
+	tab->cval == NULL || tab->ctotal == NULL ||
+	tab->f == NULL) {
+	return E_ALLOC;
+    }
+
+    for (i=0; i<rows; i++) {
+	tab->f[i] = NULL;
+    }
+
+    for (i=0; i<rows; i++) {
+	tab->f[i] = malloc(cols * sizeof tab->f[i]);
+	if (tab->f[i] == NULL) {
+	    return E_ALLOC;
+	} else {
+	    for (j=0; j<cols; j++) {
+		tab->f[i][j] = 0;
+	    }
+	}
+    }
+
+    return 0;
+}
+
+int compare_xtab_rows (const void *a, const void *b) 
+{
+    const int **da = (const int **) a;
+    const int **db = (const int **) b;
+    int ret;
+    
+    ret = da[0][0] - db[0][0];
+
+    if (ret == 0) {
+	ret = da[0][1] - db[0][1];
+    }
+    
+    return ret;
+}
+
+/**
+  crosstab struct creation function
+*/
+
+static Xtab *get_xtab (int rvarno, int cvarno, const double **Z, 
+		       const DATAINFO *pdinfo)
+{
+    Xtab *tab = NULL;
+    int **X = NULL;
+    int rx = 0, cx = 0;
+    int rows, cols;
+
+    FreqDist *rowfreq, *colfreq;
+
+    int t, i, nr = 0, nc = 0;
+    int t1 = pdinfo->t1;
+    int t2 = pdinfo->t2;
+    int n = 0;
+    int err = 0;
+
+    /* count non-missing values */
+    for (t=t1; t<=t2; t++) {
+	if (!(na(Z[rvarno][t]) || na(Z[cvarno][t]))) {
+	    n++;
+	}
+    }
+
+    if (n == 0) {
+	fprintf(stderr, "All values invalid!\n");
+	return NULL;
+    }
+
+    /* Put in some info we already know */
+
+    tab = xtab_new(n, t1, t2);
+    if (tab == NULL) {
+	return NULL;
+    }
+
+    tab->missing = (t2 - t1 + 1) - n;
+    strcpy(tab->rvarname, pdinfo->varname[rvarno]);
+    strcpy(tab->cvarname, pdinfo->varname[cvarno]);
+
+    /* 
+       start allocating stuff; we use temporary FreqDists for rows 
+       and columns to retrieve values with non-zero frequencies
+       and get dimensions for the cross table
+     */
+
+    rowfreq = get_freq(rvarno, Z, pdinfo, 0, OPT_D); 
+    colfreq = get_freq(cvarno, Z, pdinfo, 0, OPT_D); 
+    X = malloc(n * sizeof *X);
+
+    if (rowfreq == NULL || colfreq == NULL || X == NULL) {
+	free_freq(rowfreq);
+	free_freq(colfreq);
+	free(X);
+	return NULL;
+    }
+
+    for (i=0; i<n; i++) {
+	X[i] = NULL;
+    }
+
+    rows = rowfreq->numbins;
+    cols = colfreq->numbins;
+
+    if (xtab_allocate_arrays(tab, rows, cols)) {
+	free_xtab(tab);
+	return NULL;
+    }
+
+    for (i=0; i<rows; i++) {
+	tab->rval[i] = rowfreq->midpt[i];
+	tab->rtotal[i] = 0;
+    }
+
+    for (i=0; i<cols; i++) {
+	tab->cval[i] = colfreq->midpt[i];
+	tab->ctotal[i] = 0;
+    }
+
+    /* done with FreqDists */
+    free_freq(rowfreq);
+    free_freq(colfreq);
+
+    /* 
+       build the matrix X holding the values to be sorted
+    */
+
+    i = 0;
+    for (t=t1; t<=t2 && i<n; t++) {
+	X[i] = malloc(2 * sizeof **X);
+	if (X[i] == NULL) {
+	    err = E_ALLOC;
+	    break;
+	} else if (!(na(Z[rvarno][t]) || na(Z[cvarno][t]))) { 
+	    X[i][0] = (int) Z[rvarno][t];
+	    X[i][1] = (int) Z[cvarno][t];
+	    i++;
+	}
+    }
+
+    if (err) {
+	free_xtab(tab);
+	tab = NULL;
+    } else {
+	qsort(X, n, sizeof *X, compare_xtab_rows);
+
+	rx = cx = 0;
+	nr = tab->rval[0];
+	nc = tab->cval[0];
+    }
+
+    /* compute frequencies by going through sorted X */
+
+    for (i=0; i<n && !err; i++) {
+	while (X[i][0] > nr) { /* skip row */
+	    nr = tab->rval[++rx];
+	    cx = 0;
+	    nc = tab->cval[0];
+	}
+	while (X[i][1] > nc) { /* skip column */
+	    nc = tab->cval[++cx];
+	}
+#if XTAB_DEBUG
+	fprintf(stderr,"%d: (%d,%d) [%d,%d] %d,%d\n",
+		i, rx, cx, nr, nc, X[i][0], X[i][1]);
+#endif
+	tab->f[rx][cx] += 1;
+	tab->rtotal[rx] += 1;
+	tab->ctotal[cx] += 1;
+    }
+
+    /* we're done; free stuff and quit */
+
+    if (X != NULL) {
+	for (i=0; i<n; i++) {
+	    free(X[i]);
+	}
+	free(X);
+    }
+
+    return tab;
+}
+
+int crosstab (const int *list, const double **Z, 
+	      const DATAINFO *pdinfo, gretlopt opt, 
+	      PRN *prn)
+{
+    int *rowvar = NULL;
+    int *colvar = NULL;
+    int i, j, k;
+
+    int pos = gretl_list_separator_position(list);
+    int nrv = pos - 1;
+    int ncv = list[0] - pos;
+    int err = 0;
+
+    if (pos == 0 || nrv == 0 || ncv == 0) {
+	return E_PARSE;
+    }
+
+    rowvar = gretl_list_new(nrv);
+    colvar = gretl_list_new(ncv);
+    if (rowvar == NULL || colvar == NULL) {
+	free(rowvar);
+	free(colvar);
+	return E_ALLOC;
+    }
+
+    j = 1;
+    for (i=1; i<=nrv; i++) {
+	if (var_is_discrete(pdinfo, list[i]) && !var_is_scalar(pdinfo, list[i])) {
+	    rowvar[j++] = list[i];
+	} else {
+	    rowvar[0] -= 1;
+	}
+    }
+
+    j = 1;
+    for (i=1; i<=ncv; i++) {
+	k = pos + i;
+	if (var_is_discrete(pdinfo, list[k]) && !var_is_scalar(pdinfo, list[k])) {
+	    colvar[j++] = list[k];
+	} else {
+	    colvar[0] -= 1;
+	}
+    }
+
+    if (rowvar[0] == 0 || colvar[0] == 0) {
+	free(rowvar);
+	free(colvar);
+	return E_DATATYPE;
+    }
+
+    for (i=1; i<=rowvar[0] && !err; i++) {
+	for (j=1; j<=colvar[0] && !err; j++) {
+	    Xtab *tab = get_xtab(rowvar[i], colvar[j], Z, pdinfo); 
+
+	    if (tab == NULL) {
+		err = E_ALLOC;
+	    } else {
+		print_xtab(tab, opt, prn); 
+		free_xtab(tab);
+	    }
+	}
+    }
+
+    free(rowvar);
+    free(colvar);
+
+    return err;
+}
+
+/* ---------- end crosstab stuff -------------------- */
+
 int model_error_dist (const MODEL *pmod, double ***pZ,
 		      DATAINFO *pdinfo, PRN *prn)
 {
     FreqDist *freq = NULL;
     int err = 0;
 
+    if (pmod == NULL || pmod->uhat == NULL) {
+	return E_DATA;
+    }
+
     if (genr_fit_resid(pmod, pZ, pdinfo, GENR_RESID, 1)) {
-	pputs(prn, _("Out of memory attempting to add variable\n"));
-	err = E_ALLOC;
+	return E_ALLOC;
     }
 
     if (!err) {
@@ -1393,7 +1803,6 @@ int model_error_dist (const MODEL *pmod, double ***pZ,
     dataset_drop_last_variables(1, pZ, pdinfo);
 
     return err;
-
 }
 
 /* PACF via Durbin-Levinson algorithm */
@@ -1496,6 +1905,9 @@ static char *corrgm_crit_string (void)
  * corrgram:
  * @varno: ID number of variable to process.
  * @order: integer order for autocorrelation function.
+ * @nparam: number of estimated parameters (e.g. for the
+ * case of ARMA), used to correct the degrees of freedom 
+ * for Q test.
  * @pZ: pointer to data matrix.
  * @pdinfo: information on the data set.
  * @prn: gretl printing struct.
@@ -1509,12 +1921,14 @@ static char *corrgm_crit_string (void)
  * Returns: 0 on successful completion, error code on error.
  */
 
-int corrgram (int varno, int order, double ***pZ, 
+int corrgram (int varno, int order, int nparam, double ***pZ, 
 	      DATAINFO *pdinfo, PRN *prn, gretlopt opt)
 {
-    double *acf, box, pm;
+    double box, pm90, pm95, pm99;
+    double *acf = NULL;
     double *pacf = NULL;
-    int k, l, acf_m, pacf_m, nobs; 
+    int k, l, acf_m, pacf_m; 
+    int nobs, dfQ;
     int t, t1 = pdinfo->t1, t2 = pdinfo->t2;
     int list[2];
     FILE *fq = NULL;
@@ -1560,34 +1974,6 @@ int corrgram (int varno, int order, double ***pZ,
 	acf[l-1] = gretl_acf(nobs, l, &(*pZ)[varno][t1]);
     }
 
-    if (opt & OPT_R) {
-	pprintf(prn, "\n%s\n\n", _("Residual autocorrelation function"));
-    } else {
-	sprintf(gretl_tmp_str, _("Autocorrelation function for %s"), 
-		pdinfo->varname[varno]);
-	pprintf(prn, "\n%s\n\n", gretl_tmp_str);
-    }
-
-    /* add Ljung-Box statistic */
-    box = 0;
-    for (t=0; t<acf_m; t++) { 
-	box += acf[t] * acf[t] / (nobs - t + 1);
-    }
-    box *= nobs * (nobs + 2.0);
-
-    pprintf(prn, "Ljung-Box Q' = %.4f\n", box);
-    pprintf(prn, _("Degrees of freedom = %d, p-value = %.4f\n\n"),
-	    acf_m, chisq(box, acf_m));
-
-    /* print acf */
-    for (t=0; t<acf_m; t++) {
-	pprintf(prn, "%5d)%8.4f", t + 1, acf[t]);
-	if ((t + 1) % 5 == 0) {
-	    pputc(prn, '\n');
-	}
-    }
-    pputc(prn, '\n');
-
     if (opt & OPT_A) { 
 	/* use ASCII graphics, not gnuplot */
 	double *xl = malloc(acf_m * sizeof *xl);
@@ -1605,6 +1991,14 @@ int corrgram (int varno, int order, double ***pZ,
 	free(xl);
     } 
 
+    if (opt & OPT_R) {
+	pprintf(prn, "\n%s\n\n", _("Residual autocorrelation function"));
+    } else {
+	sprintf(gretl_tmp_str, _("Autocorrelation function for %s"), 
+		pdinfo->varname[varno]);
+	pprintf(prn, "\n%s\n\n", gretl_tmp_str);
+    }
+
     /* determine lag order for pacf (may have to be shorter than acf_m) */
     if (acf_m > nobs / 2 - 1) {
 	pacf_m = nobs / 2 - 1;
@@ -1622,36 +2016,51 @@ int corrgram (int varno, int order, double ***pZ,
     }
 
     /* for confidence bands */
-    pm = 1.96 / sqrt((double) nobs);
+    pm90 = 1.65 / sqrt((double) nobs);
+    pm95 = 1.96 / sqrt((double) nobs);
+    pm99 = 2.58 / sqrt((double) nobs);
 
     err = pacf_err = get_pacf(pacf, acf, pacf_m);
 
-    if (!err) {
-	pacf[0] = acf[0];
-	pprintf(prn, "\n%s", _("Partial autocorrelations"));
-	if (pacf_m < acf_m) {
-	    pprintf(prn, " (%s %d):\n\n", _("to lag"), pacf_m);
+    pputs(prn, _("  LAG      ACF          PACF         Q-stat. [p-value]"));
+    pputs(prn, "\n\n");
+
+    box = 0.0;
+    dfQ = 1;
+
+    for (t=0; t<acf_m; t++) {
+	pprintf(prn, "%5d%9.4f ", t + 1, acf[t]);
+	if (fabs(acf[t]) > pm99) {
+	    pputs(prn, " ***");
+	} else if (fabs(acf[t]) > pm95) {
+	    pputs(prn, " ** ");
+	} else if (fabs(acf[t]) > pm90) {
+	    pputs(prn, " *  ");
 	} else {
-	    pputs(prn, ":\n\n");
+	    pputs(prn, "    ");
 	}
-	for (k=0; k<pacf_m; k++) {
-	    pprintf(prn, "%5d)%8.4f", k+1, pacf[k]);
-	    if ((k + 1) % 5 == 0) {
-		pputc(prn, '\n');
+
+	if (t < pacf_m) {
+	    pprintf(prn, "%9.4f", pacf[t]);
+	    if (fabs(pacf[t]) > pm99) {
+		pputs(prn, " ***");
+	    } else if (fabs(pacf[t]) > pm95) {
+		pputs(prn, " ** ");
+	    } else if (fabs(pacf[t]) > pm90) {
+		pputs(prn, " *  ");
+	    } else {
+		pputs(prn, "    ");
 	    }
 	}
-    }
-    pputc(prn, '\n');
-    if (pacf_m % 5 > 0) {
+
+	box += (nobs * (nobs + 2.0)) * acf[t] * acf[t] / (nobs - t + 1);
+	pprintf(prn, "%12.4f", box);
+	if (t >= nparam) {
+	    pprintf(prn, "  [%5.3f]", chisq_cdf_comp(box, dfQ++));
+	}
 	pputc(prn, '\n');
     }
 
-    pprintf(prn, "%s: %s = %g\n", 
-	    /* xgettext:no-c-format */
-	    _("5% critical value"),
-	    corrgm_crit_string(),
-	    pm);
-
     if (opt & OPT_A) {
 	goto acf_getout;
     } else if (gnuplot_init(PLOT_CORRELOGRAM, &fq)) {
@@ -1682,9 +2091,9 @@ int corrgram (int varno, int order, double ***pZ,
     }
     fprintf(fq, "set xrange [0:%d]\n", acf_m + 1);
     fprintf(fq, "plot \\\n"
-	    "'-' using 1:2 notitle w impulses, \\\n"
+	    "'-' using 1:2 notitle w impulses lw 5, \\\n"
 	    "%g title '+- %s' lt 2, \\\n"
-	    "%g notitle lt 2\n", pm, corrgm_crit_string(), -pm);
+	    "%g notitle lt 2\n", pm95, corrgm_crit_string(), -pm95);
     for (k=0; k<acf_m; k++) {
 	fprintf(fq, "%d %g\n", k + 1, acf[k]);
     }
@@ -1701,9 +2110,9 @@ int corrgram (int varno, int order, double ***pZ,
 	}
 	fprintf(fq, "set xrange [0:%d]\n", pacf_m + 1);
 	fprintf(fq, "plot \\\n"
-		"'-' using 1:2 notitle w impulses, \\\n"
+		"'-' using 1:2 notitle w impulses lw 5, \\\n"
 		"%g title '+- %s' lt 2, \\\n"
-		"%g notitle lt 2\n", pm, corrgm_crit_string(), -pm);
+		"%g notitle lt 2\n", pm95, corrgm_crit_string(), -pm95);
 	for (k=0; k<pacf_m; k++) {
 	    fprintf(fq, "%d %g\n", k + 1, pacf[k]);
 	}
@@ -1858,7 +2267,7 @@ gretl_matrix *gretl_matrix_periodogram (const gretl_matrix *x, int m)
 gretl_matrix *LWE_lambda (const gretl_matrix *I, int n, double *lcm)
 {
     int m = gretl_vector_get_length(I);
-    gretl_matrix *lambda, *llambda;
+    gretl_matrix *lambda;
     int i;
 
     lambda = gretl_column_vector_alloc(m);
@@ -1867,20 +2276,20 @@ gretl_matrix *LWE_lambda (const gretl_matrix *I, int n, double *lcm)
 	gretl_vector_set(lambda, i, (2.0 * M_PI / n) * (i + 1));
 #if LWE_DEBUG
 	fprintf(stderr, "LWE_obj_func: lambda[%d] = %g\n",
-		i, gretl_vector_get(lambda, i));
+		i, lambda->val[i]);
 #endif
     }
 
-    llambda = gretl_matrix_copy(lambda);
-    gretl_matrix_transform_elements(llambda, T_LOG);
-    *lcm = gretl_vector_mean(llambda);
+    *lcm = 0.0;
+    for (i=0; i<m; i++) {
+	*lcm += log(lambda->val[i]);
+    }
+    *lcm /= m;
 
 #if LWE_DEBUG
     fprintf(stderr, "LWE_lambda: col mean of log lambda = %g\n", *lcm);
 #endif
 
-    gretl_matrix_free(llambda);
-
     return lambda;
 }
 
@@ -2027,10 +2436,10 @@ int fract_int_LWE (const double **Z, int varno, int t1, int t2,
  * @varno: ID number of variable to process.
  * @pZ: pointer to data matrix.
  * @pdinfo: information on the data set.
- * @prn: gretl printing struct.
  * @opt: if includes %OPT_O, use Bartlett lag window for periodogram;
  * if includes %OPT_N, don't display gnuplot graph; if includes
  * %OPT_R, the variable is a model residual.
+ * @prn: gretl printing struct.
  *
  * Computes and displays the periodogram for the variable specified 
  * by @varno.
@@ -2040,7 +2449,7 @@ int fract_int_LWE (const double **Z, int varno, int t1, int t2,
  */
 
 int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo, 
-		 PRN *prn, gretlopt opt)
+		 gretlopt opt, PRN *prn)
 {
     double *autocov = NULL;
     double *omega = NULL;
@@ -2186,6 +2595,8 @@ int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo,
     pprintf(prn, _("Number of observations = %d\n"), nobs);
     if (window) {
 	pprintf(prn, _("Using Bartlett lag window, length %d\n\n"), L);
+    } else {
+	pputc(prn, '\n');
     }
     pputs(prn, _(" omega  scaled frequency  periods  spectral density\n\n"));
 
@@ -2345,7 +2756,7 @@ static void print_summary_single (const Summary *summ, int j,
 
     for (i=0; i<8; i++) {
 	if (strlen(_(labels[i])) > slen) {
-#if defined(ENABLE_NLS) && defined(USE_GTK2)
+#if defined(ENABLE_NLS) && defined(USE_GLIB2)
 	    slen = g_utf8_strlen(_(labels[i]), -1);	    
 #else
 	    slen = strlen(_(labels[i]));
@@ -2369,8 +2780,7 @@ static void print_summary_single (const Summary *summ, int j,
  * @pdinfo: information on the data set.
  * @prn: gretl printing struct.
  *
- * Print the summary statistics for a given variable.
- *
+ * Prints the summary statistics for a given variable.
  */
 
 void print_summary (const Summary *summ,
@@ -2470,21 +2880,12 @@ void print_summary (const Summary *summ,
  * @summ: gretl summary statistics struct
  *
  * Frees all malloced elements of the struct.
- *
  */
 
 void free_summary (Summary *summ)
 {
     free(summ->list);
-
-    free(summ->mean);
-    free(summ->median);
-    free(summ->sd);
-    free(summ->skew);
-    free(summ->xkurt);
-    free(summ->low);
-    free(summ->high); 
-    free(summ->cv);
+    free(summ->stats);
 
     free(summ);
 }
@@ -2508,27 +2909,21 @@ static Summary *summary_new (const int *list)
     summ->n = 0;
     summ->missing = 0;
 
-    summ->mean = summ->median = summ->sd = NULL;
-    summ->skew = summ->xkurt = NULL;
-    summ->low = summ->high = summ->cv = NULL;
-
-    summ->mean = malloc(nv * sizeof *summ->mean);
-    summ->median = malloc(nv * sizeof *summ->median);
-    summ->sd = malloc(nv * sizeof *summ->sd);
-    summ->skew = malloc(nv * sizeof *summ->skew);
-    summ->xkurt = malloc(nv * sizeof *summ->xkurt);
-    summ->low = malloc(nv * sizeof *summ->low);
-    summ->high = malloc(nv * sizeof *summ->high);
-    summ->cv = malloc(nv * sizeof *summ->cv);
-
-    if (summ->mean == NULL || summ->median == NULL ||
-	summ->sd == NULL || summ->skew == NULL ||
-	summ->xkurt == NULL || summ->low == NULL ||
-	summ->high == NULL || summ->cv == NULL) {
+    summ->stats = malloc(8 * nv * sizeof *summ->stats);
+    if (summ->stats == NULL) {
 	free_summary(summ);
 	return NULL;
     }
 
+    summ->mean = summ->stats;
+    summ->median = summ->mean + nv;
+    summ->sd = summ->median + nv;
+    summ->skew = summ->sd + nv;
+    summ->xkurt = summ->skew + nv;
+    summ->low = summ->xkurt + nv;
+    summ->high = summ->low + nv;
+    summ->cv = summ->high + nv;
+
     return summ;
 }
 
@@ -2542,12 +2937,10 @@ static Summary *summary_new (const int *list)
  * Calculates descriptive summary statistics for the specified variables.
  *
  * Returns: struct containing the summary statistics.
- *
  */
 
-Summary *summary (const int *list, 
-		       const double **Z, const DATAINFO *pdinfo,
-		       PRN *prn) 
+Summary *summary (const int *list, const double **Z, 
+		  const DATAINFO *pdinfo, PRN *prn) 
 {
     Summary *summ;
     int i, vi, sn, gn;
@@ -2694,7 +3087,7 @@ VMatrix *corrlist (int *list, const double **Z, const DATAINFO *pdinfo)
     corrmat->dim = lo = list[0];  
     mm = (lo * (lo + 1)) / 2;
 
-    corrmat->names = create_strings_array(lo);
+    corrmat->names = strings_array_new(lo);
     if (corrmat->names == NULL) {
 	free(corrmat);
 	return NULL;
@@ -2943,7 +3336,7 @@ int vars_test (const int *list, const double **Z, const DATAINFO *pdinfo,
 	dfd = n1 - 1;
     }
 
-    pval = fdist(F, dfn, dfd);
+    pval = f_cdf_comp(F, dfn, dfd);
 
     pputs(prn, _("\nEquality of variances test\n\n"));
     pprintf(prn, "   %s: ", pdinfo->varname[list[1]]);
@@ -2955,7 +3348,7 @@ int vars_test (const int *list, const double **Z, const DATAINFO *pdinfo,
 	    _("The two population variances are equal"));
     pprintf(prn, "   %s: F(%d,%d) = %g\n", _("Test statistic"), dfn, dfd, F);
     pprintf(prn, _("   p-value (two-tailed) = %g\n\n"), pval);
-    if (fdist(F, dfn, dfd) > .10)
+    if (f_cdf_comp(F, dfn, dfd) > .10)
 	pputs(prn, _("   The difference is not statistically significant.\n\n"));
 
     record_test_result(F, pval, _("difference of variances"));
@@ -3323,7 +3716,7 @@ static int lorenz_graph (const char *vname, double *lz, int n)
 /**
  * gini:
  * @vnum: ID number of variable to examine.
- * @Z: data matrix.
+ * @pZ: pointer to data matrix.
  * @pdinfo: data information struct.
  * @opt: not used yet.
  * @prn: gretl printing struct.
