@@ -22,12 +22,14 @@
 
 typedef struct Summary_ Summary;
 typedef struct FreqDist_ FreqDist;
+typedef struct Xtab_ Xtab;
 typedef struct MahalDist_ MahalDist;
 
 struct Summary_ {
     int n;
     int missing;
     int *list;
+    double *stats;
     double *mean;
     double *median;
     double *sd;
@@ -54,6 +56,17 @@ struct FreqDist_ {
     int t1, t2;
 };
 
+struct Xtab_ {
+    char rvarname[VNAMELEN]; 
+    char cvarname[VNAMELEN]; 
+    int rows, cols;
+    int *rval, *cval;
+    int *rtotal, *ctotal;
+    int **f;
+    int n, missing;
+    int t1, t2;
+};
+
 /* functions follow */
 
 int eval_ytest (double y, GretlOp op, double test);
@@ -61,6 +74,12 @@ int eval_ytest (double y, GretlOp op, double test);
 int gretl_minmax (int t1, int t2, const double *x, 
 		  double *min, double *max);
 
+double gretl_min (int t1, int t2, const double *x);
+
+double gretl_max (int t1, int t2, const double *x);
+
+double gretl_sum (int t1, int t2, const double *x);
+
 double gretl_mean (int t1, int t2, const double *x);
 
 double gretl_restricted_mean (int t1, int t2, const double *x,
@@ -102,23 +121,27 @@ FreqDist *get_freq (int varno, const double **Z, const DATAINFO *pdinfo,
 		    int params, gretlopt opt);
 
 int freqdist (int varno, const double **Z, const DATAINFO *pdinfo,
-	      int graph, PRN *prn, gretlopt opt);
+	      int graph, gretlopt opt, PRN *prn);
+
+int crosstab (const int *list, const double **Z, 
+	      const DATAINFO *pdinfo, gretlopt opt,
+	      PRN *prn);
 
 int model_error_dist (const MODEL *pmod, double ***pZ,
 		      DATAINFO *pdinfo, PRN *prn);
 
 int auto_acf_order (int pd, int nobs);
 
-int corrgram (int varno, int order, 
+int corrgram (int varno, int order, int nparam,
 	      double ***pZ, DATAINFO *pdinfo, 
 	      PRN *prn, gretlopt opt);
 
 int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo, 
-		 PRN *prn, gretlopt opt);
+		 gretlopt opt, PRN *prn);
 
 Summary *summary (const int *list, const double **Z, 
-		       const DATAINFO *pdinfo,
-		       PRN *prn);
+		  const DATAINFO *pdinfo,
+		  PRN *prn);
 
 void print_summary (const Summary *summ,
 		    const DATAINFO *pdinfo,
