@@ -20,6 +20,7 @@
 /* discrete.c for gretl: logit and probit models */
 
 #include "libgretl.h"
+#include "libset.h"
 
 #define TINY 1.0e-13
 
@@ -127,7 +128,9 @@ static int add_slopes_to_model (MODEL *pmod, double fbx)
 	slopes[i] = pmod->coeff[i] * fbx;
     }
 
-    if (gretl_model_set_data(pmod, "slopes", slopes, ssize)) {
+    if (gretl_model_set_data(pmod, "slopes", slopes, 
+			     MODEL_DATA_DOUBLE_ARRAY,
+			     ssize)) {
 	free(slopes);
 	return 1;
     }
@@ -135,7 +138,7 @@ static int add_slopes_to_model (MODEL *pmod, double fbx)
     return 0;
 }
 
-static double *hess_wts (MODEL *pmod, const double **Z, int opt) 
+static double *hess_wts (MODEL *pmod, const double **Z, int ci) 
 {
     int t, tw, n = pmod->t2 - pmod->t1 + 1;
     double q, bx, xx;
@@ -156,7 +159,7 @@ static double *hess_wts (MODEL *pmod, const double **Z, int opt)
 	q = 2.0 * Z[pmod->list[1]][t] - 1.0;
 	bx = pmod->yhat[t];
 
-	if (opt == LOGIT) {
+	if (ci == LOGIT) {
 	    w[tw] = -1.0 * logit(bx) * (1.0 - logit(bx));
 	} else {
 	    xx = (q * normal_pdf(q * bx)) / normal_cdf(q * bx);
@@ -167,7 +170,7 @@ static double *hess_wts (MODEL *pmod, const double **Z, int opt)
     return w;
 }
 
-static double *hessian (MODEL *pmod, const double **Z, int opt) 
+static double *hessian (MODEL *pmod, const double **Z, int ci) 
 {
     int i, j, li, lj, m, t;
     const int l0 = pmod->list[0];
@@ -181,7 +184,7 @@ static double *hessian (MODEL *pmod, const double **Z, int opt)
 	return NULL;
     }
 
-    wt = hess_wts(pmod, Z, opt);
+    wt = hess_wts(pmod, Z, ci);
     if (wt == NULL) {
 	free(xpx);
 	return NULL;
@@ -298,7 +301,7 @@ compute_QML_vcv (MODEL *pmod, const double **Z)
 	}
     }
 
-    gretl_model_set_int(pmod, "robust", 1);
+    gretl_model_set_int(pmod, "ml_vcv", VCV_QML);
 
  bailout:
 
@@ -637,6 +640,7 @@ MODEL logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo,
 
     if (act_pred != NULL) {
 	gretl_model_set_data(&dmod, "discrete_act_pred", act_pred, 
+			     MODEL_DATA_INT_ARRAY, 
 			     4 * sizeof *act_pred);
     }
 
@@ -662,49 +666,50 @@ MODEL logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo,
    X'X must be Cholesky-decomposed already.
 */
 
-static int neginv (const double *xpx, double *diag, int nv)
+static int neginv (const double *xpx, double *diag, int n)
 {
     int kk, l, m, k, i, j;
-    const int nxpx = nv * (nv + 1) / 2;
-    double d, e, *tmp;
+    const int nxpx = n * (n + 1) / 2;
+    double *tmp;
+    double d, e;
 
-    tmp = malloc((nv + 1) * sizeof *tmp);
+    tmp = malloc((n + 1) * sizeof *tmp);
     if (tmp == NULL) {
-	return 1;
+	return E_ALLOC;
     }
 
-    for (i=0; i<=nv; i++) {
+    for (i=0; i<=n; i++) {
 	tmp[i] = 0.0;
     }
 
     kk = 0;
 
-    for (l=1; l<=nv-1; l++) {
+    for (l=1; l<=n-1; l++) {
         d = xpx[kk];
         tmp[l] = d;
         e = d * d;
         m = 0;
         if (l > 1) {
 	    for (j=1; j<=l-1; j++) {
-		m += nv - j;
+		m += n - j;
 	    }
 	}
-        for (i=l+1; i<=nv; i++) {
+        for (i=l+1; i<=n; i++) {
             d = 0.0;
             k = i + m - 1;
             for (j=l; j<=i-1; j++) {
                 d += tmp[j] * xpx[k];
-                k += nv - j;
+                k += n - j;
             }
             d = -d * xpx[k];
             tmp[i] = d;
             e += d * d;
         }
-        kk += nv + 1 - l;
+        kk += n + 1 - l;
         diag[l-1] = e;
     }
 
-    diag[nv - 1] = xpx[nxpx - 1] * xpx[nxpx - 1];
+    diag[n - 1] = xpx[nxpx - 1] * xpx[nxpx - 1];
 
     free(tmp);
 
@@ -851,7 +856,7 @@ static int rewrite_logistic_stats (const double **Z, const DATAINFO *pdinfo,
     pmod->sdy = gretl_stddev(pmod->t1, pmod->t2, Z[dv]);
 
     /* make the VCV matrix before messing with the model stats */
-    makevcv(pmod);
+    makevcv(pmod, pmod->sigma);
 
     pmod->ess = 0.0;
     for (t=0; t<pdinfo->n; t++) {
