@@ -21,6 +21,7 @@
 
 #include "libgretl.h"
 #include "qr_estimate.h"
+#include "gretl_panel.h"
 #include "libset.h"
 #include "compat.h"
 #include "missing_private.h"
@@ -48,7 +49,7 @@
 			     squares value to be effectively zero */
 
 /* define for lots of debugging info */
-#undef XPX_DEBUG
+#define XPX_DEBUG 0
 
 /* private function prototypes */
 static int form_xpxxpy (const int *list, int t1, int t2, 
@@ -56,8 +57,8 @@ static int form_xpxxpy (const int *list, int t1, int t2,
 			double *xpx, double *xpy, const char *mask);
 static void regress (MODEL *pmod, double *xpy, double **Z, 
 		     int n, double rho);
-static int cholbeta (double *xpx, double *xpy, double *coeff, double *rss,
-		     int nv);
+static int cholbeta (double *xpx, double *xpy, double *coeff, int nv, 
+		     double *rss);
 static void diaginv (double *xpx, double *xpy, double *diag, int nv);
 
 static int hatvar (MODEL *pmod, int n, double **Z);
@@ -130,50 +131,6 @@ static void model_depvar_stats (MODEL *pmod, const double **Z)
     pmod->sdy = (sum >= 0)? sqrt(sum) : NADBL;
 }
 
-/* initial setup for structure to hold info on autoregressive
-   coefficients 
-*/
-
-static int ar_info_init (MODEL *pmod, int nterms)
-{
-    int i;
-
-    pmod->arinfo = malloc(sizeof *pmod->arinfo);
-    if (pmod->arinfo == NULL) {
-	return 1;
-    }
-
-    pmod->arinfo->arlist = gretl_list_new(nterms);
-    if (pmod->arinfo->arlist == NULL) {
-	free(pmod->arinfo);
-	pmod->arinfo = NULL;
-	return 1; 
-    }
-
-    pmod->arinfo->rho = malloc(nterms * sizeof *pmod->arinfo->rho);
-    if (pmod->arinfo->rho == NULL) {
-	free(pmod->arinfo->arlist);
-	free(pmod->arinfo);
-	pmod->arinfo = NULL;
-	return 1; 
-    }
-
-    pmod->arinfo->sderr = malloc(nterms * sizeof *pmod->arinfo->sderr);
-    if (pmod->arinfo->sderr == NULL) {
-	free(pmod->arinfo->arlist);
-	free(pmod->arinfo->rho);
-	free(pmod->arinfo);
-	pmod->arinfo = NULL;
-	return 1; 
-    }
-
-    for (i=0; i<nterms; i++) {
-	pmod->arinfo->sderr[i] = pmod->arinfo->rho[i] = NADBL;
-    }
-
-    return 0;
-}
-
 /* determine the degrees of freedom for a model */
 
 static int get_model_df (MODEL *pmod)
@@ -204,7 +161,7 @@ transcribe_ld_vcv (MODEL *targ, MODEL *src)
     int nxpx = (nv * nv + nv) / 2;
     int i, j;
 
-    if (makevcv(src)) {
+    if (makevcv(src, src->sigma)) {
 	return 1;
     }
 
@@ -360,7 +317,7 @@ static int compute_ar_stats (MODEL *pmod, const double **Z, double rho)
     int i, t, yno = pmod->list[1];
     double x, pw1 = 0.0;
 
-    if (ar_info_init(pmod, 1)) {
+    if (gretl_model_add_arinfo(pmod, 1)) {
 	pmod->errcode = E_ALLOC;
 	return 1;
     }
@@ -408,7 +365,7 @@ static int compute_ar_stats (MODEL *pmod, const double **Z, double rho)
 static void get_wls_stats (MODEL *pmod, const double **Z)
 {
     int t, wobs = pmod->nobs, yno = pmod->list[1];
-    double x, dy, w2, wmean = 0.0, wsum = 0.0;
+    double x, dy, wmean = 0.0, wsum = 0.0;
 
     for (t=pmod->t1; t<=pmod->t2; t++) { 
 	if (model_missing(pmod, t)) {
@@ -418,9 +375,8 @@ static void get_wls_stats (MODEL *pmod, const double **Z)
 	    wobs--;
 	    pmod->dfd -= 1;
 	} else {
-	    w2 = Z[pmod->nwt][t] * Z[pmod->nwt][t];
-	    wmean += w2 * Z[yno][t];
-	    wsum += w2;
+	    wmean += Z[pmod->nwt][t] * Z[yno][t];
+	    wsum += Z[pmod->nwt][t];
 	}
     }
 
@@ -431,12 +387,11 @@ static void get_wls_stats (MODEL *pmod, const double **Z)
 	if (model_missing(pmod, t) || Z[pmod->nwt][t] == 0.0) {
 	    continue;
 	}	
-	w2 = Z[pmod->nwt][t] * Z[pmod->nwt][t]; 
 	dy = Z[yno][t] - wmean;
-	x += w2 * dy * dy;
+	x += Z[pmod->nwt][t] * dy * dy;
     }
 
-    pmod->fstt = ((x - pmod->ess) * pmod->dfd)/(pmod->dfn * pmod->ess);
+    pmod->fstt = ((x - pmod->ess) * pmod->dfd) / (pmod->dfn * pmod->ess);
     pmod->rsq = (1 - (pmod->ess / x));
     pmod->adjrsq = 1 - ((1 - pmod->rsq) * (pmod->nobs - 1)/pmod->dfd);
 }
@@ -452,9 +407,9 @@ static void fix_wls_values (MODEL *pmod, double **Z)
 	    }
 	}
     } else {
-	pmod->ess_wt = pmod->ess;
-	pmod->sigma_wt = pmod->sigma;
-	pmod->ess = 0.0;
+	double ess_orig = 0.0;
+	double sw, sigma_orig;
+
 	for (t=pmod->t1; t<=pmod->t2; t++) {
 	    if (model_missing(pmod, t)) {
 		continue;
@@ -463,12 +418,16 @@ static void fix_wls_values (MODEL *pmod, double **Z)
 		pmod->yhat[t] = pmod->uhat[t] = NADBL;
 		pmod->nobs -= 1;
 	    } else {
-		pmod->yhat[t] /= Z[pmod->nwt][t];
-		pmod->uhat[t] /= Z[pmod->nwt][t];
-		pmod->ess += pmod->uhat[t] * pmod->uhat[t];
+		sw = sqrt(Z[pmod->nwt][t]);
+		pmod->yhat[t] /= sw;
+		pmod->uhat[t] /= sw;
+		ess_orig += pmod->uhat[t] * pmod->uhat[t];
 	    }
 	}
-	pmod->sigma = sqrt(pmod->ess / pmod->dfd);
+
+	sigma_orig = sqrt(ess_orig / pmod->dfd);
+	gretl_model_set_double(pmod, "ess_orig", ess_orig);
+	gretl_model_set_double(pmod, "sigma_orig", sigma_orig);
     }
 }
 
@@ -486,8 +445,8 @@ static void dropwt (int *list)
 
 static void model_stats_init (MODEL *pmod)
 {
-    pmod->ess = pmod->ess_wt = NADBL;
-    pmod->sigma = pmod->sigma_wt = NADBL;
+    pmod->ess = NADBL;
+    pmod->sigma = NADBL;
     pmod->fstt = pmod->lnL = NADBL;
     pmod->rsq = pmod->adjrsq = NADBL;
 }
@@ -539,11 +498,6 @@ lsq_check_for_missing_obs (MODEL *pmod, gretlopt opts,
 	/* we'll try to compensate for missing obs */
 	missv = adjust_t1t2(pmod, pmod->list, &pmod->t1, &pmod->t2,
 			    pdinfo->n, Z, NULL);
-	if (pmod->ci == POOLED && pmod->missmask != NULL) {
-	    if (!model_mask_leaves_balanced_panel(pmod, pdinfo)) {
-		gretl_model_set_int(pmod, "unbalanced", 1);
-	    } 
-	}
     }
 
 #if SMPL_DEBUG
@@ -575,6 +529,27 @@ lagged_depvar_check (MODEL *pmod, const double **Z, const DATAINFO *pdinfo)
     }
 }
 
+static void 
+log_depvar_ll (MODEL *pmod, const double **Z, const DATAINFO *pdinfo)
+{
+    char parent[VNAMELEN];
+
+    if (is_log_variable(pmod->list[1], pdinfo, parent)) {
+	double jll = pmod->lnL;
+	int t;
+
+	for (t=0; t<pdinfo->n; t++) {
+	    if (!na(pmod->uhat[t])) {
+		jll -= Z[pmod->list[1]][t];
+	    }
+	}
+	gretl_model_set_double(pmod, "jll", jll);
+	gretl_model_set_string_as_data(pmod, 
+				       "log-parent", 
+				       gretl_strdup(parent));
+    }
+}
+
 #define COLL_DEBUG 0
 
 int 
@@ -676,6 +651,114 @@ redundant_var (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, int **droplist)
     return ret;
 }
 
+static int check_weight_var (MODEL *pmod, const double *w, int *effobs)
+{
+    int t;
+
+    if (gretl_iszero(pmod->t1, pmod->t2, w)) {
+	pmod->errcode = E_WTZERO;
+	return 1;
+    }
+
+    for (t=pmod->t1; t<=pmod->t2; t++) {
+	if (w[t] < 0.0) {
+	    pmod->errcode = E_WTNEG;
+	    return 1;
+	}
+    }
+
+    *effobs = gretl_isdummy(pmod->t1, pmod->t2, w);
+
+    if (*effobs) {
+	/* the weight var is a dummy, with effobs 1s */
+	gretl_model_set_int(pmod, "wt_dummy", 1);
+    }
+
+    return 0;
+}
+
+static int gretl_choleski_regress (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, 
+				   double rho, int pwe, gretlopt opt)
+{
+    double *xpy = NULL;
+    int *droplist = NULL;
+    int l0 = pmod->list[0];
+    int i, k, nxpx;
+
+ trim_var:
+
+    if (droplist != NULL) {
+	l0 = pmod->list[0];
+	free(pmod->xpx);
+	free(pmod->coeff);
+	free(pmod->sderr);
+	pmod->errcode = 0;
+    }
+ 
+    k = l0 - 1;
+    nxpx = k * (k + 1) / 2;
+
+    if (nxpx == 0) {
+	fprintf(stderr, "problem: nxpx = 0 (l0 = %d)\n", l0);
+	pmod->errcode = E_DATA;
+	return pmod->errcode;
+    }
+
+    xpy = malloc((l0 + 1) * sizeof *xpy);
+    pmod->xpx = malloc(nxpx * sizeof *pmod->xpx);
+    pmod->coeff = malloc(pmod->ncoeff * sizeof *pmod->coeff);
+    pmod->sderr = malloc(pmod->ncoeff * sizeof *pmod->sderr);
+
+    if (pmod->yhat == NULL) {
+	pmod->yhat = malloc(pdinfo->n * sizeof *pmod->yhat);
+    } 
+
+    if (pmod->uhat == NULL) {
+	pmod->uhat = malloc(pdinfo->n * sizeof *pmod->uhat);
+    }
+
+    if (xpy == NULL || pmod->xpx == NULL || pmod->coeff == NULL ||
+	pmod->sderr == NULL || pmod->yhat == NULL || pmod->uhat == NULL) {
+	pmod->errcode = E_ALLOC;
+	return pmod->errcode;
+    }
+
+    for (i=0; i<=l0; i++) {
+	xpy[i] = 0.0;
+    }
+    for (i=0; i<nxpx; i++) {
+	pmod->xpx[i] = 0.0;
+    }
+
+    /* calculate regression results, Cholesky style */
+    form_xpxxpy(pmod->list, pmod->t1, pmod->t2, (const double **) *pZ, 
+		pmod->nwt, rho, pwe, pmod->xpx, xpy, pmod->missmask);
+
+#if XPX_DEBUG
+    for (i=0; i<=l0; i++) {
+	fprintf(stderr, "xpy[%d] = %g\n", i, xpy[i]);
+    }
+    for (i=0; i<nxpx; i++) {
+	fprintf(stderr, "xpx[%d] = %g\n", i, pmod->xpx[i]);
+    }
+    fputc('\n', stderr);
+#endif
+
+    regress(pmod, xpy, *pZ, pdinfo->n, rho);
+    free(xpy);
+
+    if (pmod->errcode == E_SINGULAR && !(opt & OPT_Z) &&
+	redundant_var(pmod, pZ, pdinfo, &droplist)) {
+	goto trim_var;
+    }
+
+    if (droplist != NULL) {
+	gretl_model_set_list_as_data(pmod, "droplist", droplist);
+    }
+
+    return pmod->errcode;
+}
+
 /* as lsq() below, except that we allow for a non-zero value
    of the first-order quasi-differencing coefficient, rho,
    and there's no PRN.
@@ -684,14 +767,13 @@ redundant_var (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, int **droplist)
 MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo, 
 	       GretlCmdIndex ci, gretlopt opt, double rho)
 {
-    int l0, yno, i;
+    MODEL mdl;
     int effobs = 0;
     int missv = 0, misst = 0;
     int jackknife = 0;
     int use_qr = get_use_qr();
     int pwe = (ci == PWE || (opt & OPT_P));
-    double *xpy;
-    MODEL mdl;
+    int yno, i;
 
     *gretl_errmsg = '\0';
 
@@ -735,16 +817,11 @@ MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
 
     /* Doing weighted least squares? */
     if (ci == WLS) { 
-	mdl.nwt = mdl.list[1];
-	if (gretl_iszero(mdl.t1, mdl.t2, (*pZ)[mdl.nwt])) {
-	    mdl.errcode = E_WTZERO;
+	check_weight_var(&mdl, (*pZ)[mdl.list[1]], &effobs);
+	if (mdl.errcode) {
 	    return mdl;
 	}
-	effobs = gretl_isdummy(mdl.t1, mdl.t2, (*pZ)[mdl.nwt]);
-	if (effobs) {
-	    /* the weight var is a dummy, with effobs 1s */
-	    gretl_model_set_int(&mdl, "wt_dummy", 1);
-	}
+	mdl.nwt = mdl.list[1];
     } else {
 	mdl.nwt = 0;
     }
@@ -790,7 +867,7 @@ MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
             mdl.errcode = E_UNKVAR;
             goto lsq_abort;
         }
-    }       
+    } 
 
     /* check for zero dependent var */
     if (depvar_zero(mdl.t1, mdl.t2, yno, mdl.nwt, (const double **) *pZ)) {  
@@ -817,8 +894,7 @@ MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
 	mdl.t1 += 1;
     }
 
-    l0 = mdl.list[0];  /* holds 1 + number of coeffs */
-    mdl.ncoeff = l0 - 1; 
+    mdl.ncoeff = mdl.list[0] - 1; 
     if (effobs) {
 	mdl.nobs = effobs; /* FIXME? */
     } else {
@@ -850,76 +926,7 @@ MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
 	mdl.rho = rho;
 	gretl_qr_regress(&mdl, pZ, pdinfo, opt);
     } else {
-	int *droplist = NULL;
-	int l, nxpx;
-
-    trim_var:
-
-	if (droplist != NULL) {
-	    l0 = mdl.list[0];
-	    free(mdl.xpx);
-	    free(mdl.coeff);
-	    free(mdl.sderr);
-	    mdl.errcode = 0;
-	}
- 
-	l = l0 - 1;
-	nxpx = l * (l + 1) / 2;
-
-	if (nxpx == 0) {
-	    fprintf(stderr, "problem: nxpx = 0 (l0 = %d)\n", l0);
-	}
-
-	xpy = malloc((l0 + 1) * sizeof *xpy);
-	mdl.xpx = malloc(nxpx * sizeof *mdl.xpx);
-	mdl.coeff = malloc(mdl.ncoeff * sizeof *mdl.coeff);
-	mdl.sderr = malloc(mdl.ncoeff * sizeof *mdl.sderr);
-
-	if (mdl.yhat == NULL) {
-	    mdl.yhat = malloc(pdinfo->n * sizeof *mdl.yhat);
-	} 
-	if (mdl.uhat == NULL) {
-	    mdl.uhat = malloc(pdinfo->n * sizeof *mdl.uhat);
-	}
-
-	if (xpy == NULL || mdl.xpx == NULL || mdl.coeff == NULL ||
-	    mdl.sderr == NULL || mdl.yhat == NULL || mdl.uhat == NULL) {
-	    mdl.errcode = E_ALLOC;
-	    return mdl;
-	}
-
-	for (i=0; i<=l0; i++) {
-	    xpy[i] = 0.0;
-	}
-	for (i=0; i<nxpx; i++) {
-	    mdl.xpx[i] = 0.0;
-	}
-
-	/* calculate regression results, Cholesky style */
-	form_xpxxpy(mdl.list, mdl.t1, mdl.t2, (const double **) *pZ, 
-		    mdl.nwt, rho, pwe, mdl.xpx, xpy, mdl.missmask);
-
-#ifdef XPX_DEBUG
-	for (i=0; i<=l0; i++) {
-	    fprintf(stderr, "xpy[%d] = %g\n", i, xpy[i]);
-	}
-	for (i=0; i<nxpx; i++) {
-	    fprintf(stderr, "xpx[%d] = %g\n", i, mdl.xpx[i]);
-	}
-	fputc('\n', stderr);
-#endif
-
-	regress(&mdl, xpy, *pZ, pdinfo->n, rho);
-	free(xpy);
-
-	if (mdl.errcode == E_SINGULAR && !(opt & OPT_Z) &&
-	    redundant_var(&mdl, pZ, pdinfo, &droplist)) {
-	    goto trim_var;
-	}
-
-	if (droplist != NULL) {
-	    gretl_model_set_list_as_data(&mdl, "droplist", droplist);
-	}
+	gretl_choleski_regress(&mdl, pZ, pdinfo, rho, pwe, opt);
     }
 
     if (mdl.errcode) {
@@ -944,15 +951,21 @@ MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
-    /* weighted least squares: fix fitted values, ESS, sigma */
+    /* weighted least squares: fix yhat and uhat; add calculation of
+       ESS and sigma based on unweighted data
+    */
     if (ci == WLS) {
 	get_wls_stats(&mdl, (const double **) *pZ);
 	fix_wls_values(&mdl, *pZ);
     }
 
-    if ((opt & OPT_T) && mdl.missmask == NULL) {
-	mdl.rho = rhohat(0, mdl.t1, mdl.t2, mdl.uhat);
-	mdl.dw = dwstat(0, &mdl, (const double **) *pZ);
+    if (mdl.missmask == NULL) {
+	if (opt & OPT_T) {
+	    mdl.rho = rhohat(1, mdl.t1, mdl.t2, mdl.uhat);
+	    mdl.dw = dwstat(1, &mdl, (const double **) *pZ);
+	} else if (dataset_is_panel(pdinfo)) {
+	    panel_dwstat(&mdl, pdinfo);
+	}
     } else {
 	mdl.rho = mdl.dw = NADBL;
     }
@@ -964,11 +977,10 @@ MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
     }
 
     /* Generate model selection statistics */
-    gretl_calculate_criteria((mdl.ci == WLS)? mdl.ess_wt : mdl.ess, 
-			     mdl.nobs, mdl.ncoeff, &mdl.lnL, 
-			     &mdl.criterion[C_AIC],
-			     &mdl.criterion[C_BIC],
-			     &mdl.criterion[C_HQC]);
+    ls_criteria(&mdl);
+    if (!(opt & OPT_A) && !na(mdl.lnL)) {
+	log_depvar_ll(&mdl, (const double **) *pZ, pdinfo);
+    }
 
     /* hccm command or HC3a */
     if (jackknife) {
@@ -1033,11 +1045,10 @@ MODEL lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
     Z[v][t] = observation t on variable v
     n = number of obs in data set
     t1, t2 = starting and ending observations
-    rho = first order serial correlation coefficent
+    rho = quasi-differencing coefficent
     nwt = ID number of variable used as weight
 
-    xpx = X'X matrix as a lower triangle
-          stacked by columns
+    xpx = X'X matrix as a lower triangle, stacked by columns
     xpy = X'y vector
     xpy[0] = sum of y
     xpy[list[0]] = y'y
@@ -1050,7 +1061,7 @@ static int form_xpxxpy (const int *list, int t1, int t2,
     int i, j, t;
     int li, lj, m;
     int l0 = list[0], yno = list[1];
-    double x, z1, pw1;
+    double x, pw1;
     int qdiff = (rho != 0.0);
 
     /* Prais-Winsten term */
@@ -1075,7 +1086,7 @@ static int form_xpxxpy (const int *list, int t1, int t2,
 		x -= rho * Z[yno][t-1];
 	    }
 	} else if (nwt) {
-	    x *= Z[nwt][t];
+	    x *= sqrt(Z[nwt][t]);
 	}
         xpy[0] += x;
         xpy[l0] += x * x;
@@ -1127,8 +1138,7 @@ static int form_xpxxpy (const int *list, int t1, int t2,
 		x = 0.0;
 		for (t=t1; t<=t2; t++) {
 		    if (!missing_masked(mask, t)) {
-			z1 = Z[nwt][t];
-			x += z1 * z1 * Z[li][t] * Z[lj][t];
+			x += Z[nwt][t] * Z[li][t] * Z[lj][t];
 		    }
 		}
 		if (floateq(x, 0.0) && li == lj)  {
@@ -1139,8 +1149,7 @@ static int form_xpxxpy (const int *list, int t1, int t2,
 	    x = 0.0;
 	    for (t=t1; t<=t2; t++) {
 		if (!missing_masked(mask, t)) {
-		    z1 = Z[nwt][t];
-		    x += z1 * z1 * Z[yno][t] * Z[li][t];
+		    x += Z[nwt][t] * Z[yno][t] * Z[li][t];
 		}
 	    }
 	    xpy[i-1] = x;
@@ -1196,7 +1205,7 @@ static int make_ess (MODEL *pmod, double **Z)
 	}
 	resid = Z[yno][t] - yhat;
 	if (nwt) {
-	    resid *= Z[nwt][t];
+	    resid *= sqrt(Z[nwt][t]);
 	}
 	pmod->ess += resid * resid;
     }
@@ -1204,17 +1213,57 @@ static int make_ess (MODEL *pmod, double **Z)
     return 0;
 }
 
-static void compute_r_squared (MODEL *pmod, double *y)
+int check_for_effective_const (MODEL *pmod, const double *y)
+{
+    double x1 = 0.0, x2 = 0.0;
+    double reldiff;
+    int t, ret = 0;
+
+    for (t=pmod->t1; t<=pmod->t2; t++) {
+	if (!na(pmod->yhat[t])) {
+	    x1 += pmod->yhat[t];
+	    x2 += y[t];
+	}
+    }
+
+    reldiff = fabs((x1 - x2) / x2);
+
+    if (floateq(reldiff, 0.0)) {
+	gretl_model_set_int(pmod, "effconst", 1);
+	pmod->dfn -= 1;
+	ret = 1;
+    } else if (gretl_model_get_int(pmod, "effconst")) {
+	gretl_model_set_int(pmod, "effconst", 0);
+	pmod->dfn += 1;
+    }
+
+    return ret;
+}
+
+static void compute_r_squared (MODEL *pmod, const double *y, int *ifc)
 {
     pmod->rsq = 1.0 - (pmod->ess / pmod->tss);
 
-    if (pmod->dfd > 0) {
-	double den = pmod->tss * pmod->dfd;
+    if (*ifc == 0) {
+	*ifc = check_for_effective_const(pmod, y);
+    }
 
-	if (pmod->ifc) {
+    if (pmod->dfd > 0) {
+	double den;
+
+	if (*ifc) {
+	    den = pmod->tss * pmod->dfd;
 	    pmod->adjrsq = 1 - (pmod->ess * (pmod->nobs - 1) / den);
 	} else {
-	    pmod->rsq = gretl_corr_rsq(pmod->t1, pmod->t2, y, pmod->yhat);
+	    int t;
+
+	    den = 0.0;
+	    for (t=pmod->t1; t<=pmod->t2; t++) {
+		if (!na(y[t])) {
+		    den += y[t] * y[t];
+		}
+	    }
+	    pmod->rsq = 1 - pmod->ess / den; /* NIST method */
 	    pmod->adjrsq = 
 		1.0 - ((1.0 - pmod->rsq) * (pmod->nobs - 1.0) / pmod->dfd);
 	} 
@@ -1229,20 +1278,21 @@ static void compute_r_squared (MODEL *pmod, double *y)
   regress: takes xpx, the X'X matrix produced by form_xpxxpy(), and
   xpy (X'y), and computes ols estimates and associated statistics.
 
-  n = number of observations per series in data set
-  ifc = 1 if constant is present, else = 0
+  n = total number of observations per series in data set
+  ifc = 1 if constant is present in model, else = 0
 
   ess = error sum of squares
   sigma = standard error of regression
   fstt = F-statistic
-  coeff = vector of regression coefficients
-  sderr = vector of standard errors of regression coefficients
+  coeff = array of regression coefficients
+  sderr = corresponding array of standard errors
 */
 
 static void regress (MODEL *pmod, double *xpy, double **Z, 
 		     int n, double rho)
 {
     int v, yno = pmod->list[1];
+    int ifc = pmod->ifc;
     double ysum, ypy, zz, rss = 0.0;
     double sgmasq = 0.0;
     double *diag = NULL;
@@ -1254,19 +1304,11 @@ static void regress (MODEL *pmod, double *xpy, double **Z,
 
     ysum = xpy[0];
     ypy = xpy[pmod->ncoeff + 1];
-
-#ifdef NO_LHS_ZERO
-    if (floateq(ypy, 0.0)) { 
-        pmod->errcode = E_YPY;
-        return; 
-    }
-#endif
-
     zz = ysum * ysum / pmod->nobs;
     pmod->tss = ypy - zz;
 
     /*  Cholesky-decompose X'X and find the coefficients */
-    err = cholbeta(pmod->xpx, xpy, pmod->coeff, &rss, pmod->ncoeff);
+    err = cholbeta(pmod->xpx, xpy, pmod->coeff, pmod->ncoeff, &rss);
     if (err) {
         pmod->errcode = err;
         return;
@@ -1279,7 +1321,7 @@ static void regress (MODEL *pmod, double *xpy, double **Z,
 	rss = ypy - pmod->ess;
     }
 
-    if (pmod->ess < ESSZERO && pmod->ess > (-ESSZERO)) {
+    if (fabs(pmod->ess) < ESSZERO) {
 	pmod->ess = 0.0;
     } else if (pmod->ess < 0.0) { 
 	sprintf(gretl_errmsg, _("Error sum of squares (%g) is not > 0"),
@@ -1307,7 +1349,7 @@ static void regress (MODEL *pmod, double *xpy, double **Z,
     if (pmod->errcode) return;
 
     if (pmod->tss > 0.0) {
-	compute_r_squared(pmod, Z[yno]);
+	compute_r_squared(pmod, Z[yno], &ifc);
     }
 
 #if 0
@@ -1319,8 +1361,10 @@ static void regress (MODEL *pmod, double *xpy, double **Z,
 
     if (sgmasq <= 0.0 || pmod->dfd == 0 || pmod->dfn == 0) {
 	pmod->fstt = NADBL;
+    } else if (pmod->rsq == 1.0) {
+	pmod->fstt = NADBL;
     } else {
-	pmod->fstt = (rss - zz * pmod->ifc) / (sgmasq * pmod->dfn);
+	pmod->fstt = (rss - zz * ifc) / (sgmasq * pmod->dfn);
 	if (pmod->fstt < 0.0) {
 	    pmod->fstt = 0.0;
 	}
@@ -1343,8 +1387,6 @@ static void regress (MODEL *pmod, double *xpy, double **Z,
     }
 
     free(diag); 
-    
-    return;  
 }
 
 /*
@@ -1356,20 +1398,21 @@ static void regress (MODEL *pmod, double *xpy, double **Z,
   xpy = the X'y vector on input and Cholesky-transformed t
         vector on output 
   coeff = array of estimated coefficients 
-  rss = pointer to receive regression sum of squares
   nv = number of regression coefficients including the constant
+  rss = location to receive regression sum of squares
 
   The number of floating-point operations is basically 3.5 * nv^2
   plus (nv^3) / 3.
 */
 
 static int 
-cholbeta (double *xpx, double *xpy, double *coeff, double *rss, int nv)
+cholbeta (double *xpx, double *xpy, double *coeff, int nv, double *rss)
 {
     int i, j, k, kk, l, jm1;
     double e, d, d1, d2, test, xx;
 
     if (xpx[0] <= 0.0) {
+	fprintf(stderr, "%s %d: xpx <= 0.0\n", __FILE__, __LINE__);
 	return E_NAN;
     }
 
@@ -1395,9 +1438,6 @@ cholbeta (double *xpx, double *xpy, double *coeff, double *rss, int nv)
         d2 = xpx[kk] - d;
 	test = d2 / xpx[kk];
         if (test < TINY) {
-#if 0
-	    fprintf(stderr, "cholbeta: test = %g\n", test);
-#endif
 	    *rss = -1.0;
 	    return E_SINGULAR;
         }
@@ -1423,7 +1463,7 @@ cholbeta (double *xpx, double *xpy, double *coeff, double *rss, int nv)
 
     kk--;
 
-    /* regression sum of squares */
+    /* calculate regression sum of squares */
     d = 0.0;
     for (j=1; j<=nv; j++) {
 	d += xpy[j] * xpy[j];
@@ -1434,7 +1474,9 @@ cholbeta (double *xpx, double *xpy, double *coeff, double *rss, int nv)
     for (j=0; j<nv-1; j++) {
 	coeff[j] = 0.0;
     }
+
     coeff[nv-1] = xpy[nv] * xpx[kk];
+
     for (j=nv-1; j>=1; j--) {
 	d = xpy[j];
 	for (i=nv-1; i>=j; i--) {
@@ -1444,8 +1486,10 @@ cholbeta (double *xpx, double *xpy, double *coeff, double *rss, int nv)
 	kk--;
 	coeff[j-1] = d * xpx[kk];
     }
+
     for (j=0; j<nv; j++) {
 	if (isnan(coeff[j])) {
+	    fprintf(stderr, "%s %d: coeff %d is NaN\n", __FILE__, __LINE__, j);
 	    return E_NAN;
 	}
     }	
@@ -1501,6 +1545,8 @@ static void diaginv (double *xpx, double *xpy, double *diag, int nv)
 /**
  * makevcv:
  * @pmod: pointer to model.
+ * @sigma: square root of error variance, or 1.0 to
+ * produce just X'X^{-1}.
  *
  * Inverts the Cholesky-decomposed X'X matrix and computes the 
  * coefficient covariance matrix.
@@ -1508,7 +1554,7 @@ static void diaginv (double *xpx, double *xpy, double *diag, int nv)
  * Returns: 0 on successful completion, non-zero code on error.
  */
 
-int makevcv (MODEL *pmod)
+int makevcv (MODEL *pmod, double sigma)
 {
     int dec, mst, kk, i, j, kj, icnt, m, k, l = 0;
     const int nv = pmod->ncoeff;
@@ -1568,21 +1614,15 @@ int makevcv (MODEL *pmod)
 	}
     }
 
-    if (pmod->ci == CUSUM) {
-	return 0;
+    if (pmod->ci == LOGIT || pmod->ci == PROBIT) {
+	sigma = 1.0;
     }
 
-    /* some estimators need special treatment */
+    if (sigma != 1.0) {
+	double s2 = sigma * sigma;
 
-    if (pmod->ci != HCCM && pmod->ci != LOGIT && pmod->ci != PROBIT) {
-	double sigma = pmod->sigma;
-
-	if ((pmod->ci == WLS && !(gretl_model_get_int(pmod, "wt_dummy"))) || 
-	    pmod->ci == ARCH || pmod->ci == HSK) {
-	    sigma = pmod->sigma_wt;
-	} 
 	for (k=0; k<nxpx; k++) {
-	    pmod->vcv[k] *= sigma * sigma;
+	    pmod->vcv[k] *= s2;
 	}
     }
 
@@ -1591,7 +1631,7 @@ int makevcv (MODEL *pmod)
 
 /**
  * dwstat:
- * @order: order of autoregression, 0 for OLS.
+ * @order: order of autoregression (usually 1).
  * @pmod: pointer to model.
  * @Z: data array.
  *
@@ -1602,46 +1642,58 @@ int makevcv (MODEL *pmod)
 
 double dwstat (int order, MODEL *pmod, const double **Z)
 {
-    double diff, ut, ut1;
-    double diffsq = 0.0;
-    int t;
-
-    if (order) order--;
+    double ut, u1;
+    double num = 0.0;
+    double den = 0.0;
+    int t, t1;
 
     if (pmod->ess <= 0.0) {
 	return NADBL;
     }
 
-    for (t=pmod->t1+1+order; t<=pmod->t2; t++)  {
-        ut = pmod->uhat[t];
-        ut1 = pmod->uhat[t-1];
-        if (na(ut) || na(ut1) ||
-	    (pmod->nwt && (floateq(Z[pmod->nwt][t], 0.0) || 
-			   floateq(Z[pmod->nwt][t-1], 0.0)))) { 
-	    continue;
+    t1 = pmod->t1 + order;
+
+    if (pmod->nwt) {
+	ut = pmod->uhat[t1 - 1];
+	if (!na(ut)) {
+	    den += ut * ut;
 	}
-        diff = ut - ut1;
-        diffsq += diff * diff;
+    } else {
+	den = pmod->ess;
     }
 
-    return diffsq / pmod->ess;
+    for (t=t1; t<=pmod->t2; t++)  {
+        ut = pmod->uhat[t];
+        u1 = pmod->uhat[t-1];
+        if (na(ut) || na(u1) ||
+	    (pmod->nwt && (Z[pmod->nwt][t] == 0.0 || 
+			   Z[pmod->nwt][t-1] == 0.0))) { 
+	    continue;
+	}
+        num += (ut - u1) * (ut - u1);
+	if (pmod->nwt) {
+	    den += ut * ut;
+	}
+    }
+
+    return num / den;
 }
 
 /* altrho: alternative calculation of rho */
 
 static double altrho (int order, int t1, int t2, const double *uhat)
 {
-    double *ut, *ut1;    
+    double *ut, *u1;    
     int t, n, len = t2 - (t1 + order) + 1;
-    double uh, uh1, rho;
+    double uht, uh1, rho;
 
     ut = malloc(len * sizeof *ut);
     if (ut == NULL) {
 	return NADBL;
     }
 
-    ut1 = malloc(len * sizeof *ut1);
-    if (ut1 == NULL) {
+    u1 = malloc(len * sizeof *u1);
+    if (u1 == NULL) {
 	free(ut);
 	return NADBL;
     }
@@ -1649,26 +1701,26 @@ static double altrho (int order, int t1, int t2, const double *uhat)
     n = 0;
 
     for (t=t1+order; t<=t2; t++) { 
-        uh = uhat[t];
+        uht = uhat[t];
 	uh1 = (t > 0)? uhat[t-1] : NADBL;
-        if (!na(uh) && !na(uh1)) {
-	    ut[n] = uh;
-	    ut1[n] = uh1;
+        if (!na(uht) && !na(uh1)) {
+	    ut[n] = uht;
+	    u1[n] = uh1;
 	    n++;
 	}
     }
 
-    rho = gretl_corr(0, n - 1, ut, ut1, NULL);
+    rho = gretl_corr(0, n - 1, ut, u1, NULL);
 
     free(ut);
-    free(ut1);
+    free(u1);
 
     return rho;
 }
 
 /**
  * rhohat:
- * @order: order of autoregression, 0 for OLS residuals.
+ * @order: order of autoregression, usually 1.
  * @t1: start of sample range.
  * @t2: end of sample range.
  * @uhat: array of regression residuals.
@@ -1681,22 +1733,18 @@ static double altrho (int order, int t1, int t2, const double *uhat)
 
 double rhohat (int order, int t1, int t2, const double *uhat)
 {
-    double ut, ut1, uu = 0.0, xx = 0.0;
+    double ut, u1, uu = 0.0, xx = 0.0;
     double rho;
     int t;
 
-    if (order) {
-	order--;
-    }
-
-    for (t=t1+order+1; t<=t2; t++) { 
+    for (t=t1+order; t<=t2; t++) { 
         ut = uhat[t];
-        ut1 = uhat[t-1];
-        if (na(ut) || na(ut1)) {
+        u1 = uhat[t-1];
+        if (na(ut) || na(u1)) {
 	    continue;
 	}
-        uu += ut * ut1;
-        xx += ut1 * ut1;
+        uu += ut * u1;
+        xx += u1 * u1;
     }
 
     if (floateq(xx, 0.0)) {
@@ -1733,13 +1781,13 @@ static int hatvar (MODEL *pmod, int n, double **Z)
             xno = pmod->list[i+2];
 	    x = Z[xno][t];
 	    if (pmod->nwt) {
-		x *= Z[pmod->nwt][t];
+		x *= sqrt(Z[pmod->nwt][t]);
 	    }
             pmod->yhat[t] += pmod->coeff[i] * x;
         }
 	x = Z[yno][t];
 	if (pmod->nwt) {
-	    x *= Z[pmod->nwt][t];
+	    x *= sqrt(Z[pmod->nwt][t]);
 	}
         pmod->uhat[t] = x - pmod->yhat[t];                
     }
@@ -1781,8 +1829,6 @@ static int hilu_plot (double *ssr, double *rho, int n)
     return 0;
 }
 
-#undef USE_DW /* base rho-hat on the D-W statistic? */
-
 static double autores (MODEL *pmod, const double **Z, int ci)
 {
     int t, v, t1 = pmod->t1;
@@ -1800,22 +1846,12 @@ static double autores (MODEL *pmod, const double **Z, int ci)
 	}
 	pmod->uhat[t] = x;
 	if (t > t1) {
-#ifdef USE_DW
-	    x = pmod->uhat[t] - pmod->uhat[t-1];
-	    num += x * x;
-#else
 	    num += pmod->uhat[t] * pmod->uhat[t-1];
-#endif
 	    den += pmod->uhat[t-1] * pmod->uhat[t-1];
 	}
     } 
 
-#ifdef USE_DW
-    den += pmod->uhat[pmod->t2] * pmod->uhat[pmod->t2];
-    rhohat = 1.0 - num / (den * 2.0);
-#else
     rhohat = num / den;
-#endif
 
     return rhohat;
 }
@@ -1968,11 +2004,7 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
 	    clear_model(&corc_model);
 	    goto bailout;
 	}
-#ifdef USE_DW
-	rho0 = rho = 1.0 - (corc_model.dw / 2.0);
-#else
 	rho0 = rho = corc_model.rho;
-#endif
     }
 
     if (na(rho)) {
@@ -2191,7 +2223,7 @@ static int get_hsk_weights (MODEL *pmod, double ***pZ, DATAINFO *pdinfo)
 		(*pZ)[oldv][t] = NADBL;
 	    } else {
 		xx = aux.yhat[t];
-		(*pZ)[oldv][t] = 1.0 / sqrt(exp(xx));
+		(*pZ)[oldv][t] = 1.0 / exp(xx);
 	    }
 	}
 	shrink = pdinfo->v - oldv - 1;
@@ -2343,7 +2375,7 @@ static int jackknife_vcv (MODEL *pmod, const double **Z)
 
     pmod->ci = HCCM;
 
-    if (makevcv(pmod)) {
+    if (makevcv(pmod, 1.0)) {
 	err = E_ALLOC;
 	goto bailout;
     }
@@ -2409,7 +2441,7 @@ static int jackknife_vcv (MODEL *pmod, const double **Z)
 	    }
 	    xx -= st[i] * st[j] / nobs;
 	    /* MacKinnon and White: "It is tempting to omit the factor
-	       (n - 1)/n from HC3" (1985, p. 309).  Here we leave it in
+	       (n - 1) / n from HC3" (1985, p. 309).  Here we leave it in
 	       place, as in their simulations.
 	    */
 	    xx *= (nobs - 1.0) / nobs;
@@ -2519,7 +2551,7 @@ int whites_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	printmodel(&white, pdinfo, OPT_NONE, prn);
 
 	TR2 = white.rsq * white.nobs;
-	pval = chisq(TR2, white.ncoeff - 1);
+	pval = chisq_cdf_comp(TR2, white.ncoeff - 1);
 
 	if (opt & OPT_S) {
 	    ModelTest *test = model_test_new(GRETL_TEST_WHITES);
@@ -2734,7 +2766,9 @@ MODEL ar_func (const int *list, double ***pZ,
 	ar.ifc = 1;
     }
     if (ar.ifc) {
-	ar.dfn -= 1;
+	if (!gretl_model_get_int(&ar, "effconst")) {
+	    ar.dfn -= 1;
+	}
     }
     ar.ci = AR;
 
@@ -2773,7 +2807,7 @@ MODEL ar_func (const int *list, double ***pZ,
 
     dataset_drop_last_variables(arlist[0] + 1 + reglist[0], pZ, pdinfo);
 
-    if (ar_info_init(&ar, maxlag)) {
+    if (gretl_model_add_arinfo(&ar, maxlag)) {
 	ar.errcode = E_ALLOC;
     } else {
 	for (i=0; i<=arlist[0]; i++) { 
@@ -2788,7 +2822,6 @@ MODEL ar_func (const int *list, double ***pZ,
 
     if (!ar.errcode) {
 	set_model_id(&ar);
-	printmodel(&ar, pdinfo, opt, prn);  
     }  
 
  bailout:
@@ -2942,7 +2975,11 @@ real_arch_test (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo,
 
     gretl_model_init(&archmod);
 
-    /* assess the lag order */
+    if (order == 0) {
+	/* use data frequency as default lag order */
+	order = pdinfo->pd;
+    }
+
     if (order < 1 || order > T - pmod->list[0]) {
 	archmod.errcode = E_UNSPEC;
 	sprintf(gretl_errmsg, _("Invalid lag order for arch (%d)"), order);
@@ -2999,9 +3036,9 @@ real_arch_test (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo,
 
     if (!err) {
 	archmod.aux = AUX_ARCH;
-	archmod.order = order;
+	gretl_model_set_int(&archmod, "arch_order", order);
 	LM = archmod.nobs * archmod.rsq;
-	xx = chisq(LM, order);
+	xx = chisq_cdf_comp(LM, order);
 
 	if (full) {
 	    printmodel(&archmod, pdinfo, OPT_NONE, prn);
@@ -3009,7 +3046,7 @@ real_arch_test (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo,
 		    archmod.nobs, archmod.rsq);
 	}
 
-	if (opt & OPT_S) {
+	if ((opt & OPT_S) || (opt & OPT_P)) {
 	    ModelTest *test = model_test_new(GRETL_TEST_ARCH);
 
 	    if (test != NULL) {
@@ -3018,16 +3055,21 @@ real_arch_test (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo,
 		model_test_set_dfn(test, order);
 		model_test_set_value(test, LM);
 		model_test_set_pvalue(test, xx);
-		maybe_add_test_to_model(pmod, test);
+		if (opt & OPT_S) {
+		    maybe_add_test_to_model(pmod, test);
+		} else {
+		    gretl_model_test_print_direct(test, prn);
+		    model_test_free(test);
+		}
 	    }	    
 	}
 
+	record_test_result(LM, xx, "ARCH");
+
 	if (!full) {
 	    goto arch_test_exit;
 	}
 
-	record_test_result(LM, xx, "ARCH");
-
 	pprintf(prn, _("LM test statistic (%f) is distributed as Chi-square "
 		"(%d)\nArea to the right of LM = %f  "), LM, order, xx);
 
@@ -3053,7 +3095,7 @@ real_arch_test (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo,
 		    if (xx <= 0.0) {
 			xx = (*pZ)[nv][t];
 		    }
-		    (*pZ)[nwt][t] = 1.0 / sqrt(xx);
+		    (*pZ)[nwt][t] = 1.0 / xx; /* FIXME is this right? */
 		}
 
 		strcpy(pdinfo->varname[nwt], "1/sigma");
@@ -3062,7 +3104,7 @@ real_arch_test (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo,
 		archmod = lsq(wlist, pZ, pdinfo, WLS, OPT_NONE);
 
 		archmod.ci = ARCH;
-		archmod.order = order;
+		gretl_model_set_int(&archmod, "arch_order", order);
 		printmodel(&archmod, pdinfo, opt, prn);
 	    }
 	}
@@ -3107,6 +3149,8 @@ MODEL arch_test (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo,
  * @order: lag order for ARCH process.
  * @pZ: pointer to data matrix.
  * @pdinfo: information on the data set.
+ * @opt: if %OPT_S, the test is saved to @pmod and not printed,
+ * otherwise it is printed to @prn and discarded.
  * @prn: gretl printing struct.
  *
  * Tests @pmod for Auto-Regressive Conditional Heteroskedasticity.  
@@ -3115,12 +3159,17 @@ MODEL arch_test (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo,
  */
 
 int arch_test_simple (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo, 
-		      PRN *prn)
+		      gretlopt opt, PRN *prn)
 {
     MODEL amod;
     int err;
 
-    amod = real_arch_test(pmod, order, pZ, pdinfo, OPT_S, prn, 0);
+    if (!(opt & OPT_S)) {
+	/* if not saving, then print test */
+	opt = OPT_P;
+    }
+
+    amod = real_arch_test(pmod, order, pZ, pdinfo, opt, prn, 0);
     err = amod.errcode;
     clear_model(&amod);
 
@@ -3422,49 +3471,110 @@ MODEL garch (const int *list, double ***pZ, DATAINFO *pdinfo, gretlopt opt,
 } 
 
 /**
- * pooled:
- * @list: regression list (dependent variable plus independent 
- * variables).
- * @pZ: pointer to data matrix.
- * @pdinfo: information on the (panel) data set.
- * @opt: can include %OPT_W to do weighted least squares
- * using weights based on the error variance for the 
- * respective cross-sectional units.
- * @prn: for printing details of iterations (or %NULL).
+ * mp_ols:
+ * @list: specification of variables to use.
+ * @Z: data array.
+ * @pdinfo: information on the data set.
  *
- * Calculate pooled OLS or WLS estimates for a panel dataset.
+ * Estimate an OLS model using multiple-precision arithmetic
+ * via the GMP library.
  * 
  * Returns: a #MODEL struct, containing the estimates.
  */
 
-MODEL pooled (const int *list, double ***pZ, DATAINFO *pdinfo,
-	      gretlopt opt, PRN *prn)
+MODEL mp_ols (const int *list, const double **Z, DATAINFO *pdinfo)
 {
-    MODEL wmod;
+    void *handle = NULL;
+    int (*mplsq)(const int *, const int *, const double **, 
+		 DATAINFO *, char *, MODEL *, gretlopt);
+    MODEL mpmod;
+
+    gretl_model_init(&mpmod);
+
+    mplsq = get_plugin_function("mplsq", &handle);
+    if (mplsq == NULL) {
+	mpmod.errcode = 1;
+	return mpmod;
+    }
+
+    if (gretl_list_has_separator(list)) {
+	int *base = NULL;
+	int *poly = NULL;
+
+	gretl_list_split_on_separator(list, &base, &poly);
+	if (base == NULL || poly == NULL) {
+	    mpmod.errcode = E_ALLOC;
+	} else {
+	    mpmod.errcode = (*mplsq)(base, poly, Z, pdinfo,  
+				     gretl_errmsg, &mpmod, OPT_S);
+	}
+	free(base);
+	free(poly);
+    } else {
+	mpmod.errcode = (*mplsq)(list, NULL, Z, pdinfo,  
+				 gretl_errmsg, &mpmod, OPT_S); 
+    }
+
+    close_plugin(handle);
+
+    set_model_id(&mpmod);
+
+    return mpmod;
+}
+
+static int check_panel_options (gretlopt opt)
+{
+    int err = 0;
+
+    if ((opt & OPT_R) && (opt & OPT_W)) {
+	/* can't specify random effects + weighted least squares */
+	err = E_DATA;
+    } else if ((opt & OPT_T) && !(opt & OPT_W)) {
+	/* iterate option requires weighted least squares option */
+	err = E_DATA;
+    }
+
+    return err;
+}
+
+/**
+ * panel_model:
+ * @list: regression list (dependent variable plus independent 
+ * variables).
+ * @pZ: pointer to data matrix.
+ * @pdinfo: information on the (panel) data set.
+ * @opt: can include %OPT_Q (quiet estimation), %OPT_S
+ * (silent estimation), %OPT_R (random effects model),
+ * %OPT_W (weights based on the error variance for the
+ * respective cross-sectional units), %OPT_T (iterate, only
+ * available in conjunction with %OPT_W).
+ * @prn: printing struct (or %NULL).
+ *
+ * Calculate estimates for a panel dataset, using fixed
+ * effects (the default), random effects, or weighted
+ * least squares based on the respective variances for the
+ * cross-sectional units.
+ * 
+ * Returns: a #MODEL struct, containing the estimates.
+ */
+
+MODEL panel_model (const int *list, double ***pZ, DATAINFO *pdinfo,
+		   gretlopt opt, PRN *prn)
+{
+    MODEL mod;
 
     *gretl_errmsg = '\0';
 
-    if (opt & OPT_W) {
-	void *handle;
-	MODEL (*panel_wls_by_unit) (const int *, double ***, DATAINFO *,
-				    gretlopt, PRN *);
-
-	panel_wls_by_unit = get_plugin_function("panel_wls_by_unit", &handle);
-
-	if (panel_wls_by_unit == NULL) {
-	    gretl_model_init(&wmod);
-	    wmod.errcode = E_FOPEN;
-	    return wmod;
-	}
-
-	wmod = (*panel_wls_by_unit) (list, pZ, pdinfo, opt, prn);
-
-	close_plugin(handle);
+    if (check_panel_options(opt)) {
+	gretl_model_init(&mod);
+	mod.errcode = E_BADOPT;
+    } else if (opt & OPT_W) {
+	mod = panel_wls_by_unit(list, pZ, pdinfo, opt, prn);
     } else {
-	wmod = lsq(list, pZ, pdinfo, POOLED, opt);
+	mod = real_panel_model(list, pZ, pdinfo, opt, prn);
     }
 
-    return wmod;
+    return mod;
 }
 
 /**
@@ -3488,12 +3598,16 @@ int groupwise_hetero_test (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     MODEL wmod;
     int err;
 
+    if (pmod->ci != OLS) {
+	return E_NOTIMP;
+    }
+
     if (!dataset_is_panel(pdinfo)) {
 	strcpy(gretl_errmsg, _("This test is only available for panel data"));
 	return 1;
     }
 
-    wmod = pooled(pmod->list, pZ, pdinfo, OPT_W | OPT_T | OPT_A, prn);
+    wmod = panel_wls_by_unit(pmod->list, pZ, pdinfo, OPT_T | OPT_A, prn);
     err = wmod.errcode;
 
     if (!err) {
