@@ -45,8 +45,10 @@ MODEL poisson_model (const int *list, double ***pZ, DATAINFO *pdinfo,
 MODEL garch (const int *list, double ***pZ, DATAINFO *pdinfo, gretlopt opt,
 	     PRN *prn);
 
-MODEL pooled (const int *list, double ***pZ, DATAINFO *pdinfo,
-	      gretlopt opt, PRN *prn);
+MODEL mp_ols (const int *list, const double **Z, DATAINFO *pdinfo);
+
+MODEL panel_model (const int *list, double ***pZ, DATAINFO *pdinfo,
+		   gretlopt opt, PRN *prn);
 
 int groupwise_hetero_test (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 			   PRN *prn);
@@ -66,13 +68,13 @@ MODEL arch_test (MODEL *pmod, int order,
 		 gretlopt opt, PRN *prn);
 
 int arch_test_simple (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo, 
-		      PRN *prn);
+		      gretlopt opt, PRN *prn);
 
 MODEL arch_model (const int *list, int order, 
 		  double ***pZ, DATAINFO *pdinfo, 
 		  gretlopt opt, PRN *prn);
 
-int makevcv (MODEL *pmod);
+int makevcv (MODEL *pmod, double sigma);
 
 int *augment_regression_list (const int *orig, int aux, 
 			      double ***pZ, DATAINFO *pdinfo);
