@@ -969,6 +969,63 @@ static double arma_variance (const double *phi, int p,
     return sspsi;
 }
 
+/* When forecasting based on an armax model estimated using X12A,
+   or via the Kalman filter, we need to form the series X\beta
+   so that we can subtract X\beta_{t-i} from y_{t-i} in
+   computing the AR portion of the forecast.
+*/
+
+static double *create_Xb_series (Forecast *fc, const MODEL *pmod,
+				 const double *beta, int *xlist, 
+				 const double **Z)
+{
+    double *Xb;
+    double x;
+    int miss;
+    int i, j, t;
+
+    Xb = malloc((fc->t2 + 1) * sizeof *Xb);
+    if (Xb == NULL) {
+	return NULL;
+    }
+
+    for (t=0; t<=fc->t2; t++) {
+	Xb[t] = 0.0;
+	miss = 0;
+	j = 0;
+	for (i=1; i<=xlist[0] && !miss; i++) {
+	    if (xlist[i] == 0) {
+		Xb[t] += pmod->coeff[0];
+	    } else {
+		x = Z[xlist[i]][t];
+		if (na(x)) {
+		    Xb[t] = NADBL;
+		    miss = 1;
+		} else {
+		    Xb[t] += beta[j++] * x;
+		}
+	    }
+	}
+    }
+
+    return Xb;
+}
+
+static int want_x_beta_prep (const MODEL *pmod, const int *xlist)
+{
+    int ret = 0;
+
+    if (xlist != NULL) {
+	int aflags = gretl_model_get_int(pmod, "arma_flags");
+
+	if ((aflags & ARMA_EXACT) || (aflags & ARMA_X12A)) {
+	    ret = 1;
+	}
+    }
+
+    return ret;
+}
+
 /* generate forecasts for AR(I)MA (or ARMAX) models, including
    forecast standard errors if we're doing out-of-sample
    forecasting
@@ -980,15 +1037,19 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
     double *psi = NULL;
     double *phi = NULL;
     double *theta = NULL;
+    double *phi0 = NULL;
+    double *Xb = NULL;
     const double *beta;
     const double *y;
 
     double xval, yval, vl;
+    double mu = NADBL;
     int xvars, yno;
     int *xlist = NULL;
-    int p, q, npsi = 0;
+    int p, q, px = 0, npsi = 0;
     int t1 = fc->t1;
     int ar_smax, ma_smax;
+    int regarma = 0;
     int i, s, t;
     int err = 0;
 
@@ -1010,8 +1071,8 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
 	t1 = pmod->t2 + 1;
     }
 
-    p = gretl_arma_model_max_AR_lag(pmod);
-    q = gretl_arma_model_max_MA_lag(pmod);
+    p = arma_model_max_AR_lag(pmod);
+    q = arma_model_max_MA_lag(pmod);
 
     xlist = model_xlist(pmod);
     yno = gretl_model_get_depvar(pmod);
@@ -1019,18 +1080,43 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
     DPRINTF(("forecasting variable %d (%s), obs %d to %d, with p=%d, q=%d\n", yno, 
 	     pdinfo->varname[yno], fc->t1, fc->t2, p, q));
 
-    if (xlist != NULL) {
-	xvars = xlist[0];
-    } else {
-	xvars = 0;
-    }
+    xvars = (xlist != NULL)? xlist[0] : 0;
 
-    err = gretl_arma_model_get_AR_MA_coeffs(pmod, &phi, &theta);
+    err = arma_model_integrated_AR_MA_coeffs(pmod, &phi, &theta);
     if (err) {
 	goto bailout;
     }
 
-    beta = gretl_arma_model_get_x_coeffs(pmod);
+    beta = arma_model_get_x_coeffs(pmod);
+
+    if (want_x_beta_prep(pmod, xlist)) {
+	if (gretl_is_arima_model(pmod)) {
+	    regarma = 1;
+	    err = regarma_model_AR_coeffs(pmod, &phi0, &px);
+	}
+	if (xlist[0] == 1 && xlist[1] == 0) {
+	    /* just a const, no ARMAX */
+	    mu = pmod->coeff[0];
+	    if (phi0 != NULL) {
+		mu = 1.0;
+		for (i=1; i<=px; i++) {
+		    mu -= phi0[i];
+		}
+		mu *= pmod->coeff[0];
+	    } else {
+		mu = pmod->coeff[0];
+	    }
+	} else {
+	    /* we have ARMAX terms */
+	    Xb = create_Xb_series(fc, pmod, beta, xlist, Z);
+	    if (Xb == NULL) {
+		err = E_ALLOC;
+	    }
+	}
+	if (err) {
+	    goto bailout;
+	}
+    }
 
     /* setup for forecast error variance */
     if (fc->sderr != NULL) {
@@ -1058,30 +1144,57 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
 
     /* do real forecast */
     for (t=t1; t<=fc->t2 && !err; t++) {
-	int miss = 0;
 	double yh = 0.0;
+	int miss = 0;
 
 	DPRINTF(("\n *** Doing forecast for obs %d\n", t));
 
-	/* contribution of independent variables */
-	for (i=1; i<=xvars; i++) {
+	/* contribution of const and/or independent variables */
+
+	if (!na(mu)) {
+	    yh = mu;
+	} else if (Xb != NULL) {
+	    /* X\beta series is pre-computed */
+	    if (na(Xb[t])) {
+		miss = 1;
+	    } else { 
+		yh = Xb[t];
+	    }
+	} else {
 	    int j = 0;
 
-	    if (xlist[i] == 0) {
-		yh += pmod->coeff[0];
-	    } else {
-		xval = Z[xlist[i]][t];
-		if (na(xval)) {
-		    miss = 1;
+	    for (i=1; i<=xvars; i++) {
+		if (xlist[i] == 0) {
+		    yh += pmod->coeff[0];
 		} else {
-		    yh += beta[j++] * xval;
+		    xval = Z[xlist[i]][t];
+		    if (na(xval)) {
+			miss = 1;
+		    } else {
+			yh += beta[j++] * xval;
+		    }
 		}
 	    }
 	}
 
-	DPRINTF((" x contribution = %g\n", yh));
+	if (phi0 != NULL && Xb != NULL) {
+	    /* the regarma case */
+	    for (i=1; i<=px; i++) {
+		s = t - i;
+		if (s < 0) {
+		    miss = 1;
+		} else if (na(Xb[s])) {
+		    miss = 1;
+		} else {
+		    yh -= phi0[i] * Xb[s];
+		} 
+	    }
+	}
+
+	DPRINTF((" Xb contribution = %g\n", yh));
 
 	/* AR contribution (incorporating any differencing) */
+
 	for (i=1; i<=p && !miss; i++) {
 	    if (phi[i] == 0.0) {
 		continue;
@@ -1100,14 +1213,25 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
 		DPRINTF(("  AR: lag %d, missing value\n", i));
 		miss = 1;
 	    } else {
-		DPRINTF(("  AR: lag %d, using coeff %g\n", i, phi[i]));
-		yh += phi[i] * yval;
+		DPRINTF(("  AR: lag %d, using coeff %#.8g\n", i, phi[i]));
+		if (!regarma && Xb != NULL) {
+		    if (na(Xb[s])) {
+			miss = 1;
+		    } else {
+			yh += phi[i] * (yval - Xb[s]);
+		    }
+		} else if (!regarma && !na(mu)) {
+		    yh += phi[i] * (yval - mu);
+		} else {
+		    yh += phi[i] * yval;
+		}
 	    }
 	}
 
 	DPRINTF((" with AR contribution: %g\n", yh));
 
 	/* MA contribution */
+
 	for (i=1; i<=q && !miss; i++) {
 	    if (theta[i] == 0.0) {
 		continue;
@@ -1151,6 +1275,8 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
     free(psi);
     free(phi);
     free(theta);
+    free(Xb);
+    free(phi0);
 
     return err;
 }
