@@ -25,6 +25,22 @@ enum {
     SORT_DESCENDING
 };
 
+enum {
+    HIGHNUM = 5000,
+    TNUM,
+    INDEXNUM
+};
+
+typedef enum {
+    R_NOBS = 1,  /* number of observations in current sample range */
+    R_NVARS,     /* number of variables in dataset (including the constant) */
+    R_PD,        /* periodicity of dataset */
+    R_DSET_MAX,  /* separator */
+    R_TEST_STAT, /* test statistic from last explicit test performed */
+    R_TEST_PVAL, /* p-value from last explicit test performed */
+    R_MAX
+} RetrievalIndex;
+
 typedef struct _GENERATOR GENERATOR;
 
 int generate (const char *line, double ***pZ, DATAINFO *pdinfo, 
@@ -59,21 +75,26 @@ int genr_function_from_string (const char *s);
 int get_t_from_obs_string (char *s, const double **Z, 
 			   const DATAINFO *pdinfo);
 
+const char *get_retriever_word (int idx);
+
 /* genrfuncs.c, public functions */
 
+int hp_filter (const double *x, double *hp, const DATAINFO *pdinfo, 
+	       gretlopt opt);
+
+int bkbp_filter (const double *y, double *bk, const DATAINFO *pdinfo);
+
 int dummy (double ***pZ, DATAINFO *pdinfo, int center);
 
 int panel_unit_first_obs (int t, const DATAINFO *pdinfo);
 
-int panel_unit_dummies (double ***pZ, DATAINFO *pdinfo);
-
-int paneldum (double ***pZ, DATAINFO *pdinfo);
-
-int plotvar (double ***pZ, DATAINFO *pdinfo);
-
-int plotvar_from_varname (double ***pZ, DATAINFO *pdinfo, const char *vname);
+int panel_dummies (double ***pZ, DATAINFO *pdinfo, gretlopt opt);
 
 int genrtime (double ***pZ, DATAINFO *pdinfo, int tm);
 
+const double *gretl_plotx (const DATAINFO *pdinfo);
+
+const char *get_model_stat_word (int idx);
+
 #endif /* GENERATE_H */
 
