@@ -26,6 +26,57 @@
 
 #include <errno.h>
 
+struct model_grab {
+    int idx;
+    const char *mword;
+};
+
+struct model_grab grabs[] = {
+    { M_ESS,        "$ess" },
+    { M_T,          "$t" },
+    { M_RSQ,        "$rsq" },
+    { M_SIGMA,      "$sigma" },
+    { M_DF,         "$df" },
+    { M_NCOEFF,     "$ncoeff" },
+    { M_LNL,        "$lnl" },
+    { M_AIC,        "$aic" },
+    { M_BIC,        "$bic" },
+    { M_HQC,        "$hqc" },
+    { M_TRSQ,       "$trsq" },
+    { M_SCALAR_MAX,  NULL },
+    { M_COEFF_S,    "$coeff" },
+    { M_SE_S,       "$stderr" },
+    { M_VCV_S,      "$vcv" },
+    { M_RHO_S,      "$rho" },
+    { M_ELEM_MAX,    NULL },
+    { M_UHAT,       "$uhat" },
+    { M_YHAT,       "$yhat" },
+    { M_AHAT,       "$ahat" },
+    { M_H,          "$h" },
+    { M_SERIES_MAX,  NULL },
+    { M_COEFF,      "$coeff" },
+    { M_SE,         "$ess" },
+    { M_VCV,        "$vcv" },
+    { M_RHO,        "$rho" },
+    { M_JALPHA,     "$jalpha" }, 
+    { M_JBETA,      "$jbeta" },
+    { M_JVBETA,     "$jvbeta" },
+    { M_MAX,         NULL }
+};
+
+const char *get_model_stat_word (int idx)
+{
+    int i;
+
+    for (i=0; grabs[i].idx != M_MAX; i++) {
+	if (idx == grabs[i].idx) {
+	    return grabs[i].mword;
+	}
+    }
+
+    return NULL;
+}
+
 static double hp_lambda (const DATAINFO *pdinfo)
 {
     double la = get_hp_lambda();
@@ -40,8 +91,9 @@ static double hp_lambda (const DATAINFO *pdinfo)
 /**
  * hp_filter:
  * @x: array of original data.
- * @hp: array in which Hodrick-Prescott "cycle" is computed.
+ * @hp: array in which filtered series is computed.
  * @pdinfo: data set information.
+ * @opt: if %OPT_T, return the trend rather than the cycle.
  *
  * Calculates the "cycle" component of the time series in
  * array @x, using the Hodrick-Prescott filter.  Adapted from the 
@@ -50,7 +102,8 @@ static double hp_lambda (const DATAINFO *pdinfo)
  * Returns: 0 on success, non-zero error code on failure.
  */
 
-int hp_filter (const double *x, double *hp, const DATAINFO *pdinfo)
+int hp_filter (const double *x, double *hp, const DATAINFO *pdinfo,
+	       gretlopt opt)
 {
     int i, t, T, t1 = pdinfo->t1, t2 = pdinfo->t2;
     int err = 0;
@@ -82,20 +135,9 @@ int hp_filter (const double *x, double *hp, const DATAINFO *pdinfo)
 
     lambda = hp_lambda(pdinfo);
 
-    V = malloc(4 * sizeof *V);
-    if (V == NULL) return E_ALLOC;
-
-    for (i=0; i<4; i++) {
-	V[i] = malloc(T * sizeof **V);
-	if (V[i] == NULL) {
-	    int j;
-	    
-	    for (j=0; j<i; j++) {
-		free(V[j]);
-	    }
-	    free(V);
-	    return E_ALLOC;
-	}
+    V = doubles_array_new(4, T);
+    if (V == NULL) {
+	return E_ALLOC;
     }
 
     /* adjust starting points */
@@ -187,8 +229,14 @@ int hp_filter (const double *x, double *hp, const DATAINFO *pdinfo)
     V[3][0] = m[0];
     V[3][1] = m[1];
 
-    for (t=0; t<T; t++) {
-	hp[t] = x[t] - V[3][t];
+    if (opt & OPT_T) {
+	for (t=0; t<T; t++) {
+	    hp[t] = V[3][t];
+	}
+    } else {
+	for (t=0; t<T; t++) {
+	    hp[t] = x[t] - V[3][t];
+	}
     }
 
  bailout:
@@ -215,7 +263,7 @@ int hp_filter (const double *x, double *hp, const DATAINFO *pdinfo)
 int bkbp_filter (const double *y, double *bk, const DATAINFO *pdinfo)
 {
     int t1 = pdinfo->t1, t2 = pdinfo->t2;
-    int periods[2];
+    int bkl, bku;
 
     double omubar, omlbar;
     double avg_a;
@@ -230,15 +278,15 @@ int bkbp_filter (const double *y, double *bk, const DATAINFO *pdinfo)
     */
 
     /* get user settings if available (or the defaults) */
-    get_bkbp_periods(periods);
-    k = get_bkbp_k();
+    get_bkbp_periods(pdinfo, &bkl, &bku);
+    k = get_bkbp_k(pdinfo);
 
 #if BK_DEBUG
     fprintf(stderr, "lower limit = %d, upper limit = %d, \n", 
-	    periods[0], periods[1]);
+	    bkl, bku);
 #endif
 
-    if (periods[0] >= periods[1]) {
+    if (bkl >= bku) {
 	strcpy(gretl_errmsg, "Error in Baxter-King frequencies");
 	return 1;
     }
@@ -258,8 +306,8 @@ int bkbp_filter (const double *y, double *bk, const DATAINFO *pdinfo)
 	return E_ALLOC;
     }
     
-    omubar = 2.0 * M_PI / periods[0];
-    omlbar = 2.0 * M_PI / periods[1];
+    omubar = 2.0 * M_PI / bkl;
+    omlbar = 2.0 * M_PI / bku;
     
     /* first we compute the coefficients */
 
@@ -368,30 +416,6 @@ static int panel_x_offset (const DATAINFO *pdinfo, int *bad)
     return offset;
 }
 
-static void 
-make_x_panel_dummy (double *x, const DATAINFO *pdinfo, int i)
-{
-    int t, offset, bad = 0;
-    int dmin, dmax;
-
-    offset = panel_x_offset(pdinfo, &bad);
-
-    dmin = (i - 1) * pdinfo->pd;
-    dmax = i * pdinfo->pd - offset;
-
-    if (i > 1) dmin -= offset;
-
-    for (t=0; t<pdinfo->n; t++) {
-	if (bad) {
-	    x[t] = NADBL;
-	} else if (t >= dmin && t < dmax) {
-	    x[t] = 1.0;
-	} else {
-	    x[t] = 0.0;
-	}
-    }
-}
-
 static int n_new_dummies (const DATAINFO *pdinfo,
 			  int nunits, int nperiods)
 {
@@ -470,19 +494,10 @@ int dummy (double ***pZ, DATAINFO *pdinfo, int center)
     char vname[VNAMELEN];
     char vlabel[MAXLABEL];
     int vi, t, yy, pp, mm;
-    int ndums, nnew = 0;
+    int ndums = pdinfo->pd, nnew = 0;
     int di, di0 = pdinfo->v;
     double xx, dx;
 
-    if (pdinfo->structure == STACKED_CROSS_SECTION) {
-	ndums = pdinfo->n / pdinfo->pd;
-	if (pdinfo->n % pdinfo->pd) {
-	    ndums++;
-	}
-    } else {
-	ndums = pdinfo->pd;
-    }
-
     if (ndums == 1 || ndums > 99999) {
 	strcpy(gretl_errmsg, _("This command won't work with the current periodicity"));
 	return 0;
@@ -526,19 +541,15 @@ int dummy (double ***pZ, DATAINFO *pdinfo, int center)
 	strcpy(pdinfo->varname[di], vname);
 	strcpy(VARLABEL(pdinfo, di), vlabel);
 
-	if (pdinfo->structure == STACKED_CROSS_SECTION) {
-	    make_x_panel_dummy((*pZ)[di], pdinfo, vi);
-	} else {
-	    for (t=0; t<pdinfo->n; t++) {
-		xx = date(t, pdinfo->pd, pdinfo->sd0);
-		if (dataset_is_daily(pdinfo)) { /* FIXME weekly? */
-		    xx += .1;
-		}
-		yy = (int) xx;
-		pp = (int) (mm * (xx - yy) + 0.5);
-		dx = (pp == vi)? 1.0 : 0.0;
-		(*pZ)[di][t] = dx;
+	for (t=0; t<pdinfo->n; t++) {
+	    xx = date(t, pdinfo->pd, pdinfo->sd0);
+	    if (dataset_is_daily(pdinfo)) { /* FIXME weekly? */
+		xx += .1;
 	    }
+	    yy = (int) xx;
+	    pp = (int) (mm * (xx - yy) + 0.5);
+	    dx = (pp == vi)? 1.0 : 0.0;
+	    (*pZ)[di][t] = dx;
 	}
     }
 
@@ -556,50 +567,45 @@ int dummy (double ***pZ, DATAINFO *pdinfo, int center)
     return di0;
 }
 
-/* if both == 1, generate both unit and period dummies,
-   otherwise just generate unit dummies
-*/
+/**
+ * panel_dummies:
+ * @pZ: pointer to data matrix.
+ * @pdinfo: data information struct.
+ * @opt: %OPT_T for time dummies, otherwise unit dummies.
+ *
+ * Adds to the data set a set of dummy variables corresponding
+ * to either the cross-sectional units in a panel, or the
+ * time periods.
+ *
+ * Returns: 0 on successful completion, error code on error.
+ */
 
-static int real_paneldum (double ***pZ, DATAINFO *pdinfo,
-			  int both)
+int panel_dummies (double ***pZ, DATAINFO *pdinfo, gretlopt opt)
 {
     char vname[16];
     int vi, t, yy, pp, mm;
-    int xsect, orig_v = pdinfo->v;
-    int ndum, nnew, n_blockdum = 0, n_freqdum = 0;
+    int orig_v = pdinfo->v;
+    int ndum, nnew;
+    int n_unitdum = 0;
+    int n_timedum = 0;
     int newvnum, offset, bad = 0;
     double xx;
 
-    xsect = (pdinfo->structure == STACKED_CROSS_SECTION);
-
-    /* in case xsect, block dummies are per time-period,
-       frequency dummies are for units;
-       in case of stacked time series, block dummies are
-       per cross-sectional unit, frequency ones are for periods
-    */
-
-    if (both || xsect) {
-	n_freqdum = pdinfo->pd;
-	if (n_freqdum == 1) {
-	    return E_PDWRONG;
-	}
-    }
-
-    if (both || !xsect) {
-	n_blockdum = pdinfo->n / pdinfo->pd;
+    if (opt & OPT_T) {
+	ndum = n_timedum = pdinfo->pd;
+    } else {	
+	n_unitdum = pdinfo->n / pdinfo->pd;
 	if (pdinfo->n % pdinfo->pd) {
-	    n_blockdum++;
-	}
-	if (n_blockdum == 1) {
-	    return E_PDWRONG;
+	    n_unitdum++;
 	}
+	ndum = n_unitdum;
+    }
+    
+    if (ndum == 1) {
+	return E_PDWRONG;
     }
 
-    ndum = n_freqdum + n_blockdum;
-
-    nnew = n_new_dummies(pdinfo, 
-			 (xsect)? n_freqdum : n_blockdum,
-			 (xsect)? n_blockdum : n_freqdum);
+    nnew = n_new_dummies(pdinfo, n_unitdum, n_timedum);
 
     if (dataset_add_series(nnew, pZ, pdinfo)) {
 	return E_ALLOC;
@@ -613,15 +619,12 @@ static int real_paneldum (double ***pZ, DATAINFO *pdinfo,
 
     newvnum = orig_v;
 
-    /* first generate the frequency-based dummies */
-    for (vi=1; vi<=n_freqdum; vi++) {
+    /* generate time-based dummies, if wanted */
+
+    for (vi=1; vi<=n_timedum; vi++) {
 	int dnum;
 
-	if (xsect) {
-	    sprintf(vname, "du_%d", vi);
-	} else {
-	    sprintf(vname, "dt_%d", vi);
-	}
+	sprintf(vname, "dt_%d", vi);
 
 	dnum = varindex(pdinfo, vname);
 	if (dnum >= orig_v) {
@@ -631,7 +634,7 @@ static int real_paneldum (double ***pZ, DATAINFO *pdinfo,
 	strcpy(pdinfo->varname[dnum], vname);
 	sprintf(VARLABEL(pdinfo, dnum), 
 		_("%s = 1 if %s is %d, 0 otherwise"), vname, 
-		(xsect)? _("unit"): _("period"), vi);
+		_("period"), vi);
 
 	for (t=0; t<pdinfo->n; t++) {
 	    xx = date(t, pdinfo->pd, pdinfo->sd0);
@@ -643,19 +646,16 @@ static int real_paneldum (double ***pZ, DATAINFO *pdinfo,
 
     offset = panel_x_offset(pdinfo, &bad);
 
-    /* and then the block-based ones */
-    for (vi=1; vi<=n_blockdum; vi++) {
+    /* generate unit-based dummies, if wanted */
+
+    for (vi=1; vi<=n_unitdum; vi++) {
 	int dmin = (vi-1) * pdinfo->pd;
 	int dmax = vi * pdinfo->pd - offset;
 	int dnum;
 
 	if (vi > 1) dmin -= offset;
 
-	if (xsect) {
-	    sprintf(vname, "dt_%d", vi);
-	} else {
-	    sprintf(vname, "du_%d", vi);
-	}
+	sprintf(vname, "du_%d", vi);
 
 	dnum = varindex(pdinfo, vname);
 	if (dnum >= orig_v) {
@@ -665,7 +665,7 @@ static int real_paneldum (double ***pZ, DATAINFO *pdinfo,
 	strcpy(pdinfo->varname[dnum], vname);
 	sprintf(VARLABEL(pdinfo, dnum), 
 		_("%s = 1 if %s is %d, 0 otherwise"), vname, 
-		(xsect)? _("period"): _("unit"), vi);
+		_("unit"), vi);
 
 	for (t=0; t<pdinfo->n; t++) {
 	    if (bad) {
@@ -681,38 +681,6 @@ static int real_paneldum (double ***pZ, DATAINFO *pdinfo,
     return 0;
 }
 
-/**
- * panel_unit_dummies:
- * @pZ: pointer to data matrix.
- * @pdinfo: data information struct.
- *
- * Adds to the data set a set of dummy variables corresponding
- * to the cross-sectional units in a panel.
- *
- * Returns: 0 on successful completion, error code on error.
- */
-
-int panel_unit_dummies (double ***pZ, DATAINFO *pdinfo)
-{
-    return real_paneldum(pZ, pdinfo, 0);
-}
-
-/**
- * paneldum:
- * @pZ: pointer to data matrix.
- * @pdinfo: data information struct.
- *
- * Adds to the data set a set of panel data dummy variables (for
- * both unit and period).
- *
- * Returns: 0 on successful completion, error code on error.
- */
-
-int paneldum (double ***pZ, DATAINFO *pdinfo)
-{
-    return real_paneldum(pZ, pdinfo, 1);
-}
-
 /**
  * genrunit:
  * @pZ: pointer to data matrix.
@@ -729,8 +697,7 @@ int genrunit (double ***pZ, DATAINFO *pdinfo)
     int xt = 0;
     int i, t;
 
-    if (pdinfo->structure != STACKED_TIME_SERIES &&
-	pdinfo->structure != STACKED_CROSS_SECTION) {
+    if (pdinfo->structure != STACKED_TIME_SERIES) {
 	strcpy(gretl_errmsg, "'genr unit' can be used only with "
 	       "panel data");
 	return 1;
@@ -738,29 +705,19 @@ int genrunit (double ***pZ, DATAINFO *pdinfo)
 
     i = varindex(pdinfo, "unit");
 
-    if (i == pdinfo->v) {
-	if (dataset_add_series(1, pZ, pdinfo)) return E_ALLOC;
+    if (i == pdinfo->v && dataset_add_series(1, pZ, pdinfo)) {
+	return E_ALLOC;
     }
 
     strcpy(pdinfo->varname[i], "unit");
     strcpy(VARLABEL(pdinfo, i), _("cross-sectional unit index"));
 
-    if (pdinfo->structure == STACKED_CROSS_SECTION) {
-	for (t=0; t<pdinfo->n; t++) {
-	    if (t % pdinfo->pd == 0) {
-		xt = 1;
-	    }
-	    (*pZ)[i][t] = (double) xt++;
+    for (t=0; t<pdinfo->n; t++) {
+	if (t % pdinfo->pd == 0) {
+	    xt++;
 	}
-    } else {
-	/* stacked time series */
-	for (t=0; t<pdinfo->n; t++) {
-	    if (t % pdinfo->pd == 0) {
-		xt++;
-	    }
-	    (*pZ)[i][t] = (double) xt;
-	}
-    }    
+	(*pZ)[i][t] = (double) xt;
+    }
 
     return 0;
 }
@@ -796,21 +753,11 @@ make_panel_time_var (double *x, const DATAINFO *pdinfo)
 {
     int t, xt = 0;
 
-    if (pdinfo->structure == STACKED_TIME_SERIES) {
-	for (t=0; t<pdinfo->n; t++) {
-	    if (t % pdinfo->pd == 0) {
-		xt = 1;
-	    }
-	    x[t] = (double) xt++;
-	}
-    } else {
-	/* stacked cross-sections */
-	for (t=0; t<pdinfo->n; t++) {
-	    if (t % pdinfo->pd == 0) {
-		xt++;
-	    }
-	    x[t] = (double) xt;
+    for (t=0; t<pdinfo->n; t++) {
+	if (t % pdinfo->pd == 0) {
+	    xt = 1;
 	}
+	x[t] = (double) xt++;
     }
 }
 
@@ -850,9 +797,7 @@ int genrtime (double ***pZ, DATAINFO *pdinfo, int tm)
 	strcpy(VARLABEL(pdinfo, i), _("data index variable"));
     }
     
-    if (tm && 
-	(pdinfo->structure == STACKED_TIME_SERIES ||
-	 pdinfo->structure == STACKED_CROSS_SECTION)) {
+    if (tm && pdinfo->structure == STACKED_TIME_SERIES) {
 	make_panel_time_var((*pZ)[i], pdinfo);
     } else {
 	for (t=0; t<pdinfo->n; t++) {
@@ -863,17 +808,44 @@ int genrtime (double ***pZ, DATAINFO *pdinfo, int tm)
     return 0;
 }
 
-static int plotvar_is_full_size (int v, int n, const double *x)
-{
-    int t;
+/**
+ * genrwkday:
+ * @pZ: pointer to data array.
+ * @pdinfo: data information struct.
+ *
+ * Generates (and adds to the dataset, if it's not already
+ * present) an index representing the day of the week for
+ * each observation (for dated daily data only).
+ * The index has value 0 for Sunday, 1 for Monday, and
+ * so on.
+ *
+ * Returns: 0 on success, non-zero code on error.
+ */
 
-    for (t=0; t<n; t++) {
-	if (na(x[t])) {
-	    return 0;
-	}
+int genrwkday (double ***pZ, DATAINFO *pdinfo)
+{
+    char datestr[OBSLEN];
+    int i, t;
+
+    if (!dated_daily_data(pdinfo)) {
+	return E_PDWRONG;
     }
 
-    return 1;
+    i = varindex(pdinfo, "weekday");
+
+    if (i == pdinfo->v && dataset_add_series(1, pZ, pdinfo)) {
+	return E_ALLOC;
+    }
+
+    strcpy(pdinfo->varname[i], "weekday");
+    strcpy(VARLABEL(pdinfo, i), _("day of week (1 = Monday)"));
+    
+    for (t=0; t<pdinfo->n; t++) {
+	ntodate_full(datestr, t, pdinfo);
+	(*pZ)[i][t] = get_day_of_week(datestr);
+    }
+
+    return 0;
 }
 
 typedef enum {
@@ -888,35 +860,6 @@ typedef enum {
     PLOTVAR_MAX
 } plotvar_type; 
 
-struct plotvar {
-    int ptype;
-    char *pname;
-};
-
-static struct plotvar pvars[] = {
-    { PLOTVAR_INDEX,    "index" },
-    { PLOTVAR_TIME,     "time" },
-    { PLOTVAR_ANNUAL,   "annual" },
-    { PLOTVAR_QUARTERS, "qtrs" },
-    { PLOTVAR_MONTHS,   "months" },
-    { PLOTVAR_CALENDAR, "caldate" },
-    { PLOTVAR_DECADES,  "decades" },
-    { PLOTVAR_HOURLY,   "hrs" }
-};
-
-static int ptype_from_name (const char *pname)
-{
-    int i;
-
-    for (i=0; i<PLOTVAR_MAX; i++) {
-	if (!strcmp(pvars[i].pname, pname)) {
-	    return pvars[i].ptype;
-	}
-    }
-
-    return -1;
-}
-
 int plotvar_code (const DATAINFO *pdinfo)
 {
     if (!dataset_is_time_series(pdinfo)) {
@@ -940,135 +883,114 @@ int plotvar_code (const DATAINFO *pdinfo)
 }
 
 /**
- * plotvar_from_varname:
- * @pZ: pointer to data matrix.
+ * gretl_plotx:
  * @pdinfo: data information struct.
- * @vname: name for the plotting variable.
  *
- * Adds to the dataset a special dummy variable named @vname,
- * for use on the x-axis in plotting, or simply retrieves the 
- * ID number of the variable if it already exists.  
+ * Finds or creates a special dummy variable for use on the
+ * x-axis in plotting; this will have the full length of the
+ * data series as given in @pdinfo, and will be appropriately
+ * configured for the data frequency.  Do not try to free this
+ * variable.
  *
- * Returns: the ID number of the variable (> 0) or -1 on failure.
+ * Returns: pointer to plot x-variable, or %NULL on failure.
  */
 
-int plotvar_from_varname (double ***pZ, DATAINFO *pdinfo, const char *vname)
+const double *gretl_plotx (const DATAINFO *pdinfo)
 {
-    int t, v, y1, n = pdinfo->n;
-    const char *pname;
-    int ptype;
+    static double *x;
+    static int ptype;
+    static int T;
+
+    int t, y1;
+    int new_ptype;
     float rm;
 
-    if (vname != NULL) {
-	pname = vname;
-	ptype = ptype_from_name(pname);
-	if (ptype < 0) {
-	    return -1;
-	}
-    } else {
-	ptype = plotvar_code(pdinfo);
-	pname = pvars[ptype].pname;
+    if (pdinfo == NULL) {
+	/* cleanup signal */
+	free(x);
+	ptype = 0;
+	T = 0;
+	return NULL;
     }
 
-    v = varindex(pdinfo, pname);
+    new_ptype = plotvar_code(pdinfo);
 
-    if (v < pdinfo->v) {
-	if (plotvar_is_full_size(v, pdinfo->n, (*pZ)[v])) {
-	    return v;
-	} 
-    } else if (dataset_add_series(1, pZ, pdinfo)) {
-	return -1;
+    if (x != NULL && new_ptype == ptype && T == pdinfo->n) {
+	/* a suitable array is already at hand */
+	return x;
     }
 
-    strcpy(pdinfo->varname[v], pname);
+    if (x != NULL) {
+	free(x);
+    }
+
+    x = malloc(pdinfo->n * sizeof *x);
+    if (x == NULL) {
+	return NULL;
+    }
+
+    T = pdinfo->n;
+    ptype = new_ptype;
 
     y1 = (int) pdinfo->sd0;
     rm = pdinfo->sd0 - y1;
 
     switch (ptype) {
     case PLOTVAR_ANNUAL: 
-	strcpy(VARLABEL(pdinfo, v), _("annual plotting variable")); 
-	for (t=0; t<n; t++) {
-	    (*pZ)[v][t] = (double) (t + atoi(pdinfo->stobs));
+	for (t=0; t<T; t++) {
+	    x[t] = (double) (t + atoi(pdinfo->stobs));
 	}
 	break;
     case PLOTVAR_QUARTERS:
-	strcpy(VARLABEL(pdinfo, v), _("quarterly plotting variable"));
-	(*pZ)[v][0] = y1 + (10.0 * rm - 1.0) / 4.0;
-	for (t=1; t<n; t++) {
-	    (*pZ)[v][t] = (*pZ)[v][t-1] + .25;
+	x[0] = y1 + (10.0 * rm - 1.0) / 4.0;
+	for (t=1; t<T; t++) {
+	    x[t] = x[t-1] + .25;
 	}
 	break;
     case PLOTVAR_MONTHS:
-	strcpy(VARLABEL(pdinfo, v), _("monthly plotting variable"));
-	(*pZ)[v][0] = y1 + (100.0 * rm - 1.0) / 12.0;
-	for (t=1; t<n; t++) {
-	    (*pZ)[v][t] = (*pZ)[v][t-1] + (1.0 / 12.0);
+	x[0] = y1 + (100.0 * rm - 1.0) / 12.0;
+	for (t=1; t<T; t++) {
+	    x[t] = x[t-1] + (1.0 / 12.0);
 	}
 	break;
     case PLOTVAR_HOURLY:
-	strcpy(VARLABEL(pdinfo, v), _("hourly plotting variable"));
-	(*pZ)[v][0] = y1 + (100.0 * rm - 1.0) / 24.0;
-	for (t=1; t<n; t++) {
-	    (*pZ)[v][t] = (*pZ)[v][t-1] + (1.0 / 24.0);
+	x[0] = y1 + (100.0 * rm - 1.0) / 24.0;
+	for (t=1; t<T; t++) {
+	    x[t] = x[t-1] + (1.0 / 24.0);
 	}
 	break;
     case PLOTVAR_CALENDAR:
-	strcpy(VARLABEL(pdinfo, v), _("daily plotting variable"));
-	for (t=0; t<n; t++) {
+	for (t=0; t<T; t++) {
 	    if (pdinfo->S != NULL) {
-		(*pZ)[v][t] = get_dec_date(pdinfo->S[t]);
+		x[t] = get_dec_date(pdinfo->S[t]);
 	    } else {
 		char datestr[OBSLEN];
 		    
 		calendar_date_string(datestr, t, pdinfo);
-		(*pZ)[v][t] = get_dec_date(datestr);
+		x[t] = get_dec_date(datestr);
 	    }
 	}
 	break;
     case PLOTVAR_DECADES:
-	strcpy(pdinfo->varname[v], "time");
-	strcpy(VARLABEL(pdinfo, v), _("time trend variable"));
-	for (t=0; t<n; t++) {
-	    (*pZ)[v][t] = pdinfo->sd0 + 10 * t;
+	for (t=0; t<T; t++) {
+	    x[t] = pdinfo->sd0 + 10 * t;
 	}
 	break;
     case PLOTVAR_INDEX:
-	strcpy(VARLABEL(pdinfo, v), _("index variable"));
-	for (t=0; t<n; t++) {
-	    (*pZ)[v][t] = (double) (t + 1);
+	for (t=0; t<T; t++) {
+	    x[t] = (double) (t + 1);
 	}
 	break;
     case PLOTVAR_TIME:
-	strcpy(VARLABEL(pdinfo, v), _("time trend variable"));
-	for (t=0; t<n; t++) {
-	    (*pZ)[v][t] = (double) (t + 1);
+	for (t=0; t<T; t++) {
+	    x[t] = (double) (t + 1);
 	}
 	break;
     default:
 	break;
     }
 
-    return v;
-}
-
-/**
- * plotvar:
- * @pZ: pointer to data matrix.
- * @pdinfo: data information struct.
- *
- * Adds to the dataset a special dummy variable for use on the
- * x-axis in plotting, or simple retrieves the ID number of the
- * plotting variable if it already exists.  The name of the variable 
- * is assigned automatically based on the characteristics of the 
- * dataset.
- *
- * Returns: the ID number of the variable (> 0) or -1 on failure.
- */
-
-int plotvar (double ***pZ, DATAINFO *pdinfo)
-{
-    return plotvar_from_varname(pZ, pdinfo, NULL);
+    return x;
 }
 
 static int varnames_from_arg (const char *s, char *v1str, char *v2str)
@@ -1127,29 +1049,10 @@ double genr_cov_corr (const char *s, double ***pZ,
     return ret;
 }
 
-static int 
-get_model_param_number (const MODEL *pmod, const char *vname)
-{
-    int i, ret = 0;
-
-    if (pmod->params == NULL) {
-	return 0;
-    }
-
-    for (i=0; i<=pmod->ncoeff; i++) {
-	if (!strcmp(vname, pmod->params[i])) {
-	    ret = i + 1;
-	    break;
-	}
-    }
-
-    return ret;
-}
-
 double genr_vcv (const char *s, const DATAINFO *pdinfo, MODEL *pmod)
 {
     int v1 = 0, v2 = 0;
-    int i, j, k, v1l, v2l;
+    int i, j, k;
     char v1str[VNAMELEN], v2str[VNAMELEN];
     int gotit;
     double ret = NADBL;
@@ -1162,41 +1065,26 @@ double genr_vcv (const char *s, const DATAINFO *pdinfo, MODEL *pmod)
 	return NADBL;
     }
 
-    /* are the varnames valid? */
-    if (pmod->ci != NLS && pmod->ci != ARMA) {
-	v1 = varindex(pdinfo, v1str);
-	v2 = varindex(pdinfo, v2str);
-	if (v1 >= pdinfo->v || v2 >= pdinfo->v) {
-	    return NADBL;
-	}
-    }
-
-    /* check model list */
-    if (pmod->ci == NLS || pmod->ci == ARMA) {
-	v1l = get_model_param_number(pmod, v1str);
-	v2l = get_model_param_number(pmod, v2str);
-    } else {
-	v1l = gretl_list_position(v1, pmod->list);
-	v2l = gretl_list_position(v2, pmod->list);
-    }
-
-    if (v1l == 0 || v2l == 0) {
+    v1 = gretl_model_get_param_number(pmod, pdinfo, v1str);
+    if (v1 < 0) {
 	return NADBL;
     }
 
-    v1l -= 2;
-    v2l -= 2;
+    v2 = gretl_model_get_param_number(pmod, pdinfo, v2str);
+    if (v2 < 0) {
+	return NADBL;
+    }
 
     /* make model vcv matrix if need be */
-    if (pmod->vcv == NULL && makevcv(pmod)) {
+    if (pmod->vcv == NULL && makevcv(pmod, pmod->sigma)) {
 	return NADBL;
     }
 
     /* now find the right entry */
-    if (v1l > v2l) {
-	k = v1l;
-	v1l = v2l;
-	v2l = k;
+    if (v1 > v2) {
+	k = v1;
+	v1 = v2;
+	v2 = k;
     }
 
     gotit = 0;
@@ -1206,7 +1094,7 @@ double genr_vcv (const char *s, const DATAINFO *pdinfo, MODEL *pmod)
 	    if (j < i) {
 		continue;
 	    }
-	    if (i == v1l && j == v2l) {
+	    if (i == v1 && j == v2) {
 		ret = pmod->vcv[k];
 		gotit = 1;
 		break;
@@ -1236,13 +1124,16 @@ int genr_fit_resid (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 		    int code, int undo)
 {
     char vname[VNAMELEN], vlabel[MAXLABEL];
+    const double *x = NULL;
     int i, t;
-    double *h = NULL;
 
     if (code == GENR_H) {
-	h = gretl_model_get_data(pmod, "garch_h");
-	if (h == NULL) return E_DATA;
-    }
+	x = gretl_model_get_data(pmod, "garch_h");
+	if (x == NULL) return E_DATA;
+    } else if (code == GENR_AHAT) {
+	x = gretl_model_get_data(pmod, "ahat");
+	if (x == NULL) return E_DATA;
+    }	
 
     if (dataset_add_series(1, pZ, pdinfo)) {
 	return E_ALLOC;
@@ -1282,9 +1173,16 @@ int genr_fit_resid (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	sprintf(vname, "h%d", pmod->ID);
 	sprintf(vlabel, _("fitted variance from model %d"), pmod->ID);
 	for (t=pmod->t1; t<=pmod->t2; t++) {
-	    (*pZ)[i][t] = h[t];
+	    (*pZ)[i][t] = x[t];
 	}
-    }
+    } else if (code == GENR_AHAT) { 
+	/* fixed-effects constants */
+	sprintf(vname, "ahat%d", pmod->ID);
+	sprintf(vlabel, _("per-unit constants from model %d"), pmod->ID);
+	for (t=pmod->t1; t<=pmod->t2; t++) {
+	    (*pZ)[i][t] = x[t];
+	}
+    }	
 
     strcpy(pdinfo->varname[i], vname);
 
@@ -1311,10 +1209,12 @@ int get_observation_number (const char *s, const DATAINFO *pdinfo)
 
     if (pdinfo->markers && pdinfo->S != NULL) {
 	for (t=0; t<pdinfo->n; t++) {
-	    if (!strcmp(test, pdinfo->S[t])) return t + 1;
+	    if (!strcmp(test, pdinfo->S[t])) {
+		return t + 1;
+	    }
 	}
 	if (calendar_data(pdinfo)) {
-	    charsub(test, ':', '/');
+	    charsub(test, ':', '/'); /* FIXME */
 	    for (t=0; t<pdinfo->n; t++) {
 		if (!strcmp(test, pdinfo->S[t]) ||
 		    !strcmp(test, pdinfo->S[t] + 2)) {
@@ -1326,13 +1226,15 @@ int get_observation_number (const char *s, const DATAINFO *pdinfo)
 
     if (pdinfo->structure == TIME_SERIES) {
 	t = dateton(test, pdinfo);
-	if (t >= 0) return t + 1;
+	if (t >= 0) {
+	    return t + 1;
+	}
     }
 
     if (calendar_data(pdinfo)) {
 	char datestr[OBSLEN];
 
-	charsub(test, ':', '/');
+	charsub(test, ':', '/'); /* FIXME */
 	for (t=0; t<pdinfo->n; t++) {
 	    calendar_date_string(datestr, t, pdinfo);
 	    if (!strcmp(test, datestr) ||
@@ -1451,26 +1353,13 @@ int get_t_from_obs_string (char *s, const double **Z,
     return t;
 }
 
-static int arma_model_stat_pos (const char *s, const MODEL *pmod)
+static void invalid_stat_error (int idx, int *err)
 {
-    int p = -1;
+    const char *mword = get_model_stat_word(idx);
 
-    if (numeric_string(s)) {
-	p = atoi(s) - 1;
-	if (p >= pmod->ncoeff) p = -1;
-	return p;
-    } else if (pmod->params != NULL) {
-	int i;
-
-	for (i=1; i<=pmod->ncoeff; i++) {
-	    if (!strcmp(s, pmod->params[i])) {
-		p = i - 1;
-		break;
-	    }
-	}
-    }
-
-    return p;
+    sprintf(gretl_errmsg, _("Invalid argument for %s"), 
+	    (mword != NULL)? mword : _("function"));
+    *err = E_INVARG;    
 }
 
 /* retrieve a specific element from one of the arrays of data
@@ -1482,7 +1371,7 @@ get_model_data_element (MODEL *pmod, int idx, const char *key,
 {
     char s[32] = {0};
     const char *p;
-    int lv, vi = 0;
+    int vi = 0;
     double x = NADBL;
 
     /* extract arg string in parentheses */
@@ -1497,7 +1386,7 @@ get_model_data_element (MODEL *pmod, int idx, const char *key,
 
     if (idx == M_RHO_S) {
 	if (!(numeric_string(s))) {
-	    *err = E_INVARG;
+	    invalid_stat_error(idx, err);
 	} else if (dot_atof(s) == 1 && AR1_MODEL(pmod->ci)) {
 	    x = gretl_model_get_double(pmod, "rho_in");
 	} else if (pmod->ci != AR && dot_atof(s) == 1) {
@@ -1505,35 +1394,26 @@ get_model_data_element (MODEL *pmod, int idx, const char *key,
 	} else if (pmod->arinfo == NULL || 
 		   pmod->arinfo->arlist == NULL || 
 		   pmod->arinfo->rho == NULL) {
-	    *err = E_INVARG;
+	    invalid_stat_error(idx, err);
 	} else if (!(vi = gretl_list_position(atoi(s), pmod->arinfo->arlist))) {
-	    *err = E_INVARG;
+	    invalid_stat_error(idx, err);
 	} else {
 	    x = pmod->arinfo->rho[vi-1];
 	}
     } else if (idx == M_VCV_S) {
 	x = genr_vcv(s, pdinfo, pmod);
 	if (na(x)) {
-	    *err = E_INVARG;
+	    invalid_stat_error(idx, err);
 	}
     } else if (idx == M_COEFF_S || idx == M_SE_S) {
 	if (pmod == NULL || pmod->list == NULL) {
-	    *err = E_INVARG;
-	} else if (pmod->ci == ARMA) {
-	    vi = arma_model_stat_pos(s, pmod);
-	    if (vi < 0) {
-		*err = E_INVARG;
-	    }
+	    invalid_stat_error(idx, err);
 	} else {
-	    lv = numeric_string(s)? atoi(s) : varindex(pdinfo, s);
-	    vi = gretl_list_position(lv, pmod->list);
-
-	    if (vi < 2) {
-		*err = E_INVARG;
-	    } else {
-		vi -= 2;
+	    vi = gretl_model_get_param_number(pmod, pdinfo, s);
+	    if (vi < 0) {
+		invalid_stat_error(idx, err);
 	    }
-	}
+	} 
 
 	if (!*err) {
 	    if (idx == M_COEFF_S && pmod->coeff != NULL) { 
@@ -1541,7 +1421,7 @@ get_model_data_element (MODEL *pmod, int idx, const char *key,
 	    } else if (pmod->sderr != NULL) {
 		x = pmod->sderr[vi];
 	    } else {
-		*err = E_INVARG;
+		invalid_stat_error(idx, err);
 	    }
 	}
     } 
