@@ -22,15 +22,13 @@
 
 /* private functions, used only in generate.c */
 
-int hp_filter (const double *x, double *hp, const DATAINFO *pdinfo);
-
-int bkbp_filter (const double *y, double *bk, const DATAINFO *pdinfo);
-
 int get_fracdiff (const double *y, double *diffvec, double d,
 		  const DATAINFO *pdinfo);
 
 int genrunit (double ***pZ, DATAINFO *pdinfo);
 
+int genrwkday (double ***pZ, DATAINFO *pdinfo);
+
 double genr_cov_corr (const char *s, double ***pZ, 
 		      const DATAINFO *pdinfo, int fn);
 
