@@ -114,6 +114,7 @@ enum transformations {
     T_MEDIAN,
     T_GINI,
     T_ZEROMISS,
+    T_CDF,
     T_PVALUE,
     T_CRIT,
     T_OBSNUM,
@@ -141,14 +142,21 @@ enum transformations {
     T_TRACE,
     T_SUMR,
     T_SUMC,
+    T_MEANR,
+    T_MEANC,
     T_1NORM,
     T_RCOND,
     T_ROWS,
     T_COLS,
     T_TRANSP,
+    T_VEC,
+    T_VECH,
+    T_UNVECH,
     T_IMAT,
     T_ZEROS,
     T_ONES,
+    T_MUNIF,
+    T_MNORM,
     T_IDENTITY,
     T_UFUNC
 };
