@@ -28,15 +28,17 @@
 
 typedef enum {
     GP_IMPULSES   = 1 << 0,  /* use impulses for plotting */
-    GP_RESIDS     = 1 << 1,  /* doing residual plot */
-    GP_FA         = 1 << 2,  /* doing fitted/actual plot */
-    GP_DUMMY      = 1 << 3,  /* using a dummy for separation */
-    GP_BATCH      = 1 << 4,  /* working in batch mode */
-    GP_GUI        = 1 << 5,  /* called from GUI context */
-    GP_OLS_OMIT   = 1 << 6,  /* Don't draw fitted line on graph */
-    GP_DATA_STYLE = 1 << 7,  /* data style is set by user */
-    GP_FILE       = 1 << 8   /* send output to named file */
-} gnuplot_flags;
+    GP_LINES      = 1 << 1,  /* force use of lines for plotting */
+    GP_RESIDS     = 1 << 2,  /* doing residual plot */
+    GP_FA         = 1 << 3,  /* doing fitted/actual plot */
+    GP_DUMMY      = 1 << 4,  /* using a dummy for separation */
+    GP_BATCH      = 1 << 5,  /* working in batch mode */
+    GP_GUI        = 1 << 6,  /* called from GUI context */
+    GP_OLS_OMIT   = 1 << 7,  /* Don't draw fitted line on graph */
+    GP_DATA_STYLE = 1 << 8,  /* data style is set by user */
+    GP_FILE       = 1 << 9,  /* send output to named file */
+    GP_IDX        = 1 << 10  /* plot against time or obs index */
+} GnuplotFlags;
 
 typedef enum {
     GPTSPEC_TS             = 1 << 0,
@@ -48,7 +50,7 @@ typedef enum {
     GPTSPEC_ALL_MARKERS    = 1 << 6,
     GPTSPEC_ALL_MARKERS_OK = 1 << 7,
     GPTSPEC_NO_BORDER      = 1 << 8
-} gptspec_flags; 
+} PlotSpecFlags; 
 
 #define MAXTITLE 128
 #define MAX_PLOT_LABELS 3
@@ -62,6 +64,7 @@ typedef struct {
     char scale[8];         /* string representation of scale factor */
     int yaxis;             /* 1 for left, 2 for right */
     int type;              /* 1, 2, ... (color) */
+    int width;             /* default 1, could be bigger */
     int ncols;             /* number of data columns (0 for formula) */
 } GPT_LINE;
 
@@ -73,12 +76,6 @@ typedef enum {
     GP_JUST_RIGHT
 } gp_just_codes;
 
-typedef enum {
-    DIST_NONE = 0,
-    DIST_NORMAL,
-    DIST_GAMMA
-} DistCode;
-
 typedef enum {
     PLOT_REGULAR = 0,
     PLOT_SAMPLING_DIST,
@@ -101,6 +98,7 @@ typedef enum {
     PLOT_ELLIPSE,
     PLOT_MULTI_IRF,
     PLOT_PANEL,
+    PLOT_BI_GRAPH,
     PLOT_TYPE_MAX
 } PlotType;
 
@@ -114,7 +112,7 @@ typedef struct {
     FILE *fp;
     char fname[MAXLEN];        /* for gui purposes */
     PlotType code;             /* to deal with FREQ, FCASTERR... */
-    unsigned int flags;        /* bitwise OR of options (gptspec_flags) */
+    PlotSpecFlags flags;        /* bitwise OR of options */
     int nobs;                  /* number of observations */
     char titles[4][MAXTITLE];  /* main, x, y, y2 */
     double range[3][2];        /* axis range specifiers */
@@ -156,21 +154,23 @@ PlotType plot_type_from_string (const char *str);
 
 int gnuplot_make_graph (void);
 
-int gnuplot (int *list, const int *lines, const char *literal,
-	     double ***pZ, DATAINFO *pdinfo, 
-	     int *plot_count, gnuplot_flags flags);
+GnuplotFlags gp_flags (int batch, gretlopt opt);
 
-int multi_scatters (const int *list, 
-		    double ***pZ, const DATAINFO *pdinfo, 
-		    int *plot_count, gnuplot_flags flags);
+int gnuplot (const int *plotlist, const int *lines, const char *literal,
+	     double ***pZ, DATAINFO *pdinfo, 
+	     int *plot_count, GnuplotFlags flags);
+
+int multi_scatters (const int *list, const double **Z,
+		    const DATAINFO *pdinfo, 
+		    int *plot_count, GnuplotFlags flags);
 
 int gnuplot_3d (int *list, const char *literal,
 		double ***pZ, DATAINFO *pdinfo, 
-		int *plot_count, gnuplot_flags flags);
+		int *plot_count, GnuplotFlags flags);
 
 int plot_freq (FreqDist *freq, DistCode dist);
 
-int garch_resid_plot (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo); 
+int garch_resid_plot (const MODEL *pmod, const DATAINFO *pdinfo); 
 
 int print_plotspec_details (const GPT_SPEC *spec, FILE *fp);
 
@@ -205,8 +205,9 @@ gretl_VAR_plot_multiple_irf (GRETL_VAR *var, int periods,
 			     const double **Z,
 			     const DATAINFO *pdinfo);
 
-int gretl_VAR_residual_plot (const GRETL_VAR *var, 
-			     double ***pZ, DATAINFO *pdinfo);
+int gretl_VAR_residual_plot (const GRETL_VAR *var, const DATAINFO *pdinfo);
+
+int gretl_VAR_residual_mplot (const GRETL_VAR *var, const DATAINFO *pdinfo); 
 
 int gretl_VAR_roots_plot (GRETL_VAR *var);
 
