@@ -55,6 +55,8 @@ static struct gretl_cmd gretl_cmds[] = {
     { DATA,     "data" },
     { DELEET,   "delete" },
     { DIFF,     "diff" },
+    { DISCRETE, "discrete" },
+    { DUMMIFY,  "dummify" },
     { ELSE,     "else" },
     { END,      "end" },
     { ENDIF,    "endif" },
@@ -84,7 +86,6 @@ static struct gretl_cmd gretl_cmds[] = {
     { INCLUDE,  "include" },
     { INFO,     "info" }, 
     { KPSS,     "kpss" },
-    { LABEL,    "label" },
     { LABELS,   "labels" },
     { LAD,      "lad" },
     { LAGS,     "lags" },    
@@ -114,12 +115,12 @@ static struct gretl_cmd gretl_cmds[] = {
     { PERGM,    "pergm" },
     { PLOT,     "plot" },    
     { POISSON,  "poisson" },
-    { POOLED,   "pooled" },
     { PRINT,    "print" }, 
     { PRINTF,   "printf" },
     { PROBIT,   "probit" },
     { PVALUE,   "pvalue" },  
     { PWE,      "pwe" },
+    { QLRTEST,  "qlrtest" }, 
     { QUIT,     "quit" }, 
     { REMEMBER, "remember" },
     { RENAME,   "rename" },
@@ -132,6 +133,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { SCATTERS, "scatters" },
     { SDIFF,    "sdiff" },
     { SET,      "set" },
+    { SETINFO,  "setinfo" },
     { SETOBS,   "setobs" },
     { SETMISS,  "setmiss" },
     { SHELL,    "shell" },   
@@ -152,6 +154,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { VECM,     "vecm" },
     { VIF,      "vif" },
     { WLS,      "wls" },
+    { XTAB,     "xtab" },
     { NC,       NULL}
 }; 
 
