@@ -45,6 +45,8 @@ typedef enum {
     DATA,
     DELEET,
     DIFF,
+    DISCRETE,
+    DUMMIFY,
     ELSE,
     END,
     ENDIF,
@@ -74,7 +76,6 @@ typedef enum {
     INCLUDE,
     INFO,
     KPSS,
-    LABEL,
     LABELS, 
     LAD,
     LAGS,    
@@ -104,12 +105,12 @@ typedef enum {
     PERGM,
     PLOT,    
     POISSON,
-    POOLED,
     PRINT, 
     PRINTF,
     PROBIT,
     PVALUE, 
     PWE,
+    QLRTEST,
     QUIT,
     REMEMBER,
     RENAME,
@@ -122,6 +123,7 @@ typedef enum {
     SCATTERS,
     SDIFF,
     SET,
+    SETINFO,
     SETOBS,
     SETMISS,
     SHELL,   
@@ -142,14 +144,16 @@ typedef enum {
     VECM,
     VIF,
     WLS,
+    XTAB,
     NC
 } GretlCmdIndex;
 
 #define NEEDS_TWO_VARS(c)  ((c) == AR || (c) == ARCH || (c) == COINT || \
-                           (c) == CORC || (c) == HCCM || (c) == HILU || \
-                           (c) == HSK || (c) == LOGIT || (c) == SPEARMAN || \
-                           (c) == OLS || (c) == POOLED || (c) == PROBIT || \
-                           (c) == TSLS || (c) == VAR || (c) == WLS || (c) == PWE)
+                            (c) == CORC || (c) == HCCM || (c) == HILU || \
+                            (c) == HSK || (c) == LOGIT || (c) == SPEARMAN || \
+                            (c) == OLS || (c) == PROBIT || \
+                            (c) == TSLS || (c) == VAR || (c) == WLS || \
+			    (c) == PWE || (c) == XTAB)
 
 #define TEXTSAVE_OK(c) (c == ADD || \
                         c == ADDTO || \
@@ -183,6 +187,7 @@ typedef enum {
                         c == PRINT || \
                         c == PRINTF || \
                         c == PVALUE || \
+                        c == QLRTEST || \
                         c == RESET || \
                         c == RMPLOT || \
                         c == RUNS || \
@@ -190,7 +195,26 @@ typedef enum {
                         c == SUMMARY || \
                         c == TESTUHAT || \
                         c == VIF || \
-                        c == VARTEST)
+                        c == VARTEST || \
+                        c == XTAB)
+
+#define NEEDS_MODEL_CHECK(c) (c == ADD || \
+                              c == OMIT || \
+                              c == COEFFSUM || \
+                              c == CUSUM || \
+                              c == RESET || \
+                              c == CHOW || \
+                              c == QLRTEST || \
+                              c == VIF || \
+                              c == TABPRINT || \
+                              c == EQNPRINT || \
+                              c == FCAST || \
+                              c == FIT || \
+                              c == FCASTERR || \
+                              c == HAUSMAN || \
+                              c == LEVERAGE || \
+                              c == LMTEST || \
+                              c == TESTUHAT)
 
 int gretl_command_number (const char *s);
 
