@@ -49,8 +49,7 @@ const char *gretl_error_messages[] = {
        "exceed those in the data set"),                          /* E_NODATA */
     N_("Command has insufficient arguments"),                    /* E_ARGS */
     N_("This command is implemented only for OLS models"),       /* E_OLSONLY */
-    N_("Invalid argument for coeff, corr, stderr, rho, "
-       "pvalue, critical or mpow"),                              /* E_INVARG */
+    N_("Invalid argument for function"),                         /* E_INVARG */
     N_("Invalid sample split for Chow test"),                    /* E_SPLIT */
     N_("Syntax error in command line"),                          /* E_PARSE */
     N_("No independent variables left after omissions"),         /* E_NOVARS */
@@ -65,6 +64,7 @@ const char *gretl_error_messages[] = {
     N_("Attempting to take square root of negative number"),     /* E_SQRT */
     N_("Excessive exponent in genr formula"),                    /* E_HIGH */
     N_("Weight variable is all zeros, aborting regression"),     /* E_WTZERO */
+    N_("Weight variable contains negative values"),              /* E_WTNEG */
     N_("Need valid starting and ending observations"),           /* E_OBS */
     N_("You must include a constant in this sort of model"),     /* E_NOCONST */
     N_("There were missing observations for the added "
@@ -78,6 +78,8 @@ const char *gretl_error_messages[] = {
     N_("Not a Number in calculation"),                           /* E_NAN */
     N_("Matrices not conformable for operation"),                /* E_NONCONF */
     N_("Data types not conformable for operation"),              /* E_TYPES */
+    N_("Wrong data type"),                                       /* E_DATATYPE */
+    N_("Incompatible options"),                                  /* E_BADOPT */
     NULL,                                                        /* E_DB_DUP */
     NULL,                                                        /* E_OK */
     NULL                                                         /* E_MAX */
