@@ -21,13 +21,16 @@
 #include "gretl_list.h"
 #include "gretl_func.h"
 #include "libset.h"
+#include "gretl_xml.h"
 
 #define LDEBUG 0
 
+#define LNAMELEN 32
+
 typedef struct saved_list_ saved_list;
 
 struct saved_list_ {
-    char name[VNAMELEN];
+    char name[LNAMELEN];
     int *list;
     int level;
 };
@@ -55,7 +58,7 @@ static saved_list *saved_list_new (const int *list, const char *name)
 	    sl = NULL;
 	} else {
 	    *sl->name = 0;
-	    strncat(sl->name, name, VNAMELEN - 1);
+	    strncat(sl->name, name, LNAMELEN - 1);
 	}
     }
 
@@ -64,8 +67,10 @@ static saved_list *saved_list_new (const int *list, const char *name)
 
 static void free_saved_list (saved_list *sl)
 {
-    free(sl->list);
-    free(sl);
+    if (sl != NULL) {
+	free(sl->list);
+	free(sl);
+    }
 }
 
 static saved_list *get_saved_list_by_name (const char *name)
@@ -87,6 +92,34 @@ static saved_list *get_saved_list_by_name (const char *name)
     return NULL;
 }
 
+/**
+ * n_saved_lists:
+ *
+ * Returns: the number of saved (named) lists currently defined.
+ */
+
+int n_saved_lists (void)
+{
+    return n_lists;
+}
+
+/**
+ * get_list_name_by_index:
+ * @idx: 0-based index into array of saved lists.
+ *
+ * Returns: the name of the specified saved list, or %NULL if
+ * @idx is out of bounds.
+ */
+
+const char *get_list_name_by_index (int idx)
+{
+    if (idx >= 0 && idx < n_lists) {
+	return list_stack[idx]->name;
+    } else {
+	return NULL;
+    }
+}
+
 /**
  * get_list_by_name:
  * @name: the name of the list to be found.
@@ -149,7 +182,13 @@ static int real_remember_list (const int *list, const char *name,
 	    err = E_ALLOC;
 	} else {
 	    if (gretl_messages_on()) {
-		pprintf(prn, "Added list '%s'\n", name);
+		const char *realname = list_stack[n_lists]->name;
+
+		pprintf(prn, "Added list '%s'\n", realname);
+		if (strlen(realname) < strlen(name)) {
+		    pprintf(prn, "Warning: the name was truncated to %d characters\n", 
+			    LNAMELEN - 1);
+		}
 	    }
 	    n_lists++;
 	}
@@ -325,7 +364,8 @@ void gretl_lists_cleanup (void)
  * holds a count of the number of elements that follow.  This
  * leading element is initialized appropriately.  For example, if
  * @nterms = 4, space for 5 integers is allocated and the first
- * element of the array is set to 4.
+ * element of the array is set to 4.  The other elements of 
+ * the list are initialized to 0.
  *
  * Returns: the newly allocated list, or %NULL on failure.
  */
@@ -468,14 +508,6 @@ int *gretl_list_append_term (int **plist, int v)
     return list;
 }
 
-int gretl_compare_ints (const void *a, const void *b)
-{
-    const int *ia = (const int *) a;
-    const int *ib = (const int *) b;
-     
-    return (*ia > *ib) - (*ia < *ib);
-}
-
 /**
  * gretl_list_sort:
  * @list: list to be sorted.
@@ -507,7 +539,7 @@ int *gretl_list_sort (int *list)
 /**
  * gretl_list_cmp:
  * @list1: gretl list.
- * @list1: gretl list.
+ * @list2: gretl list.
  * 
  * Returns: 0 if @list1 and @list2 have identical content,
  * otherwise 1.
@@ -590,6 +622,10 @@ int *gretl_list_from_string (const char *liststr)
     int *list;
     int n = 0;
 
+    if (liststr == NULL) {
+	return NULL;
+    }
+
     while (*s) {
 	s += strspn(s, " ,()");
 	if (*s && sscanf(s, "%7[^ ,()]", numstr)) {
@@ -612,7 +648,11 @@ int *gretl_list_from_string (const char *liststr)
     while (*s) {
 	s += strspn(s, " ,()");
 	if (*s && sscanf(s, "%7[^ ,()]", numstr)) {
-	    list[n++] = atoi(numstr);
+	    if (*numstr == ';') {
+		list[n++] = LISTSEP;
+	    } else {
+		list[n++] = atoi(numstr);
+	    }
 	    s += strlen(numstr);
 
 	}
@@ -968,10 +1008,10 @@ static int list_count (const int *list)
 
 int *gretl_list_omit (const int *orig, const int *omit, int minpos, int *err)
 {
-    int i, j, k;
-    int *smal;
     const int nomit = omit[0];
     const int norig = list_count(orig);
+    int *smal = NULL;
+    int i, j, k;
 
     *err = 0;
 
@@ -1095,6 +1135,10 @@ int *gretl_list_diff_new (const int *biglist, const int *sublist,
     int i, j, n, k = 0;
     int match;
 
+    if (biglist == NULL || sublist == NULL) {
+	return NULL;
+    }
+
     n = biglist[0] - sublist[0];
 
     if (n <= 0) {
@@ -1388,7 +1432,7 @@ int gretl_list_split_on_separator (const int *list, int **plist1, int **plist2)
     }
 
     if (n < 0) {
-	return 1;
+	return E_PARSE;
     }
 
     list1 = gretl_list_new(n - 1);
@@ -1490,7 +1534,7 @@ int *full_var_list (const DATAINFO *pdinfo, int *nvars)
     int *list = NULL;
 
     for (i=1; i<pdinfo->v; i++) {
-	if (pdinfo->vector[i] && !is_hidden_variable(i, pdinfo)) {
+	if (var_is_series(pdinfo, i) && !var_is_hidden(pdinfo, i)) {
 	    nv++;
 	}
     }
@@ -1506,7 +1550,7 @@ int *full_var_list (const DATAINFO *pdinfo, int *nvars)
     if (list != NULL) {
 	j = 1;
 	for (i=1; i<pdinfo->v; i++) {
-	    if (pdinfo->vector[i] && !is_hidden_variable(i, pdinfo)) {
+	    if (var_is_series(pdinfo, i) && !var_is_hidden(pdinfo, i)) {
 		list[j++] = i;
 	    }
 	}
@@ -1569,3 +1613,118 @@ int gretl_list_is_consecutive (const int *list)
     return ret;
 }
 
+int load_user_lists_file (const char *fname)
+{
+    xmlDocPtr doc = NULL;
+    xmlNodePtr node = NULL;
+    int i, nl, err = 0;
+
+    xmlKeepBlanksDefault(0);
+
+    err = gretl_xml_open_doc_root(fname, "gretl-lists", &doc, &node);
+    if (err) {
+	return err;
+    }
+
+    if (!gretl_xml_get_prop_as_int(node, "count", &nl)) {
+	err = E_DATA;
+    } else if (nl <= 0) {
+	err = E_DATA;
+    }
+
+    if (!err) {
+	list_stack = malloc(nl * sizeof *list_stack);
+	if (list_stack == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    n_lists = nl;
+	    for (i=0; i<nl; i++) {
+		list_stack[i] = NULL;
+	    }
+	    for (i=0; i<nl && !err; i++) {
+		list_stack[i] = malloc(sizeof **list_stack);
+		if (list_stack[i] == NULL) {
+		    err = E_ALLOC;
+		} else {
+		    list_stack[i]->name[0] = '\0';
+		    list_stack[i]->list = NULL;
+		    list_stack[i]->level = 0;
+		}
+	    }
+	}
+    }
+
+    if (!err) {
+	xmlNodePtr cur = node->xmlChildrenNode;
+	char *lname;
+
+	i = 0;
+	while (cur != NULL && !err) {
+	    if (!xmlStrcmp(cur->name, (XUC) "list")) {
+		if (!gretl_xml_get_prop_as_string(cur, "name", &lname)) {
+		    err = E_DATA;
+		} else {
+		    strncat(list_stack[i]->name, lname, LNAMELEN - 1);
+		    free(lname);
+		    list_stack[i]->list = 
+			gretl_xml_node_get_list(cur, doc, &err);
+		    i++;
+		}
+	    }
+	    cur = cur->next;
+	}
+    }
+
+    if (err && list_stack != NULL) {
+	gretl_lists_cleanup();
+    }
+
+    if (doc != NULL) {
+	xmlFreeDoc(doc);
+	xmlCleanupParser();
+    }
+
+    return err;
+}
+
+/**
+ * gretl_serialize_lists:
+ * @fname: name of file to which output should be written.
+ *
+ * Prints an XML representation of the current saved lists,
+ * if any.
+ *
+ * Returns: 0 on success, or if there are no saved lists, 
+ * non-zero code on error.
+ */
+
+int gretl_serialize_lists (const char *fname)
+{
+    FILE *fp;
+    int i;
+
+    if (n_lists == 0) {
+	return 0;
+    }
+
+    fp = gretl_fopen(fname, "w");
+    if (fp == NULL) {
+	return E_FOPEN;
+    }
+
+    gretl_xml_header(fp); 
+
+    fprintf(fp, "<gretl-lists count=\"%d\">\n", n_lists);
+
+    for (i=0; i<n_lists; i++) {
+	gretl_xml_put_named_list(list_stack[i]->name, 
+				 list_stack[i]->list, 
+				 fp);
+    }
+
+    fputs("</gretl-lists>\n", fp);
+
+    fclose(fp);
+
+    return 0;
+}
