@@ -126,16 +126,18 @@ gretl_matrix *gretl_matrix_alloc (int rows, int cols);
 
 gretl_matrix *gretl_matrix_reuse (gretl_matrix *m, int rows, int cols);
 
-gretl_matrix *gretl_packed_matrix_alloc (int rows);
-
 gretl_matrix *gretl_identity_matrix_new (int n);
 
 gretl_matrix *gretl_zero_matrix_new (int r, int c);
 
 gretl_matrix *gretl_unit_matrix_new (int r, int c);
 
+gretl_matrix *gretl_null_matrix_new (void);
+
 gretl_matrix *gretl_matrix_copy (const gretl_matrix *m);
 
+int gretl_matrix_inscribe_I (gretl_matrix *m, int row, int col, int n);
+
 gretl_matrix *gretl_matrix_copy_transpose (const gretl_matrix *m);
 
 gretl_vector *gretl_column_vector_from_array (const double *x, 
@@ -158,6 +160,8 @@ double gretl_matrix_trace (const gretl_matrix *m, int *err);
 
 int gretl_matrix_random_fill (gretl_matrix *m, int dist);
 
+gretl_matrix *gretl_random_matrix_new (int r, int c, int dist);
+
 int gretl_matrix_transform_elements (gretl_matrix *m, GretlMathFunc fn);
 
 double gretl_vector_mean (const gretl_vector *v);
@@ -170,6 +174,8 @@ int gretl_matrix_zero_upper (gretl_matrix *m);
 
 int gretl_matrix_zero_lower (gretl_matrix *m);
 
+void gretl_matrix_fill (gretl_matrix *m, double x);
+
 void gretl_matrix_multiply_by_scalar (gretl_matrix *m, double x);
 
 int gretl_matrix_divide_by_scalar (gretl_matrix *m, double x);
@@ -188,12 +194,26 @@ int gretl_matrix_add_to (gretl_matrix *targ, const gretl_matrix *src);
 int 
 gretl_matrix_subtract_from (gretl_matrix *targ, const gretl_matrix *src);
 
+int gretl_matrix_I_minus (gretl_matrix *m);
+
 int gretl_matrix_transpose (gretl_matrix *targ, const gretl_matrix *src);
 
 int gretl_square_matrix_transpose (gretl_matrix *m);
 
 int gretl_matrix_add_self_transpose (gretl_matrix *m);
 
+int 
+gretl_matrix_vectorize (gretl_matrix *targ, const gretl_matrix *src);
+
+int 
+gretl_matrix_unvectorize (gretl_matrix *targ, const gretl_matrix *src);
+
+int 
+gretl_matrix_vectorize_h (gretl_matrix *targ, const gretl_matrix *src);
+
+int 
+gretl_matrix_unvectorize_h (gretl_matrix *targ, const gretl_matrix *src);
+
 int gretl_matrix_multiply_mod (const gretl_matrix *a, GretlMatrixMod amod,
 			       const gretl_matrix *b, GretlMatrixMod bmod,
 			       gretl_matrix *c);
@@ -201,8 +221,13 @@ int gretl_matrix_multiply_mod (const gretl_matrix *a, GretlMatrixMod amod,
 int gretl_matrix_multiply (const gretl_matrix *a, const gretl_matrix *b,
 			   gretl_matrix *c);
 
+int
+gretl_matrix_kronecker_product (const gretl_matrix *A, const gretl_matrix *B,
+				gretl_matrix *K);
+
 gretl_matrix *
-gretl_matrix_kronecker_product (const gretl_matrix *A, const gretl_matrix *B);
+gretl_matrix_kronecker_product_new (const gretl_matrix *A, 
+				    const gretl_matrix *B);
 
 double gretl_matrix_dot_product (const gretl_matrix *a, GretlMatrixMod amod,
 				 const gretl_matrix *b, GretlMatrixMod bmod,
@@ -223,9 +248,13 @@ gretl_matrix *gretl_matrix_row_sum (const gretl_matrix *m);
 
 gretl_matrix *gretl_matrix_column_sum (const gretl_matrix *m);
 
-double gretl_matrix_row_mean (const gretl_matrix *m, int row);
+gretl_matrix *gretl_matrix_row_mean (const gretl_matrix *m);
 
-double gretl_matrix_column_mean (const gretl_matrix *m, int col);
+gretl_matrix *gretl_matrix_column_mean (const gretl_matrix *m);
+
+double gretl_matrix_row_i_mean (const gretl_matrix *m, int row);
+
+double gretl_matrix_column_j_mean (const gretl_matrix *m, int col);
 
 void gretl_matrix_demean_by_row (gretl_matrix *m);
 
@@ -249,6 +278,10 @@ int gretl_invert_general_matrix (gretl_matrix *a);
 
 int gretl_invert_symmetric_matrix (gretl_matrix *a);
 
+int gretl_invert_symmetric_matrix2 (gretl_matrix *a, double *ldet);
+
+int gretl_invert_packed_symmetric_matrix (gretl_matrix *v);
+
 int gretl_invert_diagonal_matrix (gretl_matrix *a);
 
 int gretl_invert_matrix (gretl_matrix *a);
@@ -273,6 +306,8 @@ gretl_matrix *
 gretl_matrix_col_concat (const gretl_matrix *a, const gretl_matrix *b,
 			 int *err);
 
+gretl_matrix *gretl_matrix_lag (gretl_matrix *m, int k, double missval);
+
 int gretl_matrix_cholesky_decomp (gretl_matrix *a);
 
 int gretl_matrix_QR_decomp (gretl_matrix *M, gretl_matrix *R);
@@ -315,6 +350,8 @@ gretl_matrix_print_to_prn (const gretl_matrix *m, const char *msg, PRN *prn);
 
 void gretl_matrix_print (const gretl_matrix *m, const char *msg);
 
+void gretl_packed_matrix_print (const gretl_matrix *m, const char *msg);
+
 void debug_print_matrix (const gretl_matrix *m, const char *msg);
 
 void gretl_matrix_set_int (gretl_matrix *m, int t);
@@ -323,7 +360,7 @@ int gretl_matrix_get_int (const gretl_matrix *m);
 
 int gretl_is_identity_matrix (const gretl_matrix *m);
 
-int gretl_is_zero_vector (const gretl_vector *v);
+int gretl_is_zero_matrix (const gretl_matrix *m);
 
 int gretl_matrices_are_equal (const gretl_matrix *a, const gretl_matrix *b,
 			      int *err);
@@ -343,6 +380,12 @@ gretl_matrix_array_alloc_with_size (int n, int rows, int cols);
 void gretl_matrix_array_free (gretl_matrix **A, int n);
 
 gretl_matrix *gretl_matrix_data_subset (const int *list, const double **Z,
-					int t1, int t2, char **pmask);
+					int t1, int t2, const char *mask);
+
+gretl_matrix *
+gretl_matrix_data_subset_no_missing (const int *list, const double **Z,
+				     int t1, int t2, int *err);
+
+void lapack_mem_free (void);
 
 #endif /* GRETL_MATRIX_H */
