@@ -20,6 +20,8 @@
 #ifndef GRETL_MODEL_H
 #define GRETL_MODEL_H
 
+#include "objstack.h"
+
 typedef enum {
     M_ESS = 1,    /* error sum of squares */
     M_T,          /* observations used */
@@ -40,12 +42,16 @@ typedef enum {
     M_ELEM_MAX,   /* -- separator -- */
     M_UHAT,       /* residuals */
     M_YHAT,       /* fitted values */
+    M_AHAT,       /* per-unit intercepts in panel model */
     M_H,          /* GARCH predicted variances */
     M_SERIES_MAX, /* -- separator -- */
     M_COEFF,      /* parameter estimates */
     M_SE,         /* parameter standard errors */
     M_VCV,        /* parameter covariance matrix */
     M_RHO,        /* autoregressive coefficients */
+    M_JALPHA,     /* Johansen's alpha */
+    M_JBETA,      /* Johansen's beta */
+    M_JVBETA,     /* Covariance matrix for Johansen's normalised beta */
     M_MAX         /* sentinel */
 } ModelDataIndex;
 
@@ -54,6 +60,29 @@ typedef enum {
 #define model_data_is_series(i) (i > M_ELEM_MAX && i < M_SERIES_MAX)
 #define model_data_is_matrix(i) (i > M_ELEM_MAX && i != M_SERIES_MAX)
 
+typedef enum {
+    MODEL_DATA_NONE,
+    MODEL_DATA_INT,
+    MODEL_DATA_LIST,
+    MODEL_DATA_DOUBLE,
+    MODEL_DATA_INT_ARRAY,
+    MODEL_DATA_DOUBLE_ARRAY,
+    MODEL_DATA_STRING,
+    MODEL_DATA_CHAR_ARRAY,
+    MODEL_DATA_CMPLX_ARRAY,
+    MODEL_DATA_STRUCT
+} ModelDataType;
+
+typedef enum {
+    ARMA_SEAS  = 1 << 0, /* includes seasonal component */
+    ARMA_DSPEC = 1 << 1, /* input list includes differences */
+    ARMA_X12A  = 1 << 2, /* using X-12-ARIMA to generate estimates */
+    ARMA_EXACT = 1 << 3, /* using exact ML */
+    ARMA_VECH  = 1 << 4  /* using vech representation when computing
+			    variance matrix of state for Kalman filter
+			 */
+} ArmaFlags;
+
 typedef struct CoeffIntervals_ CoeffIntervals;
 
 struct CoeffIntervals_ {
@@ -129,6 +158,7 @@ typedef enum {
     GRETL_TEST_AUTOCORR,
     GRETL_TEST_CHOW,
     GRETL_TEST_CUSUM,
+    GRETL_TEST_QLR,
     GRETL_TEST_GROUPWISE,
     GRETL_TEST_LOGS,
     GRETL_TEST_NORMAL,
@@ -138,15 +168,24 @@ typedef enum {
     GRETL_TEST_WHITES,
     GRETL_TEST_SARGAN,
     GRETL_TEST_TSLS_HAUSMAN,
+    GRETL_TEST_PANEL_HAUSMAN,
+    GRETL_TEST_PANEL_F,
+    GRETL_TEST_PANEL_BP,
     GRETL_TEST_MAX
 } ModelTestType;
 
+#ifndef GRETLCLI
+int attach_model_tests_from_xml (MODEL *pmod, xmlNodePtr node);
+#endif
+
 MODEL *gretl_model_new (void);
 
 MODEL *gretl_model_new_protected (void);
 
 void gretl_model_init (MODEL *pmod);
 
+int gretl_model_allocate_storage (MODEL *pmod);
+
 MODEL **gretl_model_array_new (int n);
 
 void gretl_model_array_destroy (MODEL **models, int n);
@@ -163,14 +202,17 @@ void gretl_model_free (MODEL *pmod);
 
 void gretl_model_free_on_exit (MODEL *pmod);
 
-int gretl_model_set_data_with_destructor (MODEL *pmod, const char *key, 
-					  void *ptr, size_t size, 
+int gretl_model_set_data_with_destructor (MODEL *pmod, const char *key, void *ptr, 
+					  ModelDataType type, size_t size, 
 					  void (*destructor) (void *));
 
-int gretl_model_set_data (MODEL *pmod, const char *key, void *ptr, size_t size);
+int gretl_model_set_data (MODEL *pmod, const char *key, void *ptr, 
+			  ModelDataType type, size_t size);
 
 int gretl_model_set_list_as_data (MODEL *pmod, const char *key, int *list);
 
+int gretl_model_set_string_as_data (MODEL *pmod, const char *key, char *str);
+
 int gretl_model_destroy_data_item (MODEL *pmod, const char *key);
 
 int gretl_model_detach_data_item (MODEL *pmod, const char *key);
@@ -188,10 +230,16 @@ int gretl_model_get_int (const MODEL *pmod, const char *key);
 
 double gretl_model_get_double (const MODEL *pmod, const char *key);
 
+int *gretl_model_get_list (const MODEL *pmod, const char *key);
+
 char *gretl_model_get_param_name (const MODEL *pmod, 
 				  const DATAINFO *pdinfo,
 				  int i, char *targ);
 
+int gretl_model_get_param_number (const MODEL *pmod, 
+				  const DATAINFO *pdinfo,
+				  char *pname);
+
 void free_coeff_intervals (CoeffIntervals *cf);
 
 CoeffIntervals *
@@ -202,24 +250,34 @@ int gretl_model_get_depvar (const MODEL *pmod);
 
 int *gretl_model_get_x_list (const MODEL *pmod);
 
-int gretl_arma_model_nonseasonal_AR_order (const MODEL *pmod);
+int arma_model_nonseasonal_AR_order (const MODEL *pmod);
 
-int gretl_arma_model_nonseasonal_MA_order (const MODEL *pmod);
+int arma_model_nonseasonal_MA_order (const MODEL *pmod);
 
-int gretl_arma_model_max_AR_lag (const MODEL *pmod);
+int arma_model_max_AR_lag (const MODEL *pmod);
 
-int gretl_arma_model_max_MA_lag (const MODEL *pmod);
+int arma_model_max_MA_lag (const MODEL *pmod);
 
-int gretl_arma_model_get_AR_MA_coeffs (const MODEL *pmod,
-				       double **arvec,
-				       double **mavec);
+int arma_model_integrated_AR_MA_coeffs (const MODEL *pmod,
+					double **phi_star,
+					double **theta_star);
 
-const double *gretl_arma_model_get_x_coeffs (const MODEL *pmod);
+int regarma_model_AR_coeffs (const MODEL *pmod,
+			     double **phi0,
+			     int *pp);
+
+const double *arma_model_get_x_coeffs (const MODEL *pmod);
+
+int regarima_model_get_AR_coeffs (const MODEL *pmod,
+				  double **phi0,
+				  int *pp);
 
 int gretl_model_new_vcv (MODEL *pmod, int *nelem);
 
 VMatrix *gretl_model_get_vcv (MODEL *pmod, const DATAINFO *pdinfo);
 
+int gretl_model_add_arinfo (MODEL *pmod, int nterms);
+
 MODEL *gretl_model_copy (const MODEL *pmod);
 
 void swap_models (MODEL *targ, MODEL *src);
@@ -230,6 +288,15 @@ int is_quiet_model_test (int ci, gretlopt opt);
 
 int command_ok_for_model (int test_ci, int model_ci);
 
+int model_test_ok (int ci, gretlopt opt, const MODEL *pmod, 
+		   const DATAINFO *pdinfo);
+
+int gretl_is_arima_model (const MODEL *pmod);
+
+int get_first_model_stat (const char **word, const char **desc);
+
+int get_next_model_stat (const char **word, const char **desc);
+
 int get_model_count (void);
 
 void reset_model_count (void);
@@ -241,6 +308,7 @@ void model_count_minus (void);
 void set_model_id (MODEL *pmod);
 
 ModelTest *model_test_new (ModelTestType ttype);
+void model_test_free (ModelTest *test);
 
 int maybe_add_test_to_model (MODEL *pmod, ModelTest *test);
 
@@ -252,9 +320,13 @@ void model_test_set_value (ModelTest *test, double val);
 void model_test_set_pvalue (ModelTest *test, double pval);
 void model_test_set_param (ModelTest *test, const char *s);
 void model_test_set_allocated_param (ModelTest *test, char *s);
+void model_test_set_crit_and_alpha (ModelTest *test, 
+				    double crit,
+				    double alpha);
 
 void gretl_model_test_print (const MODEL *pmod, int i, PRN *prn);
 void gretl_model_print_last_test (const MODEL *pmod, PRN *prn);
+void gretl_model_test_print_direct (const ModelTest *test, PRN *prn);
 
 void gretl_model_destroy_tests (MODEL *pmod);
 
@@ -271,6 +343,11 @@ int gretl_model_add_arma_varnames (MODEL *pmod, const DATAINFO *pdinfo,
 				   int yno, int p, int q, int P, int Q, 
 				   int r);
 
+int gretl_model_add_panel_varnames (MODEL *pmod, const DATAINFO *pdinfo,
+				    const int *ulist);
+
+void gretl_model_add_allocated_varnames (MODEL *pmod, char **vnames);
+
 char *gretl_model_get_fitted_formula (const MODEL *pmod, int xvar,
 				      const double **Z,
 				      const DATAINFO *pdinfo);
@@ -291,4 +368,13 @@ gretl_model_get_series (const MODEL *pmod, const DATAINFO *pdinfo,
 gretl_matrix *gretl_model_get_matrix (MODEL *pmod, ModelDataIndex idx, 
 				      int *err);
 
+int gretl_model_serialize (const MODEL *pmod, SavedObjectFlags flags,
+			   FILE *fp);
+
+#ifndef GRETLCLI
+
+MODEL *gretl_model_from_XML (xmlNodePtr node, xmlDocPtr doc, int *err);
+
+#endif
+
 #endif /* GRETL_MODEL_H */
