@@ -30,7 +30,7 @@
 # include <dirent.h>
 #endif
 
-#ifdef USE_GTK2
+#ifdef USE_GLIB2
 # include <glib.h>
 # if (GLIB_MAJOR_VERSION >= 2) && (GLIB_MINOR_VERSION >= 6)
 #  ifdef WIN32
@@ -43,21 +43,20 @@ enum {
     CURRENT_DIR,
     DATA_SEARCH,
     SCRIPT_SEARCH,
+    FUNCS_SEARCH,
     USER_SEARCH
 };
 
 static void ensure_slash (char *str);
 
-static int add_gdt_suffix (char *fname)
+static int add_suffix (char *fname, const char *sfx)
 {
-    int added = 0;
-
     if (strrchr(fname, '.') == NULL) {
-	strcat(fname, ".gdt");
-	added = 1;
+	strcat(fname, sfx);
+	return 1;
     }
 
-    return added;
+    return 0;
 }
 
 FILE *gretl_fopen (const char *filename, const char *mode)
@@ -109,6 +108,24 @@ gzFile gretl_gzopen (const char *filename, const char *mode)
     return fz;
 }
 
+int gretl_is_xml_file (const char *fname)
+{
+    gzFile fz;
+    char test[6];
+    int ret = 0;
+
+    fz = gretl_gzopen(fname, "rb");
+    if (fz != Z_NULL) {
+	if (gzread(fz, test, 5)) {
+	    test[5] = '\0';
+	    if (!strcmp(test, "<?xml")) ret = 1;
+	} 
+	gzclose(fz);
+    } 
+
+    return ret;
+} 
+
 int gretl_path_prepend (char *file, const char *path)
 {
     char temp[MAXLEN];
@@ -150,7 +167,7 @@ static int try_open_file (char *targ, const char *finddir,
 
     fp = gretl_fopen(tmp, "r");
     if (fp == NULL && code == DATA_SEARCH) {
-	if (add_gdt_suffix(tmp)) {
+	if (add_suffix(tmp, ".gdt")) {
 	    fp = gretl_fopen(tmp, "r");
 	}
     }
@@ -233,7 +250,7 @@ static int try_open_file (char *targ, const char *finddir,
 
     fp = gretl_fopen(tmp, "r");
     if (fp == NULL && code == DATA_SEARCH) {
-	if (add_gdt_suffix(tmp)) {
+	if (add_suffix(tmp, ".gdt")) {
 	    fp = gretl_fopen(tmp, "r");
 	}
     }
@@ -316,7 +333,13 @@ static char *search_dir (char *fname, const char *topdir, int code)
 	    fclose(test);
 	    return fname;
 	}
-	if (code == DATA_SEARCH && add_gdt_suffix(fname)) {
+	if (code == DATA_SEARCH && add_suffix(fname, ".gdt")) {
+	    test = gretl_fopen(fname, "r");
+	    if (test != NULL) {
+		fclose(test);
+		return fname;
+	    }
+	} else if (code == FUNCS_SEARCH && add_suffix(fname, ".gfn")) {
 	    test = gretl_fopen(fname, "r");
 	    if (test != NULL) {
 		fclose(test);
@@ -402,6 +425,10 @@ char *addpath (char *fname, PATHS *ppaths, int script)
 	return NULL;
     }
 
+    if (ppaths == NULL) {
+	return NULL;
+    }
+
     /* try looking where script was found */
     if (*ppaths->currdir) {
 	if ((fname = search_dir(fname, ppaths->currdir, CURRENT_DIR))) {
@@ -416,6 +443,15 @@ char *addpath (char *fname, PATHS *ppaths, int script)
 	/* for a script, try system script dir (and subdirs) */
 	if ((fname = search_dir(fname, ppaths->scriptdir, SCRIPT_SEARCH))) { 
 	    return fname;
+	} else {
+	    char fndir[MAXLEN];
+
+	    fname = tmp;
+	    strcpy(fname, orig);
+	    sprintf(fndir, "%sfunctions", ppaths->gretldir);
+	    if ((fname = search_dir(fname, fndir, FUNCS_SEARCH))) { 
+		return fname;
+	    }
 	}
     } else {
 	/* for a data file, try system data dir (and subdirs) */
@@ -570,7 +606,7 @@ struct INTERNAL_PATHS {
     char libpath[MAXLEN];
     char x12a[MAXLEN];
     char x12adir[MAXLEN];
-    char pngfont[32];
+    char pngfont[128];
     unsigned char status;
 };
 
@@ -581,11 +617,7 @@ static void set_gretl_libpath (const char *path)
 #ifdef WIN32
     strcpy(gretl_paths.libpath, path);
 #else
-# ifdef USE_GTK2
     const char *sfx = "-gtk2/";
-# else
-    const char *sfx = "-gtk1/";
-# endif
     char *p = strstr(path, "/share");
 
     if (p) {
@@ -830,19 +862,11 @@ int set_paths (PATHS *ppaths, gretlopt opt)
 	    /* force english */
 	    sprintf(ppaths->helpfile, "%sgretlgui.hlp", ppaths->gretldir);
 	    sprintf(ppaths->cli_helpfile, "%sgretlcli.hlp", ppaths->gretldir);
-#ifdef USE_GTK2
 	    sprintf(ppaths->cmd_helpfile, "%sgretlcmd.hlp", ppaths->gretldir);
-#else
-	    sprintf(ppaths->cmd_helpfile, "%sgretlcli.hlp", ppaths->gretldir);
-#endif
 	} else {
 	    sprintf(ppaths->helpfile, "%s%s", ppaths->gretldir, _("gretlgui.hlp"));
 	    sprintf(ppaths->cli_helpfile, "%s%s", ppaths->gretldir, _("gretlcli.hlp"));
-#ifdef USE_GTK2
 	    sprintf(ppaths->cmd_helpfile, "%s%s", ppaths->gretldir, _("gretlcmd.hlp"));
-#else
-	    sprintf(ppaths->cmd_helpfile, "%s%s", ppaths->gretldir, _("gretlcli.hlp"));
-#endif
 	}
     } else {
 	sprintf(ppaths->helpfile, "%s%s", ppaths->gretldir, _("gretlcli.hlp"));
