@@ -37,7 +37,7 @@ typedef enum {
     GRETL_FORMAT_EQN       = 1 << 5,
     GRETL_FORMAT_SELECTION = 1 << 6,
     GRETL_FORMAT_CSV       = 1 << 7,
-    GRETL_FORMAT_TABLE     = 1 << 8,
+    GRETL_FORMAT_TAB       = 1 << 8,
 } PrnFormat;
 
 /* functions follow */
@@ -58,6 +58,8 @@ void gretl_print_set_format (PRN *prn, PrnFormat format);
 
 void gretl_print_toggle_doc_flag (PRN *prn);
 
+void gretl_print_set_delim (PRN *prn, char delim);
+
 int pprintf (PRN *prn, const char *template, ...);
 
 int pputs (PRN *prn, const char *s);
@@ -90,4 +92,6 @@ int tex_eqn_format (PRN *prn);
 
 int csv_format (PRN *prn);
 
+char prn_delim (PRN *prn);
+
 #endif /* GRETL_PRN_H */
