@@ -40,10 +40,9 @@ struct restriction_set_ {
     int cross;                    /* includes cross-equation restrictions? */
     char *mask;                   /* selection mask for coeffs */
     restriction **restrictions;
-    MODEL *pmod;
-    gretl_equation_system *sys;
-    GRETL_VAR *var;
-    const DATAINFO *pdinfo;
+    void *obj;
+    GretlObjType type;
+    gretlopt opt;
 };
 
 static void destroy_restriction_set (gretl_restriction_set *rset);
@@ -72,7 +71,8 @@ static int
 get_R_vecm_column (const gretl_restriction_set *rset, int i, int j)
 {
     const restriction *r = rset->restrictions[i];
-    int nb = gretl_VECM_n_beta(rset->var);
+    GRETL_VAR *var = rset->obj;
+    int nb = gretl_VECM_n_beta(var);
     int col = r->coeff[j];
     int k;
 
@@ -89,12 +89,13 @@ static int
 get_R_sys_column (const gretl_restriction_set *rset, int i, int j)
 {
     const restriction *r = rset->restrictions[i];
+    gretl_equation_system *sys = rset->obj;
     const int *list;
     int col = r->coeff[j];
     int k;
 
     for (k=0; k<r->eq[j]; k++) {
-	list = system_get_list(rset->sys, k);
+	list = system_get_list(sys, k);
 	col += list[0] - 1;
     }
 
@@ -103,9 +104,9 @@ get_R_sys_column (const gretl_restriction_set *rset, int i, int j)
 
 static int get_R_column (const gretl_restriction_set *rset, int i, int j)
 {
-    if (rset->sys != NULL) {
+    if (rset->type == GRETL_OBJ_SYS) {
 	return get_R_sys_column(rset, i, j);
-    } else if (rset->var != NULL) {
+    } else if (rset->type == GRETL_OBJ_VAR) {
 	return get_R_vecm_column(rset, i, j);
     } else {
 	return 0;
@@ -131,12 +132,14 @@ static int R_n_columns (gretl_restriction_set *rset)
 {
     int i, cols = 0;
 
-    if (rset->var) {
-	cols = gretl_VECM_n_beta(rset->var);
-    } else if (rset->sys) {
-	cols = system_n_indep_vars(rset->sys);
+    if (rset->type == GRETL_OBJ_VAR) {
+	cols = gretl_VECM_n_beta(rset->obj);
+    } else if (rset->type == GRETL_OBJ_SYS) {
+	cols = system_n_indep_vars(rset->obj);
     } else {
-	for (i=0; i<rset->pmod->ncoeff; i++) {
+	MODEL *pmod = rset->obj;
+
+	for (i=0; i<pmod->ncoeff; i++) {
 	    if (rset->mask[i]) {
 		cols++;
 	    }
@@ -147,22 +150,31 @@ static int R_n_columns (gretl_restriction_set *rset)
 }
 
 static int 
-restriction_set_form_matrices (gretl_restriction_set *rset,
-			       gretl_matrix **Rin,
-			       gretl_vector **qin)
+restriction_set_form_full_matrices (gretl_restriction_set *rset,
+				    gretl_matrix **Rin,
+				    gretl_vector **qin)
 {
+    MODEL *pmod;
     gretl_matrix *R;
     gretl_vector *q;
     restriction *r;
     double x;
-    int col, i, j;
+    int i, j, k;
 
-    R = gretl_matrix_alloc(rset->k, R_n_columns(rset));
+    if (rset->type != GRETL_OBJ_EQN ||
+	rset->obj == NULL) {
+	return 1;
+    }
 
-    if (R == NULL) return E_ALLOC;
+    pmod = rset->obj;
+    k = pmod->ncoeff;
+
+    R = gretl_matrix_alloc(rset->k, k);
+    if (R == NULL) {
+	return E_ALLOC;
+    }
 
     q = gretl_column_vector_alloc(rset->k);
-
     if (q == NULL) {
 	gretl_matrix_free(R);
 	return E_ALLOC;
@@ -173,9 +185,56 @@ restriction_set_form_matrices (gretl_restriction_set *rset,
 
     for (i=0; i<rset->k; i++) { 
 	r = rset->restrictions[i];
-	if (rset->pmod != NULL) {
+	for (j=0; j<k; j++) {
+	    if (rset->mask[j]) {
+		x = get_restriction_param(r, j);
+		gretl_matrix_set(R, i, j, x);
+	    }
+	}
+	gretl_vector_set(q, i, r->rhs);
+    }
+
+    *Rin = R;
+    *qin = q;
+
+    return 0;
+}
+
+static int 
+restriction_set_form_matrices (gretl_restriction_set *rset,
+			       gretl_matrix **Rin,
+			       gretl_vector **qin)
+{
+    MODEL *pmod = NULL;
+    gretl_matrix *R;
+    gretl_vector *q;
+    restriction *r;
+    double x;
+    int col, i, j;
+
+    R = gretl_matrix_alloc(rset->k, R_n_columns(rset));
+    if (R == NULL) {
+	return E_ALLOC;
+    }
+
+    q = gretl_column_vector_alloc(rset->k);
+    if (q == NULL) {
+	gretl_matrix_free(R);
+	return E_ALLOC;
+    }
+
+    gretl_matrix_zero(R);
+    gretl_matrix_zero(q);
+
+    if (rset->type == GRETL_OBJ_EQN) {
+	pmod = rset->obj;
+    }
+
+    for (i=0; i<rset->k; i++) { 
+	r = rset->restrictions[i];
+	if (pmod != NULL) {
 	    col = 0;
-	    for (j=0; j<rset->pmod->ncoeff; j++) {
+	    for (j=0; j<pmod->ncoeff; j++) {
 		if (rset->mask[j]) {
 		    x = get_restriction_param(r, j);
 		    gretl_matrix_set(R, i, col++, x);
@@ -196,13 +255,14 @@ restriction_set_form_matrices (gretl_restriction_set *rset,
     gretl_matrix_print(q, "q");
 #endif
 
-    if (rset->var != NULL) {
-	gretl_matrix *D = gretl_matrix_right_nullspace(R);
+    if (rset->type == GRETL_OBJ_VAR) {
+	GRETL_VAR *var = rset->obj;
 
+	gretl_matrix *D = gretl_matrix_right_nullspace(R);
 #if RDEBUG
 	gretl_matrix_print(D, "D");
 #endif
-	gretl_VAR_attach_restrictions(rset->var, D);
+	gretl_VAR_attach_restrictions(var, D);
 	gretl_matrix_free(R);
 	gretl_matrix_free(q);
     } else {
@@ -220,14 +280,17 @@ restriction_set_form_matrices (gretl_restriction_set *rset,
 
 static int restriction_set_make_mask (gretl_restriction_set *rset)
 {
+    MODEL *pmod;
     restriction *r;
     int i, j;
 
-    if (rset->pmod == NULL) {
+    if (rset->type != GRETL_OBJ_EQN ||
+	rset->obj == NULL) {
 	return 1;
     }
 
-    rset->mask = calloc(rset->pmod->ncoeff, 1);
+    pmod = rset->obj;
+    rset->mask = calloc(pmod->ncoeff, 1);
 
     if (rset->mask == NULL) {
 	destroy_restriction_set(rset);
@@ -267,13 +330,27 @@ static int parse_b_bit (const char *s, int *eq, int *bnum)
     } else if (*s == '[') {
 	if (sscanf(s, "[%d,%d]", eq, bnum) == 2) {
 	    err = 0;
-	} else if (sscanf(s, "[%d]", bnum)) {
+	} else if (sscanf(s, "[%d]", bnum) == 1) {
+	    *eq = 1;
 	    err = 0;
 	}	
     }
 
-    if (*eq > 0) {
-	*eq -= 1;
+    if (*bnum < 1) {
+	sprintf(gretl_errmsg, _("Coefficient number (%d) is out of range"), 
+		*bnum);
+	err = 1;
+    } else {
+	*bnum -= 1; /* convert to zero base */
+    }
+
+    if (*eq < 1) {
+	fprintf(stderr, "borked HERE\n");
+	sprintf(gretl_errmsg, _("Equation number (%d) is out of range"), 
+		*eq);
+	err = 1;
+    } else {
+	*eq -= 1; /* convert to zero base */
     }
 
     return err;
@@ -285,7 +362,7 @@ parse_coeff_chunk (const char *s, double *x, int *eq, int *bnum)
     const char *s0 = s;
     int err = E_PARSE;
 
-    *eq = 0;
+    *eq = 1;
 
     while (isspace((unsigned char) *s)) s++;
 
@@ -303,7 +380,7 @@ parse_coeff_chunk (const char *s, double *x, int *eq, int *bnum)
 	}
     }
 
-    if (err) {
+    if (err && *gretl_errmsg == '\0') {
 	sprintf(gretl_errmsg, _("parse error in '%s'\n"), s0);
     } 
 
@@ -341,6 +418,7 @@ static void destroy_restriction_set (gretl_restriction_set *rset)
 static restriction *restriction_new (int n, int cross)
 {
     restriction *r;
+    int i;
 
     r = malloc(sizeof *r);
     if (r == NULL) return NULL;
@@ -356,6 +434,11 @@ static restriction *restriction_new (int n, int cross)
 	return NULL;
     }
 
+    for (i=0; i<n; i++) {
+	r->mult[i] = 0.0;
+	r->coeff[i] = 0;
+    }
+
     if (cross) {
 	r->eq = malloc(n * sizeof *r->eq);
 	if (r->eq == NULL) {
@@ -377,12 +460,16 @@ augment_restriction_set (gretl_restriction_set *rset, int n_terms)
     int n = rset->k;
 
     rlist = realloc(rset->restrictions, (n + 1) * sizeof *rlist);
-    if (rlist == NULL) return NULL;
+    if (rlist == NULL) {
+	return NULL;
+    }
 
     rset->restrictions = rlist;
 
     rset->restrictions[n] = restriction_new(n_terms, rset->cross);
-    if (rset->restrictions[n] == NULL) return NULL;
+    if (rset->restrictions[n] == NULL) {
+	return NULL;
+    }
 
     rset->k += 1;
 
@@ -411,27 +498,23 @@ static void print_restriction (const gretl_restriction_set *rset,
 {
     const restriction *r = rset->restrictions[j];
     char vname[24];
-    int i;
+    int i, k;
 
     for (i=0; i<r->nterms; i++) {
+	k = r->coeff[i];
 	print_mult(r->mult[i], i == 0, prn);
 	if (rset->cross) {
-	    pprintf(prn, "b[%d,%d]", r->eq[i] + 1, r->coeff[i]);
-	} else if (rset->var) {
-	    const int *list = gretl_VECM_list(rset->var);
-	    int li = (list == NULL)? 0 : r->coeff[i] + 1;
-
-	    if (li > 0 && li <= list[0]) {
-		pprintf(prn, "b[%s]", pdinfo->varname[list[li]]);
-	    } else {
-		pprintf(prn, "b[%d]", r->coeff[i]);
-	    }
+	    pprintf(prn, "b[%d,%d]", r->eq[i] + 1, k + 1);
+	} else if (rset->type == GRETL_OBJ_VAR) {
+	    pprintf(prn, "b[%d]", k + 1);
 	} else {
-	    gretl_model_get_param_name(rset->pmod, pdinfo, 
-				       r->coeff[i], vname);
+	    MODEL *pmod = rset->obj;
+
+	    gretl_model_get_param_name(pmod, pdinfo, k, vname);
 	    pprintf(prn, "b[%s]", vname);
 	}
     }
+
     pprintf(prn, " = %g\n", r->rhs);
 }
 
@@ -456,21 +539,22 @@ print_restriction_set (const gretl_restriction_set *rset,
 	}
 	print_restriction(rset, i, pdinfo, prn);
     }
-
-#if RDEBUG
-    if (rset->pmod != NULL) {
-	pprintf(prn, "Selection mask for coefficients:");
-	for (i=0; i<rset->pmod->ncoeff; i++) {
-	    pprintf(prn, " %d", (int) rset->mask[i]);
-	}
-	pputc(prn, '\n');
-    }
-#endif
 }
 
 static int 
 add_term_to_restriction (restriction *r, double mult, int eq, int bnum, int i)
 {
+    int j;
+
+    for (j=0; j<i; j++) {
+	if (bnum == r->coeff[j] && (r->eq == NULL || eq == r->eq[j])) {
+	    /* additional reference to a previously referenced coeff */
+	    r->mult[j] += mult;
+	    r->nterms -= 1;
+	    return 0;
+	}
+    }
+
     r->mult[i] = mult;
     r->coeff[i] = bnum;
 
@@ -482,31 +566,32 @@ add_term_to_restriction (restriction *r, double mult, int eq, int bnum, int i)
 }
 
 static gretl_restriction_set *
-real_restriction_set_start (MODEL *pmod, const DATAINFO *pdinfo,
-			    gretl_equation_system *sys,
-			    GRETL_VAR *var)
+real_restriction_set_start (void *ptr, GretlObjType type,
+			    gretlopt opt)
 {
     gretl_restriction_set *rset;
 
     rset = malloc(sizeof *rset);
     if (rset == NULL) return NULL;
 
-    rset->pmod = pmod;
-    rset->pdinfo = pdinfo;
-    rset->sys = sys;
-    rset->var = var;
+    rset->obj = ptr;
+    rset->type = type;
+    rset->opt = opt;
 
     rset->k = 0;
     rset->mask = NULL;
     rset->restrictions = NULL;
+    rset->cross = 0;
 
-    if (rset->sys != NULL) {
+    if (rset->type == GRETL_OBJ_SYS) {
 	rset->cross = 1;
-    } else if (rset->var != NULL && gretl_VECM_rank(rset->var) > 1) {
-	rset->cross = 1;
-    } else {
-	rset->cross = 0;
-    }
+    } else if (rset->type == GRETL_OBJ_VAR) {
+	GRETL_VAR *var = ptr;
+
+	if (var != NULL && gretl_VECM_rank(var) > 1) {
+	    rset->cross = 1;
+	}
+    } 
 
     return rset;
 }
@@ -520,35 +605,40 @@ static int bnum_out_of_bounds (const gretl_restriction_set *rset,
 {
     int ret = 1;
 
-    if (rset->var) {
-	if (eq >= gretl_VECM_rank(rset->var)) {
+    if (rset->type == GRETL_OBJ_VAR) {
+	GRETL_VAR *var = rset->obj;
+
+	if (eq >= gretl_VECM_rank(var)) {
 	    sprintf(gretl_errmsg, _("Equation number (%d) is out of range"), 
 		    eq + 1);
-	} else if (bnum >= gretl_VECM_n_beta(rset->var)) {
+	} else if (bnum >= gretl_VECM_n_beta(var)) {
 	    sprintf(gretl_errmsg, _("Coefficient number (%d) is out of range"), 
-		    bnum);
+		    bnum + 1);
 	} else {
 	    ret = 0;
 	}
-    } else if (rset->sys) {
-	const int *list = system_get_list(rset->sys, eq);
+    } else if (rset->type == GRETL_OBJ_SYS) {
+	gretl_equation_system *sys = rset->obj;
+	const int *list = system_get_list(sys, eq);
 
 	if (list == NULL) {
 	    sprintf(gretl_errmsg, _("Equation number (%d) is out of range"), 
 		    eq + 1);
 	} else if (bnum >= list[0] - 1) {
 	    sprintf(gretl_errmsg, _("Coefficient number (%d) out of range "
-				    "for equation %d"), bnum, eq + 1);
+				    "for equation %d"), bnum + 1, eq + 1);
 	} else {
 	    ret = 0;
 	}
     } else {
+	MODEL *pmod = rset->obj;
+
 	if (eq > 0) {
 	    sprintf(gretl_errmsg, _("Equation number (%d) is out of range"), 
 		    eq + 1);
-	} else if (bnum >= rset->pmod->ncoeff) {
+	} else if (bnum >= pmod->ncoeff || bnum < 0) {
 	    sprintf(gretl_errmsg, _("Coefficient number (%d) is out of range"), 
-		    bnum);
+		    bnum + 1);
 	} else {
 	    ret = 0;
 	}
@@ -603,7 +693,7 @@ real_restriction_set_parse_line (gretl_restriction_set *rset,
 
     for (i=0; i<nt; i++) {
 	char chunk[32];
-	int len, bnum, eq;
+	int len, bnum = 1, eq = 1;
 	double mult;
 
 	len = strcspn(p, "+-=");
@@ -643,7 +733,7 @@ real_restriction_set_parse_line (gretl_restriction_set *rset,
     if (!err) {
 	if (!sscanf(p, " = %lf", &r->rhs)) {
 	    err = E_PARSE;
-	} else if (rset->var != NULL && r->rhs != 0.0) {
+	} else if (rset->type == GRETL_OBJ_VAR && r->rhs != 0.0) {
 	    strcpy(gretl_errmsg, "VECM restrictions: the equations must "
 		   "be homogeneous");
 	    err = 1;
@@ -662,12 +752,14 @@ restriction_set_parse_line (gretl_restriction_set *rset, const char *line)
 {
     int nx = 0;
 
-    if (rset->pmod != NULL) {
-	nx = rset->pmod->ncoeff;
-    } else if (rset->sys != NULL) {
-	nx = system_n_indep_vars(rset->sys);
-    } else if (rset->var != NULL) {
-	nx = gretl_VECM_n_beta(rset->var);
+    if (rset->type == GRETL_OBJ_EQN) {
+	MODEL *pmod = rset->obj;
+
+	nx = pmod->ncoeff;
+    } else if (rset->type == GRETL_OBJ_SYS) {
+	nx = system_n_indep_vars(rset->obj);
+    } else if (rset->type == GRETL_OBJ_VAR) {
+	nx = gretl_VECM_n_beta(rset->obj);
     }
 
     if (rset->k >= nx) {
@@ -680,15 +772,14 @@ restriction_set_parse_line (gretl_restriction_set *rset, const char *line)
     return real_restriction_set_parse_line(rset, line, 0);
 }
 
-/* special set-up for a set of restrictions for a VAR (vecm,
-   actually) */
+/* set-up for a set of restrictions for a VAR (vecm, actually) */
 
 gretl_restriction_set *
 var_restriction_set_start (const char *line, GRETL_VAR *var)
 {
     gretl_restriction_set *rset;
 
-    rset = real_restriction_set_start(NULL, NULL, NULL, var);
+    rset = real_restriction_set_start(var, GRETL_OBJ_VAR, OPT_NONE);
     if (rset == NULL) {
 	strcpy(gretl_errmsg, _("Out of memory!"));
 	return NULL;
@@ -706,15 +797,36 @@ var_restriction_set_start (const char *line, GRETL_VAR *var)
     return rset;
 }
 
-/* special set-up for a set of cross-equation restrictions, for
-   a system of simultaneous equations */
+/* set-up for a set of cross-equation restrictions, for a system of
+   simultaneous equations */
 
 gretl_restriction_set *
 cross_restriction_set_start (const char *line, gretl_equation_system *sys)
 {
     gretl_restriction_set *rset;
 
-    rset = real_restriction_set_start(NULL, NULL, sys, NULL);
+    rset = real_restriction_set_start(sys, GRETL_OBJ_SYS, OPT_NONE);
+    if (rset == NULL) {
+	strcpy(gretl_errmsg, _("Out of memory!"));
+	return NULL;
+    }
+
+    if (real_restriction_set_parse_line(rset, line, 1)) {
+	sprintf(gretl_errmsg, _("parse error in '%s'\n"), line);
+	return NULL;
+    }
+
+    return rset;
+}
+
+/* set-up for a set of restrictions on a single equation */
+
+gretl_restriction_set *
+eqn_restriction_set_start (const char *line, MODEL *pmod)
+{
+    gretl_restriction_set *rset;
+
+    rset = real_restriction_set_start(pmod, GRETL_OBJ_EQN, OPT_NONE);
     if (rset == NULL) {
 	strcpy(gretl_errmsg, _("Out of memory!"));
 	return NULL;
@@ -729,63 +841,247 @@ cross_restriction_set_start (const char *line, gretl_equation_system *sys)
 }
 
 gretl_restriction_set *
-restriction_set_start (const char *line, MODEL *pmod, const DATAINFO *pdinfo)
+restriction_set_start (const char *line, gretlopt opt, int *err)
 {
     gretl_restriction_set *rset = NULL;
-    char *sname = NULL;
+    char *name = NULL;
+    GretlObjType type;
+    void *ptr = NULL;
 
 #if RDEBUG
     fprintf(stderr, "restriction_set_start: line='%s'\n", line);
 #endif
 
     if (!strncmp(line, "restrict", 8)) {
-	sname = get_system_name_from_line(line);
+	name = get_system_name_from_line(line);
     }
 
-    /* are we applying a restriction to a named system of equations? */
-    if (sname != NULL) {
-	gretl_equation_system *sys = NULL;
-	GRETL_VAR *var = NULL;
+    if (name != NULL) {
+	/* get pointer to named object */
+	*err = gretl_get_object_and_type(name, &ptr, &type);
+	if (ptr == NULL) {
+	    sprintf(gretl_errmsg, "'%s': unrecognized name", name);
+	}
+    } else {
+	/* get pointer to last-created object */
+	ptr = get_last_model(&type);  
+    }
 
-	sys = get_equation_system_by_name(sname);
-	if (sys != NULL) {
-	    rset = real_restriction_set_start(NULL, NULL, sys, NULL);
-	    if (rset == NULL) {
-		strcpy(gretl_errmsg, _("Out of memory!"));
-	    }	    
-	} else {
-	    var = get_VECM_by_name(sname);
-	    if (var != NULL) {
-		rset = real_restriction_set_start(NULL, NULL, NULL, var);
-		if (rset == NULL) {
-		    strcpy(gretl_errmsg, _("Out of memory!"));
-		}
+    if (ptr == NULL) {
+	*err = E_DATA;
+	return NULL;
+    }
+
+    if (type != GRETL_OBJ_EQN && type != GRETL_OBJ_SYS &&
+	type != GRETL_OBJ_VAR) {
+	*err = E_DATA;
+	return NULL;
+    }
+
+    rset = real_restriction_set_start(ptr, type, opt);
+    if (rset == NULL) {
+	*err = E_ALLOC;
+    }
+
+    if (!*err && name == NULL) {
+	*err = real_restriction_set_parse_line(rset, line, 1);
+	if (*err) {
+	    rset = NULL;
+	    if (*err == E_PARSE) {
+		sprintf(gretl_errmsg, _("parse error in '%s'\n"), line);
 	    }
 	}
-
-	if (sys == NULL && var == NULL) {
-	    sprintf(gretl_errmsg, "'%s': unrecognized name", sname);
-	}
-
-	free(sname);
-    } else {
-	rset = real_restriction_set_start(pmod, pdinfo, NULL, NULL);
-
-	if (rset == NULL) {
-	    strcpy(gretl_errmsg, _("Out of memory!"));
-	} else if (real_restriction_set_parse_line(rset, line, 1)) {
-	    sprintf(gretl_errmsg, _("parse error in '%s'\n"), line);
-	    rset = NULL;
-	}
-    }
+    }	
 
     return rset;
 }
 
+static void print_pval_str (double pval, char *str)
+{
+    if (pval < .00001) {
+	sprintf(str, "<%.5f", 0.00001);
+    } else {
+	sprintf(str, "%.5f", pval);
+    }
+}
+
+static int print_coeff (const MODEL *pmod, int i,
+			double coeff, double sderr, int k,
+			const DATAINFO *pdinfo, 
+			PRN *prn)
+{
+    int do_pval = 1;
+    double t, pvalue = 999.0;
+    int gotnan = 0;
+    char varname[24];
+
+    gretl_model_get_param_name(pmod, pdinfo, i, varname);
+    pprintf(prn, "  %-15s ", varname);
+    
+    if (isnan(coeff) || na(coeff)) {
+	pprintf(prn, "%*s", UTF_WIDTH(_("undefined"), 17), _("undefined"));
+	gotnan = 1;
+    } else {
+	gretl_print_value(coeff, prn);
+    }
+
+    if (isnan(sderr) || na(sderr)) {
+	pprintf(prn, "%*s\n", UTF_WIDTH(_("undefined"), 16), _("undefined"));
+	return 1;
+    }
+
+    gretl_print_value(sderr, prn); 
+
+    if (sderr > 0.0) {
+	t = coeff / sderr;
+	if (fabs(t) >= 1000.0) {
+	    char numstr[9];
+
+	    sprintf(numstr, "%#8.2G", t);
+	    pprintf(prn, " %8s", numstr);
+	} else {
+	    pprintf(prn, " %7.3f", t);
+	}
+
+	if (do_pval) {
+	    char pvalstr[16];
+	    int dfd = pmod->dfd + k;
+
+	    pvalue = coeff_pval(pmod, t, dfd);
+	    print_pval_str(pvalue, pvalstr);
+	    pprintf(prn, "%*s", UTF_WIDTH(pvalstr, 10), pvalstr);
+	}
+    } else if (do_pval) { 
+	do_pval = 0;
+	pprintf(prn, "     %*s", UTF_WIDTH(_("undefined"), 10), _("undefined"));
+    }
+
+    if (do_pval) {
+	if (pvalue < 0.01) {
+	    pputs(prn, " ***");
+	} else if (pvalue < 0.05) {
+	    pputs(prn, " **");
+	} else if (pvalue < 0.10) {
+	    pputs(prn, " *");
+	}
+    } 
+
+    pputc(prn, '\n');
+
+    return gotnan;
+}
+
+static void coeff_header (const MODEL *pmod, PRN *prn)
+{
+    int use_param = pmod->ci == NLS || pmod->ci == MLE;
+
+    if (use_param) {
+	pputs(prn, _("      PARAMETER       ESTIMATE          STDERROR"
+		     "      T STAT   P-VALUE\n\n"));
+    } else {
+	pputs(prn, _("      VARIABLE       COEFFICIENT        STDERROR"
+		     "      T STAT   P-VALUE\n\n"));
+    }
+}
+
+static int 
+do_restricted_estimates (gretl_restriction_set *rset,
+			 const double **Z, const DATAINFO *pdinfo,
+			 PRN *prn)
+{
+    MODEL *pmod = rset->obj;
+    gretl_matrix *X = NULL;
+    gretl_matrix *y = NULL;
+    gretl_matrix *R = NULL;
+    gretl_matrix *q = NULL;
+    gretl_matrix *b = NULL;
+    gretl_matrix *S = NULL;
+    int *xlist = NULL;
+    double s2 = 0.0;
+    int T = pmod->nobs;
+    int k = pmod->ncoeff;
+    int i, s, t;
+    int yno, err = 0;
+
+    X = gretl_matrix_alloc(T, k);
+    y = gretl_matrix_alloc(T, 1);
+    b = gretl_matrix_alloc(k, 1);
+    S = gretl_matrix_alloc(k, k);
+
+    if (X == NULL || y == NULL || b == NULL || S == NULL) {
+	err = E_ALLOC;
+	goto bailout;
+    }
+
+    err = restriction_set_form_full_matrices(rset, &R, &q);
+    if (err) {
+	goto bailout;
+    }
+
+    yno = gretl_model_get_depvar(pmod);
+    xlist = gretl_model_get_x_list(pmod);
+    if (xlist == NULL) {
+	err = E_ALLOC;
+	goto bailout;
+    }
+
+    s = 0;
+    for (t=pmod->t1; t<=pmod->t2; t++) {
+	if (na(pmod->uhat[t])) {
+	    continue;
+	}
+	gretl_vector_set(y, s, Z[yno][t]);
+	for (i=0; i<k; i++) {
+	    gretl_matrix_set(X, s, i, Z[xlist[i+1]][t]);
+	}
+	s++;
+    }
+
+#if RDEBUG
+    gretl_matrix_print(R, "R");
+    gretl_matrix_print(q, "q");
+#endif
+
+    err = gretl_matrix_restricted_ols(y, X, R, q, b, S, &s2);
+
+    if (!err) {
+	double v, coeff, se;
+
+	pprintf(prn, "%s:\n\n", _("Restricted estimates"));
+	coeff_header(pmod, prn);
+	for (i=0; i<k; i++) {
+	    coeff = gretl_vector_get(b, i);
+	    v = gretl_matrix_get(S, i, i);
+	    se = (v > 1.0e-16)? sqrt(v) : 0.0;
+	    print_coeff(pmod, i, coeff, se, rset->k, pdinfo, prn);
+	}
+	pputc(prn, '\n');
+	pprintf(prn, "  %s = %.*g\n", _("Standard error of residuals"), 
+		GRETL_DIGITS, sqrt(s2));
+    } 
+
+ bailout:
+    
+    gretl_matrix_free(X);
+    gretl_matrix_free(y);
+    gretl_matrix_free(R);
+    gretl_matrix_free(q);
+    gretl_matrix_free(b);
+    gretl_matrix_free(S);
+
+    free(xlist);
+
+    return err;
+}
+
 /* execute the test, for a single equation */
 
-static int test_restriction_set (gretl_restriction_set *rset, PRN *prn)
+static int test_restriction_set (gretl_restriction_set *rset, 
+				 const double **Z,
+				 const DATAINFO *pdinfo,
+				 PRN *prn)
 {
+    MODEL *pmod = rset->obj;
     gretl_matrix *R;
     gretl_vector *q;
     gretl_matrix *vcv = NULL;
@@ -795,7 +1091,7 @@ static int test_restriction_set (gretl_restriction_set *rset, PRN *prn)
     double test_stat, pval;
     int err, robust, freeRv = 1;
 
-    int asym = ASYMPTOTIC_MODEL(rset->pmod->ci);
+    int asym = ASYMPTOTIC_MODEL(pmod->ci);
 
     *gretl_errmsg = '\0';
 
@@ -819,8 +1115,8 @@ static int test_restriction_set (gretl_restriction_set *rset, PRN *prn)
 	goto bailout;
     }
 
-    b = gretl_coeff_vector_from_model(rset->pmod, rset->mask);
-    vcv = gretl_vcv_matrix_from_model(rset->pmod, rset->mask);
+    b = gretl_coeff_vector_from_model(pmod, rset->mask);
+    vcv = gretl_vcv_matrix_from_model(pmod, rset->mask);
     if (b == NULL || vcv == NULL) {
 	err = E_ALLOC;
 	goto bailout;
@@ -847,7 +1143,7 @@ static int test_restriction_set (gretl_restriction_set *rset, PRN *prn)
     gretl_matrix_print(br, "br");
 #endif  
 
-    if (!gretl_is_zero_vector(q)) {
+    if (!gretl_is_zero_matrix(q)) {
 	err = gretl_matrix_subtract_from(br, q);
 	if (err) {
 	    fprintf(stderr, "Failed: gretl_matrix_subtract_from(br, q)\n");
@@ -882,19 +1178,19 @@ static int test_restriction_set (gretl_restriction_set *rset, PRN *prn)
 	goto bailout;
     }
 
-    robust = gretl_model_get_int(rset->pmod, "robust");
+    robust = gretl_model_get_int(pmod, "robust");
 
     if (asym) {
-	pval = chisq(test_stat, rset->k);
+	pval = chisq_cdf_comp(test_stat, rset->k);
 	pprintf(prn, "\n%s: %s(%d) = %g, ", _("Test statistic"), 
 		(robust)? _("Robust chi^2"): "chi^2",
 		rset->k, test_stat);
     } else {
 	test_stat /= rset->k;
-	pval = fdist(test_stat, rset->k, rset->pmod->dfd);
+	pval = f_cdf_comp(test_stat, rset->k, pmod->dfd);
 	pprintf(prn, "\n%s: %s(%d, %d) = %g, ", _("Test statistic"), 
 		(robust)? _("Robust F"): "F",
-		rset->k, rset->pmod->dfd, test_stat);
+		rset->k, pmod->dfd, test_stat);
     }
 
     pprintf(prn, _("with p-value = %g\n"), pval);
@@ -902,6 +1198,10 @@ static int test_restriction_set (gretl_restriction_set *rset, PRN *prn)
 
     record_test_result(test_stat, pval, _("restriction"));
 
+    if (pmod != NULL && !(rset->opt & OPT_Q) && pmod->ci == OLS) {
+	do_restricted_estimates(rset, Z, pdinfo, prn);
+    }
+
  bailout:
 
     gretl_matrix_free(R);
@@ -910,7 +1210,9 @@ static int test_restriction_set (gretl_restriction_set *rset, PRN *prn)
     gretl_vector_free(b);
     gretl_vector_free(br);
     
-    if (freeRv) gretl_matrix_free(Rv);
+    if (freeRv) {
+	gretl_matrix_free(Rv);
+    }
 
     return err;
 }
@@ -923,6 +1225,7 @@ static int test_restriction_set (gretl_restriction_set *rset, PRN *prn)
 
 int
 gretl_restriction_set_finalize (gretl_restriction_set *rset, 
+				const double **Z,
 				const DATAINFO *pdinfo,
 				PRN *prn)
 {
@@ -932,14 +1235,14 @@ gretl_restriction_set_finalize (gretl_restriction_set *rset,
 	return 1;
     }
 
-    if (rset->var != NULL) {
+    if (rset->type == GRETL_OBJ_VAR) {
 	/* vecm */
 	err = restriction_set_form_matrices(rset, NULL, NULL);
 	if (!err) {
 	    print_restriction_set(rset, pdinfo, prn);
-	    gretl_VECM_test_beta(rset->var, prn);
+	    gretl_VECM_test_beta(rset->obj, prn);
 	}
-    } else if (rset->sys != NULL) {
+    } else if (rset->type == GRETL_OBJ_SYS) {
 	/* simultaneous equations system */
 	gretl_matrix *R;
 	gretl_matrix *q;
@@ -958,7 +1261,7 @@ gretl_restriction_set_finalize (gretl_restriction_set *rset,
 	    }
 	}	
 	if (!err) {
-	    system_set_restriction_matrices(rset->sys, R, q);
+	    system_set_restriction_matrices(rset->obj, R, q);
 	    destroy_restriction_set(rset);
 	}
     } else {
@@ -966,7 +1269,7 @@ gretl_restriction_set_finalize (gretl_restriction_set *rset,
 	err = restriction_set_make_mask(rset);
 	if (!err) {
 	    print_restriction_set(rset, pdinfo, prn);
-	    test_restriction_set(rset, prn);
+	    test_restriction_set(rset, Z, pdinfo, prn);
 	    destroy_restriction_set(rset);
 	}
     }	
