@@ -23,7 +23,7 @@
 typedef struct restriction_set_ gretl_restriction_set;
 
 gretl_restriction_set *
-restriction_set_start (const char *line, MODEL *pmod, const DATAINFO *pdinfo);
+restriction_set_start (const char *line, gretlopt opt, int *err);
 
 gretl_restriction_set *
 cross_restriction_set_start (const char *line, gretl_equation_system *sys);
@@ -31,12 +31,15 @@ cross_restriction_set_start (const char *line, gretl_equation_system *sys);
 gretl_restriction_set *
 var_restriction_set_start (const char *line, GRETL_VAR *var);
 
+gretl_restriction_set *
+eqn_restriction_set_start (const char *line, MODEL *pmod);
+
 int 
 restriction_set_parse_line (gretl_restriction_set *rset, const char *line);
 
 int
 gretl_restriction_set_finalize (gretl_restriction_set *rset, 
-				const DATAINFO *pdinfo,
+				const double **Z, const DATAINFO *pdinfo,
 				PRN *prn);
 
 #endif
