@@ -23,12 +23,13 @@
 #include "cmd_private.h"
 #include "libset.h"
 #include "usermat.h"
+#include "gretl_panel.h"
 
 #include <errno.h>
 
 #ifndef WIN32
 # include <signal.h>
-# ifdef USE_GTK2
+# ifdef USE_GLIB2
 #  include <glib.h>
 #  define USE_GSPAWN
 # endif
@@ -117,7 +118,7 @@ int ztox (int i, double *px, const double **Z, const DATAINFO *pdinfo)
     int t, m = 0;
     double xx;
 
-    if (!pdinfo->vector[i]) {
+    if (var_is_scalar(pdinfo, i)) {
 	px[0] = Z[i][0];
 	return 1;
     }
@@ -267,8 +268,9 @@ int gretl_isunits (int t1, int t2, const double *x)
  * @t2: ending observation. 
  *
  * Returns: 1 if @x contains only values whose decimal part is 
- * zero in the given sample range and 0 otherwise (missing 
- * values are skipped).
+ * zero in the given sample range and 0 otherwise.  Missing 
+ * values are skipped, but numbers that cannot be represented
+ * as integers as rejected as "not discrete".
  */
 
 int gretl_isdiscrete (int t1, int t2, const double *x)
@@ -276,7 +278,10 @@ int gretl_isdiscrete (int t1, int t2, const double *x)
     int t, ret = 1;
 
     for (t=t1; t<=t2; t++) {
-	if (!na(x[t]) && x[t] != floor(x[t])) {
+	if (na(x[t])) {
+	    continue;
+	}
+	if (!ok_int(x[t]) || x[t] != floor(x[t])) {
 	    ret = 0;
 	    break;
 	}
@@ -344,6 +349,77 @@ int gretl_inverse_compare_doubles (const void *a, const void *b)
     return (*da < *db) - (*da > *db);
 }
 
+/**
+ * gretl_compare_ints:
+ * @a: pointer to first element to compare.
+ * @b: pointer to second element to compare.
+ *
+ * Comparison function for use with %qsort.  Sorts integers in
+ * ascending order.
+ * 
+ * Returns: appropriate value for %qsort.
+ */
+
+int gretl_compare_ints (const void *a, const void *b)
+{
+    const int *ia = (const int *) a;
+    const int *ib = (const int *) b;
+     
+    return *ia - *ib;
+}
+
+/**
+ * count_distinct_values:
+ * @x: sorted array of doubles.
+ * @n: number of elements in array.
+ *
+ * Returns: the number of distinct values in array @x,
+ * provided that @x is already sorted.
+ */
+
+int count_distinct_values (const double *x, int n)
+{
+    int i, c = 1;
+
+    for (i=1; i<n; i++) {
+	if (x[i] != x[i-1]) {
+	    c++;
+	}
+    }
+
+    return c;
+}
+
+/**
+ * rearrange_id_array:
+ * @x: sorted array of doubles.
+ * @m: number of distinct values in array.
+ * @n: number of elements in array.
+ *
+ * Rearranges the sorted array @x such that the first @m 
+ * elements contain the @m distinct values in sorted order.
+ *
+ * Returns: 0 on success, 1 on error (in case @m is greater
+ * than @n).
+ */
+
+int rearrange_id_array (double *x, int m, int n)
+{
+    int i, k = 1;
+
+    if (m >= n || m == 1) {
+	return 1;
+    }
+
+    for (i=1; i<n && k<m; i++) {
+	if (x[i] != x[i-1]) {
+	    x[k++] = x[i];
+	}
+    }
+
+    return 0;
+}
+
 /**
  * printlist:
  * @list: array of integers.
@@ -387,7 +463,7 @@ int gretl_calculate_criteria (double ess, int n, int k,
     if (na(ess) || ess <= 0.0 || k < 1 || n <= k) {
 	err = 1;
     } else {
-	const double ln2pi1 = 2.837877066409345;
+	const double ln2pi1 = 2.837877066409345; /* log(2*pi) + 1 */
 
 	errno = 0;
 
@@ -423,8 +499,6 @@ int ls_criteria (MODEL *pmod)
     double ll, aic, bic, hqc;
     int err;
 
-    /* should this be ncoeff + 1 below? (1 for variance) */
-
     err = gretl_calculate_criteria(pmod->ess, pmod->nobs, pmod->ncoeff,
 				   &ll, &aic, &bic, &hqc);
 
@@ -454,7 +528,8 @@ int gretl_print_criteria (double ess, int nobs, int ncoeff, PRN *prn)
     return err;
 }
 
-char *real_format_obs (char *obs, int maj, int min, int pd, char sep)
+static char *
+real_format_obs (char *obs, int maj, int min, int pd, char sep)
 {
     if (pd >= 10) {
 	int pdp = pd / 10, minlen = 2;
@@ -564,23 +639,44 @@ catch_setobs_errors (const char *stobs, int pd, int n, int min, gretlopt opt)
 /**
  * set_obs:
  * @line: command line.
+ * @pZ: pointer to data array.
  * @pdinfo: data information struct.
- * @opt: OPT_S for stacked time-series, OPT_C for stacked cross-section,
- * OPT_T for time series, OPT_X for cross section.
+ * @opt: %OPT_S for stacked time-series, %OPT_C for stacked cross-section,
+ * %OPT_T for time series, %OPT_X for cross section, %OPT_P to set
+ * panel structure via two variables representing unit and period
+ * respectively.
  * 
  * Set the frequency and initial observation for a dataset.
  *
  * Returns: 0 on successful completion, 1 on error.
  */
 
-int set_obs (const char *line, DATAINFO *pdinfo, gretlopt opt)
+int set_obs (const char *line, double ***pZ, DATAINFO *pdinfo, 
+	     gretlopt opt)
 {
     char stobs[OBSLEN];
     int structure = STRUCTURE_UNKNOWN;
     int pd, dated = 0;
+    int err = 0;
 
     *gretl_errmsg = '\0';
 
+    if (opt & OPT_R) {
+	/* restructure panel: "hidden" option */
+	return switch_panel_orientation(*pZ, pdinfo);
+    }    
+
+    if (!strcmp(line, "setobs")) {
+	/* we'll just print current obs info */
+	return 0;
+    }
+
+    if (opt & OPT_P) {
+	return set_panel_structure_from_line(line, *pZ, pdinfo);
+    }
+
+    /* now we get down to business */
+
     if (sscanf(line, "%*s %d %10s", &pd, stobs) != 2) {
 	strcpy(gretl_errmsg, _("Failed to parse line as frequency, startobs"));
 	return 1;
@@ -681,10 +777,29 @@ int set_obs (const char *line, DATAINFO *pdinfo, gretlopt opt)
     pdinfo->pd = pd;
     pdinfo->structure = structure;
 
+    if (pdinfo->structure != STACKED_TIME_SERIES &&
+	pdinfo->structure != STACKED_CROSS_SECTION &&
+	pdinfo->paninfo != NULL) {
+	/* This could be a problem in some cases, like
+	   if the dataset is subsampled? */
+	dataset_destroy_panel_info(pdinfo);
+    }
+
     ntodate_full(pdinfo->stobs, 0, pdinfo); 
     ntodate_full(pdinfo->endobs, pdinfo->n - 1, pdinfo);
 
-    return 0;
+    /* pre-process stacked cross-sectional panels: put into canonical
+       stacked time series form
+    */
+    if (pdinfo->structure == STACKED_CROSS_SECTION) {
+	err = switch_panel_orientation(*pZ, pdinfo);
+    }	
+
+    if (!err && pdinfo->structure == STACKED_TIME_SERIES) {
+	err = dataset_add_default_panel_indices(pdinfo);
+    }
+
+    return err;
 }
 
 /**
@@ -708,8 +823,21 @@ int gretl_int_from_string (const char *s, const double **Z,
     char *test;
     int n = 0;
 
+    if (s == NULL || *s == 0) {
+	*err = E_DATA;
+	return 0;
+    }
+
+    errno = 0;
+
     n = strtol(s, &test, 10);
 
+    if (errno == ERANGE) {
+	*err = E_DATA;
+	errno = 0;
+	return 0;
+    }
+
     if (*test == '\0') {
 	return n;
     } else if (Z == NULL || pdinfo == NULL) {
@@ -718,9 +846,11 @@ int gretl_int_from_string (const char *s, const double **Z,
 	int v = varindex(pdinfo, s);
 	double x;
 
-	if (v == pdinfo->v) {
+	if (v == INDEXNUM) {
+	    n = loop_scalar_read(*s);
+	} else if (v >= pdinfo->v) {
 	    *err = E_UNKVAR;
-	} else if (pdinfo->vector[v]) {
+	} else if (var_is_series(pdinfo, v)) {
 	    *err = E_TYPES;
 	} else {
 	    x = Z[v][0];
@@ -735,6 +865,16 @@ int gretl_int_from_string (const char *s, const double **Z,
     return n;    
 }
 
+/**
+ * positive_int_from_string:
+ * @s: string to examine.
+ * 
+ * If @s is a valid string representation of a positive integer,
+ * return that integer, otherwise return -1.
+ *
+ * Returns: integer value.
+ */
+
 int positive_int_from_string (const char *s)
 {
     int ret = -1;
@@ -753,6 +893,14 @@ int positive_int_from_string (const char *s)
     return ret;
 }
 
+/**
+ * varnum_from_string:
+ * @str: string representation of an integer ID number.
+ * @pdinfo: dataset information.
+ * 
+ * Returns: integer ID number, or -1 on failure.
+ */
+
 int varnum_from_string (const char *str, DATAINFO *pdinfo)
 {
     int varno = positive_int_from_string(str);
@@ -764,12 +912,22 @@ int varnum_from_string (const char *str, DATAINFO *pdinfo)
     return varno;
 }
 
-int rename_var_by_id (const char *str, const char *vname, 
+/**
+ * rename_var_by_id:
+ * @idstr: string representation of the ID number of the
+ * variable to be renamed.
+ * @vname: new name to give the variable.
+ * @pdinfo: dataset information.
+ * 
+ * Returns: 0 on sucess, %E_DATA on error.
+ */
+
+int rename_var_by_id (const char *idstr, const char *vname, 
 		      DATAINFO *pdinfo)
 {
-    int varno = varnum_from_string(str, pdinfo);
+    int v = varnum_from_string(idstr, pdinfo);
 
-    if (varno < 0) {
+    if (v < 0) {
 	return E_DATA;
     }
 
@@ -777,12 +935,19 @@ int rename_var_by_id (const char *str, const char *vname,
        non-duplication (see interact.c under RENAME)
     */
 
-    strcpy(pdinfo->varname[varno], vname);
+    strcpy(pdinfo->varname[v], vname);
 
     return 0;
 }
 
-/* ........................................................... */
+/**
+ * copyvec:
+ * @src: array of doubles.
+ * @n: number of elements to copy.
+ * 
+ * Returns: an allocated copy of the first @n elements of
+ * array @src, or %NULL on failure.
+ */
 
 double *copyvec (const double *src, int n)
 {
@@ -805,6 +970,43 @@ double *copyvec (const double *src, int n)
     return targ;
 }
 
+/**
+ * doubles_array_new:
+ * @m: number of sub-arrays.
+ * @n: length of each sub-array.
+ *
+ * Allocates a 2-dimensional array of doubles, that is,
+ * @m arrays each containing @n elements.
+ * 
+ * Returns: the allocated array, or %NULL on failure.
+ */
+
+double **doubles_array_new (int m, int n)
+{
+    double **X;
+    int i, j;
+
+    X = malloc(m * sizeof *X);
+
+    if (X == NULL) {
+	return X;
+    }
+
+    for (i=0; i<m; i++) {
+	X[i] = malloc(n * sizeof **X);
+	if (X[i] == NULL) {
+	    for (j=0; j<i; j++) {
+		free(X[j]);
+	    }
+	    free(X);
+	    X = NULL;
+	    break;
+	}
+    }
+
+    return X;
+}
+
 int re_estimate (char *model_spec, MODEL *tmpmod, 
 		 double ***pZ, DATAINFO *pdinfo) 
 {
@@ -853,7 +1055,6 @@ int re_estimate (char *model_spec, MODEL *tmpmod,
     case OLS:
     case WLS:
     case HCCM:
-    case POOLED:
 	*tmpmod = lsq(cmd.list, pZ, pdinfo, cmd.ci, cmd.opt);
 	break;
     case TSLS:
@@ -872,210 +1073,24 @@ int re_estimate (char *model_spec, MODEL *tmpmod,
     return err;
 }
 
-int guess_panel_structure (double **Z, DATAINFO *pdinfo)
-{
-    int v, panel;
-
-    v = varindex(pdinfo, "year");
-
-    if (v == pdinfo->v) {
-	v = varindex(pdinfo, "Year");
-    }
-
-    if (v == pdinfo->v) {
-	panel = 0; /* can't guess */
-    } else {
-	if (floateq(Z[v][0], Z[v][1])) { /* "year" is same for first two obs */
-	    pdinfo->structure = STACKED_CROSS_SECTION; 
-	    panel = STACKED_CROSS_SECTION;
-	} else {
-	    pdinfo->structure = STACKED_TIME_SERIES; 
-	    panel = STACKED_TIME_SERIES;
-	}
-    }
-
-    return panel;
-}
-
-/* 
-   nunits = number of cross-sectional units
-   T = number pf time-periods per cross-sectional unit
-*/
-
-int get_panel_structure (const DATAINFO *pdinfo, int *nunits, int *T)
-{
-    int err = 0;
-
-    if (pdinfo->structure == STACKED_TIME_SERIES) {
-        *nunits = pdinfo->n / pdinfo->pd;
-        *T = pdinfo->pd;
-    } else if (pdinfo->structure == STACKED_CROSS_SECTION) {
-	*nunits = pdinfo->pd;
-	*T = pdinfo->n / pdinfo->pd;
-    } else {
-	err = 1;
-    }
-
-    return err;
-}
-
-int set_panel_structure (gretlopt opt, DATAINFO *pdinfo, PRN *prn)
-{
-    int nunits, T;
-    int old_ts = pdinfo->structure;
-
-    if (pdinfo->pd == 1) {
-	pputs(prn, _("The current data frequency, 1, is not "
-		"compatible with panel data.\nPlease see the 'setobs' "
-		"command.\n"));
-	return 1;
-    }
-
-    if (opt == OPT_C) {
-	pdinfo->structure = STACKED_CROSS_SECTION;
-    } else {
-	pdinfo->structure = STACKED_TIME_SERIES;
-    }
-
-    if (get_panel_structure(pdinfo, &nunits, &T)) {
-	pputs(prn, _("Failed to set panel structure\n"));
-	pdinfo->structure = old_ts;
-	return 1;
-    } else {
-	pprintf(prn, _("Panel structure set to %s\n"),
-		(pdinfo->structure == STACKED_CROSS_SECTION)? 
-		_("stacked cross sections") : _("stacked time series"));
-	pprintf(prn, _("(%d units observed in each of %d periods)\n"),
-		nunits, T);
-    }
-
-    return 0;
-}
-
-int balanced_panel (const DATAINFO *pdinfo)
-{
-    char unit[OBSLEN], period[OBSLEN];
-
-    if ((pdinfo->t2 - pdinfo->t1 + 1) % pdinfo->pd) {
-        return 0;
-    }
-
-    if (sscanf(pdinfo->endobs, "%[^:]:%s", unit, period) == 2) {
-        if (atoi(period) != pdinfo->pd) return 0;
-    } else {
-        return 0;
-    }
-
-    return 1;
-}
-
 double get_xvalue (int i, const double **Z, const DATAINFO *pdinfo)
 {
-    if (pdinfo->vector[i]) {
+    if (var_is_series(pdinfo, i)) {
 	return Z[i][pdinfo->t1];
     } else {
 	return Z[i][0];
     }	
 }
 
-static void free_mp_varnames (mp_results *mpvals)
-{
-    int i, n = mpvals->ncoeff + 1;
-
-    if (mpvals->varnames != NULL) {
-	for (i=0; i<n; i++) {
-	    free(mpvals->varnames[i]);
-	}
-	free(mpvals->varnames);
-    }
-}
-
-int allocate_mp_varnames (mp_results *mpvals)
-{
-    int i, j, n = mpvals->ncoeff + 1;
-
-    mpvals->varnames = malloc(n * sizeof *mpvals->varnames);
-
-    if (mpvals->varnames == NULL) {
-	return 1;
-    }
-
-    for (i=0; i<n; i++) {
-	mpvals->varnames[i] = malloc(VNAMELEN + 8);
-	if (mpvals->varnames[i] == NULL) {
-	    for (j=0; j<i; j++) {
-		free(mpvals->varnames[j]);
-	    }
-	    free(mpvals->varnames);
-	    return 1;
-	}
-	mpvals->varnames[i][0] = 0;
-    }
-
-    return 0;
-}
-
-void free_gretl_mp_results (mp_results *mpvals)
-{
-    if (mpvals != NULL) {
-	free(mpvals->coeff);
-	free(mpvals->sderr);
-	if (mpvals->varnames != NULL) {
-	    free_mp_varnames(mpvals);
-	}
-	if (mpvals->varlist != NULL) {
-	    free(mpvals->varlist);
-	}
-	free(mpvals);
-    }
-}
-
-mp_results *gretl_mp_results_new (int nc)
-{
-    mp_results *mpvals;
-    int i;
-
-    mpvals = malloc(sizeof *mpvals);
-    if (mpvals == NULL) {
-	return NULL;
-    }
-
-    mpvals->ncoeff = nc;
-
-    mpvals->coeff = malloc(nc * sizeof *mpvals->coeff);
-    mpvals->sderr = malloc(nc * sizeof *mpvals->sderr);
-    mpvals->varnames = NULL;
-    mpvals->varlist = NULL;
-
-    if (mpvals->coeff == NULL || 
-	mpvals->sderr == NULL) {
-	free_gretl_mp_results(mpvals);
-	return NULL;
-    }
-
-    for (i=0; i<nc; i++) {
-	mpvals->coeff[i] = NADBL;
-	mpvals->sderr[i] = NADBL;
-    }
-
-    mpvals->sigma = mpvals->ess = NADBL;
-    mpvals->rsq = mpvals->fstt = NADBL;
-    mpvals->adjrsq = NADBL;
-
-    mpvals->t1 = mpvals->t2 = mpvals->ifc = 0;
-    mpvals->dfn = mpvals->dfd = 0;
-
-    return mpvals;
-}
-
 #ifndef WIN32
 # ifdef USE_GSPAWN
 
 static int font_not_found (const char *s)
 {
-    /* "Could not find/open font when opening font X, using default" */
+    /* "Could not find/open font when opening font X, using default" 
+       or "gnuplot_x11: Some character sets not available" */
 
-    if (strstr(s, "using default")) {
+    if (strstr(s, "using default") || strstr(s, "character sets not available")) {
 	return 1;
     } else {
 	return 0;
@@ -1105,9 +1120,9 @@ int gretl_spawn (char *cmdline)
 	g_error_free(error);
 	ret = 1;
     } else if (errout && *errout) {
-	strcpy(gretl_errmsg, errout);
 	fprintf(stderr, "stderr: '%s'\n", errout);
 	if (!font_not_found(errout)) {
+	    strcpy(gretl_errmsg, errout);
 	    ret = 1;
 	}
     } else if (status != 0) {
@@ -1209,6 +1224,8 @@ void libgretl_cleanup (void)
     gretl_lists_cleanup();
     gretl_command_hash_cleanup();
     destroy_user_matrices();
+    lapack_mem_free();
+    gretl_plotx(NULL);
 
     p = strstr(gretl_plotfile(), "gpttmp");
     if (p != NULL) {
