@@ -30,13 +30,14 @@ typedef enum {
 } model_selection_criteria;
 
 #include <float.h>
+#include <limits.h>
 
 #define floateq(x, y)  (fabs((x) - (y)) < DBL_EPSILON)
 #define floatneq(x, y) (fabs((x) - (y)) > DBL_EPSILON)
 #define floatgt(x, y)  ((x) - (y) > DBL_EPSILON)
 #define floatlt(x, y)  ((y) - (x) > DBL_EPSILON)
 
-/* functions follow */
+#define ok_int(x) (x <= (double) INT_MAX && x >= (double) INT_MIN)
 
 void libgretl_init (void);
 
@@ -62,9 +63,22 @@ int true_const (int v, const double **Z, const DATAINFO *pdinfo);
 
 char *format_obs (char *obs, int maj, int min, int pd);
 
-int set_obs (const char *line, DATAINFO *pdinfo, gretlopt opt);
+int set_obs (const char *line, double ***pZ, DATAINFO *pdinfo, 
+	     gretlopt opt);
 
-/* other */
+/* sorting and comparison */
+
+int gretl_compare_doubles (const void *a, const void *b);
+
+int gretl_inverse_compare_doubles (const void *a, const void *b);
+
+int count_distinct_values (const double *x, int n);
+
+int rearrange_id_array (double *x, int m, int n);
+
+int gretl_compare_ints (const void *a, const void *b);
+
+/* miscellaneous */
 
 void printlist (const int *list, const char *msg);
 
@@ -75,7 +89,7 @@ int positive_int_from_string (const char *s);
 
 int varnum_from_string (const char *str, DATAINFO *pdinfo);
 
-int rename_var_by_id (const char *str, const char *vname, 
+int rename_var_by_id (const char *idstr, const char *vname, 
 		      DATAINFO *pdinfo);
 
 int re_estimate (char *model_spec, MODEL *tmpmod, 
@@ -83,16 +97,14 @@ int re_estimate (char *model_spec, MODEL *tmpmod,
 
 double *copyvec (const double *src, int n);
 
+double **doubles_array_new (int m, int n);
+
 int ijton (int i, int j, int nrows);
 
 int ztox (int i, double *px, const double **Z, const DATAINFO *pdinfo);
 
 double get_xvalue (int i, const double **Z, const DATAINFO *pdinfo);
 
-int gretl_compare_doubles (const void *a, const void *b);
-
-int gretl_inverse_compare_doubles (const void *a, const void *b);
-
 int gretl_copy_file (const char *src, const char *dest);
 
 #ifndef WIN32
@@ -109,22 +121,6 @@ int gretl_print_criteria (double ess, int nobs, int ncoeff, PRN *prn);
 
 int ls_criteria (MODEL *pmod);
 
-/* panel data utilities */
-
-int get_panel_structure (const DATAINFO *pdinfo, int *nunits, int *T);
-
-int set_panel_structure (gretlopt opt, DATAINFO *pdinfo, PRN *prn); 
-
-int balanced_panel (const DATAINFO *pdinfo);
-
-/* multiple-precision utilities */
-
-void free_gretl_mp_results (mp_results *mpvals);
-
-mp_results *gretl_mp_results_new (int totvar);
-
-int allocate_mp_varnames (mp_results *mpvals);
-
 /* hypothesis tests mechanism */
 
 void record_test_result (double teststat, double pval, char *blurb);
