@@ -94,7 +94,7 @@ static int filter_comments (char *line, CMD *cmd)
     }
     
     for (i=0; i<n; i++) {
-	if (line[i] == '(' && line [i+1] == '*') {
+	if (!strncmp(line + i, "(*", 2) || !strncmp(line + i, "/*", 2)) {
 	    cmd->ignore = 1;
 	    if (line[i+3] == '!') { /* special code for data file to open */
 		sscanf(line + 4, "%s", datfile);
@@ -102,7 +102,7 @@ static int filter_comments (char *line, CMD *cmd)
 		cmd->ignore = 0;  /* FIXME ? */
 		return 0;
 	    }
-	} else if (line[i] == '*' && line [i+1] == ')') {
+	} else if (!strncmp(line + i, "*)", 2) || !strncmp(line + i, "*/", 2)) {
 	    cmd->ignore = 0; 
 	    i += 2;
 	    while (isspace((unsigned char) line[i]) && i < n) {
@@ -119,6 +119,10 @@ static int filter_comments (char *line, CMD *cmd)
     tmpstr[j] = '\0';
     strcpy(line, tmpstr);
 
+#if CMD_DEBUG
+    fprintf(stderr, "filter_comments: cmd->ignore = %d\n", cmd->ignore);
+#endif
+
     return (*line == '\0');
 }
 
@@ -159,6 +163,10 @@ static int catch_command_alias (char *line, CMD *cmd)
 	cmd->ci = BXPLOT;
     } else if (!strcmp(s, "man")) {
 	cmd->ci = HELP;
+    } else if (!strcmp(s, "pooled")) {
+	cmd->ci = OLS;
+    } else if (!strcmp(s, "label")) {
+	cmd->ci = SETINFO;
     } else if (!strcmp(line, "smpl full")) {
 	strcpy(line, "smpl");
 	cmd->opt = OPT_F;
@@ -223,7 +231,6 @@ static int catch_command_alias (char *line, CMD *cmd)
 	               c == IMPORT || \
                        c == INCLUDE || \
     	               c == INFO || \
- 	               c == LABEL || \
  	               c == LABELS || \
                        c == LEVERAGE || \
                        c == LMTEST || \
@@ -235,15 +242,16 @@ static int catch_command_alias (char *line, CMD *cmd)
                        c == NULLDATA || \
  	               c == OPEN || \
                        c == OUTFILE || \
-	               c == PANEL || \
                        c == PRINTF || \
 	               c == PVALUE || \
+                       c == QLRTEST || \
 	               c == QUIT || \
                        c == RENAME || \
                        c == RESET || \
                        c == RESTRICT || \
 	               c == RUN || \
                        c == SET || \
+                       c == SETINFO || \
 	               c == SETOBS || \
 	               c == SHELL || \
                        c == SYSTEM || \
@@ -259,14 +267,16 @@ static int catch_command_alias (char *line, CMD *cmd)
                          c == GARCH || \
                          c == MPOLS || \
                          c == POISSON || \
+                         c == PRINT || \
                          c == SCATTERS || \
-                         c == TSLS)
+                         c == TSLS || \
+                         c == XTAB)
 
 #define NEEDS_LISTSEP(c) (c == AR || \
                           c == ARMA || \
                           c == GARCH || \
-                          c == SCATTERS || \
-                          c == TSLS)
+                          c == TSLS || \
+                          c == XTAB)
 
 #define DEFAULTS_TO_FULL_LIST(c) (c == CORR || \
                                   c == DIFF || \
@@ -286,6 +296,7 @@ static int catch_command_alias (char *line, CMD *cmd)
                                c == STORE)
 
 #define RETURNS_LIST(c) (c == DIFF || \
+                         c == DUMMIFY || \
                          c == LDIFF || \
                          c == SDIFF || \
                          c == LAGS || \
@@ -686,9 +697,9 @@ static void parse_laglist_spec (const char *s, int *order, char **lname,
     }
 }
 
-int auto_transform_ok (const char *s, int *lnum,
-		       double ***pZ, DATAINFO *pdinfo,
-		       CMD *cmd)
+static int auto_transform_ok (const char *s, int *lnum,
+			      double ***pZ, DATAINFO *pdinfo,
+			      CMD *cmd)
 {
     char fword[9];
     int *genlist = NULL;
@@ -698,7 +709,7 @@ int auto_transform_ok (const char *s, int *lnum,
     int err = 0, ok = 1;
 
     if (sscanf(s, "%8[^(](", fword)) {
-	char *lname = NULL;
+	char *param = NULL;
 	int *gotlist;
 	int vnum = 0;
 
@@ -714,19 +725,30 @@ int auto_transform_ok (const char *s, int *lnum,
 
 	if (trans > 0) {
 	    s = strchr(s, '(') + 1;
+
 	    if (trans == LAGS) {
-		parse_laglist_spec(s, &order, &lname, &vnum,
+		parse_laglist_spec(s, &order, &param, &vnum,
 				   (const double **) *pZ, pdinfo);
 	    } else {
-		lname = gretl_word_strdup(s, NULL);
+		param = gretl_word_strdup(s, NULL);
 	    }
-	    if (lname != NULL) {
-		gotlist = get_list_by_name(lname);
+
+	    if (param != NULL) {
+		/* try for a named list */
+		gotlist = get_list_by_name(param);
 		if (gotlist != NULL) {
 		    genlist = gretl_list_copy(gotlist);
+		} else {
+		    vnum = varindex(pdinfo, param);
+		    if (vnum == pdinfo->v) {
+			vnum = 0;
+		    }
 		}
-		free(lname);
-	    } else if (vnum > 0) {
+		free(param);
+	    } 
+
+	    if (genlist == NULL && vnum > 0) {
+		/* try for a single variable */
 		genlist = gretl_list_new(1);
 		if (genlist != NULL) {
 		    genlist[1] = vnum;
@@ -748,6 +770,8 @@ int auto_transform_ok (const char *s, int *lnum,
 	err = list_xpxgenr(&genlist, pZ, pdinfo, opt);
     } else if (trans == LAGS) {
 	err = list_laggenr(&genlist, order, pZ, pdinfo);
+    } else if (trans == DUMMIFY) {
+	err = list_dumgenr(&genlist, pZ, pdinfo);
     }
 
     if (err) {
@@ -764,34 +788,31 @@ int auto_transform_ok (const char *s, int *lnum,
     return ok;
 } 
 
-static int plot_var_ok (const char *s, int *lnum,
+static int add_time_ok (const char *s, int *lnum,
 			double ***pZ, DATAINFO *pdinfo,
 			CMD *cmd)
 {
-    int pnum, ok = 1;
-    
-    if (strcmp(s, "qtrs") &&
-	strcmp(s, "months") &&
-	strcmp(s, "time")) { 
-	return 0;
-    }
-
-    pnum = plotvar_from_varname(pZ, pdinfo, s);
-
-    if (pnum < 0) {
-	cmd->errcode = 1;
-	ok = 0;
-	sprintf(gretl_errmsg, 
-		_("Failed to add plotting index variable"));
+    if (strcmp(s, "time")) {
+	return 0; /* not handled */
+    } else if (cmd->ci == GNUPLOT) {
+	cmd->list[0] -= 1;
+	cmd->opt |= OPT_T;
+	return 1; /* handled */
     } else {
-	cmd->list[*lnum] = pnum;
-	*lnum += 1;
-    }
+	int err = genrtime(pZ, pdinfo, 1);
 
-    return ok;
+	if (err) {
+	    cmd->errcode = err;
+	} else {
+	    cmd->list[*lnum] = varindex(pdinfo, "time");
+	    *lnum += 1;
+	}
+
+	return err == 0;
+    }
 } 
 
-#if defined(USE_GTK2) || defined (HAVE_FNMATCH_H)
+#if defined(USE_GLIB2) || defined (HAVE_FNMATCH_H)
 
 static int wildcard_expand_ok (const char *s, int *lnum,
 			       const DATAINFO *pdinfo, CMD *cmd)
@@ -1058,7 +1079,7 @@ static int fix_semicolon_after_var (char *s)
 
 static int check_end_command (CMD *cmd)
 {
-    if (*cmd->param) {
+    if (cmd->param != NULL && *cmd->param != 0) {
 	int cmdcode = gretl_command_number(cmd->param);
 
 	if (cmdcode == LOOP) {
@@ -1076,32 +1097,26 @@ static int check_end_command (CMD *cmd)
     return cmd->errcode;
 }
 
-static int resize_cmd_param (CMD *cmd, const char *s, int inlen)
+static void cmd_param_grab_string (CMD *cmd, const char *s)
 {
-    char *param;
-    int len;
+    free(cmd->param);
+    cmd->param = gretl_strdup(s);
+    if (cmd->param == NULL) {
+	cmd->errcode = E_ALLOC;
+    }
+}
 
-    if (inlen > 0) {
-	len = inlen;
-    } else if (s != NULL) {
-	if (strchr(s, ' ') == NULL) {
-	    len = strlen(s);
-	} else {
-	    len = strcspn(s, " ");
+static void cmd_param_grab_word (CMD *cmd, const char *s)
+{
+    int n = strcspn(s, " \n\t");
+
+    if (n > 0) {
+	free(cmd->param);
+	cmd->param = gretl_strndup(s, n);
+	if (cmd->param == NULL) {
+	    cmd->errcode = E_ALLOC;
 	}
-	len++;
-    } else {
-	return 1;
     }
-
-    param = realloc(cmd->param, len);
-    if (param == NULL) {
-	return 1;
-    }
-
-    cmd->param = param;
-    
-    return 0;
 }
 
 /* Capture the next 'word' found following the initial command word
@@ -1131,14 +1146,12 @@ static int capture_param (const char *s, CMD *cmd,
 		    cmd->word);
 	}
     } else {
-	if (resize_cmd_param(cmd, NULL, strlen(s) + 1)) {
-	    cmd->errcode = E_ALLOC;
-	} else if (cmd->ci == PRINT || cmd->ci == FUNCERR) {
+	if (cmd->ci == PRINT || cmd->ci == FUNCERR) {
 	    /* grab the whole remainder of line */
-	    strcpy(cmd->param, s);
+	    cmd_param_grab_string(cmd, s);
 	} else {
 	    /* grab one 'word' */
-	    sscanf(s, "%s", cmd->param);
+	    cmd_param_grab_word(cmd, s);
 	}
 #if CMD_DEBUG
 	fprintf(stderr, "capture_param: s='%s', param='%s'\n",
@@ -1372,7 +1385,7 @@ int parse_command_line (char *line, CMD *cmd, double ***pZ, DATAINFO *pdinfo)
     fprintf(stderr, "parsing '%s'\n", line);
 #endif
 
-    /* look for ramu practice files */
+    /* legacy: look for Ramanathan practice files */
     if (line[0] == '(' && line[1] == '*') {
 	cmd->ignore = 1;
 	gotdata = trydatafile(line, cmd);
@@ -1487,8 +1500,8 @@ int parse_command_line (char *line, CMD *cmd, double ***pZ, DATAINFO *pdinfo)
 
     /** now for a few command which may or may not take a list **/
 
-    /* PRINT can take a list, but not in its string literal variant */
     if (cmd->ci == PRINT && strstr(line, "\"")) {
+	/* no list in string literal variant */
 	cmd->nolist = 1;
 	capture_param(line, cmd, NULL, NULL);
 	return cmd->errcode;
@@ -1675,11 +1688,7 @@ int parse_command_line (char *line, CMD *cmd, double ***pZ, DATAINFO *pdinfo)
 	/* special: optional lag order for correlogram */
 	if (cmd->ci == CORRGM && j == 2) {
 	    cmd->list[0] = 1;
-	    if (resize_cmd_param(cmd, remainder, 0)) {
-		cmd->errcode = E_ALLOC;
-		goto bailout;
-	    }
-	    sscanf(remainder, "%s", cmd->param);
+	    cmd_param_grab_word(cmd, remainder);
 	    break;
 	}
 
@@ -1729,15 +1738,15 @@ int parse_command_line (char *line, CMD *cmd, double ***pZ, DATAINFO *pdinfo)
 		    }
 		}
 
-		/* Case 3: special plotting variable */
+		/* Case 3: add "time" automatically? */
 		else if (!cmd->errcode && 
-			 plot_var_ok(field, &lnum, pZ, pdinfo, cmd)) {
+			 add_time_ok(field, &lnum, pZ, pdinfo, cmd)) {
 		    /* handled, get on with it */
 		    pos += strlen(field) + 1;
 		    continue; 
 		} 
 
-#if defined(USE_GTK2) || defined (HAVE_FNMATCH_H)
+#if defined(USE_GLIB2) || defined (HAVE_FNMATCH_H)
 		/* wildcard expansion? */
 		else if (!cmd->errcode && 
 			 wildcard_expand_ok(field, &lnum, pdinfo, cmd)) {
@@ -1771,7 +1780,7 @@ int parse_command_line (char *line, CMD *cmd, double ***pZ, DATAINFO *pdinfo)
 	    }
 	} /* end if isalpha(*field) */
 
-#if defined(USE_GTK2) || defined (HAVE_FNMATCH_H)
+#if defined(USE_GLIB2) || defined (HAVE_FNMATCH_H)
 	else if (*field == '*') {
 	    if (wildcard_expand_ok(field, &lnum, pdinfo, cmd)) {
 		/* handled, get on with it */
@@ -1803,8 +1812,8 @@ int parse_command_line (char *line, CMD *cmd, double ***pZ, DATAINFO *pdinfo)
 		    /* turn off acceptance of AR lags etc */
 		    read_lags = 0;
 		}
-		if (cmd->ci == MPOLS) {
-		    poly = 1;
+		if (cmd->ci == MPOLS) { 	 
+		    poly = 1; 	 
 		}
 		continue;
 	    } else if (cmd->ci == VAR || cmd->ci == VECM) {
@@ -1825,7 +1834,7 @@ int parse_command_line (char *line, CMD *cmd, double ***pZ, DATAINFO *pdinfo)
 
 	/* check cmd->list for scalars */
 	if (!read_lags && !poly && !(SCALARS_OK_IN_LIST(cmd->ci))) {
-	    if (!pdinfo->vector[cmd->list[lnum-1]]) {
+	    if (var_is_scalar(pdinfo, cmd->list[lnum-1])) {
 		cmd->errcode = 1;
 		sprintf(gretl_errmsg, _("variable %s is a scalar"), field);
 		goto bailout;
@@ -1861,6 +1870,10 @@ int parse_command_line (char *line, CMD *cmd, double ***pZ, DATAINFO *pdinfo)
 	cmd->errcode = E_ARGS;
     }
 
+    if (cmd->ci == GNUPLOT && !(cmd->opt & OPT_T) && cmd->list[0] < 2) {
+	cmd->errcode = E_ARGS;
+    }
+
     /* check list for duplicated variables? */
     if (!cmd->errcode && !cmd->nolist) {
 	int dupv = gretl_list_duplicates(cmd->list, cmd->ci);
@@ -1942,8 +1955,12 @@ int help (const char *cmdword, const char *helpfile, PRN *prn)
     ok = gretl_command_number(cmdword) > 0;
 
     if (!ok) {
-	pprintf(prn, _("\"%s\" is not a gretl command.\n"), cmdword);
-	return 1;
+	if (gretl_is_public_user_function(cmdword)) {
+	    return user_function_help(cmdword, prn);
+	} else {
+	    pprintf(prn, _("\"%s\" is not a gretl command.\n"), cmdword);
+	    return 1;
+	}
     }
 
     if ((fp = gretl_fopen(helpfile, "r")) == NULL) {
@@ -2323,7 +2340,7 @@ static int n_separators (const int *list)
                            c == MPOLS)
 
 #define hold_param(c) (c == TSLS || c == AR || c == ARMA || \
-                       c == CORRGM || c == MPOLS || c == SCATTERS || \
+                       c == CORRGM || c == SCATTERS || c == MPOLS || \
                        c == GNUPLOT || c == LOGISTIC || c == GARCH || \
                        c == EQUATION || c == POISSON)
 
@@ -2467,9 +2484,10 @@ print_cmd_list (const CMD *cmd, const DATAINFO *pdinfo,
 
 #undef ECHO_DEBUG
 
-static int is_silent (const CMD *cmd)
+static int is_silent (const CMD *cmd, const char *line)
 {
-    if (cmd->ci == FUNCERR) {
+    if (cmd->ci == FUNCERR || cmd->ci == PRINTF ||
+	(cmd->ci == PRINT && strchr(line, '"'))) {
 	return 1;
     }
 
@@ -2514,7 +2532,7 @@ void echo_cmd (const CMD *cmd, const DATAINFO *pdinfo, const char *line,
 #endif
 
     /* don't echo certain things */
-    if (is_silent(cmd)) {
+    if (is_silent(cmd, line)) {
 	return;
     }
 
@@ -2718,18 +2736,24 @@ static void get_optional_filename (const char *line, CMD *cmd)
     }    
 }
 
-static int make_var_label (const char *line, const DATAINFO *pdinfo, 
-			   PRN *prn)
+static int set_var_info (const char *line, gretlopt opt, 
+			 DATAINFO *pdinfo, PRN *prn)
 {
     char *p;
     char vname[VNAMELEN];
+    int cmdlen = 0;
     int v, setstuff = 0;
 
     if (pdinfo->varinfo == NULL) {
 	return 1;
     }
 
-    if (sscanf(line, "label %15s", vname) != 1) {
+    /* skip command word */
+    cmdlen = strcspn(line, " ");
+    line += cmdlen++;
+    line += strspn(line, " ");
+
+    if (sscanf(line, "%15s", vname) != 1) {
 	return E_PARSE;
     }
 
@@ -2739,7 +2763,13 @@ static int make_var_label (const char *line, const DATAINFO *pdinfo,
 	return E_UNKVAR;
     }
 
-    p = get_flag_field(line + 6, 'd');
+    if (opt & OPT_D) {
+	set_var_discrete(pdinfo, v, 1);
+    } else if (opt & OPT_C) {
+	set_var_discrete(pdinfo, v, 0);
+    }
+
+    p = get_flag_field(line + cmdlen, 'd');
     if (p != NULL) {
 	setstuff = 1;
 	*VARLABEL(pdinfo, v) = 0;
@@ -2747,7 +2777,7 @@ static int make_var_label (const char *line, const DATAINFO *pdinfo,
 	free(p);
     }
 
-    p = get_flag_field(line + 6, 'n');
+    p = get_flag_field(line + cmdlen, 'n');
     if (p != NULL) {
 	setstuff = 1;
 	*DISPLAYNAME(pdinfo, v) = 0;
@@ -2755,10 +2785,6 @@ static int make_var_label (const char *line, const DATAINFO *pdinfo,
 	free(p);
     } 
 
-    if (!setstuff && *VARLABEL(pdinfo, v) != 0) {
-	pprintf(prn, "%s\n", VARLABEL(pdinfo, v));
-    }
-
     return 0;
 }
 
@@ -2889,7 +2915,7 @@ static int add_obs (int n, double ***pZ, DATAINFO *pdinfo, PRN *prn)
     } else if (n <= 0) {
 	err = E_PARSE;
     } else {
-	err = dataset_add_observations(n, pZ, pdinfo);
+	err = dataset_add_observations(n, pZ, pdinfo, OPT_A);
 	if (!err) {
 	    pprintf(prn, _("Dataset extended by %d observations"), n);
 	    pputc(prn, '\n');
@@ -2899,6 +2925,40 @@ static int add_obs (int n, double ***pZ, DATAINFO *pdinfo, PRN *prn)
     return err;
 }
 
+static void print_info (gretlopt opt, DATAINFO *pdinfo, PRN *prn)
+{
+#if 0 /* not really ready for this */
+    if (opt & OPT_V) {
+	const char *word;
+	const char *desc;
+	int m = get_first_model_stat(&word, &desc);
+
+	while (m > 0) {
+	    pprintf(prn, "%-8s  %s\n", word, _(desc));
+	    m = get_next_model_stat(&word, &desc);
+	}
+    } 
+
+    if (opt & OPT_T) {
+	/* print_available_tests(pdinfo, prn); */
+    }
+
+    if (opt == OPT_NONE) {
+	if (pdinfo->descrip != NULL) {
+	    pprintf(prn, "%s\n", pdinfo->descrip);
+	} else {
+	    pputs(prn, _("No data information is available.\n"));
+	}
+    }
+#else
+    if (pdinfo->descrip != NULL) {
+	pprintf(prn, "%s\n", pdinfo->descrip);
+    } else {
+	pputs(prn, _("No data information is available.\n"));
+    }
+#endif
+}
+
 /* common code for command-line and GUI client programs, where the
    command doesn't require special handling on the client side 
 */
@@ -2910,11 +2970,12 @@ int simple_commands (CMD *cmd, const char *line,
     int err = 0, order = 0;
     VMatrix *corrmat;
     Summary *summ;
-    int *genlist = NULL;
+    int *listcpy = NULL;
 
     if (RETURNS_LIST(cmd->ci)) {
-	genlist = gretl_list_copy(cmd->list);
-	if (genlist == NULL) {
+	/* list is potentially modified -> make a copy */
+	listcpy = gretl_list_copy(cmd->list);
+	if (listcpy == NULL) {
 	    return E_ALLOC;
 	}
     }
@@ -2959,6 +3020,10 @@ int simple_commands (CMD *cmd, const char *line,
 	}
 	break;
 
+    case DISCRETE:
+	err = list_makediscrete(cmd->list, pdinfo, cmd->opt);
+	break;
+
     case ESTIMATE:
 	err = estimate_named_system(line, pZ, pdinfo, cmd->opt, prn);
 	break;
@@ -3006,7 +3071,7 @@ int simple_commands (CMD *cmd, const char *line,
     case DIFF:
     case LDIFF:
     case SDIFF:
-	err = list_diffgenr(genlist, cmd->ci, pZ, pdinfo);
+	err = list_diffgenr(listcpy, cmd->ci, pZ, pdinfo);
 	if (err) {
 	    if (cmd->ci == LDIFF) {
 		pputs(prn, _("Error adding log differences of variables.\n"));
@@ -3018,9 +3083,18 @@ int simple_commands (CMD *cmd, const char *line,
 	}
 	break;
 
+    case DUMMIFY:
+	err = list_dumgenr(&listcpy, pZ, pdinfo);
+	if (err) {
+	    pputs(prn, _("Error adding dummy variables.\n"));
+	} else {
+	    maybe_list_vars(pdinfo, prn);
+	}
+	break;
+
     case LAGS:
 	order = atoi(cmd->param);
-	err = list_laggenr(&genlist, order, pZ, pdinfo); 
+	err = list_laggenr(&listcpy, order, pZ, pdinfo); 
 	if (err) {
 	    pputs(prn, _("Error adding lags of variables.\n"));
 	} else {
@@ -3029,7 +3103,7 @@ int simple_commands (CMD *cmd, const char *line,
 	break;
 
     case LOGS:
-	err = list_loggenr(genlist, pZ, pdinfo);
+	err = list_loggenr(listcpy, pZ, pdinfo);
 	if (err) {
 	    pputs(prn, _("Error adding logs of variables.\n"));
 	} else {
@@ -3038,7 +3112,7 @@ int simple_commands (CMD *cmd, const char *line,
 	break;
 
     case SQUARE:
-	err = list_xpxgenr(&genlist, pZ, pdinfo, cmd->opt);
+	err = list_xpxgenr(&listcpy, pZ, pdinfo, cmd->opt);
 	if (err) {
 	    pputs(prn, _("Failed to generate squares\n"));
 	} else {
@@ -3080,11 +3154,7 @@ int simple_commands (CMD *cmd, const char *line,
 	break;
 
     case INFO:
-	if (pdinfo->descrip != NULL) {
-	    pprintf(prn, "%s\n", pdinfo->descrip);
-	} else {
-	    pputs(prn, _("No data information is available.\n"));
-	}
+	print_info(cmd->opt, pdinfo, prn);
 	break;
 
     case RENAME:
@@ -3096,8 +3166,8 @@ int simple_commands (CMD *cmd, const char *line,
 	}
 	break;
 
-    case LABEL:
-	err = make_var_label(line, pdinfo, prn);
+    case SETINFO:
+	err = set_var_info(line, cmd->opt, pdinfo, prn);
 	break;
 
     case LABELS:
@@ -3136,6 +3206,11 @@ int simple_commands (CMD *cmd, const char *line,
 	}
 	break; 
 
+    case XTAB:
+	err = crosstab(cmd->list, (const double **) *pZ, 
+		       pdinfo, cmd->opt, prn);
+	break;
+
     case MAHAL:
 	err = mahalanobis_distance(cmd->list, pZ, pdinfo, 
 				   cmd->opt, prn);
@@ -3213,8 +3288,8 @@ int simple_commands (CMD *cmd, const char *line,
 	break;
     }
 
-    if (genlist != NULL) {
-	free(genlist);
+    if (listcpy != NULL) {
+	free(listcpy);
     }
 
     if (err == E_OK) {
@@ -3359,6 +3434,8 @@ int ready_for_command (const char *line)
 	ok = 1;
     } else if (*line == 'q' || *line == 'x' || *line == '#') {
 	ok = 1;
+    } else if (*line == '/' && *(line+1) == '*') {
+	ok = 1;
     } else {
 	for (i=0; ok_cmds[i] != NULL && !ok; i++) {
 	    if (strncmp(line, ok_cmds[i], strlen(ok_cmds[i])) == 0) {
@@ -3466,3 +3543,4 @@ char *gretl_cmd_get_savename (char *sname)
 }
 
 
+
