@@ -50,7 +50,8 @@ typedef enum {
     BASQUE
 } ForcedLangs;
 
-#define HIDDEN_COMMAND(c) (c == FUNCERR || \
+#define HIDDEN_COMMAND(c) (c == FNCALL || \
+                           c == FUNCERR || \
                            c == REMEMBER)
     
 /* functions follow */
@@ -67,7 +68,7 @@ void gretl_cmd_set_context (CMD *cmd, int ci);
 
 void gretl_cmd_destroy_context (CMD *cmd);
 
-const char *gretl_cmd_get_savename (const CMD *cmd);
+char *gretl_cmd_get_savename (char *sname);
 
 gretlopt gretl_cmd_get_opt (const CMD *cmd);
 
@@ -76,7 +77,7 @@ void gretl_cmd_set_opt (CMD *cmd, gretlopt opt);
 int parse_command_line (char *line, CMD *cmd, double ***pZ, 
 			DATAINFO *pdinfo); 
 
-int get_command_index (const char *line, CMD *cmd);
+int get_command_index (char *line, CMD *cmd, const DATAINFO *pdinfo);
 
 int command_number (const char *cmd);
 
@@ -85,7 +86,7 @@ int help (const char *cmdword, const char *helpfile, PRN *prn);
 int parseopt (const char **argv, int argc, char *fname, 
 	      int *force_lang);
 
-int shell (const char *arg);
+int gretl_shell (const char *arg);
 
 void echo_cmd (const CMD *cmd, const DATAINFO *pdinfo, const char *line, 
 	       unsigned char flags, PRN *prn);
@@ -93,7 +94,7 @@ void echo_cmd (const CMD *cmd, const DATAINFO *pdinfo, const char *line,
 void echo_function_call (const char *line, unsigned char flags, PRN *prn);
 
 int simple_commands (CMD *cmd, const char *line, 
-		     double ***pZ, DATAINFO *datainfo,
+		     double ***pZ, DATAINFO *pdinfo,
 		     PRN *prn);
 
 int call_pca_plugin (VMatrix *corrmat, double ***pZ,
@@ -102,7 +103,7 @@ int call_pca_plugin (VMatrix *corrmat, double ***pZ,
 
 int ready_for_command (const char *line);
 
-void safe_print_line (const char *line, int loopstack, PRN *prn);
+void safe_print_line (const char *line, PRN *prn);
 
 #endif /* INTERACT_H */
 
