@@ -25,19 +25,28 @@ typedef struct kalman_ kalman;
 kalman *kalman_new (const gretl_matrix *S, const gretl_matrix *P,
 		    const gretl_matrix *F, const gretl_matrix *A,
 		    const gretl_matrix *H, const gretl_matrix *Q,
-		    const gretl_matrix *R, int *err);
+		    const gretl_matrix *R, const gretl_matrix *y,
+		    const gretl_matrix *x, gretl_matrix *E,
+		    int ncoeff, int ifc, int *err);
 
 void kalman_free (kalman *K);
 
-int kalman_forecast (kalman *K, const gretl_matrix *y, const gretl_matrix *x,
-		     gretl_matrix *E);
+int kalman_forecast (kalman *K);
+
+int kalman_get_ncoeff (const kalman *K);
 
 double kalman_get_loglik (const kalman *K);
 
+double kalman_get_arma_variance (const kalman *K);
+
 int kalman_set_initial_state_vector (kalman *K, const gretl_matrix *S);
 
 int kalman_set_initial_MSE_matrix (kalman *K, const gretl_matrix *P);
 
+void kalman_set_nonshift (kalman *K, int n);
+
+void kalman_use_ARMA_ll (kalman *K);
+
 #endif /* KALMAN_H_ */
 
 
