@@ -28,6 +28,11 @@
 
 #include <zlib.h>
 
+#ifndef GRETLCLI
+# include <libxml/xmlmemory.h>
+# include <libxml/parser.h>
+#endif
+
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
@@ -43,13 +48,8 @@
 #endif
 
 #ifdef ENABLE_NLS
-# ifdef USE_GTK2
-#  define I_(String) iso_gettext (String) 
-#  define M_(String) maybe_iso_gettext (String)
-# else
-#  define I_(String) _(String)
-#  define M_(String) _(String)
-# endif /* USE_GTK2 */
+# define I_(String) iso_gettext (String) 
+# define M_(String) maybe_iso_gettext (String)
 #else
 # define I_(String) String
 # define M_(String) String
@@ -94,6 +94,7 @@ enum ts_codes {
     TIME_SERIES,
     STACKED_TIME_SERIES,
     STACKED_CROSS_SECTION,
+    PANEL_UNKNOWN,
     SPECIAL_TIME_SERIES,
     STRUCTURE_UNKNOWN
 };
@@ -102,7 +103,8 @@ enum auto_genr {
     GENR_RESID,
     GENR_FITTED,
     GENR_RESID2,
-    GENR_H
+    GENR_H,
+    GENR_AHAT
 };
 
 enum progress_flags {
@@ -129,7 +131,8 @@ enum test_stats {
     GRETL_STAT_HARVEY_COLLIER,
     GRETL_STAT_RESET,
     GRETL_STAT_LR,
-    GRETL_STAT_WALD_CHISQ
+    GRETL_STAT_WALD_CHISQ,
+    GRETL_STAT_SUP_WALD
 };
 
 enum gretl_opt_flags {
@@ -141,20 +144,24 @@ enum gretl_opt_flags {
     OPT_E = 1 <<  4,
     OPT_F = 1 <<  5,
     OPT_G = 1 <<  6,
-    OPT_I = 1 <<  7,
-    OPT_L = 1 <<  8,
-    OPT_M = 1 <<  9,
-    OPT_N = 1 << 10,
-    OPT_O = 1 << 11,
-    OPT_P = 1 << 12,
-    OPT_Q = 1 << 13,
-    OPT_R = 1 << 14,
-    OPT_S = 1 << 15,
-    OPT_T = 1 << 16,
-    OPT_V = 1 << 17,
-    OPT_W = 1 << 18,
-    OPT_X = 1 << 19,
-    OPT_Z = 1 << 20,
+    OPT_H = 1 <<  7,
+    OPT_I = 1 <<  8,
+    OPT_J = 1 <<  9,
+    OPT_K = 1 << 10,
+    OPT_L = 1 << 11,
+    OPT_M = 1 << 12,
+    OPT_N = 1 << 13,
+    OPT_O = 1 << 14,
+    OPT_P = 1 << 15,
+    OPT_Q = 1 << 16,
+    OPT_R = 1 << 17,
+    OPT_S = 1 << 18,
+    OPT_T = 1 << 19,
+    OPT_U = 1 << 20,
+    OPT_V = 1 << 21,
+    OPT_W = 1 << 22,
+    OPT_X = 1 << 23,
+    OPT_Z = 1 << 24,
     OPT_UNSET = 1 << 30
 };
 
@@ -172,9 +179,16 @@ typedef enum {
     OP_KRON    = 27,
 } GretlOp;
 
+typedef enum {
+    D_NONE = 0,
+    D_UNIFORM,
+    D_NORMAL,
+    D_GAMMA
+} DistCode;
+
 typedef enum {
     T_NONE,
-    T_LOG, 
+    T_LOG,
     T_EXP, 
     T_SIN, 
     T_COS,
@@ -188,39 +202,64 @@ typedef enum {
     T_QNORM,
     T_GAMMA,
     T_LNGAMMA,
-    T_NORMAL, 
     T_UNIFORM, 
+    T_NORMAL, 
+    T_CHISQ, 
+    T_STUDENT,
+    T_LOG10,
+    T_LOG2,
     T_MATHMAX
 } GretlMathFunc;
 
+#ifndef CMPLX
+typedef struct _cmplx cmplx;
+struct _cmplx {
+    double r;
+    double i;
+};
+#endif
+
 typedef unsigned long gretlopt;
 
 typedef struct VARINFO_ VARINFO;
+typedef struct PANINFO_ PANINFO;
 typedef struct DATAINFO_ DATAINFO;
 typedef struct PATHS_ PATHS;
 typedef struct VMatrix_ VMatrix;
 typedef struct SAMPLE_ SAMPLE;
 typedef struct ARINFO_ ARINFO;
 typedef struct MODEL_ MODEL;
-typedef struct GRAPHT_ GRAPHT;
 typedef struct PRN_ PRN;
 typedef struct FITRESID_ FITRESID;
 typedef struct DATASET_ DATASET;
 typedef struct GRETL_VAR_ GRETL_VAR;
 
-typedef struct mp_results_ mp_results;
 typedef struct model_data_item_ model_data_item;
 typedef struct ModelTest_ ModelTest;
+typedef struct _gretl_equation_system gretl_equation_system;
 
 /* information on individual variable */
 struct VARINFO_ {
     char label[MAXLABEL];
     char display_name[MAXDISP];
+    int flags;
     char compact_method;
     char stack_level;
+    char line_width;
     char **sorted_markers;
 };
 
+/* panel indexing information */
+struct PANINFO_ {
+    int nunits;    /* number of cross-sectional units */
+    int Tmin;      /* min. number of time-series observations per unit */
+    int Tmax;      /* max. number of time-series observations per unit */
+    int olen;      /* length in digits of highest time-series index value */
+    int *unit;     /* index array, cross-sectional units */
+    int *period;   /* index array, time periods */
+    char *padmask; /* mask recording padding, when subsampled */
+};
+
 /* information on data set */
 struct DATAINFO_ { 
     int v;              /* number of variables */
@@ -233,6 +272,7 @@ struct DATAINFO_ {
     char endobs[OBSLEN]; /* string representation of ending obs */
     char **varname;     /* array of names of variables */
     VARINFO **varinfo;  /* array of specific info on vars */
+    PANINFO *paninfo;   /* additional info for panel data */
     char markers;       /* whether (1) or not (0) the data file has
 			   observation markers */
     char delim;         /* default delimiter for "CSV" files */
@@ -240,7 +280,6 @@ struct DATAINFO_ {
     char submode;       /* mode of sub-sampling in force, if any */
     char **S;           /* to hold observation markers */
     char *descrip;      /* to hold info on data sources etc. */
-    char *vector;       /* hold info on vars: vector versus scalar */
     char *submask;      /* subsampling mask */
     void *data;         /* all-purpose pointer */
 };
@@ -267,7 +306,7 @@ struct PATHS_ {
     char x12a[MAXLEN];
     char x12adir[MAXLEN];
     char dbhost[32];
-    char pngfont[32];
+    char pngfont[128];
 };
 
 struct VMatrix_ {
@@ -310,20 +349,17 @@ struct MODEL_ {
 				    else = 0 */
     int ci;                      /* "command index" -- depends on 
 				    estimation method */
-    int nwt;                     /* ID of the weight variable (WLS) */
-    int order;                   /* lag order (e.g. for ARCH) */
+    int nwt;                     /* ID number of the weight variable (WLS) */
     int aux;                     /* code representing the sort of
 				    auxiliary regression this is (or not) */
     double *coeff;               /* array of coefficient estimates */
     double *sderr;               /* array of estimated std. errors */
     double *uhat;                /* regression residuals */
     double *yhat;                /* fitted values from regression */
-    double *xpx;
-    double *vcv;                 /* VCV matrix for coeff. estimates */
+    double *xpx;                 /* X'X matrix, in packed form */
+    double *vcv;                 /* VCV matrix for coefficient estimates */
     double ess, tss;             /* Error and Total Sums of Squares */
     double sigma;                /* Standard error of regression */
-    double ess_wt;               /* ESS using weighted data (WLS) */
-    double sigma_wt;             /* same thing for std. error */
     double rsq, adjrsq;          /* Unadjusted and adjusted R^2 */     
     double fstt;                 /* F-statistic */
     double lnL;                  /* log-likelihood */
@@ -334,39 +370,17 @@ struct MODEL_ {
     ARINFO *arinfo;              /* pointer to struct to hold special info for 
 				    autoregressive model */ 
     int errcode;                 /* Error code in case of failure */
-    char *name;
+    char *name;                  /* for use in GUI */
     int nparams;                 /* number of named model parameters */
     char **params;               /* for named model parameters */
-    int ntests;
-    ModelTest *tests;
-    void *data;                  /* pointer for use in re. missing data */
+    int ntests;                  /* number of attached test results */
+    ModelTest *tests;            /* attached hypothesis test results */
     DATASET *dataset;            /* for handling models estimated on a
 				    sub-sampled portion of the dataset */
     int n_data_items;            /* number of extra data items */
     model_data_item **data_items; /* pointer to additional data */
 };
 
-struct GRAPHT_ {
-    int ID;
-    int sort;
-    char name[32];
-    char fname[MAXLEN];
-}; 
-
-struct mp_results_ {
-    int ncoeff;
-    int t1, t2, ifc;
-    int dfn, dfd;
-    int *varlist;
-    char **varnames;
-    double *coeff;
-    double *sderr;
-    double sigma;
-    double ess;
-    double rsq, adjrsq;
-    double fstt;
-};
-
 #define VARLABEL(p,i)        ((p->varinfo[i])->label)
 #define DISPLAYNAME(p,i)     ((p->varinfo[i])->display_name)
 #define COMPACT_METHOD(p,i)  ((p->varinfo[i])->compact_method)
