@@ -21,6 +21,7 @@
 
 #include "libgretl.h"
 #include "libset.h"
+#include "usermat.h"
 
 #define PDEBUG 0
 
@@ -60,7 +61,7 @@ struct sample_info {
 };
     
 struct bhhh_opts {
-    double tolerance;
+    double toler;
     int maxiter;
 };
 
@@ -75,6 +76,8 @@ struct set_vars_ {
     int gretl_echo;             /* echoing commands or not */
     int gretl_msgs;             /* emitting non-error messages or not */
     char delim;                 /* delimiter for CSV data export */
+    int longdigits;             /* digits for printing data in long form */
+    gretl_matrix *initvals;     /* for parameter initialization */
     struct robust_opts ropts;   /* robust standard error options */
     struct garch_opts gopts;    /* GARCH covariance matrix */
     struct bkbp_opts bkopts;    /* Baxter-King filter */
@@ -115,9 +118,9 @@ static void garch_opts_copy (struct garch_opts *opts)
 
 static void bkbp_opts_init (struct bkbp_opts *opts)
 {
-    opts->k = 8;
-    opts->periods[0] = 8;
-    opts->periods[1] = 32;
+    opts->k = UNSET_INT;
+    opts->periods[0] = UNSET_INT;
+    opts->periods[1] = UNSET_INT;
 }
 
 static void bkbp_opts_copy (struct bkbp_opts *opts)
@@ -129,16 +132,25 @@ static void bkbp_opts_copy (struct bkbp_opts *opts)
 
 static void bhhh_opts_init (struct bhhh_opts *opts)
 {
-    opts->tolerance = NADBL;
+    opts->toler = NADBL;
     opts->maxiter = 500;
 }
 
 static void bhhh_opts_copy (struct bhhh_opts *opts)
 {
-    opts->tolerance = state->maxopts.tolerance;
+    opts->toler = state->maxopts.toler;
     opts->maxiter = state->maxopts.maxiter;
 }
 
+static void print_initvals (const gretl_matrix *ivals, PRN *prn)
+{
+    if (ivals == NULL) {
+	pputs(prn, " initvals = auto\n");
+    } else {
+	gretl_matrix_print_to_prn(ivals, " initvals =", prn);
+    }
+}
+
 static void sample_info_init (struct sample_info *sinfo)
 {
     sinfo->t1 = UNSET_INT;
@@ -149,10 +161,12 @@ static void sample_info_init (struct sample_info *sinfo)
 
 #define sinfo_is_set(s) (s.t1 != UNSET_INT && s.t2 != UNSET_INT)
 
-static void check_for_state (void) 
+static int check_for_state (void) 
 {
     if (state != NULL) {
-	libset_init();
+	return libset_init();
+    } else {
+	return 0;
     }
 }
 
@@ -171,6 +185,8 @@ static void state_vars_copy (set_vars *sv, const DATAINFO *pdinfo)
     sv->gretl_echo = state->gretl_echo; 
     sv->gretl_msgs = state->gretl_msgs; 
     sv->delim = state->delim; 
+    sv->longdigits = state->longdigits; 
+    sv->initvals = gretl_matrix_copy(state->initvals);
 
     robust_opts_copy(&sv->ropts);
     garch_opts_copy(&sv->gopts);
@@ -208,6 +224,8 @@ static void state_vars_init (set_vars *sv)
     sv->gretl_echo = 1; 
     sv->gretl_msgs = 1; 
     sv->delim = UNSET_INT;
+    sv->longdigits = 10;
+    sv->initvals = NULL;
 
     robust_opts_init(&sv->ropts);
     garch_opts_init(&sv->gopts);
@@ -218,38 +236,141 @@ static void state_vars_init (set_vars *sv)
 
 int get_hc_version (void)
 {
-    check_for_state();
-    return state->ropts.hc_version;
+    if (check_for_state()) {
+	return 0;
+    } else {
+	return state->ropts.hc_version;
+    }
 }
 
 double get_hp_lambda (void)
 {
-    check_for_state();
-    return state->hp_lambda;
+    if (check_for_state()) {
+	return 0.0;
+    } else {
+	return state->hp_lambda;
+    }
 }
 
-int get_bkbp_k (void)
+int set_hp_lambda (double d)
 {
-    check_for_state();
-    return state->bkopts.k;
+    if (check_for_state()) {
+	return E_ALLOC;
+    }
+
+    if (d > 0) {
+	state->hp_lambda = d;
+	return 0;
+    } else {
+	return 1;
+    }
 }
 
-void get_bkbp_periods (int *periods)
+int get_bkbp_k (const DATAINFO *pdinfo)
 {
-    check_for_state();
-    periods[0] = state->bkopts.periods[0];
-    periods[1] = state->bkopts.periods[1];
+    if (check_for_state()) {
+	return 0;
+    }
+
+    if (is_unset(state->bkopts.k)) {
+	if (pdinfo->pd == 1) {
+	    return 3;
+	} else if (pdinfo->pd == 4) {
+	    return 12;
+	} else if (pdinfo->pd == 12) {
+	    return 36;
+	} else {
+	    return 3;
+	}
+    } else {
+	return state->bkopts.k;
+    }
+}
+
+int set_bkbp_k (int k)
+{
+    if (check_for_state()) {
+	return E_ALLOC;
+    }
+
+    if (k > 0) {
+	state->bkopts.k = k;
+	return 0;
+    } else {
+	return 1;
+    }
+}
+
+void unset_bkbp_k (void)
+{
+    if (check_for_state()) {
+	return;
+    }
+
+    state->bkopts.k = UNSET_INT;
+}
+
+void get_bkbp_periods (const DATAINFO *pdinfo, int *l, int *u)
+{
+    if (check_for_state()) {
+	return;
+    }
+
+    if (is_unset(state->bkopts.periods[0])) {
+	*l = (pdinfo->pd == 4)? 6 :
+	    (pdinfo->pd == 12)? 18 : 2;
+    } else {
+	*l = state->bkopts.periods[0];
+    }
+
+    if (is_unset(state->bkopts.periods[1])) {
+	*u = (pdinfo->pd == 4)? 32 :
+	    (pdinfo->pd == 12)? 96 : 8;
+    } else {
+	*u = state->bkopts.periods[1];
+    }
+}
+
+int set_bkbp_periods (int l, int u)
+{
+    if (check_for_state()) {
+	return E_ALLOC;
+    }
+
+    if (l > 0 && u > l) {
+	state->bkopts.periods[0] = l;
+	state->bkopts.periods[1] = u;
+	return 0;
+    } else {
+	return 1;
+    }
+}
+
+void unset_bkbp_periods (void)
+{
+    if (check_for_state()) {
+	return;
+    }
+
+    state->bkopts.periods[0] = UNSET_INT;
+    state->bkopts.periods[1] = UNSET_INT;
 }
 
 double get_bhhh_toler (void)
 {
-    check_for_state();
-    return state->maxopts.tolerance;
+    if (check_for_state()) {
+	return 1.0;
+    }
+
+    return state->maxopts.toler;
 }
 
 int get_bhhh_maxiter (void)
 {
-    check_for_state();
+    if (check_for_state()) {
+	return 0;
+    }
+
     return state->maxopts.maxiter;
 }
 
@@ -257,12 +378,14 @@ int set_bhhh_toler (double tol)
 {
     int err = 0;
 
-    check_for_state();
+    if (check_for_state()) {
+	return E_ALLOC;
+    }
 
     if (tol <= 0.0) {
 	err = 1;
     } else {
-	state->maxopts.tolerance = tol;
+	state->maxopts.toler = tol;
     }
 
     return err;
@@ -272,7 +395,9 @@ int set_bhhh_maxiter (int n)
 {
     int err = 0;
 
-    check_for_state();
+    if (check_for_state()) {
+	return E_ALLOC;
+    }
 
     if (n < 1) {
 	err = 1;
@@ -283,16 +408,38 @@ int set_bhhh_maxiter (int n)
     return err;
 }
 
+int set_long_digits (int n)
+{
+    int err = 0;
+
+    if (check_for_state()) {
+	return E_ALLOC;
+    }
+
+    if (n < 1 || n > 20) {
+	err = 1;
+    } else {
+	state->longdigits = n;
+    }
+
+    return err;
+}
+
 
 int get_VAR_horizon (void)
 {
-    check_for_state();
-    return state->horizon;
+    if (check_for_state()) {
+	return 0;
+    } else {
+	return state->horizon;
+    }
 }
 
 double get_nls_toler (void)
 {
-    check_for_state();
+    if (check_for_state()) {
+	return 1.0;
+    }
 
     if (na(state->nls_toler)) {
 	state->nls_toler = get_default_nls_toler();
@@ -305,7 +452,9 @@ int set_nls_toler (double tol)
 {
     int err = 0;
 
-    check_for_state();
+    if (check_for_state()) {
+	return E_ALLOC;
+    }
 
     if (tol <= 0.0) {
 	err = 1;
@@ -318,7 +467,9 @@ int set_nls_toler (double tol)
 
 static int get_or_set_force_hc (int f)
 {
-    check_for_state();
+    if (check_for_state()) {
+	return 0;
+    }
 
     if (f >= 0) {
 	state->ropts.force_hc = f;
@@ -329,7 +480,9 @@ static int get_or_set_force_hc (int f)
 
 static int get_or_set_garch_vcv (int v)
 {
-    check_for_state();
+    if (check_for_state()) {
+	return 0;
+    }
 
     if (v >= 0) {
 	state->gopts.vcv_variant = v;
@@ -340,7 +493,9 @@ static int get_or_set_garch_vcv (int v)
 
 static int get_or_set_garch_robust_vcv (int v)
 {
-    check_for_state();
+    if (check_for_state()) {
+	return 0;
+    }
 
     if (v >= 0) {
 	state->gopts.robust_vcv_variant = v;
@@ -449,6 +604,17 @@ char get_csv_delim (const DATAINFO *pdinfo)
     }
 }
 
+int get_long_digits (void)
+{
+    check_for_state();
+    return state->longdigits;
+}
+
+const gretl_matrix *get_init_vals (void)
+{
+    return state->initvals;
+}    
+
 int get_hac_lag (int m)
 {
     check_for_state();
@@ -559,8 +725,40 @@ void set_garch_robust_vcv (const char *s)
     free(scpy);
 }
 
-static int set_bkbp_periods (const char *s0, const char *s1,
-			     PRN *prn)
+static int set_line_width (const char *s0, const char *s1,
+			   DATAINFO *pdinfo, PRN *prn)
+{
+    int v, w, err = 0;
+
+    if (!isdigit((unsigned char) *s1)) {
+	return 1;
+    }
+
+    if (isdigit((unsigned char) *s0)) {
+	v = atoi(s0);
+    } else {
+	v = varindex(pdinfo, s0);
+    }
+
+    if (v < 1 || v >= pdinfo->v) {
+	return E_DATA;
+    }
+
+    w = atoi(s1);
+
+    if (w < 0 || w > 32) {
+	err = E_DATA;
+    } else {
+	var_set_linewidth(pdinfo, v, w);
+	pprintf(prn, _("Line width for %s = %d\n"), 
+		pdinfo->varname[v], w);
+    }
+
+    return err;
+}
+
+static int set_bkbp_limits (const char *s0, const char *s1,
+			    PRN *prn)
 {
     int p0, p1;
 
@@ -587,6 +785,34 @@ static int set_bkbp_periods (const char *s0, const char *s1,
     return 0;
 }
 
+static int set_initvals (const char *s, const DATAINFO *pdinfo, PRN *prn)
+{
+    gretl_matrix *m;
+    char mname[VNAMELEN];
+    int err = 0;
+
+    /* skip past "set initvals" */
+    s += 12;
+
+    if (sscanf(s, "%15s", mname) != 1 || !strcmp(mname, "auto")) {
+	gretl_matrix_free(state->initvals);
+	state->initvals = NULL;
+    } else {
+	m = get_matrix_by_name(mname, pdinfo);
+	if (m == NULL) {
+	    pprintf(prn, _("'%s': no such matrix\n"), mname);
+	    err = E_DATA;
+	} else {
+	    state->initvals = gretl_matrix_copy(m);
+	    if (state->initvals == NULL) {
+		err = E_ALLOC;
+	    }
+	}
+    }
+
+    return err;
+}
+
 static int parse_set_plotfile (const char *s)
 {
     char *fname;
@@ -624,6 +850,8 @@ static char delim_from_arg (const char *s)
 	ret = ' ';
     } else if (!strcmp(s, "tab")) {
 	ret = '\t';
+    } else if (!strcmp(s, "semicolon")) {
+	ret = ';';
     }
 
     return ret;
@@ -639,10 +867,11 @@ static const char *arg_from_delim (char c)
 	ret = "space";
     } else if (c == '\t') {
 	ret = "tab";
+    } else if (c == ';') {
+	ret = "semicolon";
     }
 
     return ret;
-
 }
 
 static int display_settings (PRN *prn)
@@ -674,9 +903,19 @@ static int display_settings (PRN *prn)
 	pprintf(prn, " hp_lambda = %g\n", state->hp_lambda);
     }
 
-    pprintf(prn, " bkbp_limits = (%d, %d)\n", state->bkopts.periods[0], 
-	    state->bkopts.periods[1]);
-    pprintf(prn, " bkbp_k = %d\n", state->bkopts.k);
+    if (is_unset(state->bkopts.periods[0]) ||
+	is_unset(state->bkopts.periods[1])) {
+	pputs(prn, " bkbp_limits: auto\n");
+    } else {
+	pprintf(prn, " bkbp_limits = (%d, %d)\n", state->bkopts.periods[0], 
+		state->bkopts.periods[1]);
+    }
+
+    if (is_unset(state->bkopts.k)) {
+	pputs(prn, " bkbp_k: auto\n");
+    } else {
+	pprintf(prn, " bkbp_k = %d\n", state->bkopts.k);
+    }
 
     if (is_unset(state->horizon)) {
 	pputs(prn, " horizon: auto\n");
@@ -701,6 +940,8 @@ static int display_settings (PRN *prn)
 
     pprintf(prn, " shell_ok = %d\n", state->shell_ok);
     pprintf(prn, " csv_delim = %s\n", arg_from_delim(state->delim));
+    pprintf(prn, " longdigits = %d\n", state->longdigits);
+    print_initvals(state->initvals, prn);
 
     return 0;
 }
@@ -711,7 +952,7 @@ static int display_settings (PRN *prn)
 #define boolean_off(s) (!strcmp(s, "off") || !strcmp(s, "0") || \
                         !strcmp(s, "false"))
 
-int execute_set_line (const char *line, PRN *prn)
+int execute_set_line (const char *line, DATAINFO *pdinfo, PRN *prn)
 {
     char setobj[16], setarg[16], setarg2[16];
     int nw, err = E_PARSE;
@@ -726,11 +967,15 @@ int execute_set_line (const char *line, PRN *prn)
 	return display_settings(prn);
     }
 
-    /* special: plotfile */
-    if (nw > 1 && !strcmp(setobj, "plotfile")) {
-	return parse_set_plotfile(line);
+    /* specials which need the whole line */
+    if (nw > 1) {
+	if (!strcmp(setobj, "plotfile")) {
+	    return parse_set_plotfile(line);
+	} else if (!strcmp(setobj, "initvals")) {
+	    return set_initvals(line, pdinfo, prn);
+	}
     }
-    
+
     if (nw == 1) {
 	if (!strcmp(setobj, "echo")) {
 	    state->gretl_echo = 1;
@@ -874,10 +1119,16 @@ int execute_set_line (const char *line, PRN *prn)
 	    if (isdigit(*setarg)) {
 		err = set_bhhh_maxiter(atoi(setarg));
 	    }
-	}
+	} else if (!strcmp(setobj, "longdigits")) {
+	    if (isdigit(*setarg)) {
+		err = set_long_digits(atoi(setarg));
+	    }
+	} 
     } else if (nw == 3) {
 	if (!strcmp(setobj, "bkbp_limits")) {
-	    err = set_bkbp_periods(setarg, setarg2, prn);
+	    err = set_bkbp_limits(setarg, setarg2, prn);
+	} else if (!strcmp(setobj, "linewidth")) {
+	    err = set_line_width(setarg, setarg2, pdinfo, prn);
 	}
     }
 		    
@@ -1062,6 +1313,11 @@ static void free_state (set_vars *sv)
     if (sv->sinfo.submask != NULL) {
 	free(sv->sinfo.submask);
     }
+
+    if (sv->initvals != NULL) {
+	gretl_matrix_free(sv->initvals);
+    }
+
     free(sv);
 }
 
