@@ -56,9 +56,14 @@ int get_hac_lag (int m);
 int get_halt_on_error (void);
 
 double get_hp_lambda (void);
+int set_hp_lambda (double d);
 
-int get_bkbp_k (void);
-void get_bkbp_periods (int *periods);
+int get_bkbp_k (const DATAINFO *pdinfo);
+void get_bkbp_periods (const DATAINFO *pdinfo, int *l, int *u);
+int set_bkbp_k (int k);
+int set_bkbp_periods (int bkl, int bku);
+void unset_bkbp_k (void);
+void unset_bkbp_periods (void);
 
 int gretl_get_text_pause (void);
 
@@ -67,6 +72,8 @@ int get_bhhh_maxiter (void);
 int set_bhhh_toler (double tol);
 int set_bhhh_maxiter (int n);
 
+const gretl_matrix *get_init_vals (void);
+
 int get_VAR_horizon (void);
 
 double get_nls_toler (void);
@@ -88,9 +95,12 @@ int gretl_echo_on (void);
 void set_gretl_messages (int e);
 int gretl_messages_on (void);
 
+int set_long_digits (int n);
+int get_long_digits (void);
+
 char get_csv_delim (const DATAINFO *pdinfo);
 
-int execute_set_line (const char *line, PRN *prn);
+int execute_set_line (const char *line, DATAINFO *pdinfo, PRN *prn);
 
 #endif /* LIBSET_H */
 
