@@ -22,10 +22,12 @@
 #include "libgretl.h"
 #include "missing_private.h"
 
-typedef struct {
+typedef struct MISSOBS_ MISSOBS;
+
+struct MISSOBS_ {
     int misscount;
     char *missvec;
-} MISSOBS;
+};
 
 static char *model_missmask (const int *list, int t1, int t2,
 			     int n, const double **Z, int dwt,
@@ -42,9 +44,8 @@ static char *model_missmask (const int *list, int t1, int t2,
    observations had been temporarily purged.
 */
 
-static int reorganize_uhat_yhat (MODEL *pmod) 
+static int reorganize_uhat_yhat (MODEL *pmod, MISSOBS *mobs) 
 {
-    MISSOBS *mobs = (MISSOBS *) pmod->data;
     double *tmp;
     int t, g;
 
@@ -112,13 +113,15 @@ int undo_daily_repack (MODEL *pmod, double **Z,
     if (!gretl_model_get_int(pmod, "daily_repack")) {
 	return 1;
     }
-    
-    if (pmod->data == NULL) {
+
+    mobs = (MISSOBS *) gretl_model_get_data(pmod, "missobs");
+    if (mobs == NULL) {
 	return E_DATA;
     }
-			   
-    mobs = (MISSOBS *) pmod->data;
 
+    /* take charge of MISSOBS pointer */
+    gretl_model_detach_data_item(pmod, "missobs");
+			   
     tmpmiss = malloc(mobs->misscount * sizeof *tmpmiss);
     if (tmpmiss == NULL) {
 	err = E_ALLOC;
@@ -161,7 +164,7 @@ int undo_daily_repack (MODEL *pmod, double **Z,
     free(tmpgood);
 
     if (!err) {
-	err = reorganize_uhat_yhat(pmod); 
+	err = reorganize_uhat_yhat(pmod, mobs); 
     }  
 
     /* undo temporary shrinkage of pmod->t2 */
@@ -171,7 +174,6 @@ int undo_daily_repack (MODEL *pmod, double **Z,
        for future use (e.g. in forecasting) */
     free(mobs->missvec);
     free(mobs);
-    pmod->data = NULL;
 
     pmod->errcode = err;
 
@@ -282,7 +284,9 @@ int repack_missing_daily_obs (MODEL *pmod, double **Z,
 	pmod->t2 -= misscount;
 	mobs->missvec = missvec;
 	mobs->misscount = misscount;
-	pmod->data = mobs;
+	err = gretl_model_set_data(pmod, "missobs", mobs, 
+				   MODEL_DATA_STRUCT,
+				   sizeof *mobs);
     }
 
     return err;
@@ -407,7 +411,7 @@ int adjust_t1t2 (MODEL *pmod, const int *list, int *t1, int *t2,
 		 int n, const double **Z, int *misst)
 {
     int i, t, dwt = 0, t1min = *t1, t2max = *t2;
-    int missobs, ret = 0;
+    int vi, missobs, ret = 0;
     double xx;
 
     if (pmod != NULL && gretl_model_get_int(pmod, "wt_dummy")) {
@@ -419,10 +423,11 @@ int adjust_t1t2 (MODEL *pmod, const int *list, int *t1, int *t2,
     for (t=t1min; t<t2max; t++) {
 	missobs = 0;
 	for (i=1; i<=list[0]; i++) {
-	    if (list[i] == 0 || list[i] == LISTSEP) {
+	    vi = list[i];
+	    if (vi == 0 || vi == LISTSEP) {
 		continue;
 	    }
-	    xx = Z[list[i]][t];
+	    xx = Z[vi][t];
 	    if (dwt) {
 		xx *= Z[dwt][t];
 	    }
@@ -442,10 +447,11 @@ int adjust_t1t2 (MODEL *pmod, const int *list, int *t1, int *t2,
     for (t=t2max; t>t1min; t--) {
 	missobs = 0;
 	for (i=1; i<=list[0]; i++) {
-	    if (list[i] == 0 || list[i] == LISTSEP) {
+	    vi = list[i];
+	    if (vi == 0 || vi == LISTSEP) {
 		continue;
 	    }
-	    xx = Z[list[i]][t];
+	    xx = Z[vi][t];
 	    if (dwt) {
 		xx *= Z[dwt][t];
 	    }
@@ -466,17 +472,18 @@ int adjust_t1t2 (MODEL *pmod, const int *list, int *t1, int *t2,
 	   flag an error in case any are found */
 	for (t=t1min; t<=t2max; t++) {
 	    for (i=1; i<=list[0]; i++) {
-		if (list[i] == 0 || list[i] == LISTSEP) {
+		vi = list[i];
+		if (vi == 0 || vi == LISTSEP) {
 		    continue;
 		}
-		xx = Z[list[i]][t];
+		xx = Z[vi][t];
 		if (dwt) {
 		    xx *= Z[dwt][t];
 		}
 		if (na(xx)) {
-		    /* indentify first missing obs and var */
+		    /* identify first missing obs and var */
 		    *misst = t + 1;
-		    ret = list[i];
+		    ret = vi;
 		    break;
 		}
 	    }
@@ -490,10 +497,11 @@ int adjust_t1t2 (MODEL *pmod, const int *list, int *t1, int *t2,
 	missobs = 0;
 	for (t=t1min; t<=t2max; t++) {
 	    for (i=1; i<=list[0]; i++) {
-		if (list[i] == 0 || list[i] == LISTSEP) {
+		vi = list[i];
+		if (vi == 0 || vi == LISTSEP) {
 		    continue;
 		}
-		xx = Z[list[i]][t];
+		xx = Z[vi][t];
 		if (dwt) {
 		    xx *= Z[dwt][t];
 		}
@@ -524,6 +532,25 @@ int adjust_t1t2 (MODEL *pmod, const int *list, int *t1, int *t2,
     return ret;
 }
 
+/* drop first/last observations from sample if missing obs 
+   encountered -- also check for missing vals within the
+   remaining sample: return non-zero if there are such.
+   Adjust the t1 andt2 members of pdinfo if need be.
+*/
+
+int list_adjust_t1t2 (const int *list, const double **Z, DATAINFO *pdinfo)
+{
+    int err, misst = 0;
+
+    err = adjust_t1t2(NULL, list, &pdinfo->t1, &pdinfo->t2, 
+		      pdinfo->n, Z, &misst);
+    if (err) {
+	err = E_MISSDATA;
+    }
+    
+    return err;
+}
+
 /* drop first/last observations from sample if missing obs 
    encountered -- also check for missing vals within the
    remaining sample */
@@ -737,7 +764,7 @@ int set_miss (const int *list, const char *param, double **Z,
 	}
     } else {
 	for (i=1; i<=list[0]; i++) {
-	    if (!pdinfo->vector[list[i]]) {
+	    if (var_is_scalar(pdinfo, list[i])) {
 		pprintf(prn, _("The variable %s is a scalar\n"), 
 			pdinfo->varname[list[i]]);
 		continue;
