@@ -19,6 +19,9 @@
 
 /* modelprint.h for gretl */
 
+#ifndef MODELPRINT_H
+#define MODELPRINT_H
+
 int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 		PRN *prn);
 
@@ -26,5 +29,4 @@ const char *estimator_string (int ci, PRN *prn);
 
 void print_model_vcv_info (const MODEL *pmod, PRN *prn);
 
-void print_mpols_results (const mp_results *mpvals, DATAINFO *pdinfo,
-			  PRN *prn);
+#endif /* MODELPRINT_H */
