@@ -17,8 +17,7 @@
  *
  */
 
-/*  monte_carlo.c - loop simulation procedures
-*/  
+/*  monte_carlo.c - loop simulation procedures */
 
 #include "libgretl.h" 
 #include "loop_private.h"
@@ -132,7 +131,6 @@ struct LOOPSET_ {
     int n_models;
     int n_loop_models;
     int n_prints;
-    int n_storevars;
 
     char **lines;
     int *ci;
@@ -141,9 +139,9 @@ struct LOOPSET_ {
     LOOP_MODEL *lmodels;
     LOOP_PRINT *prns;
     char storefile[MAXLEN];
-    char **storename;
-    char **storelbl;
-    double *storeval;
+    gretlopt storeopt;
+    double **sZ;
+    DATAINFO *sdinfo;
     LOOPSET *parent;
     LOOPSET **children;
     int n_children;
@@ -320,7 +318,7 @@ static void set_loop_opts (LOOPSET *loop, gretlopt opt)
 #define OK_LOOP_MODEL(c) (c == ARMA || c == CORC || c == GARCH || \
                           c == HCCM || c == HILU || c == HSK || \
                           c == LAD || c == OLS || c == TSLS || \
-                          c == PWE || c == WLS)
+                          c == PWE || c == WLS || c == MPOLS)
 
 /**
  * ok_in_loop:
@@ -330,76 +328,81 @@ static void set_loop_opts (LOOPSET *loop, gretlopt opt)
  * 0 otherwise.
  */
 
-int ok_in_loop (int ci)
+int ok_in_loop (int c)
 {
-    int ok = 0;
-
-    if (ci == GENR ||
-	ci == LOOP ||
-	ci == FNCALL || 
-	ci == MATRIX ||
-	ci == STORE ||
-	ci == PRINT ||
-	ci == PRINTF ||
-	ci == PVALUE ||
-	ci == SMPL ||
-	ci == IF ||
-	ci == ELSE ||
-	ci == ENDIF ||
-	ci == BREAK ||
-	ci == ENDLOOP) { 
-	ok = 1;
+    if (c == GENR ||
+	c == LOOP ||
+	c == FNCALL || 
+	c == MATRIX ||
+	c == STORE ||
+	c == PRINT ||
+	c == PRINTF ||
+	c == PVALUE ||
+	c == SMPL ||
+	c == IF ||
+	c == ELSE ||
+	c == ENDIF ||
+	c == BREAK ||
+	c == ENDLOOP) { 
+	return 1;
     }
 
     /* "simple_commands" */
-    else if (ci == ADF || 
-	     ci == COINT || 
-	     ci == COINT2 || 
-	     ci == CORR ||
-	     ci == CRITERIA || 
-	     ci == CRITICAL || 
-	     ci == DIFF || 
-	     ci == HURST ||	
-	     ci == KPSS ||
-	     ci == LABEL ||
-	     ci == LAGS || 
-	     ci == LDIFF || 
-	     ci == LOGS ||
-	     ci == MEANTEST || 
-	     ci == MULTIPLY || 
-	     ci == OUTFILE ||
-	     ci == PCA ||
-	     ci == RHODIFF ||
-	     ci == RUNS || 
-	     ci == SPEARMAN || 
-	     ci == SQUARE || 
-	     ci == SUMMARY ||
-	     ci == VARLIST ||
-	     ci == VARTEST) {
-	ok = 1;
+    if (c == ADF || 
+	c == COINT || 
+	c == COINT2 || 
+	c == CORR ||
+	c == CRITERIA || 
+	c == CRITICAL || 
+	c == DIFF || 
+	c == HURST ||	
+	c == KPSS ||
+	c == LAGS || 
+	c == LDIFF || 
+	c == LOGS ||
+	c == MEANTEST || 
+	c == MULTIPLY || 
+	c == OUTFILE ||
+	c == PCA ||
+	c == REMEMBER || 
+	c == RHODIFF ||
+	c == RUNS || 
+	c == SETINFO ||
+	c == SPEARMAN || 
+	c == SQUARE || 
+	c == SUMMARY ||
+	c == VARLIST ||
+	c == VARTEST ||
+	c == XTAB ) {
+	return 1;
+    }
+
+    /* frequencies --- no graph allowed */
+    if (c == FREQ) {
+	return 1;
     }
 
     /* modeling commands */
-    else if (OK_LOOP_MODEL(ci)) {
+    if (OK_LOOP_MODEL(c)) {
 	return 1;
     }
 
     /* vector models */
-    else if (ci == VAR || ci == VECM) {
-	ok = 1;
+    if (c == VAR || c == VECM) {
+	return 1;
     }
     
     /* nonlinear models */
-    else if (ci == NLS || ci == MLE || ci == END) {
-	ok = 1;
+    if (c == NLS || c == MLE || c == END) {
+	return 1;
     }
 
     /* basic model tests */
-    else if (ci == ADD || ci == OMIT) {
-	ok = 1;
+    if (c == ADD || c == OMIT || c == TESTUHAT) {
+	return 1;
     }
 
-    return ok;
+    return 0;
 }
 
 static int loop_attach_child (LOOPSET *loop, LOOPSET *child)
@@ -526,7 +529,7 @@ ok_loop_var (const LOOPSET *loop, const DATAINFO *pdinfo, const char *vname)
 	       and will become available at runtime? */
 	    v = LOOP_VAL_UNDEF;
 	}
-    } else if (v == 0 || pdinfo->vector[v]) {
+    } else if (var_is_series(pdinfo, v)) {
 	strcpy(gretl_errmsg, _("The loop control variable "
 	       "must be a scalar"));
 	v = LOOP_VAL_BAD;
@@ -776,7 +779,7 @@ static double *save_delta_state (int v, double **Z, DATAINFO *pdinfo)
     double *x0;
     int i, n;
 
-    n = (pdinfo->vector[v])? pdinfo->n : 1;
+    n = (var_is_series(pdinfo, v))? pdinfo->n : 1;
 
     x0 = malloc(n * sizeof *x0);
     if (x0 != NULL) {
@@ -793,7 +796,7 @@ restore_delta_state (int v, double *x0, double **Z, DATAINFO *pdinfo)
 {
     int i, n;
 
-    n = (pdinfo->vector[v])? pdinfo->n : 1;
+    n = (var_is_series(pdinfo, v))? pdinfo->n : 1;
 
     for (i=0; i<n; i++) {
 	Z[v][i] = x0[i];
@@ -1387,7 +1390,6 @@ static void gretl_loop_init (LOOPSET *loop)
     loop->n_models = 0;
     loop->n_loop_models = 0;
     loop->n_prints = 0;
-    loop->n_storevars = 0;
 
     loop->lines = NULL;
     loop->ci = NULL;
@@ -1399,9 +1401,9 @@ static void gretl_loop_init (LOOPSET *loop)
     loop->prns = NULL;
 
     loop->storefile[0] = '\0';
-    loop->storename = NULL;
-    loop->storelbl = NULL;
-    loop->storeval = NULL;
+    loop->storeopt = OPT_NONE;
+    loop->sZ = NULL;
+    loop->sdinfo = NULL;
 
     loop->parent = NULL;
     loop->children = NULL;
@@ -1545,30 +1547,9 @@ static int loop_print_init (LOOP_PRINT *lprn, const int *list)
 
 static void loop_store_free (LOOPSET *loop)
 {
-    int i;
-
-    if (loop->storename != NULL) {
-	for (i=0; i<loop->n_storevars; i++) {
-	    free(loop->storename[i]);
-	}
-	free(loop->storename);
-	loop->storename = NULL;
-    }
-
-    if (loop->storelbl != NULL) {
-	for (i=0; i<loop->n_storevars; i++) {
-	    free(loop->storelbl[i]);
-	}
-	free(loop->storelbl);
-	loop->storelbl = NULL;
-    }
-
-    if (loop->storeval != NULL) { 
-	free(loop->storeval);
-	loop->storeval = NULL;
-    }
-
-    loop->n_storevars = 0;
+    destroy_dataset(loop->sZ, loop->sdinfo);
+    loop->sZ = NULL;
+    loop->sdinfo = NULL;
 }
 
 /**
@@ -1577,6 +1558,7 @@ static void loop_store_free (LOOPSET *loop)
  * @fname: name of file in which to store data.
  * @list: list of variables to be stored (written to file).
  * @pdinfo: data information struct.
+ * @opt: option for data storage format.
  *
  * Set up @loop for saving a set of variables.
  *
@@ -1584,63 +1566,38 @@ static void loop_store_free (LOOPSET *loop)
  */
 
 static int loop_store_init (LOOPSET *loop, const char *fname, 
-			    const int *list, DATAINFO *pdinfo)
+			    const int *list, DATAINFO *pdinfo,
+			    gretlopt opt)
 {
-    int i, tot = list[0] * loop->itermax;
+    int i;
 
-    if (loop->n_storevars > 0) {
+    if (loop->sZ != NULL) {
 	strcpy(gretl_errmsg, "Only one 'store' command is allowed in a "
 	       "progressive loop");
 	return E_DATA;
     }
 
-    loop->n_storevars = list[0];
-
-    loop->storename = NULL;
-    loop->storelbl = NULL;
-    loop->storeval = NULL;
+    loop->sdinfo = create_new_dataset(&loop->sZ, list[0] + 1, 
+				      loop->itermax, 0);
+    if (loop->sdinfo == NULL) {
+	return E_ALLOC;
+    }
 
     loop->storefile[0] = '\0';
     strncat(loop->storefile, fname, MAXLEN - 1);
+    loop->storeopt = opt;
 
-    loop->storename = malloc(list[0] * sizeof *loop->storename);
-    if (loop->storename == NULL) return E_ALLOC;
-
-    loop->storelbl = malloc(list[0] * sizeof *loop->storelbl);
-    if (loop->storelbl == NULL) goto cleanup;
-
-    loop->storeval = malloc(tot * sizeof *loop->storeval);
-    if (loop->storeval == NULL) goto cleanup;
-
-    for (i=0; i<loop->n_storevars; i++) {
-	loop->storename[i] = NULL;
-	loop->storelbl[i] = NULL;
-    }
-
-    for (i=0; i<loop->n_storevars; i++) {
+    for (i=1; i<loop->sdinfo->v; i++) {
 	char *p;
 
-	loop->storename[i] = malloc(VNAMELEN);
-	if (loop->storename[i] == NULL) goto cleanup;
-
-	strcpy(loop->storename[i], pdinfo->varname[list[i+1]]);
-
-	loop->storelbl[i] = malloc(MAXLABEL);
-	if (loop->storelbl[i] == NULL) goto cleanup;
-
-	strcpy(loop->storelbl[i], VARLABEL(pdinfo, list[i+1]));
-	if ((p = strstr(loop->storelbl[i], "(scalar)"))) {
+	strcpy(loop->sdinfo->varname[i], pdinfo->varname[list[i]]);
+	strcpy(VARLABEL(loop->sdinfo, i), VARLABEL(pdinfo, list[i]));
+	if ((p = strstr(VARLABEL(loop->sdinfo, i), "(scalar)"))) {
 	    *p = 0;
 	}
     }
 
     return 0;
-
- cleanup:
-
-    loop_store_free(loop);
-
-    return E_ALLOC;
 }
 
 static int add_model_record (LOOPSET *loop)
@@ -1651,12 +1608,12 @@ static int add_model_record (LOOPSET *loop)
 
     models = realloc(loop->models, (nm + 1) * sizeof *models);
     if (models == NULL) {
-	err = 1;
+	err = E_ALLOC;
     } else {
 	loop->models = models;
 	loop->models[nm] = gretl_model_new();
 	if (loop->models[nm] == NULL) {
-	    err = 1;
+	    err = E_ALLOC;
 	} else {
 	    loop->models[nm]->ID = nm + 1;
 	}
@@ -1669,6 +1626,23 @@ static int add_model_record (LOOPSET *loop)
     return err;
 }
 
+int model_is_in_loop (const MODEL *pmod)
+{
+    LOOPSET *loop = currloop;
+    int i;
+
+    while (loop != NULL) {
+	for (i=0; i<loop->n_models; i++) {
+	    if (pmod == loop->models[i]) {
+		return 1;
+	    }
+	}
+	loop = loop->parent;
+    }
+
+    return 0;
+}
+
 static int add_loop_model (LOOPSET *loop)
 {
     int nlm = loop->n_loop_models;
@@ -1758,8 +1732,8 @@ static int add_loop_print (LOOPSET *loop, const int *list)
     loop->prns = prns;
 
     if (loop_print_init(&loop->prns[np], list)) { 
-	strcpy(gretl_errmsg, _("Failed to initalize print struct for loop\n"));
-	err = 1;
+	fprintf(stderr, "Failed to initialize print struct for loop\n");
+	err = E_ALLOC;
     }
 
     if (!err) {
@@ -1794,8 +1768,11 @@ static void update_loop_print (LOOPSET *loop, int i,
 #endif
     
     for (j=1; j<=list[0]; j++) {
-	if (pdinfo->vector[list[j]]) t = pdinfo->t1;
-	else t = 0;
+	if (var_is_series(pdinfo, list[j])) {
+	    t = pdinfo->t1;
+	} else {
+	    t = 0;
+	}
 #ifdef ENABLE_GMP
 	mpf_set_d(m, (*pZ)[list[j]][t]); 
 	mpf_add(lprn->sum[j-1], lprn->sum[j-1], m);
@@ -2188,88 +2165,33 @@ static void print_loop_prn (LOOP_PRINT *lprn, int n,
 
 static int save_loop_store (LOOPSET *loop, PRN *prn)
 {
-    FILE *fp;
-    char gdtfile[MAXLEN], infobuf[1024];
-    char *xmlbuf = NULL;
-    time_t writetime;
-    int i, t;
+    char fname[MAXLEN];
+    int *list;
+    int err = 0;
+
+    list = gretl_consecutive_list_new(1, loop->sdinfo->v - 1);
+    if (list == NULL) {
+	return E_ALLOC;
+    }
 
     /* organize filename */
     if (loop->storefile[0] == '\0') {
-	sprintf(gdtfile, "%sloopdata.gdt", gretl_user_dir());	
+	sprintf(fname, "%sloopdata.gdt", gretl_user_dir());	
     } else {
-	strcpy(gdtfile, loop->storefile);
+	strcpy(fname, loop->storefile);
     }
 
-    if (strchr(gdtfile, '.') == NULL) {
-	strcat(gdtfile, ".gdt");
+    if (loop->storeopt == OPT_NONE &&
+	strchr(fname, '.') == NULL) {
+	strcat(fname, ".gdt");
     }
 
-    fp = gretl_fopen(gdtfile, "w");
-    if (fp == NULL) {
-	return E_FOPEN;
-    }
+    err = write_data(fname, list, (const double **) loop->sZ, 
+		     loop->sdinfo, loop->storeopt, NULL);
 
-    writetime = time(NULL);
+    free(list);
 
-    pprintf(prn, _("printing %d values of variables to %s\n"), 
-	    loop->itermax, gdtfile);
-
-    fprintf(fp, "<?xml version=\"1.0\"?>\n"
-	    "<!DOCTYPE gretldata SYSTEM \"gretldata.dtd\">\n\n"
-	    "<gretldata name=\"%s\" frequency=\"1\" "
-	    "startobs=\"1\" endobs=\"%d\" ", 
-	    gdtfile, loop->itermax);
-
-    fprintf(fp, "type=\"cross-section\">\n");
-
-    sprintf(infobuf, "%s %s", _("simulation data written"),
-	    print_time(&writetime)); 
-    xmlbuf = gretl_xml_encode(infobuf);
-    fprintf(fp, "<description>\n%s\n</description>\n", xmlbuf);
-    free(xmlbuf);
-
-    gretl_push_c_numeric_locale();
-
-    /* print info on variables */
-    fprintf(fp, "<variables count=\"%d\">\n", loop->n_storevars);
-
-    for (i=0; i<loop->n_storevars; i++) {
-	xmlbuf = gretl_xml_encode(loop->storename[i]);
-	fprintf(fp, "<variable name=\"%s\"", xmlbuf);
-	free(xmlbuf);
-	xmlbuf = gretl_xml_encode(loop->storelbl[i]);
-	fprintf(fp, "\n label=\"%s\"/>\n", xmlbuf);
-	free(xmlbuf);
-    }
-
-    fputs("</variables>\n", fp);
-
-    fprintf(fp, "<observations count=\"%d\" labels=\"false\">\n",
-	    loop->itermax);
-
-    for (t=0; t<loop->itermax; t++) {
-	double x;
-
-	fputs("<obs>", fp);
-	for (i=0; i<loop->n_storevars; i++) {
-	    x = loop->storeval[loop->itermax * i + t];
-	    if (na(x)) {
-		fputs("NA ", fp);
-	    } else {
-		fprintf(fp, "%g ", x);
-	    }
-	}
-	fputs("</obs>\n", fp);
-    }
-
-    fprintf(fp, "</observations>\n</gretldata>\n");
-
-    gretl_pop_c_numeric_locale();
-
-    fclose(fp);
-
-    return 0;
+    return err;
 }
 
 static int 
@@ -2346,16 +2268,16 @@ substitute_dollar_targ (char *str, const LOOPSET *loop,
 static void update_loop_store (const int *list, LOOPSET *loop,
 			       const double **Z, DATAINFO *pdinfo)
 {
-    int i, sv;
+    int i;
 
-    for (i=0; i<list[0]; i++) {
-	sv = i * loop->itermax + loop->iter;
-	if (pdinfo->vector[list[i+1]]) { 
-	    loop->storeval[sv] = Z[list[i+1]][pdinfo->t1 + 1];
+    for (i=1; i<=list[0]; i++) {
+	if (var_is_series(pdinfo, list[i])) { 
+	    loop->sZ[i][loop->iter] = Z[list[i]][pdinfo->t1 + 1]; /* ?? */
 	} else {
-	    loop->storeval[sv] = Z[list[i+1]][0];
+	    loop->sZ[i][loop->iter] = Z[list[i]][0];
 	}
     }
+
 }
 
 static int clr_attach_var (controller *clr, const DATAINFO *pdinfo)
@@ -2715,7 +2637,6 @@ int gretl_loop_exec (char *line, double ***pZ, DATAINFO **ppdinfo,
 	    case DIFF: 
 	    case HURST:	
 	    case KPSS:
-	    case LABEL:
 	    case LAGS: 
 	    case LDIFF: 
 	    case LOGS:
@@ -2726,11 +2647,13 @@ int gretl_loop_exec (char *line, double ***pZ, DATAINFO **ppdinfo,
 	    case PCA:
 	    case RHODIFF:
 	    case RUNS: 
+	    case SETINFO:
 	    case SPEARMAN: 
 	    case SQUARE: 
 	    case SUMMARY:
 	    case VARLIST:
 	    case VARTEST: 
+	    case XTAB:
 		err = simple_commands(&cmd, linecpy, pZ, pdinfo, prn);
 		break;
 
@@ -2761,6 +2684,12 @@ int gretl_loop_exec (char *line, double ***pZ, DATAINFO **ppdinfo,
 		} 
 		break;
 
+	    case FREQ:
+		/* note: no graphs in loops */
+		err = freqdist(cmd.list[1], (const double **) *pZ, pdinfo, 0, 
+			       cmd.opt, prn);
+		break;
+
 	    case GENR:
 		err = generate(linecpy, pZ, pdinfo, cmd.opt, 
 			       (loop_is_verbose(loop))? prn : NULL);
@@ -2773,6 +2702,7 @@ int gretl_loop_exec (char *line, double ***pZ, DATAINFO **ppdinfo,
 	    case HILU:
 	    case HSK:
 	    case LAD:
+	    case MPOLS:
 	    case OLS:
 	    case TSLS:
 	    case PWE:
@@ -2797,6 +2727,8 @@ int gretl_loop_exec (char *line, double ***pZ, DATAINFO **ppdinfo,
 		    *models[0] = lsq(cmd.list, pZ, pdinfo, cmd.ci, cmd.opt);
 		} else if (cmd.ci == LAD) {
 		    *models[0] = lad(cmd.list, pZ, pdinfo);
+		} else if (cmd.ci == MPOLS) {
+		    *models[0] = mp_ols(cmd.list, (const double **) *pZ, pdinfo);
 		} else if (cmd.ci == HSK) {
 		    *models[0] = hsk_func(cmd.list, pZ, pdinfo);
 		} else if (cmd.ci == ARMA) {
@@ -2864,14 +2796,19 @@ int gretl_loop_exec (char *line, double ***pZ, DATAINFO **ppdinfo,
 		}
 		if (err) {
 		    errmsg(err, prn);
-		    clear_model(models[1]);
-		} else {
+		} else if (!(cmd.opt & OPT_Q) && !(cmd.opt & OPT_W)) {
 		    swap_models(models[0], models[1]);
 		    set_as_last_model(models[0], GRETL_OBJ_EQN);
+		}
+		if (!(cmd.opt & OPT_W)) {
 		    clear_model(models[1]);
 		}
 		break;	
 
+	    case TESTUHAT:
+		err = model_error_dist(models[0], pZ, pdinfo, prn);
+		break;
+
 	    case MLE:
 	    case NLS:
 		if (loop_is_progressive(loop) || (cmd.opt & OPT_P)) {
@@ -2946,7 +2883,8 @@ int gretl_loop_exec (char *line, double ***pZ, DATAINFO **ppdinfo,
 	    case STORE:
 		if (loop_is_progressive(loop)) {
 		    if (loop->iter == 0) {
-			err = loop_store_init(loop, cmd.param, cmd.list, pdinfo);
+			err = loop_store_init(loop, cmd.param, cmd.list, 
+					      pdinfo, cmd.opt);
 			if (err) {
 			    break;
 			}
@@ -3016,11 +2954,15 @@ int gretl_loop_exec (char *line, double ***pZ, DATAINFO **ppdinfo,
 	/* need to update models[0] */
 	GretlObjType type;
 	void *ptr = get_last_model(&type);
+	int i;
 
 	if (type == GRETL_OBJ_EQN && models[0] != ptr) {
 	    swap_models(models[0], loop->models[loop->n_models - 1]);
 	    set_as_last_model(models[0], GRETL_OBJ_EQN);
 	}
+	for (i=0; i<loop->n_models; i++) {
+	    gretl_model_free(loop->models[i]);
+	}
     }
 
     gretl_cmd_free(&cmd);
