@@ -34,4 +34,8 @@ int gretl_loop_append_line (char *line, int ci, gretlopt opt,
 int gretl_loop_exec (char *line, double ***pZ, DATAINFO **ppdinfo, 
 		     MODEL **models, PRN *prn);
 
+int model_is_in_loop (const MODEL *pmod);
+
+int loop_scalar_read (int c);
+
 #endif /* MONTE_CARLO_H */
