@@ -21,10 +21,10 @@
 #include "var.h"
 #include "johansen.h"
 #include "system.h"
-#include "system_private.h"
 #include "objstack.h"
 #include "genrfuncs.h"
 #include "usermat.h"
+#include "modelspec.h"
 
 #define ODEBUG 0
 
@@ -71,6 +71,10 @@ void gretl_object_ref (void *ptr, GretlObjType type)
 
 	if (pmod != NULL) {
 	    pmod->refcount += 1;
+#if ODEBUG
+	    fprintf(stderr, "gretl_object_ref: refcount on %p is now %d\n",
+		    (void *) pmod, pmod->refcount);
+#endif
 	}
     } else if (type == GRETL_OBJ_VAR) {
 	GRETL_VAR *var = (GRETL_VAR *) ptr;
@@ -189,18 +193,26 @@ void gretl_model_protect (MODEL *pmod)
 
 static int model_is_protected (MODEL *pmod)
 {
-    int i;
+    int i, prot = 0;
 
     for (i=0; i<NPROT; i++) {
 	if (pmod == protected_models[i]) {
-#if ODEBUG
-	    fprintf(stderr, "model at %p is protected\n", (void *) pmod);
-#endif
-	    return 1;
+	    prot = 1;
+	    break;
 	}
     }
 
-    return 0;
+    if (!prot) {
+	prot = model_is_in_loop(pmod);
+    }
+
+#if ODEBUG
+    if (prot) {
+	fprintf(stderr, "model at %p is protected\n", (void *) pmod);
+    }
+#endif
+
+    return prot;
 }
 
 /**
@@ -310,11 +322,11 @@ void *get_last_model (GretlObjType *type)
 
 /**
  * gretl_object_get_name:
- * @ptr: pointer to gretl object (e.g. #MODEL).
+ * @p: pointer to gretl object (e.g. #MODEL).
  * @type: type of object.
  *
  * Returns: the name of the object of type @type with
- * location @ptr, or %NULL if the object is not found.
+ * location @p, or %NULL if the object is not found.
  */
 
 const char *gretl_object_get_name (void *p, GretlObjType type)
@@ -363,6 +375,18 @@ get_object_by_name (const char *oname, GretlObjType type, int *onum)
 #if ODEBUG
     fprintf(stderr, "get_object_by_name: name='%s', type=%d, ptr=%p\n",
 	    oname, type, (void *) ptr);
+    if (ptr == NULL) {
+	if (n_obj == 0) {
+	    fprintf(stderr, "(no currently saved objects)\n");
+	} else {
+	    fprintf(stderr, "failed: current objects:\n");
+	    for (i=0; i<n_obj; i++) {
+		fprintf(stderr, " %02d: '%s' (%p, type %d)\n", i,
+			gretl_object_get_name(ostack[i].ptr, ostack[i].type),
+			ostack[i].ptr, ostack[i].type);
+	    }
+	}
+    }
 #endif    
 
     return ptr;
@@ -493,7 +517,7 @@ void remove_model_from_stack (MODEL *pmod)
     }
 }
 
-static int object_on_stack (void *p)
+static int object_stack_index (const void *p)
 {
     int i, ret = -1;
 
@@ -514,6 +538,11 @@ static int object_on_stack (void *p)
     return ret;
 }
 
+int object_is_on_stack (const void *ptr)
+{
+    return (object_stack_index(ptr) >= 0);
+}
+
 static int 
 real_stack_object (void *p, GretlObjType type, const char *name, PRN *prn)
 {
@@ -524,7 +553,7 @@ real_stack_object (void *p, GretlObjType type, const char *name, PRN *prn)
 	return 1;
     }
 
-    if (object_on_stack(p) >= 0) {
+    if (object_stack_index(p) >= 0) {
 	return 0;
     }
 
@@ -794,12 +823,30 @@ real_get_obj_matrix (void *p, GretlObjType type, int idx, int *err)
     return M;
 }
 
+static stacker genr_model;
+
+void set_genr_model (MODEL *pmod)
+{
+    genr_model.type = GRETL_OBJ_EQN;
+    genr_model.ptr = pmod;
+}
+
+void unset_genr_model (void)
+{
+    genr_model.type = GRETL_OBJ_UNKNOWN;
+    genr_model.ptr = NULL;
+}
+
 static stacker *find_smatch (const char *oname)
 {
     stacker *smatch = NULL;
 
     if (oname == NULL || *oname == '\0') {
-	smatch = &last_model;
+	if (genr_model.ptr != NULL) {
+	    smatch = &genr_model;
+	} else {
+	    smatch = &last_model;
+	}
     } else {
 	const char *test;
 	int i;
@@ -831,7 +878,7 @@ double saved_object_get_scalar (const char *oname, const char *key,
     }
 
     if (ret == INVALID_STAT) {
-	*err = 1;
+	*err = E_BADSTAT;
     }
 
     return ret;
@@ -844,6 +891,11 @@ double saved_object_get_scalar_element (const char *oname, const char *key,
     stacker *smatch;
     int idx;
 
+#if ODEBUG
+    fprintf(stderr, "saved_object_get_scalar_element: oname='%s', key='%s'\n",
+	    oname, key);
+#endif
+
     smatch = find_smatch(oname);
 
     if (smatch != NULL) {
@@ -853,7 +905,7 @@ double saved_object_get_scalar_element (const char *oname, const char *key,
     }
 
     if (ret == INVALID_STAT && !*err) {
-	*err = 1;
+	*err = E_BADSTAT;
     }
 
     return ret;
@@ -875,7 +927,7 @@ double *saved_object_get_series (const char *oname, const char *key,
     }
 
     if (x == NULL && !*err) {
-	*err = 1;
+	*err = E_BADSTAT;
     }
 
     return x;
@@ -907,7 +959,7 @@ saved_object_get_matrix (const char *oname, const char *key,
     }
 
     if (M == NULL && !*err) {
-	*err = 1;
+	*err = E_BADSTAT;
     }    
 
     return M;
@@ -1052,6 +1104,81 @@ static void saved_object_free (stacker *s)
     }
 }
 
+int last_model_test_ok (int ci, gretlopt opt, const DATAINFO *pdinfo, 
+			PRN *prn)
+{
+    GretlObjType type;
+    void *ptr;
+    int err = 0;
+
+    ptr = get_last_model(&type);  
+    if (ptr == NULL) {
+	pputs(prn, _("Can't do this: no model has been estimated yet\n"));
+	return 1;
+    }
+
+    if (type == GRETL_OBJ_EQN) {
+	MODEL *pmod = (MODEL *) ptr;
+  
+	if (!model_test_ok(ci, opt, pmod, pdinfo)) {
+	    err = E_NOTIMP;
+	}
+	if (model_sample_problem(pmod, pdinfo)) {
+	    pputs(prn, _("Can't do: the current data set is different from "
+			 "the one on which\nthe reference model was estimated\n"));
+	    err = 1;
+	}
+    } else if (type == GRETL_OBJ_SYS) {
+	if (ci != RESTRICT && ci != TESTUHAT) {
+	    err = E_NOTIMP;
+	}
+    } else if (type == GRETL_OBJ_VAR) {
+	GRETL_VAR *var = (GRETL_VAR *) ptr;
+	int r = gretl_VECM_rank(var);
+
+	err = E_NOTIMP;
+
+	if (ci == RESTRICT && r > 0) {
+	    err = 0;
+	} else if (ci == TESTUHAT) {
+	    err = 0;
+	} else if (ci == LMTEST && ((opt & OPT_A) || (opt & OPT_H))) {
+	    err = 0;
+	} 
+    }
+
+    if (err == E_NOTIMP) {
+	pputs(prn, _("Sorry, command not available for this estimator"));
+	pputc(prn, '\n');
+    }
+
+    return err;
+}
+
+int last_model_test_uhat (double ***pZ, DATAINFO *pdinfo, PRN *prn)
+{
+    GretlObjType type;
+    void *ptr;
+    int err = 0;
+
+    ptr = get_last_model(&type);  
+    if (ptr == NULL) {
+	return E_DATA;
+    }
+
+    if (type == GRETL_OBJ_EQN) {
+	err = model_error_dist(ptr, pZ, pdinfo, prn);
+    } else if (type == GRETL_OBJ_SYS) {
+	err = system_normality_test(ptr, prn);
+    } else if (type == GRETL_OBJ_VAR) {
+	err = gretl_VAR_normality_test(ptr, prn);
+    } else {
+	err = E_DATA;
+    }
+
+    return err;
+}
+
 void gretl_saved_objects_cleanup (void)
 {
     int i;
@@ -1063,7 +1190,7 @@ void gretl_saved_objects_cleanup (void)
 	}
 #if ODEBUG
 	fprintf(stderr, "gretl_saved_objects_cleanup:\n"
-		" calling gretl_object_destroy on ostack[%d]\n", i);
+		" calling saved_object_free on ostack[%d]\n", i);
 #endif
 	saved_object_free(&ostack[i]);
     }
