@@ -20,8 +20,6 @@
 #ifndef OBJSTACK_H
 #define OBJSTACK_H
 
-#include "system.h"
-
 typedef enum {
     GRETL_OBJ_ANY,
     GRETL_OBJ_EQN,
@@ -42,6 +40,14 @@ typedef enum {
     GRETL_OBJ_MAX
 } GretlObjType;
 
+typedef enum {
+    IN_GUI_SESSION = 1 << 0,
+    IN_NAMED_STACK = 1 << 1,
+    IN_MODEL_TABLE = 1 << 2,
+    IS_LAST_MODEL  = 1 << 3,
+    IN_GRAPH_PAGE  = 1 << 4
+} SavedObjectFlags;
+
 enum {
     OBJ_ACTION_NONE,
     OBJ_ACTION_INVALID,
@@ -73,6 +79,8 @@ void *gretl_get_object_by_name (const char *name);
 int gretl_get_object_and_type (const char *name, void **pp, 
 			       GretlObjType *type);
 
+int object_is_on_stack (const void *ptr);
+
 int gretl_stack_object (void *ptr, GretlObjType type);
 
 int gretl_stack_object_as (void *ptr, GretlObjType type, const char *name);
@@ -112,6 +120,15 @@ int parse_object_command (const char *s, char *name, char **cmd);
 
 int match_object_command (const char *s, GretlObjType type);
 
+int last_model_test_ok (int ci, gretlopt opt, const DATAINFO *pdinfo, 
+			PRN *prn);
+
+int last_model_test_uhat (double ***pZ, DATAINFO *pdinfo, PRN *prn);
+
+void set_genr_model (MODEL *pmod);
+
+void unset_genr_model (void);
+
 void gretl_saved_objects_cleanup (void);
 
 #endif
