@@ -31,12 +31,13 @@
                        c == LAD || \
                        c == LOGISTIC || \
                        c == LOGIT || \
+                       c == MPOLS || \
                        c == OLS || \
                        c == OMIT || \
                        c == MLE || \
                        c == NLS || \
+                       c == PANEL || \
                        c == POISSON || \
-                       c == POOLED || \
                        c == PROBIT || \
                        c == PWE || \
                        c == TOBIT || \
@@ -74,19 +75,24 @@ struct gretl_option gretl_opts[] = {
     { ADF,      OPT_T, "ct" }, 
     { ADF,      OPT_V, "verbose" },
     { ADF,      OPT_Q, "quiet" },
+    { ADF,      OPT_F, "difference" },
     { ARMA,     OPT_C, "conditional" },
     { ARMA,     OPT_N, "nc" },    
     { ARMA,     OPT_V, "verbose" },
     { ARMA,     OPT_X, "x-12-arima" },
     { BXPLOT,   OPT_O, "notches" },
     { COINT,    OPT_N, "nc" },
+    { COINT,    OPT_R, "ctt" },     
+    { COINT,    OPT_S, "skip-df" },
+    { COINT,    OPT_T, "ct" },
     { COINT2,   OPT_A, "crt" },
     { COINT2,   OPT_D, "seasonals" },
     { COINT2,   OPT_N, "nc" },
     { COINT2,   OPT_R, "rc" },
     { COINT2,   OPT_T, "ct" },
-    { COINT2,   OPT_V, "verbose" },
+    { DISCRETE, OPT_R, "reverse" },
     { EQNPRINT, OPT_O, "complete" },
+    { EQNPRINT, OPT_T, "t-ratios" },
     { TABPRINT, OPT_O, "complete" },
     { ESTIMATE, OPT_M, "geomean" },
     { ESTIMATE, OPT_N, "no-df-corr" },
@@ -101,24 +107,27 @@ struct gretl_option gretl_opts[] = {
     { GARCH,    OPT_A, "arma-init" },    
     { GARCH,    OPT_R, "robust" },
     { GARCH,    OPT_V, "verbose" },
-    { GENR,     OPT_L, "local" },
-    { GENR,     OPT_P, "private" },
     { GNUPLOT,  OPT_O, "with-lines" },
     { GNUPLOT,  OPT_M, "with-impulses" },
     { GNUPLOT,  OPT_S, "suppress-fitted" },
+    { GNUPLOT,  OPT_T, "time-series" },
     { GNUPLOT,  OPT_Z, "dummy" },
     { GRAPH,    OPT_O, "tall" },
-    { HAUSMAN,  OPT_T, "time-effects" },
     { HILU,     OPT_B, "no-corc" },
     { IMPORT,   OPT_B, "box1" },
     { IMPORT,   OPT_O, "octave" },
+#if 0 /* not ready */
+    { INFO,     OPT_V, "variables" },
+    { INFO,     OPT_T, "tests" },
+#endif
     { KPSS,     OPT_T, "trend" },
     { KPSS,     OPT_V, "verbose" },
     { KPSS,     OPT_Q, "quiet" },
+    { KPSS,     OPT_F, "difference" },
     { LEVERAGE, OPT_S, "save" },
+    { LMTEST,   OPT_A, "autocorr" },
+    { LMTEST,   OPT_H, "arch" },
     { LMTEST,   OPT_L, "logs" },
-    { LMTEST,   OPT_M, "autocorr" },
-    { LMTEST,   OPT_O, "autocorr" },
     { LMTEST,   OPT_S, "squares" }, 
     { LMTEST,   OPT_P, "panel" },
     { LMTEST,   OPT_W, "white" },
@@ -129,7 +138,14 @@ struct gretl_option gretl_opts[] = {
     { MAHAL,    OPT_S, "save" },
     { MAHAL,    OPT_V, "vcv" },
     { MEANTEST, OPT_O, "unequal-vars" },
+    { MLE,      OPT_H, "hessian" },
+    { MLE,      OPT_R, "robust" },
     { MLE,      OPT_V, "verbose" },
+    { MPOLS,    OPT_O, "vcv" },
+    { MPOLS,    OPT_Q, "quiet" },
+    { MPOLS,    OPT_S, "simple-print" },
+    { NLS,      OPT_O, "vcv" },
+    { NLS,      OPT_R, "robust" },
     { NLS,      OPT_V, "verbose" },
     { OLS,      OPT_N, "no-df-corr" },
     { OLS,      OPT_O, "vcv" }, 
@@ -141,28 +157,41 @@ struct gretl_option gretl_opts[] = {
     { OMIT,     OPT_I, "silent" },
     { OMIT,     OPT_Q, "quiet" },
     { OMIT,     OPT_T, "inst" },
+    { OMIT,     OPT_W, "wald" },
     { OMITFROM, OPT_B, "both" },
     { OMITFROM, OPT_Q, "quiet" },
     { OMITFROM, OPT_T, "inst" },
     { OUTFILE,  OPT_A, "append" },
     { OUTFILE,  OPT_C, "close" },
     { OUTFILE,  OPT_W, "write" },
-    { PANEL,    OPT_C, "cross-section" },
-    { PANEL,    OPT_S, "time-series" },
+    { PANEL,    OPT_B, "between" },
+    { PANEL,    OPT_H, "hausman-reg" },
+    { PANEL,    OPT_O, "vcv" },
+    { PANEL,    OPT_Q, "quiet" },
+    { PANEL,    OPT_R, "robust" },
+    { PANEL,    OPT_S, "silent" },
+    { PANEL,    OPT_T, "iterate" },
+    { PANEL,    OPT_U, "random-effects" },
+    { PANEL,    OPT_V, "verbose" },
+    { PANEL,    OPT_W, "unit-weights" },
     { POISSON,  OPT_V, "verbose" },
-    { POOLED,   OPT_T, "iterate" },
-    { POOLED,   OPT_V, "verbose" },
-    { POOLED,   OPT_W, "unit-weights" },
     { PCA,      OPT_A, "save-all" },
     { PCA,      OPT_O, "save" },
     { PERGM,    OPT_O, "bartlett" },
     { PLOT,     OPT_O, "one-scale" },
     { PRINT,    OPT_O, "byobs" },
+    { PRINT,    OPT_L, "long" },
     { PRINT,    OPT_N, "no-dates" },
     { PRINT,    OPT_T, "ten" },
     { PROBIT,   OPT_R, "robust" },
     { QUIT,     OPT_X, "exit" },
+    { RESTRICT, OPT_Q, "quiet" },
+    { SCATTERS, OPT_L, "with-lines" },
+    { SETINFO,  OPT_C, "continuous" },
+    { SETINFO,  OPT_D, "discrete" },
     { SETOBS,   OPT_C, "stacked-cross-section" },
+    { SETOBS,   OPT_P, "panel-vars" },
+    { SETOBS,   OPT_R, "restructure" },
     { SETOBS,   OPT_S, "stacked-time-series" },
     { SETOBS,   OPT_T, "time-series" },
     { SETOBS,   OPT_X, "cross-section" },
@@ -179,6 +208,7 @@ struct gretl_option gretl_opts[] = {
     { STORE,    OPT_D, "database" },
     { STORE,    OPT_F, "overwrite" },
     { STORE,    OPT_G, "dat" },
+    { STORE,    OPT_J, "jmulti" },
     { STORE,    OPT_M, "gnu-octave" },
     { STORE,    OPT_R, "gnu-R" },
     { STORE,    OPT_T, "traditional" },
@@ -208,9 +238,66 @@ struct gretl_option gretl_opts[] = {
     { VECM,     OPT_V, "verbose" },
     { WLS,      OPT_R, "robust" },    
     { WLS,      OPT_Q, "quiet" },
+    { XTAB,     OPT_C, "column" },
+    { XTAB,     OPT_R, "row" },
+    { XTAB,     OPT_X, "chi-square" },
+    { XTAB,     OPT_Z, "zeros" },
     { 0,        0L,    NULL }
 };
 
+static int compare_strings (const void *a, const void *b)
+{
+    const char **sa = (const char **) a;
+    const char **sb = (const char **) b;
+     
+    return strcmp(*sa, *sb);
+}
+
+char **get_all_option_strings (int *pn)
+{
+    char **optstrs;
+    int i, j, m, n = 0;
+
+    for (i=0; gretl_opts[i].ci != 0; i++) {
+	n++;
+    }
+
+    optstrs = strings_array_new(n);
+
+    if (optstrs != NULL) {
+	for (i=0; i<n; i++) {
+	    optstrs[i] = gretl_strdup(gretl_opts[i].longopt);
+	    if (optstrs[i] == NULL) {
+		free_strings_array(optstrs, n);
+		optstrs = NULL;
+		break;
+	    }
+	}
+    }
+
+    if (optstrs != NULL) {
+	qsort(optstrs, n, sizeof *optstrs, compare_strings);
+	m = n;
+	for (i=0; i<m-1; i++) {
+	    if (!strcmp(optstrs[i], optstrs[i+1])) {
+		free(optstrs[i+1]);
+		for (j=i+1; j<m-1; j++) {
+		    optstrs[j] = optstrs[j+1];
+		}
+		optstrs[m-1] = NULL;
+		i--;
+		m--;
+	    }
+	}
+	if (m < n) {
+	    optstrs = realloc(optstrs, m * sizeof *optstrs);
+	}
+	*pn = m;
+    }
+    
+    return optstrs;
+}
+
 const char **get_opts_for_command (int ci, int *nopt)
 {
     int i, j, n = 0;
@@ -256,7 +343,10 @@ struct flag_match flag_matches[] = {
     { OPT_E, 'e' },
     { OPT_F, 'f' },
     { OPT_G, 'g' },
+    { OPT_H, 'h' },
     { OPT_I, 'i' },
+    { OPT_J, 'i' },
+    { OPT_K, 'k' },
     { OPT_L, 'l' },
     { OPT_M, 'm' },
     { OPT_N, 'n' },
@@ -266,6 +356,7 @@ struct flag_match flag_matches[] = {
     { OPT_R, 'r' },
     { OPT_S, 's' },
     { OPT_T, 't' },
+    { OPT_U, 'u' },
     { OPT_V, 'v' },
     { OPT_W, 'w' },
     { OPT_X, 'x' },
@@ -273,7 +364,7 @@ struct flag_match flag_matches[] = {
     { 0L,   '\0' }
 };
 
-static const char *ok_flags = "abcdefgilmnopqrstvwxz";
+static const char *ok_flags = "abcdefghijklmnopqrstuvwxz";
 
 #define isflag(c) (strchr(ok_flags, c) != NULL)
 
@@ -471,7 +562,7 @@ gretlopt get_gretl_options (char *line, int *err)
 	return oflags;
     }
 
-    /* to enable reading of trad. esl input files */
+    /* to enable reading of trad. ESL input files */
     if (line[n-2] == ';' && isspace(line[n-1])) {
 	line[n-2] = '\0';
     } else if (line[n-1] == ';') {
@@ -490,14 +581,16 @@ gretlopt get_gretl_options (char *line, int *err)
 
     /* some commands do not take a "flag", and "-%c" may have
        some other meaning */
-    if (ci == 0 || ci == GENR || ci == LABEL) {
+    if (ci == 0 || ci == GENR) {
 	return oflags;
     }
 
-    /* try for short-form options (e.g. "-o") */
-    opt = get_short_opts(line, ci, &myerr);
-    if (!myerr && opt) {
-	oflags |= opt;
+    if (ci != SETINFO) {
+	/* try for short-form options (e.g. "-o") */
+	opt = get_short_opts(line, ci, &myerr);
+	if (!myerr && opt) {
+	    oflags |= opt;
+	}
     }
 
     /* try for long-form options (e.g. "--vcv") */
