@@ -19,8 +19,10 @@
 
 #include "libgretl.h"
 
-#ifdef WIN32
+#if defined(WIN32)
 # include <windows.h>
+#elif defined(OSX_NATIVE)
+# include <mach-o/dyld.h>
 #else
 # include <dlfcn.h>
 #endif
@@ -33,7 +35,6 @@ enum {
     P_VIF,
     P_LEVERAGE,
     P_MP_OLS,
-    P_PANEL_DATA,
     P_PCA,
     P_PROGRESS_BAR,
     P_RANGE_MEAN,
@@ -52,7 +53,9 @@ enum {
     P_POISSON,
     P_MAILER,
     P_EVIEWS_IMPORT,
-    P_STATA_IMPORT
+    P_STATA_IMPORT,
+    P_JMULTI_IMPORT,
+    P_ZIPFILE
 } plugin_codes;
 
 struct plugin_info {
@@ -74,7 +77,6 @@ struct plugin_info plugins[] = {
     { P_VIF,             "vif" },
     { P_LEVERAGE,        "leverage" },
     { P_MP_OLS,          "mp_ols" },
-    { P_PANEL_DATA,      "panel_data" },
     { P_PCA,             "pca" },
     { P_PROGRESS_BAR,    "progress_bar" },
     { P_RANGE_MEAN,      "range-mean" },
@@ -94,14 +96,19 @@ struct plugin_info plugins[] = {
     { P_MAILER,          "mailer" },
     { P_EVIEWS_IMPORT,   "eviews_import" },
     { P_STATA_IMPORT,    "stata_import" },
+    { P_JMULTI_IMPORT,   "jmulti_import" },
+    { P_ZIPFILE,         "gretlzip" }
 };  
 
 struct plugin_function plugin_functions[] = { 
     /* data importers */
-    { "excel_get_data", P_EXCEL_IMPORT },
-    { "wbook_get_data", P_GNUMERIC_IMPORT },
-    { "wf1_get_data",   P_EVIEWS_IMPORT },
-    { "dta_get_data",   P_STATA_IMPORT },
+    { "excel_get_data",   P_EXCEL_IMPORT },
+    { "cli_get_xls",      P_EXCEL_IMPORT },
+    { "wbook_get_data",   P_GNUMERIC_IMPORT },
+    { "cli_get_gnumeric", P_GNUMERIC_IMPORT },
+    { "wf1_get_data",     P_EVIEWS_IMPORT },
+    { "dta_get_data",     P_STATA_IMPORT },
+    { "jmulti_get_data",  P_JMULTI_IMPORT },
 
     /* Johansen cointegration test and VECM */
     { "johansen_analysis",        P_JOHANSEN },
@@ -125,12 +132,6 @@ struct plugin_function plugin_functions[] = {
     { "mp_vector_ln",             P_MP_OLS },
 #endif
 
-    /* panel data methods */
-    { "panel_autocorr_test",      P_PANEL_DATA },
-    { "panel_diagnostics",        P_PANEL_DATA },
-    { "switch_panel_orientation", P_PANEL_DATA },
-    { "panel_wls_by_unit",        P_PANEL_DATA },
-
     /* principal components analysis */
     { "pca_from_corrmat", P_PCA },
 
@@ -178,6 +179,11 @@ struct plugin_function plugin_functions[] = {
     /* Send email */
     { "email_file",    P_MAILER },
 
+    /* zip and unzip */
+    { "gretl_make_zipfile", P_ZIPFILE},
+    { "gretl_unzip_file",   P_ZIPFILE},
+    { "gretl_is_zipfile",   P_ZIPFILE},
+
     /* sentinel */
     { NULL, 0 }
 };
@@ -198,12 +204,16 @@ static const char *get_plugin_name_for_function (const char *func)
 
 static void *get_plugin_handle (const char *plugin)
 {
+#ifdef OSX_NATIVE
+    NSObjectFileImage file;
+    NSObjectFileImageReturnCode rc;
+#endif
     char pluginpath[MAXLEN];
     void *handle = NULL;
 
     strcpy(pluginpath, gretl_lib_path());
 
-#ifdef WIN32
+#if defined(WIN32)
     append_dir(pluginpath, "plugins");
     strcat(pluginpath, plugin);
     strcat(pluginpath, ".dll");
@@ -211,6 +221,19 @@ static void *get_plugin_handle (const char *plugin)
     if (handle == NULL) {
         sprintf(gretl_errmsg, _("Couldn't load plugin %s"), pluginpath);
     }
+#elif defined(OSX_NATIVE)
+    strcat(pluginpath, plugin);
+    strcat(pluginpath, ".so");
+    rc = NSCreateObjectFileImageFromFile(pluginpath, &file);
+    if (rc == NSObjectFileImageSuccess) {
+	handle = NSLinkModule(file, pluginpath,
+			      NSLINKMODULE_OPTION_BINDNOW |
+			      NSLINKMODULE_OPTION_PRIVATE |
+			      NSLINKMODULE_OPTION_RETURN_ON_ERROR);
+    }
+    if (handle == NULL) {
+        sprintf(gretl_errmsg, _("Couldn't load plugin %s"), pluginpath);
+    }
 #else
     strcat(pluginpath, plugin);
     strcat(pluginpath, ".so");
@@ -241,8 +264,10 @@ void *get_plugin_function (const char *funcname, void **handle)
 	return NULL;
     }
 
-#ifdef WIN32
+#if defined(WIN32)
     funp = GetProcAddress(*handle, funcname);
+#elif defined(OSX_NATIVE)
+    funp = NSLookupSymbolInModule(*handle, funcname);
 #else
     funp = dlsym(*handle, funcname);
     if (funp == NULL) {
@@ -269,8 +294,10 @@ void close_plugin (void *handle)
 {
     if (handle == NULL) return;
 
-#ifdef WIN32
+#if defined(WIN32)
     FreeLibrary(handle);
+#elif defined(OSX_NATIVE)
+    NSUnLinkModule(handle, NSUNLINKMODULE_OPTION_NONE);
 #else
     dlclose(handle);
 #endif
