@@ -274,14 +274,14 @@ static void print_freq_test (const FreqDist *freq, PRN *prn)
 {
     double pval = NADBL;
 
-    if (freq->dist == DIST_NORMAL) {
-	pval = chisq(freq->test, 2);
+    if (freq->dist == D_NORMAL) {
+	pval = chisq_cdf_comp(freq->test, 2);
 	pprintf(prn, "\n%s:\n", 
 		_("Test for null hypothesis of normal distribution"));
 	pprintf(prn, "%s(2) = %.3f %s %.5f\n", 
 		_("Chi-square"), freq->test, 
 		_("with p-value"), pval);
-    } else if (freq->dist == DIST_GAMMA) {
+    } else if (freq->dist == D_GAMMA) {
 	pval = normal_pvalue_2(freq->test);
 	pprintf(prn, "\n%s:\n", 
 		_("Test for null hypothesis of gamma distribution"));
@@ -291,7 +291,7 @@ static void print_freq_test (const FreqDist *freq, PRN *prn)
 
     if (!na(pval)) {
 	record_test_result(freq->test, pval, 
-			   (freq->dist == DIST_NORMAL)? 
+			   (freq->dist == D_NORMAL)? 
 			   "normality" : "gamma");
     }
 }
@@ -332,7 +332,6 @@ void print_freq (const FreqDist *freq, PRN *prn)
     if (!freq->discrete) {
 	pprintf(prn, _("number of bins = %d, mean = %g, sd = %g\n"), 
 		freq->numbins, freq->xbar, freq->sdx);
-
 	pputs(prn, 
 	      _("\n       interval          midpt   frequency    rel.     cum.\n\n"));
     } else {
@@ -349,18 +348,18 @@ void print_freq (const FreqDist *freq, PRN *prn)
 	    } else if (k == K) {
 		pputs(prn, "          >= ");
 	    } else {
-		pprintf(prn, "%10g - ", freq->endpt[k]);
+		pprintf(prn, "%#10.5g - ", freq->endpt[k]);
 	    }
 	    if (k == K) {
-		sprintf(word, "%g", freq->endpt[k]);
+		sprintf(word, "%#.5g", freq->endpt[k]);
 	    } else {
-		sprintf(word, "%g", freq->endpt[k+1]);
+		sprintf(word, "%#.5g", freq->endpt[k+1]);
 	    }
 
 	    pprintf(prn, "%s", word);
 	    nlw = 10 - strlen(word);
 	    bufspace(nlw, prn);
-	    sprintf(word, " %g", freq->midpt[k]);
+	    sprintf(word, " %#.5g", freq->midpt[k]);
 	}
 
 	pputs(prn, word);
@@ -381,10 +380,8 @@ void print_freq (const FreqDist *freq, PRN *prn)
     missing = total - valid;
 
     if (missing > 0) {
-	pputc(prn, '\n');
-	pprintf(prn, _("Invalid cases = %d (%5.2f%%)"), missing, 
-		100 * (double) missing / total);
-	pputc(prn, '\n');
+	pprintf(prn, "\n%s = %d (%5.2f%%)\n", _("Missing observations"), 
+		missing, 100 * (double) missing / total);
     }
 
     if (!na(freq->test)) {
@@ -392,6 +389,115 @@ void print_freq (const FreqDist *freq, PRN *prn)
     }
 }
 
+/**
+ * print_xtab:
+ * @tab: gretl cross-tabulation struct.
+ * @prn: gretl printing struct.
+ *
+ * Print crosstab to @prn.
+ */
+
+void print_xtab (const Xtab *tab, gretlopt opt, PRN *prn)
+{
+    int i, j, r, c;
+    r = tab->rows;
+    c = tab->cols;
+    double x, y;
+    int n5 = 0;
+    double pearson = 0.0;
+
+    pputc(prn, '\n');
+    pprintf(prn, _("Cross-tabulation of %s (rows) against %s (columns)"),
+	    tab->rvarname, tab->cvarname);
+
+    pputs(prn, "\n\n       ");
+    for (j=0; j<c; j++) {
+	pprintf(prn, "[%4d]", (tab->cval)[j]);
+    } 
+
+    pprintf(prn,"  %s\n  \n", _("TOT."));
+
+    for (i=0; i<r; i++) {
+
+	if (tab->rtotal[i]) {
+	    pprintf(prn,"[%4d] ",(tab->rval)[i]);
+
+	    for (j=0; j<c; j++) {
+		if (tab->ctotal[j]) {
+		    if ((tab->f)[i][j] || (opt & OPT_Z)) {
+			if (opt & (OPT_C | OPT_R)) {
+			    if (opt & OPT_C) {
+				x = 100.0 * (tab->f)[i][j] / tab->ctotal[j];
+			    } else {
+				x = 100.0 * (tab->f)[i][j] / tab->rtotal[i];
+			    }
+			    pprintf(prn, "%5.1f%%", x);
+			} else {
+			    pprintf(prn, "%5d ", (tab->f)[i][j]);
+			}
+		    } else {
+			pputs(prn,"      ");
+		    }
+		} 
+
+		if (!na(pearson)) {
+		    y = (double) (tab->rtotal[i] * tab->ctotal[j]) / tab->n;
+		    x = (double) (tab->f)[i][j] - y;
+		    if (y < 1.0e-7) {
+			fprintf(stderr, "Error computing chi2 test: "
+				"expected n in cell (%d,%d) = %g\n",
+				i, j, y);
+			pearson = NADBL;
+		    } else {
+			pearson += x * x / y;
+			if (y >= 5) n5++;
+		    }
+		}
+	    }
+
+	    if (opt & OPT_C) {
+		x = 100.0 * tab->rtotal[i] / tab->n;
+		pprintf(prn, "%5.1f%%\n", x);
+	    } else {
+		pprintf(prn, "%6d\n", tab->rtotal[i]);
+	    }
+	}
+    }
+
+    pputc(prn, '\n');
+    pputs(prn, _("TOTAL  "));
+
+    for (j=0; j<c; j++) {
+	if (opt & OPT_R) {
+	    x = 100.0 * tab->ctotal[j] / tab->n;
+	    pprintf(prn, "%5.1f%%", x);
+	} else {
+	    pprintf(prn, "%5d ", tab->ctotal[j]);
+	}
+    }
+    
+    pprintf(prn, "%6d\n", tab->n);
+
+    if (tab->missing) {
+	pputc(prn, '\n');
+	pprintf(prn, _("%d missing values"), tab->missing);
+	pputc(prn, '\n');
+    }
+
+    if (!na(pearson)) {
+	double n5p = (double) n5 / (r * c);
+	int df;
+
+	if (n5p >= .80 || (opt & OPT_X)) {
+	    df = (r - 1) * (c - 1);
+	    pputc(prn, '\n');
+	    pprintf(prn, _("Pearson chi-square test = %g (%d df, p-value = %g)"), 
+		    pearson, df, chisq_cdf_comp(pearson, df));
+	    pputc(prn, '\n');
+	}
+    }
+}
+
 /**
  * print_smpl:
  * @pdinfo: data information struct
@@ -435,6 +541,15 @@ void print_smpl (const DATAINFO *pdinfo, int fulln, PRN *prn)
     } else {
 	pputc(prn, '\n');
     } 
+
+    if (dataset_is_panel(pdinfo)) {
+	pprintf(prn, "%s\n", _("panel"));
+    } 
+#if 0
+    else if (dataset_is_time_series(pdinfo)) {
+	pprintf(prn, "%s\n", _("time series"));
+    }
+#endif
 }
 
 /**
@@ -808,9 +923,10 @@ static void fit_resid_head (const FITRESID *fr,
     ntodate(mdate2, fr->t2, pdinfo);
 
     pprintf(prn, _("Model estimation range: %s - %s"), mdate1, mdate2);
+    pputc(prn, '\n');
 
     if (!na(fr->sigma)) {
-	pprintf(prn, _("Standard error of residuals = %f\n"), fr->sigma);
+	pprintf(prn, _("Standard error of residuals = %g\n"), fr->sigma);
     }
     
     pprintf(prn, "\n     %s ", _("Obs"));
@@ -888,15 +1004,15 @@ static void fcast_print_x (double x, int n, int pmax, PRN *prn)
     }
 }
 
-static void printstr_ten (PRN *prn, double xx, int *ls)
+static void printstr_long (PRN *prn, double xx, int d, int *ls)
 {
     int lwrd;    
-    char str[32];
+    char str[64];
 
     if (na(xx)) {
 	strcpy(str, "NA");
     } else {
-	sprintf(str, "%.10g", xx);
+	sprintf(str, "%#.*E", d, xx);
     }
     strcat(str, "  ");
     lwrd = strlen(str);
@@ -933,19 +1049,25 @@ static void printstr (PRN *prn, double xx, int *ls)
 static void printz (const double *z, const DATAINFO *pdinfo, 
 		    PRN *prn, gretlopt opt)
 {
-    int t, t1 = pdinfo->t1, t2 = pdinfo->t2, ls = 0;
+    int t1 = pdinfo->t1, t2 = pdinfo->t2;
+    int t, dig = 10, ls = 0;
     double xx;
 
+    if (opt & OPT_L) {
+	dig = get_long_digits();
+	opt = OPT_T;
+    }
+
     if (gretl_isconst(t1, t2, z)) {
 	if (opt & OPT_T) {
-	    printstr_ten(prn, z[t1], &ls);
+	    printstr_long(prn, z[t1], dig, &ls);
 	} else {
 	    printstr(prn, z[t1], &ls);
 	}
     } else for (t=t1; t<=t2; t++) {
 	xx = z[t];
 	if (opt & OPT_T) {
-	    printstr_ten(prn, xx, &ls);
+	    printstr_long(prn, xx, dig, &ls);
 	} else {
 	    printstr(prn, xx, &ls);
 	}
@@ -1349,8 +1471,10 @@ static void print_scalar (double x, const char *vname,
 	if (x >= 0.0) {
 	    pputc(prn, ' ');
 	}
-	if (opt & OPT_T) {
-	    pprintf(prn, "%.10e", x);
+	if (opt & OPT_L) {
+	    pprintf(prn, "%#.*E", get_long_digits(), x);
+	} else if (opt & OPT_T) {
+	    pprintf(prn, "%#.10E", x);
 	} else {
 	    pprintf(prn, "%#.6g", x);
 	}
@@ -1368,14 +1492,36 @@ int get_printdata_blocks (void)
     return printdata_blocks;
 }
 
+static int adjust_print_list (int *list, int *screenvar,
+			      gretlopt opt)
+{
+    int pos;
+
+    if (!(opt & OPT_O)) {
+	return E_PARSE;
+    }
+
+    pos = gretl_list_separator_position(list);
+
+    if (list[0] < 3 || pos != list[0] - 1) {
+	return E_PARSE;
+    } else {
+	*screenvar = list[list[0]];
+	list[0] = pos - 1;
+    }
+
+    return 0;
+}
+
 /**
  * printdata:
  * @list: list of variables to print.
  * @Z: data matrix.
  * @pdinfo: data information struct.
- * @opt: if OPT_O, print the data by observation (series in columns);
- *       if OPT_N, use simple obs numbers, not dates; 
- *       if OPT_T, print the data to 10 significant digits.
+ * @opt: if %OPT_O, print the data by observation (series in columns);
+ * if %OPT_N, use simple obs numbers, not dates; if %OPT_T, print the 
+ * data to 10 significant digits; if %OPT_L, print the data to a number 
+ * of digits set by "set longdigits" (default 10).
  * @prn: gretl printing struct.
  *
  * Print the data for the variables in @list, from observations t1 to
@@ -1388,6 +1534,7 @@ int printdata (const int *list, const double **Z, const DATAINFO *pdinfo,
 	       gretlopt opt, PRN *prn)
 {
     int j, v, v1, v2, jc, nvjc, lineno, ncol;
+    int screenvar = 0;
     int allconst, scalars = 0;
     int nvars = 0, sortvar = 0;
     int maxlen = 0, bplen = 13;
@@ -1419,13 +1566,21 @@ int printdata (const int *list, const double **Z, const DATAINFO *pdinfo,
 	}
     }
 
+    if (gretl_list_has_separator(plist)) {
+	err = adjust_print_list(plist, &screenvar, opt);
+	if (err) {
+	    free(plist);
+	    return err;
+	}
+    }
+
     lineno = 1;
 
     /* screen out any scalars and print them first */
     for (j=1; j<=plist[0]; j++) {
 	int len, v = plist[j];
 
-	if (!pdinfo->vector[v]) {
+	if (var_is_scalar(pdinfo, v)) {
 	    print_scalar(Z[v][0], pdinfo->varname[v], opt, 0, prn);
 	    scalars = 1;
 	    gretl_list_delete_at_pos(plist, j);
@@ -1522,6 +1677,11 @@ int printdata (const int *list, const double **Z, const DATAINFO *pdinfo,
 
 	    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 
+		if (screenvar && Z[screenvar][t] == 0.0) {
+		    /* screened out by boolean */
+		    continue;
+		}
+
 		if (sortvar && plist[0] == 1) {
 		    strcpy(obs_string, SORTED_MARKER(pdinfo, sortvar, t));
 		} else if (opt & OPT_N) {
@@ -1618,7 +1778,7 @@ int print_data_sorted (const int *list, const int *obsvec,
 
     /* ...with no scalars or bad variable numbers */
     for (i=1; i<=list[0]; i++) {
-	if (list[i] >= pdinfo->v || !pdinfo->vector[list[i]]) {
+	if (list[i] >= pdinfo->v || var_is_scalar(pdinfo, list[i])) {
 	    return E_DATA;
 	}
     }
@@ -1752,7 +1912,7 @@ int text_print_forecast (const FITRESID *fr,
 			 double ***pZ, DATAINFO *pdinfo, 
 			 gretlopt opt, PRN *prn)
 {
-    int t, pv, err = 0;
+    int t, err = 0;
     int do_errs = (fr->sderr != NULL);
     int pmax = fr->pmax;
     int errpmax = fr->pmax;
@@ -1833,11 +1993,12 @@ int text_print_forecast (const FITRESID *fr,
     /* do we really want a plot for non-time series? */
 
     if (plot && fr->nobs > 0) {
-	pv = plotvar(pZ, pdinfo);
-	if (pv < 0) {
+	const double *obs = gretl_plotx(pdinfo);
+
+	if (obs == NULL) {
 	    err = 1;
 	} else {
-	    err = plot_fcast_errs(fr->t0, fr->t2, (*pZ)[pv], 
+	    err = plot_fcast_errs(fr->t0, fr->t2, obs, 
 				  fr->actual, fr->fitted, maxerr, 
 				  fr->depvar, 
 				  (time_series)? pdinfo->pd : 0);
@@ -1883,8 +2044,20 @@ int print_fit_resid (const MODEL *pmod, const double **Z,
 
 #define PRINTF_DEBUG 0
 
-#define is_format_char(c) (c == 'f' || c == 'g' || c == 'd' || c == 's')
-#define numeric_conv(c)   (c == 'f' || c == 'g' || c == 'd')
+#define is_format_char(c) (c == 'e' || \
+                           c == 'E' || \
+                           c == 'f' || \
+                           c == 'g' || \
+                           c == 'G' || \
+                           c == 'd' || \
+                           c == 's')
+
+#define numeric_conv(c) (c == 'e' || \
+                         c == 'E' || \
+                         c == 'f' || \
+                         c == 'g' || \
+                         c == 'G' || \
+			 c == 'd')
 
 static int print_arg (const char **pfmt, double val, 
 		      const char *str, PRN *prn)
@@ -1901,7 +2074,7 @@ static int print_arg (const char **pfmt, double val,
 	fmt[2] = '\0';
 	n = 2;
     } else {
-	sscanf(*pfmt + 1, "%14[^gfsd]", fmt + 1);
+	sscanf(*pfmt + 1, "%14[^eEfgGsd]", fmt + 1);
 	n = strlen(fmt);
 	fc = *(*pfmt + n);
 	fmt[n] = fc;
@@ -2159,7 +2332,7 @@ static int real_do_printf (const char *line, double ***pZ,
     /* play safe with sizes here */
     xvals = malloc(argc * sizeof *xvals);
     str = malloc(strlen(line) + 1);
-    svals = create_strings_array(argc);
+    svals = strings_array_new(argc);
 
     if (xvals == NULL || svals == NULL || str == NULL) {
 	err = E_ALLOC;
@@ -2202,10 +2375,10 @@ static int real_do_printf (const char *line, double ***pZ,
 	    if (v < pdinfo->v) {
 #if PRINTF_DEBUG
 		fprintf(stderr, "'%s' is variable #%d (vector = %d)\n",
-			argv, v, pdinfo->vector[v]);
+			argv, v, var_is_series(pdinfo, v));
 #endif
 		/* simple existent varname */
-		if (pdinfo->vector[v]) {
+		if (var_is_series(pdinfo, v)) {
 		    xvals[i] = (*pZ)[v][t];
 		} else {
 		    xvals[i] = (*pZ)[v][0];
