@@ -44,31 +44,47 @@ const char negval[] = N_("\nEnter x value (value < 0 will exit menu): ");
  * @p: probability of success on each trial.
  *
  * Returns: the probability of @k or less successes on
- * @n trials given binomial probability @p.
+ * @n trials given binomial probability @p, or
+ * #NADBL on failure.
  */
 
 double binomial_cdf (int k, int n, double p)
 {
-    double x = bdtr(k, n, p);
+    double x = NADBL;
 
-    return (isnan(x))? NADBL : x;
+    if (p >= 0.0 && n >= 0 && k >= 0) {
+	x = bdtr(k, n, p);
+	if (get_cephes_errno()) {
+	    x = NADBL;
+	}
+    }
+
+    return x;
 }
 
 /**
- * binomial_pvalue:
- * @k: number of successes.
+ * binomial_cdf_comp:
+ * @k: maximum number of successes.
  * @n: number of trials.
  * @p: probability of success on each trial.
  *
  * Returns: the probability of @k + 1 or more successes on
- * @n trials given binomial probability @p.
+ * @n trials given binomial probability @p, or
+ * #NADBL on failure.
  */
 
-double binomial_pvalue (int k, int n, double p)
+double binomial_cdf_comp (int k, int n, double p)
 {
-    double x = bdtrc(k, n, p);
+    double x = NADBL;
 
-    return (isnan(x))? NADBL : x;
+    if (p >= 0.0 && n >= 0 && k >= 0) {
+	x = bdtrc(k, n, p);
+	if (get_cephes_errno()) {
+	    x = NADBL;
+	}
+    }
+
+    return x;
 }
 
 /**
@@ -88,6 +104,9 @@ double x_factorial (double x)
 	fact = NADBL;
     } else if (x > 12.0) {
 	fact = cephes_gamma(1.0 + x);
+	if (get_cephes_errno()) {
+	    fact = NADBL;
+	}
     } else if (n == 0) {
 	fact = 1.0;
     } else {
@@ -100,17 +119,60 @@ double x_factorial (double x)
     return fact;
 }
 
+/**
+ * log_x_factorial:
+ * @x: input value.
+ * 
+ * Returns: the log of the factorial of int(x), cast to a double, or
+ * NADBL on failure.
+ */
+
+double log_x_factorial (double x)
+{
+    double lfact;
+    int n = x;
+
+    if (x < 0.0) {
+	lfact = NADBL;
+    } else if (x > 12.0) {
+	lfact = cephes_lgamma(1.0 + x);
+	if (get_cephes_errno()) {
+	    lfact = NADBL;
+	}
+    } else if (n == 0) {
+	lfact = 0.0;
+    } else {
+	lfact = n;
+	while (--n > 1) {
+	    lfact *= n;
+	}
+	lfact = log(lfact);
+    }
+
+    return lfact;
+}
+
 /**
  * tcrit95:
  * @df: degrees of freedom.
  * 
  * Returns: the two-sided 95 percent critical value for the t 
- * distribution with @df degrees of freedom.
+ * distribution with @df degrees of freedom, or #NADBL on
+ * failure.
  */
 
 double tcrit95 (int df)
 {
-    return stdtri(df, 0.975);
+    double x = NADBL;
+
+    if (df > 0) {
+	x = stdtri(df, 0.975);
+	if (get_cephes_errno()) {
+	    x = NADBL;
+	}
+    }
+
+    return x;
 }
 
 /**
@@ -118,14 +180,26 @@ double tcrit95 (int df)
  * @n: sample size.
  * 
  * Returns: the two-sided 95 percent critical value for the sample
- * correlation coefficient, sample size @n.
+ * correlation coefficient, sample size @n, or #NADBL on
+ * failure.
  */
 
 double rhocrit95 (int n)
 {
-    double x = stdtri(n - 2, 0.975);
+    double x = NADBL;
+
+    if (n - 2 > 0) {
+	x = stdtri(n - 2, 0.975);
+	if (get_cephes_errno()) {
+	    x = NADBL;
+	} else {
+	    double x2 = x * x;
+
+	    x = sqrt(x2 / (x2 - 2 + n));
+	}
+    }
     
-    return sqrt(x*x / (x*x - 2 + n));
+    return x;
 }
 
 /**
@@ -141,7 +215,9 @@ double rhocrit95 (int n)
 
 double normal_pvalue_2 (double x)
 {
-    return 2.0 * (1.0 - ndtr(fabs(x)));
+    double p = (x < 0.0)? ndtr(x) : ndtr(-x);
+
+    return 2.0 * p;
 }
 
 /**
@@ -161,80 +237,232 @@ double normal_pvalue_1 (double x)
     return 1.0 - ndtr(x);
 }
 
+/**
+ * t_cdf:
+ * @x: the cutoff point in the distribution.
+ * @df: degrees of freedom.
+ * 
+ * Returns: the integral from minus infinity to @x of
+ * the t distribution with @df degrees of freedom, or
+ * #NADBL on failure.
+ */
+
+double t_cdf (double x, int df)
+{
+    double p = NADBL;
+
+    if (df > 0) {
+	p = stdtr(df, x);
+	if (get_cephes_errno()) {
+	    p = NADBL;
+	}
+    }
+
+    return p;
+}
+
+/**
+ * t_cdf_comp:
+ * @x: the cutoff point in the distribution.
+ * @df: degrees of freedom.
+ * 
+ * Returns: the integral from @x to infinity of
+ * the t distribution with @df degrees of freedom, or
+ * #NADBL on failure.
+ */
+
+double t_cdf_comp (double x, int df)
+{
+    double p = NADBL;
+
+    if (df > 0) {
+	p = stdtr(df, x);
+	if (get_cephes_errno()) {
+	    p = NADBL;
+	} else {
+	    p = 1.0 - p;
+	}
+    }
+
+    return p;
+}
+
 /**
  * t_pvalue_2:
  * @x: the cutoff point in the distribution.
  * @df: degrees of freedom.
  * 
  * Returns: the probability that t(@df) is greater than @x
- * (two-sided, using the absolute value of @x).
+ * (two-sided, using the absolute value of @x), or
+ * #NADBL on failure.
  */
 
 double t_pvalue_2 (double x, int df)
 {
-    double ret = -1.0;
+    double p = NADBL;
 
     if (df > 0) {
-	double s = stdtr(df, fabs(x));
-	
-	ret = 2.0 * (1.0 - s);
-	if (ret < 0.0) {
-	    ret = 0.0;
+	if (x < 0.0) {
+	    p = stdtr(df, x);
+	} else {
+	    p = stdtr(df, -x);
+	}
+	if (get_cephes_errno()) {
+	    p = NADBL;
+	} else {
+	    p *= 2.0;
 	}
     }
 
-    return ret;
+    return p;
 }
 
 /**
- * fdist:
+ * chisq_cdf:
+ * @x: the cutoff point in the distribution.
+ * @df: degrees of freedom.
+ * 
+ * Returns: the integral from 0 to @x of the chi-squared
+ * distribution with @df degrees of freedom, or #NADBL
+ * on failure.
+ */
+
+double chisq_cdf (double x, int df)
+{
+    double p = NADBL;
+
+    if (df > 0 && x >= 0.0) {
+	p = chdtr(df, x);
+	if (get_cephes_errno()) {
+	    p = NADBL;
+	}
+    }
+
+    return p;
+}
+
+/**
+ * chisq_cdf_comp:
+ * @x: the cutoff point in the distribution.
+ * @df: degrees of freedom.
+ * 
+ * Returns: the integral from @x to infinity of the chi-squared
+ * distribution with @df degrees of freedom, or #NADBL
+ * on failure.
+ */
+
+double chisq_cdf_comp (double x, int df)
+{
+    double p = NADBL;
+
+    if (df > 0 && x >= 0.0) {
+	p = chdtrc(df, x);
+	if (get_cephes_errno()) {
+	    p = NADBL;
+	}
+    }
+
+    return p;
+}
+
+/**
+ * chisq_critval:
+ * @a: tail probability.
+ * @df: degrees of freedom.
+ * 
+ * Returns: the Chi-square argument x such that the integral
+ * from x to infinity of the Chi-square density is equal
+ * to the given cumulative probability @a, or #NADBL
+ * on failure.
+ */
+
+double chisq_critval (double a, int df)
+{
+    double x = NADBL;
+
+    if (df > 0 && a >= 0.0) {
+	x = chdtri(df, a);
+	if (get_cephes_errno()) {
+	    x = NADBL;
+	}
+    }
+
+    return x;
+}
+
+/**
+ * f_cdf:
  * @x: the cutoff point in the distribution.
  * @dfn: numerator degrees of freedom.
  * @dfd: denominator degrees of freedom.
  * 
- * Returns: the probability a random variable distributed as
- * F(@dfn, @dfd) is greater than @x, or -1 if either @dfn or @dfd is
- * negative.
+ * Returns: the integral of the F distribution with @dfn and
+ * @dfd degrees of freedom, from 0 to @x, or #NADBL on failure.
  */
 
-double fdist (double x, int dfn, int dfd)
+double f_cdf (double x, int dfn, int dfd)
 {
-    double ret = 1.0;
+    double p = NADBL;
 
-    if (dfn < 1 || dfd < 1) {
-        ret = -1.0;
-    } else if (x > 0.0) {
-	if (0 && dfd > 300) {
-	    /* FIXME igamc underflow ?? */
-	    ret = chisq(x * dfn, dfn);
-	} else {
-	    ret = fdtrc(dfn, dfd, x);
+    if (dfn > 0 && dfd > 0 && x >= 0.0) {
+	p = fdtr(dfn, dfd, x);
+	if (get_cephes_errno()) {
+	    p = NADBL;
 	}
     }
 
-    return ret;
+    return p;
 }
 
 /**
- * chisq:
+ * f_cdf_comp:
  * @x: the cutoff point in the distribution.
- * @df: degrees of freedom.
+ * @dfn: numerator degrees of freedom.
+ * @dfd: denominator degrees of freedom.
  * 
- * Returns: the probability that a random variable distributed as
- * Chi-squared(@df) is greater than @x.
+ * Returns: the integral of the F distribution with @dfn and
+ * @dfd degrees of freedom, from @x to infinity, or #NADBL 
+ * on failure.
  */
 
-double chisq (double x, int df)
+double f_cdf_comp (double x, int dfn, int dfd)
 {
-    double ret = 1.0;
+    double p = NADBL;
 
-    if (df < 0) {
-	ret = -1.0;
-    } else if (x > 0.0) {
-	ret = chdtrc(df, x);
+    if (dfn > 0 && dfd > 0 && x >= 0.0) {
+	p = fdtrc(dfn, dfd, x);
+	if (get_cephes_errno()) {
+	    p = NADBL;
+	}
     }
 
-    return ret;
+    return p;
+}
+
+/**
+ * f_critval:
+ * @a: tail probability.
+ * @dfn: numerator degrees of freedom.
+ * @dfd: denominator degrees of freedom.
+ * 
+ * Returns: the F argument x such that the integral
+ * from x to infinity of the F density is equal
+ * to the given cumulative probability @a, or #NADBL
+ * on failure.
+ */
+
+double f_critval (double a, int dfn, int dfd)
+{
+    double x = NADBL;
+
+    if (dfn > 0 && dfd > 0 && a >= 0.0) {
+	x = fdtri(dfn, dfd, a);
+	if (get_cephes_errno()) {
+	    x = NADBL;
+	}
+    }
+
+    return x;
 }
 
 /**
@@ -285,7 +513,7 @@ static double get_number_or_val (const char *s,
     } else {
 	int v = varindex(pdinfo, s);
 
-	if (v > 0 && v < pdinfo->v && !pdinfo->vector[v]) {
+	if (v > 0 && v < pdinfo->v && var_is_scalar(pdinfo, v)) {
 	    return Z[v][0];
 	}
     } 
@@ -304,7 +532,7 @@ static int val_from_string (char *s, const double **Z,
 	int v = varindex(pdinfo, s);
 
 	if (v < pdinfo->v) {
-	    if (pdinfo->vector[v]) {
+	    if (var_is_series(pdinfo, v)) {
 		x = Z[v][pdinfo->t1];
 	    } else {
 		x = Z[v][0];
@@ -369,65 +597,121 @@ static char normalize_stat (char c)
     return 0;
 }
 
-static double find_pvalue (char st, int n[3], double x[3], PRN *prn)
+static double find_cdf (char st, int n[3], double x[3])
 {
-    double tmp, pv = NADBL;
+    double p = NADBL;
+
+    if (st == 'z') {
+	p = normal_cdf(x[0]);
+    } else if (st == 't') {
+	p = t_cdf(x[1], n[0]);
+    } else if (st == 'X') {
+	p = chisq_cdf(x[1], n[0]);
+    } else if (st == 'F') {
+	p = f_cdf(x[2], n[0], n[1]);
+    } else if (st == 'G') {
+	p = 1.0 - gamma_cdf_comp(x[0], x[1], x[2], 2);
+    } else if (st == 'B') {
+	p = binomial_cdf(n[2], n[1], x[0]);
+    }
+
+    return p;
+}
+
+static double find_pvalue (char st, int n[3], double x[3])
+{
+    double p = NADBL;
+
+    if (st == 'z') {
+	p = 1.0 - normal_cdf(x[0]);
+    } else if (st == 't') {
+	p = t_cdf_comp(x[1], n[0]);
+    } else if (st == 'X') {
+	p = chisq_cdf_comp(x[1], n[0]);
+    } else if (st == 'F') {
+	p = f_cdf_comp(x[2], n[0], n[1]);
+    } else if (st == 'G') {
+	p = gamma_cdf_comp(x[0], x[1], x[2], 2);
+    } else if (st == 'B') {
+	p = binomial_cdf_comp(n[2], n[1], x[0]);
+    }
+
+    return p;
+}
+
+static void 
+print_pv_string (double x, double p, int left, PRN *prn)
+{
+    if (left) {
+	if (p == 1.0) {
+	    pprintf(prn, _("area to the left of %g =~ %g\n"), x, p);
+	} else {
+	    pprintf(prn, _("area to the left of %g = %g\n"), x, p);
+	}
+    } else {
+	if (p == 1.0) {
+	    pprintf(prn, _("area to the right of %g =~ %g\n"), x, p);
+	} else {
+	    pprintf(prn, _("area to the right of %g = %g\n"), x, p);
+	}
+    }
+}
+
+static double 
+find_and_print_pvalue (char st, int n[3], double x[3], PRN *prn)
+{
+    double pv = NADBL;
 
     switch (st) {
 
     case 'z':
-	tmp = x[0];
-	if (x[0] > 0.0) tmp = -tmp;
-	pv = normal_cdf(tmp);
-	if (pv < 0) {
-	    pv = NADBL;
-	} else if (!na(pv)) {	
-	    pprintf(prn, _("\nStandard normal: area to the %s "
-			   "of %g = %g\n"), (x[0] > 0)? _("right"): _("left"), 
-		    x[0], pv);
+	if (x[0] < 0.0) {
+	    pv = normal_cdf(x[0]);
+	} else {
+	    pv = normal_cdf(-x[0]);
+	}
+	if (!na(pv)) {	
+	    pprintf(prn, "\n%s: ", _("Standard normal"));
+	    print_pv_string(x[0], pv, (x[0] < 0), prn);
 	    pprintf(prn, _("(two-tailed value = %g; complement = %g)\n"), 
 		    2.0 * pv, 1.0 - 2.0 * pv);
 	}
 	break;
 
     case 't':
-	pv = t_pvalue_2(x[1], n[0]);
-	if (pv < 0) {
-	    pv = NADBL;
-	} else if (!na(pv)) {
-	    pv *= 0.5;
-	    pprintf(prn, _("\nt(%d): area to the %s of %g = %g\n"), 
-		    n[0], (x[1] > 0)? _("right"): _("left"),
-		    x[1], pv);
+	if (x[1] < 0.0) {
+	    pv = t_cdf(x[1], n[0]);
+	} else {
+	    pv = t_cdf_comp(x[1], n[0]);
+	}
+	if (!na(pv)) {
+	    pprintf(prn, "\nt(%d): ", n[0]);
+	    print_pv_string(x[1], pv, (x[1] < 0), prn);
 	    pprintf(prn, _("(two-tailed value = %g; complement = %g)\n"), 
 		    2.0 * pv, 1.0 - 2.0 * pv);
 	}
 	break;
 
     case 'X':
-	pv = chisq(x[1], n[0]);
-	if (pv < 0) {
-	    pv = NADBL;
-	} else if (!na(pv)) {
-	    pprintf(prn, _("\nChi-square(%d): area to the right of %g = %g\n"), 
-		    n[0], x[1], pv);
-	    pprintf(prn, _("(to the left: %g)\n"), 1.0 - pv);
+	pv = chisq_cdf_comp(x[1], n[0]);
+	if (!na(pv)) {
+	    pprintf(prn, "\n%s(%d): ", _("Chi-square"), n[0]);
+	    print_pv_string(x[1], pv, 0, prn);
+	    pprintf(prn, _("(to the left: %g)\n"), chisq_cdf(x[1], n[0]));
 	}
 	break;
 
     case 'F':
-	pv = fdist(x[2], n[0], n[1]);
-	if (pv < 0) {
-	    pv = NADBL;
-	} else if (!na(pv)) {
-	    pprintf(prn, _("\nF(%d, %d): area to the right of %g = %g\n"), 
-		    n[0], n[1], x[2], pv);
-	    pprintf(prn, _("(to the left: %g)\n"), 1.0 - pv);
+	pv = f_cdf_comp(x[2], n[0], n[1]);
+	if (!na(pv)) {
+	    pprintf(prn, "\nF(%d, %d): ", n[0], n[1]);
+	    print_pv_string(x[2], pv, 0, prn);
+	    pprintf(prn, _("(to the left: %g)\n"), f_cdf(x[2], n[0], n[1]));
 	}
 	break;
 
     case 'G':
-	pv = gamma_dist(x[0], x[1], x[2], 2);
+	pv = gamma_cdf_comp(x[0], x[1], x[2], 2);
 	if (pv < 0) {
 	    pv = NADBL;
 	} else if (!na(pv)) {
@@ -439,13 +723,18 @@ static double find_pvalue (char st, int n[3], double x[3], PRN *prn)
 	break;
 
     case 'B':
-	pv = binomial_pvalue(n[2], n[1], x[0]);
-	if (pv < 0) {
-	    pv = NADBL;
-	} else if (!na(pv)) {
+	pv = binomial_cdf_comp(n[2], n[1], x[0]);
+	if (!na(pv)) {
+	    double pc = binomial_cdf(n[2], n[1], x[0]);
+
 	    pprintf(prn, _("\nBinomial (p = %g, n = %d):"
 			   "\n Prob(x > %d) = %g\n"), 
 		    x[0], n[1], n[2], pv);
+	    pprintf(prn, _(" Prob(x <= %d) = %g\n"), n[2], pc);
+	    if (n[2] > 0) {
+		pprintf(prn, _(" Prob(x = %d) = %g\n"), n[2],
+			pc - binomial_cdf(n[2] - 1, n[1], x[0]));
+	    }		
 	}
 	break;
 
@@ -468,8 +757,9 @@ static double find_pvalue (char st, int n[3], double x[3], PRN *prn)
  * @Z: the data matrix.
  * @pdinfo: data information struct.
  * @prn: gretl printing struct.
- * @opt: OPT_G (for use within genr) forces uses of '.' as
- * decimal separator.
+ * @opt: (for internal use) %OPT_G forces uses of '.' as
+ * decimal separator; %OPT_C actually evaluate cdf instead of
+ * p-value.
  * 
  * Returns: the probability that a random variable distributed as
  * specified in the command line @str exceeds the value indicated
@@ -552,7 +842,13 @@ double batch_pvalue (const char *str,
 	return NADBL;
     }
 
-    pv = find_pvalue(st, n, x, prn);
+    if (opt & OPT_C) {
+	pv = find_cdf(st, n, x);
+    } else if (opt & OPT_G) {
+	pv = find_pvalue(st, n, x);
+    } else {
+	pv = find_and_print_pvalue(st, n, x, prn);
+    }
 
     if (na(pv)) {
 	pputs(prn, _("\nError computing pvalue\n"));
@@ -652,7 +948,7 @@ static void ptvalue (void)
     zx = getx();
     if(zx < 0.0) return;
     xsq = zx * zx;
-    xx = fdist(xsq, 1, n)/2.0;
+    xx = f_cdf_comp(xsq, 1, n)/2.0;
     printf(_("\nFor Student's t(%d), area (one-tail) to the "
 	   "right of %g is "), n, zx);
     putxx(xx);
@@ -669,7 +965,7 @@ static void pchisq (void)
     printf("%s", _(negval));
     zx = getx();
     if(zx < 0.0) return;
-    xx = chisq(zx, n);
+    xx = chisq_cdf_comp(zx, n);
     printf(_("\nFor Chi-square(%d), area to the right of %g is "), 
 	   n, zx);
     putxx(xx);
@@ -689,7 +985,7 @@ static void pfvalue (void)
     printf("%s", _(negval));
     zx = getx();
     if (zx < 0.0) return;
-    xx = fdist(zx, m, n);
+    xx = f_cdf_comp(zx, m, n);
     printf(_("\nFor F(%d, %d), area to the right of %g is "),
 	   m, n, zx);
     putxx(xx);
@@ -709,7 +1005,7 @@ static void pgamma (void)
     printf("%s", _(negval));
     zx = getx();
     if (zx < 0.0) return;
-    xx = 1.0 - gamma_dist(mean, variance, zx, 2);
+    xx = 1.0 - gamma_cdf_comp(mean, variance, zx, 2);
     printf(_("\nFor Gamma (mean %g, variance %g), area to the right of %g is "),
 	   mean, variance, zx);
     putxx(xx);
@@ -730,33 +1026,6 @@ static void getdf (const char *str)
     printf(_("\nEnter d.f.%s(value <= 0 will exit menu): "), str);
 }
 
-/**
- * f_crit_a:
- * @a: significance level.
- * @df1: numerator degrees of freedom.
- * @df2: denominator degrees of freedom.
- *
- * Returns: the one-sided critical value for F(@df1, @df2, @a).
- */
-
-double f_crit_a (double a, int df1, int df2)
-{
-    if (df1 < 1 || df2 < 1 || a < 0.0) {
-	return NADBL;
-    } else {
-	return fdtri(df1, df2, a);
-    }
-}
-
-static double chi_crit_a (double a, int df)
-{
-    if (df < 1 || a < 0.0) {
-	return NADBL;
-    } else {
-	return chdtri(df, a);
-    }
-}
-
 static int 
 parse_critical_input (const char *s, int *df, int *n)
 {
@@ -859,9 +1128,9 @@ int print_critical (const char *line, PRN *prn)
     case 'F':
 	pprintf(prn, _("Approximate critical values of F(%d, %d)\n\n"),
 		df, n);
-	pprintf(prn, _(" 10%% in right tail %.2f\n"), f_crit_a(.10, df, n));
-	pprintf(prn, "  5%%               %.2f\n", f_crit_a(.05, df, n));	
-	pprintf(prn, "  1%%               %.2f\n", f_crit_a(.01, df, n));
+	pprintf(prn, _(" 10%% in right tail %.2f\n"), f_critval(.10, df, n));
+	pprintf(prn, "  5%%               %.2f\n", f_critval(.05, df, n));	
+	pprintf(prn, "  1%%               %.2f\n", f_critval(.01, df, n));
 	break;
     case 'd':
 	(*dw)(n, prn);
@@ -957,15 +1226,9 @@ double genr_get_critical (const char *line, const double **Z,
     }	
 
     if (st == 'F') {
-	ret = f_crit_a(alpha, dfn, dfd);
-	if (ret < 0) {
-	    ret = NADBL;
-	}
+	ret = f_critval(alpha, dfn, dfd);
     } else if (st == 'X') {
-	ret = chi_crit_a(alpha, dfn);
-	if (ret < 0) {
-	    ret = NADBL;
-	}
+	ret = chisq_critval(alpha, dfn);
     } else if (st == 't') {
 	if (alpha > 0.5) {
 	    ret = stdtri(dfn, 1.0 - alpha);
@@ -991,51 +1254,32 @@ double genr_get_critical (const char *line, const double **Z,
 
 static const double gamma_tol = 1e-7;
 
-/* internal functions */
-
-static double gamma_integral (double lambda, double x);
-static double gamma_integral_expansion (double lambda, double x);
-static double gamma_integral_fraction (double lambda, double x);
-static double gammadist_wilson_hilferty (double shape, double scale, double x);
-
-/* Control 1 : s1, s2 = shape, scale
-           2 : s1, s2 = expectation, variance
-   Returns NADBL on error 
+/* Gamma distribution function.
+   See Johnson, Kotz and Balakrishnan: 
+   Continuous Univariate Distributions
+   vol 1, 2nd ed, Wiley 1994
 */
 
-double gamma_dist (double s1, double s2, double x, int control)
+static double gammadist_wilson_hilferty (double shape, double scale, double x)
 {
-    double shape, scale, xx;
+    double df = 2.0 * shape;
+    double xscaled = x * 2.0 / scale;
+    double xx;
 
-    if (control == 1) {
-	shape = s1; 
-	scale = s2; 
-    } else {
-	scale = s2 / s1; 
-	shape = s1 / scale; 
-    }	
+    xx = exp(log(xscaled/df)/3) - 1 + (double)(2) / 9 / df;
+    xx *= sqrt(9 * df / 2);
 
-    if ((shape > 20) && (x / scale < 0.9 * shape) && (x > 1)) {
-	xx = gammadist_wilson_hilferty(shape, scale, x);
-    } else {
-	xx = gamma_integral(shape, x / scale);
-	if (!na(xx)) {
-	    xx /= cephes_gamma(shape);
-	}
-    }
+    return normal_cdf(xx);
+} 
 
-    return xx;
-}
-
-/* end exported functions */
+/* Expansion of Gamma Integral int_0^x t^lambda-1 exp(-t)
+   Abramowitz and Stegun p. 262
+   Note that the series is alternating.
+*/
 
 static double gamma_integral_expansion (double lambda, double x)
-     /* Expansion of Gamma Integral int_0^x t^lambda-1 exp(-t)
-	Abramowitz and Stegun p. 262
-	Note that the series is alternating.
-     */
 {
-    double xx, x1 = 1.0;
+    double g, x1 = 1.0;
     double x2, x3 = 1.0 / lambda;
     int i = 0;
 
@@ -1047,23 +1291,24 @@ static double gamma_integral_expansion (double lambda, double x)
     } while (fabs(x2) >= gamma_tol && i <= 100);
 
     if (i == 100) {
-	xx = NADBL;
+	g = NADBL;
     } else {
-	xx = x3 * exp(lambda * log(x));
+	g = x3 * exp(lambda * log(x));
     }
 
-    return xx;
+    return g;
 }
 
+/* Continued Fraction Expansion for Gamma Integral
+   int_0^x t^lambda-1 exp(-t) dx
+   Abramowitz and Stegun p. 263
+   Implemented in Fortran by
+   B. L. Shea (1988): Chi-squared and incomplete gamma integral,
+   Applied Statistics, vol 37, pp. 466-473.
+   See also Schwartz p. 120.      
+*/
+
 static double gamma_integral_fraction (double lambda, double x)
-     /* Continued Fraction Expansion for Gamma Integral
-	int_0^x t^lambda-1 exp(-t) dx
-	Abramowitz and Stegun p. 263
-	Implemented in Fortran by
-	B. L. Shea (1988): Chi-squared and incomplete gamma integral,
-	Applied Statistics, vol 37, pp. 466-473.
-	See also Schwartz p. 120.      
-     */
 {
     double a = 1 - lambda;
     double b = a + x + 1;
@@ -1096,37 +1341,52 @@ static double gamma_integral_fraction (double lambda, double x)
 
 static double gamma_integral (double lambda, double x)
 {
-    double xx;
+    double g;
 
     if (x < 0.0)  { 
-	xx = NADBL;
+	g = NADBL;
     } else if (x < gamma_tol) {
-	xx = 0.0;
+	g = 0.0;
     } else if (x <= 1.0 || x < 0.9 * lambda) {
-	xx = gamma_integral_expansion(lambda, x);
+	g = gamma_integral_expansion(lambda, x);
     } else {
-	xx = gamma_integral_fraction(lambda, x);
+	g = gamma_integral_fraction(lambda, x);
+    }
+
+    return g;
+}
+
+/* Control 1 : s1, s2 = shape, scale
+           2 : s1, s2 = expectation, variance
+   Returns NADBL on error 
+*/
+
+double gamma_cdf_comp (double s1, double s2, double x, int control)
+{
+    double shape, scale, xx;
+
+    if (control == 1) {
+	shape = s1; 
+	scale = s2; 
+    } else {
+	scale = s2 / s1; 
+	shape = s1 / scale; 
+    }	
+
+    if ((shape > 20) && (x / scale < 0.9 * shape) && (x > 1)) {
+	xx = gammadist_wilson_hilferty(shape, scale, x);
+    } else {
+	xx = gamma_integral(shape, x / scale);
+	if (!na(xx)) {
+	    xx /= cephes_gamma(shape);
+	}
     }
 
     return xx;
 }
 
-static double gammadist_wilson_hilferty (double shape, double scale, double x)
-     /* Gamma distribution function.
-	See Johnson, Kotz and Balakrishnan: 
-	Continuous Univariate Distributions
-        vol 1, 2nd ed, Wiley 1994
-     */
-{
-    double df = 2.0 * shape;
-    double xscaled = x * 2.0 / scale;
-    double xx;
 
-    xx = exp(log(xscaled/df)/3) - 1 + (double)(2) / 9 / df;
-    xx *= sqrt(9 * df / 2);
 
-    return normal_cdf(xx);
-} 
 
 
 
