@@ -21,47 +21,103 @@
 #include "qr_estimate.h"
 #include "gretl_matrix.h"
 #include "libset.h"
+#include "estim_private.h"
 
 #include "f2c.h"
 #include "clapack_double.h"
 
 #define QR_RCOND_MIN 1e-15 /* experiment with this? */
-#define ESSZERO      1e-22 /* SSR < this counts as zero */
+#define ESSZERO      1e-22 /* SSR less than this counts as zero */
 
-/* In fortran arrays, column entries are contiguous.
+/* General note: in fortran arrays, column entries are contiguous.
    Columns of data matrix X hold variables, rows hold observations.
-   So in a fortran array, entries for a given variable are
-   contiguous.
+   So in a fortran array, entries for a given variable are contiguous.
 */
 
-static double get_tss (const double *y, int n, int ifc)
+static double qr_get_tss (MODEL *pmod, const double *y, int *ifc)
 {
+    int pwe = gretl_model_get_int(pmod, "pwe");
     double ymean = 0.0;
     double x, tss = 0.0;
-    int i;
+    int t;
 
-    if (ifc) {
-	ymean = gretl_mean(0, n-1, y);
+    if (*ifc == 0) {
+	*ifc = check_for_effective_const(pmod, y);
     }
 
-    for (i=0; i<n; i++) {
-	x = y[i] - ymean;
-	tss += x * x;
-    }
+    if (pmod->rho != 0.0) {
+	double ry, d;
+
+	if (*ifc) {
+	    for (t=pmod->t1; t<=pmod->t2; t++) {
+		ry = y[t];
+		if (t == pmod->t1 && pwe) {
+		    ry *= sqrt(1.0 - pmod->rho * pmod->rho);
+		} else {
+		    ry -= pmod->rho * y[t-1];
+		}
+		ymean += ry;
+	    }
+	    ymean /= pmod->nobs;
+	}
+
+	for (t=pmod->t1; t<=pmod->t2; t++) {
+	    ry = y[t];
+	    if (t == pmod->t1 && pwe) {
+		ry *= sqrt(1.0 - pmod->rho * pmod->rho);
+	    } else {
+		ry -= pmod->rho * y[t-1];
+	    }
+	    d = ry - ymean;
+	    tss += d * d;
+	}
+    } else {
+	if (*ifc) {
+	    for (t=pmod->t1; t<=pmod->t2; t++) {
+		if (!na(pmod->yhat[t])) {
+		    ymean += y[t];
+		}
+	    }
+	    ymean /= pmod->nobs;
+	}
+
+	for (t=pmod->t1; t<=pmod->t2; t++) {
+	    if (!na(pmod->yhat[t])) {
+		x = y[t] - ymean;
+		tss += x * x;
+	    }
+	}
+    } 
 
     return tss;
 }
 
-static void qr_compute_f_stat (MODEL *pmod, gretlopt opts)
+static void qr_compute_stats (MODEL *pmod, const double *y, int n,
+			      gretlopt opt)
 {
+    int ifc = pmod->ifc;
+
+    pmod->tss = qr_get_tss(pmod, y, &ifc);
+
+    if (pmod->dfd > 0) {
+	double den = pmod->tss * pmod->dfd;
+
+	pmod->rsq = 1.0 - (pmod->ess / pmod->tss);
+	pmod->adjrsq = 1.0 - (pmod->ess * (n - 1) / den);
+    } else {
+	pmod->rsq = 1.0;
+    }
+
     if (pmod->ncoeff == 1 && pmod->ifc) {
 	pmod->fstt = NADBL;
 	return;
     }
 
     if (pmod->dfd > 0 && pmod->dfn > 0) {
-	if (opts & OPT_R) {
+	if (opt & OPT_R) {
 	    pmod->fstt = robust_omit_F(NULL, pmod);
+	} else if (pmod->rsq == 1.0) {
+	    pmod->fstt = NADBL;
 	} else {
 	    pmod->fstt = (pmod->tss - pmod->ess) * pmod->dfd / 
 		(pmod->ess * pmod->dfn);
@@ -71,30 +127,6 @@ static void qr_compute_f_stat (MODEL *pmod, gretlopt opts)
     }
 }
 
-static void qr_compute_r_squared (MODEL *pmod, const double *y, int n)
-{
-    if (pmod->dfd > 0) {
-	if (pmod->ifc) {
-	    double den = pmod->tss * pmod->dfd;
-
-	    pmod->rsq = 1.0 - (pmod->ess / pmod->tss);
-	    pmod->adjrsq = 1.0 - (pmod->ess * (n - 1) / den);
-	} else {
-	    double alt = 
-		gretl_corr_rsq(pmod->t1, pmod->t2, y, pmod->yhat);
-
-	    if (na(alt)) {
-		pmod->rsq = pmod->adjrsq = NADBL;
-	    } else {
-		pmod->rsq = alt;
-		pmod->adjrsq = 1.0 - ((1 - alt) * (n - 1.0) / pmod->dfd);
-	    }
-	}
-    } else {
-	pmod->rsq = 1.0;
-    }
-}
-
 static int get_vcv_index (MODEL *pmod, int i, int j, int n)
 {
     int k, vi, vj;
@@ -144,9 +176,11 @@ static void get_resids_and_SSR (MODEL *pmod, const double **Z,
     int qdiff = (pmod->rho != 0.0);
     int pwe = gretl_model_get_int(pmod, "pwe");
     int yvar = pmod->list[1];
-    double u, y;
+    double y;
 
-    if (dwt) dwt = pmod->nwt;
+    if (dwt) {
+	dwt = pmod->nwt;
+    }
 
     pmod->ess = 0.0;
 
@@ -161,9 +195,9 @@ static void get_resids_and_SSR (MODEL *pmod, const double **Z,
 		} else {
 		    y -= pmod->rho * Z[yvar][t-1];
 		}
-		u = y - yhat->val[i];
-		pmod->uhat[t] = u;
-		pmod->ess += u * u;
+		pmod->yhat[t] = yhat->val[i];
+		pmod->uhat[t] = y - yhat->val[i];
+		pmod->ess += pmod->uhat[t] * pmod->uhat[t];
 		i++;
 	    }
 	}
@@ -177,7 +211,7 @@ static void get_resids_and_SSR (MODEL *pmod, const double **Z,
 		    pmod->yhat[t] = NADBL;
 		} else {
 		    if (!dwt) {
-			y *= Z[pmod->nwt][t];
+			y *= sqrt(Z[pmod->nwt][t]);
 		    }
 		    pmod->yhat[t] = yhat->val[i];
 		    pmod->uhat[t] = y - yhat->val[i];
@@ -200,7 +234,7 @@ static void get_resids_and_SSR (MODEL *pmod, const double **Z,
     }
 
     /* if SSR is small enough, treat it as zero */
-    if (pmod->ess < ESSZERO && pmod->ess > (-ESSZERO)) {
+    if (fabs(pmod->ess) < ESSZERO) {
 	pmod->ess = 0.0;
     } 
 }
@@ -217,7 +251,7 @@ get_data_X (gretl_matrix *X, const MODEL *pmod, const double **Z)
 	for (t=pmod->t1; t<=pmod->t2; t++) {
 	    if (!model_missing(pmod, t)) {
 		if (pmod->nwt) {
-		    X->val[j++] = Z[pmod->nwt][t] * Z[pmod->list[i]][t];
+		    X->val[j++] = sqrt(Z[pmod->nwt][t]) * Z[pmod->list[i]][t];
 		} else {
 		    X->val[j++] = Z[pmod->list[i]][t];
 		}
@@ -230,7 +264,7 @@ get_data_X (gretl_matrix *X, const MODEL *pmod, const double **Z)
 	for (t=pmod->t1; t<=pmod->t2; t++) {
 	    if (!model_missing(pmod, t)) {
 		if (pmod->nwt) {
-		    X->val[j++] = Z[pmod->nwt][t];
+		    X->val[j++] = sqrt(Z[pmod->nwt][t]);
 		} else {
 		    X->val[j++] = 1.0;
 		}
@@ -499,7 +533,9 @@ static void get_model_data (MODEL *pmod, const double **Z,
 
     start = (pmod->ifc)? 3 : 2;
 
-    if (dwt) dwt = pmod->nwt;
+    if (dwt) {
+	dwt = pmod->nwt;
+    }
 
     /* copy independent vars into matrix Q */
     j = 0;
@@ -512,7 +548,7 @@ static void get_model_data (MODEL *pmod, const double **Z,
 	    if (dwt) {
 		if (Z[dwt][t] == 0.0) continue;
 	    } else if (pmod->nwt) {
-		x *= Z[pmod->nwt][t];
+		x *= sqrt(Z[pmod->nwt][t]);
 	    } else if (qdiff) {
 		if (pwe && t == pmod->t1) {
 		    x *= pw1;
@@ -537,7 +573,7 @@ static void get_model_data (MODEL *pmod, const double **Z,
 	    if (dwt) {
 		if (Z[dwt][t] == 0.0) continue;
 	    } else if (pmod->nwt) {
-		x = Z[pmod->nwt][t];
+		x = sqrt(Z[pmod->nwt][t]);
 	    } else if (qdiff) {
 		if (pwe && t == pmod->t1) {
 		    x = pw1;
@@ -559,7 +595,7 @@ static void get_model_data (MODEL *pmod, const double **Z,
 	if (dwt) {
 	    if (Z[dwt][t] == 0.0) continue;
 	} else if (pmod->nwt) {
-	    x *= Z[pmod->nwt][t];
+	    x *= sqrt(Z[pmod->nwt][t]);
 	} else if (qdiff) {
 	    if (pwe && t == pmod->t1) {
 		x *= pw1;
@@ -569,9 +605,6 @@ static void get_model_data (MODEL *pmod, const double **Z,
 	}
 	y->val[j++] = x;
     }
-
-    /* fetch tss based on the (possibly transformed) y */
-    pmod->tss = get_tss(y->val, pmod->nobs, pmod->ifc);
 }
 
 static void save_coefficients (MODEL *pmod, gretl_matrix *b,
@@ -677,22 +710,9 @@ int gretl_qr_regress (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 
     err = QR_decomp_plus(Q, R);
 
-#if 0 /* experimenting here */
-    if (1) {
-	fprintf(stderr, "without pivot\n");
-	gretl_matrix_print(Q, "Q");
-	gretl_matrix_print(R, "R");
-	get_model_data(pmod, (const double **) *pZ, Q, y);
-	err = QR_decomp_plus(Q, R, 1);
-	fprintf(stderr, "with pivot\n");
-	gretl_matrix_print(Q, "Q");
-	gretl_matrix_print(R, "R");
-    }
-#endif
-
     if (err == E_SINGULAR && !(opts & OPT_Z) &&
 	redundant_var(pmod, pZ, pdinfo, &droplist)) {
-	/* FIXME this can be done more efficiently, using R */
+	/* FIXME: can't this be done more efficiently, using R? */
 	gretl_matrix_null(&Q);
 	gretl_matrix_null(&R);
 	gretl_matrix_null(&xpxinv);
@@ -761,8 +781,7 @@ int gretl_qr_regress (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     }
 
     /* get R^2, F */
-    qr_compute_r_squared(pmod, (*pZ)[pmod->list[1]], T);
-    qr_compute_f_stat(pmod, opts);
+    qr_compute_stats(pmod, (*pZ)[pmod->list[1]], T, opts);
 
  qr_cleanup:
 
