@@ -24,7 +24,7 @@
 #include <errno.h>
 #include <time.h>
 
-#if defined(USE_GTK2)
+#if defined(USE_GLIB2)
 # include <glib.h>
 #else
 # include <fnmatch.h>
@@ -82,7 +82,6 @@ double dot_atof (const char *s)
  * Returns: the integer position of the last "." within @str,
  * or strlen(@str) in case a dot is not found, or the string
  * ends with a (backward or forward) slash.
- *
  */
 
 int dotpos (const char *str)
@@ -183,7 +182,6 @@ char *gretl_delete (char *str, int idx, int count)
  *
  * Returns: the first position of @c in @s, or -1 if @c is not
  * found.
- *
  */
 
 int haschar (char c, const char *s)
@@ -206,7 +204,6 @@ int haschar (char c, const char *s)
  * @s: the string to examine.
  *
  * Returns: 1 if @c is the last character in @s, 0 otherwise
- *
  */
 
 int lastchar (char c, const char *s)
@@ -282,7 +279,6 @@ int has_suffix (const char *str, const char *sfx)
  * @str: the string to examine.
  *
  * Returns: 1 if the given @str is numeric, otherwise 0.
- *
  */
 
 int numeric_string (const char *str)
@@ -319,7 +315,6 @@ int numeric_string (const char *str)
  *
  * Returns: 1 if the last non-space character in @s is a backslash,
  * otherwise 0.
- *
  */
 
 int ends_with_backslash (const char *s)
@@ -446,7 +441,7 @@ char *gretl_word_strdup (const char *src, const char **ptr)
 	const char *p;
 	int len = 0;
 
-	while (*src && !is_word_char(*src) ) {
+	while (*src && !is_word_char(*src)) {
 	    src++;
 	}
 
@@ -469,6 +464,52 @@ char *gretl_word_strdup (const char *src, const char **ptr)
     return targ;
 }
 
+/**
+ * gretl_double_from_string:
+ * @s: the source string.
+ * @ptr: location to receive end of field pointer, or %NULL.
+ *
+ * Tries to parse the first 'word' found in @s as a 
+ * floating-point number, and returns either the number or
+ * #NADBL on failure.  If the parsing was successful and if
+ * @ptr is not %NULL, on exit it points at the next position 
+ * in @s after the just-parsed numeric portion.
+ *
+ * Returns: floating-point value or #NADBL.
+ */
+
+double gretl_double_from_string (const char *s, const char **ptr)
+{
+    double x = NADBL;
+
+    if (s == NULL) {
+	if (ptr != NULL) {
+	    *ptr = NULL;
+	}
+    } else if (*s == '\0') {
+	if (ptr != NULL) {
+	    *ptr = s;
+	}
+    } else {
+	int n = strspn(s, " ");
+
+	s += n;
+	n = strcspn(s, " ");
+	if (n <= 31) {
+	    if (sscanf(s, "%lf", &x) != 1) {
+		x = NADBL;
+		n = 0;
+	    } 
+	}
+
+	if (n > 0 && ptr != NULL) {
+	    *ptr = s + n;
+	}
+    }
+
+    return x;
+}
+
 /**
  * gretl_trunc:
  * @str: the string to truncate.
@@ -490,7 +531,7 @@ char *gretl_trunc (char *str, size_t n)
 
 /**
  * gretl_varchar_spn:
- * @s: the string to truncate.
+ * @s: the string to examine.
  *
  * Returns: the length of the intial segment of @s which
  * consists of characters that are valid in a gretl
@@ -518,7 +559,6 @@ int gretl_varchar_spn (const char *s)
  * @len: the length of the string to be cleared.
  *
  * Sets all bytes in @str to 0.
- *
  */
 
 void clear (char *str, int len)
@@ -554,7 +594,9 @@ int count_fields (const char *s)
 	    p = strpbrk(s, " ");
 	    if (p != NULL) {
 		s = p + strspn(p, " ");
-		if (*s) nf++;
+		if (*s) {
+		    nf++;
+		}
 	    } else {
 		break;
 	    }
@@ -628,8 +670,7 @@ char *chopstr (char *str)
  * For processing filenames: copies @src to @targ, minus any existing
  * filename extension, and adds to @targ the specified extension.
  *
- * Returns: the output string.
- *
+ * Returns: the output string, @targ.
  */
 
 char *switch_ext (char *targ, const char *src, char *ext)
@@ -657,7 +698,6 @@ char *switch_ext (char *targ, const char *src, char *ext)
  * including the last occurrence of @c within @src.
  *
  * Returns: 1 if @c is found in @str, otherwise 0.
- *
  */
 
 int get_base (char *targ, const char *src, char c)
@@ -688,7 +728,6 @@ int get_base (char *targ, const char *src, char c)
  * then replace a trailing backslash (if any) with a space.
  * 
  * Returns: 1 if a trailing backslash was found, otherwise 0.
- *
  */
 
 int top_n_tail (char *str)
@@ -908,7 +947,6 @@ char *space_to_score (char *s)
  * @targ[@n] is a NUL byte.
  * 
  * Returns: the output string.
- *
  */
 
 char *safecpy (char *targ, const char *src, int n)
@@ -919,7 +957,7 @@ char *safecpy (char *targ, const char *src, int n)
 }
 
 /**
- * create_strings_array:
+ * strings_array_new:
  * @nstrs: number of strings in array.
  *
  * Allocates storage for @nstrs strings and initalizes all 
@@ -928,11 +966,15 @@ char *safecpy (char *targ, const char *src, int n)
  * Returns: the allocated array, or %NULL on failure.
  */
 
-char **create_strings_array (int nstrs)
+char **strings_array_new (int nstrs)
 {
     char **s;
     int i;
 
+    if (nstrs <= 0) {
+	return NULL;
+    }
+
     s = malloc(nstrs * sizeof *s);
     if (s != NULL) {
 	for (i=0; i<nstrs; i++) {
@@ -943,6 +985,83 @@ char **create_strings_array (int nstrs)
     return s;
 }
 
+/**
+ * strings_array_new_with_length:
+ * @nstrs: number of strings in array.
+ * @len: number of bytes per string.
+ *
+ * Allocates storage for @nstrs strings, each of them 
+ * @len bytes long.  The first byte of each string is
+ * initialized to 0.
+ * 
+ * Returns: the allocated array, or %NULL on failure.
+ */
+
+char **strings_array_new_with_length (int nstrs, int len)
+{
+    char **S;
+    int i, j;
+
+    if (nstrs <= 0) {
+	return NULL;
+    }
+
+    S = malloc(nstrs * sizeof *S);
+    if (S == NULL) return NULL;
+
+    for (i=0; i<nstrs; i++) {
+	S[i] = malloc(len);
+	if (S[i] == NULL) {
+	    for (j=0; j<i; j++) {
+		free(S[j]);
+	    }
+	    free(S);
+	    return NULL;
+	}
+	S[i][0] = '\0';
+    }
+
+    return S;
+}
+
+/**
+ * strings_array_dup:
+ * @strs: array of strings to be copied.
+ * @n: number of strings in array.
+ *
+ * Returns: an allocated copy of @strs, or %NULL on failure.
+ */
+
+char **strings_array_dup (char **strs, int n)
+{
+    char **S;
+    int i, j;
+
+    if (n <= 0 || strs == NULL) {
+	return NULL;
+    }
+
+    S = malloc(n * sizeof *S);
+    if (S == NULL) return NULL;
+
+    for (i=0; i<n; i++) {
+	if (strs[i] == NULL) {
+	    S[i] = NULL;
+	} else {
+	    S[i] = gretl_strdup(strs[i]);
+	    if (S[i] == NULL) {
+		for (j=0; j<i; j++) {
+		    free(S[j]);
+		}
+		free(S);
+		return NULL;
+	    }
+	}
+    }
+
+    return S;
+}
+
 /**
  * free_strings_array:
  * @strs: array of allocated strings.
@@ -1126,7 +1245,16 @@ void csv_obs_to_prn (int t, const DATAINFO *pdinfo, PRN *prn)
 	    }
 	}
     }
-}	
+}
+	
+/**
+ * print_time:
+ * @timep: time to print.
+ *
+ * Returns: pointer to a static string containing a locale-dependent
+ * representation of @timep.  In English, this will be in the format
+ * Y/m/d H:M.
+ */
 
 const char *print_time (const time_t *timep)
 {
@@ -1140,59 +1268,93 @@ const char *print_time (const time_t *timep)
     return timestr;
 }
 
-char *gretl_xml_encode (char *buf)
+/**
+ * gretl_xml_validate:
+ * @s: string to be tested.
+ *
+ * Returns: 1 if @s is acceptable for insertion into an XML file
+ * as is, 0 if it contains special characters that need to be 
+ * escaped.  See also gretl_xml_encode().
+ */
+
+int gretl_xml_validate (const char *s)
 {
-    char *xmlbuf, *p;
-    size_t sz = strlen(buf) + 1;
+    while (*s) {
+	if (*s == '&' || *s == '<' || *s == '>' || *s == '"') {
+	    return 0;
+	}
+	s++;
+    }
+
+    return 1;
+}
+
+/**
+ * gretl_xml_encode:
+ * @str: NUL-terminated source string.
+ *
+ * Returns: an allocated re-write of @buf, with characters that are
+ * special in XML encoded as character entities.  See also
+ * gretl_xml_validate().
+ */
+
+char *gretl_xml_encode (const char *str)
+{
+    char *xmlstr, *p;
+    const char *s = str;
+    size_t sz = strlen(str) + 1;
 
 #ifdef XML_DEBUG
     fprintf(stderr, "gretl_xml_encode: original buffer size=%d\n", sz);
 #endif
 
-    p = buf;
-    while (*buf) {
-	if (*buf == '&') sz += 4;
-	else if (*buf == '<') sz += 3;
-	else if (*buf == '>') sz += 3;
-	else if (*buf == '"') sz += 5;
-	buf++;
+    while (*s) {
+	if (*s == '&') sz += 4;
+	else if (*s == '<') sz += 3;
+	else if (*s == '>') sz += 3;
+	else if (*s == '"') sz += 5;
+	s++;
     }
-    buf = p;
 
-    xmlbuf = malloc(sz);
-    if (xmlbuf == NULL) {
+    xmlstr = malloc(sz);
+    if (xmlstr == NULL) {
 	sprintf(gretl_errmsg, _("out of memory in XML encoding"));
 	return NULL;
     }
+
 #ifdef XML_DEBUG
-    fprintf(stderr, "gretl_xml_encode: malloc'd xmlbuf at size %d\n", sz);
+    fprintf(stderr, "gretl_xml_encode: malloc'd xmlstr at size %d\n", sz);
 #endif
-    p = xmlbuf;
-    while (*buf) {
-	if (*buf == '&') {
+
+    s = str;
+    p = xmlstr;
+    
+    while (*s) {
+	if (*s == '&') {
 	    strcpy(p, "&amp;");
 	    p += 5;
-	} else if (*buf == '<') {
+	} else if (*s == '<') {
 	    strcpy(p, "&lt;");
 	    p += 4;
-	} else if (*buf == '>') {
+	} else if (*s == '>') {
 	    strcpy(p, "&gt;");
 	    p += 4;
-	} else if (*buf == '"') {
+	} else if (*s == '"') {
 	    strcpy(p, "&quot;");
 	    p += 6;
 	} else {
-	    *p++ = *buf;
+	    *p++ = *s;
 	}
-	buf++;
+	s++;
     }
-    xmlbuf[sz-1] = '\0';
+
+    xmlstr[sz-1] = '\0';
 
 #ifdef XML_DEBUG
-    fprintf(stderr, "done gretl_xml_encode: xmlbuf='%s'\n", xmlbuf);
+    fprintf(stderr, "done gretl_xml_encode: xmlstr='%s'\n", xmlstr);
 #endif
 
-    return xmlbuf;
+    return xmlstr;
 }
 
 static char x2c (char *s) 
@@ -1205,6 +1367,12 @@ static char x2c (char *s)
     return digit;
 }
 
+/**
+ * unescape_url:
+ * @url: string representing a URL.
+ *
+ */
+
 void unescape_url (char *url) 
 {
     register int x, y;
@@ -1302,43 +1470,91 @@ char *append_dir (char *fname, const char *dir)
     return fname;
 }
 
-int build_path (const char *dir, const char *fname, char *path, 
-		const char *ext)
+/**
+ * build_path:
+ * @targ: target string to write to (must be pre-allocated).
+ * @dirname: first part of path.
+ * @fname: filename.
+ * @ext: filename extension to be appended (or %NULL).
+ *
+ * Writes to @targ a full path composed of @dirname,
+ * @fname and (optionally) @ext.  This function ensures
+ * that an appropriate separator is inserted between 
+ * @dirname and @fname, if @dirname is not already
+ * terminated with such a separator.
+ *
+ * Returns: the target string, @targ.
+ */
+
+char *build_path (char *targ, const char *dirname, const char *fname, 
+		  const char *ext)
 {
     size_t len;
 
-    if (dir == NULL || fname == NULL || path == NULL) {
-	return 1;
+    if (dirname == NULL || fname == NULL || targ == NULL) {
+	return NULL;
     }
 
-    *path = '\0';
-    strcat(path, dir);
-    len = strlen(path);
-    if (len == 0) return 1;
+    *targ = '\0';
+    strcat(targ, dirname);
+    len = strlen(targ);
+    if (len == 0) {
+	return NULL;
+    }
 
     /* strip a trailing single dot */
-    if (len > 1 && path[len-1] == '.' && 
-	(path[len-2] == '/' || path[len-2] == '\\')) {
-	    path[len-1] = '\0';
+    if (len > 1 && targ[len-1] == '.' && 
+	(targ[len-2] == '/' || targ[len-2] == '\\')) {
+	    targ[len-1] = '\0';
     }
 
-    if (path[len-1] == '/' || path[len-1] == '\\') {
-        /* dir is already properly terminated */
-        strcat(path, fname);
+    if (targ[len-1] == '/' || targ[len-1] == '\\') {
+        /* dirname is already properly terminated */
+        strcat(targ, fname);
     } else {
         /* otherwise put a separator in */
-        strcat(path, SLASHSTR);
-        strcat(path, fname);
+        strcat(targ, SLASHSTR);
+        strcat(targ, fname);
     }
 
     if (ext != NULL) {
-	strcat(path, ext);
+	strcat(targ, ext);
     }
 
-    return 0;
+    return targ;
 }
 
-#if defined(USE_GTK2)
+/**
+ * path_last_element:
+ * @path: path to work on.
+ *
+ * Returns: a pointer to the last element of @path, that is, 
+ * the element following the last path separator character, if any.
+ * If @path does not contain a separator, @path itself is returned.
+ * Note that the return value may be the empty string, if @path
+ * ends with a separator.
+ */
+
+const char *path_last_element (const char *path)
+{
+    const char *p = strrchr(path, SLASH);
+
+#ifdef WIN32
+    if (p == NULL) {
+	p = strrchr(path, '\\');
+    }
+#endif
+
+    if (p == NULL) {
+	p = path;
+    } else {
+	p++;
+    }
+
+    return p;
+}
+
+#if defined(USE_GLIB2)
 
 int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
 {
@@ -1348,7 +1564,7 @@ int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
 
     pspec = g_pattern_spec_new(pattern);
     for (i=1; i<pdinfo->v; i++) { 
-	if (pdinfo->vector[i] &&
+	if (var_is_series(pdinfo, i) &&
 	    g_pattern_match_string(pspec, pdinfo->varname[i])) {
 	    n++;
 	}
@@ -1361,7 +1577,7 @@ int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
 
 	    list[0] = n;
 	    for (i=1; i<pdinfo->v; i++) { 
-		if (pdinfo->vector[i] &&
+		if (var_is_series(pdinfo, i) &&
 		    g_pattern_match_string(pspec, pdinfo->varname[i])) {
 		    list[j++] = i;
 		}
@@ -1382,7 +1598,7 @@ int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
     int i, n = 0;
 
     for (i=1; i<pdinfo->v; i++) { 
-	if (pdinfo->vector[i] &&
+	if (var_is_series(pdinfo, i) &&
 	    fnmatch(pattern, pdinfo->varname[i], 0) == 0) {
 	    n++;
 	}
@@ -1395,7 +1611,7 @@ int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
 
 	    list[0] = n;
 	    for (i=1; i<pdinfo->v; i++) { 
-		if (pdinfo->vector[i] &&
+		if (var_is_series(pdinfo, i) &&
 		    fnmatch(pattern, pdinfo->varname[i], 0) == 0) {
 		    list[j++] = i;
 		}
