@@ -71,6 +71,8 @@ char *gretl_strndup (const char *src, size_t n);
 
 char *gretl_word_strdup (const char *src, const char **ptr);
 
+double gretl_double_from_string (const char *s, const char **ptr);
+
 char *charsub (char *str, char find, char repl);
 
 char *shift_string_left (char *str, size_t move);
@@ -97,7 +99,11 @@ char *space_to_score (char *s);
 
 char *safecpy (char *targ, const char *src, int n);
 
-char **create_strings_array (int nstrs);
+char **strings_array_new (int nstrs);
+
+char **strings_array_new_with_length (int nstrs, int len);
+
+char **strings_array_dup (char **strs, int n);
 
 void free_strings_array (char **strs, int nstrs);
 
@@ -115,7 +121,9 @@ void csv_obs_to_prn (int t, const DATAINFO *pdinfo, PRN *prn);
 
 const char *print_time (const time_t *timep);
 
-char *gretl_xml_encode (char *buf);
+int gretl_xml_validate (const char *s);
+
+char *gretl_xml_encode (const char *str);
 
 void unescape_url (char *url);
 
@@ -125,10 +133,12 @@ int fix_varname_duplicates (DATAINFO *pdinfo);
 
 char *append_dir (char *fname, const char *dir);
 
-int build_path (const char *dir, const char *fname, char *path, 
-		const char *ext);
+char *build_path (char *targ, const char *dirname, const char *fname, 
+		  const char *ext);
 
-#if defined(USE_GTK2) || defined (HAVE_FNMATCH_H)
+const char *path_last_element (const char *path);
+
+#if defined(USE_GLIB2) || defined (HAVE_FNMATCH_H)
 int *varname_match_list (const DATAINFO *pdinfo, const char *pattern);
 #endif
 
