@@ -22,6 +22,7 @@
 #include "libgretl.h"
 #include "libset.h"
 #include "gretl_func.h"
+#include "gretl_panel.h"
 
 #define SUBDEBUG 0
 
@@ -50,7 +51,7 @@ static const DATAINFO *peerinfo;
 
 #define SUBMASK_SENTINEL 127
 
-/* special functions shared with dataset.c */
+/* accessors for full dataset, when sub-sampled */
 
 double ***fetch_full_Z (void)
 {
@@ -67,8 +68,6 @@ DATAINFO *fetch_full_datainfo (void)
     return fullinfo;
 }
 
-/* end specials */
-
 static int full_data_length (const DATAINFO *pdinfo)
 {
     int n = 0;
@@ -120,6 +119,42 @@ char *copy_datainfo_submask (const DATAINFO *pdinfo)
     return mask;
 }
 
+int write_model_submask (const MODEL *pmod, FILE *fp)
+{
+    int ret = 0;
+
+    if (pmod->submask != NULL) {
+	int i, n = get_submask_length(pmod->submask);
+
+	fprintf(fp, "<submask length=\"%d\">", n);
+	for (i=0; i<n; i++) {
+	    fprintf(fp, "%d ", (int) pmod->submask[i]);
+	}
+	fputs("</submask>\n", fp);
+	ret = 1;
+    }
+
+    return ret;
+}
+
+int write_datainfo_submask (const DATAINFO *pdinfo, FILE *fp)
+{
+    int ret = 0;
+
+    if (complex_subsampled()) {
+	int i, n = get_submask_length(pdinfo->submask);
+
+	fprintf(fp, "<submask length=\"%d\" mode=\"%d\">", n, pdinfo->submode);
+	for (i=0; i<n; i++) {
+	    fprintf(fp, "%d ", (int) pdinfo->submask[i]);
+	}
+	fputs("</submask>\n", fp);
+	ret = 1;
+    }
+
+    return ret;
+}
+
 int submask_cmp (const char *m1, const char *m2)
 {
     while (*m1 != SUBMASK_SENTINEL && *m2 != SUBMASK_SENTINEL) {
@@ -179,7 +214,6 @@ static void sync_dataset_elements (const DATAINFO *pdinfo)
 {
     fullinfo->varname = pdinfo->varname;
     fullinfo->varinfo = pdinfo->varinfo;
-    fullinfo->vector = pdinfo->vector;
     fullinfo->descrip = pdinfo->descrip;
 }
 
@@ -222,12 +256,15 @@ update_full_data_values (const double **subZ, const DATAINFO *pdinfo)
     for (i=1; i<fullinfo->v; i++) {
 	int subt = 0;
 
-	if (!pdinfo->vector[i]) {
+	if (var_is_scalar(pdinfo, i)) {
 	    fullZ[i][0] = subZ[i][0];
 	} else {
 	    for (t=0; t<fullinfo->n; t++) {
-		if (pdinfo->submask[t]) {
+		if (pdinfo->submask[t] == 1) {
 		    fullZ[i][t] = subZ[i][subt++];
+		} else if (pdinfo->submask[t] == 'p') {
+		    /* skip panel padding (?) */
+		    subt++;
 		}
 	    }
 	}
@@ -260,14 +297,14 @@ static int update_case_markers (const DATAINFO *pdinfo)
 
 static int add_new_vars_to_full (const double **Z, DATAINFO *pdinfo)
 {
-    int newvars = pdinfo->v - fullinfo->v;
-    int V = fullinfo->v;
+    int V1 = pdinfo->v;
+    int V0 = fullinfo->v;
     int N = fullinfo->n;
     double **newZ = NULL;
     int i, t, subt;
     int err = 0;
 
-    if (newvars <= 0) {
+    if (V1 <= V0) {
 	return 0;
     }
 
@@ -276,7 +313,7 @@ static int add_new_vars_to_full (const double **Z, DATAINFO *pdinfo)
     }
 
     /* allocate expanded data array */
-    newZ = realloc(fullZ, pdinfo->v * sizeof *fullZ);
+    newZ = realloc(fullZ, V1 * sizeof *fullZ);
 
     if (newZ == NULL) {
 	return E_ALLOC;
@@ -284,15 +321,13 @@ static int add_new_vars_to_full (const double **Z, DATAINFO *pdinfo)
 
     fullZ = newZ;
 
-    for (i=0; i<newvars && !err; i++) {
-	int vi = V + i;
-
-	if (pdinfo->vector[vi]) {
-	    fullZ[vi] = malloc(N * sizeof **newZ);
+    for (i=V0; i<pdinfo->v && !err; i++) {
+	if (var_is_series(pdinfo, i)) {
+	    fullZ[i] = malloc(N * sizeof **newZ);
 	} else {
-	    fullZ[vi] = malloc(sizeof **newZ);
+	    fullZ[i] = malloc(sizeof **newZ);
 	}
-	if (fullZ[vi] == NULL) {
+	if (fullZ[i] == NULL) {
 	    err = E_ALLOC;
 	}
     }
@@ -300,9 +335,9 @@ static int add_new_vars_to_full (const double **Z, DATAINFO *pdinfo)
     if (err) {
 	return E_ALLOC;
     }
-    
-    for (i=V; i<pdinfo->v; i++) {
-	if (!pdinfo->vector[i]) {
+
+    for (i=V0; i<pdinfo->v; i++) {
+	if (var_is_scalar(pdinfo, i)) {
 	   fullZ[i][0] = Z[i][0]; 
 	}
     }
@@ -311,22 +346,22 @@ static int add_new_vars_to_full (const double **Z, DATAINFO *pdinfo)
 
     for (t=0; t<N; t++) {
 	if (pdinfo->submask[t]) {
-	    for (i=V; i<pdinfo->v; i++) {
-		if (pdinfo->vector[i]) {
+	    for (i=V0; i<pdinfo->v; i++) {
+		if (var_is_series(pdinfo, i)) {
 		    fullZ[i][t] = Z[i][subt];
 		}
 	    }
 	    subt++;
 	} else {
-	    for (i=V; i<pdinfo->v; i++) { 
-		if (pdinfo->vector[i]) {
+	    for (i=V0; i<pdinfo->v; i++) { 
+		if (var_is_series(pdinfo, i)) {
 		    fullZ[i][t] = NADBL;
 		}
 	    }
 	}
     }
 
-    fullinfo->v = pdinfo->v;
+    fullinfo->v = V1;
 
     return 0;
 }
@@ -380,8 +415,8 @@ int restore_full_sample (double ***pZ, DATAINFO **ppdinfo)
     *gretl_errmsg = '\0';
 
 #if SUBDEBUG
-    fprintf(stderr, "real_restore_full_sample: pZ=%p, ppdinfo=%p cum=%d\n",
-	    (void *) pZ, (void *) ppdinfo, cumulate);
+    fprintf(stderr, "restore_full_sample: pZ=%p, ppdinfo=%p\n",
+	    (void *) pZ, (void *) ppdinfo);
     fprintf(stderr, "*pZ=%p, *ppdinfo=%p\n",
 	    (void *) *pZ, (void *) *ppdinfo);
 #endif
@@ -406,16 +441,24 @@ int restore_full_sample (double ***pZ, DATAINFO **ppdinfo)
        which may have moved */
     sync_dataset_elements(*ppdinfo);
 
-    /* update values for pre-existing series, which may have been
-       modified via genr etc */
-    update_full_data_values((const double **) *pZ, *ppdinfo);
+    if (!err) {
+	/* update values for pre-existing series, which may have been
+	   modified via genr etc */
+	update_full_data_values((const double **) *pZ, *ppdinfo);
 
-    /* if case markers were added when subsampled, carry them back */
-    update_case_markers(*ppdinfo);
+	/* if case markers were added when subsampled, carry them back */
+	update_case_markers(*ppdinfo);
+
+	/* delete any newly added hidden vars */
+	err = dataset_destroy_hidden_variables(pZ, *ppdinfo, fullinfo->v);
+
+	/* in case any new vars were added when subsampled, try to merge
+	   them into the full dataset */
+	if (!err) {
+	    err = add_new_vars_to_full((const double **) *pZ, *ppdinfo);
+	}
+    }
 
-    /* in case any new vars were added when subsampled, try to merge
-       them into the full dataset */
-    err = add_new_vars_to_full((const double **) *pZ, *ppdinfo);
     if (err == E_ALLOC) {
         sprintf(gretl_errmsg, _("Out of memory expanding data set\n"));
     } else if (err == E_NOMERGE) {
@@ -460,7 +503,7 @@ make_missing_mask (const int *list, const double **Z, const DATAINFO *pdinfo,
 	    mask[t] = 1;
 	    for (i=1; i<=list[0]; i++) {
 		vi = list[i];
-		if (pdinfo->vector[vi] && na(Z[vi][t])) {
+		if (var_is_series(pdinfo, vi) && na(Z[vi][t])) {
 		    mask[t] = 0;
 		    break;
 		}
@@ -471,7 +514,7 @@ make_missing_mask (const int *list, const double **Z, const DATAINFO *pdinfo,
 	for (t=0; t<pdinfo->n; t++) {
 	    mask[t] = 1;
 	    for (i=1; i<pdinfo->v; i++) {
-		if (pdinfo->vector[i] && na(Z[i][t])) {
+		if (var_is_series(pdinfo, i) && na(Z[i][t])) {
 		    mask[t] = 0;
 		    break;
 		}
@@ -551,19 +594,49 @@ static int mask_from_dummy (const char *line,
     }
 
     return err;
-} 
+}
 
-static int count_selected_cases (const char *x, int n)
+/* how many observations are selected by the given 
+   subsample mask? */
+
+static int 
+count_selected_cases (const char *mask, const DATAINFO *pdinfo)
 {
-    int i, count = 0;
+    int i, n = 0;
 
-    for (i=0; i<n; i++) {
-	if (x[i] != 0) {
-	    count++;
+    for (i=0; i<pdinfo->n; i++) {
+	if (mask[i]) {
+	    n++;
 	}
     }
 
-    return count;
+    return n;
+}
+
+/* panel: how many distinct cross-sectional units are included 
+   in the masked subset of observations? */
+
+static int 
+count_panel_units (const char *mask, const DATAINFO *pdinfo)
+{
+    int u, ubak = -1;
+    int i, n = 0;
+
+    for (i=0; i<pdinfo->n; i++) {
+	if (mask[i]) {
+	    u = pdinfo->paninfo->unit[i];
+	    if (u != ubak) {
+		n++;
+		ubak = u;
+	    }
+	}
+    }
+
+#if SUBDEBUG
+    fprintf(stderr, "count_panel_units: got n = %d\n", n);
+#endif
+
+    return n;
 }
 
 static int make_random_mask (const char *oldmask, const char *line, 
@@ -620,7 +693,7 @@ static int make_random_mask (const char *oldmask, const char *line,
 	    mask[u] = 1;
 	}
 	if (i >= subn - 1) {
-	    cases = count_selected_cases(mask, pdinfo->n);
+	    cases = count_selected_cases(mask, pdinfo);
 	}
     }
 
@@ -651,132 +724,6 @@ int get_full_length_n (void)
     return n;
 }
 
-static int block_retained (const char *s)
-{
-    int nrows = 0;
-    int ret = 0;
-
-#if SUBDEBUG
-    fprintf(stderr, "block_retained: s = '%s'\n", s);
-#endif
-
-    while (*s) {
-	if (*s == '1') {
-	    nrows++;
-	}
-	if (nrows > 1) {
-	    ret = 1;
-	    break;
-	}
-	s++;
-    }
-
-    return ret;
-}
-
-static char *make_panel_signature (const DATAINFO *pdinfo,
-				   const char *mask,
-				   int mt1, int mt2,
-				   int neg)
-{
-    char *sig;
-    size_t sz = pdinfo->n + pdinfo->n / pdinfo->pd;
-    int t;
-
-    sig = calloc(sz, 1);
-
-#if SUBDEBUG
-    for (t=0; t<pdinfo->n; t++) {
-	if (t >= mt1 && t <= mt2) {
-	    fprintf(stderr, "mask[%d]=%c\n", t-mt1, mask[t-mt1]);
-	}
-    }
-#endif
-
-    if (sig != NULL) {
-	int i = 0;
-
-	for (t=0; t<pdinfo->n; t++) {
-	    if (t > 0 && t % pdinfo->pd == 0) {
-		i++;
-	    } 
-	    if (t < mt1 || t > mt2) {
-		sig[i++] = '0';
-	    } else {
-		if (neg) {
-		    /* using model missmask: '1' for missing vs '0' */
-		    sig[i++] = (mask[t - mt1] != '1')? '1' : '0';
-		} else {
-		    /* using sample missmask: 0 for missing vs 1 */
-		    sig[i++] = (mask[t - mt1] != 0)? '1' : '0';
-		}
-	    }
-	}
-    }
-
-    return sig;
-}
-
-static int 
-real_mask_leaves_balanced_panel (const char *mask,
-				 const DATAINFO *pdinfo,
-				 int *new_blocks,
-				 int mt1, int mt2, int neg)
-{
-    int i;
-    int ok_blocks = 0, unbal = 0;
-    int nblocks = pdinfo->n / pdinfo->pd;
-    char *sig = make_panel_signature(pdinfo, mask, mt1, mt2, neg);
-    char *sig_0 = NULL;
-
-    /* FIXME: allow for possibility that starting obs is not 1:1 */
-
-    if (sig == NULL) {
-	return 0;
-    }
-
-    for (i=0; i<nblocks; i++) {
-	char *sig_n = sig + i * (pdinfo->pd + 1);
-
-#if SUBDEBUG
-	fprintf(stderr, "block %d: sig_n = '%s'\n", i, sig_n);
-#endif
-
-	if (block_retained(sig_n)) {
-	    if (ok_blocks == 0) {
-		sig_0 = sig_n;
-	    } else {
-		if (strcmp(sig_n, sig_0)) {
-		    unbal = 1;
-		}
-	    }
-	    if (!unbal) {
-		ok_blocks++;
-	    }
-	} 
-	if (unbal) {
-	    break;
-	}
-    }
-
-    free(sig);
-
-    if (new_blocks != NULL) {
-	*new_blocks = ok_blocks;
-    }
-
-    return (ok_blocks > 0 && unbal == 0);
-}
-
-int model_mask_leaves_balanced_panel (const MODEL *pmod,
-				      const DATAINFO *pdinfo)
-{
-    return real_mask_leaves_balanced_panel(pmod->missmask, 
-					   pdinfo, NULL,
-					   pmod->t1, pmod->t2,
-					   1);
-}
-
 /* when subsampling cases from a time series, if the resulting dataset
    is still a time series without internal "holes" then automatically
    set a time series interpretation of the reduced dataset
@@ -811,35 +758,7 @@ maybe_reconstitute_time_series (const DATAINFO *pdinfo,
 
 	ntodate(stobs, t1, pdinfo);
 	sprintf(line, "setobs %d %s", pdinfo->pd, stobs);
-	set_obs(line, subinfo, OPT_NONE);
-    } 
-}
-
-/* when subsampling cases from a panel dataset, if the resulting
-   dataset is still a balanced panel then automatically set the
-   interpretation of the reduced dataset as a panel
-*/
-
-static void
-maybe_reconstitute_panel (const DATAINFO *pdinfo,
-			  const char *mask,
-			  DATAINFO *subinfo)
-{
-    int bal, ok_blocks = 0;
-
-    bal = real_mask_leaves_balanced_panel(mask, pdinfo, &ok_blocks,
-					  0, pdinfo->n - 1, 0);
-
-    if (bal && ok_blocks > 1) {
-	char line[32];
-	int pd = subinfo->n / ok_blocks;
-	gretlopt opt = OPT_C;
-
-	sprintf(line, "setobs %d 1.1", pd);
-	if (pdinfo->structure == STACKED_TIME_SERIES) {
-	    opt = OPT_S;
-	} 
-	set_obs(line, subinfo, opt);
+	set_obs(line, NULL, subinfo, OPT_NONE);
     } 
 }
 
@@ -850,13 +769,22 @@ copy_data_to_subsample (double **subZ, DATAINFO *subinfo,
 {
     int i, t, st;
 
+#if SUBDEBUG
+    fprintf(stderr, "copying data to subsample\n");
+#endif
+
     /* copy data values */
     for (i=1; i<pdinfo->v; i++) {
-	if (pdinfo->vector[i]) {
+	if (var_is_series(pdinfo, i)) {
 	    st = 0;
 	    for (t=0; t<pdinfo->n; t++) {
-		if (mask == NULL || mask[t] == 1) {
+		if (mask == NULL) {
 		    subZ[i][st++] = Z[i][t];
+		} else if (mask[t] == 1) {
+		    subZ[i][st++] = Z[i][t];
+		} else if (mask[t] == 'p') {
+		    /* panel padding */
+		    subZ[i][st++] = NADBL;
 		}
 	    }
 	} else {
@@ -868,7 +796,7 @@ copy_data_to_subsample (double **subZ, DATAINFO *subinfo,
    if (pdinfo->markers && subinfo->markers) {
        st = 0;
        for (t=0; t<pdinfo->n; t++) {
-	   if (mask == NULL || mask[t] == 1) {
+	   if (mask == NULL || mask[t] == 1 || mask[t] == 'p') {
 	       strcpy(subinfo->S[st++], pdinfo->S[t]);
 	   }
        }
@@ -895,6 +823,54 @@ int get_restriction_mode (gretlopt opt)
     return mode;
 }
 
+static int 
+make_panel_submask (char *mask, const DATAINFO *pdinfo, int *err)
+{
+    char *umask = NULL;
+    char *pmask = NULL;
+    int i, np = 0;
+
+    umask = calloc(pdinfo->paninfo->nunits, 1);
+    if (umask == NULL) {
+	*err = E_ALLOC;
+	return 0;
+    }
+
+    pmask = calloc(pdinfo->paninfo->Tmax, 1);
+    if (pmask == NULL) {
+	free(umask);
+	*err = E_ALLOC;
+	return 0;
+    }
+
+    for (i=0; i<pdinfo->n; i++) {
+	if (mask[i]) {
+	    umask[pdinfo->paninfo->unit[i]] = 1;
+	    pmask[pdinfo->paninfo->period[i]] = 1;
+	}
+    }
+
+    for (i=0; i<pdinfo->n; i++) {
+	if (!mask[i]) {
+	    if (umask[pdinfo->paninfo->unit[i]] &&
+		pmask[pdinfo->paninfo->period[i]]) {
+		mask[i] = 'p'; /* mark as padding row */
+		np++;
+	    }
+	}
+    }
+
+#if SUBDEBUG
+    fprintf(stderr, "make_panel_submask: number of padding rows = %d\n",
+	    np);
+#endif
+
+    free(umask);
+    free(pmask);
+
+    return np;
+}
+
 static int 
 make_restriction_mask (int mode, const char *line, const int *list, 
 		       double ***pZ, DATAINFO *pdinfo,
@@ -933,7 +909,7 @@ make_restriction_mask (int mode, const char *line, const int *list,
     if (oldmask != NULL && mode != SUBSAMPLE_RANDOM) {
 	sn = overlay_masks(mask, oldmask, pdinfo->n);
     } else {
-	sn = count_selected_cases(mask, pdinfo->n);
+	sn = count_selected_cases(mask, pdinfo);
     }
 
     /* does this policy lead to an empty sample, or no change in the
@@ -962,8 +938,8 @@ make_restriction_mask (int mode, const char *line, const int *list,
 }
 
 int 
-restrict_sample_from_mask (const char *mask, int mode, double ***pZ, 
-			   DATAINFO **ppdinfo)
+restrict_sample_from_mask (char *mask, int mode, 
+			   double ***pZ, DATAINFO **ppdinfo)
 {
     double **subZ = NULL;
     DATAINFO *subinfo = NULL;
@@ -975,9 +951,32 @@ restrict_sample_from_mask (const char *mask, int mode, double ***pZ,
 	return E_ALLOC;
     }
 
-    /* set up the sub-sampled datainfo */
-    subinfo->n = count_selected_cases(mask, (*ppdinfo)->n);
+    subinfo->n = count_selected_cases(mask, *ppdinfo);
     subinfo->v = (*ppdinfo)->v;
+
+#if SUBDEBUG
+    fprintf(stderr, "count_selected_cases: %d obs in subsample\n",
+	    subinfo->n);
+#endif
+
+    if (dataset_is_panel(*ppdinfo)) {
+	/* are we able to reconstitute a panel? */
+	int np, n = count_panel_units(mask, *ppdinfo);
+
+	if (n > 1 && subinfo->n > n) {
+	    /* add padding rows if need be */
+	    np = make_panel_submask(mask, *ppdinfo, &err);
+	    if (err) {
+		free(subinfo);
+		return err;
+	    }
+	    subinfo->structure = STACKED_TIME_SERIES;
+	    subinfo->n += np;
+	    subinfo->pd = subinfo->n / n;
+	}
+    }
+
+    /* set up the sub-sampled datainfo */
     if (start_new_Z(&subZ, subinfo, 1)) {
 	free(subinfo);
 	return E_ALLOC;
@@ -988,7 +987,19 @@ restrict_sample_from_mask (const char *mask, int mode, double ***pZ,
     subinfo->varname = (*ppdinfo)->varname;
     subinfo->varinfo = (*ppdinfo)->varinfo;
     subinfo->descrip = (*ppdinfo)->descrip;
-    subinfo->vector = (*ppdinfo)->vector;
+
+    if (subinfo->structure == STACKED_TIME_SERIES) {
+#if SUBDEBUG
+	fprintf(stderr, "panel subset: nT = %d, pd = %d\n", 
+		subinfo->n, subinfo->pd);
+#endif
+	err = dataset_add_default_panel_indices(subinfo);
+	if (err) {
+	    free_Z(subZ, subinfo);
+	    free(subinfo);
+	    return E_ALLOC;
+	}
+    }
 
     /* set up case markers? */
     if ((*ppdinfo)->markers) {
@@ -998,19 +1009,16 @@ restrict_sample_from_mask (const char *mask, int mode, double ***pZ,
 	    free(subinfo);
 	    return E_ALLOC;
 	}
-	subinfo->markers = (*ppdinfo)->markers;
     }
 
     /* copy across data (and case markers, if any) */
     copy_data_to_subsample(subZ, subinfo, (const double **) *pZ, 
 			   *ppdinfo, mask);
 
-    if (mode == SUBSAMPLE_USE_DUMMY || 
-	mode == SUBSAMPLE_BOOLEAN ||
-	mode == SUBSAMPLE_DROP_MISSING) {
-	if (dataset_is_panel(*ppdinfo)) {
-	    maybe_reconstitute_panel(*ppdinfo, mask, subinfo);
-	} else if (dataset_is_time_series(*ppdinfo)) {
+    if (dataset_is_time_series(*ppdinfo)) {
+	if (mode == SUBSAMPLE_USE_DUMMY || 
+	    mode == SUBSAMPLE_BOOLEAN ||
+	    mode == SUBSAMPLE_DROP_MISSING) {
 	    maybe_reconstitute_time_series(*ppdinfo, mask, subinfo);
 	}
     }
@@ -1159,7 +1167,9 @@ int set_sample (const char *line, const double **Z, DATAINFO *pdinfo)
 	return db_set_sample(line, pdinfo);
     }
 
-    if (nf == 1) return 0;
+    if (nf == 1) {
+	return 0;
+    }
 	
     if (nf == 2) {
 	if (sscanf(line, "%4s %10s", cmd, newstart) != 2) {
@@ -1224,7 +1234,7 @@ int count_missing_values (double ***pZ, DATAINFO *pdinfo, PRN *prn)
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	tmiss = 0;
 	for (i=1; i<pdinfo->v; i++) {
-	    if (is_hidden_variable(i, pdinfo) || !pdinfo->vector[i]) {
+	    if (var_is_hidden(pdinfo, i) || var_is_scalar(pdinfo, i)) {
 		continue;
 	    }
 	    if (na((*pZ)[i][t])) {
@@ -1277,13 +1287,6 @@ int count_missing_values (double ***pZ, DATAINFO *pdinfo, PRN *prn)
     return missval;
 }
 
-static void copy_varinfo (VARINFO *dest, const VARINFO *src)
-{
-    strcpy(dest->label, src->label);
-    strcpy(dest->display_name, src->display_name);
-    dest->compact_method = src->compact_method;
-}
-
 static void copy_series_info (DATAINFO *dest, const DATAINFO *src)
 {
     int i;
@@ -1293,7 +1296,6 @@ static void copy_series_info (DATAINFO *dest, const DATAINFO *src)
 	if (src->varinfo != NULL) {
 	    copy_varinfo(dest->varinfo[i], src->varinfo[i]);
 	}
-	dest->vector[i] = src->vector[i];	
     }
 }
 
