@@ -31,10 +31,20 @@ typedef enum {
     SUBSAMPLE_UNKNOWN
 } SubsampleMode;
 
+double ***fetch_full_Z (void);
+
+void reset_full_Z (double ***pZ);
+
+DATAINFO *fetch_full_datainfo (void);
+
 char *copy_subsample_mask (const char *src);
 
 char *copy_datainfo_submask (const DATAINFO *pdinfo);
 
+int write_datainfo_submask (const DATAINFO *pdinfo, FILE *fp);
+
+int write_model_submask (const MODEL *pmod, FILE *fp);
+
 int submask_cmp (const char *m1, const char *m2);
 
 int attach_subsample_to_model (MODEL *pmod, const DATAINFO *pdinfo);
@@ -44,7 +54,7 @@ int restrict_sample (const char *line, const int *list,
 		     gretlopt oflag, PRN *prn);
 
 int 
-restrict_sample_from_mask (const char *mask, int mode, 
+restrict_sample_from_mask (char *mask, int mode, 
 			   double ***pZ, DATAINFO **ppdinfo);
 
 int complex_subsampled (void);
@@ -63,7 +73,4 @@ void free_model_dataset (MODEL *pmod);
 
 void maybe_free_full_dataset (const DATAINFO *pdinfo);
 
-int model_mask_leaves_balanced_panel (const MODEL *pmod,
-				      const DATAINFO *pdinfo);
-
 #endif /* SUBSAMPLE_H */
