@@ -21,8 +21,8 @@
 
 #include "libgretl.h"
 #include "system.h"
-#include "system_private.h"
 #include "objstack.h"
+#include "gretl_xml.h"
 
 enum {
     OP_PLUS,
@@ -89,43 +89,90 @@ const char *nosystem = N_("No system of equations has been defined");
 const char *badsystem = N_("Unrecognized equation system type");
 const char *toofew = N_("An equation system must have at least two equations");
 
-static void destroy_ident (identity *pident);
+static void destroy_ident (identity *ident);
 static int make_instrument_list (gretl_equation_system *sys);
 
 static void 
-print_system_identity (const identity *pident, const DATAINFO *pdinfo, 
+print_system_equation (const int *list, const DATAINFO *pdinfo, 
 		       PRN *prn)
 {
-    int i;
+    int i, v;
 
-    pprintf(prn, "Identity: %s = %s ", 
-	    pdinfo->varname[pident->depvar],
-	    pdinfo->varname[pident->atoms[0].varnum]);
+    pputs(prn, "equation");
 
-    for (i=1; i<pident->n_atoms; i++) {
-	pprintf(prn, "%c %s ", (pident->atoms[i].op == OP_PLUS)? '+' : '-',
-		pdinfo->varname[pident->atoms[i].varnum]);
+    for (i=1; i<=list[0]; i++) {
+	v = list[i];
+	if (v == LISTSEP) {
+	    pputs(prn, " ;");
+	} else if (v > 0 && v < pdinfo->v) {
+	    pprintf(prn, " %s", pdinfo->varname[v]);
+	} else {
+	    pprintf(prn, " %d", v);
+	}
     }
 
     pputc(prn, '\n');
 }
 
-void 
-print_equation_system_info (const gretl_equation_system *sys, 
-			    const DATAINFO *pdinfo, PRN *prn)
+static void 
+print_system_identity (const identity *ident, const DATAINFO *pdinfo, 
+		       gretlopt opt, PRN *prn)
 {
     int i;
 
-    if (sys->name != NULL) {
+    if (opt & OPT_H) {
+	pprintf(prn, "Identity: %s = %s ", 
+		pdinfo->varname[ident->depvar],
+		pdinfo->varname[ident->atoms[0].varnum]);
+    } else {
+	pprintf(prn, "identity %s = %s ", 
+		pdinfo->varname[ident->depvar],
+		pdinfo->varname[ident->atoms[0].varnum]);
+    }
+
+    for (i=1; i<ident->n_atoms; i++) {
+	pprintf(prn, "%c %s ", (ident->atoms[i].op == OP_PLUS)? '+' : '-',
+		pdinfo->varname[ident->atoms[i].varnum]);
+    }
+
+    pputc(prn, '\n');
+}
+
+/**
+ * print_equation_system_info:
+ * @sys: gretl equation system.
+ * @pdinfo: dataset information.
+ * @opt: use %OPT_H for printing in a form designed to appear
+ * in the header when results of estimation are printed.
+ * @prn: printing struct.
+ * 
+ * Prints details of an equation system to @prn.
+ */
+
+void 
+print_equation_system_info (const gretl_equation_system *sys, 
+			    const DATAINFO *pdinfo, 
+			    gretlopt opt, PRN *prn)
+{
+    int i;
+
+    
+    if ((opt & OPT_H) && sys->name != NULL) {
 	pprintf(prn, "Equation system %s\n", sys->name);
     }
 
+    if (!(opt & OPT_H)) {
+	for (i=0; i<sys->n_equations; i++) {
+	    print_system_equation(sys->lists[i], pdinfo, prn);
+	}    
+    }
+
     for (i=0; i<sys->n_identities; i++) {
-	print_system_identity(sys->idents[i], pdinfo, prn);
+	print_system_identity(sys->idents[i], pdinfo, opt, prn);
     }
 
     if (sys->endog_vars != NULL) {
-	pputs(prn, "Endogenous variables:");
+	pputs(prn, (opt & OPT_H)? "Endogenous variables:" : "endog");
 	for (i=1; i<=sys->endog_vars[0]; i++) {
 	    pprintf(prn, " %s", pdinfo->varname[sys->endog_vars[i]]);
 	}
@@ -133,13 +180,12 @@ print_equation_system_info (const gretl_equation_system *sys,
     }
 
     if (sys->instr_vars != NULL) {
-	pputs(prn, "Exogenous variables:");
+	pputs(prn, (opt & OPT_H)? "Exogenous variables:" : "instr");
 	for (i=1; i<=sys->instr_vars[0]; i++) {
 	    pprintf(prn, " %s", pdinfo->varname[sys->instr_vars[i]]);
 	}
 	pputc(prn, '\n');
     }
-
 }
 
 int gretl_system_method_from_string (const char *s)
@@ -616,7 +662,7 @@ system_print_F_test (const gretl_equation_system *sys,
 
     pprintf(prn, "%s:\n", _("F test for the specified restrictions"));
     pprintf(prn, "  F(%d,%d) = %g %s %g\n", dfn, dfu, F,
-	    _("with p-value"), fdist(F, dfn, dfu));
+	    _("with p-value"), f_cdf_comp(F, dfn, dfu));
     pputc(prn, '\n');    
 
  bailout:
@@ -630,7 +676,7 @@ static void
 system_print_LR_test (const gretl_equation_system *sys,
 		      double llu, PRN *prn)
 {
-    double llr = system_get_ll(sys);
+    double llr = sys->ll;
     int df = gretl_matrix_rows(sys->R);
     double X2;
 
@@ -645,7 +691,7 @@ system_print_LR_test (const gretl_equation_system *sys,
     pprintf(prn, "  %s = %g\n", _("Restricted log-likelihood"), llr);
     pprintf(prn, "  %s = %g\n", _("Unrestricted log-likelihood"), llu);
     pprintf(prn, "  %s(%d) = %g %s %g\n", _("Chi-square"),
-	    df, X2, _("with p-value"), chisq(X2, df));
+	    df, X2, _("with p-value"), chisq_cdf_comp(X2, df));
     pputc(prn, '\n');    
 }
 
@@ -738,6 +784,60 @@ static int estimate_with_test (gretl_equation_system *sys,
     return err;
 }
 
+static void 
+adjust_sys_flags_for_method (gretl_equation_system *sys, int method)
+{
+    char old_flags = sys->flags;
+
+    sys->flags = 0;
+
+    if (old_flags & GRETL_SYS_ITERATE) {
+	/* the iterate option is only available for WLS, SUR or 3SLS */
+	if (sys->method == SYS_WLS || sys->method == SYS_SUR ||
+	    sys->method == SYS_3SLS) {
+	    sys->flags |= GRETL_SYS_ITERATE;
+	}
+    }
+
+    /* by default, apply a df correction for single-equation methods */
+    if (sys->method == SYS_OLS || sys->method == SYS_WLS ||
+	sys->method == SYS_TSLS || sys->method == SYS_LIML) {
+	if (old_flags & GRETL_SYSTEM_DFCORR) {
+	    sys->flags |= GRETL_SYSTEM_DFCORR;
+	}
+    } 
+
+    if (old_flags & GRETL_SYS_VCV_GEOMEAN) {
+	sys->flags |= GRETL_SYS_VCV_GEOMEAN;
+    }    
+}
+
+static void 
+set_sys_flags_from_opt (gretl_equation_system *sys, gretlopt opt)
+{
+    sys->flags = 0;
+
+    if (opt & OPT_T) {
+	/* the iterate option is only available for WLS, SUR or 3SLS */
+	if (sys->method == SYS_WLS || sys->method == SYS_SUR ||
+	    sys->method == SYS_3SLS) {
+	    sys->flags |= GRETL_SYS_ITERATE;
+	}
+    }
+
+    /* by default, apply a df correction for single-equation methods */
+    if (sys->method == SYS_OLS || sys->method == SYS_WLS ||
+	sys->method == SYS_TSLS || sys->method == SYS_LIML) {
+	if (!(opt & OPT_N)) {
+	    sys->flags |= GRETL_SYSTEM_DFCORR;
+	}
+    } 
+
+    if (opt & OPT_M) {
+	sys->flags |= GRETL_SYS_VCV_GEOMEAN;
+    } 
+}
+
 /**
  * gretl_equation_system_estimate:
  * @sys: pre-defined equation system.
@@ -765,6 +865,18 @@ gretl_equation_system_estimate (gretl_equation_system *sys,
 
     *gretl_errmsg = 0;
 
+    if (opt == OPT_UNSET) {
+	opt = OPT_NONE;
+	adjust_sys_flags_for_method(sys, sys->method);
+    } else {
+	set_sys_flags_from_opt(sys, opt);
+    }
+
+    /* if restrictions are in place, reset the restricted flag */
+    if (sys->R != NULL && sys->q != NULL) {
+	sys->flags |= GRETL_SYS_RESTRICT;
+    } 
+
     /* in case we're re-estimating */
     system_clear_results(sys);
 
@@ -785,7 +897,7 @@ gretl_equation_system_estimate (gretl_equation_system *sys,
     stest = sys_test_type(sys);
 
     if (stest == SYS_TEST_NOTIMP) {
-	pprintf(prn, _("Sorry, command not available for this estimator"));
+	pputs(prn, _("Sorry, command not available for this estimator"));
 	pputc(prn, '\n');
 	err = 1;
     } else if (stest != SYS_NO_TEST) {
@@ -802,7 +914,7 @@ gretl_equation_system_estimate (gretl_equation_system *sys,
     }
 
     if (!err) {
-	set_as_last_model(sys, SYSTEM);
+	set_as_last_model(sys, GRETL_OBJ_SYS);
     } 
 
     return err;
@@ -862,59 +974,6 @@ int gretl_equation_system_finalize (gretl_equation_system *sys,
     return err;
 }
 
-static void 
-adjust_sys_flags_for_method (gretl_equation_system *sys, int method)
-{
-    char old_flags = sys->flags;
-
-    sys->flags = 0;
-
-    if (old_flags & GRETL_SYS_ITERATE) {
-	/* the iterate option is only available for WLS, SUR or 3SLS */
-	if (sys->method == SYS_WLS || sys->method == SYS_SUR ||
-	    sys->method == SYS_3SLS) {
-	    sys->flags |= GRETL_SYS_ITERATE;
-	}
-    }
-
-    /* by default, apply a df correction for single-equation methods */
-    if (sys->method == SYS_OLS || sys->method == SYS_WLS ||
-	sys->method == SYS_TSLS || sys->method == SYS_LIML) {
-	if (old_flags & GRETL_SYSTEM_DFCORR) {
-	    sys->flags |= GRETL_SYSTEM_DFCORR;
-	}
-    } 
-
-    if (old_flags & GRETL_SYS_VCV_GEOMEAN) {
-	sys->flags |= GRETL_SYS_VCV_GEOMEAN;
-    }    
-}
-
-static void 
-set_sys_flags_from_opt (gretl_equation_system *sys, gretlopt opt)
-{
-    sys->flags = 0;
-
-    if (opt & OPT_T) {
-	/* the iterate option is only available for WLS, SUR or 3SLS */
-	if (sys->method == SYS_WLS || sys->method == SYS_SUR ||
-	    sys->method == SYS_3SLS) {
-	    sys->flags |= GRETL_SYS_ITERATE;
-	}
-    }
-
-    /* by default, apply a df correction for single-equation methods */
-    if (sys->method == SYS_OLS || sys->method == SYS_WLS ||
-	sys->method == SYS_TSLS || sys->method == SYS_LIML) {
-	if (!(opt & OPT_N)) {
-	    sys->flags |= GRETL_SYSTEM_DFCORR;
-	}
-    } 
-
-    if (opt & OPT_M) {
-	sys->flags |= GRETL_SYS_VCV_GEOMEAN;
-    } 
-}
 
 /* Implement the "estimate" command, which must give the name of a pre-defined
    equation system and an estimation method, as in:
@@ -942,7 +1001,7 @@ int estimate_named_system (const char *line, double ***pZ, DATAINFO *pdinfo,
 
     sys = get_equation_system_by_name(sysname);
     if (sys == NULL) {
-	sprintf(gretl_errmsg, "'%s': unrecognized name", sysname);
+	sprintf(gretl_errmsg, _("'%s': unrecognized name"), sysname);
 	free(sysname);
 	return 1;
     }
@@ -961,33 +1020,9 @@ int estimate_named_system (const char *line, double ***pZ, DATAINFO *pdinfo,
 
     sys->method = method;
 
-    if (opt == OPT_UNSET) {
-	opt = OPT_NONE;
-	adjust_sys_flags_for_method(sys, method);
-    } else {
-	set_sys_flags_from_opt(sys, opt);
-    }
-
-    /* if restrictions are in place, reset the restricted flag */
-    if (sys->R != NULL && sys->q != NULL) {
-	sys->flags |= GRETL_SYS_RESTRICT;
-    } 
-
     return gretl_equation_system_estimate(sys, pZ, pdinfo, opt, prn);
 }
 
-int estimate_saved_equation_system (gretl_equation_system *sys, 
-				    double ***pZ, DATAINFO *pdinfo,
-				    PRN *prn)
-{
-    char line[128];
-    int err;
-
-    sprintf(line, "estimate \"%s\"", sys->name);
-    err = estimate_named_system(line, pZ, pdinfo, OPT_UNSET, prn);
-    return err;
-}
-
 static int get_real_list_length (const int *list)
 {
     int i, len = list[0];
@@ -1046,11 +1081,6 @@ void gretl_system_set_name (gretl_equation_system *sys, const char *name)
     sys->name = gretl_strdup(name);
 }
 
-const char *gretl_system_get_name (const gretl_equation_system *sys)
-{
-    return sys->name;
-}
-
 int system_adjust_t1t2 (gretl_equation_system *sys,
 			int *t1, int *t2, const double **Z)
 {
@@ -1105,16 +1135,6 @@ int *compose_tsls_list (gretl_equation_system *sys, int i)
     return list;
 }
 
-void system_set_n_obs (gretl_equation_system *sys, int n)
-{
-    sys->n_obs = n;
-}
-
-void system_set_iters (gretl_equation_system *sys, int n)
-{
-    sys->iters = n;
-}
-
 int system_normality_test (const gretl_equation_system *sys, PRN *prn)
 {
     int err = 0;
@@ -1138,7 +1158,7 @@ void make_system_data_info (gretl_equation_system *sys, int eqn,
 	} else if (sys->method == SYS_3SLS) {
 	    sprintf(VARLABEL(pdinfo, v), _("3SLS residual, equation %d"), eqn);
 	} else {
-	    sprintf(VARLABEL(pdinfo, v), "system residual, equation %d", eqn);
+	    sprintf(VARLABEL(pdinfo, v), _("system residual, equation %d"), eqn);
 	}
     } else if (code == GRETL_SYSTEM_SAVE_YHAT) {
 	sprintf(pdinfo->varname[v], "yhat_s%02d", eqn);
@@ -1147,7 +1167,7 @@ void make_system_data_info (gretl_equation_system *sys, int eqn,
 	} else if (sys->method == SYS_3SLS) {
 	    sprintf(VARLABEL(pdinfo, v), _("3SLS fitted value, equation %d"), eqn);
 	} else {
-	    sprintf(VARLABEL(pdinfo, v), "system fitted value, equation %d", eqn);
+	    sprintf(VARLABEL(pdinfo, v), _("system fitted value, equation %d"), eqn);
 	}
     }
 }
@@ -1194,20 +1214,6 @@ const char *system_get_full_string (const gretl_equation_system *sys)
     }
 }
 
-const gretl_matrix *system_get_R_matrix (const gretl_equation_system *sys)
-{
-    if (sys->flags & GRETL_SYS_RESTRICT) {
-	return sys->R;
-    } else {
-	return NULL;
-    }
-}
-
-const gretl_matrix *system_get_q_matrix (const gretl_equation_system *sys)
-{
-    return sys->q;
-}
-
 int system_save_uhat (const gretl_equation_system *sys)
 {
     return sys->flags & GRETL_SYSTEM_SAVE_UHAT;
@@ -1228,16 +1234,6 @@ int system_want_df_corr (const gretl_equation_system *sys)
     return sys->flags & GRETL_SYSTEM_DFCORR;
 }
 
-int system_n_equations (const gretl_equation_system *sys)
-{
-    return sys->n_equations;
-}
-
-int system_n_indentities (const gretl_equation_system *sys)
-{
-    return sys->n_identities;
-}
-
 int system_n_restrictions (const gretl_equation_system *sys)
 {
     int nr = 0;
@@ -1249,16 +1245,6 @@ int system_n_restrictions (const gretl_equation_system *sys)
     return nr;
 }
 
-int system_n_obs (const gretl_equation_system *sys)
-{
-    return sys->n_obs;
-}
-
-int system_iters (const gretl_equation_system *sys)
-{
-    return sys->iters;
-}
-
 int *system_get_list (const gretl_equation_system *sys, int i)
 {
     if (i >= sys->n_equations) return NULL;
@@ -1288,11 +1274,6 @@ int *system_get_instr_vars (const gretl_equation_system *sys)
     return sys->instr_vars;
 }
 
-gretl_matrix *system_get_uhat (const gretl_equation_system *sys)
-{
-    return sys->uhat;
-}
-
 void system_attach_coeffs (gretl_equation_system *sys, gretl_matrix *b)
 {
     if (sys->b != NULL) {
@@ -1329,11 +1310,6 @@ void system_attach_uhat (gretl_equation_system *sys, gretl_matrix *uhat)
     sys->uhat = uhat;
 }
 
-gretl_matrix *system_get_sigma (const gretl_equation_system *sys)
-{
-    return sys->sigma;
-}
-
 MODEL *system_get_model (const gretl_equation_system *sys, int i)
 {
     if (sys->models == NULL || i >= sys->n_equations) {
@@ -1343,56 +1319,6 @@ MODEL *system_get_model (const gretl_equation_system *sys, int i)
     return sys->models[i];
 }
 
-double system_get_ll (const gretl_equation_system *sys)
-{
-    return sys->ll;
-}
-
-double system_get_llu (const gretl_equation_system *sys)
-{
-    return sys->llu;
-}
-
-double system_get_ess (const gretl_equation_system *sys)
-{
-    return sys->ess;
-}
-
-double system_get_X2 (const gretl_equation_system *sys)
-{
-    return sys->X2;
-}
-
-double system_get_diag_stat (const gretl_equation_system *sys)
-{
-    return sys->diag;
-}
-
-void system_set_ll (gretl_equation_system *sys, double ll)
-{
-    sys->ll = ll;
-}
-
-void system_set_llu (gretl_equation_system *sys, double llu)
-{
-    sys->llu = llu;
-}
-
-void system_set_X2 (gretl_equation_system *sys, double X2)
-{
-    sys->X2 = X2;
-}
-
-void system_set_ess (gretl_equation_system *sys, double ess)
-{
-    sys->ess = ess;
-}
-
-void system_set_diag_stat (gretl_equation_system *sys, double s)
-{
-    sys->diag = s;
-}
-
 /* for case of applying df correction to cross-equation 
    residual covariance matrix */
 
@@ -1501,33 +1427,33 @@ int rhs_var_in_identity (const gretl_equation_system *sys, int lhsvar,
     return 0;
 }
 
-static void destroy_ident (identity *pident)
+static void destroy_ident (identity *ident)
 {
-    free(pident->atoms);
-    free(pident);
+    free(ident->atoms);
+    free(ident);
 }
 
 static identity *ident_new (int nv)
 {
-    identity *pident;
+    identity *ident;
 
-    pident = malloc(sizeof *pident);
-    if (pident == NULL) return NULL;
+    ident = malloc(sizeof *ident);
+    if (ident == NULL) return NULL;
 
-    pident->n_atoms = nv;
-    pident->atoms = malloc(nv * sizeof *pident->atoms);
-    if (pident->atoms == NULL) {
-	free(pident);
-	pident = NULL;
+    ident->n_atoms = nv;
+    ident->atoms = malloc(nv * sizeof *ident->atoms);
+    if (ident->atoms == NULL) {
+	free(ident);
+	ident = NULL;
     }
 
-    return pident;
+    return ident;
 }
 
 static identity *
 parse_identity (const char *str, const DATAINFO *pdinfo, int *err)
 {
-    identity *pident;
+    identity *ident;
     const char *p;
     char f1[24], f2[16];
     char op, vname1[VNAMELEN], vname2[VNAMELEN];
@@ -1548,23 +1474,23 @@ parse_identity (const char *str, const DATAINFO *pdinfo, int *err)
 	p++;
     }
 
-    pident = ident_new(nv);
-    if (pident == NULL) {
+    ident = ident_new(nv);
+    if (ident == NULL) {
 	*err = E_ALLOC;
 	return NULL;
     }
 
-    pident->depvar = varindex(pdinfo, vname1);
-    if (pident->depvar == pdinfo->v) {
-	destroy_ident(pident);
+    ident->depvar = varindex(pdinfo, vname1);
+    if (ident->depvar == pdinfo->v) {
+	destroy_ident(ident);
 	*err = E_UNKVAR;
 	return NULL;
     }
 
-    pident->atoms[0].op = OP_PLUS;
-    pident->atoms[0].varnum = varindex(pdinfo, vname2);
-    if (pident->atoms[0].varnum == pdinfo->v) {
-	destroy_ident(pident);
+    ident->atoms[0].op = OP_PLUS;
+    ident->atoms[0].varnum = varindex(pdinfo, vname2);
+    if (ident->atoms[0].varnum == pdinfo->v) {
+	destroy_ident(ident);
 	*err = E_UNKVAR;
 	return NULL;
     }
@@ -1577,9 +1503,9 @@ parse_identity (const char *str, const DATAINFO *pdinfo, int *err)
 	else if (op == '-') op = OP_MINUS;
 	else *err = E_PARSE;
 	if (!*err) {
-	    pident->atoms[i].op = op;
-	    pident->atoms[i].varnum = varindex(pdinfo, vname1);
-	    if (pident->atoms[i].varnum == pdinfo->v) {
+	    ident->atoms[i].op = op;
+	    ident->atoms[i].varnum = varindex(pdinfo, vname1);
+	    if (ident->atoms[i].varnum == pdinfo->v) {
 		*err = E_UNKVAR;
 	    }
 	}
@@ -1587,34 +1513,34 @@ parse_identity (const char *str, const DATAINFO *pdinfo, int *err)
     }
 
     if (*err) {
-	destroy_ident(pident);
-	pident = NULL;
+	destroy_ident(ident);
+	ident = NULL;
     }
        
-    return pident;
+    return ident;
 }
 
 static int 
 add_identity_to_sys (gretl_equation_system *sys, const char *line,
 		     const DATAINFO *pdinfo)
 {
-    identity **ppident;
-    identity *pident;
+    identity **pident;
+    identity *ident;
     int ni = sys->n_identities;
     int err = 0;
 
-    pident = parse_identity(line, pdinfo, &err);
-    if (pident == NULL) return err;
+    ident = parse_identity(line, pdinfo, &err);
+    if (ident == NULL) return err;
 
     /* connect the identity to the equation system */
-    ppident = realloc(sys->idents, (ni + 1) * sizeof *sys->idents);
-    if (ppident == NULL) {
-	destroy_ident(pident);
+    pident = realloc(sys->idents, (ni + 1) * sizeof *sys->idents);
+    if (pident == NULL) {
+	destroy_ident(ident);
 	return E_ALLOC;
     }
 
-    sys->idents = ppident;
-    sys->idents[ni] = pident;
+    sys->idents = pident;
+    sys->idents[ni] = ident;
     sys->n_identities += 1;
 
     return 0;
@@ -1679,7 +1605,7 @@ add_aux_list_to_sys (gretl_equation_system *sys, const char *line,
 	    int *sublist = get_list_by_name(vname);
 
 	    if (sublist == NULL) {
-		sprintf(gretl_errmsg, "Undefined variable '%s'.", vname);
+		sprintf(gretl_errmsg, _("Undefined variable '%s'."), vname);
 		err = 1;
 	    } else {
 		err = gretl_list_insert_list_minus(&list, sublist, j);
@@ -1975,3 +1901,176 @@ int highest_numbered_var_in_system (const gretl_equation_system *sys,
 
     return vmax;
 }
+
+static identity *
+sys_retrieve_identity (xmlNodePtr node, xmlDocPtr doc, int *err)
+{
+    identity *ident;
+    xmlNodePtr cur;
+    int n_atoms, depvar;
+    int i, got = 0;
+
+    got += gretl_xml_get_prop_as_int(node, "n_atoms", &n_atoms);
+    got += gretl_xml_get_prop_as_int(node, "depvar", &depvar);
+    if (got < 2) {
+	*err = E_DATA;
+	return NULL;
+    }
+
+    ident = ident_new(n_atoms);
+    if (ident == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    ident->depvar = depvar;
+
+    cur = node->xmlChildrenNode;
+
+    i = 0;
+    while (cur != NULL && !*err) {
+	if (!xmlStrcmp(cur->name, (XUC) "id_atom")) {
+	    got = gretl_xml_get_prop_as_int(cur, "op", &ident->atoms[i].op);
+	    got += gretl_xml_get_prop_as_int(cur, "varnum", &ident->atoms[i].varnum);
+	    if (got < 2) {
+		*err = E_DATA;
+	    } else {
+		i++;
+	    }
+	}
+	cur = cur->next;
+    }
+
+    if (!*err && i != n_atoms) {
+	*err = E_DATA;
+    }
+
+    if (*err) {
+	destroy_ident(ident);
+	ident = NULL;
+    }
+
+    return ident;
+}
+
+gretl_equation_system *
+gretl_system_from_XML (xmlNodePtr node, xmlDocPtr doc, int *err)
+{
+    gretl_equation_system *sys;
+    xmlNodePtr cur;
+    char *name;
+    int method = 0;
+    int i, j, got = 0;
+
+    got += gretl_xml_get_prop_as_string(node, "name", &name);
+    got += gretl_xml_get_prop_as_int(node, "method", &method);
+
+    if (got < 2) {
+	*err = E_DATA;
+	return NULL;
+    }     
+
+    sys = gretl_equation_system_new(method, name);
+    if (sys == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    got = 0;
+    got += gretl_xml_get_prop_as_int(node, "n_equations", &sys->n_equations);
+    got += gretl_xml_get_prop_as_int(node, "n_identities", &sys->n_identities);
+    got += gretl_xml_get_prop_as_char(node, "flags", &sys->flags);
+
+    if (got < 3) {
+	*err = E_DATA;
+	goto bailout;
+    } 
+
+    sys->lists = malloc(sys->n_equations * sizeof sys->lists);
+    if (sys->lists == NULL) {
+	*err = E_ALLOC;
+	goto bailout;
+    }
+
+    if (sys->n_identities > 0) {
+	sys->idents = malloc(sys->n_identities * sizeof sys->idents);
+	if (sys->idents == NULL) {
+	    *err = E_ALLOC;
+	    goto bailout;
+	}
+    }
+
+    cur = node->xmlChildrenNode;
+
+    i = j = 0;
+    while (cur != NULL && !*err) {
+	if (!xmlStrcmp(cur->name, (XUC) "eqnlist")) {
+	    sys->lists[i++] = gretl_xml_node_get_list(cur, doc, err); 
+	} else if (!xmlStrcmp(cur->name, (XUC) "endog_vars")) {
+	    sys->endog_vars = gretl_xml_node_get_list(cur, doc, err);
+	} else if (!xmlStrcmp(cur->name, (XUC) "instr_vars")) {
+	    sys->instr_vars = gretl_xml_node_get_list(cur, doc, err);
+	} else if (!xmlStrcmp(cur->name, (XUC) "identity")) {
+	    sys->idents[j++] = sys_retrieve_identity(cur, doc, err); 
+	} else if (!xmlStrcmp(cur->name, (XUC) "R")) {
+	    sys->R = gretl_xml_get_matrix(cur, doc, err);
+	} else if (!xmlStrcmp(cur->name, (XUC) "q")) {
+	    sys->q = gretl_xml_get_matrix(cur, doc, err);
+	}
+	cur = cur->next;
+    } 
+
+    if (!*err && (i != sys->n_equations || j != sys->n_identities)) {
+	*err = E_DATA;
+	gretl_equation_system_destroy(sys);
+	sys = NULL;
+    }
+
+ bailout:
+
+    return sys;
+}
+
+static void xml_print_identity (identity *ident, FILE *fp)
+{
+    int i;
+
+    fprintf(fp, "<identity n_atoms=\"%d\" depvar=\"%d\">\n",
+	    ident->n_atoms, ident->depvar);
+    for (i=0; i<ident->n_atoms; i++) {
+	fprintf(fp, " <id_atom op=\"%d\" varnum=\"%d\"/>\n",
+		ident->atoms[i].op, ident->atoms[i].varnum);
+    }
+    fputs("</identity>\n", fp);
+}
+
+int gretl_system_serialize (gretl_equation_system *sys, SavedObjectFlags flags,
+			    FILE *fp)
+{
+    int i, err = 0;
+
+    fprintf(fp, "<gretl-equation-system name=\"%s\" saveflags=\"%d\" method=\"%d\" ",  
+	    (sys->name != NULL)? sys->name : "none", flags, sys->method);
+
+    fprintf(fp, "n_equations=\"%d\" n_identities=\"%d\" flags=\"%d\">\n",
+	    sys->n_equations, sys->n_identities, (int) sys->flags);
+
+    for (i=0; i<sys->n_equations; i++) {
+	gretl_xml_put_tagged_list("eqnlist", sys->lists[i], fp);
+    }
+
+    gretl_xml_put_tagged_list("endog_vars", sys->endog_vars, fp);
+    gretl_xml_put_tagged_list("instr_vars", sys->instr_vars, fp);
+
+    for (i=0; i<sys->n_identities; i++) {
+	xml_print_identity(sys->idents[i], fp);
+    }    
+
+    gretl_xml_put_matrix(sys->R, "R", fp);
+    gretl_xml_put_matrix(sys->q, "q", fp);
+
+    fputs("</gretl-equation-system>\n", fp);
+
+    return err;
+}
+
