@@ -61,60 +61,67 @@ static int check_vals (const double *x, const double *y, int n)
 }
 
 static int 
-make_transform_varname (char *vname, const char *orig, int cmd, 
-			int lag, int len)
+make_transform_varname (char *vname, const char *orig, int ci, 
+			int aux, int len)
 {
     *vname = '\0';
 
-    if (cmd == DIFF) {
+    if (ci == DIFF) {
 	strcpy(vname, "d_");
 	strncat(vname, orig, len - 2);
-    } else if (cmd == LDIFF) {
+    } else if (ci == LDIFF) {
 	strcpy(vname, "ld_");
 	strncat(vname, orig, len - 3);
-    } else if (cmd == SDIFF) {
+    } else if (ci == SDIFF) {
 	strcpy(vname, "sd_");
 	strncat(vname, orig, len - 3);
-    } else if (cmd == LOGS) {
+    } else if (ci == LOGS) {
 	strcpy(vname, "l_");
 	strncat(vname, orig, len - 2);
-    } else if (cmd == SQUARE) {
+    } else if (ci == SQUARE) {
 	strcpy(vname, "sq_");
 	strncat(vname, orig, len - 3);
-    } else if (cmd == LAGS) {
+    } else if (ci == LAGS) {
 	char ext[6];
 
-	if (lag >= 0) {
+	if (aux >= 0) {
 	    /* an actual lag */
-	    sprintf(ext, "_%d", lag);
+	    sprintf(ext, "_%d", aux);
 	} else {
 	    /* in fact a lead */
-	    sprintf(ext, "%d", -lag);
+	    sprintf(ext, "%d", -aux);
 	}
 	strncat(vname, orig, len - strlen(ext));
 	strcat(vname, ext);
-    }
+    } else if (ci == DUMMIFY) {
+	char ext[6];
+
+	sprintf(ext, "_%d", aux);
+	strcpy(vname, "D");
+	strncat(vname, orig, len - strlen(ext) - 1);
+	strcat(vname, ext);
+    }	
 
     return 0;
 }
 
 static int
 make_transform_label (char *label, const char *parent,
-		      int cmd, int lag)
+		      int ci, int lag)
 {
     int err = 0;
 
-    if (cmd == DIFF) {
+    if (ci == DIFF) {
 	sprintf(label, _("= first difference of %s"), parent);
-    } else if (cmd == LDIFF) {
+    } else if (ci == LDIFF) {
 	sprintf(label, _("= log difference of %s"), parent);
-    } else if (cmd == SDIFF) {
+    } else if (ci == SDIFF) {
 	sprintf(label, _("= seasonal difference of %s"), parent);
-    } else if (cmd == LOGS) {
+    } else if (ci == LOGS) {
 	sprintf(label, _("= log of %s"), parent);
-    } else if (cmd == SQUARE) {
+    } else if (ci == SQUARE) {
 	sprintf(label, _("= %s squared"), parent);
-    } else if (cmd == LAGS) {
+    } else if (ci == LAGS) {
 	if (lag >= 0) {
 	    sprintf(label, "= %s(t - %d)", parent, lag);
 	} else {
@@ -127,13 +134,18 @@ make_transform_label (char *label, const char *parent,
     return err;
 }
 
-int is_standard_lag (int v, const DATAINFO *pdinfo)
+int is_standard_lag (int v, const DATAINFO *pdinfo, int *parent)
 {
     const char *test = VARLABEL(pdinfo, v);
     char pm, vname[VNAMELEN];
     int lag, ret = 0;
 
     if (sscanf(test, "= %15[^(](t %c %d)", vname, &pm, &lag) == 3) {
+	if (parent != NULL) {
+	    int pv = varindex(pdinfo, vname);
+
+	    *parent = (pv < pdinfo->v)? pv : 0;
+	}
 	ret = 1;
     }
 
@@ -209,16 +221,7 @@ static int get_lag (int v, int lag, double *lagvec,
 	lagvec[t] = NADBL;
     }
 
-    if (pdinfo->structure == STACKED_CROSS_SECTION) {
-	/* needs rather special handling */
-	for (t=t1; t<=t2; t++) { 
-	    lt = t - lag * pdinfo->pd;
-	    if (lt < 0 || lt >= pdinfo->n) {
-		continue;
-	    }
-	    lagvec[t] = Z[v][lt];
-	}
-    } else if (dated_daily_data(pdinfo)) {
+    if (dated_daily_data(pdinfo)) {
 	for (t=t1; t<=t2; t++) {
 	    lt = t - lag;
 	    while (lt >= 0 && na(Z[v][lt])) {
@@ -264,10 +267,10 @@ static int get_log (int v, double *logvec, const double **Z,
     int t, err = 0;
 
     for (t=pdinfo->t1; t<=pdinfo->t2 && !err; t++) {
-	xx = (pdinfo->vector[v])? Z[v][t] : Z[v][0];
+	xx = (var_is_series(pdinfo, v))? Z[v][t] : Z[v][0];
 	if (na(xx)) {
 	    logvec[t] = NADBL;
-	} else if (xx <= 0.0) {
+	} else if (xx <= 0.0) { /* ?? treatment of 0 arg */
 	    sprintf(gretl_errmsg, 
 		    _("Log error: Variable '%s', obs %d,"
 		      " value = %g\n"), pdinfo->varname[v],
@@ -297,12 +300,10 @@ static int get_diff (int v, double *diffvec, int ci,
 	    panel_unit_first_obs(t, pdinfo)) {
 	    continue;
 	}
+
 	x0 = Z[v][t];
-	if (pdinfo->structure == STACKED_CROSS_SECTION) {
-	    x1 = (t - pdinfo->pd >= 0)? Z[v][t-pdinfo->pd] : NADBL;
-	} else {
-	    x1 = Z[v][t - t0];
-	}
+	x1 = Z[v][t - t0];
+
 	if (ci == LDIFF) {
 	    if (!na(x0) && !na(x1) && x0 > 0 && x1 > 0) {
 		diffvec[t] = log(x0) - log(x1);
@@ -326,8 +327,8 @@ static int get_xpx (int vi, int vj, double *xvec, const double **Z,
     int t;
 
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	xit = (pdinfo->vector[vi])? Z[vi][t] : Z[vi][0];
-	xjt = (pdinfo->vector[vj])? Z[vj][t] : Z[vj][0];
+	xit = (var_is_series(pdinfo, vi))? Z[vi][t] : Z[vi][0];
+	xjt = (var_is_series(pdinfo, vj))? Z[vj][t] : Z[vj][0];
 	if (na(xit) || na(xjt)) {
 	    xvec[t] = NADBL;
 	} else {
@@ -338,6 +339,26 @@ static int get_xpx (int vi, int vj, double *xvec, const double **Z,
     return 0;
 }
 
+/* write dummy for (v == value) into xvec */
+
+static int get_discdum (int v, double val, double *xvec, const double **Z, 
+			const DATAINFO *pdinfo)
+{
+    double xt;
+    int t;
+
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	xt = Z[v][t];
+	if (na(xt)) {
+	    xvec[t] = NADBL;
+	} else {
+	    xvec[t] = (xt == val)? 1.0 : 0.0;
+	}
+    }
+
+    return 0;
+}
+
 enum transform_results {
     VAR_ADDED_OK,
     VAR_EXISTS_OK,
@@ -409,10 +430,13 @@ check_add_transform (int vnum, const double *x,
 /* get_transform: create specified transformation of variable v if
    this variable does not already exist.
 
+   The dummies resulting from DUMMIFY are automatically marked as
+   discrete.
+
    Return the ID number of the transformed var, or -1 on error.
 */
 
-static int get_transform (int ci, int v, int aux, 
+static int get_transform (int ci, int v, int aux, double x, 
 			  double ***pZ, DATAINFO *pdinfo,
 			  int startlen)
 {
@@ -436,6 +460,9 @@ static int get_transform (int ci, int v, int aux,
     } else if (ci == SQUARE) {
 	/* "aux" = second variable number */
 	err = get_xpx(v, aux, vx, (const double **) *pZ, pdinfo);
+    } else if (ci == DUMMIFY) {
+	/* "x" = value for dummy */
+	err = get_discdum(v, x, vx, (const double **) *pZ, pdinfo);
     }
 
     if (err) {
@@ -445,6 +472,8 @@ static int get_transform (int ci, int v, int aux,
     if (ci == SQUARE && v != aux) {
 	sprintf(label, _("= %s times %s"), pdinfo->varname[v], 
 		pdinfo->varname[aux]);
+    } else if (ci == DUMMIFY) {
+	sprintf(label, _("dummy for %s = %g"), pdinfo->varname[v], x);
     } else {
 	make_transform_label(label, pdinfo->varname[v], ci, aux);
     }
@@ -473,6 +502,10 @@ static int get_transform (int ci, int v, int aux,
 	}
     }
 
+    if (!err && ci == DUMMIFY && vno > 0) {
+	set_var_discrete(pdinfo, vno, 1);
+    }
+
     return vno;
 }
 
@@ -493,12 +526,12 @@ int laggenr (int v, int lag, double ***pZ, DATAINFO *pdinfo)
 {
     int lno;
 
-    if (!pdinfo->vector[v] || lag > pdinfo->n) {
+    if (var_is_scalar(pdinfo, v) || lag > pdinfo->n) {
 	lno = -1;
     } else if (lag == 0) {
 	lno = v;
     } else {
-	lno = get_transform(LAGS, v, lag, pZ, pdinfo, VNAMELEN - 3);
+	lno = get_transform(LAGS, v, lag, 0.0, pZ, pdinfo, VNAMELEN - 3);
     }
 
     return lno;
@@ -518,7 +551,7 @@ int laggenr (int v, int lag, double ***pZ, DATAINFO *pdinfo)
 
 int loggenr (int v, double ***pZ, DATAINFO *pdinfo)
 {
-    return get_transform(LOGS, v, 0, pZ, pdinfo, VNAMELEN - 3);
+    return get_transform(LOGS, v, 0, 0.0, pZ, pdinfo, VNAMELEN - 3);
 }
 
 /**
@@ -538,7 +571,7 @@ int loggenr (int v, double ***pZ, DATAINFO *pdinfo)
 
 int diffgenr (int v, int ci, double ***pZ, DATAINFO *pdinfo)
 {
-    if (!pdinfo->vector[v]) {
+    if (var_is_scalar(pdinfo, v)) {
 	return -1;
     }
 
@@ -550,7 +583,7 @@ int diffgenr (int v, int ci, double ***pZ, DATAINFO *pdinfo)
 	return -1;
     }
 
-    return get_transform(ci, v, 0, pZ, pdinfo, VNAMELEN - 3);
+    return get_transform(ci, v, 0, 0.0, pZ, pdinfo, VNAMELEN - 3);
 }
 
 /**
@@ -575,7 +608,7 @@ int xpxgenr (int vi, int vj, double ***pZ, DATAINFO *pdinfo)
 	}
     }
 
-    return get_transform(SQUARE, vi, vj, pZ, pdinfo, VNAMELEN - 3);
+    return get_transform(SQUARE, vi, vj, 0.0, pZ, pdinfo, VNAMELEN - 3);
 }
 
 static int 
@@ -653,14 +686,14 @@ int list_loggenr (int *list, double ***pZ, DATAINFO *pdinfo)
     for (i=1; i<=list[0]; i++) {
 	v = list[i];
 
-	if (v == 0 || !pdinfo->vector[v]) {
+	if (v == 0 || var_is_scalar(pdinfo, v)) {
 	    continue; 
 	}
 	if (gretl_isdummy(pdinfo->t1, pdinfo->t2, (*pZ)[v])) {
 	    continue;
 	}
 
-	tnum = get_transform(LOGS, v, 0, pZ, pdinfo, startlen);
+	tnum = get_transform(LOGS, v, 0, 0.0, pZ, pdinfo, startlen);
 
 	if (tnum > 0) {
 	    n_ok++;
@@ -677,7 +710,7 @@ static int *make_lags_list (int *list, int order, DATAINFO *pdinfo)
 
     for (i=1; i<=list[0]; i++) {
 	v = list[i];
-	if (v > 0 && pdinfo->vector[v]) {
+	if (v > 0 && var_is_series(pdinfo, v)) {
 	    nl += order;
 	}
     }
@@ -687,15 +720,14 @@ static int *make_lags_list (int *list, int order, DATAINFO *pdinfo)
 
 /**
  * list_laggenr:
- * @plist: pointer to list of variables to process.  On exit
+ * @plist: on entry, pointer to list of variables to process.  On exit
  * the list holds the ID numbers of the lag variables.
  * @order: number of lags to generate (or 0 for automatic).
- * @list: list of variables to process.
- * @pZ: pointer to data matrix.
+ * @pZ: pointer to data array.
  * @pdinfo: data information struct.
  *
  * Generates and adds to the data set @order lagged values of the 
- * variables given in @list.
+ * variables given in the list pointed to by @plist.
  *
  * Returns: 0 on successful completion, 1 on error.
  */
@@ -727,12 +759,12 @@ int list_laggenr (int **plist, int order, double ***pZ, DATAINFO *pdinfo)
     for (i=1; i<=list[0]; i++) {
 	int lv, v = list[i];
 
-	if (v == 0 || !pdinfo->vector[v]) {
+	if (v == 0 || var_is_scalar(pdinfo, v)) {
 	    continue;
 	}
 
 	for (l=1; l<=order; l++) {
-	    lv = get_transform(LAGS, v, l, pZ, pdinfo, startlen);
+	    lv = get_transform(LAGS, v, l, 0.0, pZ, pdinfo, startlen);
 #if TRDEBUG > 1
 	    fprintf(stderr, "base var '%s', lag %d: lv = %d\n",
 		    pdinfo->varname[v], l, lv);
@@ -807,7 +839,7 @@ int list_diffgenr (int *list, int ci, double ***pZ, DATAINFO *pdinfo)
     
     for (i=1; i<=list[0] && !err; i++) {
 	v = list[i];
-	tnum = get_transform(ci, v, 0, pZ, pdinfo, startlen);
+	tnum = get_transform(ci, v, 0, 0.0, pZ, pdinfo, startlen);
 	if (tnum < 0) {
 	    err = 1;
 	} else {
@@ -862,14 +894,14 @@ int list_xpxgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
     for (i=1; i<=l0; i++) {
 	vi = list[i];
 
-	if (vi == 0 || !pdinfo->vector[vi]) {
+	if (vi == 0 || var_is_scalar(pdinfo, vi)) {
 	    continue; 
 	}
 	if (gretl_isdummy(pdinfo->t1, pdinfo->t2, (*pZ)[vi])) {
 	    continue;
 	}
 
-	tnum = get_transform(SQUARE, vi, vi, pZ, pdinfo, startlen);
+	tnum = get_transform(SQUARE, vi, vi, 0.0, pZ, pdinfo, startlen);
 	if (tnum > 0) {
 	    xpxlist[k++] = tnum;
 	    xpxlist[0] += 1;
@@ -895,6 +927,142 @@ int list_xpxgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
     return (xpxlist[0] > 0)? 0 : E_SQUARES;
 }
 
+#define DUMDEBUG 0
+
+/**
+ * list_dumgenr:
+ * @plist: pointer to list of variables to process; on exit
+ * the list holds the ID numbers of the generated dummies.
+ * @pZ: pointer to data matrix.
+ * @pdinfo: data information struct.
+ *
+ * Generates and adds to the data set dummy variables for the
+ * distinct values of the variables given in the list pointed to
+ * by @plist. The latter must have already been marked as discrete.
+ *
+ * Returns: 0 on success, error code on error.
+*/
+
+int list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo)
+{
+    int *list = *plist;
+    int *tmplist = NULL;
+    double *x = NULL;
+    int i, j, t, n;
+    int startlen;
+    int err = 0;
+
+    tmplist = gretl_null_list();
+    if (tmplist == NULL) {
+	return E_ALLOC;
+    }
+    
+    x = malloc(pdinfo->n * sizeof *x);
+    if (x == NULL) {
+	free(tmplist);
+	return E_ALLOC;
+    }
+
+    startlen = get_starting_length(list, pdinfo, 3); /* ?? */
+
+    for (i=1; i<=list[0] && !err; i++) {
+	int vi = list[i];
+	int nuniq, tnum;
+	double xt;
+
+	if (vi == 0 || var_is_scalar(pdinfo, vi) || !var_is_discrete(pdinfo, vi)) {
+	    continue; 
+	}
+
+	n = 0;
+	for (t=0; t<pdinfo->n; t++) {
+	    xt = (*pZ)[vi][t];
+	    if (!na(xt)) {
+		x[n++] = xt;
+	    }
+	}
+
+	if (n == 0) {
+	    continue;
+	}
+
+	qsort(x, n, sizeof *x, gretl_compare_doubles);
+	nuniq = count_distinct_values(x, n);
+
+	if (nuniq == 1) {
+	    continue;
+	}
+
+	rearrange_id_array(x, nuniq, n);
+
+#if DUMDEBUG 
+	fprintf(stderr, "variable %d has %d distinct values:\n", vi, nuniq);
+#endif
+
+	for (j=0; j<nuniq && !err; j++) {
+	    tnum = get_transform(DUMMIFY, vi, j+1, x[j], pZ, pdinfo, startlen);
+#if DUMDEBUG   
+	    fprintf(stderr, "VALUE = %g, tnum = %d\n", x[j], tnum);
+#endif
+	    if (tnum > 0) {
+		tmplist = gretl_list_append_term(&tmplist, tnum);
+		if (tmplist == NULL) {
+		    err = E_ALLOC;
+		} 
+	    } else {
+		err = E_DATA;
+	    }
+	}
+    }
+
+    if (!err && tmplist[0] == 0) {
+	err = E_DATA;
+    }
+
+    free(x);
+
+    if (!err) {
+	free(*plist);
+	*plist = tmplist;
+#if DUMDEBUG   
+	printlist(tmplist, "output list");
+#endif
+    } else {
+	free(tmplist);
+    }
+
+    return err;
+}
+
+/**
+ * list_makediscrete:
+ * @list: list of variables to process.
+ * @pdinfo: data information struct.
+ * @opt: if %OPT_R, reverse the operation.
+ *
+ * Sets the variables given in @list as discrete, unless
+ * opt is %OPT_R, in which case the variables are set as
+ * continuous.
+ *
+ * Returns: 0 on success, error code on error.
+ */
+
+int list_makediscrete (const int *list, DATAINFO *pdinfo, gretlopt opt)
+{
+    int disc = !(opt & OPT_R);
+    int i, v, err = 0;
+
+    for (i=1; i<=list[0]; i++) {
+	v = list[i];
+	if (v == 0 || var_is_scalar(pdinfo, v)) {
+	    continue;
+	}
+	set_var_discrete(pdinfo, v, disc);
+    }
+
+    return err;
+}
+
 int gettrend (double ***pZ, DATAINFO *pdinfo, int square)
 {
     int idx, t, v = pdinfo->v;
@@ -905,13 +1073,13 @@ int gettrend (double ***pZ, DATAINFO *pdinfo, int square)
     if (idx < v) {
 	return idx;
     }
-    
+
     if (dataset_add_series(1, pZ, pdinfo)) {
 	return 0; /* error: valid value cannot == 0 */
     }
 
     for (t=0; t<pdinfo->n; t++) {
-	x = (double) t + 1;
+	x = (double) t + 1; 
 	(*pZ)[v][t] = (square)? x * x : x;
     }
 
