@@ -22,7 +22,7 @@
 
 int default_lag_order (const DATAINFO *pdinfo);
 
-int is_standard_lag (int v, const DATAINFO *pdinfo);
+int is_standard_lag (int v, const DATAINFO *pdinfo, int *parent);
 
 int diffgenr (int v, int ci, double ***pZ, DATAINFO *pdinfo);
 
@@ -41,6 +41,10 @@ int list_loggenr (int *list, double ***pZ, DATAINFO *pdinfo);
 int list_xpxgenr (int **plist, double ***pZ, DATAINFO *pdinfo, 
 		  gretlopt opt);
 
+int list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo);
+
+int list_makediscrete (const int *list, DATAINFO *pdinfo, gretlopt opt);
+
 int gettrend (double ***pZ, DATAINFO *pdinfo, int square);
 
 void gretl_transforms_cleanup (void);
