@@ -24,6 +24,8 @@
 #include "libset.h"
 #include "estim_private.h"
 
+#define TDEBUG 0
+
 static void tsls_omitzero (int *list, const double **Z, int t1, int t2)
 {
     int i, v;
@@ -128,10 +130,12 @@ tsls_save_data (MODEL *pmod, const int *hatlist, const int *exolist,
 
     /* now attach X and endog to the model */
     if (!recycle_X) {
-	gretl_model_set_data(pmod, "tslsX", X, Xsize);
+	gretl_model_set_data(pmod, "tslsX", X, MODEL_DATA_DOUBLE_ARRAY,
+			     Xsize);
     }
     if (!recycle_e) {
-	gretl_model_set_data(pmod, "endog", endog, esize);
+	gretl_model_set_data(pmod, "endog", endog, MODEL_DATA_CHAR_ARRAY,
+			     esize);
     }
 
     return err;
@@ -284,7 +288,7 @@ tsls_list_omit (const int *orig, const int *drop, gretlopt opt, int *err)
 
     if ((opt & OPT_T) && (opt & OPT_B)) {
 	/* incoherent options */
-	*err = E_PARSE;
+	*err = E_BADOPT;
 	return NULL;
     }
 
@@ -331,7 +335,7 @@ tsls_list_add (const int *orig, const int *add, gretlopt opt, int *err)
 
     if ((opt & OPT_T) && (opt & OPT_B)) {
 	/* incoherent options */
-	*err = E_PARSE;
+	*err = E_BADOPT;
 	return NULL;
     }
 
@@ -448,6 +452,7 @@ tsls_sargan_test (MODEL *tsls_model, int Orank, int *instlist,
 
     smod = lsq(OT_list, pZ, pdinfo, OLS, OPT_A);
     if (smod.errcode) {
+	fprintf(stderr, "tsls_sargan_test: smod.errcode = %d\n", smod.errcode);
 	err = smod.errcode;
     } else {
 	ModelTest *test = model_test_new(GRETL_TEST_SARGAN);
@@ -457,7 +462,7 @@ tsls_sargan_test (MODEL *tsls_model, int Orank, int *instlist,
 	    model_test_set_teststat(test, GRETL_STAT_TR2);
 	    model_test_set_dfn(test, Orank);
 	    model_test_set_value(test, OTest);
-	    model_test_set_pvalue(test, chisq(OTest, Orank));
+	    model_test_set_pvalue(test, chisq_cdf_comp(OTest, Orank));
 	    maybe_add_test_to_model(tsls_model, test);
 	}	
     }
@@ -481,6 +486,7 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
 
     hmod = lsq(reglist, pZ, pdinfo, OLS, OPT_A);
     if (hmod.errcode) {
+	fprintf(stderr, "tsls_hausman_test: hmod.errcode (R) = %d\n", hmod.errcode);
 	err = hmod.errcode;
 	goto bailout;
     } 
@@ -497,6 +503,7 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
     hmod = lsq(HT_list, pZ, pdinfo, OLS, OPT_A);
 
     if (hmod.errcode) {
+	fprintf(stderr, "tsls_hausman_test: hmod.errcode (U) = %d\n", hmod.errcode);
 	err = hmod.errcode;
 	goto bailout;
     } else if ((df = hmod.list[0] - reglist[0]) > 0) {
@@ -512,7 +519,7 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
 	    model_test_set_teststat(test, GRETL_STAT_WALD_CHISQ);
 	    model_test_set_dfn(test, df);
 	    model_test_set_value(test, HTest);
-	    model_test_set_pvalue(test, chisq(HTest, df));
+	    model_test_set_pvalue(test, chisq_cdf_comp(HTest, df));
 	    maybe_add_test_to_model(tmod, test);
 	}
     }
@@ -529,18 +536,17 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
    of this matrix; return Q */
 
 static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
-			     const double **Z, int t1, int t2, char **pmask,
+			     const double **Z, int t1, int t2, char *mask,
 			     int *err)
 {
     gretl_matrix *Q = NULL;
     gretl_matrix *R = NULL;
-    char *mask = NULL;
     int rank, ndrop = 0;
     int *droplist = NULL;
     double test;
     int i, k, v;
 
-    Q = gretl_matrix_data_subset(instlist, Z, t1, t2, &mask);
+    Q = gretl_matrix_data_subset(instlist, Z, t1, t2, mask);
     if (Q == NULL) {
 	*err = E_ALLOC;
 	return NULL;
@@ -560,6 +566,12 @@ static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
     }
 
     rank = gretl_matrix_QR_rank(R, NULL, err);
+#if TDEBUG
+    fprintf(stderr, "tsls_Q: k = Q->cols = %d, rank = %d\n", k, rank);
+    if (rank == 0) {
+	gretl_matrix_print(R, "R");
+    }
+#endif
     if (*err) {
 	goto bailout;
     } else if (rank < k) {
@@ -588,8 +600,7 @@ static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
 
 	k = instlist[0];
 	gretl_matrix_free(Q);
-	free(mask);
-	Q = gretl_matrix_data_subset(instlist, Z, t1, t2, &mask);
+	Q = gretl_matrix_data_subset(instlist, Z, t1, t2, mask);
 	R = gretl_matrix_reuse(R, k, k);
 	*err = gretl_matrix_QR_decomp(Q, R);
     }
@@ -599,12 +610,10 @@ static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
     gretl_matrix_free(R);
 
     if (*err) {
-	free(mask);
 	free(droplist);
 	gretl_matrix_free(Q);
 	Q = NULL;
     } else {
-	*pmask = mask;
 	*pdlist = droplist;
     }
 
@@ -616,13 +625,18 @@ static int tsls_form_xhat (gretl_matrix *Q, double *x, double *xhat,
 {
     double *r;
     int k = gretl_matrix_cols(Q);
-    int i, t, s;
+    int i, t, s = 0;
 
     r = malloc(k * sizeof *r);
     if (r == NULL) {
 	return E_ALLOC;
     }
 
+#if TDEBUG
+    fprintf(stderr, "tsls_form_xhat: t1=%d, t2=%d, mask=%p\n",
+	    pdinfo->t1, pdinfo->t2, (void *) mask);
+#endif
+
     /* form r = Q'y */
     for (i=0; i<k; i++) {
 	r[i] = 0.0;
@@ -709,8 +723,8 @@ static void tsls_extra_stats (MODEL *pmod, const double **Z)
 
     if (pmod->missmask == NULL) {
 	/* no missing obs within sample range */
-	pmod->rho = rhohat(0, pmod->t1, pmod->t2, pmod->uhat);
-	pmod->dw = dwstat(0, pmod, Z);
+	pmod->rho = rhohat(1, pmod->t1, pmod->t2, pmod->uhat);
+	pmod->dw = dwstat(1, pmod, Z);
     } else {
 	pmod->rho = pmod->dw = NADBL;
     }
@@ -776,6 +790,145 @@ reglist_remove_redundant_vars (const MODEL *tmod, int *s2list, int *reglist)
     return 0;
 }
 
+static void compute_first_stage_F (MODEL *pmod, int v, int fitv, 
+				   gretl_matrix *Q, 
+				   double **Z, DATAINFO *pdinfo)
+{
+    int n = 0;
+    int k = gretl_matrix_cols(Q);
+    int ifc = pmod->ifc;
+    double essr = 0.0, essu = 0.0;
+    double x, F, ybar = 0.0;
+    int t, dfn, dfd;
+
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	if (!na(Z[v][t])) {
+	    ybar += Z[v][t];
+	    n++;
+	}
+    }
+
+    ybar /= n;
+
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	if (!na(Z[v][t])) {
+	    x = Z[v][t] - ybar;
+	    essr += x * x;
+	    x = Z[v][t] - Z[fitv][t];
+	    essu += x * x;
+	}
+    }  
+
+    dfd = n - k;
+    dfn = k - ifc;
+
+    F = ((essr - essu) / dfn) / (essu / dfd);
+
+    gretl_model_set_double(pmod, "stage1-F", F);
+    gretl_model_set_int(pmod, "stage1-dfn", dfn);
+    gretl_model_set_int(pmod, "stage1-dfd", dfd);
+}
+
+static int 
+tsls_adjust_sample (const int *list, int *t1, int *t2, 
+		    const double **Z, char **pmask)
+{
+    int i, t, t1min = *t1, t2max = *t2;
+    char *mask = NULL;
+    int T, vi, missobs;
+    int err = 0;
+
+    /* advance start of sample range to skip missing obs? */
+    for (t=t1min; t<t2max; t++) {
+	missobs = 0;
+	for (i=1; i<=list[0]; i++) {
+	    vi = list[i];
+	    if (vi == 0 || vi == LISTSEP) {
+		continue;
+	    }
+	    if (na(Z[vi][t])) {
+		missobs = 1;
+		break;
+	    }
+	}
+	if (missobs) {
+	    t1min++;
+	} else {
+	    break;
+	}
+    }
+
+    /* retard end of sample range to skip missing obs? */
+    for (t=t2max; t>t1min; t--) {
+	missobs = 0;
+	for (i=1; i<=list[0]; i++) {
+	    vi = list[i];
+	    if (vi == 0 || vi == LISTSEP) {
+		continue;
+	    }
+	    if (na(Z[vi][t])) {
+		missobs = 1;
+		break;
+	    }
+	}
+	if (missobs) {
+	    t2max--;
+	} else {
+	    break;
+	}	
+    }
+
+    /* count missing values in mid-range of data */
+    missobs = 0;
+    for (t=t1min; t<=t2max; t++) {
+	for (i=1; i<=list[0]; i++) {
+	    vi = list[i];
+	    if (vi == 0 || vi == LISTSEP) {
+		continue;
+	    }
+	    if (na(Z[vi][t])) {
+		missobs++;
+		break;
+	    }
+	}
+    }
+    
+    T = t2max - t1min + 1;
+
+    if (missobs == T) {
+	err = E_MISSDATA;
+    } else if (missobs > 0) {
+	mask = calloc(T, 1);
+	if (mask == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    for (t=t1min; t<=t2max; t++) {
+		for (i=1; i<=list[0]; i++) {
+		    vi = list[i];
+		    if (vi == 0 || vi == LISTSEP) {
+			continue;
+		    }
+		    if (na(Z[vi][t])) {
+			mask[t - t1min] = 1;
+			break;
+		    }
+		}
+	    }
+	}
+    }
+
+#if TDEBUG
+    fprintf(stderr, "tsls_adjust_sample: t1=%d, t2=%d, missobs=%d, ok obs=%d\n", 
+	    t1min, t2max, missobs, t2max - t1min + 1 - missobs);
+#endif
+
+    *t1 = t1min; 
+    *t2 = t2max;
+    *pmask = mask;
+
+    return err;
+}
+
 /**
  * tsls_func:
  * @list: dependent variable plus list of regressors.
@@ -805,7 +958,7 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
     int *exolist = NULL;
     int *droplist = NULL;
     int pos, orig_nvar = pdinfo->v;
-    int rlen, ninst;
+    int rlen, ninst, ev = 0;
     int OverIdRank = 0;
     int i;
 
@@ -836,8 +989,11 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
     }
 
     /* adjust sample range for missing observations */
-    varlist_adjust_sample(list, &pdinfo->t1, &pdinfo->t2, 
-			  (const double **) *pZ); 
+    tsls.errcode = tsls_adjust_sample(list, &pdinfo->t1, &pdinfo->t2, 
+				      (const double **) *pZ, &missmask); 
+    if (tsls.errcode) {
+	return tsls;
+    }
 
     /* 
        reglist: dependent var plus list of regressors
@@ -893,7 +1049,7 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
 
     Q = tsls_Q(instlist, reglist, &droplist,
 	       (const double **) *pZ, pdinfo->t1, pdinfo->t2,
-	       &missmask, &tsls.errcode);
+	       missmask, &tsls.errcode);
     if (tsls.errcode) {
 	goto bailout;
     } 
@@ -902,9 +1058,8 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
     OverIdRank = instlist[0] - reglist[0] + 1;
     if (OverIdRank < 0) {
         sprintf(gretl_errmsg, 
-		_("Order condition for identification is not satisfied.\n"
-		"varlist 2 needs at least %d more variable(s) not in "
-		"varlist1."), -OverIdRank);
+		_("The order condition for identification is not satisfied.\n"
+		  "At least %d more instruments are needed."), -OverIdRank);
 	tsls.errcode = E_UNSPEC; 
 	goto bailout;
     }
@@ -929,6 +1084,11 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
        into the data matrix Z.
     */
 
+    /* prepare for first-stage F-test, if only one endogenous regressor */
+    if (hatlist[0] == 1) {
+	ev = hatlist[1];
+    }    
+
     for (i=1; i<=hatlist[0]; i++) {
 	int orig = hatlist[i];
 	int newv = orig_nvar + i - 1;
@@ -954,9 +1114,14 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
     /* second-stage regression */
     tsls = lsq(s2list, pZ, pdinfo, OLS, (ci == TSLS)? OPT_NONE : OPT_Z);
     if (tsls.errcode) {
+	fprintf(stderr, "tsls_func, stage 2: tsls.errcode = %d\n", tsls.errcode);
 	goto bailout;
     }
 
+    if (ci == TSLS && hatlist[0] == 1) {
+	compute_first_stage_F(&tsls, ev, hatlist[1], Q, *pZ, pdinfo);
+    } 
+
     if (tsls.list[0] < s2list[0]) {
 	/* collinear regressors dropped? If so, adjustments are needed */
 	OverIdRank += s2list[0] - tsls.list[0];
