@@ -52,6 +52,50 @@ static int delete_matrix_by_name (const char *name);
 static double ***gZ;
 static DATAINFO *gdinfo;
 
+#if MDEBUG > 1
+static void print_matrix_stack (const char *msg)
+{
+    int i;
+
+    fprintf(stderr, "\nmatrix stack, %s:\n", msg);
+    for (i=0; i<n_matrices; i++) {
+	if (matrices[i] == NULL) {
+	    fprintf(stderr, " %d: NULL\n", i);
+	} else {
+	    fprintf(stderr, " %d: '%s' at %p (%d x %d)\n",
+		    i, matrices[i]->name, (void *) matrices[i]->M,
+		    matrices[i]->M->rows, matrices[i]->M->cols);
+	}
+    }
+    fputc('\n', stderr);
+}
+#endif
+
+int n_user_matrices (void)
+{
+    return n_matrices;
+}
+
+gretl_matrix *user_matrix_by_index (int i, const char **name)
+{
+    if (i >= 0 && i < n_matrices) {
+	*name = matrices[i]->name;
+	return matrices[i]->M;
+    } else {
+	*name = NULL;
+	return NULL;
+    }
+}
+
+const char *get_matrix_name_by_index (int idx)
+{
+    if (idx >= 0 && idx < n_matrices) {
+	return matrices[idx]->name;
+    } else {
+	return NULL;
+    }
+}
+
 static void 
 usermat_publish_dataset (double ***pZ, DATAINFO *pdinfo)
 {
@@ -61,8 +105,10 @@ usermat_publish_dataset (double ***pZ, DATAINFO *pdinfo)
 
 static void usermat_unpublish_dataset (double ***pZ)
 {
-    *pZ = *gZ;
-    gZ = NULL;
+    if (gZ != NULL) {
+	*pZ = *gZ;
+	gZ = NULL;
+    }
     gdinfo = NULL;
 }
 
@@ -83,7 +129,7 @@ static user_matrix *user_matrix_new (gretl_matrix *M, const char *name)
     return u;
 }
 
-static int add_user_matrix (gretl_matrix *M, const char *name)
+int add_user_matrix (gretl_matrix *M, const char *name)
 {
     user_matrix **tmp;
 
@@ -229,7 +275,7 @@ static int replace_user_matrix (user_matrix *u, gretl_matrix *M,
 	   into the original matrix, u->M */
 	err = matrix_insert_submatrix(u->M, M, mask);
 	if (!is_user_matrix(M)) {
-	    /* is this always right? */
+	    /* FIXME is this always right? */
 	    gretl_matrix_free(M);
 	}
     } else if (M != u->M) {
@@ -294,12 +340,14 @@ real_get_matrix_by_name (const char *name, int slevel, const DATAINFO *pdinfo)
 	level = slevel;
     }
 
-    /* if a series or scalar has been created with the same
-       name as the requested matrix, delete the matrix
-    */
-    if (name_is_variable(name, pdinfo)) {
-	delete_matrix_by_name(name);
-	return NULL;
+    if (pdinfo != NULL) {
+	/* if a series or scalar has been created with the same
+	   name as the requested matrix, delete the matrix
+	*/
+	if (name_is_variable(name, pdinfo)) {
+	    delete_matrix_by_name(name);
+	    return NULL;
+	}
     }
 
     for (i=0; i<n_matrices; i++) {
@@ -661,7 +709,7 @@ static int *parse_slice_spec (const char *s, int n, int *err)
 	}
     }
 
-    if (slice == NULL && !*err) {
+    if (slice == NULL && gZ != NULL && !*err) {
 	/* not found yet: keep trying, for p:q or plain p */
 	char f1[32], f2[32]; /* arbitrary? */
 	double x[2];
@@ -853,6 +901,10 @@ matrix_get_submatrix (const gretl_matrix *M, const char *s,
     int n = gretl_matrix_cols(M);
     int nr, nc;
 
+#if MDEBUG
+    fprintf(stderr, "matrix_get_submatrix: s = '%s'\n", s);
+#endif
+
     /* the selection string should end with ']' */
     p = strrchr(s, ']');
     if (p == NULL || *(p+1) != '\0') {
@@ -923,7 +975,7 @@ matrix_get_submatrix (const gretl_matrix *M, const char *s,
 /**
  * user_matrix_get_slice:
  * @s: string specifying a sub-matrix.
- * @Z: data array.
+ * @pZ: pointer to data array.
  * @pdinfo: dataset information.
  * @err: location to receive error code.
  *
@@ -959,10 +1011,10 @@ gretl_matrix *user_matrix_get_slice (const char *s,
 }
 
 /**
- * add_or_replace_user_matrix:
+ * add_or_replace_user_matrix_full:
  * @M: gretl matrix.
  * @name: name for the matrix.
- * @mask: submatrix specification (or empty string).
+ * @mask: submatrix specification (or %NULL).
  * @R: location to receive address of matrix that was
  * replaced, if any (or %NULL).
  * @pZ: pointer to data array.
@@ -976,9 +1028,9 @@ gretl_matrix *user_matrix_get_slice (const char *s,
  * Returns: 0 on success, %E_ALLOC on failure.
  */
 
-int add_or_replace_user_matrix (gretl_matrix *M, const char *name,
-				const char *mask, gretl_matrix **R,
-				double ***pZ, DATAINFO *pdinfo)
+int add_or_replace_user_matrix_full (gretl_matrix *M, const char *name,
+				     const char *mask, gretl_matrix **R,
+				     double ***pZ, DATAINFO *pdinfo)
 {
     user_matrix *u;
     int err = 0;
@@ -989,6 +1041,11 @@ int add_or_replace_user_matrix (gretl_matrix *M, const char *name,
 	if (pZ != NULL && pdinfo != NULL) {
 	    usermat_publish_dataset(pZ, pdinfo);
 	}
+#if MDEBUG
+	fprintf(stderr, "add_or_replace_user_matrix: source M=%p, name='%s', "
+		"mask='%s', R=%p\n",
+		(void *) M, name, mask, (void *) R);
+#endif
 	err = replace_user_matrix(u, M, R, mask);
 	usermat_unpublish_dataset(pZ);
     } else {
@@ -998,7 +1055,7 @@ int add_or_replace_user_matrix (gretl_matrix *M, const char *name,
 	/* if we created a matrix with the same name as an existing
 	   scalar, delete the scalar */
 	v = varindex(pdinfo, name);
-	if (v < pdinfo->v && !pdinfo->vector[v]) {
+	if (v < pdinfo->v && var_is_scalar(pdinfo, v)) {
 	    dataset_drop_variable(v, pZ, pdinfo);
 	}
     }
@@ -1006,6 +1063,33 @@ int add_or_replace_user_matrix (gretl_matrix *M, const char *name,
     return err;
 }
 
+/**
+ * add_or_replace_user_matrix:
+ * @M: gretl matrix.
+ * @name: name for the matrix.
+ *
+ * Checks whether a matrix of the given @name already exists.
+ * If so, the original matrix is replaced by @M; if not, the
+ * the matrix @M is added to the stack of user-defined
+ * matrices.
+ *
+ * Returns: 0 on success, %E_ALLOC on failure.
+ */
+
+int add_or_replace_user_matrix (gretl_matrix *M, const char *name)
+{
+    user_matrix *u = get_user_matrix_by_name(name);
+    int err = 0;
+
+    if (u != NULL) {
+	err = replace_user_matrix(u, M, NULL, NULL);
+    } else {
+	err = add_user_matrix(M, name);
+    }
+
+    return err;
+}
+
 static void destroy_user_matrix (user_matrix *u)
 {
     if (u == NULL) {
@@ -1043,7 +1127,10 @@ int destroy_user_matrices_at_level (int level)
 
 #if MDEBUG
     fprintf(stderr, "destroy_user_matrices_at_level: level = %d, "
-	    "n_matrices = %d\n", level, n_matrices);
+	    "total n_matrices = %d\n", level, n_matrices);
+#endif
+#if MDEBUG > 1
+    print_matrix_stack("at top of destroy_user_matrices_at_level");
 #endif
 
     for (i=0; i<n_matrices; i++) {
@@ -1052,14 +1139,15 @@ int destroy_user_matrices_at_level (int level)
 	}
 	if (matrices[i]->level == level) {
 #if MDEBUG
-	    fprintf(stderr, "destroying matrix[%d] (M at %p)\n",
-		    i, (void *) matrices[i]->M);
+	    fprintf(stderr, "destroying matrix[%d] ('%s' at %p)\n",
+		    i, matrices[i]->name, (void *) matrices[i]->M);
 #endif
 	    destroy_user_matrix(matrices[i]);
 	    for (j=i; j<n_matrices - 1; j++) {
 		matrices[j] = matrices[j+1];
 	    }
 	    matrices[n_matrices - 1] = NULL;
+	    i--;
 	} else {
 	    nm++;
 	}
@@ -1080,6 +1168,10 @@ int destroy_user_matrices_at_level (int level)
 	}
     }
 
+#if MDEBUG > 1
+    print_matrix_stack("at end of destroy_user_matrices_at_level");
+#endif
+
     return err;
 }
 
@@ -1171,7 +1263,7 @@ static int first_field_is_series (const char *s, const DATAINFO *pdinfo)
     *word = '\0';
     sscanf(s, "%15[^,;} ]", word);
     v = varindex(pdinfo, word);
-    if (v < pdinfo->v && pdinfo->vector[v]) {
+    if (v < pdinfo->v && var_is_series(pdinfo, v)) {
 	ret = 1;
     }
 
@@ -1287,7 +1379,7 @@ static double get_var_double (const char **s, const double **Z,
 	v = varindex(pdinfo, vname);
 	if (v == pdinfo->v) {
 	    *err = E_UNKVAR;
-	} else if (pdinfo->vector[v]) {
+	} else if (var_is_series(pdinfo, v)) {
 	    *err = E_DATA;
 	} else {
 	    x = Z[v][0];
@@ -1401,7 +1493,7 @@ static int fill_matrix_from_series (gretl_matrix *M, const char *s,
 		v = get_varnum(&s, pdinfo, &err);
 		if (!err) {
 		    for (t=0; t<T; t++) {
-			if (pdinfo->vector[v]) {
+			if (var_is_series(pdinfo, v)) {
 			    x = Z[v][t + pdinfo->t1];
 			} else {
 			    x = Z[v][0];
@@ -1424,7 +1516,7 @@ static int fill_matrix_from_series (gretl_matrix *M, const char *s,
 		v = get_varnum(&s, pdinfo, &err);
 		if (!err) {
 		    for (t=0; t<T; t++) {
-			if (pdinfo->vector[v]) {
+			if (var_is_series(pdinfo, v)) {
 			    x = Z[v][t + pdinfo->t1]; 
 			} else {
 			    x = Z[v][0];
@@ -1462,14 +1554,56 @@ static int matrix_genr (const char *name, const char *mask,
     return err;
 }
 
+static int list_has_scalars (const int *list, const DATAINFO *pdinfo)
+{
+    int i;
+
+    for (i=1; i<=list[0]; i++) {
+	if (var_is_scalar(pdinfo, list[i])) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
+static int *full_series_list (const DATAINFO *pdinfo)
+{
+    int *list = NULL;
+    int i, j, n = 0;
+
+    for (i=1; i<pdinfo->v; i++) {
+	if (var_is_series(pdinfo, i)) {
+	    n++;
+	}
+    }
+
+    if (n == 0) {
+	return NULL;
+    }
+
+    list = gretl_list_new(n);
+    if (list == NULL) {
+	return NULL;
+    }
+
+    for (i=1, j=1; i<pdinfo->v; i++) {
+	if (var_is_series(pdinfo, i)) {
+	    list[j++] = i;
+	}
+    }
+
+    return list;
+}
+
 gretl_matrix *fill_matrix_from_list (const char *s, const double **Z,
 				     const DATAINFO *pdinfo, int transp,
 				     int *err)
 {
     gretl_matrix *M = NULL;
     char word[VNAMELEN];
-    char *mask = NULL;
-    const int *list;
+    int *list = NULL;
+    int freelist = 0;
     int len;
 
     while (isspace(*s)) s++;
@@ -1481,23 +1615,23 @@ gretl_matrix *fill_matrix_from_list (const char *s, const double **Z,
 
     *word = '\0';
     strncat(word, s, len);
-    list = get_list_by_name(word);
-    if (list == NULL) {
-	return NULL;
+
+    if (!strcmp(word, "dataset")) {
+	list = full_series_list(pdinfo);
+	freelist = 1;
+    } else {
+	list = get_list_by_name(word);
+	if (list == NULL) {
+	    return NULL;
+	} 
+	if (list_has_scalars(list, pdinfo)) {
+	    *err = E_TYPES;
+	    return NULL;
+	}
     }
 
-    M = gretl_matrix_data_subset(list, Z, pdinfo->t1, pdinfo->t2, &mask);
-
-    if (M == NULL) {
-	*err = 1;
-    }
-
-    if (mask != NULL) {
-	*err = E_MISSDATA;
-	free(mask);
-	gretl_matrix_free(M);
-	M = NULL;
-    }
+    M = gretl_matrix_data_subset_no_missing(list, Z, pdinfo->t1, pdinfo->t2, 
+					    err);
 
     if (M != NULL && transp) {
 	gretl_matrix *R = gretl_matrix_copy_transpose(M);
@@ -1512,6 +1646,10 @@ gretl_matrix *fill_matrix_from_list (const char *s, const double **Z,
 	}
     }
 
+    if (freelist) {
+	free(list);
+    }
+
     return M;
 }
 
@@ -1520,7 +1658,7 @@ static int name_is_series (const char *name, const DATAINFO *pdinfo)
     int v = varindex(pdinfo, name);
     int ret = 0;
 
-    if (v < pdinfo->v && pdinfo->vector[v]) {
+    if (v < pdinfo->v && var_is_series(pdinfo, v)) {
 	sprintf(gretl_errmsg, _("'%s' is the name of a data series"), name);
 	ret = 1;
     }
@@ -1646,8 +1784,8 @@ static int create_matrix (const char *name, const char *mask,
 	if (err) {
 	    goto finalize;
 	} else if (M != NULL) {
-	    err = add_or_replace_user_matrix(M, name, mask, NULL,
-					     pZ, pdinfo);
+	    err = add_or_replace_user_matrix_full(M, name, mask, NULL,
+						  pZ, pdinfo);
 	    goto finalize;
 	}
     }
@@ -1695,8 +1833,8 @@ static int create_matrix (const char *name, const char *mask,
     }
     
     if (!err) {
-	err = add_or_replace_user_matrix(M, name, mask, NULL,
-					 pZ, pdinfo);
+	err = add_or_replace_user_matrix_full(M, name, mask, NULL,
+					      pZ, pdinfo);
     }
 
  finalize:
@@ -1747,6 +1885,9 @@ print_matrix_by_name (const char *name, const char *mask,
 	    }
 	    gretl_matrix_free(S);
 	} else {
+#if MDEBUG
+	    fprintf(stderr, "printing matrix '%s' (%p)\n", name, (void *) M);
+#endif
 	    gretl_matrix_print_to_prn(M, name, prn);
 	}
 	if (transp) {
@@ -1775,10 +1916,13 @@ print_matrix_address_by_name (const char *name, const DATAINFO *pdinfo, PRN *prn
 
 /**
  * matrix_command:
- * @s: string that specifies matrix command.
+ * @line: string that specifies matrix command.
+ * @pZ: pointer to data array.
+ * @pdinfo: dataset information struct.
  * @prn: pointer to printing struct.
  *
- * To be written.
+ * Creates, deletes or prints a user-defined matrix, depending
+ * on the content of @line.
  * 
  * Returns: 0 on success, non-zero code on failure.
  */
@@ -1791,6 +1935,10 @@ int matrix_command (const char *line, double ***pZ, DATAINFO *pdinfo, PRN *prn)
     char *p;
     int err = 0;
 
+#if MDEBUG
+    fprintf(stderr, "*** matrix_command: '%s'\n", line);
+#endif
+
     if (!strncmp(line, "matrix ", 7)) line += 7;
     while (isspace(*line)) line++;
 
@@ -1854,21 +2002,30 @@ static void print_calc_input_info (gretl_matrix *A, gretl_matrix *B, char op)
 {
     fprintf(stderr, "matrix_calc_AB: A = %p", (void *) A);
     if (A != NULL) {
-	fprintf(stderr, " (%dx%d)", gretl_matrix_rows(A), gretl_matrix_cols(A));
+	fprintf(stderr, " (%dx%d)", A->rows, A->cols);
     } 
     fprintf(stderr, ", B = %p", (void *) B);
     if (B != NULL) {
-	fprintf(stderr, " (%dx%d), ", gretl_matrix_rows(B), gretl_matrix_cols(B));
+	fprintf(stderr, " (%dx%d), ", B->rows, B->cols);
     }
     if (isprint(op)) {
 	fprintf(stderr, "op='%c'\n", op);
     } else {
 	fprintf(stderr, "op=%d\n", (int) op);
     }
+
 # if MDEBUG > 1
     if (A != NULL) gretl_matrix_print(A, "input A");
     if (B != NULL) gretl_matrix_print(B, "input B");
 # endif
+
+    if (op == '\0') {
+	if (is_user_matrix(B)) {
+	    fprintf(stderr, "B is a user matrix: copy to target\n");
+	} else {
+	    fprintf(stderr, "B is a not a user matrix: assign to target\n");
+	}
+    }
 }
 #endif
 
@@ -1879,9 +2036,9 @@ gretl_matrix *matrix_calc_AB (gretl_matrix *A, gretl_matrix *B,
 {
     gretl_matrix *C = NULL;
     gretl_matrix *D = NULL;
-    double x;
     int ra, ca;
     int rb, cb;
+    int r, c;
 
     *err = 0;
 
@@ -1891,20 +2048,40 @@ gretl_matrix *matrix_calc_AB (gretl_matrix *A, gretl_matrix *B,
 
     switch (op) {
     case '\0':
-	C = B;
+	if (is_user_matrix(B)) {
+	    C = gretl_matrix_copy(B);
+	} else {
+	    C = B;
+	}
 	break;
     case '=':
 	C = matrix_test_equality(A, B, err);
 	break;
     case '+':
     case '-':
-	C = gretl_matrix_copy(A);
+	ra = gretl_matrix_rows(A);
+	ca = gretl_matrix_cols(A);
+
+	if (ra == 1 && ca == 1) {
+	    C = gretl_matrix_copy(B);
+	} else {
+	    C = gretl_matrix_copy(A);
+	}
 	if (C == NULL) {
 	    *err = E_ALLOC;
-	} else if (op == '+') {
-	    *err = gretl_matrix_add_to(C, B);
+	} else if (ra == 1 && ca == 1) {
+	    if (op == '+') {
+		*err = gretl_matrix_add_to(C, A);
+	    } else {
+		gretl_matrix_multiply_by_scalar(C, -1.0);
+		*err = gretl_matrix_add_to(C, A);
+	    }
 	} else {
-	    *err = gretl_matrix_subtract_from(C, B);
+	    if (op == '+') {
+		*err = gretl_matrix_add_to(C, B);
+	    } else {
+		*err = gretl_matrix_subtract_from(C, B);
+	    }
 	}
 	break;
     case '~':
@@ -1918,41 +2095,25 @@ gretl_matrix *matrix_calc_AB (gretl_matrix *A, gretl_matrix *B,
 	cb = gretl_matrix_cols(B);
 
 	if (ra == 1 && ca == 1) {
-	    C = gretl_matrix_copy(B);
-#if MDEBUG
-	    fprintf(stderr, " allocated copy 'C' of B (%dx%d) at %p\n", 
-		    rb, cb, (void *) C);
-#endif
-	    if (C == NULL) {
-		*err = E_ALLOC;	
-	    } else {
-		x = gretl_matrix_get(A, 0, 0);
-		gretl_matrix_multiply_by_scalar(C, x);
-	    }
+	    r = rb;
+	    c = cb;
 	} else if (rb == 1 && cb == 1) {
-	    C = gretl_matrix_copy(A);
-#if MDEBUG
-	    fprintf(stderr, " allocated copy 'C' of A (%dx%d) at %p\n", 
-		    ra, ca, (void *) C);
-#endif
-	    if (C == NULL) {
-		*err = E_ALLOC;	
-	    } else {
-		x = gretl_matrix_get(B, 0, 0);
-		gretl_matrix_multiply_by_scalar(C, x);
-	    }
+	    r = ra;
+	    c = ca;
+	} else {
+	    r = ra;
+	    c = cb;
+	}
+	C = gretl_matrix_alloc(r, c);
+	if (C == NULL) {
+	    *err = E_ALLOC;
 	} else {
-	    C = gretl_matrix_alloc(ra, cb);
 #if MDEBUG
 	    fprintf(stderr, " allocated new 'C' (%dx%d) at %p\n", 
-		    ra, cb, (void *) C);
+		    r, c, (void *) C);
 #endif
-	    if (C == NULL) {
-		*err = E_ALLOC;
-	    } else {
-		*err = gretl_matrix_multiply(A, B, C);
-	    }
-	}
+	    *err = gretl_matrix_multiply(A, B, C);
+	}	
 	break;
     case '/':
 	/* matrix "division" */
@@ -1964,8 +2125,7 @@ gretl_matrix *matrix_calc_AB (gretl_matrix *A, gretl_matrix *B,
 	    *err = E_ALLOC;
 	} else {
 	    if (rb == 1 && cb == 1) {
-		x = gretl_vector_get(B, 0);
-		*err = gretl_matrix_divide_by_scalar(C, x);
+		*err = gretl_matrix_divide_by_scalar(C, B->val[0]);
 	    } else {
 		D = gretl_matrix_copy(B);
 		if (D == NULL) {
@@ -1996,14 +2156,13 @@ gretl_matrix *matrix_calc_AB (gretl_matrix *A, gretl_matrix *B,
 	    if (C == NULL) {
 		*err = E_ALLOC;
 	    } else {
-		x = gretl_matrix_get(B, 0, 0);
-		gretl_matrix_dot_pow(C, x);
+		gretl_matrix_dot_pow(C, B->val[0]);
 	    }
 	}
 	break;
     case OP_KRON:
 	/* Kronecker product */
-	C = gretl_matrix_kronecker_product(A, B);
+	C = gretl_matrix_kronecker_product_new(A, B);
 	if (C == NULL) {
 	    *err = E_ALLOC;
 	}
@@ -2108,6 +2267,57 @@ gretl_matrix *user_matrix_column_demean (const gretl_matrix *m)
     return R;
 }
 
+gretl_matrix *user_matrix_vec (const gretl_matrix *m)
+{
+    gretl_matrix *R = NULL;
+
+    if (m != NULL) {
+	R = gretl_matrix_alloc(m->rows * m->cols, 1);
+	if (R != NULL) {
+	    gretl_matrix_vectorize(R, m);
+	} 
+    }
+
+    return R;
+}
+
+gretl_matrix *user_matrix_vech (const gretl_matrix *m, int *err)
+{
+    gretl_matrix *R = NULL;
+
+    if (m != NULL) {
+	if (m->rows != m->cols) {
+	    *err = E_NONCONF;
+	} else {
+	    int n = m->rows;
+	    int k = n * (n + 1) / 2;
+
+	    R = gretl_matrix_alloc(k, 1);
+	    if (R != NULL) {
+		*err = gretl_matrix_vectorize_h(R, m);
+	    }
+	} 
+    }
+
+    return R;
+}
+
+gretl_matrix *user_matrix_unvech (const gretl_matrix *m, int *err)
+{
+    gretl_matrix *R = NULL;
+    int n;
+
+    if (m != NULL && m->cols == 1) {
+	n = (int) ((sqrt(1.0 + 8.0 * m->rows) - 1.0) / 2.0);
+	R = gretl_matrix_alloc(n, n);
+	if (R != NULL) {
+	    *err = gretl_matrix_unvectorize_h(R, m);
+	} 
+    }
+
+    return R;
+}
+
 static int 
 real_user_matrix_QR_decomp (const gretl_matrix *m, gretl_matrix **Q, 
 			    gretl_matrix **R)
@@ -2177,7 +2387,7 @@ static int add_or_replace_aux_matrix (gretl_matrix *A,
 {
     int err;
 
-    err = add_or_replace_user_matrix(A, aname, NULL, NULL, pZ, pdinfo);
+    err = add_or_replace_user_matrix_full(A, aname, NULL, NULL, pZ, pdinfo);
 
     if (!err && gretl_messages_on()) {
 	if (n_matrices > old_nm) {
