@@ -20,6 +20,12 @@
 #ifndef USERMAT_H_
 #define USERMAT_H_
 
+int n_user_matrices (void);
+
+gretl_matrix *user_matrix_by_index (int i, const char **name);
+
+const char *get_matrix_name_by_index (int idx);
+
 gretl_matrix *get_matrix_by_name (const char *name, const DATAINFO *pdinfo);
 
 gretl_matrix *get_matrix_by_name_at_level (const char *name, int level,
@@ -32,9 +38,13 @@ int named_matrix_get_variable (const char *mspec,
 			       double ***Z, DATAINFO *pdinfo,
 			       double **px, int *plen);
 
-int add_or_replace_user_matrix (gretl_matrix *M, const char *name,
-				const char *mask, gretl_matrix **R,
-				double ***pZ, DATAINFO *pdinfo);
+int add_user_matrix (gretl_matrix *M, const char *name);
+
+int add_or_replace_user_matrix (gretl_matrix *M, const char *name);
+
+int add_or_replace_user_matrix_full (gretl_matrix *M, const char *name,
+				     const char *mask, gretl_matrix **R,
+				     double ***pZ, DATAINFO *pdinfo);
 
 int copy_named_matrix_as (const char *orig, const char *new);
 
@@ -66,6 +76,12 @@ gretl_matrix *user_matrix_cholesky_decomp (const gretl_matrix *m);
 
 gretl_matrix *user_matrix_column_demean (const gretl_matrix *m);
 
+gretl_matrix *user_matrix_vec (const gretl_matrix *m);
+
+gretl_matrix *user_matrix_vech (const gretl_matrix *m, int *err);
+
+gretl_matrix *user_matrix_unvech (const gretl_matrix *m, int *err);
+
 gretl_matrix *
 user_matrix_QR_decomp (const char *str, double ***pZ, DATAINFO *pdinfo,
 		       PRN *prn, int *err);
