@@ -24,11 +24,13 @@
 
 /**
  * gretl_VAR_print_VCV:
- * @var:
- * @prn:
+ * @var: pointer to gretl VAR structure.
+ * @prn: printing object.
  *
+ * Prints to @prn the contemporaneous (cross-equation) variance
+ * matrix for @var.
  *
- * Returns:
+ * Returns: 0 on success, 1 on failure.
  */
 
 int gretl_VAR_print_VCV (const GRETL_VAR *var, PRN *prn)
@@ -240,13 +242,19 @@ static int varprint_namelen (const GRETL_VAR *var, const DATAINFO *pdinfo,
 /**
  * gretl_VAR_print_impulse_response:
  * @var: pointer to VAR struct.
- * @shock:
+ * @shock: index number of the "shock" variable.
  * @periods: number of periods over which to print response.
  * @pdinfo: dataset information.
  * @pause: if non-zero, pause between sections of output.
- * @prn: gretl printing struct.
- *
+ * @prn: gretl printing object.
  *
+ * Prints to @prn the estimated responses of the endogenous
+ * variables in @var to a one-standard deviation shock in
+ * the specified variable: @shock is a zero-based index into
+ * the equations of the VAR so for example if @shock = 1,
+ * the responses are to a shock in the second endogenous
+ * variable in the VAR specification.
+ * 
  * Returns: 0 on success, non-zero code on error.
  */
 
@@ -887,19 +895,19 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 		pputs(prn, "& ");
 		pprintf(prn, "$F(%d, %d) = %g$ & ", var->order, dfd, var->Fvals[k]);
 		pprintf(prn, "%s %.4f\\\\\n", I_("p-value"), 
-			fdist(var->Fvals[k], var->order, dfd));
+			f_cdf_comp(var->Fvals[k], var->order, dfd));
 	    } else if (rtf) {
 		pprintf(prn, I_("All lags of %-15s "), pdinfo->varname[v]);
 		pprintf(prn, "F(%d, %d) = %8.5g, ", var->order, dfd, var->Fvals[k]);
 		pprintf(prn, "%s %.4f\\par\n", I_("p-value"), 
-			fdist(var->Fvals[k], var->order, dfd));
+			f_cdf_comp(var->Fvals[k], var->order, dfd));
 	    } else {
 		pputs(prn, "  ");
 		pprintf(prn, _("All lags of %-15s "), pdinfo->varname[v]);
 		sprintf(Fstr, "F(%d, %d)", var->order, dfd);
 		pprintf(prn, "%12s = %#8.5g, ", Fstr, var->Fvals[k]);
 		pprintf(prn, "%s %.4f\n", _("p-value"), 
-			fdist(var->Fvals[k], var->order, dfd));
+			f_cdf_comp(var->Fvals[k], var->order, dfd));
 	    }
 	    k++;
 	}
@@ -910,19 +918,19 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 		pputs(prn, "& ");
 		pprintf(prn, "$F(%d, %d) = %g$ & ", var->neqns, dfd, var->Fvals[k]);
 		pprintf(prn, "%s %.4f\\\\\n", I_("p-value"), 
-			fdist(var->Fvals[k], var->neqns, dfd));
+			f_cdf_comp(var->Fvals[k], var->neqns, dfd));
 	    } else if (rtf) {
 		pprintf(prn, I_("All vars, lag %-13d "), var->order);
 		pprintf(prn, "F(%d, %d) = %8.5g, ", var->neqns, dfd, var->Fvals[k]);
 		pprintf(prn, "%s %.4f\\par\n", I_("p-value"), 
-			fdist(var->Fvals[k], var->neqns, dfd));
+			f_cdf_comp(var->Fvals[k], var->neqns, dfd));
 	    } else {
 		pputs(prn, "  ");
 		pprintf(prn, _("All vars, lag %-13d "), var->order);
 		sprintf(Fstr, "F(%d, %d)", var->neqns, dfd);
 		pprintf(prn, "%12s = %#8.5g, ", Fstr, var->Fvals[k]);
 		pprintf(prn, "%s %.4f\n", _("p-value"), 
-			fdist(var->Fvals[k], var->neqns, dfd));
+			f_cdf_comp(var->Fvals[k], var->neqns, dfd));
 	    } 
 	    k++;
 	}
@@ -960,14 +968,14 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 	    pprintf(prn, "%s: %s\\par\n", I_("Alternative hypothesis"), h1str);
 	    pprintf(prn, "%s: $\\chi^2_{%d}$ = %.3f (%s %f)\\par\n",
 		    I_("Likelihood ratio test"), 
-		    df, var->LR, I_("p-value"), chisq(var->LR, df));
+		    df, var->LR, I_("p-value"), chisq_cdf_comp(var->LR, df));
 	} else if (rtf) {
 	    pprintf(prn, "\\par %s\n", I_("For the system as a whole"));
 	    pprintf(prn, "\\par %s: %s\n", I_("Null hypothesis"), h0str);
 	    pprintf(prn, "\\par %s: %s\n", I_("Alternative hypothesis"), h1str);
 	    pprintf(prn, "\\par %s: %s(%d) = %g (%s %f)\n",
 		    I_("Likelihood ratio test"), I_("Chi-square"), 
-		    df, var->LR, I_("p-value"), chisq(var->LR, df));
+		    df, var->LR, I_("p-value"), chisq_cdf_comp(var->LR, df));
 	} else {
 	    int ordlen = (var->order > 10)? 2 : 1;
 
@@ -976,7 +984,7 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 	    pprintf(prn, "  %s: %s\n", _("Alternative hypothesis"), h1str);
 	    pprintf(prn, "  %s: %s(%d) = %g (%s %f)\n",
 		    _("Likelihood ratio test"), _("Chi-square"), 
-		    df, var->LR, _("p-value"), chisq(var->LR, df));
+		    df, var->LR, _("p-value"), chisq_cdf_comp(var->LR, df));
 	    /* Info criteria comparison */
 	    pprintf(prn, "\n  %s:\n", _("Comparison of information criteria"));
 	    pputs(prn, "  ");
