@@ -1,41 +1,36 @@
 #define MAX_ARMA_ORDER 6
 #define MAX_ARIMA_DIFF 2
 
-enum {
-    ARMA_IFC   = 1 << 0, /* specification includes a constant */
-    ARMA_SEAS  = 1 << 1, /* includes seasonal component */
-    ARMA_DSPEC = 1 << 2, /* input list includes differences */
-    ARMA_X12A  = 1 << 3  /* using X-12-ARIMA to generate estimates */
-} ArmaFlags;
-
 struct arma_info {
-    int yno;      /* ID of dependent variable */
-    char flags;   /* from ArmaFlags */
-    int p;        /* non-seasonal AR order */
-    int d;        /* non-seasonal difference */
-    int q;        /* non-seasonal MA order */
-    int P;        /* seasonal AR order */
-    int D;        /* seasonal difference */
-    int Q;        /* seasonal MA order */
-    int maxlag;   /* longest lag in model */
-    int r;        /* number of other regressors (ARMAX) */
-    int nc;       /* total number of coefficients */
-    int t1;       /* starting observation */
-    int t2;       /* ending observation */
-    int pd;       /* periodicity of data */
-    int T;        /* full length of data series */
-    double *dy;   /* differenced dependent variable */
+    int yno;         /* ID of dependent variable */
+    ArmaFlags flags; /* specification flags */
+    int ifc;         /* specification includes a constant? */
+    int p;           /* non-seasonal AR order */
+    int d;           /* non-seasonal difference */
+    int q;           /* non-seasonal MA order */
+    int P;           /* seasonal AR order */
+    int D;           /* seasonal difference */
+    int Q;           /* seasonal MA order */
+    int maxlag;      /* longest lag in model */
+    int nexo;        /* number of other regressors (ARMAX) */
+    int nc;          /* total number of coefficients */
+    int t1;          /* starting observation */
+    int t2;          /* ending observation */
+    int pd;          /* periodicity of data */
+    int T;           /* full length of data series */
+    double *dy;      /* differenced dependent variable */
 };
 
-#define arma_has_const(a)      ((a)->flags & ARMA_IFC)
 #define arma_has_seasonal(a)   ((a)->flags & ARMA_SEAS)
 #define arma_is_arima(a)       ((a)->flags & ARMA_DSPEC)
 #define arma_by_x12a(a)        ((a)->flags & ARMA_X12A)
+#define arma_exact_ml(a)       ((a)->flags & ARMA_EXACT)
+#define arma_using_vech(a)     ((a)->flags & ARMA_VECH)
 
-#define set_arma_has_const(a)     ((a)->flags |= ARMA_IFC)
 #define set_arma_has_seasonal(a)  ((a)->flags |= ARMA_SEAS)
 #define set_arma_is_arima(a)      ((a)->flags |= ARMA_DSPEC)
 #define unset_arma_is_arima(a)    ((a)->flags &= ~ARMA_DSPEC)
+#define set_arma_use_vech(a)      ((a)->flags |= ARMA_VECH)
 
 static void 
 arma_info_init (struct arma_info *ainfo, char flags, const DATAINFO *pdinfo)
@@ -51,7 +46,8 @@ arma_info_init (struct arma_info *ainfo, char flags, const DATAINFO *pdinfo)
     ainfo->Q = 0; 
 
     ainfo->maxlag = 0;
-    ainfo->r = 0;
+    ainfo->ifc = 0;
+    ainfo->nexo = 0;
     ainfo->nc = 0;
 
     ainfo->t1 = pdinfo->t1;
@@ -75,46 +71,6 @@ static int arma_list_y_position (struct arma_info *ainfo)
     return ypos;
 }
 
-#if 0
-
-static void arma_R2_and_F (MODEL *pmod, const double *y)
-{
-    int t;
-
-    pmod->tss = 0.0;
-
-    for (t=pmod->t1; t<=pmod->t2; t++) {
-	pmod->tss += (y[t] - pmod->ybar) * (y[t] - pmod->ybar);
-    }
-
-    if (!pmod->ifc) {
-	double syh2 = 0.0;
-
-	for (t=pmod->t1; t<=pmod->t2; t++) {
-	    syh2 += pmod->yhat[t] * pmod->yhat[t];
-	}
-	pmod->fstt = pmod->dfd * syh2 / (pmod->dfn * pmod->ess);
-    } else if (pmod->tss > pmod->ess) {
-	pmod->fstt = pmod->dfd * (pmod->tss - pmod->ess) / (pmod->dfn * pmod->ess);
-    } 
-
-    if (!pmod->ifc) {
-	double r2 = gretl_corr_rsq(pmod->t1, pmod->t2, y, pmod->yhat);
-
-	pmod->rsq = r2;
-	pmod->adjrsq = 1.0 - ((1.0 - r2) * (pmod->nobs - 1.0) / pmod->dfd);
-    } else if (pmod->tss > 0) {
-	pmod->rsq = 1.0 - (pmod->ess / pmod->tss);
-	if (pmod->dfd > 0) {
-	    double den = pmod->tss * pmod->dfd;
-
-	    pmod->adjrsq = 1.0 - (pmod->ess * (pmod->nobs - 1) / den);
-	}
-    }
-}
-
-#endif
-
 #define INT_DEBUG 0
 
 static int arima_integrate (double *dx, const double *x,
@@ -194,39 +150,45 @@ static int arima_integrate (double *dx, const double *x,
     return 0;
 }
 
-/* write the various statistics from ARMA estimation into
-   a gretl MODEL struct */
-
-static void write_arma_model_stats (MODEL *pmod, model_info *arma,
-				    const int *list, const double **Z, 
-				    const double *theta, 
-				    struct arma_info *ainfo)
+static void ainfo_data_to_model (struct arma_info *ainfo, MODEL *pmod)
 {
-    double **series = NULL;
-    const double *e = NULL;
-    const double *y = NULL;
-    double mean_error;
-    int i, t;
-
-    if (arma != NULL) {
-	series = model_info_get_series(arma);
-	e = series[0];
-	pmod->lnL = model_info_get_ll(arma);
-    }
-
-    pmod->ci = ARMA;
-    pmod->ifc = arma_has_const(ainfo);
-
+    pmod->ifc = ainfo->ifc;
     pmod->dfn = ainfo->nc - pmod->ifc;
     pmod->dfd = pmod->nobs - pmod->dfn;
     pmod->ncoeff = ainfo->nc;
 
-    if (theta != NULL) {
-	for (i=0; i<pmod->ncoeff; i++) {
-	    pmod->coeff[i] = theta[i];
-	}
+    if (arma_has_seasonal(ainfo)) {
+	gretl_model_set_int(pmod, "arma_P", ainfo->P);
+	gretl_model_set_int(pmod, "arma_Q", ainfo->Q);
+	gretl_model_set_int(pmod, "arma_pd", ainfo->pd);	
     }
 
+    if (ainfo->d > 0 || ainfo->D > 0) {
+	gretl_model_set_int(pmod, "arima_d", ainfo->d);
+	gretl_model_set_int(pmod, "arima_D", ainfo->D);
+    }
+
+    if (ainfo->nexo > 0) {
+	gretl_model_set_int(pmod, "armax", 1);
+    }
+}
+
+/* write the various statistics from ARMA estimation into
+   a gretl MODEL struct */
+
+static void write_arma_model_stats (MODEL *pmod, const int *list, 
+				    struct arma_info *ainfo,
+				    const double **Z, 
+				    const DATAINFO *pdinfo)
+{
+    const double *y = NULL;
+    double mean_error;
+    int t;
+
+    pmod->ci = ARMA;
+
+    ainfo_data_to_model(ainfo, pmod);
+
     free(pmod->list);
     pmod->list = gretl_list_copy(list);
 
@@ -242,9 +204,6 @@ static void write_arma_model_stats (MODEL *pmod, model_info *arma,
     mean_error = pmod->ess = 0.0;
 
     for (t=pmod->t1; t<=pmod->t2; t++) {
-	if (e != NULL) {
-	    pmod->uhat[t] = e[t];
-	}
 	if (!na(y[t]) && !na(pmod->uhat[t])) {
 	    pmod->yhat[t] = y[t] - pmod->uhat[t];
 	    pmod->ess += pmod->uhat[t] * pmod->uhat[t];
@@ -260,36 +219,22 @@ static void write_arma_model_stats (MODEL *pmod, model_info *arma,
     mean_error /= pmod->nobs;
     gretl_model_set_double(pmod, "mean_error", mean_error);
 
-    if (arma != NULL) {
-	/* in X12A case we read this from file */
+    if (na(pmod->sigma)) {
+	/* in X12A or native exact cases this is already done */
 	pmod->sigma = sqrt(pmod->ess / pmod->nobs);
     } 
 
     pmod->rsq = pmod->adjrsq = pmod->fstt = NADBL;
     pmod->tss = NADBL;
 
-#if 0
-    arma_R2_and_F(pmod, y);
-#endif
-
-    if (arma != NULL) {
+    if (!arma_by_x12a(ainfo)) {
 	mle_criteria(pmod, 1);
     }
 
-    if (arma_has_seasonal(ainfo)) {
-	gretl_model_set_int(pmod, "arma_P", ainfo->P);
-	gretl_model_set_int(pmod, "arma_Q", ainfo->Q);
-	gretl_model_set_int(pmod, "arma_pd", ainfo->pd);	
-    }
-
-    if (ainfo->d > 0 || ainfo->D > 0) {
-	gretl_model_set_int(pmod, "arima_d", ainfo->d);
-	gretl_model_set_int(pmod, "arima_D", ainfo->D);
-    }
-
-    if (ainfo->r > 0) {
-	gretl_model_set_int(pmod, "armax", 1);
-    }
+    gretl_model_add_arma_varnames(pmod, pdinfo, ainfo->yno,
+				  ainfo->p, ainfo->q, 
+				  ainfo->P, ainfo->Q,
+				  ainfo->nexo);
 }
 
 static void calc_max_lag (struct arma_info *ainfo)
@@ -303,23 +248,33 @@ static void calc_max_lag (struct arma_info *ainfo)
     }
 
     ainfo->maxlag = pmax + dmax;
+
+#if ARMA_DEBUG
+    fprintf(stderr, "calc_max_lag: ainfo->maxlag = %d\n", ainfo->maxlag);
+#endif
 }
 
+#define SAMPLE_DEBUG 0
+
 static int 
 arma_adjust_sample (const DATAINFO *pdinfo, const double **Z, const int *list,
 		    struct arma_info *ainfo)
 {
     int t1 = pdinfo->t1, t2 = pdinfo->t2;
-    int an, i, v, t, t1min = 0;
+    int an, i, v, t, t1min;
     int vstart, pmax, anymiss;
 
+#if SAMPLE_DEBUG
+    fprintf(stderr, "arma_adjust_sample: at start, t1=%d, t2=%d\n",
+	    t1, t2);
+#endif
+
     vstart = arma_list_y_position(ainfo);
 
-    pmax = ainfo->p;
-    if (ainfo->P > 0) {
-	pmax += ainfo->P * ainfo->pd;
-    }   
+    pmax = ainfo->p + ainfo->P * ainfo->pd;
 
+    /* determine starting point for valid data, t1min */
+    t1min = 0;
     for (t=0; t<=pdinfo->t2; t++) {
 	anymiss = 0;
 	for (i=vstart; i<=list[0]; i++) {
@@ -336,19 +291,22 @@ arma_adjust_sample (const DATAINFO *pdinfo, const double **Z, const int *list,
 	}
     }
 
-#if 0
-    t1min += ainfo->maxlag;
-#else
-    if (!arma_by_x12a(ainfo)) {
-	/* not required for X-12-ARIMA? */
+    if (arma_by_x12a(ainfo) || arma_exact_ml(ainfo)) {
+	/* FIXME x12a in conditional mode? */
+	;
+    } else {
 	t1min += ainfo->maxlag;
     }
-#endif
 
+    /* if the notional starting point is before the start of
+       valid data, advance it */
     if (t1 < t1min) {
 	t1 = t1min;
     }
 
+    /* trim any missing obs from the end of the specified sample
+       range 
+    */
     for (t=pdinfo->t2; t>=t1; t--) {
 	anymiss = 0;
 	for (i=vstart; i<=list[0]; i++) {
@@ -365,7 +323,18 @@ arma_adjust_sample (const DATAINFO *pdinfo, const double **Z, const int *list,
 	}
     }
 
-    for (t=t1-pmax; t<t2; t++) {
+    if (arma_by_x12a(ainfo) || arma_exact_ml(ainfo)) {
+	/* FIXME x12a in conditional mode? */
+	t1min = t1;
+    } else {    
+	t1min = t1 - pmax; /* wrong? */
+	if (t1min < 0) {
+	    t1min = 0;
+	}
+    }
+
+    /* check for missing obs within the sample range */
+    for (t=t1min; t<t2; t++) {
 	for (i=vstart; i<=list[0]; i++) {
 	    if (t < t1 && i > vstart) {
 		continue;
@@ -387,6 +356,11 @@ arma_adjust_sample (const DATAINFO *pdinfo, const double **Z, const int *list,
 	return 1; 
     }
 
+#if SAMPLE_DEBUG
+    fprintf(stderr, "arma_adjust_sample: at end, t1=%d, t2=%d\n",
+	    t1, t2);
+#endif
+
     ainfo->t1 = t1;
     ainfo->t2 = t2;
 
@@ -519,16 +493,16 @@ static int check_arma_list (int *list, gretlopt opt,
 	if ((opt & OPT_N) || (armax && !hadconst)) {
 	    ;
 	} else {
-	    set_arma_has_const(ainfo);
+	    ainfo->ifc = 1;
 	}
     }
 
     if (err) {
 	gretl_errmsg_set(_("Error in arma command"));
     } else {
-	ainfo->r = list[0] - ((arma_has_seasonal(ainfo))? 7 : 4);
+	ainfo->nexo = list[0] - ((arma_has_seasonal(ainfo))? 7 : 4);
 	ainfo->nc = ainfo->p + ainfo->q + ainfo->P + ainfo->Q
-	    + ainfo->r + arma_has_const(ainfo);
+	    + ainfo->nexo + ainfo->ifc;
 	ainfo->yno = (arma_has_seasonal(ainfo))? list[7] : list[4];
     }
 
@@ -601,16 +575,16 @@ static int check_arima_list (int *list, gretlopt opt,
 	if ((opt & OPT_N) || (armax && !hadconst)) {
 	    ;
 	} else {
-	    set_arma_has_const(ainfo);
+	    ainfo->ifc = 1;
 	}
     }
 
     if (err) {
 	gretl_errmsg_set(_("Error in arma command"));
     } else {
-	ainfo->r = list[0] - ((arma_has_seasonal(ainfo))? 9 : 5);
+	ainfo->nexo = list[0] - ((arma_has_seasonal(ainfo))? 9 : 5);
 	ainfo->nc = ainfo->p + ainfo->q + ainfo->P + ainfo->Q
-	    + ainfo->r + arma_has_const(ainfo);
+	    + ainfo->nexo + ainfo->ifc;
 	ainfo->yno = (arma_has_seasonal(ainfo))? list[9] : list[5];
     }
 
