@@ -36,11 +36,6 @@
 
 static void free_sheet (void);
 static int allocate_row_col (int row, int col, wbook *book);
-static char *copy_unicode_string (unsigned char *src, int remlen, 
-				  int *skip, int *slop);
-static char *convert8to7 (const char *s, int count);
-static char *convert16to7 (const unsigned char *s, int count);
-static char *make_string (char *str);
 
 int debug_print;
 
@@ -203,6 +198,129 @@ static double biff_get_rk (const unsigned char *ptr)
     return NADBL;
 }
 
+static char *convert8to7 (const char *s, int count) 
+{
+    char *dest;
+
+    if (count > VNAMELEN - 1) {
+	count = VNAMELEN - 1;
+    }
+
+    dest = malloc(VNAMELEN);
+    *dest = '\0';
+    strncat(dest, s, count);
+    iso_to_ascii(dest);
+    top_n_tail(dest);
+
+    dprintf("convert8to7: returning '%s'\n", dest);
+
+    return dest;
+}
+
+static char *convert16to7 (const unsigned char *s, int count) 
+{
+    char *dest;
+    int i, u, j = 0;
+
+    dest = malloc(VNAMELEN);
+    if (dest == NULL) {
+	return NULL;
+    }
+
+    memset(dest, 0, VNAMELEN);
+
+    for (i=0; i<count && j<VNAMELEN-1; i++) {
+	u = MS_OLE_GET_GUINT16(s);
+	s += 2;
+	if ((isalnum(u) || ispunct(u)) && u < 128) {
+	    dest[j++] = u;
+	}
+    }
+
+    dprintf("convert16to7: returning '%s'\n", dest);
+
+    return dest;    
+}
+
+static char *
+copy_unicode_string (unsigned char *src, int remlen, 
+		     int *skip, int *slop) 
+{
+    int count = MS_OLE_GET_GUINT16(src);
+    unsigned char flags = *(src + 2);
+    int this_skip = 3, skip_to_next = 3;
+    int csize = (flags & 0x01)? 2 : 1;
+
+    dprintf("copy_unicode_string: count = %d, csize = %d\n",
+	    count, csize);
+
+    if (flags & 0x08) {
+	dprintf(" contains Rich-Text info\n");
+    }
+    if (flags & 0x04) {
+	dprintf(" contains Far-East info\n");
+    }    
+
+    skip_to_next += count * csize;
+
+    if (flags & 0x08) {
+	guint16 rich_text_info_len = 0;
+
+	rich_text_info_len = 4 * MS_OLE_GET_GUINT16(src + 3);
+	this_skip += 2;
+	skip_to_next += 2 + rich_text_info_len;
+    } 
+
+    if (flags & 0x04) {
+	guint32 far_east_info_len = 0;
+	int far_east_offset = 3;
+
+	if (flags & 0x08) {
+	    far_east_offset = 5;
+	}
+	far_east_info_len = MS_OLE_GET_GUINT32(src + far_east_offset);
+	this_skip += 4;
+	skip_to_next += 4 + far_east_info_len;
+    }
+
+    /* skip for the next read */
+    if (skip != NULL) {
+	*skip = skip_to_next;
+    }
+
+    /* size check */
+    if (slop != NULL) {
+	if (remlen > 0 && this_skip + count > remlen) {
+	    *slop = this_skip + count - remlen;
+	} else {
+	    *slop = 0;
+	}
+    }
+
+    if (count > 64) {
+	/* let's not mess with excessive strings */
+	return g_strdup("bigstr");
+    } else if (csize == 1) {
+	return convert8to7((char *) src + this_skip, count);
+    } else { 
+	return convert16to7(src + this_skip, count);
+    }
+}
+
+static char *make_string (char *str) 
+{
+    char *ret = NULL;
+
+    if (str != NULL) {
+	ret = g_strdup_printf("\"%s", str);
+	free(str);
+    } else {
+	ret = g_strdup("\"");
+    }
+
+    return ret;
+}    
+
 static int row_col_err (int row, int col, PRN *prn) 
 {
     static int prevrow = -1, prevcol = -1;
@@ -231,6 +349,12 @@ static int check_copy_string (struct sheetrow *prow, int row, int col,
 	int commas = 0;
 	static int warned = 0;
 
+	if (len == 0) {
+	    dprintf("converting sst[%d] '%s' to NA\n", idx, s);
+	    prow->cells[col] = g_strdup("NA");
+	    return 0;
+	}	    
+
 	for (i=0; i<len; i++) {
 	    if (!isdigit(s[i]) && s[i] != ' ' && s[i] != '-' &&
 		s[i] != ',' && s[i] != '.') {
@@ -264,10 +388,9 @@ static int check_copy_string (struct sheetrow *prow, int row, int col,
 	    }
 	    *p = '\0';
 	    dprintf("converting sst[%d] '%s' to numeric as %s\n", idx, s, q);
-	    prow->cells[col] = g_strdup_printf("%s", q);
-	    free(q);
+	    prow->cells[col] = q;
 	    return 0;
-	}
+	} 
     }
 
     dprintf("copying sst[%d] '%s' into place\n", idx, s);
@@ -396,6 +519,8 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 		unsigned char flags = *ptr;
 		int csize = (flags & 0x01)? 2 : 1;
 
+		dprintf("BIFF_CONTINUE: slop = %d, csize = %d\n", (int) slop,
+			(int) csize);
 		ptr += 1 + csize * slop;
 	    }
 	    for (k=sstnext; k<sstsize; k++) {
@@ -403,7 +528,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 		if (remlen <= 0) {
 		    break;
 		}
-		dprintf("Working on sst[%d]\n", k);
+		dprintf("Working on sst[%d], remlen = %d\n", k, remlen);
 		sst[k] = copy_unicode_string(ptr, remlen, &skip, &slop);
 		ptr += skip;
 	    }
@@ -422,6 +547,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	
 	    prow = rows + i;
 	    ptr = q->data + 8;
+	    fprintf(stderr, "BIFF_LABEL: calling convert8to7\n");
 	    prow->cells[j] = make_string(convert8to7((char *) ptr, len));
 	}
 	break;
@@ -440,6 +566,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	    } else if (sst[sidx] != NULL) {
 		check_copy_string(prow, i, j, sidx, sst[sidx]);
 	    } else {
+		dprintf("sst[%d] seems to be NULL, leaving string blank\n", (int) sidx);
 		prow->cells[j] = malloc(2);
 		if (prow->cells[j] != NULL) {
 		    prow->cells[j][0] = '\0';
@@ -454,7 +581,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	} else {
 	    val = get_le_double(q->data + 6);
 	    prow = rows + i;
-	    prow->cells[j] = g_strdup_printf("%.10g", val);
+	    prow->cells[j] = g_strdup_printf("%.15g", val);
 	    dprintf("Got NUMBER (%g), row=%d, col=%d\n", val, i, j);
 	}
 	break;
@@ -465,7 +592,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	} else {
 	    val = biff_get_rk(q->data + 6);
 	    prow = rows + i;
-	    prow->cells[j] = g_strdup_printf("%.10g", val);
+	    prow->cells[j] = g_strdup_printf("%.15g", val);
 	    dprintf("Got RK (%g), row=%d, col=%d\n", val, i, j);
 	}
 	break;
@@ -480,7 +607,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	    }
 	    val = biff_get_rk(q->data + 6 + 6 * k);
 	    prow = rows + i; /* might have moved */
-	    prow->cells[j] = g_strdup_printf("%.10g", val);
+	    prow->cells[j] = g_strdup_printf("%.15g", val);
 	    dprintf(" MULRK[col=%d] = %g\n", j, val);
 	    j++;
 	}
@@ -520,7 +647,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 		    fprintf(stderr, "Got a NaN\n");
 		    prow->cells[j] = g_strdup("-999.0");
 		} else {
-		    prow->cells[j] = g_strdup_printf("%.10g", val);
+		    prow->cells[j] = g_strdup_printf("%.15g", val);
 		}
 	    }
 	}
@@ -645,17 +772,16 @@ static int process_sheet (const char *filename, wbook *book, PRN *prn)
     result = ms_ole_stream_open(&stream, file, "/", "workbook", 'r');
 
     if (result != MS_OLE_ERR_OK) {
-	ms_ole_stream_close (&stream);
-
+	ms_ole_stream_close(&stream);
 	result = ms_ole_stream_open(&stream, file, "/", "book", 'r');
 	if (result != MS_OLE_ERR_OK) {
 	    ms_ole_stream_close(&stream);
-	    fprintf(stderr, _("No book or workbook streams found."));
+	    fputs("No book or workbook streams found\n", stderr);
 	    return 1;
 	}
     }
 
-    fprintf(stderr, _("Reading file...\n"));
+    fputs("Reading file...\n", stderr);
     q = ms_biff_query_new(stream);
 
     while (!gotbof && ms_biff_query_next(q)) {
@@ -713,154 +839,6 @@ static int process_sheet (const char *filename, wbook *book, PRN *prn)
     return err;    
 }
 
-static char *make_string (char *str) 
-{
-    char *ret = NULL;
-
-    if (str != NULL) {
-	ret = g_strdup_printf("\"%s", str);
-	free(str);
-    } else {
-	ret = g_strdup("\"");
-    }
-
-    return ret;
-}    
-
-static char *
-copy_unicode_string (unsigned char *src, int remlen, 
-		     int *skip, int *slop) 
-{
-    int count = MS_OLE_GET_GUINT16(src);
-    unsigned char flags = *(src + 2);
-    int this_skip = 3, skip_to_next = 3;
-    int csize = (flags & 0x01)? 2 : 1;
-
-    dprintf("copy_unicode_string: count = %d, csize = %d\n",
-	    count, csize);
-
-    if (flags & 0x08) {
-	dprintf(" contains Rich-Text info\n");
-    }
-    if (flags & 0x04) {
-	dprintf(" contains Far-East info\n");
-    }    
-
-    skip_to_next += count * csize;
-
-    if (flags & 0x08) {
-	guint16 rich_text_info_len = 0;
-
-	rich_text_info_len = 4 * MS_OLE_GET_GUINT16(src + 3);
-	this_skip += 2;
-	skip_to_next += 2 + rich_text_info_len;
-    } 
-
-    if (flags & 0x04) {
-	guint32 far_east_info_len = 0;
-	int far_east_offset = 3;
-
-	if (flags & 0x08) {
-	    far_east_offset = 5;
-	}
-	far_east_info_len = MS_OLE_GET_GUINT32(src + far_east_offset);
-	this_skip += 4;
-	skip_to_next += 4 + far_east_info_len;
-    }
-
-    /* skip for the next read */
-    if (skip != NULL) {
-	*skip = skip_to_next;
-    }
-
-    /* size check */
-    if (slop != NULL) {
-	if (remlen > 0 && this_skip + count > remlen) {
-	    *slop = this_skip + count - remlen;
-	} else {
-	    *slop = 0;
-	}
-    }
-
-    if (count > 64) {
-	/* let's not mess with excessive strings */
-	return g_strdup("bigstr");
-    } else if (csize == 1) {
-	return convert8to7((char *) src + this_skip, count);
-    } else { 
-	return convert16to7(src + this_skip, count);
-    }
-}
-
-static char *convert8to7 (const char *s, int count) 
-{
-    char *dest;
-
-    if (count > VNAMELEN - 1) {
-	count = VNAMELEN - 1;
-    }
-
-    dest = malloc(VNAMELEN);
-    *dest = '\0';
-    strncat(dest, s, count);
-    iso_to_ascii(dest);
-
-    if (*dest == '\0') {
-	strcpy(dest, "varname");
-    }
-
-    dprintf("convert8to7: returning '%s'\n", dest);
-
-    return dest;
-}
-
-static char *convert16to7 (const unsigned char *s, int count) 
-{
-    char *dest;
-    int i, u, j = 0;
-
-    dest = malloc(VNAMELEN);
-    if (dest == NULL) {
-	return NULL;
-    }
-
-    memset(dest, 0, VNAMELEN);
-
-#if 1
-    for (i=0; i<count && j<VNAMELEN-1; i++) {
-	u = MS_OLE_GET_GUINT16(s);
-	s += 2;
-	if ((isalnum(u) || ispunct(u)) && u < 128) {
-	    dest[j++] = u;
-	}
-    }
-#else
-    printf("convert16to7: count = %d\n", count);
-    for (i=0; i<count; i++) {
-	u = MS_OLE_GET_GUINT16(s);
-	s += 2;
-	if ((isalnum(u) || ispunct(u) || u == ' ') && u < 128) {
-	    putchar(u);
-	    if (j<VNAMELEN-1) {
-		dest[j++] = u;
-	    }
-	    if (i % 56 == 0) {
-		putchar('\n');
-	    }	
-	}
-    }
-    printf("\nconvert16to7: returning '%s'\n", dest);
-#endif
-
-    if (*dest == '\0') {
-	strcpy(dest, "varname");
-    }
-
-    dprintf("convert16to7: returning '%s'\n", dest);
-
-    return dest;    
-}
-
 static void row_init (struct sheetrow *row)
 {
     row->last = 0;
@@ -1346,8 +1324,9 @@ static void book_time_series_setup (wbook *book, DATAINFO *newinfo, int pd)
     book_unset_obs_labels(book);
 }
 
-int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
-		    PRN *prn)
+static int 
+real_excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
+		     int gui, PRN *prn)
 {
     wbook xbook;
     wbook *book = &xbook;
@@ -1379,7 +1358,7 @@ int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	wbook_print_info(book);
     }
 
-    if (!err) {
+    if (!err && gui) {
 	wsheet_menu(book, book->nsheets > 1);
 	if (book_debugging(book)) {
 	    debug_print = 1;
@@ -1548,3 +1527,17 @@ int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 
     return err;
 }  
+
+int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
+		    PRN *prn)
+{
+    return real_excel_get_data(fname, pZ, pdinfo, 1, prn);
+}
+
+int cli_get_xls (const char *fname, double ***pZ, DATAINFO *pdinfo,
+		 PRN *prn)
+{
+    return real_excel_get_data(fname, pZ, pdinfo, 0, prn);
+}
+
+
