@@ -2,7 +2,7 @@
 #define FCP_H
 
 int garch_estimate (int t1, int t2, int nobs, 
-		    const double **X, int nx, double *ydet, 
+		    const double **X, int nx, 
 		    double *coeff, int nc, double *vc, 
 		    double *res2, double *res, double *h,
 		    const double *ystoc, double *amax, double *b, 
