@@ -19,11 +19,6 @@
 
 #ifdef EXCEL_IMPORTER
 
-# ifndef WIN32
-#  define BUILDING_PLUGIN
-#  include "../gui2/dialogs.h" /* for infobox */
-# endif
-
 static void set_all_missing (double **Z, DATAINFO *pdinfo)
 {
     int i, t;
@@ -43,11 +38,12 @@ static void invalid_varname (PRN *prn)
     pputs(prn, _("\nPlease rename this variable and try again"));
 }
 
-static int label_is_date (char *str)
+static int label_is_date (char *str, int *submax)
 {
     int len = strlen(str);
-    int i, d, pd = 0;
-    double dd, sub;
+    int i, d, sub = 0, pd = 0;
+    char *p;
+    double dd;
 
 #if 0
     fprintf(stderr, "label_is_date: looking at '%s'\n", str);
@@ -60,14 +56,29 @@ static int label_is_date (char *str)
 	}
     }
 
-    if (len == 4 && sscanf(str, "%4d", &d) && d > 0 && d < 3000) {
+    p = strchr(str, '.');
+
+    if (len == 4 && sscanf(str, "%d", &d) && d > 0 && d < 3000) {
 	pd = 1;
-    } else if (len == 6 && sscanf(str, "%lf", &dd) && dd > 0 && dd < 3000) { 
-	sub = 10.0 * (dd - (int) dd);
-	if (sub >= .999 && sub <= 4.001) pd = 4;
-    } else if (len == 7 && sscanf(str, "%lf", &dd) && dd > 0 && dd < 3000) {
-	sub = 100.0 * (dd - (int) dd);
-	if (sub >= .9999 && sub <= 12.0001) pd = 12;
+    } else if (p != NULL) {
+	if (len == 6) {
+	    if (sscanf(str, "%lf", &dd) && dd > 0 && dd < 3000) { 
+		sub = atoi(p + 1);
+		if (sub > 0 && sub < 5) {
+		    pd = 4;
+		}
+	    }
+	} else if (len == 7) {
+	    if (sscanf(str, "%lf", &dd) && dd > 0 && dd < 3000) {
+		sub = atoi(p + 1);
+		if (sub > 0 && sub < 13) {
+		    pd = 12;
+		}
+	    }
+	}
+	if (sub > *submax) {
+	    *submax = sub;
+	}
     }
 
     return pd;
@@ -251,6 +262,7 @@ consistent_date_labels (int nrows, int row_offset, int col_offset, char **labels
 {
     int t, tstart = 1 + row_offset;
     int pd = 0, pdbak = 0;
+    int submax = 0;
     double x, xbak = 0.0;
     char *test;
 
@@ -267,14 +279,18 @@ consistent_date_labels (int nrows, int row_offset, int col_offset, char **labels
 	    test++;
 	}
 
-	pd = label_is_date(test);
-
+	pd = label_is_date(test, &submax);
 	if (pd == 0) {
 	    fprintf(stderr, " no: label '%s' on row %d is not a valid date\n", 
 		    test, t + 1);
 	    return 0;
 	}
 
+	if (pd == 12 && t - tstart > 3 && submax == 4) {
+	    /* appeared to be monthly but really quarterly? */
+	    pd = pdbak = 4;
+	}
+
 	x = atof(test);
 
 	if (t > tstart) {
@@ -405,6 +421,28 @@ static void colspin_changed (GtkEditable *ed, GtkWidget *w)
 }
 
 #ifdef EXCEL_IMPORTER
+# ifndef WIN32
+
+void infobox (const char *template, ...)
+{
+    GtkWidget *dialog;
+    char msg[MAXLEN];
+    va_list args;
+
+    va_start(args, template);
+    vsprintf(msg, template, args);
+    va_end(args);
+
+    dialog = gtk_message_dialog_new (NULL, 
+				     GTK_DIALOG_DESTROY_WITH_PARENT,
+				     GTK_MESSAGE_INFO,
+				     GTK_BUTTONS_CLOSE,
+				     msg);
+    gtk_dialog_run (GTK_DIALOG (dialog));
+    gtk_widget_destroy (dialog);
+}
+
+# endif /* !WIN32 */
 
 static 
 void debug_callback (GtkWidget *w, wbook *book)
@@ -416,27 +454,23 @@ void debug_callback (GtkWidget *w, wbook *book)
     }
 
     if (book_debugging(book) && !done) {
+# ifdef WIN32
 	gchar *msg;
 
-# ifdef WIN32
 	make_debug_fname();
 	msg = g_strdup_printf(I_("Sending debugging output to %s"),
 			      debug_fname);
 	MessageBox(NULL, msg, "gretl", MB_OK | MB_ICONINFORMATION);
+	g_free(msg);	
 # else
-	msg = g_strdup_printf(_("Sending debugging output to %s"),
-			      "stderr");
-	infobox(msg);
+	infobox(_("Sending debugging output to %s"), "stderr");
 # endif
-	g_free(msg);
 	done = 1;
     }
 }
 
 #endif /* EXCEL_IMPORTER */
 
-#if GTK_MAJOR_VERSION >= 2
-
 static
 void wsheet_menu_select_row (GtkTreeSelection *selection, wbook *book)
 {
@@ -547,8 +581,7 @@ static void wsheet_menu (wbook *book, int multisheet)
     GtkWidget *vbox, *hbox;
     GtkObject *c_adj, *r_adj;
 
-    w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-
+    w = gtk_dialog_new();
     gtk_window_set_title(GTK_WINDOW(w), _("gretl: spreadsheet import"));
 
     g_signal_connect_after(G_OBJECT(w), "delete_event",
@@ -556,38 +589,37 @@ static void wsheet_menu (wbook *book, int multisheet)
     g_signal_connect(G_OBJECT(w), "destroy",  
 		     G_CALLBACK(gtk_main_quit), NULL);
 
-    vbox = gtk_vbox_new (FALSE, 5);
-    gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
-    gtk_container_add(GTK_CONTAINER(w), vbox);
+    vbox = GTK_DIALOG(w)->vbox;
+    gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
 
     /* selection of starting column and row */
     label = gtk_label_new(_("Start import at:"));
-    gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 5);
 
-    hbox = gtk_hbox_new (FALSE, 5);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 5);
+    hbox = gtk_hbox_new(FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 5);
 
     /* starting column spinner */
     tmp = gtk_label_new(_("column:"));
     c_adj = gtk_adjustment_new(1, 1, 256, 1, 1, 1);
-    book->colspin = gtk_spin_button_new (GTK_ADJUSTMENT(c_adj), 1, 0);
-    g_signal_connect (c_adj, "value_changed",
-		      G_CALLBACK (wbook_get_col_offset), book);
+    book->colspin = gtk_spin_button_new(GTK_ADJUSTMENT(c_adj), 1, 0);
+    g_signal_connect(c_adj, "value_changed",
+		     G_CALLBACK(wbook_get_col_offset), book);
     gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(book->colspin),
 				      GTK_UPDATE_IF_VALID);
-    gtk_box_pack_start (GTK_BOX (hbox), tmp, FALSE, FALSE, 5);
-    gtk_box_pack_start (GTK_BOX (hbox), book->colspin, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(hbox), book->colspin, FALSE, FALSE, 5);
 
     /* starting row spinner */
     tmp = gtk_label_new(_("row:"));
     r_adj = gtk_adjustment_new(1, 1, 256, 1, 1, 1);
     book->rowspin = gtk_spin_button_new (GTK_ADJUSTMENT(r_adj), 1, 0);
-    g_signal_connect (r_adj, "value_changed",
-		      G_CALLBACK (wbook_get_row_offset), book);
+    g_signal_connect(r_adj, "value_changed",
+		     G_CALLBACK(wbook_get_row_offset), book);
     gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(book->rowspin),
 				      GTK_UPDATE_IF_VALID);
-    gtk_box_pack_start (GTK_BOX (hbox), tmp, FALSE, FALSE, 5);
-    gtk_box_pack_start (GTK_BOX (hbox), book->rowspin, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(hbox), book->rowspin, FALSE, FALSE, 5);
 
     /* column label feedback */
     hbox = gtk_hbox_new (FALSE, 5);
@@ -610,26 +642,27 @@ static void wsheet_menu (wbook *book, int multisheet)
     gtk_box_pack_start(GTK_BOX(vbox), tmp, TRUE, TRUE, 5);
 #endif
 
-    hbox = gtk_hbox_new (TRUE, 5);
-    gtk_container_add(GTK_CONTAINER(vbox), hbox);
-
-    tmp = gtk_button_new_from_stock(GTK_STOCK_OK);
-    gtk_box_pack_start (GTK_BOX (hbox), 
-                        tmp, TRUE, TRUE, 0);
-    g_signal_connect_swapped (G_OBJECT (tmp), "clicked", 
-			      G_CALLBACK (gtk_widget_destroy), 
-			      G_OBJECT (w));
-    GTK_WIDGET_SET_FLAGS (tmp, GTK_CAN_DEFAULT);
-    gtk_widget_grab_default(tmp);
+    hbox = GTK_DIALOG(w)->action_area;
+    gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
+    gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 10);
 
+    /* Cancel button */
     tmp = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
-    gtk_box_pack_start (GTK_BOX (hbox), 
-                        tmp, TRUE, TRUE, 0);
-    g_signal_connect (G_OBJECT (tmp), "clicked", 
-		      G_CALLBACK(wsheet_menu_cancel), book);
-    g_signal_connect_swapped (G_OBJECT (tmp), "clicked", 
-			      G_CALLBACK (gtk_widget_destroy), 
-			      G_OBJECT (w));
+    gtk_container_add(GTK_CONTAINER(hbox), tmp);
+    g_signal_connect(G_OBJECT (tmp), "clicked", 
+		     G_CALLBACK(wsheet_menu_cancel), book);
+    g_signal_connect_swapped(G_OBJECT (tmp), "clicked", 
+			     G_CALLBACK (gtk_widget_destroy), 
+			     G_OBJECT (w));
+
+    /* OK button */
+    tmp = gtk_button_new_from_stock(GTK_STOCK_OK);
+    gtk_container_add(GTK_CONTAINER(hbox), tmp);
+    g_signal_connect_swapped(G_OBJECT (tmp), "clicked", 
+			     G_CALLBACK (gtk_widget_destroy), 
+			     G_OBJECT (w));
+    GTK_WIDGET_SET_FLAGS(tmp, GTK_CAN_DEFAULT);
+    gtk_widget_grab_default(tmp);
 
     gtk_entry_set_activates_default(GTK_ENTRY(book->colspin), TRUE);
     gtk_entry_set_activates_default(GTK_ENTRY(book->rowspin), TRUE);
@@ -641,164 +674,4 @@ static void wsheet_menu (wbook *book, int multisheet)
     gtk_main();
 }
 
-#else /* GTK 1.2 */
-
-static
-void wsheet_menu_select_row (GtkCList *clist, gint row, gint column, 
-			     GdkEventButton *event, wbook *book) 
-{
-    book->selected = row;
-}
-
-static 
-void wsheet_menu_make_list (GtkWidget *widget, wbook *book)
-{
-    gchar *row[1];
-    int i;
-
-    gtk_clist_clear(GTK_CLIST(widget));
-    for (i=0; i<book->nsheets; i++) {
-	row[0] = book->sheetnames[i];
-        gtk_clist_append(GTK_CLIST (widget), row);
-    }
-
-    gtk_clist_select_row(GTK_CLIST(widget), 0, 0);  
-}
-
-static 
-void wsheet_menu_cancel (GtkWidget *w, wbook *book)
-{
-    book->selected = -1;
-}
-
-static 
-void wbook_get_col_offset (GtkWidget *w, wbook *book)
-{
-    book->col_offset = gtk_spin_button_get_value_as_int
-	(GTK_SPIN_BUTTON(book->colspin)) - 1;
-}
-
-static 
-void wbook_get_row_offset (GtkWidget *w, wbook *book)
-{
-    book->row_offset = gtk_spin_button_get_value_as_int
-	(GTK_SPIN_BUTTON(book->rowspin)) - 1;
-}
-
-static gint entry_activate (GtkWidget *w, GdkEventKey *key, gpointer p)
-{
-    GtkWidget *top = gtk_widget_get_toplevel(w);
-
-    gtk_window_activate_default(GTK_WINDOW(top));
-
-    return FALSE;
-}
-
-static void wsheet_menu (wbook *book, int multisheet)
-{
-    GtkWidget *w, *tmp, *frame;
-    GtkWidget *vbox, *hbox, *list;
-    GtkObject *c_adj, *r_adj;
-
-    w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-
-    gtk_window_set_title(GTK_WINDOW(w), "gretl: spreadsheet import");
-
-    gtk_signal_connect(GTK_OBJECT(w), "destroy",  
-		       GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
-
-    vbox = gtk_vbox_new (FALSE, 5);
-    gtk_container_set_border_width (GTK_CONTAINER (vbox), 10);
-    gtk_container_add(GTK_CONTAINER(w), vbox);    
-
-    /* selection of starting column and row */
-    tmp = gtk_label_new(_("Start import at:"));
-    gtk_box_pack_start (GTK_BOX (vbox), tmp, FALSE, FALSE, 5);
-
-    hbox = gtk_hbox_new (FALSE, 5);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 5);
-
-    /* starting column spinner */
-    tmp = gtk_label_new("column:");
-    c_adj = gtk_adjustment_new(1, 1, 256, 1, 1, 1);
-    book->colspin = gtk_spin_button_new (GTK_ADJUSTMENT(c_adj), 1, 0);
-    gtk_signal_connect (c_adj, "value_changed",
-			GTK_SIGNAL_FUNC (wbook_get_col_offset), book);
-    gtk_signal_connect(GTK_OBJECT(book->colspin), "activate", 
-		       GTK_SIGNAL_FUNC(entry_activate), NULL);
-    gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(book->colspin),
-				      GTK_UPDATE_IF_VALID);
-    gtk_box_pack_start (GTK_BOX (hbox), tmp, FALSE, FALSE, 5);
-    gtk_box_pack_start (GTK_BOX (hbox), book->colspin, FALSE, FALSE, 5);
-
-    /* starting row spinner */
-    tmp = gtk_label_new("row:");
-    r_adj = gtk_adjustment_new(1, 1, 256, 1, 1, 1);
-    book->rowspin = gtk_spin_button_new (GTK_ADJUSTMENT(r_adj), 1, 0);
-    gtk_signal_connect (r_adj, "value_changed",
-			GTK_SIGNAL_FUNC (wbook_get_row_offset), book);
-    gtk_signal_connect(GTK_OBJECT(book->rowspin), "activate", 
-		       GTK_SIGNAL_FUNC(entry_activate), NULL);
-    gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(book->rowspin),
-				      GTK_UPDATE_IF_VALID);
-    gtk_box_pack_start (GTK_BOX (hbox), tmp, FALSE, FALSE, 5);
-    gtk_box_pack_start (GTK_BOX (hbox), book->rowspin, FALSE, FALSE, 5);
-
-    /* column label feedback */
-    hbox = gtk_hbox_new (FALSE, 5);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-    tmp = gtk_label_new("(A)");
-    gtk_box_pack_start (GTK_BOX (hbox), tmp, FALSE, FALSE, 5);
-    gtk_signal_connect (GTK_OBJECT(GTK_EDITABLE(book->colspin)), "changed",
-			GTK_SIGNAL_FUNC (colspin_changed), tmp);
-
-    /* choose the worksheet (if applicable) */
-    if (multisheet) {
-	frame = gtk_frame_new("Sheet to import");
-	gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 5);
-	list = gtk_clist_new(1);
-	gtk_signal_connect (GTK_OBJECT (list), "select_row", 
-			    GTK_SIGNAL_FUNC (wsheet_menu_select_row), 
-			    book);
-	wsheet_menu_make_list(list, book);
-	gtk_container_set_border_width (GTK_CONTAINER (list), 10);
-	gtk_container_add(GTK_CONTAINER(frame), list);
-    }
-
-#ifdef EXCEL_IMPORTER
-    tmp = gtk_check_button_new_with_label(_("Produce debugging output"));
-    gtk_signal_connect(GTK_OBJECT(tmp), "toggled", 
-		       GTK_SIGNAL_FUNC(debug_callback), 
-		       book);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), FALSE);
-    gtk_box_pack_start(GTK_BOX(vbox), tmp, TRUE, TRUE, 5);
-#endif
-
-    hbox = gtk_hbox_new (TRUE, 5);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 5);
-
-    tmp = gtk_button_new_with_label("OK");
-    GTK_WIDGET_SET_FLAGS (tmp, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (hbox), 
-                        tmp, TRUE, TRUE, FALSE);
-    gtk_signal_connect_object (GTK_OBJECT (tmp), "clicked", 
-                               GTK_SIGNAL_FUNC (gtk_widget_destroy), 
-                               GTK_OBJECT (w));
-    gtk_widget_grab_default (tmp);
-
-    tmp = gtk_button_new_with_label("Cancel");
-    gtk_box_pack_start (GTK_BOX (hbox), 
-                        tmp, TRUE, TRUE, FALSE);
-    gtk_signal_connect (GTK_OBJECT (tmp), "clicked", 
-			GTK_SIGNAL_FUNC (wsheet_menu_cancel), book);
-    gtk_signal_connect_object (GTK_OBJECT (tmp), "clicked", 
-                               GTK_SIGNAL_FUNC (gtk_widget_destroy), 
-                               GTK_OBJECT (w));
-
-    gtk_widget_show_all(w);
-
-    gtk_main();
-}
-
-#endif /* gtk 2.0 vs 1.2 */
 
