@@ -107,7 +107,7 @@ static int density_plot (const double *x, double s, double h,
     double xt, xdt;
     int ktype, t;
 
-    if (gnuplot_init(0, &fp)) {
+    if (gnuplot_init(PLOT_KERNEL, &fp)) {
 	return E_FOPEN;
     }
 
@@ -120,14 +120,12 @@ static int density_plot (const double *x, double s, double h,
     get_xmin_xmax(x, s, n, &xmin, &xmax);
     xstep = (xmax - xmin) / kn;
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "C");
-#endif
+    gretl_push_c_numeric_locale();
 
-    fputs("# kernel density plot (no auto-parse)\n", fp);
     fputs("set nokey\n", fp); 
     fprintf(fp, "set xrange [%g:%g]\n", xmin, xmax);
 
+    fputs("# literal lines = 2\n", fp);
     fprintf(fp, "set label '%s' at graph .65, graph .97\n",
 	    (ktype == GAUSSIAN_KERNEL)? I_("Gaussian kernel") :
 	    I_("Epanechnikov kernel"));
@@ -147,9 +145,7 @@ static int density_plot (const double *x, double s, double h,
     }
     fputs("e\n", fp);
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "");
-#endif
+    gretl_pop_c_numeric_locale();
 
     fclose(fp);
 
