@@ -66,51 +66,6 @@ typedef enum {
 
 #define SBSIZE 4096
 
-#if GTK_MAJOR_VERSION < 2
-
-enum {
-    GTK_STOCK_OK,
-    GTK_STOCK_CANCEL
-};
-
-# define G_OBJECT(o)                    GTK_OBJECT(o)
-# define g_object_set_data(o,s,d)       gtk_object_set_data(o,s,d)
-# define g_object_get_data(o,s)         gtk_object_get_data(o,s)
-# define G_CALLBACK(f)                  GTK_SIGNAL_FUNC(f)
-# define g_signal_connect(o,s,f,p)      gtk_signal_connect(o,s,f,p)
-# define gtk_widget_set_size_request(g,w,h) gtk_widget_set_usize(g,w,h)
-# define gtk_notebook_set_current_page(n,p) gtk_notebook_set_page(n,p)
-
-GtkWidget *standard_button (int code)
-{
-    const char *button_strings[] = {
-	N_("OK"),
-	N_("Cancel")
-    };
-
-    return gtk_button_new_with_label(_(button_strings[code]));
-}
-
-static gint entry_activate (GtkWidget *w, GdkEventKey *key, gpointer p)
-{
-    GtkWidget *top = gtk_widget_get_toplevel(w);
-
-    gtk_window_activate_default(GTK_WINDOW(top));
-    return FALSE;
-}
-
-void gtk_entry_set_activates_default (GtkEntry *entry, gboolean setting)
-{
-    gtk_signal_connect(GTK_OBJECT(entry), "activate", 
-		       GTK_SIGNAL_FUNC(entry_activate), NULL);
-}
-
-#else
-
-# define standard_button(s) gtk_button_new_from_stock(s)
-
-#endif /* alternate gtk versions */
-
 struct msg_info {
     char *recip;
     char *sender;
@@ -458,11 +413,7 @@ static void finalize_pop_settings (GtkWidget *w, struct pop_dialog *pd)
 
 static void border_width (GtkWidget *w, int b)
 {
-#if GTK_MAJOR_VERSION < 2
-    gtk_container_border_width(GTK_CONTAINER(w), b);
-#else
     gtk_container_set_border_width(GTK_CONTAINER(w), b); 
-#endif
 }
 
 static void set_dialog_border_widths (GtkWidget *dlg)
@@ -528,7 +479,17 @@ static int is_data_file (const char *fname)
     int ret = 1;
 
     if (fname != NULL && strlen(fname) > 4) {
-	ret = !strcmp(fname + strlen(fname) - 4, ".gdt");
+	const char *ext = strrchr(fname, '.');
+
+	if (ext != NULL) {
+	    if (!strcmp(ext, ".inp")) {
+		ret = 0;
+	    } else if (!strcmp(ext, ".gfn")) {
+		ret = 0;
+	    } else if (!strcmp(ext, ".gretl")) {
+		ret = 0;
+	    }
+	}	    
     }
 
     return ret;
@@ -573,6 +534,7 @@ mail_to_dialog (const char *fname, struct mail_info *minfo, struct msg_info *msg
 
     gtk_window_set_title(GTK_WINDOW(md.dlg), _("gretl: send mail"));
     set_dialog_border_widths(md.dlg);
+    gtk_dialog_set_has_separator(GTK_DIALOG(md.dlg), FALSE);
     gtk_window_set_position(GTK_WINDOW(md.dlg), GTK_WIN_POS_MOUSE);
 
     nb = gtk_notebook_new();
@@ -625,10 +587,10 @@ mail_to_dialog (const char *fname, struct mail_info *minfo, struct msg_info *msg
 	    gchar *note;
 
 	    if (datafile) {
-		note = g_strdup_printf("Please find the gretl data file %s attached.",
+		note = g_strdup_printf(_("Please find the gretl data file %s attached."),
 				       short_fname);
 	    } else {
-		note = g_strdup_printf("Please find the gretl script %s attached.",
+		note = g_strdup_printf(_("Please find the gretl script %s attached."),
 				       short_fname);
 	    }		
 	    gtk_entry_set_text(GTK_ENTRY(w), note);
@@ -657,7 +619,7 @@ mail_to_dialog (const char *fname, struct mail_info *minfo, struct msg_info *msg
     if (md.minfo->sig != NULL) {
 	GtkWidget *w;
 
-	w = gtk_check_button_new_with_label("Append signature");
+	w = gtk_check_button_new_with_label(_("Append signature"));
 	g_signal_connect(G_OBJECT(w), "toggled", G_CALLBACK(sig_callback), minfo);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
 	gtk_table_attach_defaults(GTK_TABLE(tbl), w, 0, 2, 4, 5);
@@ -691,13 +653,10 @@ mail_to_dialog (const char *fname, struct mail_info *minfo, struct msg_info *msg
     gtk_misc_set_alignment(GTK_MISC(lbl), 1, 0.5);
     gtk_table_attach(GTK_TABLE(tbl), lbl, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
 
-#if GTK_MAJOR_VERSION < 2
-    md.port_entry = gtk_entry_new_with_max_length(5);
-#else
     md.port_entry = gtk_entry_new();
     gtk_entry_set_max_length(GTK_ENTRY(md.port_entry), 5);
     gtk_entry_set_width_chars(GTK_ENTRY(md.port_entry), 8);
-#endif
+
     gtk_table_attach_defaults(GTK_TABLE(tbl), md.port_entry, 1, 2, 1, 2);
     port_str = g_strdup_printf("%d", md.minfo->port);
     gtk_entry_set_text(GTK_ENTRY(md.port_entry), port_str);
@@ -705,23 +664,23 @@ mail_to_dialog (const char *fname, struct mail_info *minfo, struct msg_info *msg
     lbl = gtk_label_new("                     ");
     gtk_table_attach_defaults(GTK_TABLE(tbl), lbl, 2, 3, 1, 2);
 
+    hbox = GTK_DIALOG(md.dlg)->action_area;
+
+    /* Cancel button */
+    md.cancel = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+    GTK_WIDGET_SET_FLAGS(md.cancel, GTK_CAN_DEFAULT);
+    gtk_container_add(GTK_CONTAINER(hbox), md.cancel);
+    g_signal_connect(G_OBJECT(md.cancel), "clicked", 
+		     G_CALLBACK(finalize_mail_settings), &md);
+
     /* Create the "OK" button */
-    md.ok = standard_button(GTK_STOCK_OK);
+    md.ok = gtk_button_new_from_stock(GTK_STOCK_OK);
     GTK_WIDGET_SET_FLAGS(md.ok, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(md.dlg)->action_area), 
-		       md.ok, TRUE, TRUE, 0);
+    gtk_container_add(GTK_CONTAINER(hbox), md.ok);
     g_signal_connect(G_OBJECT(md.ok), "clicked", 
 		     G_CALLBACK(finalize_mail_settings), &md);
     gtk_widget_grab_default(md.ok);
 
-    /* And a Cancel button */
-    md.cancel = standard_button(GTK_STOCK_CANCEL);
-    GTK_WIDGET_SET_FLAGS(md.cancel, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(md.dlg)->action_area), 
-		       md.cancel, TRUE, TRUE, 0);
-    g_signal_connect(G_OBJECT(md.cancel), "clicked", 
-		     G_CALLBACK(finalize_mail_settings), &md);
-
     gtk_widget_set_size_request(md.dlg, 420, -1);
     gtk_widget_show_all(md.dlg);
 
@@ -742,7 +701,8 @@ static int pop_info_dialog (struct mail_info *minfo)
 	N_("Username:"),
 	N_("Password:")
     };
-    GtkWidget *tbl, *lbl, *vbox;
+    GtkWidget *tbl, *lbl;
+    GtkWidget *hbox, *vbox;
     struct pop_dialog pd;
     int i, err = 0;
 
@@ -803,23 +763,23 @@ static int pop_info_dialog (struct mail_info *minfo)
 	} 
     }
 
-    /* Create the "OK" button */
-    pd.ok = standard_button(GTK_STOCK_OK);
+    hbox = GTK_DIALOG(pd.dlg)->action_area;
+
+    /* Cancel button */
+    pd.cancel = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+    GTK_WIDGET_SET_FLAGS(pd.cancel, GTK_CAN_DEFAULT);
+    gtk_container_add(GTK_CONTAINER(hbox), pd.cancel);
+    g_signal_connect(G_OBJECT(pd.cancel), "clicked", 
+		     G_CALLBACK(finalize_pop_settings), &pd);
+
+    /* "OK" button */
+    pd.ok = gtk_button_new_from_stock(GTK_STOCK_OK);
     GTK_WIDGET_SET_FLAGS(pd.ok, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pd.dlg)->action_area), 
-		       pd.ok, TRUE, TRUE, 0);
+    gtk_container_add(GTK_CONTAINER(hbox), pd.ok);
     g_signal_connect(G_OBJECT(pd.ok), "clicked", 
 		     G_CALLBACK(finalize_pop_settings), &pd);
     gtk_widget_grab_default(pd.ok);
 
-    /* And a Cancel button */
-    pd.cancel = standard_button(GTK_STOCK_CANCEL);
-    GTK_WIDGET_SET_FLAGS(pd.cancel, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pd.dlg)->action_area), 
-		       pd.cancel, TRUE, TRUE, 0);
-    g_signal_connect(G_OBJECT(pd.cancel), "clicked", 
-		     G_CALLBACK(finalize_pop_settings), &pd);
-
     gtk_widget_set_size_request(pd.dlg, 360, -1);
     gtk_widget_show_all(pd.dlg);
 
@@ -829,49 +789,6 @@ static int pop_info_dialog (struct mail_info *minfo)
     return err;
 }
 
-#if GTK_MAJOR_VERSION < 2
-
-static void mail_infobox (const char *msg, int err) 
-{
-    GtkWidget *w, *label, *button, *vbox, *hbox;
-
-    w = gtk_window_new(GTK_WINDOW_DIALOG);
-
-    gtk_container_border_width(GTK_CONTAINER(w), 5);
-    gtk_window_position (GTK_WINDOW(w), GTK_WIN_POS_MOUSE);
-    if (err) {
-	gtk_window_set_title(GTK_WINDOW (w), _("gretl error"));
-    } else {
-	gtk_window_set_title(GTK_WINDOW (w), _("gretl info"));
-    } 
-
-    vbox = gtk_vbox_new(FALSE, 5);
-    gtk_container_add(GTK_CONTAINER(w), vbox);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    gtk_container_add(GTK_CONTAINER(vbox), hbox);
-
-    /* text of message */
-    label = gtk_label_new(msg);
-    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
-
-    /* button */
-    hbox = gtk_hbox_new(FALSE, 5);
-    gtk_container_add(GTK_CONTAINER(vbox), hbox);
-    
-    button = gtk_button_new_with_label(_("OK"));
-
-    gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 5);
-
-    gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
-			      GTK_SIGNAL_FUNC(gtk_widget_destroy), 
-			      (gpointer) w);
-
-    gtk_widget_show_all(w);
-}
-
-#else /* GTK versions switch */
-
 static void mail_infobox (const char *msg, int err)
 {
     GtkWidget *dialog;
@@ -885,8 +802,6 @@ static void mail_infobox (const char *msg, int err)
     gtk_widget_destroy(dialog);
 }
 
-#endif
-
 #define VERBOSE 1
 
 static int get_server_response (int fd, char *buf)
