@@ -20,6 +20,7 @@
 /* mp_ols.c - gretl least squares with multiple precision (GMP) */
 
 #include "libgretl.h"
+
 #include <float.h>
 #include <gmp.h>
 
@@ -27,7 +28,7 @@
 #include <mpfr.h>
 #endif
 
-/* #define MP_DEBUG 1  */
+#define MP_DEBUG 0
 
 #define DEFAULT_GRETL_MP_BITS 256   /* min. bits of precision for GMP */
 
@@ -36,7 +37,6 @@ static mpf_t MPF_ZERO;
 static mpf_t MPF_MINUS_ONE;
 static mpf_t MPF_TINY;
 
-/* struct to hold model results */
 typedef struct {
     int ID;                      /* ID number for model */
     int t1, t2, nobs;            /* starting observation, ending
@@ -50,7 +50,7 @@ typedef struct {
 				    else = 0 */
     mpf_t *coeff;                /* array of coefficient estimates */
     mpf_t *sderr;                /* array of estimated std. errors */
-    mpf_t *xpx;
+    mpf_t *xpx;                  /* X'X */
     mpf_t ess, tss;              /* Error and Total Sums of Squares */
     mpf_t sigma;                 /* Standard error of regression */
     mpf_t rsq, adjrsq;           /* Unadjusted and adjusted R^2 */     
@@ -118,23 +118,17 @@ static void free_mpZ (mpf_t **mpZ, int v, int n)
 /* reject the incoming data (a) if any values are missing, (b) if any
    vars are all zero */
 
-static int data_problems (const int *list, double **Z, DATAINFO *pdinfo,
-			  char *errbuf)
+static int data_problems (const int *list, const double **Z, 
+			  const DATAINFO *pdinfo, char *errbuf)
 {
     int i, t, allzero;
 
     for (i=1; i<=list[0]; i++) {
-	/* no need to check the constant */
 	if (list[i] == 0) {
 	    continue; 
 	}
 	allzero = 1;
 	for (t=pdinfo->t1; t<=pdinfo->t2; t++) { 
-	    if (na(Z[list[i]][t])) {
-		sprintf(errbuf, _("Missing observations for variable '%s'"), 
-			pdinfo->varname[list[i]]);	
-		return 1;
-	    }
 	    if (!floateq(Z[list[i]][t], 0.0)) {
 		allzero = 0;
 	    }
@@ -156,7 +150,7 @@ static void make_poly_series (MPMODEL *pmod, mpf_t **mpZ,
     int t, s = 0;
 
     for (t=pmod->t1; t<=pmod->t2; t++) {
-#ifdef MP_DEBUG
+#if MP_DEBUG
 	printf("generating mpZ[%d][%d] from mpZ[%d][%d],\n"
 	       "using power %lu taken from polylist[%d]\n", 
 	       mpi, s, ppos, s, pwr, pli);
@@ -169,7 +163,7 @@ static void make_poly_series (MPMODEL *pmod, mpf_t **mpZ,
     }
 }
 
-static void fill_mp_series (MPMODEL *pmod, double **Z, mpf_t **mpZ,
+static void fill_mp_series (MPMODEL *pmod, const double **Z, mpf_t **mpZ,
 			    unsigned char **digits,
 			    int i, int mpi)
 {
@@ -177,16 +171,13 @@ static void fill_mp_series (MPMODEL *pmod, double **Z, mpf_t **mpZ,
     int t, s = 0; 
 
     for (t=pmod->t1; t<=pmod->t2; t++) {
-	/* do trick with strings? */
 	if (digits != NULL && digits[i] != NULL) {
+	    /* do trick with strings */
 	    sprintf(numstr, "%.*g", digits[i][t], Z[i][t]);
-#ifdef MP_DEBUG
-	    printf("setting mpZ[%d][%d] from '%s'\n", mpi, s, numstr);
-#endif
-	    mpf_init_set_str (mpZ[mpi][s], numstr, 10);
+	    mpf_init_set_str(mpZ[mpi][s], numstr, 10);
 	} else { 
-	    /* or do straight conversion */
-	    mpf_init_set_d (mpZ[mpi][s], Z[i][t]);
+	    /* do straight conversion */
+	    mpf_init_set_d(mpZ[mpi][s], Z[i][t]);
 	}
 	s++;
     }
@@ -212,14 +203,15 @@ static void fill_mp_series (MPMODEL *pmod, double **Z, mpf_t **mpZ,
    powers of x is done in regular double precision, so we do it here.
 */
 
-static mpf_t **make_mpZ (MPMODEL *pmod, double **Z, DATAINFO *pdinfo,
-			 char **varnames)
+static mpf_t **make_mpZ (MPMODEL *mpmod, const double **Z, 
+			 const DATAINFO *pdinfo, 
+			 char **xnames)
 {
     int i, s, t;
-    int n = pmod->t2 - pmod->t1 + 1;
-    int l0 = pmod->list[0];
+    int n = mpmod->t2 - mpmod->t1 + 1;
+    int l0 = mpmod->list[0];
     int npoly, mp_poly_pos = 0;
-    int listpt, nvars = 0, vnames = 1;
+    int listpt, nvars = 0, v = 0;
     mpf_t **mpZ = NULL;
     unsigned char **digits = (unsigned char **) pdinfo->data;
     int err = 0;
@@ -228,12 +220,12 @@ static mpf_t **make_mpZ (MPMODEL *pmod, double **Z, DATAINFO *pdinfo,
 	return NULL;
     }
 
-    /* "varlist" holds the regression specification using
+    /* "varlist" holds the regression specification, using
        the numbering of variables from the original dataset 
     */
 
-    pmod->varlist = gretl_list_new(l0);
-    if (pmod->varlist == NULL) {
+    mpmod->varlist = gretl_list_new(l0);
+    if (mpmod->varlist == NULL) {
 	return NULL;
     }
 
@@ -242,14 +234,14 @@ static mpf_t **make_mpZ (MPMODEL *pmod, double **Z, DATAINFO *pdinfo,
 	return NULL;
     }
 
-    if (pmod->ifc) { 
+    if (mpmod->ifc) { 
 	mpZ[0] = malloc(n * sizeof **mpZ);
 	s = 0;
-	for (t=pmod->t1; t<=pmod->t2; t++) {
+	for (t=mpmod->t1; t<=mpmod->t2; t++) {
 	    mpf_init_set_d(mpZ[0][s++], 1.0);
 	}
-	if (varnames != NULL) {
-	    strcpy(varnames[vnames++], pdinfo->varname[0]);
+	if (xnames != NULL) { 	 
+	    strcpy(xnames[v++], pdinfo->varname[0]); 	 
 	}
 	nvars++;
     } else {
@@ -257,17 +249,17 @@ static mpf_t **make_mpZ (MPMODEL *pmod, double **Z, DATAINFO *pdinfo,
     }
 
     /* number of polynomial terms to be generated */
-    if (pmod->polylist != NULL) {
-	npoly = pmod->polylist[0];
+    if (mpmod->polylist != NULL) {
+	npoly = mpmod->polylist[0];
     } else {
 	npoly = 0;
     }
 
     /* process the ordinary data */
     for (i=1; i<=l0-npoly; i++) {
-	if (pmod->list[i] == 0) {
+	if (mpmod->list[i] == 0) {
 	    /* the constant is already handled */	    
-	    pmod->varlist[i] = 0;
+	    mpmod->varlist[i] = 0;
 	    continue;
 	}
 	mpZ[nvars] = malloc(n * sizeof **mpZ);
@@ -275,10 +267,10 @@ static mpf_t **make_mpZ (MPMODEL *pmod, double **Z, DATAINFO *pdinfo,
 	    err = 1;
 	    break;
 	}
-	if (pmod->list[i] == pmod->polyvar) {
+	if (mpmod->list[i] == mpmod->polyvar) {
 	    /* record position in mpZ of the var to be raised 
 	       to various powers, if applicable */
-#ifdef MP_DEBUG
+#if MP_DEBUG
 	    fprintf(stderr, "var to be raised to powers: it's "
 		   "at position %d in the regression list,\n"
 		   "and at slot %d in mpZ\n", i, nvars);
@@ -286,22 +278,12 @@ static mpf_t **make_mpZ (MPMODEL *pmod, double **Z, DATAINFO *pdinfo,
 	    mp_poly_pos = nvars;
 	}
 	    
-	fill_mp_series(pmod, Z, mpZ, digits, pmod->list[i], nvars); 
-	pmod->varlist[i] = pmod->list[i];
-
-	if (varnames != NULL) {
-	    int targ = 0;
-
-	    if (i > 1) {
-		targ = vnames++;
-	    }
-#ifdef MP_DEBUG
-	    fprintf(stderr, "writing varnames[%d] = '%s'\n",
-		    targ, pdinfo->varname[pmod->list[i]]);
-#endif
-	    strcpy(varnames[targ], pdinfo->varname[pmod->list[i]]);
+	fill_mp_series(mpmod, Z, mpZ, digits, mpmod->list[i], nvars); 
+	mpmod->varlist[i] = mpmod->list[i];
+        if (xnames != NULL && i > 1) { 	 
+	    strcpy(xnames[v++], pdinfo->varname[mpmod->list[i]]); 	 
 	}
-	pmod->list[i] = nvars;
+	mpmod->list[i] = nvars;
 	nvars++;
     } /* end processing ordinary data */
 
@@ -315,20 +297,17 @@ static mpf_t **make_mpZ (MPMODEL *pmod, double **Z, DATAINFO *pdinfo,
 	    err = 1;
 	    break;
 	}
-	make_poly_series(pmod, mpZ, i+1, mp_poly_pos, nvars);
-	pmod->varlist[i+listpt] = pmod->polyvar;
 
-	if (varnames != NULL) {
-	    sprintf(varnames[vnames++], "%s^%d", 
-		    pdinfo->varname[pmod->polyvar],
-		    pmod->polylist[i+1]);
-#ifdef MP_DEBUG
-	    fprintf(stderr, "wrote varname[%d] = '%s'\n", 
-		    vnames-1, varnames[vnames-1]);
-#endif
+	make_poly_series(mpmod, mpZ, i+1, mp_poly_pos, nvars);
+	mpmod->varlist[i+listpt] = mpmod->polyvar;
+
+        if (xnames != NULL) { 	 
+	    sprintf(xnames[v++], "%s^%d", 	 
+		    pdinfo->varname[mpmod->polyvar], 	 
+		    mpmod->polylist[i+1]); 	 
 	}
 
-	pmod->list[i+listpt] = nvars;
+	mpmod->list[i+listpt] = nvars;
 	nvars++;
     }
 
@@ -340,66 +319,66 @@ static mpf_t **make_mpZ (MPMODEL *pmod, double **Z, DATAINFO *pdinfo,
     return mpZ;
 }
 
-static void mp_model_free (MPMODEL *pmod)
+static void mp_model_free (MPMODEL *mpmod)
 {
-    int i, nx = pmod->list[0] - 1;
+    int i, nx = mpmod->list[0] - 1;
     int nt = nx * (nx + 1) / 2;
 
-    free(pmod->list);
-    free(pmod->varlist);
+    free(mpmod->list);
+    free(mpmod->varlist);
 
-    if (pmod->coeff != NULL) {
-	for (i=0; i<pmod->ncoeff; i++) { 
-	    mpf_clear(pmod->coeff[i]);
+    if (mpmod->coeff != NULL) {
+	for (i=0; i<mpmod->ncoeff; i++) { 
+	    mpf_clear(mpmod->coeff[i]);
 	}
-	free(pmod->coeff);
+	free(mpmod->coeff);
     }
 
-    if (pmod->sderr != NULL) {    
-	for (i=0; i<pmod->ncoeff; i++) {
-	    mpf_clear(pmod->sderr[i]);
+    if (mpmod->sderr != NULL) {    
+	for (i=0; i<mpmod->ncoeff; i++) {
+	    mpf_clear(mpmod->sderr[i]);
 	}
-	free(pmod->sderr);
+	free(mpmod->sderr);
     }
 
-    if (pmod->xpx != NULL) {
+    if (mpmod->xpx != NULL) {
 	for (i=0; i<nt; i++) {
-	    mpf_clear(pmod->xpx[i]);
+	    mpf_clear(mpmod->xpx[i]);
 	}
-	free(pmod->xpx);
+	free(mpmod->xpx);
     }
 
-    mpf_clear(pmod->ess);
-    mpf_clear(pmod->tss);
-    mpf_clear(pmod->sigma);
-    mpf_clear(pmod->rsq);
-    mpf_clear(pmod->adjrsq);
-    mpf_clear(pmod->fstt); 
+    mpf_clear(mpmod->ess);
+    mpf_clear(mpmod->tss);
+    mpf_clear(mpmod->sigma);
+    mpf_clear(mpmod->rsq);
+    mpf_clear(mpmod->adjrsq);
+    mpf_clear(mpmod->fstt); 
 }
 
-static void mp_model_init (MPMODEL *pmod, DATAINFO *pdinfo)
+static void mp_model_init (MPMODEL *mpmod, const DATAINFO *pdinfo)
 {
-    pmod->ID = 0;
-    pmod->t1 = pdinfo->t1;
-    pmod->t2 = pdinfo->t2;
+    mpmod->ID = 0;
+    mpmod->t1 = pdinfo->t1;
+    mpmod->t2 = pdinfo->t2;
 
-    pmod->list = NULL;
-    pmod->varlist = NULL;
-    pmod->polylist = NULL; /* don't free, the caller does that */
-    pmod->ifc = 1;
-    pmod->coeff = NULL;
-    pmod->sderr = NULL;
-    pmod->xpx = NULL;
+    mpmod->list = NULL;
+    mpmod->varlist = NULL;
+    mpmod->polylist = NULL; /* don't free, the caller does that */
+    mpmod->ifc = 1;
+    mpmod->coeff = NULL;
+    mpmod->sderr = NULL;
+    mpmod->xpx = NULL;
 
-    mpf_init(pmod->ess);
-    mpf_init(pmod->tss);
-    mpf_init(pmod->sigma);
-    mpf_init(pmod->rsq);
-    mpf_init(pmod->adjrsq);
-    mpf_init(pmod->fstt);
+    mpf_init(mpmod->ess);
+    mpf_init(mpmod->tss);
+    mpf_init(mpmod->sigma);
+    mpf_init(mpmod->rsq);
+    mpf_init(mpmod->adjrsq);
+    mpf_init(mpmod->fstt);
 
-    pmod->errcode = 0;
-    pmod->polyvar = 0;
+    mpmod->errcode = 0;
+    mpmod->polyvar = 0;
 }
 
 /**
@@ -441,6 +420,7 @@ int mp_vector_raise_to_power (const double *srcvec, double *targvec,
 }
 
 #ifdef HAVE_MPFR
+
 /**
  * mp_vector_ln:
  * @srcvec: source vector (doubles)
@@ -475,16 +455,17 @@ int mp_vector_ln (const double *srcvec, double *targvec, int n)
 
     return 0;
 }
+
 #endif
 
-static int poly_check (MPMODEL *pmod, const int *list)
+static int poly_check (MPMODEL *mpmod, const int *list)
 {
     int i;
 
     /* check that all powers are > 1 */
 
-    for (i=1; i<=pmod->polylist[0]; i++) {
-	if (pmod->polylist[i] < 2) {
+    for (i=1; i<=mpmod->polylist[0]; i++) {
+	if (mpmod->polylist[i] < 2) {
 	    return 1;
 	}
     }
@@ -494,12 +475,12 @@ static int poly_check (MPMODEL *pmod, const int *list)
 
     for (i=list[0]; i>1; i--) {
 	if (list[i] != 0) {
-	    pmod->polyvar = list[i];
+	    mpmod->polyvar = list[i];
 	    break;
 	}
     }
 
-    if (pmod->polyvar == 0) {
+    if (mpmod->polyvar == 0) {
 	return 1;
     }
 
@@ -539,6 +520,7 @@ static void set_gretl_mp_bits (void)
 }
 
 #ifdef HAVE_MPFR
+
 static void set_gretl_mpfr_bits (void)
 {
     char *user_bits = getenv("GRETL_MP_BITS");
@@ -549,152 +531,561 @@ static void set_gretl_mpfr_bits (void)
 	mpfr_set_default_prec((unsigned long) DEFAULT_GRETL_MP_BITS);
     }
 } 
-#endif
 
-static int copy_mp_results (MPMODEL *pmod, DATAINFO *pdinfo,
-			    mp_results *results)
+/* compute log-likelihood etc. in multiple precision, using the
+   MPFR library
+*/
+
+static void mp_ll_stats (const MPMODEL *mpmod, MODEL *pmod)
 {
-    int i, err = 0;
+    double n = mpmod->nobs;
+    double k = mpmod->ncoeff;
+    mpfr_t mll, mln, ll2;
+    mpfr_t mx1, mx2;
+    mpfr_t ln2pi1, crit;
 
-    for (i=0; i<pmod->ncoeff; i++) {
-	results->coeff[i] = mpf_get_d(pmod->coeff[i]);
-	results->sderr[i] = mpf_get_d(pmod->sderr[i]);
+    set_gretl_mpfr_bits();
+
+    mpfr_init(mll);
+    mpfr_init(mln);
+    mpfr_init(ll2);
+    mpfr_init(mx1);
+    mpfr_init(mx2);
+    mpfr_init(ln2pi1);
+    mpfr_init(crit);
+
+    mpfr_const_pi(ln2pi1, GMP_RNDN);          /* pi */
+    mpfr_set_d(mx1, 2.0, GMP_RNDN);           /* 2 */
+    mpfr_mul(ln2pi1, ln2pi1, mx1, GMP_RNDN);  /* 2 * pi */
+    mpfr_log(ln2pi1, ln2pi1, GMP_RNDN);       /* log(2*pi) */
+    mpfr_set_d(mx1, 1.0, GMP_RNDN);           /* 1 */
+    mpfr_add(ln2pi1, ln2pi1, mx1, GMP_RNDN);  /* log(2*pi) + 1 */
+
+    mpfr_set_f(mll, mpmod->ess, GMP_RNDN);
+    mpfr_set_d(mx1, -.5, GMP_RNDN);
+    mpfr_set_d(mx2, n, GMP_RNDN);
+
+    mpfr_log(mll, mll, GMP_RNDN);          /* log(ess) */
+    mpfr_log(mln, mx2, GMP_RNDN);          /* log(n) */
+
+    mpfr_mul(mll, mx2, mll, GMP_RNDN);     /* n * log(ess) */
+    mpfr_mul(mll, mx1, mll, GMP_RNDN);     /* -.5 * n * log(ess) */
+
+    mpfr_mul(mx1, mx1, mx2, GMP_RNDN);     /* -.5 * n */
+    mpfr_sub(mx2, ln2pi1, mln, GMP_RNDN);  /* log(2*pi) - log(n) */
+    mpfr_mul(mx2, mx1, mx2, GMP_RNDN);     /* -.5 * n * (log(2*pi) - log(n)) */
+
+    mpfr_add(mll, mll, mx2, GMP_RNDN);     /* now actual log-likelihood */  
+
+    pmod->lnL = mpfr_get_d(mll, GMP_RNDN);
+
+    mpfr_set_d(mx1, -2.0, GMP_RNDN);
+    mpfr_mul(ll2, mx1, mll, GMP_RNDN);    /* -2.0 * ll */
+
+    mpfr_set_d(mx1, 2.0, GMP_RNDN);
+    mpfr_set_d(mx2, k, GMP_RNDN);
+    mpfr_mul(mx1, mx1, mx2, GMP_RNDN);    /* 2 * k */
+
+    mpfr_add(crit, ll2, mx1, GMP_RNDN);   /* -2.0 * ll + 2 * k */
+    pmod->criterion[C_AIC] = mpfr_get_d(crit, GMP_RNDN);
+
+    mpfr_mul(mx1, mx2, mln, GMP_RNDN);    /* k * log(n) */
+    mpfr_add(crit, ll2, mx1, GMP_RNDN);   /* -2.0 * ll + k * log(n) */
+    pmod->criterion[C_BIC] = mpfr_get_d(crit, GMP_RNDN);
+
+    mpfr_set_d(mx1, 2.0, GMP_RNDN);
+    mpfr_set_d(mx2, (double) k, GMP_RNDN);
+    mpfr_mul(mx1, mx1, mx2, GMP_RNDN);    /* 2 * k, again */
+    mpfr_log(mx2, mln, GMP_RNDN);         /* log(log(n)) */
+    mpfr_mul(mx1, mx1, mx2, GMP_RNDN);    /* 2 * k * log(log(n) */
+    mpfr_add(crit, ll2, mx1, GMP_RNDN);   /* -2.0 * ll + 2 * k * log(log(n)) */
+    pmod->criterion[C_HQC] = mpfr_get_d(crit, GMP_RNDN);
+
+    mpfr_clear(mll);
+    mpfr_clear(mln);
+    mpfr_clear(ll2);
+    mpfr_clear(mx1);
+    mpfr_clear(mx2);
+    mpfr_clear(ln2pi1);
+    mpfr_clear(crit);
+}
+
+#else
+
+#include <errno.h>
+
+/* compute log-likelihood etc., based on the ESS from the
+   multiple-precision model but using ordinary double-precision
+   arithmetic: a fallback if the MPFR library is not available
+*/
+
+static void mp_ll_stats (const MPMODEL *mpmod, MODEL *pmod)
+{
+    double ess = mpf_get_d(mpmod->ess);
+    int k = mpmod->ncoeff;
+    int n = mpmod->nobs;
+    double ll;
+    double c[3];
+    int err = 0;
+
+    fprintf(stderr, "mp_ll_stats: non-MPFR version\n");
+
+    if (na(ess) || ess <= 0.0) {
+	err = 1;
+    } else {
+	const double ln2pi1 = 2.837877066409345;
+
+	errno = 0;
+
+	ll = -.5 * n * log(ess);
+
+	if (errno == EDOM || errno == ERANGE) {
+	    err = 1;
+	} else {
+	    ll += -.5 * n * (ln2pi1 - log((double) n));
+	    c[0] = -2.0 * ll + 2 * k;
+	    c[1] = -2.0 * ll + k * log(n);
+	    c[2] = -2.0 * ll + 2 * k * log(log(n));
+	}
     }
 
-    results->sigma = mpf_get_d(pmod->sigma);
-    results->ess = mpf_get_d(pmod->ess);
-    results->rsq = mpf_get_d(pmod->rsq);
-    results->fstt = mpf_get_d(pmod->fstt);
+    if (!err) {
+	pmod->lnL = ll;
+	pmod->criterion[C_AIC] = c[0];
+	pmod->criterion[C_BIC] = c[1];
+	pmod->criterion[C_HQC] = c[2];
+    }	
+}
 
-    if (results->varnames != NULL) { 
-	/* will use results for printing */
-	results->t1 = pmod->t1;
-	results->t2 = pmod->t2;
-	results->ifc = pmod->ifc;
-	results->dfn = pmod->dfn;
-	results->dfd = pmod->dfd;
-	results->adjrsq = mpf_get_d(pmod->adjrsq);
-	results->varlist = gretl_list_copy(pmod->varlist);
-	if (results->varlist == NULL) {
-	    err = 1;
+#endif
+
+static void mp_dwstat (const MPMODEL *mpmod, MODEL *pmod,
+		       mpf_t *uhat)
+{
+    mpf_t num, x;
+    mpf_t ut1, u11;
+    int t;
+
+    mpf_init(num);
+    mpf_init(x);
+    mpf_init(ut1);
+    mpf_init(u11);
+
+    for (t=1; t<mpmod->nobs; t++)  {
+	mpf_sub(x, uhat[t], uhat[t-1]);
+	mpf_pow_ui(x, x, 2);
+	mpf_add(num, num, x);
+	mpf_mul(x, uhat[t], uhat[t-1]);
+	mpf_add(ut1, ut1, x);
+	mpf_mul(x, uhat[t-1], uhat[t-1]);
+	mpf_add(u11, u11, x);
+    }
+
+    mpf_div(x, num, mpmod->ess);
+    pmod->dw = mpf_get_d(x);
+
+    if (isnan(pmod->dw) || isinf(pmod->dw)) {
+	pmod->dw = NADBL;
+    }    
+
+    mpf_div(x, ut1, u11);
+    pmod->rho = mpf_get_d(x);
+
+    if (isnan(pmod->rho) || isinf(pmod->rho)) {
+	pmod->dw = NADBL;
+	pmod->rho = NADBL;
+    }
+
+    mpf_clear(num);
+    mpf_clear(x);
+    mpf_clear(ut1);
+    mpf_clear(u11);
+}
+
+/* compute coefficient covariance matrix in multiple precision */
+
+static int mp_makevcv (const MPMODEL *mpmod, MODEL *pmod)
+{
+    mpf_t *vcv;
+    int dec, mst, kk, i, j, kj, icnt, m, k, l = 0;
+    const int nv = mpmod->ncoeff;
+    const int nxpx = (nv * nv + nv) / 2; 
+    mpf_t d, x;
+
+    if (mpmod->xpx == NULL) {
+	return 1;
+    }
+
+    mpf_init(d);
+    mpf_init(x);
+
+    mst = nxpx;
+    kk = nxpx - 1;
+
+    vcv = malloc(nxpx * sizeof *vcv);
+    if (vcv == NULL) {
+	return E_ALLOC;
+    }
+
+    pmod->vcv = malloc(nxpx * sizeof *pmod->vcv);
+    if (pmod->vcv == NULL) {
+	free(vcv);
+	return E_ALLOC;
+    }
+
+    for (i=0; i<nxpx; i++) {
+	mpf_init(vcv[i]);
+    }
+
+    for (i=0; i<nv; i++) {
+	mst -= i;
+	/* find diagonal element */
+	mpf_set(d, mpmod->xpx[kk]);
+	if (i > 0) {
+	    for (j=kk+1; j<=kk+i; j++) {
+		mpf_mul(x, mpmod->xpx[j], vcv[j]);
+		mpf_sub(d, d, x);
+	    }
+	}
+	mpf_mul(vcv[kk], d, mpmod->xpx[kk]);
+	/* find off-diagonal elements indexed by kj */
+	kj = kk;
+	kk = kk - i - 2;
+	if (i > nv - 2) {
+	    continue;
+	}
+	for (j=i+1; j<nv; j++) {
+	    icnt = i+1;
+	    kj -= j;
+	    mpf_set(d, MPF_ZERO);
+	    m = mst + 1;
+	    for (k=0; k<=j-1; k++) {
+		if (icnt > 0) {
+		    dec = 1;
+		    icnt--;
+		} else {
+		    dec = k;
+		}
+		m -= dec;
+		l = kj + i - k;
+		mpf_mul(x, vcv[m-1], mpmod->xpx[l]);
+		mpf_add(d, d, x);
+	    }
+	    mpf_mul(x, d, mpmod->xpx[l-1]);
+	    mpf_neg(vcv[kj], x);
+	}
+    }
+
+    for (i=0; i<nxpx; i++) {
+	mpf_mul(x, vcv[i], mpmod->sigma);
+	mpf_mul(x, x, mpmod->sigma);
+	pmod->vcv[i] = mpf_get_d(x);
+	mpf_clear(vcv[i]);
+    }
+
+    mpf_clear(d);
+    mpf_clear(x);
+    free(vcv);
+
+    return 0;
+}
+
+/* compute mean and s.d. of dependent variable in multiple precision */
+
+static void mp_depvarstats (const MPMODEL *mpmod, MODEL *pmod,
+			    mpf_t **mpZ)
+{
+    mpf_t xbar, ssx, diff, mn;
+    double xn = mpmod->nobs;
+    int yno = mpmod->list[1];
+    int t;
+    
+    mpf_init(xbar);
+    mpf_init(ssx);
+    mpf_init(diff);
+    mpf_init(mn);
+
+    mpf_set(xbar, MPF_ZERO);
+    mpf_set(ssx, MPF_ZERO);
+    mpf_set_d(mn, xn);
+
+    for (t=0; t<mpmod->nobs; t++) {
+	mpf_add(xbar, xbar, mpZ[yno][t]);
+    }
+
+    mpf_div(xbar, xbar, mn);
+    pmod->ybar = mpf_get_d(xbar);
+
+    for (t=0; t<mpmod->nobs; t++) {
+	mpf_sub(diff, mpZ[yno][t], xbar);
+	mpf_pow_ui(diff, diff, 2);
+	mpf_add(ssx, ssx, diff);
+    }
+
+    mpf_sub(mn, mn, MPF_ONE);
+    mpf_div(ssx, ssx, mn);
+    mpf_sqrt(ssx, ssx);     
+    pmod->sdy = mpf_get_d(ssx);
+
+    mpf_clear(xbar);
+    mpf_clear(ssx);
+    mpf_clear(diff);
+    mpf_clear(mn);
+}
+
+/* compute residuals and fitted values in multiple precision */
+
+static void mp_hatvars (const MPMODEL *mpmod, MODEL *pmod,
+			mpf_t **mpZ, int tseries)
+{
+    mpf_t *uhat = NULL;
+    mpf_t yht, uht, xbi;
+    int i, t;
+
+    if (tseries) {
+	uhat = malloc(mpmod->nobs * sizeof *uhat);
+	if (uhat != NULL) {
+	    for (t=0; t<mpmod->nobs; t++) {
+		mpf_init(uhat[t]);
+	    }
+	}
+    }
+    
+    mpf_init(yht);
+    mpf_init(uht);
+    mpf_init(xbi);
+
+    for (t=0; t<mpmod->nobs; t++) {
+	mpf_set_d(yht, 0.0);
+	for (i=0; i<mpmod->ncoeff; i++) {
+	    mpf_mul(xbi, mpmod->coeff[i], mpZ[mpmod->list[i+2]][t]);
+	    mpf_add(yht, yht, xbi);
+	}
+	mpf_sub(uht, mpZ[mpmod->list[1]][t], yht);
+	pmod->yhat[t + mpmod->t1] = mpf_get_d(yht);
+	pmod->uhat[t + mpmod->t1] = mpf_get_d(uht);
+	if (uhat != NULL) {
+	    mpf_set(uhat[t], uht);
+	}
+    }
+
+    mpf_clear(yht);
+    mpf_clear(uht);
+    mpf_clear(xbi);
+
+    if (uhat != NULL) {
+	mp_dwstat(mpmod, pmod, uhat);
+	for (t=0; t<mpmod->nobs; t++) {
+	    mpf_clear(uhat[t]);
+	}
+	free(uhat);
+    }
+}
+
+static int copy_mp_results (const MPMODEL *mpmod, MODEL *pmod,
+			    const DATAINFO *pdinfo, mpf_t **mpZ,
+			    char **xnames, gretlopt opt)
+{
+    int tseries = dataset_is_time_series(pdinfo);
+    int i, err = 0;
+
+    pmod->ncoeff = mpmod->ncoeff;
+    pmod->full_n = pdinfo->n;
+    pmod->ci = MPOLS;
+
+    err = gretl_model_allocate_storage(pmod);
+    if (err) {
+	if (xnames != NULL) { 	 
+	    free_strings_array(xnames, pmod->ncoeff); 	 
+	} 	 
+	return err;
+    }
+
+    if (xnames != NULL) { 	 
+	gretl_model_add_allocated_varnames(pmod, xnames); 	 
+    } 	 
+
+    for (i=0; i<mpmod->ncoeff; i++) {
+	pmod->coeff[i] = mpf_get_d(mpmod->coeff[i]);
+	pmod->sderr[i] = mpf_get_d(mpmod->sderr[i]);
+    }
+
+    pmod->sigma = mpf_get_d(mpmod->sigma);
+    pmod->ess = mpf_get_d(mpmod->ess);
+    pmod->rsq = mpf_get_d(mpmod->rsq);
+    pmod->fstt = mpf_get_d(mpmod->fstt);
+
+    if (opt & OPT_S) {
+	/* saving additional results */
+	pmod->t1 = mpmod->t1;
+	pmod->t2 = mpmod->t2;
+	pmod->nobs = mpmod->nobs;
+	pmod->ifc = mpmod->ifc;
+	pmod->dfn = mpmod->dfn;
+	pmod->dfd = mpmod->dfd;
+	pmod->adjrsq = mpf_get_d(mpmod->adjrsq);
+	pmod->list = gretl_list_copy(mpmod->varlist);
+	if (pmod->list == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    mp_depvarstats(mpmod, pmod, mpZ);
+	    mp_hatvars(mpmod, pmod, mpZ, tseries);
+	    mp_ll_stats(mpmod, pmod);
+	    mp_makevcv(mpmod, pmod);
 	}
     }
 
     return err;
 }
 
+static char **allocate_xnames (const int *list) 	 
+{ 	 
+    int n = list[0] - 1; 	 
+    char **s = strings_array_new_with_length(n, VNAMELEN + 6);
+
+    return s;
+}
+
 /**
  * mplsq:
  * @list: dependent variable plus list of regressors.
  * @polylist: list of polynomial terms (or NULL).
- * @pZ: pointer to data matrix.
+ * @Z: data array.
  * @pdinfo: information on the data set.
- * @prn: where to print results.
  * @errbuf: where to print any error message.
- * @results: pointer to struct to hold results (or NULL).
+ * @pmod: MODEL pointer to hold results.
+ * @opt: if contains %OPT_S, save additional model
+ * information (including the names of parameters in 
+ * @pmod, if required).
  *
  * Computes multiple-precision OLS estimates of the model 
- * specified by @list.  If @results is NULL, prints the estimates
- * to @prn, otherwise puts them into @results.
+ * specified by @list, and stores them in @pmod.
  * 
  * Returns: 0 on success, error code on failure.
  */
 
 int mplsq (const int *list, const int *polylist,
-	   double ***pZ, DATAINFO *pdinfo, PRN *prn,
-	   char *errbuf, mp_results *results) 
+	   const double **Z, DATAINFO *pdinfo, 
+	   char *errbuf, MODEL *pmod, gretlopt opt) 
 {
-    int l0, i, ret = 0;
+    int l0, i;
     mpf_t **mpZ = NULL;
+    char **xnames = NULL;
     MPXPXXPY xpxxpy;
-    MPMODEL model;
+    MPMODEL mpmod;
+    int orig_t1 = pdinfo->t1;
+    int orig_t2 = pdinfo->t2;
+    int err = 0;
 
     *errbuf = 0;
 
-    if (list == NULL || pZ == NULL || *pZ == NULL || pdinfo == NULL ||
+    if (list == NULL || Z == NULL || pdinfo == NULL ||
 	list[0] == 1 || pdinfo->v == 1) {
 	return E_DATA;
     }
 
     set_gretl_mp_bits();
 
-    mp_model_init (&model, pdinfo);
+    mp_model_init(&mpmod, pdinfo);
 
-    /* preserve a copy of the list supplied, for future reference */
     if (polylist == NULL) {
-	model.list = gretl_list_copy(list);
+	mpmod.list = gretl_list_copy(list);
     } else {
-	model.list = poly_copy_list(list, polylist);
+	mpmod.list = poly_copy_list(list, polylist);
     }
 
-    if (model.list == NULL) {
+    if (mpmod.list == NULL) {
 	return E_ALLOC;
     }
 
-    model.polylist = polylist; /* attached for convenience */
+    mpmod.polylist = polylist; /* attached for convenience */
 
-    if (polylist != NULL && poly_check(&model, list)) {
-	mp_model_free(&model);
-	return E_DATA;
+    if (polylist != NULL && poly_check(&mpmod, list)) {
+	err = E_DATA;
+	goto bailout;
     }
 
     /* check for missing obs in sample */
-    if (data_problems(list, *pZ, pdinfo, errbuf)) {
-	mp_model_free(&model);
-	return E_DATA;
+    err = list_adjust_t1t2(list, Z, pdinfo);
+    if (err) {
+	goto bailout;
+    }
+    
+    /* in case of any changes */
+    mpmod.t1 = pdinfo->t1;
+    mpmod.t2 = pdinfo->t2;
+
+    /* check for other data issues */
+    if (data_problems(list, Z, pdinfo, errbuf)) {
+	err = E_DATA;
+	goto bailout;
+    }
+
+    /* enable names for polynomial terms? */
+    if (polylist != NULL && (opt & OPT_S)) { 	 
+	xnames = allocate_xnames(mpmod.list); 	 
+	if (xnames == NULL) { 
+	    err = E_ALLOC;
+	    goto bailout;
+	} 	 
     }
 
     /* see if the regressor list contains a constant */
-    model.ifc = mp_rearrange(model.list);
+    mpmod.ifc = mp_rearrange(mpmod.list);
 
     /* construct multiple-precision data matrix */
-    mpZ = make_mpZ(&model, *pZ, pdinfo, results->varnames);
+    mpZ = make_mpZ(&mpmod, Z, pdinfo, xnames);
 
     if (mpZ == NULL) {
-	mp_model_free(&model);
-	return E_ALLOC;
+	err = E_ALLOC;
+	goto bailout;
     }
 
     mpf_constants_init();
 
-    l0 = model.list[0];  
-    model.ncoeff = l0 - 1; 
-    model.nobs = model.t2 - model.t1 + 1;
+    l0 = mpmod.list[0];  
+    mpmod.ncoeff = l0 - 1; 
+    mpmod.nobs = mpmod.t2 - mpmod.t1 + 1;
 
     /* check degrees of freedom */
-    if (model.nobs < model.ncoeff) { 
+    if (mpmod.nobs < mpmod.ncoeff) { 
         sprintf(errbuf, _("No. of obs (%d) is less than no. "
-			  "of parameters (%d)"), model.nobs, model.ncoeff);
-	mp_model_free(&model);
-	free_mpZ(mpZ, l0, model.nobs);
+			  "of parameters (%d)"), mpmod.nobs, mpmod.ncoeff);
+	free_mpZ(mpZ, l0, mpmod.nobs);
 	mpf_constants_clear();
-        return E_DF; 
+	err = E_DF;
+	goto bailout;
     }
 
     /* calculate regression results */
-    xpxxpy = mp_xpxxpy_func(model.list, model.nobs, mpZ);
-    mpf_set(model.tss, xpxxpy.xpy[l0]);
 
-    mp_regress(&model, xpxxpy, mpZ, model.nobs, errbuf);
+    xpxxpy = mp_xpxxpy_func(mpmod.list, mpmod.nobs, mpZ);
+    mpf_set(mpmod.tss, xpxxpy.xpy[l0]);
+
+    mp_regress(&mpmod, xpxxpy, mpZ, mpmod.nobs, errbuf);
 
     for (i=0; i<=l0; i++) {
 	mpf_clear(xpxxpy.xpy[i]);
     }
     free(xpxxpy.xpy);
-    xpxxpy.xpy = NULL;
 
-    ret = model.errcode;
-
-    if (ret == 0) {
-	copy_mp_results(&model, pdinfo, results);
-    }
+    err = mpmod.errcode;
+    if (!err) {
+	err = copy_mp_results(&mpmod, pmod, pdinfo, mpZ, xnames, opt);
+    } 
 
     /* free all the mpf stuff */
-    free_mpZ(mpZ, l0, model.nobs);
-    mp_model_free(&model);
+    free_mpZ(mpZ, l0, mpmod.nobs);
     mpf_constants_clear();
 
-    return ret;
+ bailout:
+
+    pdinfo->t1 = orig_t1;
+    pdinfo->t2 = orig_t2;
+
+    mp_model_free(&mpmod);
+
+    return err;
 }
 
 static void mp_xpxxpy_init (MPXPXXPY *m)
@@ -770,7 +1161,7 @@ static MPXPXXPY mp_xpxxpy_func (const int *list, int n, mpf_t **mpZ)
             }
             mpf_set(xpxxpy.xpx[m++], xx);
         }
-        mpf_set (xx, MPF_ZERO);
+        mpf_set(xx, MPF_ZERO);
         for (t=0; t<n; t++) {
 	    mpf_mul(tmp, mpZ[yno][t], mpZ[li][t]);
 	    mpf_add(xx, xx, tmp);
@@ -1037,7 +1428,7 @@ static MPCHOLBETA mp_cholbeta (MPXPXXPY xpxxpy)
         /* off-diagonal elements */
         for (i=j+1; i<=nv; i++) {
             kk++;
-            mpf_set (d, MPF_ZERO);
+            mpf_set(d, MPF_ZERO);
             k = j - 1;
             for (l=1; l<=jm1; l++) {
 		mpf_mul(tmp, xpxxpy.xpx[k], xpxxpy.xpx[k-j+i]);
