@@ -4,7 +4,7 @@
    Based on stataread.c from the GNU R "foreign" package with the 
    following original info:
 
-     * $Id: stata_import.c,v 1.11 2006/03/04 05:28:00 allin Exp $
+     * $Id: stata_import.c,v 1.15 2006/09/12 04:01:12 allin Exp $
   
      (c) 1999, 2000, 2001, 2002 Thomas Lumley. 
      2000 Saikat DebRoy
@@ -28,9 +28,10 @@
 #include <stdlib.h>
 #include <math.h>
 
+#include <glib.h>
+
 #include "libgretl.h"
 #include "gretl_string_table.h"
-
 #include "swap_bytes.h"
 
 enum {
@@ -72,7 +73,8 @@ enum {
 #define STATA_DOUBLE_NA pow(2.0, 1023)
 #define STATA_INT_NA 2147483647
 #define STATA_SHORTINT_NA 32767
-#define STATA_BYTE_NA 127
+
+#define STATA_BYTE_NA(b,v) ((v<8 && b==127) || b>=101)
 
 #define NA_INT -999
 
@@ -87,8 +89,6 @@ static void bin_error (int *err)
     *err = 1;
 }
 
-/** Low-level input **/
-
 static int read_int (FILE *fp, int naok, int *err)
 {
     int i;
@@ -103,28 +103,39 @@ static int read_int (FILE *fp, int naok, int *err)
     return ((i == STATA_INT_NA) & !naok ? NA_INT : i);
 }
 
-/* read a 1-byte signed integer */
 static int read_signed_byte (FILE *fp, int naok, int *err)
 { 
     signed char b;
+    int ret;
 
     if (fread(&b, 1, 1, fp) != 1) {
 	bin_error(err);
+	ret = NA_INT;
+    } else {
+	ret = (int) b;
+
+	if (!naok) {
+	    int v = abs(stata_version);
+
+	    if (STATA_BYTE_NA(b, v)) {
+		ret = NA_INT;
+	    }
+	}
     }
 
-    return ((b == STATA_BYTE_NA) & !naok)? NA_INT : (int) b;
+    return ret;
 }
 
-/* read a single byte  */
-static int read_byte (FILE *fp, int naok, int *err)
+static int read_byte (FILE *fp, int *err)
 { 
     unsigned char u;
 
     if (fread(&u, 1, 1, fp) != 1) {
 	bin_error(err);
+	return NA_INT;
     }
 
-    return ((u == STATA_BYTE_NA) & !naok)? NA_INT : (int) u;
+    return (int) u;
 }
 
 static int read_short (FILE *fp, int naok, int *err)
@@ -132,8 +143,8 @@ static int read_short (FILE *fp, int naok, int *err)
     unsigned first, second;
     int s;
 	
-    first = read_byte(fp, 1, err);
-    second = read_byte(fp, 1, err);
+    first = read_byte(fp, err);
+    second = read_byte(fp, err);
 
     if (stata_endian == CN_TYPE_BIG) {
 	s = (first << 8) | second;
@@ -230,7 +241,7 @@ static int check_variable_types (FILE *fp, int *types, int nvar, int *nsv)
     *nsv = 0;
 
     for (i=0; i<nvar && !err; i++) {
-   	unsigned char u = read_byte(fp, 1, &err);
+   	unsigned char u = read_byte(fp, &err);
 
 	types[i] = u;
 	if (stata_type_float(u) || stata_type_double(u)) {
@@ -302,7 +313,35 @@ save_dataset_info (DATAINFO *dinfo, const char *s1, const char *s2)
     return err;
 }
 
-/* use Stata's "date formats" to reonstruct time series information
+static int try_fix_varname (char *name)
+{
+    char test[VNAMELEN];
+    int err = 0;
+
+    *test = 0;
+
+    if (*name == '_') {
+	strcat(test, "x");
+	strncat(test, name, VNAMELEN - 2);
+    } else {
+	strncat(test, name, VNAMELEN - 2);
+	strcat(test, "1");
+    }
+    
+    err = check_varname(test);
+    if (!err) {
+	fprintf(stderr, "Warning: illegal name '%s' changed to '%s'\n",
+		name, test);
+	strcpy(name, test);
+    } else {
+	/* get the right error message in place */
+	check_varname(name);
+    }
+
+    return err;
+}
+
+/* use Stata's "date formats" to reconstruct time series information
    FIXME: add recognition for daily data too? 
    (Stata dates are all zero at the start of 1960.)
 */
@@ -390,12 +429,16 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
     for (i=0; i<nvar && !err; i++) {
         read_string(fp, namelen + 1, aname, &err);
 	printf("variable %d: name = '%s'\n", i+1, aname);
-	strncat(dinfo->varname[i+1], aname, VNAMELEN - 1);
+	if (check_varname(aname) && try_fix_varname(aname)) {
+	    err = 1;
+	} else {
+	    strncat(dinfo->varname[i+1], aname, VNAMELEN - 1);
+	}
     }
 
     /* sortlist -- not relevant */
     for (i=0; i<2*(nvar+1) && !err; i++) {
-        read_byte(fp, 1, &err);
+        read_byte(fp, &err);
     }
     
     /* format list (use it to identify date variables?) */
@@ -430,31 +473,44 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
 	read_string(fp, labellen, datalabel, &err);
 	if (*datalabel != '\0') {
 	    printf("variable %d: label = '%s'\n", i+1, datalabel);
-	    strncat(VARLABEL(dinfo, i+1), datalabel, MAXLABEL - 1);
+	    if (!g_utf8_validate(datalabel, -1, NULL)) {
+		gsize b;
+		gchar *tr = g_locale_to_utf8(datalabel, -1, NULL,
+					     &b, NULL); 
+
+		if (tr != NULL) {
+		    strncat(VARLABEL(dinfo, i+1), tr, MAXLABEL - 1);
+		    g_free(tr);
+		}
+	    } else {
+		strncat(VARLABEL(dinfo, i+1), datalabel, MAXLABEL - 1);
+	    }
 	}
     }
 
     /* variable 'characteristics' -- not handled */
-    while (read_byte(fp, 1, &err)) {
-	if (abs(stata_version) >= 7) { /* manual is wrong here */
+    if (!err) {
+	while (read_byte(fp, &err)) {
+	    if (abs(stata_version) >= 7) { /* manual is wrong here */
+		clen = read_int(fp, 1, &err);
+	    } else {
+		clen = read_short(fp, 1, &err);
+	    }
+	    for (i=0; i<clen; i++) {
+		read_signed_byte(fp, 1, &err);
+	    }
+	}
+	if (abs(stata_version) >= 7) {
 	    clen = read_int(fp, 1, &err);
 	} else {
 	    clen = read_short(fp, 1, &err);
 	}
-	for (i=0; i<clen; i++) {
-	    read_signed_byte(fp, 1, &err);
+	if (clen != 0) {
+	    fputs(_("something strange in the file\n"
+		    "(Type 0 characteristic of nonzero length)"), stderr);
+	    fputc('\n', stderr);
 	}
     }
-    if (abs(stata_version) >= 7) {
-        clen = read_int(fp, 1, &err);
-    } else {
-	clen = read_short(fp, 1, &err);
-    }
-    if (clen != 0) {
-	fputs(_("something strange in the file\n"
-		"(Type 0 characteristic of nonzero length)"), stderr);
-	fputc('\n', stderr);
-    }
 
     /* actual data values */
     for (t=0; t<dinfo->n && !err; t++) {
@@ -499,7 +555,7 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
 
     /* value labels (??) */
 
-    if (abs(stata_version) > 5) {
+    if (!err && abs(stata_version) > 5) {
 	for (j=0; j<nvar; j++) {
 	    /* first int not needed, use fread directly to trigger EOF */
 	    fread((int *) aname, sizeof(int), 1, fp);
@@ -512,9 +568,9 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
 	    printf("variable %d: \"aname\" = '%s'\n", i, aname);
 
 	    /* padding */
-	    read_byte(fp, 1, &err);
-	    read_byte(fp, 1, &err);
-	    read_byte(fp, 1, &err);
+	    read_byte(fp, &err);
+	    read_byte(fp, &err);
+	    read_byte(fp, &err);
 
 	    nlabels = read_int(fp, 1, &err);
 	    totlen = read_int(fp, 1, &err);
@@ -553,7 +609,7 @@ static int parse_dta_header (FILE *fp, int *namelen, int *nvar, int *nobs)
     unsigned char u;
     int err = 0;
     
-    u = read_byte(fp, 1, &err);   /* release version */
+    u = read_byte(fp, &err);   /* release version */
 
     if (!err) {
 	err = get_version_and_namelen(u, namelen);
@@ -567,18 +623,16 @@ static int parse_dta_header (FILE *fp, int *namelen, int *nvar, int *nobs)
     printf("Stata file version %d\n", abs(stata_version));
 
     /* these are file-scope globals */
-    stata_endian = (int) read_byte(fp, 1, &err);
+    stata_endian = (int) read_byte(fp, &err);
     swapends = stata_endian != CN_TYPE_NATIVE;
 
-    read_byte(fp, 1, &err);           /* filetype -- junk */
-    read_byte(fp, 1, &err);           /* padding */
+    read_byte(fp, &err);              /* filetype -- junk */
+    read_byte(fp, &err);              /* padding */
     *nvar = read_short(fp, 1, &err);  /* number of variables */
     *nobs = read_int(fp, 1, &err);    /* number of observations */
 
-    if (!err) {
-	/* sanity check */
-	if (*nvar <= 0) err = 1;
-	if (*nobs <= 0) err = 1;
+    if (!err && (*nvar <= 0 || *nobs <= 0)) {
+	err = 1;
     }
 
     if (!err) {
