@@ -20,7 +20,6 @@
 #include "libgretl.h"
 #include "gretl_matrix.h"
 #include "system.h"
-#include "system_private.h"
 
 #define SDEBUG 0
 
@@ -49,7 +48,7 @@ print_system_vcv (const gretl_equation_system *sys, PRN *prn)
 
 	    pprintf(prn, "%s:\n", _("LR test for diagonal covariance matrix"));
 	    pprintf(prn, "  %s(%d) = %g %s %g\n", _("Chi-square"),
-		    df, lr, _("with p-value"), chisq(lr, df));
+		    df, lr, _("with p-value"), chisq_cdf_comp(lr, df));
 	}
     } else {
 	double lm = sys->diag;
@@ -57,7 +56,7 @@ print_system_vcv (const gretl_equation_system *sys, PRN *prn)
 	if (lm > 0) {
 	    pprintf(prn, "%s:\n", _("Breusch-Pagan test for diagonal covariance matrix"));
 	    pprintf(prn, "  %s(%d) = %g %s %g\n", _("Chi-square"),
-		    df, lm, _("with p-value"), chisq(lm, df));
+		    df, lm, _("with p-value"), chisq_cdf_comp(lm, df));
 	}
     }
 
@@ -467,7 +466,7 @@ print_system_overidentification_test (const gretl_equation_system *sys,
 	pprintf(prn, "  %s = %g\n", _("Restricted log-likelihood"), sys->ll);
 	pprintf(prn, "  %s = %g\n", _("Unrestricted log-likelihood"), sys->llu);
 	pprintf(prn, "  %s(%d) = %g %s %g\n", _("Chi-square"),
-		df, X2, _("with p-value"), chisq(X2, df));
+		df, X2, _("with p-value"), chisq_cdf_comp(X2, df));
 	pputc(prn, '\n');
     } else if ((sys->method == SYS_3SLS || sys->method == SYS_SUR) && df > 0) {
 
@@ -480,7 +479,7 @@ print_system_overidentification_test (const gretl_equation_system *sys,
 
 	pprintf(prn, "%s:\n", _("Hansen-Sargan over-identification test"));
 	pprintf(prn, "  %s(%d) = %g %s %g\n", _("Chi-square"),
-		df, sys->X2, _("with p-value"), chisq(sys->X2, df));
+		df, sys->X2, _("with p-value"), chisq_cdf_comp(sys->X2, df));
 	pputc(prn, '\n');
     }
 }
@@ -586,7 +585,7 @@ static int hansen_sargan_test (gretl_equation_system *sys,
 
 #if SDEBUG
     fprintf(stderr, "Hansen-Sargan: Chi-square(%d) = %g (p-value %g)\n", 
-	    df, X2, chisq(X2, df));
+	    df, X2, chisq_cdf_comp(X2, df));
 #endif
     sys->X2 = X2;
 
@@ -972,7 +971,7 @@ int system_estimate (gretl_equation_system *sys, double ***pZ, DATAINFO *pdinfo,
     } 
 
     /* number of equations */
-    m = system_n_equations(sys);
+    m = sys->n_equations;
 
     /* max indep vars per equation */
     k = system_max_indep_vars(sys);
@@ -991,7 +990,7 @@ int system_estimate (gretl_equation_system *sys, double ***pZ, DATAINFO *pdinfo,
     models = sys->models;
 	    
     if ((method == SYS_FIML || method == SYS_LIML) && !(opt & OPT_Q)) {
-	print_equation_system_info(sys, pdinfo, prn);
+	print_equation_system_info(sys, pdinfo, OPT_H, prn);
     }
 
     /* First estimate the equations separately, and put the
