@@ -26,10 +26,7 @@
 #ifdef WIN32
 # include <windows.h>
 #else
-# if GLIB_CHECK_VERSION(2,0,0)
-#  define USE_GSPAWN
-#  include <signal.h>
-# endif /* GLIB_CHECK_VERSION */
+# include <signal.h>
 #endif
 
 enum opt_codes {
@@ -63,9 +60,9 @@ const char *default_mdl = {
     "(2 1 2)(0 1 1)\n"
 };  
 
-#ifdef USE_GSPAWN
+#ifndef WIN32
 
-# define SP_DEBUG 0
+#define SP_DEBUG 0
 
 static int glib_spawn (const char *workdir, const char *fmt, ...)
 {
@@ -94,12 +91,12 @@ static int glib_spawn (const char *workdir, const char *fmt, ...)
 
     nargs = i;
 
-# if SP_DEBUG
+#if SP_DEBUG
     fputs("spawning the following:\n", stderr);
     for (i=0; i<nargs; i++) {
 	fprintf(stderr, " argv[%d] = '%s'\n", i, argv[i]);
     }
-# endif
+#endif
 
     signal(SIGCHLD, SIG_DFL);
 
@@ -145,27 +142,11 @@ static int glib_spawn (const char *workdir, const char *fmt, ...)
     return ret;
 }
 
-#endif /* USE_GSPAWN */
-
-#if GTK_MAJOR_VERSION == 1
-static void tx_dialog_ok (GtkWidget *w, tx_request *request)
-{
-    request->ret = 1;
-    gtk_widget_hide(request->dialog);
-    gtk_main_quit();
-}
-
-static void tx_dialog_cancel (GtkWidget *w, tx_request *request)
-{
-    gtk_widget_hide(request->dialog);
-    gtk_main_quit();
-}
-#endif
+#endif /* !WIN32 */
 
 static int tx_dialog (tx_request *request)
 {
     GtkWidget *hbox, *vbox, *tmp;
-#if GTK_MAJOR_VERSION >= 2
     gint i, ret = 0;
 
     for (i=0; i<N_COMMON_OPTS; i++) {
@@ -179,21 +160,16 @@ static int tx_dialog (tx_request *request)
 				     NULL,
 				     GTK_DIALOG_MODAL | 
 				     GTK_DIALOG_DESTROY_WITH_PARENT,
-				     GTK_STOCK_OK,
-				     GTK_RESPONSE_ACCEPT,
 				     GTK_STOCK_CANCEL,
 				     GTK_RESPONSE_REJECT,
+				     GTK_STOCK_OK,
+				     GTK_RESPONSE_ACCEPT,
 				     NULL);
-#else
-    request->dialog = gtk_dialog_new();
-    gtk_window_set_title (GTK_WINDOW(request->dialog), 
-			  (request->code == TRAMO_SEATS)? 
-			  "TRAMO/SEATS" : "X-12-ARIMA");
-#endif
 
     vbox = gtk_vbox_new(FALSE, 0);    
 
     if (request->code == TRAMO_SEATS) {
+	gtk_dialog_set_has_separator(GTK_DIALOG(request->dialog), FALSE);
 	show_tramo_options(request, vbox);
     } else {
 	/* X-12-ARIMA */
@@ -237,37 +213,12 @@ static int tx_dialog (tx_request *request)
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(request->dialog)->vbox),
 		       hbox, FALSE, FALSE, 5);
 
-#if GTK_MAJOR_VERSION >= 2
-    ret = gtk_dialog_run (GTK_DIALOG(request->dialog));
+    ret = gtk_dialog_run(GTK_DIALOG(request->dialog));
+
     if (ret == GTK_RESPONSE_ACCEPT) ret = 1;
     else ret = 0;
+
     return ret;
-#else /* GTK 1.N */
-    request->ret = 0;
-
-    /* Create an "OK" button */
-    tmp = gtk_button_new_with_label (_("OK"));
-    GTK_WIDGET_SET_FLAGS (tmp, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (request->dialog)->action_area), 
-                        tmp, TRUE, TRUE, FALSE);
-    gtk_signal_connect(GTK_OBJECT(tmp), "clicked",
-                       GTK_SIGNAL_FUNC(tx_dialog_ok), request);
-    gtk_widget_grab_default (tmp);
-    gtk_widget_show (tmp);
-
-    /* Create a "Cancel" button */
-    tmp = gtk_button_new_with_label (_(" Cancel "));
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (request->dialog)->action_area), 
-                        tmp, TRUE, TRUE, FALSE);
-    gtk_signal_connect(GTK_OBJECT(tmp), "clicked",
-                       GTK_SIGNAL_FUNC(tx_dialog_cancel), request);
-    gtk_widget_show (tmp);
-
-    gtk_widget_show (request->dialog);
-    gtk_main();
-
-    return request->ret;
-#endif
 }
 
 static void get_seats_command (char *seats, const char *tramo)
@@ -276,16 +227,25 @@ static void get_seats_command (char *seats, const char *tramo)
 
     strcpy(seats, tramo);
     p = strrchr(seats, SLASH);
-    if (p != NULL) strcpy(p + 1, "seats");
-    else strcpy(seats, "seats");
+    if (p != NULL) {
+	strcpy(p + 1, "seats");
+    } else {
+	strcpy(seats, "seats");
+    }
 }
 
 static int graph_series (double **Z, DATAINFO *pdinfo, int opt)
 {
     FILE *fp = NULL;
+    const double *obs;
     char title[32];
     int t;
 
+    obs = gretl_plotx(pdinfo);
+    if (obs == NULL) {
+	return E_ALLOC;
+    }
+
     if (gnuplot_init(PLOT_TRI_GRAPH, &fp)) {
 	return E_FOPEN;
     }
@@ -297,8 +257,7 @@ static int graph_series (double **Z, DATAINFO *pdinfo, int opt)
 	    fputs("set xtics nomirror 0,1\n", fp); 
 	    fputs("set mxtics 4\n", fp);
 	}
-    }
-    if (pdinfo->pd == 12) {
+    } else if (pdinfo->pd == 12) {
 	if ((pdinfo->t2 - pdinfo->t1) / 12 < 8) {
 	    fputs("set xtics nomirror 0,1\n", fp); 
 	    fputs("set mxtics 12\n", fp);
@@ -313,43 +272,51 @@ static int graph_series (double **Z, DATAINFO *pdinfo, int opt)
     } else {
 	sprintf(title, "%s - 1", I_("irregular"));
     }
+
     fprintf(fp, "set bars 0\n"
 	    "set origin 0.0,0.0\n"
 	    "plot '-' using 1:($2-1.0) title '%s' w impulses\n",
 	    title);
+
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	double y = Z[D13 + 1][t];
 
-	fprintf(fp, "%g %g\n", Z[XAXIS][t], 
+	fprintf(fp, "%g %g\n", obs[t], 
 		(opt == TRAMO_SEATS && tramo_got_irfin)? y / 100.0 : y);
     }
     fputs("e\n", fp);
 
     /* actual vs trend/cycle */
+
     fprintf(fp, "set origin 0.0,0.33\n"
 	    "plot '-' using 1:2 title '%s' w l, \\\n"
 	    " '-' using 1:2 title '%s' w l\n",
 	    pdinfo->varname[0], I_("trend/cycle"));
+
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) { 
-	fprintf(fp, "%g %g\n", Z[XAXIS][t], Z[0][t]);
+	fprintf(fp, "%g %g\n", obs[t], Z[0][t]);
     }
     fputs("e , \\\n", fp);
+
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) { 
-	fprintf(fp, "%g %g\n", Z[XAXIS][t], Z[D12 + 1][t]);
+	fprintf(fp, "%g %g\n", obs[t], Z[D12 + 1][t]);
     }
     fputs("e\n", fp);
 
     /* actual vs seasonally adjusted */
+
     fprintf(fp, "set origin 0.0,0.66\n"
 	    "plot '-' using 1:2 title '%s' w l, \\\n"
 	    " '-' using 1:2 title '%s' w l\n",
 	    pdinfo->varname[0], I_("adjusted"));
+
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	fprintf(fp, "%g %g\n", Z[XAXIS][t], Z[0][t]);
+	fprintf(fp, "%g %g\n", obs[t], Z[0][t]);
     }
     fputs("e\n", fp);
+
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	fprintf(fp, "%g %g\n", Z[XAXIS][t], Z[D11 + 1][t]);
+	fprintf(fp, "%g %g\n", obs[t], Z[D11 + 1][t]);
     }
     fputs("e\n", fp);
 
@@ -558,8 +525,11 @@ static int write_tramo_file (const char *fname,
     startyr = (int) x;
     sprintf(tmp, "%g", x);
     p = strchr(tmp, '.');
-    if (p != NULL) startper = atoi(p + 1);
-    else startper = 1;
+    if (p != NULL) {
+	startper = atoi(p + 1);
+    } else {
+	startper = 1;
+    }
 
     fprintf(fp, "%s\n", pdinfo->varname[v]);
     fprintf(fp, "%d %d %d %d\n", tsamp, startyr, startper, pdinfo->pd);
@@ -606,8 +576,11 @@ static int write_spc_file (const char *fname,
     startyr = (int) x;
     sprintf(tmp, "%g", x);
     p = strchr(tmp, '.');
-    if (p != NULL) startper = atoi(p + 1);
-    else startper = 1;
+    if (p != NULL) {
+	startper = atoi(p + 1);
+    } else {
+	startper = 1;
+    }
 
     fprintf(fp, "series{\n period=%d\n title=\"%s\"\n", pdinfo->pd, 
 	    pdinfo->varname[v]);
@@ -621,7 +594,9 @@ static int write_spc_file (const char *fname,
 	} else {
 	    fprintf(fp, "%g ", Z[v][t]);
 	}
-	if ((i + 1) % 7 == 0) fputc('\n', fp);
+	if ((i + 1) % 7 == 0) {
+	    fputc('\n', fp);
+	}
 	i++;
     }
     fputs(" )\n}\n", fp);
@@ -710,28 +685,36 @@ static int save_vars_to_dataset (double ***pZ, DATAINFO *pdinfo,
     return 0;
 }
 
-#if defined(WIN32)
+#ifdef WIN32
 
 static int helper_spawn (const char *prog, const char *vname,
 			 const char *workdir, int code)
 {
-    char cmd[MAXLEN];
+    char *cmd = NULL;
+    int ret;
 
     if (code == TRAMO_ONLY) {
-	sprintf(cmd, "\"%s\" -i %s -k serie", prog, vname);
+	cmd = g_strdup_printf("\"%s\" -i %s -k serie", prog, vname);
     } else if (code == TRAMO_SEATS) {
-	sprintf(cmd, "\"%s\" -OF %s", prog, vname);
+	cmd = g_strdup_printf("\"%s\" -OF %s", prog, vname);
     } else if (code == X12A) {
-	sprintf(cmd, "\"%s\" %s -r -p -q", prog, vname);
+	cmd = g_strdup_printf("\"%s\" %s -r -p -q", prog, vname);
     } else {
 	return 1;
     }
 
-    return winfork(cmd, workdir, SW_SHOWMINIMIZED, 
-		   CREATE_NEW_CONSOLE | HIGH_PRIORITY_CLASS);
+    if (cmd == NULL) {
+	ret = E_ALLOC;
+    } else {
+	ret = winfork(cmd, workdir, SW_SHOWMINIMIZED, 
+		      CREATE_NEW_CONSOLE | HIGH_PRIORITY_CLASS);
+	g_free(cmd);
+    }
+
+    return ret;
 }
 
-#elif defined(USE_GSPAWN)
+#else
 
 static int helper_spawn (const char *prog, const char *vname,
 			 const char *workdir, int code)
@@ -751,29 +734,7 @@ static int helper_spawn (const char *prog, const char *vname,
     return err;
 }
 
-#else
-
-static int helper_spawn (const char *prog, const char *vname,
-			 const char *workdir, int code)
-{
-    char cmd[MAXLEN];
-
-    if (code == TRAMO_ONLY) {
-	sprintf(cmd, "cd \"%s\" && \"%s\" -i %s -k serie >/dev/null", 
-		workdir, prog, vname);
-    } else if (code == TRAMO_SEATS) {
-	sprintf(cmd, "cd \"%s\" && \"%s\" -OF %s", workdir, prog, vname);	
-    } else if (code == X12A) {
-	sprintf(cmd, "cd \"%s\" && \"%s\" %s -r -p -q >/dev/null", 
-		workdir, prog, vname);
-    } else {
-	return 1;
-    }
-
-    return gretl_spawn(cmd);
-}
-
-#endif /* end spawn versions switch */
+#endif
 
 int write_tx_data (char *fname, int varnum, 
 		   double ***pZ, DATAINFO *pdinfo, int *graph, 
@@ -791,7 +752,7 @@ int write_tx_data (char *fname, int varnum,
     *errmsg = 0;
 
     /* sanity check */
-    if (!pdinfo->vector[varnum]) {
+    if (var_is_scalar(pdinfo, varnum)) {
 	sprintf(errmsg, "%s %s", pdinfo->varname[varnum], 
 		_("is a scalar"));
 	return 1;
@@ -913,17 +874,12 @@ int write_tx_data (char *fname, int varnum,
 	    }
 
 	    if (request.opt[TRIGRAPH].save) {
-		int pv = plotvar(&tmpZ, tmpinfo);
-
-		if (pv < 0) {
-		    err = 1;
+		err = graph_series(tmpZ, tmpinfo, request.code);
+		if (err) {
+		    fprintf(stderr, "graph_series() failed\n");
 		} else {
-		    err = graph_series(tmpZ, tmpinfo, request.code);
-		    if (err) {
-			fprintf(stderr, "graph_series() failed\n");
-		    }
+		    *graph = 1;
 		}
-		if (!err) *graph = 1;
 	    }
 	}
 
