@@ -3,8 +3,8 @@
 ;; Copyright (C) 2005 Allin Cottrell
 
 ;; Author:     Allin Cottrell <cottrell@wfu.edu>
-;; Created:    January 18 2005
-;; Version:    0.1
+;; Created:    July 7 2006
+;; Version:    0.2
 ;; Keywords:   gretl, econometrics
 
 ;; With hefty borrowings from octave-mod.el
@@ -78,10 +78,10 @@ All Gretl abbrevs start with a grave accent (`).")
   "String to insert to start a new Gretl in-line comment.")
 
 (defvar gretl-comment-start-skip "\\s<+\\s-*"
-  "Regexp to match the start of an Gretl comment up to its body.")
+  "Regexp to match the start of a Gretl comment up to its body.")
 
 (defvar gretl-begin-keywords
-  '("loop" "function" "if" "system"))
+  '("loop" "function" "if" "system" "mle" "nls" "restrict"))
 
 (defvar gretl-else-keywords
   '("else"))
@@ -97,37 +97,70 @@ All Gretl abbrevs start with a grave accent (`).")
   "Reserved words in Gretl.")
 
 (defvar gretl-command-words
-  '("add" "addto" "adf" "append" "ar" "arch" "arma" "break" "boxplot" 
-    "chow" "coeffsum" "coint" "coint2" "corc" "corr" "corrgm" 
-    "criteria" "critical" "cusum" "data" "delete" "diff" "else" 
-    "eqnprint" "equation" "estimate"
-    "fcast" "fcasterr" "fit" "freq" "garch" "genr"      
-    "gnuplot" "graph" "hausman" "hccm" "help" "hilu" "hsk" "if" 
-    "import" "info" "kpss" "label" "labels" "lad" "lags" "ldiff" 
-    "leverage" "lmtest" "logistic" "logit" "logs" "loop"
-    "meantest" "modeltab" "mpols" "multiply" "nls" "nulldata" 
-    "ols" "omit" "omitfrom" "open" "outfile" "panel" "pca" "pergm" 
-    "plot" "pooled" "print" "printf" "probit" "pvalue" "pwe" 
-    "quit" "rename" "reset" "restrict" "rhodiff" "rmplot" "run" 
-    "runs" "scatters" "set" "setobs" "setmiss" "shell" "sim"      
-    "smpl" "spearman" "square" "store" "summary" 
-    "tabprint" "testuhat" "tobit" "tsls" "var" "varlist" 
-    "vartest" "vif" "wls")
+ '("add" "addobs" "addto" "adf" "append" "ar" "arch" "arima"
+   "break" "boxplot" "chow" "coeffsum" "coint" "coint2" "corc" "corr"
+   "corrgm" "criteria" "critical" "cusum" "data" "delete" "diff" "discrete"
+   "dummify" "else" "end" "endif" "endloop" "eqnprint" "equation" "estimate"
+   "fcast" "fcasterr" "fit" "fncall" "freq" "function" "error" "garch"
+   "genr" "gnuplot" "graph" "hausman" "hccm" "help" "hilu" "hsk"
+   "hurst" "if" "import" "include" "info" "kpss" "labels" "lad"
+   "lags" "ldiff" "leverage" "lmtest" "logistic" "logit" "logs" "loop"
+   "mahal"  "meantest" "mle" "modeltab" "mpols" "multiply" "nls"
+   "nulldata" "ols" "omit" "omitfrom" "open" "outfile" "panel" "pca"
+   "pergm" "plot" "poisson" "print" "printf" "probit" "pvalue" "pwe"
+   "qlrtest" "quit" "remember" "rename" "reset" "restrict" "rhodiff" "rmplot"
+   "run" "runs" "scatters" "sdiff" "set" "setinfo" "setobs" "setmiss"
+   "shell" "smpl" "spearman" "square" "store" "summary" "system" "tabprint"
+   "testuhat" "tobit" "transpos" "tsls" "var" "varlist" "vartest" "vecm"
+   "vif" "wls" "xtab")
   "Commands in Gretl (these names are also reserved).")
 
 (defvar gretl-genr-functions
-  '("log" "exp" "sin" "cos" "tan" "atan" "diff"
-    "ldiff" "mean" "sd" "min" "max" "sort" 
-    "int" "ln" "coeff" "abs" "rho" "sqrt" "sum"
-    "nobs" "normal" "uniform" "resample" "stderr"
-    "cum" "hpfilt" "bkfilt" "missing" "misszero"
-    "corr" "vcv" "var" "cov" "median" "zeromiss"
-    "pvalue" "mpow" "mlog" "sst")
+ '("log" "exp" "sin" "cos" "tan" "atan" "int" "abs"
+   "sqrt" "dnorm" "cnorm" "qnorm" "gamma" "lngamma" "normal" "uniform"
+   "log10" "log2" "diff" "ldiff" "sdiff" "mean" "sd" "min"
+   "max" "sort" "dsort" "sum" "nobs" "firstobs" "lastobs" "cum"
+   "missing" "ok" "misszero" "corr" "var" "lrvar" "sst" "cov"
+   "median" "gini" "zeromiss" "pvalue" "critical" "obsnum" "mpow" "mlog"
+   "resample" "hpfilt" "bkfilt" "fracdiff" "varnum" "isseries" "islist" "nelem"
+   "det" "inv" "cholesky" "cdemean" "diag" "qrdecomp" "eigensym" "eigengen"
+   "ldet" "tr" "sumr" "sumc" "onenorm" "rcond" "rows" "cols"
+   "transp" "vec" "vech" "unvech" "I" "zeros")
   "Builtin functions for Gretl's genr command.")
 
-(defvar gretl-variables
-  '("GRETL_USE_QR" "HOME")
-  "Builtin variables in Gretl.")
+(defvar gretl-option-flags
+ '("arma-init" "autocorr" "bartlett" "both"
+   "box1" "byobs" "c" "chi-square"
+   "close" "column" "complete" "conditional"
+   "continuous" "cross" "cross-section" "crt"
+   "csv" "ct" "ctt" "dat"
+   "database" "difference" "discrete" "dummy"
+   "dynamic" "exit" "full" "gamma"
+   "geomean" "gnu-R" "gnu-octave" "gzipped"
+   "hausman-reg" "hessian" "impulse-responses" "inst"
+   "iterate" "lagselect" "logs" "long"
+   "nc" "no-corc" "no-dates" "no-df-corr"
+   "no-missing" "notches" "octave" "omit-obs"
+   "one-scale" "overwrite" "panel" "panel-vars"
+   "plot" "print-final" "progressive" "quiet"
+   "random" "random-effects" "rc" "replace"
+   "restrict" "restructure" "reverse" "robust"
+   "row" "save" "save-all" "seasonals"
+   "silent" "simple-print" "special-time-series" "squares"
+   "stacked-cross-section" "stacked-time-series" "static" "suppress-fitted"
+   "t-ratios" "tall" "ten" "time-series"
+   "traditional" "trend" "unequal-vars" "unit-weights"
+   "variance-decomp" "vcv" "verbose" "white"
+   "with-impulses" "with-lines" "write" "x-12-arima"
+   "zeros")
+  "Gretl option flags.")
+
+(defvar gretl-internal-vars
+ '("ess" "t" "rsq" "sigma" "df" "ncoeff" "lnl" "aic"
+   "bic" "hqc" "trsq" "coeff" "stderr" "vcv" "rho" "uhat"
+   "yhat" "ahat" "h" "coeff" "ess" "vcv" "rho" "jalpha"
+   "jbeta" "nobs" "nvars" "pd" "pvalue" "test")
+  "Model-related variables.")
 
 (defvar gretl-function-header-regexp
   (concat "^\\s-*\\<\\(function\\)\\>"
@@ -144,7 +177,9 @@ parenthetical grouping.")
 		 "\\)\\>")
 	 'font-lock-keyword-face)
    ;; Fontify all option flags.
-   (cons "[ \t]--[-a-z]+\\>"
+   (cons (concat "[ \t]--\\("
+		 (mapconcat 'identity gretl-option-flags "\\|")
+		 "\\)")
 	 'font-lock-constant-face)
    ;; Fontify all command words.
    (cons (concat "\\<\\("
@@ -156,11 +191,12 @@ parenthetical grouping.")
 	 (if (boundp 'font-lock-builtin-face)
 	     'font-lock-builtin-face
 	   'font-lock-preprocessor-face))
-   ;; Fontify all builtin variables.
-   (cons (concat "\\<\\("
-		 (mapconcat 'identity gretl-variables "\\|")
+   ;; Fontify all internal variables.
+   (cons (concat "\\$\\("
+		 (mapconcat 'identity gretl-internal-vars "\\|")
 		 "\\)\\>")
 	 'font-lock-variable-name-face)
+
    ;; Fontify all genr functions.
    (cons (concat "\\<\\("
 		 (mapconcat 'identity gretl-genr-functions "\\|")
@@ -325,8 +361,11 @@ newline or semicolon after an else or end keyword."
   (concat gretl-block-else-regexp "\\|" gretl-block-end-regexp))
 
 (defvar gretl-block-match-alist
-  '(("loop" . ("endloop"))
+  '(("loop" . ("end" "endloop"))
     ("if" . ("else" "endif"))
+    ("nls" . ("end"))
+    ("mle" . ("end"))
+    ("restrict" . ("end"))
     ("system" . ("end")))
   "Alist with Gretl's matching block keywords.
 Has Gretl's begin keywords as keys and a list of the matching else or
@@ -1164,7 +1203,7 @@ otherwise."
 	  (mapcar '(lambda (var) (cons var var))
 		  (append gretl-command-words
 			  gretl-genr-functions
-			  gretl-variables)))))
+			  gretl-internal-vars)))))
 
 (defun gretl-complete-symbol ()
   "Perform completion on Gretl symbol preceding point.
