@@ -27,11 +27,13 @@ $verstr = $ARGV[0];
 print STDERR "Making installer script for gretl version $verstr...\n";
 
 print "; -- gretl.iss --\n";
-print "[Setup]\n";
+print "\n[Setup]\n";
 print "AppName=gretl\n";
 print "AppVerName=gretl version $verstr\n";
-print "AppCopyright=Copyright C 1999-2005 Allin Cottrell\n";
-print "DefaultDirName=c:\\userdata\\gretl\n";
+print "AppPublisherURL=http://gretl.sourceforge.net/\n";
+print "AppSupportURL=http://gretl.sourceforge.net/\n";
+print "AppCopyright=Copyright C 1999-2006 Allin Cottrell\n";
+print "DefaultDirName={pf}\\gretl\n";
 print "DefaultGroupName=gretl\n";
 print "UninstallDisplayIcon={app}\\gretlw32.exe\n";
 print "ChangesAssociations=yes\n";
@@ -44,15 +46,25 @@ print "Filename: \"{win}\\WGNUPLOT.INI\"; Section: \"WGNUPLOT\"; Key: \"TextFont
 print "\n[InstallDelete]\n";
 print "Type: files; Name: \"{app}\\*.dll\"\n";
 
+print "\n[Dirs]\n";
+print "Name: {code:GetDataDir}; Flags: uninsneveruninstall\n";
+
 print "\n[Files]\n";
 
 while ($line = <STDIN>) {
+    if ($line =~ /VERSION/ || $line =~ /DATE/) {
+        next;
+    }   
+    @pathbits = split(/ +/, $line); 
+    $line = $pathbits[@pathbits - 1];
     $line =~ s+/+\\+g;
     chomp($line);
     @pathbits = split(/\\/, $line);
     print "Source: \"$line\"; "; 
     if ($line =~ /\.ini$/) {
 	print "Destdir: \"{win}";
+    } elsif ($line =~ /README/) {
+        print "Destdir: \"{code:GetDataDir}";
     } else {
 	print "Destdir: \"{app}";
 	for ($i = 1; $i < @pathbits - 1; $i++) {
@@ -75,6 +87,8 @@ print "Root: HKCR; Subkey: \"Software\\gretl\"; Flags: uninsdeletekey\n";
 print "Root: HKCU; Subkey: \"Software\\gretl\"; Flags: uninsdeletekey\n";
 print "Root: HKCR; Subkey: \"Software\\gretl\"; ValueType: string; ValueName: ";
 print "\"gretldir\"; ValueData: \"{app}\"\n"; 
+print "Root: HKCU; Subkey: \"Software\\gretl\"; ValueType: string; ValueName: ";
+print "\"userdir\"; ValueData: \"{code:GetDataDir}\"\n"; 
 print "Root: HKCR; Subkey: \"Software\\gretl\"; ValueType: string; ValueName: ";
 print "\"gnuplot\"; ValueData: \"{app}\\wgnuplot.exe\"\n";
 print "Root: HKCR; Subkey: \"Software\\gretl\"; ValueType: string; ValueName: ";
@@ -82,8 +96,6 @@ print "\"Rcommand\"; ValueData: \"RGui.exe\"\n";
 print "Root: HKCR; Subkey: \"Software\\gretl\"; ValueType: string; ValueName: ";
 print "\"viewdvi\"; ValueData: \"windvi.exe\"\n";
 print "Root: HKCU; Subkey: \"Software\\gretl\"; ValueType: string; ValueName: ";
-print "\"userdir\"; ValueData: \"{app}\\user\\\"\n";   
-print "Root: HKCU; Subkey: \"Software\\gretl\"; ValueType: string; ValueName: ";
 print "\"expert\"; ValueData: \"false\"\n";    
 print "Root: HKCU; Subkey: \"Software\\gretl\"; ValueType: string; ValueName: ";
 print "\"binbase\"; ValueData: \"{app}\\db\\\"\n";   
@@ -158,3 +170,45 @@ print "ValueName: \"\"; ValueData: \"{app}\\gretlw32.exe,2\"\n";
 print "Root: HKCR; Subkey: \"GretlScriptFile\\shell\\open\\command\"; ";
 print "ValueType: string; ValueName: \"\"; ValueData: ";
 print "\"\"\"{app}\\gretlw32.exe\"\" -r \"\"%1\"\"\"\n";
+
+print "\n[Code]\n";
+print "var\n";
+print "  DataDirPage: TInputDirWizardPage;\n";
+print "\n";  
+print "procedure InitializeWizard;\n";
+print "begin\n";
+print "  { Create the pages }\n";
+print "   DataDirPage := CreateInputDirPage(wpSelectDir,\n";
+print "                  'Select Personal Data Directory', \n";
+print "                  'Where should personal data files be kept?',\n";
+print "                  'Select the folder for storing personal data files, then click Next.',\n";
+print "                   False, '');\n";
+print "  DataDirPage.Add('');\n";
+print "  DataDirPage.Values[0] := GetPreviousData('DataDir', '');\n";
+print "  if DataDirPage.Values[0] = '' then\n";
+print "     DataDirPage.Values[0] := ExpandConstant('{userappdata}\\gretl');\n";
+print "end;\n";
+print "\n";
+print "procedure RegisterPreviousData(PreviousDataKey: Integer);\n";
+print "begin\n";
+print "  { Store the settings so we can restore them next time }\n";
+print "  SetPreviousData(PreviousDataKey, 'DataDir', DataDirPage.Values[0]);\n";
+print "end;\n";
+print "\n";
+print "function UpdateReadyMemo(Space, NewLine, MemoUserInfoInfo, MemoDirInfo, MemoTypeInfo,\n";
+print "  MemoComponentsInfo, MemoGroupInfo, MemoTasksInfo: String): String;\n";
+print "var\n";
+print "  S: String;\n";
+print "begin\n";
+print "  { Fill the 'Ready Memo' with the normal settings and the custom settings }\n";
+print "  S := '';\n";
+print "  S := S + MemoDirInfo + NewLine;\n";
+print "  S := S + Space + DataDirPage.Values[0] + ' (personal data files)' + NewLine;\n";
+print "  Result := S;\n";
+print "end;\n\n";
+print "function GetDataDir(Param: String): String;\n";
+print "begin\n";
+print "  { Return the selected DataDir }\n";
+print "  Result := DataDirPage.Values[0];\n";
+print "end;\n";
+
